/*!
 * jQuery JavaScript Library v1.7.1
 * http://jquery.com/
 *
 * Copyright 2011, John Resig
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * Includes Sizzle.js
 * http://sizzlejs.com/
 * Copyright 2011, The Dojo Foundation
 * Released under the MIT, BSD, and GPL Licenses.
 *
 * Date: Mon Nov 21 21:11:03 2011 -0500
 */
(function( window, undefined ) {

// Use the correct document accordingly with window argument (sandbox)
var document = window.document,
	navigator = window.navigator,
	location = window.location;
var jQuery = (function() {

// Define a local copy of jQuery
var jQuery = function( selector, context ) {
		// The jQuery object is actually just the init constructor 'enhanced'
		return new jQuery.fn.init( selector, context, rootjQuery );
	},

	// Map over jQuery in case of overwrite
	_jQuery = window.jQuery,

	// Map over the $ in case of overwrite
	_$ = window.$,

	// A central reference to the root jQuery(document)
	rootjQuery,

	// A simple way to check for HTML strings or ID strings
	// Prioritize #id over <tag> to avoid XSS via location.hash (#9521)
	quickExpr = /^(?:[^#<]*(<[\w\W]+>)[^>]*$|#([\w\-]*)$)/,

	// Check if a string has a non-whitespace character in it
	rnotwhite = /\S/,

	// Used for trimming whitespace
	trimLeft = /^\s+/,
	trimRight = /\s+$/,

	// Match a standalone tag
	rsingleTag = /^<(\w+)\s*\/?>(?:<\/\1>)?$/,

	// JSON RegExp
	rvalidchars = /^[\],:{}\s]*$/,
	rvalidescape = /\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g,
	rvalidtokens = /"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g,
	rvalidbraces = /(?:^|:|,)(?:\s*\[)+/g,

	// Useragent RegExp
	rwebkit = /(webkit)[ \/]([\w.]+)/,
	ropera = /(opera)(?:.*version)?[ \/]([\w.]+)/,
	rmsie = /(msie) ([\w.]+)/,
	rmozilla = /(mozilla)(?:.*? rv:([\w.]+))?/,

	// Matches dashed string for camelizing
	rdashAlpha = /-([a-z]|[0-9])/ig,
	rmsPrefix = /^-ms-/,

	// Used by jQuery.camelCase as callback to replace()
	fcamelCase = function( all, letter ) {
		return ( letter + "" ).toUpperCase();
	},

	// Keep a UserAgent string for use with jQuery.browser
	userAgent = navigator.userAgent,

	// For matching the engine and version of the browser
	browserMatch,

	// The deferred used on DOM ready
	readyList,

	// The ready event handler
	DOMContentLoaded,

	// Save a reference to some core methods
	toString = Object.prototype.toString,
	hasOwn = Object.prototype.hasOwnProperty,
	push = Array.prototype.push,
	slice = Array.prototype.slice,
	trim = String.prototype.trim,
	indexOf = Array.prototype.indexOf,

	// [[Class]] -> type pairs
	class2type = {};

jQuery.fn = jQuery.prototype = {
	constructor: jQuery,
	init: function( selector, context, rootjQuery ) {
		var match, elem, ret, doc;

		// Handle $(""), $(null), or $(undefined)
		if ( !selector ) {
			return this;
		}

		// Handle $(DOMElement)
		if ( selector.nodeType ) {
			this.context = this[0] = selector;
			this.length = 1;
			return this;
		}

		// The body element only exists once, optimize finding it
		if ( selector === "body" && !context && document.body ) {
			this.context = document;
			this[0] = document.body;
			this.selector = selector;
			this.length = 1;
			return this;
		}

		// Handle HTML strings
		if ( typeof selector === "string" ) {
			// Are we dealing with HTML string or an ID?
			if ( selector.charAt(0) === "<" && selector.charAt( selector.length - 1 ) === ">" && selector.length >= 3 ) {
				// Assume that strings that start and end with <> are HTML and skip the regex check
				match = [ null, selector, null ];

			} else {
				match = quickExpr.exec( selector );
			}

			// Verify a match, and that no context was specified for #id
			if ( match && (match[1] || !context) ) {

				// HANDLE: $(html) -> $(array)
				if ( match[1] ) {
					context = context instanceof jQuery ? context[0] : context;
					doc = ( context ? context.ownerDocument || context : document );

					// If a single string is passed in and it's a single tag
					// just do a createElement and skip the rest
					ret = rsingleTag.exec( selector );

					if ( ret ) {
						if ( jQuery.isPlainObject( context ) ) {
							selector = [ document.createElement( ret[1] ) ];
							jQuery.fn.attr.call( selector, context, true );

						} else {
							selector = [ doc.createElement( ret[1] ) ];
						}

					} else {
						ret = jQuery.buildFragment( [ match[1] ], [ doc ] );
						selector = ( ret.cacheable ? jQuery.clone(ret.fragment) : ret.fragment ).childNodes;
					}

					return jQuery.merge( this, selector );

				// HANDLE: $("#id")
				} else {
					elem = document.getElementById( match[2] );

					// Check parentNode to catch when Blackberry 4.6 returns
					// nodes that are no longer in the document #6963
					if ( elem && elem.parentNode ) {
						// Handle the case where IE and Opera return items
						// by name instead of ID
						if ( elem.id !== match[2] ) {
							return rootjQuery.find( selector );
						}

						// Otherwise, we inject the element directly into the jQuery object
						this.length = 1;
						this[0] = elem;
					}

					this.context = document;
					this.selector = selector;
					return this;
				}

			// HANDLE: $(expr, $(...))
			} else if ( !context || context.jquery ) {
				return ( context || rootjQuery ).find( selector );

			// HANDLE: $(expr, context)
			// (which is just equivalent to: $(context).find(expr)
			} else {
				return this.constructor( context ).find( selector );
			}

		// HANDLE: $(function)
		// Shortcut for document ready
		} else if ( jQuery.isFunction( selector ) ) {
			return rootjQuery.ready( selector );
		}

		if ( selector.selector !== undefined ) {
			this.selector = selector.selector;
			this.context = selector.context;
		}

		return jQuery.makeArray( selector, this );
	},

	// Start with an empty selector
	selector: "",

	// The current version of jQuery being used
	jquery: "1.7.1",

	// The default length of a jQuery object is 0
	length: 0,

	// The number of elements contained in the matched element set
	size: function() {
		return this.length;
	},

	toArray: function() {
		return slice.call( this, 0 );
	},

	// Get the Nth element in the matched element set OR
	// Get the whole matched element set as a clean array
	get: function( num ) {
		return num == null ?

			// Return a 'clean' array
			this.toArray() :

			// Return just the object
			( num < 0 ? this[ this.length + num ] : this[ num ] );
	},

	// Take an array of elements and push it onto the stack
	// (returning the new matched element set)
	pushStack: function( elems, name, selector ) {
		// Build a new jQuery matched element set
		var ret = this.constructor();

		if ( jQuery.isArray( elems ) ) {
			push.apply( ret, elems );

		} else {
			jQuery.merge( ret, elems );
		}

		// Add the old object onto the stack (as a reference)
		ret.prevObject = this;

		ret.context = this.context;

		if ( name === "find" ) {
			ret.selector = this.selector + ( this.selector ? " " : "" ) + selector;
		} else if ( name ) {
			ret.selector = this.selector + "." + name + "(" + selector + ")";
		}

		// Return the newly-formed element set
		return ret;
	},

	// Execute a callback for every element in the matched set.
	// (You can seed the arguments with an array of args, but this is
	// only used internally.)
	each: function( callback, args ) {
		return jQuery.each( this, callback, args );
	},

	ready: function( fn ) {
		// Attach the listeners
		jQuery.bindReady();

		// Add the callback
		readyList.add( fn );

		return this;
	},

	eq: function( i ) {
		i = +i;
		return i === -1 ?
			this.slice( i ) :
			this.slice( i, i + 1 );
	},

	first: function() {
		return this.eq( 0 );
	},

	last: function() {
		return this.eq( -1 );
	},

	slice: function() {
		return this.pushStack( slice.apply( this, arguments ),
			"slice", slice.call(arguments).join(",") );
	},

	map: function( callback ) {
		return this.pushStack( jQuery.map(this, function( elem, i ) {
			return callback.call( elem, i, elem );
		}));
	},

	end: function() {
		return this.prevObject || this.constructor(null);
	},

	// For internal use only.
	// Behaves like an Array's method, not like a jQuery method.
	push: push,
	sort: [].sort,
	splice: [].splice
};

// Give the init function the jQuery prototype for later instantiation
jQuery.fn.init.prototype = jQuery.fn;

jQuery.extend = jQuery.fn.extend = function() {
	var options, name, src, copy, copyIsArray, clone,
		target = arguments[0] || {},
		i = 1,
		length = arguments.length,
		deep = false;

	// Handle a deep copy situation
	if ( typeof target === "boolean" ) {
		deep = target;
		target = arguments[1] || {};
		// skip the boolean and the target
		i = 2;
	}

	// Handle case when target is a string or something (possible in deep copy)
	if ( typeof target !== "object" && !jQuery.isFunction(target) ) {
		target = {};
	}

	// extend jQuery itself if only one argument is passed
	if ( length === i ) {
		target = this;
		--i;
	}

	for ( ; i < length; i++ ) {
		// Only deal with non-null/undefined values
		if ( (options = arguments[ i ]) != null ) {
			// Extend the base object
			for ( name in options ) {
				src = target[ name ];
				copy = options[ name ];

				// Prevent never-ending loop
				if ( target === copy ) {
					continue;
				}

				// Recurse if we're merging plain objects or arrays
				if ( deep && copy && ( jQuery.isPlainObject(copy) || (copyIsArray = jQuery.isArray(copy)) ) ) {
					if ( copyIsArray ) {
						copyIsArray = false;
						clone = src && jQuery.isArray(src) ? src : [];

					} else {
						clone = src && jQuery.isPlainObject(src) ? src : {};
					}

					// Never move original objects, clone them
					target[ name ] = jQuery.extend( deep, clone, copy );

				// Don't bring in undefined values
				} else if ( copy !== undefined ) {
					target[ name ] = copy;
				}
			}
		}
	}

	// Return the modified object
	return target;
};

jQuery.extend({
	noConflict: function( deep ) {
		if ( window.$ === jQuery ) {
			window.$ = _$;
		}

		if ( deep && window.jQuery === jQuery ) {
			window.jQuery = _jQuery;
		}

		return jQuery;
	},

	// Is the DOM ready to be used? Set to true once it occurs.
	isReady: false,

	// A counter to track how many items to wait for before
	// the ready event fires. See #6781
	readyWait: 1,

	// Hold (or release) the ready event
	holdReady: function( hold ) {
		if ( hold ) {
			jQuery.readyWait++;
		} else {
			jQuery.ready( true );
		}
	},

	// Handle when the DOM is ready
	ready: function( wait ) {
		// Either a released hold or an DOMready/load event and not yet ready
		if ( (wait === true && !--jQuery.readyWait) || (wait !== true && !jQuery.isReady) ) {
			// Make sure body exists, at least, in case IE gets a little overzealous (ticket #5443).
			if ( !document.body ) {
				return setTimeout( jQuery.ready, 1 );
			}

			// Remember that the DOM is ready
			jQuery.isReady = true;

			// If a normal DOM Ready event fired, decrement, and wait if need be
			if ( wait !== true && --jQuery.readyWait > 0 ) {
				return;
			}

			// If there are functions bound, to execute
			readyList.fireWith( document, [ jQuery ] );

			// Trigger any bound ready events
			if ( jQuery.fn.trigger ) {
				jQuery( document ).trigger( "ready" ).off( "ready" );
			}
		}
	},

	bindReady: function() {
		if ( readyList ) {
			return;
		}

		readyList = jQuery.Callbacks( "once memory" );

		// Catch cases where $(document).ready() is called after the
		// browser event has already occurred.
		if ( document.readyState === "complete" ) {
			// Handle it asynchronously to allow scripts the opportunity to delay ready
			return setTimeout( jQuery.ready, 1 );
		}

		// Mozilla, Opera and webkit nightlies currently support this event
		if ( document.addEventListener ) {
			// Use the handy event callback
			document.addEventListener( "DOMContentLoaded", DOMContentLoaded, false );

			// A fallback to window.onload, that will always work
			window.addEventListener( "load", jQuery.ready, false );

		// If IE event model is used
		} else if ( document.attachEvent ) {
			// ensure firing before onload,
			// maybe late but safe also for iframes
			document.attachEvent( "onreadystatechange", DOMContentLoaded );

			// A fallback to window.onload, that will always work
			window.attachEvent( "onload", jQuery.ready );

			// If IE and not a frame
			// continually check to see if the document is ready
			var toplevel = false;

			try {
				toplevel = window.frameElement == null;
			} catch(e) {}

			if ( document.documentElement.doScroll && toplevel ) {
				doScrollCheck();
			}
		}
	},

	// See test/unit/core.js for details concerning isFunction.
	// Since version 1.3, DOM methods and functions like alert
	// aren't supported. They return false on IE (#2968).
	isFunction: function( obj ) {
		return jQuery.type(obj) === "function";
	},

	isArray: Array.isArray || function( obj ) {
		return jQuery.type(obj) === "array";
	},

	// A crude way of determining if an object is a window
	isWindow: function( obj ) {
		return obj && typeof obj === "object" && "setInterval" in obj;
	},

	isNumeric: function( obj ) {
		return !isNaN( parseFloat(obj) ) && isFinite( obj );
	},

	type: function( obj ) {
		return obj == null ?
			String( obj ) :
			class2type[ toString.call(obj) ] || "object";
	},

	isPlainObject: function( obj ) {
		// Must be an Object.
		// Because of IE, we also have to check the presence of the constructor property.
		// Make sure that DOM nodes and window objects don't pass through, as well
		if ( !obj || jQuery.type(obj) !== "object" || obj.nodeType || jQuery.isWindow( obj ) ) {
			return false;
		}

		try {
			// Not own constructor property must be Object
			if ( obj.constructor &&
				!hasOwn.call(obj, "constructor") &&
				!hasOwn.call(obj.constructor.prototype, "isPrototypeOf") ) {
				return false;
			}
		} catch ( e ) {
			// IE8,9 Will throw exceptions on certain host objects #9897
			return false;
		}

		// Own properties are enumerated firstly, so to speed up,
		// if last one is own, then all properties are own.

		var key;
		for ( key in obj ) {}

		return key === undefined || hasOwn.call( obj, key );
	},

	isEmptyObject: function( obj ) {
		for ( var name in obj ) {
			return false;
		}
		return true;
	},

	error: function( msg ) {
		throw new Error( msg );
	},

	parseJSON: function( data ) {
		if ( typeof data !== "string" || !data ) {
			return null;
		}

		// Make sure leading/trailing whitespace is removed (IE can't handle it)
		data = jQuery.trim( data );

		// Attempt to parse using the native JSON parser first
		if ( window.JSON && window.JSON.parse ) {
			return window.JSON.parse( data );
		}

		// Make sure the incoming data is actual JSON
		// Logic borrowed from http://json.org/json2.js
		if ( rvalidchars.test( data.replace( rvalidescape, "@" )
			.replace( rvalidtokens, "]" )
			.replace( rvalidbraces, "")) ) {

			return ( new Function( "return " + data ) )();

		}
		jQuery.error( "Invalid JSON: " + data );
	},

	// Cross-browser xml parsing
	parseXML: function( data ) {
		var xml, tmp;
		try {
			if ( window.DOMParser ) { // Standard
				tmp = new DOMParser();
				xml = tmp.parseFromString( data , "text/xml" );
			} else { // IE
				xml = new ActiveXObject( "Microsoft.XMLDOM" );
				xml.async = "false";
				xml.loadXML( data );
			}
		} catch( e ) {
			xml = undefined;
		}
		if ( !xml || !xml.documentElement || xml.getElementsByTagName( "parsererror" ).length ) {
			jQuery.error( "Invalid XML: " + data );
		}
		return xml;
	},

	noop: function() {},

	// Evaluates a script in a global context
	// Workarounds based on findings by Jim Driscoll
	// http://weblogs.java.net/blog/driscoll/archive/2009/09/08/eval-javascript-global-context
	globalEval: function( data ) {
		if ( data && rnotwhite.test( data ) ) {
			// We use execScript on Internet Explorer
			// We use an anonymous function so that context is window
			// rather than jQuery in Firefox
			( window.execScript || function( data ) {
				window[ "eval" ].call( window, data );
			} )( data );
		}
	},

	// Convert dashed to camelCase; used by the css and data modules
	// Microsoft forgot to hump their vendor prefix (#9572)
	camelCase: function( string ) {
		return string.replace( rmsPrefix, "ms-" ).replace( rdashAlpha, fcamelCase );
	},

	nodeName: function( elem, name ) {
		return elem.nodeName && elem.nodeName.toUpperCase() === name.toUpperCase();
	},

	// args is for internal usage only
	each: function( object, callback, args ) {
		var name, i = 0,
			length = object.length,
			isObj = length === undefined || jQuery.isFunction( object );

		if ( args ) {
			if ( isObj ) {
				for ( name in object ) {
					if ( callback.apply( object[ name ], args ) === false ) {
						break;
					}
				}
			} else {
				for ( ; i < length; ) {
					if ( callback.apply( object[ i++ ], args ) === false ) {
						break;
					}
				}
			}

		// A special, fast, case for the most common use of each
		} else {
			if ( isObj ) {
				for ( name in object ) {
					if ( callback.call( object[ name ], name, object[ name ] ) === false ) {
						break;
					}
				}
			} else {
				for ( ; i < length; ) {
					if ( callback.call( object[ i ], i, object[ i++ ] ) === false ) {
						break;
					}
				}
			}
		}

		return object;
	},

	// Use native String.trim function wherever possible
	trim: trim ?
		function( text ) {
			return text == null ?
				"" :
				trim.call( text );
		} :

		// Otherwise use our own trimming functionality
		function( text ) {
			return text == null ?
				"" :
				text.toString().replace( trimLeft, "" ).replace( trimRight, "" );
		},

	// results is for internal usage only
	makeArray: function( array, results ) {
		var ret = results || [];

		if ( array != null ) {
			// The window, strings (and functions) also have 'length'
			// Tweaked logic slightly to handle Blackberry 4.7 RegExp issues #6930
			var type = jQuery.type( array );

			if ( array.length == null || type === "string" || type === "function" || type === "regexp" || jQuery.isWindow( array ) ) {
				push.call( ret, array );
			} else {
				jQuery.merge( ret, array );
			}
		}

		return ret;
	},

	inArray: function( elem, array, i ) {
		var len;

		if ( array ) {
			if ( indexOf ) {
				return indexOf.call( array, elem, i );
			}

			len = array.length;
			i = i ? i < 0 ? Math.max( 0, len + i ) : i : 0;

			for ( ; i < len; i++ ) {
				// Skip accessing in sparse arrays
				if ( i in array && array[ i ] === elem ) {
					return i;
				}
			}
		}

		return -1;
	},

	merge: function( first, second ) {
		var i = first.length,
			j = 0;

		if ( typeof second.length === "number" ) {
			for ( var l = second.length; j < l; j++ ) {
				first[ i++ ] = second[ j ];
			}

		} else {
			while ( second[j] !== undefined ) {
				first[ i++ ] = second[ j++ ];
			}
		}

		first.length = i;

		return first;
	},

	grep: function( elems, callback, inv ) {
		var ret = [], retVal;
		inv = !!inv;

		// Go through the array, only saving the items
		// that pass the validator function
		for ( var i = 0, length = elems.length; i < length; i++ ) {
			retVal = !!callback( elems[ i ], i );
			if ( inv !== retVal ) {
				ret.push( elems[ i ] );
			}
		}

		return ret;
	},

	// arg is for internal usage only
	map: function( elems, callback, arg ) {
		var value, key, ret = [],
			i = 0,
			length = elems.length,
			// jquery objects are treated as arrays
			isArray = elems instanceof jQuery || length !== undefined && typeof length === "number" && ( ( length > 0 && elems[ 0 ] && elems[ length -1 ] ) || length === 0 || jQuery.isArray( elems ) ) ;

		// Go through the array, translating each of the items to their
		if ( isArray ) {
			for ( ; i < length; i++ ) {
				value = callback( elems[ i ], i, arg );

				if ( value != null ) {
					ret[ ret.length ] = value;
				}
			}

		// Go through every key on the object,
		} else {
			for ( key in elems ) {
				value = callback( elems[ key ], key, arg );

				if ( value != null ) {
					ret[ ret.length ] = value;
				}
			}
		}

		// Flatten any nested arrays
		return ret.concat.apply( [], ret );
	},

	// A global GUID counter for objects
	guid: 1,

	// Bind a function to a context, optionally partially applying any
	// arguments.
	proxy: function( fn, context ) {
		if ( typeof context === "string" ) {
			var tmp = fn[ context ];
			context = fn;
			fn = tmp;
		}

		// Quick check to determine if target is callable, in the spec
		// this throws a TypeError, but we will just return undefined.
		if ( !jQuery.isFunction( fn ) ) {
			return undefined;
		}

		// Simulated bind
		var args = slice.call( arguments, 2 ),
			proxy = function() {
				return fn.apply( context, args.concat( slice.call( arguments ) ) );
			};

		// Set the guid of unique handler to the same of original handler, so it can be removed
		proxy.guid = fn.guid = fn.guid || proxy.guid || jQuery.guid++;

		return proxy;
	},

	// Mutifunctional method to get and set values to a collection
	// The value/s can optionally be executed if it's a function
	access: function( elems, key, value, exec, fn, pass ) {
		var length = elems.length;

		// Setting many attributes
		if ( typeof key === "object" ) {
			for ( var k in key ) {
				jQuery.access( elems, k, key[k], exec, fn, value );
			}
			return elems;
		}

		// Setting one attribute
		if ( value !== undefined ) {
			// Optionally, function values get executed if exec is true
			exec = !pass && exec && jQuery.isFunction(value);

			for ( var i = 0; i < length; i++ ) {
				fn( elems[i], key, exec ? value.call( elems[i], i, fn( elems[i], key ) ) : value, pass );
			}

			return elems;
		}

		// Getting an attribute
		return length ? fn( elems[0], key ) : undefined;
	},

	now: function() {
		return ( new Date() ).getTime();
	},

	// Use of jQuery.browser is frowned upon.
	// More details: http://docs.jquery.com/Utilities/jQuery.browser
	uaMatch: function( ua ) {
		ua = ua.toLowerCase();

		var match = rwebkit.exec( ua ) ||
			ropera.exec( ua ) ||
			rmsie.exec( ua ) ||
			ua.indexOf("compatible") < 0 && rmozilla.exec( ua ) ||
			[];

		return { browser: match[1] || "", version: match[2] || "0" };
	},

	sub: function() {
		function jQuerySub( selector, context ) {
			return new jQuerySub.fn.init( selector, context );
		}
		jQuery.extend( true, jQuerySub, this );
		jQuerySub.superclass = this;
		jQuerySub.fn = jQuerySub.prototype = this();
		jQuerySub.fn.constructor = jQuerySub;
		jQuerySub.sub = this.sub;
		jQuerySub.fn.init = function init( selector, context ) {
			if ( context && context instanceof jQuery && !(context instanceof jQuerySub) ) {
				context = jQuerySub( context );
			}

			return jQuery.fn.init.call( this, selector, context, rootjQuerySub );
		};
		jQuerySub.fn.init.prototype = jQuerySub.fn;
		var rootjQuerySub = jQuerySub(document);
		return jQuerySub;
	},

	browser: {}
});

// Populate the class2type map
jQuery.each("Boolean Number String Function Array Date RegExp Object".split(" "), function(i, name) {
	class2type[ "[object " + name + "]" ] = name.toLowerCase();
});

browserMatch = jQuery.uaMatch( userAgent );
if ( browserMatch.browser ) {
	jQuery.browser[ browserMatch.browser ] = true;
	jQuery.browser.version = browserMatch.version;
}

// Deprecated, use jQuery.browser.webkit instead
if ( jQuery.browser.webkit ) {
	jQuery.browser.safari = true;
}

// IE doesn't match non-breaking spaces with \s
if ( rnotwhite.test( "\xA0" ) ) {
	trimLeft = /^[\s\xA0]+/;
	trimRight = /[\s\xA0]+$/;
}

// All jQuery objects should point back to these
rootjQuery = jQuery(document);

// Cleanup functions for the document ready method
if ( document.addEventListener ) {
	DOMContentLoaded = function() {
		document.removeEventListener( "DOMContentLoaded", DOMContentLoaded, false );
		jQuery.ready();
	};

} else if ( document.attachEvent ) {
	DOMContentLoaded = function() {
		// Make sure body exists, at least, in case IE gets a little overzealous (ticket #5443).
		if ( document.readyState === "complete" ) {
			document.detachEvent( "onreadystatechange", DOMContentLoaded );
			jQuery.ready();
		}
	};
}

// The DOM ready check for Internet Explorer
function doScrollCheck() {
	if ( jQuery.isReady ) {
		return;
	}

	try {
		// If IE is used, use the trick by Diego Perini
		// http://javascript.nwbox.com/IEContentLoaded/
		document.documentElement.doScroll("left");
	} catch(e) {
		setTimeout( doScrollCheck, 1 );
		return;
	}

	// and execute any waiting functions
	jQuery.ready();
}

return jQuery;

})();


// String to Object flags format cache
var flagsCache = {};

// Convert String-formatted flags into Object-formatted ones and store in cache
function createFlags( flags ) {
	var object = flagsCache[ flags ] = {},
		i, length;
	flags = flags.split( /\s+/ );
	for ( i = 0, length = flags.length; i < length; i++ ) {
		object[ flags[i] ] = true;
	}
	return object;
}

/*
 * Create a callback list using the following parameters:
 *
 *	flags:	an optional list of space-separated flags that will change how
 *			the callback list behaves
 *
 * By default a callback list will act like an event callback list and can be
 * "fired" multiple times.
 *
 * Possible flags:
 *
 *	once:			will ensure the callback list can only be fired once (like a Deferred)
 *
 *	memory:			will keep track of previous values and will call any callback added
 *					after the list has been fired right away with the latest "memorized"
 *					values (like a Deferred)
 *
 *	unique:			will ensure a callback can only be added once (no duplicate in the list)
 *
 *	stopOnFalse:	interrupt callings when a callback returns false
 *
 */
jQuery.Callbacks = function( flags ) {

	// Convert flags from String-formatted to Object-formatted
	// (we check in cache first)
	flags = flags ? ( flagsCache[ flags ] || createFlags( flags ) ) : {};

	var // Actual callback list
		list = [],
		// Stack of fire calls for repeatable lists
		stack = [],
		// Last fire value (for non-forgettable lists)
		memory,
		// Flag to know if list is currently firing
		firing,
		// First callback to fire (used internally by add and fireWith)
		firingStart,
		// End of the loop when firing
		firingLength,
		// Index of currently firing callback (modified by remove if needed)
		firingIndex,
		// Add one or several callbacks to the list
		add = function( args ) {
			var i,
				length,
				elem,
				type,
				actual;
			for ( i = 0, length = args.length; i < length; i++ ) {
				elem = args[ i ];
				type = jQuery.type( elem );
				if ( type === "array" ) {
					// Inspect recursively
					add( elem );
				} else if ( type === "function" ) {
					// Add if not in unique mode and callback is not in
					if ( !flags.unique || !self.has( elem ) ) {
						list.push( elem );
					}
				}
			}
		},
		// Fire callbacks
		fire = function( context, args ) {
			args = args || [];
			memory = !flags.memory || [ context, args ];
			firing = true;
			firingIndex = firingStart || 0;
			firingStart = 0;
			firingLength = list.length;
			for ( ; list && firingIndex < firingLength; firingIndex++ ) {
				if ( list[ firingIndex ].apply( context, args ) === false && flags.stopOnFalse ) {
					memory = true; // Mark as halted
					break;
				}
			}
			firing = false;
			if ( list ) {
				if ( !flags.once ) {
					if ( stack && stack.length ) {
						memory = stack.shift();
						self.fireWith( memory[ 0 ], memory[ 1 ] );
					}
				} else if ( memory === true ) {
					self.disable();
				} else {
					list = [];
				}
			}
		},
		// Actual Callbacks object
		self = {
			// Add a callback or a collection of callbacks to the list
			add: function() {
				if ( list ) {
					var length = list.length;
					add( arguments );
					// Do we need to add the callbacks to the
					// current firing batch?
					if ( firing ) {
						firingLength = list.length;
					// With memory, if we're not firing then
					// we should call right away, unless previous
					// firing was halted (stopOnFalse)
					} else if ( memory && memory !== true ) {
						firingStart = length;
						fire( memory[ 0 ], memory[ 1 ] );
					}
				}
				return this;
			},
			// Remove a callback from the list
			remove: function() {
				if ( list ) {
					var args = arguments,
						argIndex = 0,
						argLength = args.length;
					for ( ; argIndex < argLength ; argIndex++ ) {
						for ( var i = 0; i < list.length; i++ ) {
							if ( args[ argIndex ] === list[ i ] ) {
								// Handle firingIndex and firingLength
								if ( firing ) {
									if ( i <= firingLength ) {
										firingLength--;
										if ( i <= firingIndex ) {
											firingIndex--;
										}
									}
								}
								// Remove the element
								list.splice( i--, 1 );
								// If we have some unicity property then
								// we only need to do this once
								if ( flags.unique ) {
									break;
								}
							}
						}
					}
				}
				return this;
			},
			// Control if a given callback is in the list
			has: function( fn ) {
				if ( list ) {
					var i = 0,
						length = list.length;
					for ( ; i < length; i++ ) {
						if ( fn === list[ i ] ) {
							return true;
						}
					}
				}
				return false;
			},
			// Remove all callbacks from the list
			empty: function() {
				list = [];
				return this;
			},
			// Have the list do nothing anymore
			disable: function() {
				list = stack = memory = undefined;
				return this;
			},
			// Is it disabled?
			disabled: function() {
				return !list;
			},
			// Lock the list in its current state
			lock: function() {
				stack = undefined;
				if ( !memory || memory === true ) {
					self.disable();
				}
				return this;
			},
			// Is it locked?
			locked: function() {
				return !stack;
			},
			// Call all callbacks with the given context and arguments
			fireWith: function( context, args ) {
				if ( stack ) {
					if ( firing ) {
						if ( !flags.once ) {
							stack.push( [ context, args ] );
						}
					} else if ( !( flags.once && memory ) ) {
						fire( context, args );
					}
				}
				return this;
			},
			// Call all the callbacks with the given arguments
			fire: function() {
				self.fireWith( this, arguments );
				return this;
			},
			// To know if the callbacks have already been called at least once
			fired: function() {
				return !!memory;
			}
		};

	return self;
};




var // Static reference to slice
	sliceDeferred = [].slice;

jQuery.extend({

	Deferred: function( func ) {
		var doneList = jQuery.Callbacks( "once memory" ),
			failList = jQuery.Callbacks( "once memory" ),
			progressList = jQuery.Callbacks( "memory" ),
			state = "pending",
			lists = {
				resolve: doneList,
				reject: failList,
				notify: progressList
			},
			promise = {
				done: doneList.add,
				fail: failList.add,
				progress: progressList.add,

				state: function() {
					return state;
				},

				// Deprecated
				isResolved: doneList.fired,
				isRejected: failList.fired,

				then: function( doneCallbacks, failCallbacks, progressCallbacks ) {
					deferred.done( doneCallbacks ).fail( failCallbacks ).progress( progressCallbacks );
					return this;
				},
				always: function() {
					deferred.done.apply( deferred, arguments ).fail.apply( deferred, arguments );
					return this;
				},
				pipe: function( fnDone, fnFail, fnProgress ) {
					return jQuery.Deferred(function( newDefer ) {
						jQuery.each( {
							done: [ fnDone, "resolve" ],
							fail: [ fnFail, "reject" ],
							progress: [ fnProgress, "notify" ]
						}, function( handler, data ) {
							var fn = data[ 0 ],
								action = data[ 1 ],
								returned;
							if ( jQuery.isFunction( fn ) ) {
								deferred[ handler ](function() {
									returned = fn.apply( this, arguments );
									if ( returned && jQuery.isFunction( returned.promise ) ) {
										returned.promise().then( newDefer.resolve, newDefer.reject, newDefer.notify );
									} else {
										newDefer[ action + "With" ]( this === deferred ? newDefer : this, [ returned ] );
									}
								});
							} else {
								deferred[ handler ]( newDefer[ action ] );
							}
						});
					}).promise();
				},
				// Get a promise for this deferred
				// If obj is provided, the promise aspect is added to the object
				promise: function( obj ) {
					if ( obj == null ) {
						obj = promise;
					} else {
						for ( var key in promise ) {
							obj[ key ] = promise[ key ];
						}
					}
					return obj;
				}
			},
			deferred = promise.promise({}),
			key;

		for ( key in lists ) {
			deferred[ key ] = lists[ key ].fire;
			deferred[ key + "With" ] = lists[ key ].fireWith;
		}

		// Handle state
		deferred.done( function() {
			state = "resolved";
		}, failList.disable, progressList.lock ).fail( function() {
			state = "rejected";
		}, doneList.disable, progressList.lock );

		// Call given func if any
		if ( func ) {
			func.call( deferred, deferred );
		}

		// All done!
		return deferred;
	},

	// Deferred helper
	when: function( firstParam ) {
		var args = sliceDeferred.call( arguments, 0 ),
			i = 0,
			length = args.length,
			pValues = new Array( length ),
			count = length,
			pCount = length,
			deferred = length <= 1 && firstParam && jQuery.isFunction( firstParam.promise ) ?
				firstParam :
				jQuery.Deferred(),
			promise = deferred.promise();
		function resolveFunc( i ) {
			return function( value ) {
				args[ i ] = arguments.length > 1 ? sliceDeferred.call( arguments, 0 ) : value;
				if ( !( --count ) ) {
					deferred.resolveWith( deferred, args );
				}
			};
		}
		function progressFunc( i ) {
			return function( value ) {
				pValues[ i ] = arguments.length > 1 ? sliceDeferred.call( arguments, 0 ) : value;
				deferred.notifyWith( promise, pValues );
			};
		}
		if ( length > 1 ) {
			for ( ; i < length; i++ ) {
				if ( args[ i ] && args[ i ].promise && jQuery.isFunction( args[ i ].promise ) ) {
					args[ i ].promise().then( resolveFunc(i), deferred.reject, progressFunc(i) );
				} else {
					--count;
				}
			}
			if ( !count ) {
				deferred.resolveWith( deferred, args );
			}
		} else if ( deferred !== firstParam ) {
			deferred.resolveWith( deferred, length ? [ firstParam ] : [] );
		}
		return promise;
	}
});




jQuery.support = (function() {

	var support,
		all,
		a,
		select,
		opt,
		input,
		marginDiv,
		fragment,
		tds,
		events,
		eventName,
		i,
		isSupported,
		div = document.createElement( "div" ),
		documentElement = document.documentElement;

	// Preliminary tests
	div.setAttribute("className", "t");
	div.innerHTML = "   <link/><table></table><a href='/a' style='top:1px;float:left;opacity:.55;'>a</a><input type='checkbox'/>";

	all = div.getElementsByTagName( "*" );
	a = div.getElementsByTagName( "a" )[ 0 ];

	// Can't get basic test support
	if ( !all || !all.length || !a ) {
		return {};
	}

	// First batch of supports tests
	select = document.createElement( "select" );
	opt = select.appendChild( document.createElement("option") );
	input = div.getElementsByTagName( "input" )[ 0 ];

	support = {
		// IE strips leading whitespace when .innerHTML is used
		leadingWhitespace: ( div.firstChild.nodeType === 3 ),

		// Make sure that tbody elements aren't automatically inserted
		// IE will insert them into empty tables
		tbody: !div.getElementsByTagName("tbody").length,

		// Make sure that link elements get serialized correctly by innerHTML
		// This requires a wrapper element in IE
		htmlSerialize: !!div.getElementsByTagName("link").length,

		// Get the style information from getAttribute
		// (IE uses .cssText instead)
		style: /top/.test( a.getAttribute("style") ),

		// Make sure that URLs aren't manipulated
		// (IE normalizes it by default)
		hrefNormalized: ( a.getAttribute("href") === "/a" ),

		// Make sure that element opacity exists
		// (IE uses filter instead)
		// Use a regex to work around a WebKit issue. See #5145
		opacity: /^0.55/.test( a.style.opacity ),

		// Verify style float existence
		// (IE uses styleFloat instead of cssFloat)
		cssFloat: !!a.style.cssFloat,

		// Make sure that if no value is specified for a checkbox
		// that it defaults to "on".
		// (WebKit defaults to "" instead)
		checkOn: ( input.value === "on" ),

		// Make sure that a selected-by-default option has a working selected property.
		// (WebKit defaults to false instead of true, IE too, if it's in an optgroup)
		optSelected: opt.selected,

		// Test setAttribute on camelCase class. If it works, we need attrFixes when doing get/setAttribute (ie6/7)
		getSetAttribute: div.className !== "t",

		// Tests for enctype support on a form(#6743)
		enctype: !!document.createElement("form").enctype,

		// Makes sure cloning an html5 element does not cause problems
		// Where outerHTML is undefined, this still works
		html5Clone: document.createElement("nav").cloneNode( true ).outerHTML !== "<:nav></:nav>",

		// Will be defined later
		submitBubbles: true,
		changeBubbles: true,
		focusinBubbles: false,
		deleteExpando: true,
		noCloneEvent: true,
		inlineBlockNeedsLayout: false,
		shrinkWrapBlocks: false,
		reliableMarginRight: true
	};

	// Make sure checked status is properly cloned
	input.checked = true;
	support.noCloneChecked = input.cloneNode( true ).checked;

	// Make sure that the options inside disabled selects aren't marked as disabled
	// (WebKit marks them as disabled)
	select.disabled = true;
	support.optDisabled = !opt.disabled;

	// Test to see if it's possible to delete an expando from an element
	// Fails in Internet Explorer
	try {
		delete div.test;
	} catch( e ) {
		support.deleteExpando = false;
	}

	if ( !div.addEventListener && div.attachEvent && div.fireEvent ) {
		div.attachEvent( "onclick", function() {
			// Cloning a node shouldn't copy over any
			// bound event handlers (IE does this)
			support.noCloneEvent = false;
		});
		div.cloneNode( true ).fireEvent( "onclick" );
	}

	// Check if a radio maintains its value
	// after being appended to the DOM
	input = document.createElement("input");
	input.value = "t";
	input.setAttribute("type", "radio");
	support.radioValue = input.value === "t";

	input.setAttribute("checked", "checked");
	div.appendChild( input );
	fragment = document.createDocumentFragment();
	fragment.appendChild( div.lastChild );

	// WebKit doesn't clone checked state correctly in fragments
	support.checkClone = fragment.cloneNode( true ).cloneNode( true ).lastChild.checked;

	// Check if a disconnected checkbox will retain its checked
	// value of true after appended to the DOM (IE6/7)
	support.appendChecked = input.checked;

	fragment.removeChild( input );
	fragment.appendChild( div );

	div.innerHTML = "";

	// Check if div with explicit width and no margin-right incorrectly
	// gets computed margin-right based on width of container. For more
	// info see bug #3333
	// Fails in WebKit before Feb 2011 nightlies
	// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
	if ( window.getComputedStyle ) {
		marginDiv = document.createElement( "div" );
		marginDiv.style.width = "0";
		marginDiv.style.marginRight = "0";
		div.style.width = "2px";
		div.appendChild( marginDiv );
		support.reliableMarginRight =
			( parseInt( ( window.getComputedStyle( marginDiv, null ) || { marginRight: 0 } ).marginRight, 10 ) || 0 ) === 0;
	}

	// Technique from Juriy Zaytsev
	// http://perfectionkills.com/detecting-event-support-without-browser-sniffing/
	// We only care about the case where non-standard event systems
	// are used, namely in IE. Short-circuiting here helps us to
	// avoid an eval call (in setAttribute) which can cause CSP
	// to go haywire. See: https://developer.mozilla.org/en/Security/CSP
	if ( div.attachEvent ) {
		for( i in {
			submit: 1,
			change: 1,
			focusin: 1
		}) {
			eventName = "on" + i;
			isSupported = ( eventName in div );
			if ( !isSupported ) {
				div.setAttribute( eventName, "return;" );
				isSupported = ( typeof div[ eventName ] === "function" );
			}
			support[ i + "Bubbles" ] = isSupported;
		}
	}

	fragment.removeChild( div );

	// Null elements to avoid leaks in IE
	fragment = select = opt = marginDiv = div = input = null;

	// Run tests that need a body at doc ready
	jQuery(function() {
		var container, outer, inner, table, td, offsetSupport,
			conMarginTop, ptlm, vb, style, html,
			body = document.getElementsByTagName("body")[0];

		if ( !body ) {
			// Return for frameset docs that don't have a body
			return;
		}

		conMarginTop = 1;
		ptlm = "position:absolute;top:0;left:0;width:1px;height:1px;margin:0;";
		vb = "visibility:hidden;border:0;";
		style = "style='" + ptlm + "border:5px solid #000;padding:0;'";
		html = "<div " + style + "><div></div></div>" +
			"<table " + style + " cellpadding='0' cellspacing='0'>" +
			"<tr><td></td></tr></table>";

		container = document.createElement("div");
		container.style.cssText = vb + "width:0;height:0;position:static;top:0;margin-top:" + conMarginTop + "px";
		body.insertBefore( container, body.firstChild );

		// Construct the test element
		div = document.createElement("div");
		container.appendChild( div );

		// Check if table cells still have offsetWidth/Height when they are set
		// to display:none and there are still other visible table cells in a
		// table row; if so, offsetWidth/Height are not reliable for use when
		// determining if an element has been hidden directly using
		// display:none (it is still safe to use offsets if a parent element is
		// hidden; don safety goggles and see bug #4512 for more information).
		// (only IE 8 fails this test)
		div.innerHTML = "<table><tr><td style='padding:0;border:0;display:none'></td><td>t</td></tr></table>";
		tds = div.getElementsByTagName( "td" );
		isSupported = ( tds[ 0 ].offsetHeight === 0 );

		tds[ 0 ].style.display = "";
		tds[ 1 ].style.display = "none";

		// Check if empty table cells still have offsetWidth/Height
		// (IE <= 8 fail this test)
		support.reliableHiddenOffsets = isSupported && ( tds[ 0 ].offsetHeight === 0 );

		// Figure out if the W3C box model works as expected
		div.innerHTML = "";
		div.style.width = div.style.paddingLeft = "1px";
		jQuery.boxModel = support.boxModel = div.offsetWidth === 2;

		if ( typeof div.style.zoom !== "undefined" ) {
			// Check if natively block-level elements act like inline-block
			// elements when setting their display to 'inline' and giving
			// them layout
			// (IE < 8 does this)
			div.style.display = "inline";
			div.style.zoom = 1;
			support.inlineBlockNeedsLayout = ( div.offsetWidth === 2 );

			// Check if elements with layout shrink-wrap their children
			// (IE 6 does this)
			div.style.display = "";
			div.innerHTML = "<div style='width:4px;'></div>";
			support.shrinkWrapBlocks = ( div.offsetWidth !== 2 );
		}

		div.style.cssText = ptlm + vb;
		div.innerHTML = html;

		outer = div.firstChild;
		inner = outer.firstChild;
		td = outer.nextSibling.firstChild.firstChild;

		offsetSupport = {
			doesNotAddBorder: ( inner.offsetTop !== 5 ),
			doesAddBorderForTableAndCells: ( td.offsetTop === 5 )
		};

		inner.style.position = "fixed";
		inner.style.top = "20px";

		// safari subtracts parent border width here which is 5px
		offsetSupport.fixedPosition = ( inner.offsetTop === 20 || inner.offsetTop === 15 );
		inner.style.position = inner.style.top = "";

		outer.style.overflow = "hidden";
		outer.style.position = "relative";

		offsetSupport.subtractsBorderForOverflowNotVisible = ( inner.offsetTop === -5 );
		offsetSupport.doesNotIncludeMarginInBodyOffset = ( body.offsetTop !== conMarginTop );

		body.removeChild( container );
		div  = container = null;

		jQuery.extend( support, offsetSupport );
	});

	return support;
})();




var rbrace = /^(?:\{.*\}|\[.*\])$/,
	rmultiDash = /([A-Z])/g;

jQuery.extend({
	cache: {},

	// Please use with caution
	uuid: 0,

	// Unique for each copy of jQuery on the page
	// Non-digits removed to match rinlinejQuery
	expando: "jQuery" + ( jQuery.fn.jquery + Math.random() ).replace( /\D/g, "" ),

	// The following elements throw uncatchable exceptions if you
	// attempt to add expando properties to them.
	noData: {
		"embed": true,
		// Ban all objects except for Flash (which handle expandos)
		"object": "clsid:D27CDB6E-AE6D-11cf-96B8-444553540000",
		"applet": true
	},

	hasData: function( elem ) {
		elem = elem.nodeType ? jQuery.cache[ elem[jQuery.expando] ] : elem[ jQuery.expando ];
		return !!elem && !isEmptyDataObject( elem );
	},

	data: function( elem, name, data, pvt /* Internal Use Only */ ) {
		if ( !jQuery.acceptData( elem ) ) {
			return;
		}

		var privateCache, thisCache, ret,
			internalKey = jQuery.expando,
			getByName = typeof name === "string",

			// We have to handle DOM nodes and JS objects differently because IE6-7
			// can't GC object references properly across the DOM-JS boundary
			isNode = elem.nodeType,

			// Only DOM nodes need the global jQuery cache; JS object data is
			// attached directly to the object so GC can occur automatically
			cache = isNode ? jQuery.cache : elem,

			// Only defining an ID for JS objects if its cache already exists allows
			// the code to shortcut on the same path as a DOM node with no cache
			id = isNode ? elem[ internalKey ] : elem[ internalKey ] && internalKey,
			isEvents = name === "events";

		// Avoid doing any more work than we need to when trying to get data on an
		// object that has no data at all
		if ( (!id || !cache[id] || (!isEvents && !pvt && !cache[id].data)) && getByName && data === undefined ) {
			return;
		}

		if ( !id ) {
			// Only DOM nodes need a new unique ID for each element since their data
			// ends up in the global cache
			if ( isNode ) {
				elem[ internalKey ] = id = ++jQuery.uuid;
			} else {
				id = internalKey;
			}
		}

		if ( !cache[ id ] ) {
			cache[ id ] = {};

			// Avoids exposing jQuery metadata on plain JS objects when the object
			// is serialized using JSON.stringify
			if ( !isNode ) {
				cache[ id ].toJSON = jQuery.noop;
			}
		}

		// An object can be passed to jQuery.data instead of a key/value pair; this gets
		// shallow copied over onto the existing cache
		if ( typeof name === "object" || typeof name === "function" ) {
			if ( pvt ) {
				cache[ id ] = jQuery.extend( cache[ id ], name );
			} else {
				cache[ id ].data = jQuery.extend( cache[ id ].data, name );
			}
		}

		privateCache = thisCache = cache[ id ];

		// jQuery data() is stored in a separate object inside the object's internal data
		// cache in order to avoid key collisions between internal data and user-defined
		// data.
		if ( !pvt ) {
			if ( !thisCache.data ) {
				thisCache.data = {};
			}

			thisCache = thisCache.data;
		}

		if ( data !== undefined ) {
			thisCache[ jQuery.camelCase( name ) ] = data;
		}

		// Users should not attempt to inspect the internal events object using jQuery.data,
		// it is undocumented and subject to change. But does anyone listen? No.
		if ( isEvents && !thisCache[ name ] ) {
			return privateCache.events;
		}

		// Check for both converted-to-camel and non-converted data property names
		// If a data property was specified
		if ( getByName ) {

			// First Try to find as-is property data
			ret = thisCache[ name ];

			// Test for null|undefined property data
			if ( ret == null ) {

				// Try to find the camelCased property
				ret = thisCache[ jQuery.camelCase( name ) ];
			}
		} else {
			ret = thisCache;
		}

		return ret;
	},

	removeData: function( elem, name, pvt /* Internal Use Only */ ) {
		if ( !jQuery.acceptData( elem ) ) {
			return;
		}

		var thisCache, i, l,

			// Reference to internal data cache key
			internalKey = jQuery.expando,

			isNode = elem.nodeType,

			// See jQuery.data for more information
			cache = isNode ? jQuery.cache : elem,

			// See jQuery.data for more information
			id = isNode ? elem[ internalKey ] : internalKey;

		// If there is already no cache entry for this object, there is no
		// purpose in continuing
		if ( !cache[ id ] ) {
			return;
		}

		if ( name ) {

			thisCache = pvt ? cache[ id ] : cache[ id ].data;

			if ( thisCache ) {

				// Support array or space separated string names for data keys
				if ( !jQuery.isArray( name ) ) {

					// try the string as a key before any manipulation
					if ( name in thisCache ) {
						name = [ name ];
					} else {

						// split the camel cased version by spaces unless a key with the spaces exists
						name = jQuery.camelCase( name );
						if ( name in thisCache ) {
							name = [ name ];
						} else {
							name = name.split( " " );
						}
					}
				}

				for ( i = 0, l = name.length; i < l; i++ ) {
					delete thisCache[ name[i] ];
				}

				// If there is no data left in the cache, we want to continue
				// and let the cache object itself get destroyed
				if ( !( pvt ? isEmptyDataObject : jQuery.isEmptyObject )( thisCache ) ) {
					return;
				}
			}
		}

		// See jQuery.data for more information
		if ( !pvt ) {
			delete cache[ id ].data;

			// Don't destroy the parent cache unless the internal data object
			// had been the only thing left in it
			if ( !isEmptyDataObject(cache[ id ]) ) {
				return;
			}
		}

		// Browsers that fail expando deletion also refuse to delete expandos on
		// the window, but it will allow it on all other JS objects; other browsers
		// don't care
		// Ensure that `cache` is not a window object #10080
		if ( jQuery.support.deleteExpando || !cache.setInterval ) {
			delete cache[ id ];
		} else {
			cache[ id ] = null;
		}

		// We destroyed the cache and need to eliminate the expando on the node to avoid
		// false lookups in the cache for entries that no longer exist
		if ( isNode ) {
			// IE does not allow us to delete expando properties from nodes,
			// nor does it have a removeAttribute function on Document nodes;
			// we must handle all of these cases
			if ( jQuery.support.deleteExpando ) {
				delete elem[ internalKey ];
			} else if ( elem.removeAttribute ) {
				elem.removeAttribute( internalKey );
			} else {
				elem[ internalKey ] = null;
			}
		}
	},

	// For internal use only.
	_data: function( elem, name, data ) {
		return jQuery.data( elem, name, data, true );
	},

	// A method for determining if a DOM node can handle the data expando
	acceptData: function( elem ) {
		if ( elem.nodeName ) {
			var match = jQuery.noData[ elem.nodeName.toLowerCase() ];

			if ( match ) {
				return !(match === true || elem.getAttribute("classid") !== match);
			}
		}

		return true;
	}
});

jQuery.fn.extend({
	data: function( key, value ) {
		var parts, attr, name,
			data = null;

		if ( typeof key === "undefined" ) {
			if ( this.length ) {
				data = jQuery.data( this[0] );

				if ( this[0].nodeType === 1 && !jQuery._data( this[0], "parsedAttrs" ) ) {
					attr = this[0].attributes;
					for ( var i = 0, l = attr.length; i < l; i++ ) {
						name = attr[i].name;

						if ( name.indexOf( "data-" ) === 0 ) {
							name = jQuery.camelCase( name.substring(5) );

							dataAttr( this[0], name, data[ name ] );
						}
					}
					jQuery._data( this[0], "parsedAttrs", true );
				}
			}

			return data;

		} else if ( typeof key === "object" ) {
			return this.each(function() {
				jQuery.data( this, key );
			});
		}

		parts = key.split(".");
		parts[1] = parts[1] ? "." + parts[1] : "";

		if ( value === undefined ) {
			data = this.triggerHandler("getData" + parts[1] + "!", [parts[0]]);

			// Try to fetch any internally stored data first
			if ( data === undefined && this.length ) {
				data = jQuery.data( this[0], key );
				data = dataAttr( this[0], key, data );
			}

			return data === undefined && parts[1] ?
				this.data( parts[0] ) :
				data;

		} else {
			return this.each(function() {
				var self = jQuery( this ),
					args = [ parts[0], value ];

				self.triggerHandler( "setData" + parts[1] + "!", args );
				jQuery.data( this, key, value );
				self.triggerHandler( "changeData" + parts[1] + "!", args );
			});
		}
	},

	removeData: function( key ) {
		return this.each(function() {
			jQuery.removeData( this, key );
		});
	}
});

function dataAttr( elem, key, data ) {
	// If nothing was found internally, try to fetch any
	// data from the HTML5 data-* attribute
	if ( data === undefined && elem.nodeType === 1 ) {

		var name = "data-" + key.replace( rmultiDash, "-$1" ).toLowerCase();

		data = elem.getAttribute( name );

		if ( typeof data === "string" ) {
			try {
				data = data === "true" ? true :
				data === "false" ? false :
				data === "null" ? null :
				jQuery.isNumeric( data ) ? parseFloat( data ) :
					rbrace.test( data ) ? jQuery.parseJSON( data ) :
					data;
			} catch( e ) {}

			// Make sure we set the data so it isn't changed later
			jQuery.data( elem, key, data );

		} else {
			data = undefined;
		}
	}

	return data;
}

// checks a cache object for emptiness
function isEmptyDataObject( obj ) {
	for ( var name in obj ) {

		// if the public data object is empty, the private is still empty
		if ( name === "data" && jQuery.isEmptyObject( obj[name] ) ) {
			continue;
		}
		if ( name !== "toJSON" ) {
			return false;
		}
	}

	return true;
}




function handleQueueMarkDefer( elem, type, src ) {
	var deferDataKey = type + "defer",
		queueDataKey = type + "queue",
		markDataKey = type + "mark",
		defer = jQuery._data( elem, deferDataKey );
	if ( defer &&
		( src === "queue" || !jQuery._data(elem, queueDataKey) ) &&
		( src === "mark" || !jQuery._data(elem, markDataKey) ) ) {
		// Give room for hard-coded callbacks to fire first
		// and eventually mark/queue something else on the element
		setTimeout( function() {
			if ( !jQuery._data( elem, queueDataKey ) &&
				!jQuery._data( elem, markDataKey ) ) {
				jQuery.removeData( elem, deferDataKey, true );
				defer.fire();
			}
		}, 0 );
	}
}

jQuery.extend({

	_mark: function( elem, type ) {
		if ( elem ) {
			type = ( type || "fx" ) + "mark";
			jQuery._data( elem, type, (jQuery._data( elem, type ) || 0) + 1 );
		}
	},

	_unmark: function( force, elem, type ) {
		if ( force !== true ) {
			type = elem;
			elem = force;
			force = false;
		}
		if ( elem ) {
			type = type || "fx";
			var key = type + "mark",
				count = force ? 0 : ( (jQuery._data( elem, key ) || 1) - 1 );
			if ( count ) {
				jQuery._data( elem, key, count );
			} else {
				jQuery.removeData( elem, key, true );
				handleQueueMarkDefer( elem, type, "mark" );
			}
		}
	},

	queue: function( elem, type, data ) {
		var q;
		if ( elem ) {
			type = ( type || "fx" ) + "queue";
			q = jQuery._data( elem, type );

			// Speed up dequeue by getting out quickly if this is just a lookup
			if ( data ) {
				if ( !q || jQuery.isArray(data) ) {
					q = jQuery._data( elem, type, jQuery.makeArray(data) );
				} else {
					q.push( data );
				}
			}
			return q || [];
		}
	},

	dequeue: function( elem, type ) {
		type = type || "fx";

		var queue = jQuery.queue( elem, type ),
			fn = queue.shift(),
			hooks = {};

		// If the fx queue is dequeued, always remove the progress sentinel
		if ( fn === "inprogress" ) {
			fn = queue.shift();
		}

		if ( fn ) {
			// Add a progress sentinel to prevent the fx queue from being
			// automatically dequeued
			if ( type === "fx" ) {
				queue.unshift( "inprogress" );
			}

			jQuery._data( elem, type + ".run", hooks );
			fn.call( elem, function() {
				jQuery.dequeue( elem, type );
			}, hooks );
		}

		if ( !queue.length ) {
			jQuery.removeData( elem, type + "queue " + type + ".run", true );
			handleQueueMarkDefer( elem, type, "queue" );
		}
	}
});

jQuery.fn.extend({
	queue: function( type, data ) {
		if ( typeof type !== "string" ) {
			data = type;
			type = "fx";
		}

		if ( data === undefined ) {
			return jQuery.queue( this[0], type );
		}
		return this.each(function() {
			var queue = jQuery.queue( this, type, data );

			if ( type === "fx" && queue[0] !== "inprogress" ) {
				jQuery.dequeue( this, type );
			}
		});
	},
	dequeue: function( type ) {
		return this.each(function() {
			jQuery.dequeue( this, type );
		});
	},
	// Based off of the plugin by Clint Helfers, with permission.
	// http://blindsignals.com/index.php/2009/07/jquery-delay/
	delay: function( time, type ) {
		time = jQuery.fx ? jQuery.fx.speeds[ time ] || time : time;
		type = type || "fx";

		return this.queue( type, function( next, hooks ) {
			var timeout = setTimeout( next, time );
			hooks.stop = function() {
				clearTimeout( timeout );
			};
		});
	},
	clearQueue: function( type ) {
		return this.queue( type || "fx", [] );
	},
	// Get a promise resolved when queues of a certain type
	// are emptied (fx is the type by default)
	promise: function( type, object ) {
		if ( typeof type !== "string" ) {
			object = type;
			type = undefined;
		}
		type = type || "fx";
		var defer = jQuery.Deferred(),
			elements = this,
			i = elements.length,
			count = 1,
			deferDataKey = type + "defer",
			queueDataKey = type + "queue",
			markDataKey = type + "mark",
			tmp;
		function resolve() {
			if ( !( --count ) ) {
				defer.resolveWith( elements, [ elements ] );
			}
		}
		while( i-- ) {
			if (( tmp = jQuery.data( elements[ i ], deferDataKey, undefined, true ) ||
					( jQuery.data( elements[ i ], queueDataKey, undefined, true ) ||
						jQuery.data( elements[ i ], markDataKey, undefined, true ) ) &&
					jQuery.data( elements[ i ], deferDataKey, jQuery.Callbacks( "once memory" ), true ) )) {
				count++;
				tmp.add( resolve );
			}
		}
		resolve();
		return defer.promise();
	}
});




var rclass = /[\n\t\r]/g,
	rspace = /\s+/,
	rreturn = /\r/g,
	rtype = /^(?:button|input)$/i,
	rfocusable = /^(?:button|input|object|select|textarea)$/i,
	rclickable = /^a(?:rea)?$/i,
	rboolean = /^(?:autofocus|autoplay|async|checked|controls|defer|disabled|hidden|loop|multiple|open|readonly|required|scoped|selected)$/i,
	getSetAttribute = jQuery.support.getSetAttribute,
	nodeHook, boolHook, fixSpecified;

jQuery.fn.extend({
	attr: function( name, value ) {
		return jQuery.access( this, name, value, true, jQuery.attr );
	},

	removeAttr: function( name ) {
		return this.each(function() {
			jQuery.removeAttr( this, name );
		});
	},

	prop: function( name, value ) {
		return jQuery.access( this, name, value, true, jQuery.prop );
	},

	removeProp: function( name ) {
		name = jQuery.propFix[ name ] || name;
		return this.each(function() {
			// try/catch handles cases where IE balks (such as removing a property on window)
			try {
				this[ name ] = undefined;
				delete this[ name ];
			} catch( e ) {}
		});
	},

	addClass: function( value ) {
		var classNames, i, l, elem,
			setClass, c, cl;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).addClass( value.call(this, j, this.className) );
			});
		}

		if ( value && typeof value === "string" ) {
			classNames = value.split( rspace );

			for ( i = 0, l = this.length; i < l; i++ ) {
				elem = this[ i ];

				if ( elem.nodeType === 1 ) {
					if ( !elem.className && classNames.length === 1 ) {
						elem.className = value;

					} else {
						setClass = " " + elem.className + " ";

						for ( c = 0, cl = classNames.length; c < cl; c++ ) {
							if ( !~setClass.indexOf( " " + classNames[ c ] + " " ) ) {
								setClass += classNames[ c ] + " ";
							}
						}
						elem.className = jQuery.trim( setClass );
					}
				}
			}
		}

		return this;
	},

	removeClass: function( value ) {
		var classNames, i, l, elem, className, c, cl;

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( j ) {
				jQuery( this ).removeClass( value.call(this, j, this.className) );
			});
		}

		if ( (value && typeof value === "string") || value === undefined ) {
			classNames = ( value || "" ).split( rspace );

			for ( i = 0, l = this.length; i < l; i++ ) {
				elem = this[ i ];

				if ( elem.nodeType === 1 && elem.className ) {
					if ( value ) {
						className = (" " + elem.className + " ").replace( rclass, " " );
						for ( c = 0, cl = classNames.length; c < cl; c++ ) {
							className = className.replace(" " + classNames[ c ] + " ", " ");
						}
						elem.className = jQuery.trim( className );

					} else {
						elem.className = "";
					}
				}
			}
		}

		return this;
	},

	toggleClass: function( value, stateVal ) {
		var type = typeof value,
			isBool = typeof stateVal === "boolean";

		if ( jQuery.isFunction( value ) ) {
			return this.each(function( i ) {
				jQuery( this ).toggleClass( value.call(this, i, this.className, stateVal), stateVal );
			});
		}

		return this.each(function() {
			if ( type === "string" ) {
				// toggle individual class names
				var className,
					i = 0,
					self = jQuery( this ),
					state = stateVal,
					classNames = value.split( rspace );

				while ( (className = classNames[ i++ ]) ) {
					// check each className given, space seperated list
					state = isBool ? state : !self.hasClass( className );
					self[ state ? "addClass" : "removeClass" ]( className );
				}

			} else if ( type === "undefined" || type === "boolean" ) {
				if ( this.className ) {
					// store className if set
					jQuery._data( this, "__className__", this.className );
				}

				// toggle whole className
				this.className = this.className || value === false ? "" : jQuery._data( this, "__className__" ) || "";
			}
		});
	},

	hasClass: function( selector ) {
		var className = " " + selector + " ",
			i = 0,
			l = this.length;
		for ( ; i < l; i++ ) {
			if ( this[i].nodeType === 1 && (" " + this[i].className + " ").replace(rclass, " ").indexOf( className ) > -1 ) {
				return true;
			}
		}

		return false;
	},

	val: function( value ) {
		var hooks, ret, isFunction,
			elem = this[0];

		if ( !arguments.length ) {
			if ( elem ) {
				hooks = jQuery.valHooks[ elem.nodeName.toLowerCase() ] || jQuery.valHooks[ elem.type ];

				if ( hooks && "get" in hooks && (ret = hooks.get( elem, "value" )) !== undefined ) {
					return ret;
				}

				ret = elem.value;

				return typeof ret === "string" ?
					// handle most common string cases
					ret.replace(rreturn, "") :
					// handle cases where value is null/undef or number
					ret == null ? "" : ret;
			}

			return;
		}

		isFunction = jQuery.isFunction( value );

		return this.each(function( i ) {
			var self = jQuery(this), val;

			if ( this.nodeType !== 1 ) {
				return;
			}

			if ( isFunction ) {
				val = value.call( this, i, self.val() );
			} else {
				val = value;
			}

			// Treat null/undefined as ""; convert numbers to string
			if ( val == null ) {
				val = "";
			} else if ( typeof val === "number" ) {
				val += "";
			} else if ( jQuery.isArray( val ) ) {
				val = jQuery.map(val, function ( value ) {
					return value == null ? "" : value + "";
				});
			}

			hooks = jQuery.valHooks[ this.nodeName.toLowerCase() ] || jQuery.valHooks[ this.type ];

			// If set returns undefined, fall back to normal setting
			if ( !hooks || !("set" in hooks) || hooks.set( this, val, "value" ) === undefined ) {
				this.value = val;
			}
		});
	}
});

jQuery.extend({
	valHooks: {
		option: {
			get: function( elem ) {
				// attributes.value is undefined in Blackberry 4.7 but
				// uses .value. See #6932
				var val = elem.attributes.value;
				return !val || val.specified ? elem.value : elem.text;
			}
		},
		select: {
			get: function( elem ) {
				var value, i, max, option,
					index = elem.selectedIndex,
					values = [],
					options = elem.options,
					one = elem.type === "select-one";

				// Nothing was selected
				if ( index < 0 ) {
					return null;
				}

				// Loop through all the selected options
				i = one ? index : 0;
				max = one ? index + 1 : options.length;
				for ( ; i < max; i++ ) {
					option = options[ i ];

					// Don't return options that are disabled or in a disabled optgroup
					if ( option.selected && (jQuery.support.optDisabled ? !option.disabled : option.getAttribute("disabled") === null) &&
							(!option.parentNode.disabled || !jQuery.nodeName( option.parentNode, "optgroup" )) ) {

						// Get the specific value for the option
						value = jQuery( option ).val();

						// We don't need an array for one selects
						if ( one ) {
							return value;
						}

						// Multi-Selects return an array
						values.push( value );
					}
				}

				// Fixes Bug #2551 -- select.val() broken in IE after form.reset()
				if ( one && !values.length && options.length ) {
					return jQuery( options[ index ] ).val();
				}

				return values;
			},

			set: function( elem, value ) {
				var values = jQuery.makeArray( value );

				jQuery(elem).find("option").each(function() {
					this.selected = jQuery.inArray( jQuery(this).val(), values ) >= 0;
				});

				if ( !values.length ) {
					elem.selectedIndex = -1;
				}
				return values;
			}
		}
	},

	attrFn: {
		val: true,
		css: true,
		html: true,
		text: true,
		data: true,
		width: true,
		height: true,
		offset: true
	},

	attr: function( elem, name, value, pass ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// don't get/set attributes on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		if ( pass && name in jQuery.attrFn ) {
			return jQuery( elem )[ name ]( value );
		}

		// Fallback to prop when attributes are not supported
		if ( typeof elem.getAttribute === "undefined" ) {
			return jQuery.prop( elem, name, value );
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		// All attributes are lowercase
		// Grab necessary hook if one is defined
		if ( notxml ) {
			name = name.toLowerCase();
			hooks = jQuery.attrHooks[ name ] || ( rboolean.test( name ) ? boolHook : nodeHook );
		}

		if ( value !== undefined ) {

			if ( value === null ) {
				jQuery.removeAttr( elem, name );
				return;

			} else if ( hooks && "set" in hooks && notxml && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				elem.setAttribute( name, "" + value );
				return value;
			}

		} else if ( hooks && "get" in hooks && notxml && (ret = hooks.get( elem, name )) !== null ) {
			return ret;

		} else {

			ret = elem.getAttribute( name );

			// Non-existent attributes return null, we normalize to undefined
			return ret === null ?
				undefined :
				ret;
		}
	},

	removeAttr: function( elem, value ) {
		var propName, attrNames, name, l,
			i = 0;

		if ( value && elem.nodeType === 1 ) {
			attrNames = value.toLowerCase().split( rspace );
			l = attrNames.length;

			for ( ; i < l; i++ ) {
				name = attrNames[ i ];

				if ( name ) {
					propName = jQuery.propFix[ name ] || name;

					// See #9699 for explanation of this approach (setting first, then removal)
					jQuery.attr( elem, name, "" );
					elem.removeAttribute( getSetAttribute ? name : propName );

					// Set corresponding property to false for boolean attributes
					if ( rboolean.test( name ) && propName in elem ) {
						elem[ propName ] = false;
					}
				}
			}
		}
	},

	attrHooks: {
		type: {
			set: function( elem, value ) {
				// We can't allow the type property to be changed (since it causes problems in IE)
				if ( rtype.test( elem.nodeName ) && elem.parentNode ) {
					jQuery.error( "type property can't be changed" );
				} else if ( !jQuery.support.radioValue && value === "radio" && jQuery.nodeName(elem, "input") ) {
					// Setting the type on a radio button after the value resets the value in IE6-9
					// Reset value to it's default in case type is set after value
					// This is for element creation
					var val = elem.value;
					elem.setAttribute( "type", value );
					if ( val ) {
						elem.value = val;
					}
					return value;
				}
			}
		},
		// Use the value property for back compat
		// Use the nodeHook for button elements in IE6/7 (#1954)
		value: {
			get: function( elem, name ) {
				if ( nodeHook && jQuery.nodeName( elem, "button" ) ) {
					return nodeHook.get( elem, name );
				}
				return name in elem ?
					elem.value :
					null;
			},
			set: function( elem, value, name ) {
				if ( nodeHook && jQuery.nodeName( elem, "button" ) ) {
					return nodeHook.set( elem, value, name );
				}
				// Does not return so that setAttribute is also used
				elem.value = value;
			}
		}
	},

	propFix: {
		tabindex: "tabIndex",
		readonly: "readOnly",
		"for": "htmlFor",
		"class": "className",
		maxlength: "maxLength",
		cellspacing: "cellSpacing",
		cellpadding: "cellPadding",
		rowspan: "rowSpan",
		colspan: "colSpan",
		usemap: "useMap",
		frameborder: "frameBorder",
		contenteditable: "contentEditable"
	},

	prop: function( elem, name, value ) {
		var ret, hooks, notxml,
			nType = elem.nodeType;

		// don't get/set properties on text, comment and attribute nodes
		if ( !elem || nType === 3 || nType === 8 || nType === 2 ) {
			return;
		}

		notxml = nType !== 1 || !jQuery.isXMLDoc( elem );

		if ( notxml ) {
			// Fix name and attach hooks
			name = jQuery.propFix[ name ] || name;
			hooks = jQuery.propHooks[ name ];
		}

		if ( value !== undefined ) {
			if ( hooks && "set" in hooks && (ret = hooks.set( elem, value, name )) !== undefined ) {
				return ret;

			} else {
				return ( elem[ name ] = value );
			}

		} else {
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, name )) !== null ) {
				return ret;

			} else {
				return elem[ name ];
			}
		}
	},

	propHooks: {
		tabIndex: {
			get: function( elem ) {
				// elem.tabIndex doesn't always return the correct value when it hasn't been explicitly set
				// http://fluidproject.org/blog/2008/01/09/getting-setting-and-removing-tabindex-values-with-javascript/
				var attributeNode = elem.getAttributeNode("tabindex");

				return attributeNode && attributeNode.specified ?
					parseInt( attributeNode.value, 10 ) :
					rfocusable.test( elem.nodeName ) || rclickable.test( elem.nodeName ) && elem.href ?
						0 :
						undefined;
			}
		}
	}
});

// Add the tabIndex propHook to attrHooks for back-compat (different case is intentional)
jQuery.attrHooks.tabindex = jQuery.propHooks.tabIndex;

// Hook for boolean attributes
boolHook = {
	get: function( elem, name ) {
		// Align boolean attributes with corresponding properties
		// Fall back to attribute presence where some booleans are not supported
		var attrNode,
			property = jQuery.prop( elem, name );
		return property === true || typeof property !== "boolean" && ( attrNode = elem.getAttributeNode(name) ) && attrNode.nodeValue !== false ?
			name.toLowerCase() :
			undefined;
	},
	set: function( elem, value, name ) {
		var propName;
		if ( value === false ) {
			// Remove boolean attributes when set to false
			jQuery.removeAttr( elem, name );
		} else {
			// value is true since we know at this point it's type boolean and not false
			// Set boolean attributes to the same name and set the DOM property
			propName = jQuery.propFix[ name ] || name;
			if ( propName in elem ) {
				// Only set the IDL specifically if it already exists on the element
				elem[ propName ] = true;
			}

			elem.setAttribute( name, name.toLowerCase() );
		}
		return name;
	}
};

// IE6/7 do not support getting/setting some attributes with get/setAttribute
if ( !getSetAttribute ) {

	fixSpecified = {
		name: true,
		id: true
	};

	// Use this for any attribute in IE6/7
	// This fixes almost every IE6/7 issue
	nodeHook = jQuery.valHooks.button = {
		get: function( elem, name ) {
			var ret;
			ret = elem.getAttributeNode( name );
			return ret && ( fixSpecified[ name ] ? ret.nodeValue !== "" : ret.specified ) ?
				ret.nodeValue :
				undefined;
		},
		set: function( elem, value, name ) {
			// Set the existing or create a new attribute node
			var ret = elem.getAttributeNode( name );
			if ( !ret ) {
				ret = document.createAttribute( name );
				elem.setAttributeNode( ret );
			}
			return ( ret.nodeValue = value + "" );
		}
	};

	// Apply the nodeHook to tabindex
	jQuery.attrHooks.tabindex.set = nodeHook.set;

	// Set width and height to auto instead of 0 on empty string( Bug #8150 )
	// This is for removals
	jQuery.each([ "width", "height" ], function( i, name ) {
		jQuery.attrHooks[ name ] = jQuery.extend( jQuery.attrHooks[ name ], {
			set: function( elem, value ) {
				if ( value === "" ) {
					elem.setAttribute( name, "auto" );
					return value;
				}
			}
		});
	});

	// Set contenteditable to false on removals(#10429)
	// Setting to empty string throws an error as an invalid value
	jQuery.attrHooks.contenteditable = {
		get: nodeHook.get,
		set: function( elem, value, name ) {
			if ( value === "" ) {
				value = "false";
			}
			nodeHook.set( elem, value, name );
		}
	};
}


// Some attributes require a special call on IE
if ( !jQuery.support.hrefNormalized ) {
	jQuery.each([ "href", "src", "width", "height" ], function( i, name ) {
		jQuery.attrHooks[ name ] = jQuery.extend( jQuery.attrHooks[ name ], {
			get: function( elem ) {
				var ret = elem.getAttribute( name, 2 );
				return ret === null ? undefined : ret;
			}
		});
	});
}

if ( !jQuery.support.style ) {
	jQuery.attrHooks.style = {
		get: function( elem ) {
			// Return undefined in the case of empty string
			// Normalize to lowercase since IE uppercases css property names
			return elem.style.cssText.toLowerCase() || undefined;
		},
		set: function( elem, value ) {
			return ( elem.style.cssText = "" + value );
		}
	};
}

// Safari mis-reports the default selected property of an option
// Accessing the parent's selectedIndex property fixes it
if ( !jQuery.support.optSelected ) {
	jQuery.propHooks.selected = jQuery.extend( jQuery.propHooks.selected, {
		get: function( elem ) {
			var parent = elem.parentNode;

			if ( parent ) {
				parent.selectedIndex;

				// Make sure that it also works with optgroups, see #5701
				if ( parent.parentNode ) {
					parent.parentNode.selectedIndex;
				}
			}
			return null;
		}
	});
}

// IE6/7 call enctype encoding
if ( !jQuery.support.enctype ) {
	jQuery.propFix.enctype = "encoding";
}

// Radios and checkboxes getter/setter
if ( !jQuery.support.checkOn ) {
	jQuery.each([ "radio", "checkbox" ], function() {
		jQuery.valHooks[ this ] = {
			get: function( elem ) {
				// Handle the case where in Webkit "" is returned instead of "on" if a value isn't specified
				return elem.getAttribute("value") === null ? "on" : elem.value;
			}
		};
	});
}
jQuery.each([ "radio", "checkbox" ], function() {
	jQuery.valHooks[ this ] = jQuery.extend( jQuery.valHooks[ this ], {
		set: function( elem, value ) {
			if ( jQuery.isArray( value ) ) {
				return ( elem.checked = jQuery.inArray( jQuery(elem).val(), value ) >= 0 );
			}
		}
	});
});




var rformElems = /^(?:textarea|input|select)$/i,
	rtypenamespace = /^([^\.]*)?(?:\.(.+))?$/,
	rhoverHack = /\bhover(\.\S+)?\b/,
	rkeyEvent = /^key/,
	rmouseEvent = /^(?:mouse|contextmenu)|click/,
	rfocusMorph = /^(?:focusinfocus|focusoutblur)$/,
	rquickIs = /^(\w*)(?:#([\w\-]+))?(?:\.([\w\-]+))?$/,
	quickParse = function( selector ) {
		var quick = rquickIs.exec( selector );
		if ( quick ) {
			//   0  1    2   3
			// [ _, tag, id, class ]
			quick[1] = ( quick[1] || "" ).toLowerCase();
			quick[3] = quick[3] && new RegExp( "(?:^|\\s)" + quick[3] + "(?:\\s|$)" );
		}
		return quick;
	},
	quickIs = function( elem, m ) {
		var attrs = elem.attributes || {};
		return (
			(!m[1] || elem.nodeName.toLowerCase() === m[1]) &&
			(!m[2] || (attrs.id || {}).value === m[2]) &&
			(!m[3] || m[3].test( (attrs[ "class" ] || {}).value ))
		);
	},
	hoverHack = function( events ) {
		return jQuery.event.special.hover ? events : events.replace( rhoverHack, "mouseenter$1 mouseleave$1" );
	};

/*
 * Helper functions for managing events -- not part of the public interface.
 * Props to Dean Edwards' addEvent library for many of the ideas.
 */
jQuery.event = {

	add: function( elem, types, handler, data, selector ) {

		var elemData, eventHandle, events,
			t, tns, type, namespaces, handleObj,
			handleObjIn, quick, handlers, special;

		// Don't attach events to noData or text/comment nodes (allow plain objects tho)
		if ( elem.nodeType === 3 || elem.nodeType === 8 || !types || !handler || !(elemData = jQuery._data( elem )) ) {
			return;
		}

		// Caller can pass in an object of custom data in lieu of the handler
		if ( handler.handler ) {
			handleObjIn = handler;
			handler = handleObjIn.handler;
		}

		// Make sure that the handler has a unique ID, used to find/remove it later
		if ( !handler.guid ) {
			handler.guid = jQuery.guid++;
		}

		// Init the element's event structure and main handler, if this is the first
		events = elemData.events;
		if ( !events ) {
			elemData.events = events = {};
		}
		eventHandle = elemData.handle;
		if ( !eventHandle ) {
			elemData.handle = eventHandle = function( e ) {
				// Discard the second event of a jQuery.event.trigger() and
				// when an event is called after a page has unloaded
				return typeof jQuery !== "undefined" && (!e || jQuery.event.triggered !== e.type) ?
					jQuery.event.dispatch.apply( eventHandle.elem, arguments ) :
					undefined;
			};
			// Add elem as a property of the handle fn to prevent a memory leak with IE non-native events
			eventHandle.elem = elem;
		}

		// Handle multiple events separated by a space
		// jQuery(...).bind("mouseover mouseout", fn);
		types = jQuery.trim( hoverHack(types) ).split( " " );
		for ( t = 0; t < types.length; t++ ) {

			tns = rtypenamespace.exec( types[t] ) || [];
			type = tns[1];
			namespaces = ( tns[2] || "" ).split( "." ).sort();

			// If event changes its type, use the special event handlers for the changed type
			special = jQuery.event.special[ type ] || {};

			// If selector defined, determine special event api type, otherwise given type
			type = ( selector ? special.delegateType : special.bindType ) || type;

			// Update special based on newly reset type
			special = jQuery.event.special[ type ] || {};

			// handleObj is passed to all event handlers
			handleObj = jQuery.extend({
				type: type,
				origType: tns[1],
				data: data,
				handler: handler,
				guid: handler.guid,
				selector: selector,
				quick: quickParse( selector ),
				namespace: namespaces.join(".")
			}, handleObjIn );

			// Init the event handler queue if we're the first
			handlers = events[ type ];
			if ( !handlers ) {
				handlers = events[ type ] = [];
				handlers.delegateCount = 0;

				// Only use addEventListener/attachEvent if the special events handler returns false
				if ( !special.setup || special.setup.call( elem, data, namespaces, eventHandle ) === false ) {
					// Bind the global event handler to the element
					if ( elem.addEventListener ) {
						elem.addEventListener( type, eventHandle, false );

					} else if ( elem.attachEvent ) {
						elem.attachEvent( "on" + type, eventHandle );
					}
				}
			}

			if ( special.add ) {
				special.add.call( elem, handleObj );

				if ( !handleObj.handler.guid ) {
					handleObj.handler.guid = handler.guid;
				}
			}

			// Add to the element's handler list, delegates in front
			if ( selector ) {
				handlers.splice( handlers.delegateCount++, 0, handleObj );
			} else {
				handlers.push( handleObj );
			}

			// Keep track of which events have ever been used, for event optimization
			jQuery.event.global[ type ] = true;
		}

		// Nullify elem to prevent memory leaks in IE
		elem = null;
	},

	global: {},

	// Detach an event or set of events from an element
	remove: function( elem, types, handler, selector, mappedTypes ) {

		var elemData = jQuery.hasData( elem ) && jQuery._data( elem ),
			t, tns, type, origType, namespaces, origCount,
			j, events, special, handle, eventType, handleObj;

		if ( !elemData || !(events = elemData.events) ) {
			return;
		}

		// Once for each type.namespace in types; type may be omitted
		types = jQuery.trim( hoverHack( types || "" ) ).split(" ");
		for ( t = 0; t < types.length; t++ ) {
			tns = rtypenamespace.exec( types[t] ) || [];
			type = origType = tns[1];
			namespaces = tns[2];

			// Unbind all events (on this namespace, if provided) for the element
			if ( !type ) {
				for ( type in events ) {
					jQuery.event.remove( elem, type + types[ t ], handler, selector, true );
				}
				continue;
			}

			special = jQuery.event.special[ type ] || {};
			type = ( selector? special.delegateType : special.bindType ) || type;
			eventType = events[ type ] || [];
			origCount = eventType.length;
			namespaces = namespaces ? new RegExp("(^|\\.)" + namespaces.split(".").sort().join("\\.(?:.*\\.)?") + "(\\.|$)") : null;

			// Remove matching events
			for ( j = 0; j < eventType.length; j++ ) {
				handleObj = eventType[ j ];

				if ( ( mappedTypes || origType === handleObj.origType ) &&
					 ( !handler || handler.guid === handleObj.guid ) &&
					 ( !namespaces || namespaces.test( handleObj.namespace ) ) &&
					 ( !selector || selector === handleObj.selector || selector === "**" && handleObj.selector ) ) {
					eventType.splice( j--, 1 );

					if ( handleObj.selector ) {
						eventType.delegateCount--;
					}
					if ( special.remove ) {
						special.remove.call( elem, handleObj );
					}
				}
			}

			// Remove generic event handler if we removed something and no more handlers exist
			// (avoids potential for endless recursion during removal of special event handlers)
			if ( eventType.length === 0 && origCount !== eventType.length ) {
				if ( !special.teardown || special.teardown.call( elem, namespaces ) === false ) {
					jQuery.removeEvent( elem, type, elemData.handle );
				}

				delete events[ type ];
			}
		}

		// Remove the expando if it's no longer used
		if ( jQuery.isEmptyObject( events ) ) {
			handle = elemData.handle;
			if ( handle ) {
				handle.elem = null;
			}

			// removeData also checks for emptiness and clears the expando if empty
			// so use it instead of delete
			jQuery.removeData( elem, [ "events", "handle" ], true );
		}
	},

	// Events that are safe to short-circuit if no handlers are attached.
	// Native DOM events should not be added, they may have inline handlers.
	customEvent: {
		"getData": true,
		"setData": true,
		"changeData": true
	},

	trigger: function( event, data, elem, onlyHandlers ) {
		// Don't do events on text and comment nodes
		if ( elem && (elem.nodeType === 3 || elem.nodeType === 8) ) {
			return;
		}

		// Event object or event type
		var type = event.type || event,
			namespaces = [],
			cache, exclusive, i, cur, old, ontype, special, handle, eventPath, bubbleType;

		// focus/blur morphs to focusin/out; ensure we're not firing them right now
		if ( rfocusMorph.test( type + jQuery.event.triggered ) ) {
			return;
		}

		if ( type.indexOf( "!" ) >= 0 ) {
			// Exclusive events trigger only for the exact event (no namespaces)
			type = type.slice(0, -1);
			exclusive = true;
		}

		if ( type.indexOf( "." ) >= 0 ) {
			// Namespaced trigger; create a regexp to match event type in handle()
			namespaces = type.split(".");
			type = namespaces.shift();
			namespaces.sort();
		}

		if ( (!elem || jQuery.event.customEvent[ type ]) && !jQuery.event.global[ type ] ) {
			// No jQuery handlers for this event type, and it can't have inline handlers
			return;
		}

		// Caller can pass in an Event, Object, or just an event type string
		event = typeof event === "object" ?
			// jQuery.Event object
			event[ jQuery.expando ] ? event :
			// Object literal
			new jQuery.Event( type, event ) :
			// Just the event type (string)
			new jQuery.Event( type );

		event.type = type;
		event.isTrigger = true;
		event.exclusive = exclusive;
		event.namespace = namespaces.join( "." );
		event.namespace_re = event.namespace? new RegExp("(^|\\.)" + namespaces.join("\\.(?:.*\\.)?") + "(\\.|$)") : null;
		ontype = type.indexOf( ":" ) < 0 ? "on" + type : "";

		// Handle a global trigger
		if ( !elem ) {

			// TODO: Stop taunting the data cache; remove global events and always attach to document
			cache = jQuery.cache;
			for ( i in cache ) {
				if ( cache[ i ].events && cache[ i ].events[ type ] ) {
					jQuery.event.trigger( event, data, cache[ i ].handle.elem, true );
				}
			}
			return;
		}

		// Clean up the event in case it is being reused
		event.result = undefined;
		if ( !event.target ) {
			event.target = elem;
		}

		// Clone any incoming data and prepend the event, creating the handler arg list
		data = data != null ? jQuery.makeArray( data ) : [];
		data.unshift( event );

		// Allow special events to draw outside the lines
		special = jQuery.event.special[ type ] || {};
		if ( special.trigger && special.trigger.apply( elem, data ) === false ) {
			return;
		}

		// Determine event propagation path in advance, per W3C events spec (#9951)
		// Bubble up to document, then to window; watch for a global ownerDocument var (#9724)
		eventPath = [[ elem, special.bindType || type ]];
		if ( !onlyHandlers && !special.noBubble && !jQuery.isWindow( elem ) ) {

			bubbleType = special.delegateType || type;
			cur = rfocusMorph.test( bubbleType + type ) ? elem : elem.parentNode;
			old = null;
			for ( ; cur; cur = cur.parentNode ) {
				eventPath.push([ cur, bubbleType ]);
				old = cur;
			}

			// Only add window if we got to document (e.g., not plain obj or detached DOM)
			if ( old && old === elem.ownerDocument ) {
				eventPath.push([ old.defaultView || old.parentWindow || window, bubbleType ]);
			}
		}

		// Fire handlers on the event path
		for ( i = 0; i < eventPath.length && !event.isPropagationStopped(); i++ ) {

			cur = eventPath[i][0];
			event.type = eventPath[i][1];

			handle = ( jQuery._data( cur, "events" ) || {} )[ event.type ] && jQuery._data( cur, "handle" );
			if ( handle ) {
				handle.apply( cur, data );
			}
			// Note that this is a bare JS function and not a jQuery handler
			handle = ontype && cur[ ontype ];
			if ( handle && jQuery.acceptData( cur ) && handle.apply( cur, data ) === false ) {
				event.preventDefault();
			}
		}
		event.type = type;

		// If nobody prevented the default action, do it now
		if ( !onlyHandlers && !event.isDefaultPrevented() ) {

			if ( (!special._default || special._default.apply( elem.ownerDocument, data ) === false) &&
				!(type === "click" && jQuery.nodeName( elem, "a" )) && jQuery.acceptData( elem ) ) {

				// Call a native DOM method on the target with the same name name as the event.
				// Can't use an .isFunction() check here because IE6/7 fails that test.
				// Don't do default actions on window, that's where global variables be (#6170)
				// IE<9 dies on focus/blur to hidden element (#1486)
				if ( ontype && elem[ type ] && ((type !== "focus" && type !== "blur") || event.target.offsetWidth !== 0) && !jQuery.isWindow( elem ) ) {

					// Don't re-trigger an onFOO event when we call its FOO() method
					old = elem[ ontype ];

					if ( old ) {
						elem[ ontype ] = null;
					}

					// Prevent re-triggering of the same event, since we already bubbled it above
					jQuery.event.triggered = type;
					elem[ type ]();
					jQuery.event.triggered = undefined;

					if ( old ) {
						elem[ ontype ] = old;
					}
				}
			}
		}

		return event.result;
	},

	dispatch: function( event ) {

		// Make a writable jQuery.Event from the native event object
		event = jQuery.event.fix( event || window.event );

		var handlers = ( (jQuery._data( this, "events" ) || {} )[ event.type ] || []),
			delegateCount = handlers.delegateCount,
			args = [].slice.call( arguments, 0 ),
			run_all = !event.exclusive && !event.namespace,
			handlerQueue = [],
			i, j, cur, jqcur, ret, selMatch, matched, matches, handleObj, sel, related;

		// Use the fix-ed jQuery.Event rather than the (read-only) native event
		args[0] = event;
		event.delegateTarget = this;

		// Determine handlers that should run if there are delegated events
		// Avoid disabled elements in IE (#6911) and non-left-click bubbling in Firefox (#3861)
		if ( delegateCount && !event.target.disabled && !(event.button && event.type === "click") ) {

			// Pregenerate a single jQuery object for reuse with .is()
			jqcur = jQuery(this);
			jqcur.context = this.ownerDocument || this;

			for ( cur = event.target; cur != this; cur = cur.parentNode || this ) {
				selMatch = {};
				matches = [];
				jqcur[0] = cur;
				for ( i = 0; i < delegateCount; i++ ) {
					handleObj = handlers[ i ];
					sel = handleObj.selector;

					if ( selMatch[ sel ] === undefined ) {
						selMatch[ sel ] = (
							handleObj.quick ? quickIs( cur, handleObj.quick ) : jqcur.is( sel )
						);
					}
					if ( selMatch[ sel ] ) {
						matches.push( handleObj );
					}
				}
				if ( matches.length ) {
					handlerQueue.push({ elem: cur, matches: matches });
				}
			}
		}

		// Add the remaining (directly-bound) handlers
		if ( handlers.length > delegateCount ) {
			handlerQueue.push({ elem: this, matches: handlers.slice( delegateCount ) });
		}

		// Run delegates first; they may want to stop propagation beneath us
		for ( i = 0; i < handlerQueue.length && !event.isPropagationStopped(); i++ ) {
			matched = handlerQueue[ i ];
			event.currentTarget = matched.elem;

			for ( j = 0; j < matched.matches.length && !event.isImmediatePropagationStopped(); j++ ) {
				handleObj = matched.matches[ j ];

				// Triggered event must either 1) be non-exclusive and have no namespace, or
				// 2) have namespace(s) a subset or equal to those in the bound event (both can have no namespace).
				if ( run_all || (!event.namespace && !handleObj.namespace) || event.namespace_re && event.namespace_re.test( handleObj.namespace ) ) {

					event.data = handleObj.data;
					event.handleObj = handleObj;

					ret = ( (jQuery.event.special[ handleObj.origType ] || {}).handle || handleObj.handler )
							.apply( matched.elem, args );

					if ( ret !== undefined ) {
						event.result = ret;
						if ( ret === false ) {
							event.preventDefault();
							event.stopPropagation();
						}
					}
				}
			}
		}

		return event.result;
	},

	// Includes some event props shared by KeyEvent and MouseEvent
	// *** attrChange attrName relatedNode srcElement  are not normalized, non-W3C, deprecated, will be removed in 1.8 ***
	props: "attrChange attrName relatedNode srcElement altKey bubbles cancelable ctrlKey currentTarget eventPhase metaKey relatedTarget shiftKey target timeStamp view which".split(" "),

	fixHooks: {},

	keyHooks: {
		props: "char charCode key keyCode".split(" "),
		filter: function( event, original ) {

			// Add which for key events
			if ( event.which == null ) {
				event.which = original.charCode != null ? original.charCode : original.keyCode;
			}

			return event;
		}
	},

	mouseHooks: {
		props: "button buttons clientX clientY fromElement offsetX offsetY pageX pageY screenX screenY toElement".split(" "),
		filter: function( event, original ) {
			var eventDoc, doc, body,
				button = original.button,
				fromElement = original.fromElement;

			// Calculate pageX/Y if missing and clientX/Y available
			if ( event.pageX == null && original.clientX != null ) {
				eventDoc = event.target.ownerDocument || document;
				doc = eventDoc.documentElement;
				body = eventDoc.body;

				event.pageX = original.clientX + ( doc && doc.scrollLeft || body && body.scrollLeft || 0 ) - ( doc && doc.clientLeft || body && body.clientLeft || 0 );
				event.pageY = original.clientY + ( doc && doc.scrollTop  || body && body.scrollTop  || 0 ) - ( doc && doc.clientTop  || body && body.clientTop  || 0 );
			}

			// Add relatedTarget, if necessary
			if ( !event.relatedTarget && fromElement ) {
				event.relatedTarget = fromElement === event.target ? original.toElement : fromElement;
			}

			// Add which for click: 1 === left; 2 === middle; 3 === right
			// Note: button is not normalized, so don't use it
			if ( !event.which && button !== undefined ) {
				event.which = ( button & 1 ? 1 : ( button & 2 ? 3 : ( button & 4 ? 2 : 0 ) ) );
			}

			return event;
		}
	},

	fix: function( event ) {
		if ( event[ jQuery.expando ] ) {
			return event;
		}

		// Create a writable copy of the event object and normalize some properties
		var i, prop,
			originalEvent = event,
			fixHook = jQuery.event.fixHooks[ event.type ] || {},
			copy = fixHook.props ? this.props.concat( fixHook.props ) : this.props;

		event = jQuery.Event( originalEvent );

		for ( i = copy.length; i; ) {
			prop = copy[ --i ];
			event[ prop ] = originalEvent[ prop ];
		}

		// Fix target property, if necessary (#1925, IE 6/7/8 & Safari2)
		if ( !event.target ) {
			event.target = originalEvent.srcElement || document;
		}

		// Target should not be a text node (#504, Safari)
		if ( event.target.nodeType === 3 ) {
			event.target = event.target.parentNode;
		}

		// For mouse/key events; add metaKey if it's not there (#3368, IE6/7/8)
		if ( event.metaKey === undefined ) {
			event.metaKey = event.ctrlKey;
		}

		return fixHook.filter? fixHook.filter( event, originalEvent ) : event;
	},

	special: {
		ready: {
			// Make sure the ready event is setup
			setup: jQuery.bindReady
		},

		load: {
			// Prevent triggered image.load events from bubbling to window.load
			noBubble: true
		},

		focus: {
			delegateType: "focusin"
		},
		blur: {
			delegateType: "focusout"
		},

		beforeunload: {
			setup: function( data, namespaces, eventHandle ) {
				// We only want to do this special case on windows
				if ( jQuery.isWindow( this ) ) {
					this.onbeforeunload = eventHandle;
				}
			},

			teardown: function( namespaces, eventHandle ) {
				if ( this.onbeforeunload === eventHandle ) {
					this.onbeforeunload = null;
				}
			}
		}
	},

	simulate: function( type, elem, event, bubble ) {
		// Piggyback on a donor event to simulate a different one.
		// Fake originalEvent to avoid donor's stopPropagation, but if the
		// simulated event prevents default then we do the same on the donor.
		var e = jQuery.extend(
			new jQuery.Event(),
			event,
			{ type: type,
				isSimulated: true,
				originalEvent: {}
			}
		);
		if ( bubble ) {
			jQuery.event.trigger( e, null, elem );
		} else {
			jQuery.event.dispatch.call( elem, e );
		}
		if ( e.isDefaultPrevented() ) {
			event.preventDefault();
		}
	}
};

// Some plugins are using, but it's undocumented/deprecated and will be removed.
// The 1.7 special event interface should provide all the hooks needed now.
jQuery.event.handle = jQuery.event.dispatch;

jQuery.removeEvent = document.removeEventListener ?
	function( elem, type, handle ) {
		if ( elem.removeEventListener ) {
			elem.removeEventListener( type, handle, false );
		}
	} :
	function( elem, type, handle ) {
		if ( elem.detachEvent ) {
			elem.detachEvent( "on" + type, handle );
		}
	};

jQuery.Event = function( src, props ) {
	// Allow instantiation without the 'new' keyword
	if ( !(this instanceof jQuery.Event) ) {
		return new jQuery.Event( src, props );
	}

	// Event object
	if ( src && src.type ) {
		this.originalEvent = src;
		this.type = src.type;

		// Events bubbling up the document may have been marked as prevented
		// by a handler lower down the tree; reflect the correct value.
		this.isDefaultPrevented = ( src.defaultPrevented || src.returnValue === false ||
			src.getPreventDefault && src.getPreventDefault() ) ? returnTrue : returnFalse;

	// Event type
	} else {
		this.type = src;
	}

	// Put explicitly provided properties onto the event object
	if ( props ) {
		jQuery.extend( this, props );
	}

	// Create a timestamp if incoming event doesn't have one
	this.timeStamp = src && src.timeStamp || jQuery.now();

	// Mark it as fixed
	this[ jQuery.expando ] = true;
};

function returnFalse() {
	return false;
}
function returnTrue() {
	return true;
}

// jQuery.Event is based on DOM3 Events as specified by the ECMAScript Language Binding
// http://www.w3.org/TR/2003/WD-DOM-Level-3-Events-20030331/ecma-script-binding.html
jQuery.Event.prototype = {
	preventDefault: function() {
		this.isDefaultPrevented = returnTrue;

		var e = this.originalEvent;
		if ( !e ) {
			return;
		}

		// if preventDefault exists run it on the original event
		if ( e.preventDefault ) {
			e.preventDefault();

		// otherwise set the returnValue property of the original event to false (IE)
		} else {
			e.returnValue = false;
		}
	},
	stopPropagation: function() {
		this.isPropagationStopped = returnTrue;

		var e = this.originalEvent;
		if ( !e ) {
			return;
		}
		// if stopPropagation exists run it on the original event
		if ( e.stopPropagation ) {
			e.stopPropagation();
		}
		// otherwise set the cancelBubble property of the original event to true (IE)
		e.cancelBubble = true;
	},
	stopImmediatePropagation: function() {
		this.isImmediatePropagationStopped = returnTrue;
		this.stopPropagation();
	},
	isDefaultPrevented: returnFalse,
	isPropagationStopped: returnFalse,
	isImmediatePropagationStopped: returnFalse
};

// Create mouseenter/leave events using mouseover/out and event-time checks
jQuery.each({
	mouseenter: "mouseover",
	mouseleave: "mouseout"
}, function( orig, fix ) {
	jQuery.event.special[ orig ] = {
		delegateType: fix,
		bindType: fix,

		handle: function( event ) {
			var target = this,
				related = event.relatedTarget,
				handleObj = event.handleObj,
				selector = handleObj.selector,
				ret;

			// For mousenter/leave call the handler if related is outside the target.
			// NB: No relatedTarget if the mouse left/entered the browser window
			if ( !related || (related !== target && !jQuery.contains( target, related )) ) {
				event.type = handleObj.origType;
				ret = handleObj.handler.apply( this, arguments );
				event.type = fix;
			}
			return ret;
		}
	};
});

// IE submit delegation
if ( !jQuery.support.submitBubbles ) {

	jQuery.event.special.submit = {
		setup: function() {
			// Only need this for delegated form submit events
			if ( jQuery.nodeName( this, "form" ) ) {
				return false;
			}

			// Lazy-add a submit handler when a descendant form may potentially be submitted
			jQuery.event.add( this, "click._submit keypress._submit", function( e ) {
				// Node name check avoids a VML-related crash in IE (#9807)
				var elem = e.target,
					form = jQuery.nodeName( elem, "input" ) || jQuery.nodeName( elem, "button" ) ? elem.form : undefined;
				if ( form && !form._submit_attached ) {
					jQuery.event.add( form, "submit._submit", function( event ) {
						// If form was submitted by the user, bubble the event up the tree
						if ( this.parentNode && !event.isTrigger ) {
							jQuery.event.simulate( "submit", this.parentNode, event, true );
						}
					});
					form._submit_attached = true;
				}
			});
			// return undefined since we don't need an event listener
		},

		teardown: function() {
			// Only need this for delegated form submit events
			if ( jQuery.nodeName( this, "form" ) ) {
				return false;
			}

			// Remove delegated handlers; cleanData eventually reaps submit handlers attached above
			jQuery.event.remove( this, "._submit" );
		}
	};
}

// IE change delegation and checkbox/radio fix
if ( !jQuery.support.changeBubbles ) {

	jQuery.event.special.change = {

		setup: function() {

			if ( rformElems.test( this.nodeName ) ) {
				// IE doesn't fire change on a check/radio until blur; trigger it on click
				// after a propertychange. Eat the blur-change in special.change.handle.
				// This still fires onchange a second time for check/radio after blur.
				if ( this.type === "checkbox" || this.type === "radio" ) {
					jQuery.event.add( this, "propertychange._change", function( event ) {
						if ( event.originalEvent.propertyName === "checked" ) {
							this._just_changed = true;
						}
					});
					jQuery.event.add( this, "click._change", function( event ) {
						if ( this._just_changed && !event.isTrigger ) {
							this._just_changed = false;
							jQuery.event.simulate( "change", this, event, true );
						}
					});
				}
				return false;
			}
			// Delegated event; lazy-add a change handler on descendant inputs
			jQuery.event.add( this, "beforeactivate._change", function( e ) {
				var elem = e.target;

				if ( rformElems.test( elem.nodeName ) && !elem._change_attached ) {
					jQuery.event.add( elem, "change._change", function( event ) {
						if ( this.parentNode && !event.isSimulated && !event.isTrigger ) {
							jQuery.event.simulate( "change", this.parentNode, event, true );
						}
					});
					elem._change_attached = true;
				}
			});
		},

		handle: function( event ) {
			var elem = event.target;

			// Swallow native change events from checkbox/radio, we already triggered them above
			if ( this !== elem || event.isSimulated || event.isTrigger || (elem.type !== "radio" && elem.type !== "checkbox") ) {
				return event.handleObj.handler.apply( this, arguments );
			}
		},

		teardown: function() {
			jQuery.event.remove( this, "._change" );

			return rformElems.test( this.nodeName );
		}
	};
}

// Create "bubbling" focus and blur events
if ( !jQuery.support.focusinBubbles ) {
	jQuery.each({ focus: "focusin", blur: "focusout" }, function( orig, fix ) {

		// Attach a single capturing handler while someone wants focusin/focusout
		var attaches = 0,
			handler = function( event ) {
				jQuery.event.simulate( fix, event.target, jQuery.event.fix( event ), true );
			};

		jQuery.event.special[ fix ] = {
			setup: function() {
				if ( attaches++ === 0 ) {
					document.addEventListener( orig, handler, true );
				}
			},
			teardown: function() {
				if ( --attaches === 0 ) {
					document.removeEventListener( orig, handler, true );
				}
			}
		};
	});
}

jQuery.fn.extend({

	on: function( types, selector, data, fn, /*INTERNAL*/ one ) {
		var origFn, type;

		// Types can be a map of types/handlers
		if ( typeof types === "object" ) {
			// ( types-Object, selector, data )
			if ( typeof selector !== "string" ) {
				// ( types-Object, data )
				data = selector;
				selector = undefined;
			}
			for ( type in types ) {
				this.on( type, selector, data, types[ type ], one );
			}
			return this;
		}

		if ( data == null && fn == null ) {
			// ( types, fn )
			fn = selector;
			data = selector = undefined;
		} else if ( fn == null ) {
			if ( typeof selector === "string" ) {
				// ( types, selector, fn )
				fn = data;
				data = undefined;
			} else {
				// ( types, data, fn )
				fn = data;
				data = selector;
				selector = undefined;
			}
		}
		if ( fn === false ) {
			fn = returnFalse;
		} else if ( !fn ) {
			return this;
		}

		if ( one === 1 ) {
			origFn = fn;
			fn = function( event ) {
				// Can use an empty set, since event contains the info
				jQuery().off( event );
				return origFn.apply( this, arguments );
			};
			// Use same guid so caller can remove using origFn
			fn.guid = origFn.guid || ( origFn.guid = jQuery.guid++ );
		}
		return this.each( function() {
			jQuery.event.add( this, types, fn, data, selector );
		});
	},
	one: function( types, selector, data, fn ) {
		return this.on.call( this, types, selector, data, fn, 1 );
	},
	off: function( types, selector, fn ) {
		if ( types && types.preventDefault && types.handleObj ) {
			// ( event )  dispatched jQuery.Event
			var handleObj = types.handleObj;
			jQuery( types.delegateTarget ).off(
				handleObj.namespace? handleObj.type + "." + handleObj.namespace : handleObj.type,
				handleObj.selector,
				handleObj.handler
			);
			return this;
		}
		if ( typeof types === "object" ) {
			// ( types-object [, selector] )
			for ( var type in types ) {
				this.off( type, selector, types[ type ] );
			}
			return this;
		}
		if ( selector === false || typeof selector === "function" ) {
			// ( types [, fn] )
			fn = selector;
			selector = undefined;
		}
		if ( fn === false ) {
			fn = returnFalse;
		}
		return this.each(function() {
			jQuery.event.remove( this, types, fn, selector );
		});
	},

	bind: function( types, data, fn ) {
		return this.on( types, null, data, fn );
	},
	unbind: function( types, fn ) {
		return this.off( types, null, fn );
	},

	live: function( types, data, fn ) {
		jQuery( this.context ).on( types, this.selector, data, fn );
		return this;
	},
	die: function( types, fn ) {
		jQuery( this.context ).off( types, this.selector || "**", fn );
		return this;
	},

	delegate: function( selector, types, data, fn ) {
		return this.on( types, selector, data, fn );
	},
	undelegate: function( selector, types, fn ) {
		// ( namespace ) or ( selector, types [, fn] )
		return arguments.length == 1? this.off( selector, "**" ) : this.off( types, selector, fn );
	},

	trigger: function( type, data ) {
		return this.each(function() {
			jQuery.event.trigger( type, data, this );
		});
	},
	triggerHandler: function( type, data ) {
		if ( this[0] ) {
			return jQuery.event.trigger( type, data, this[0], true );
		}
	},

	toggle: function( fn ) {
		// Save reference to arguments for access in closure
		var args = arguments,
			guid = fn.guid || jQuery.guid++,
			i = 0,
			toggler = function( event ) {
				// Figure out which function to execute
				var lastToggle = ( jQuery._data( this, "lastToggle" + fn.guid ) || 0 ) % i;
				jQuery._data( this, "lastToggle" + fn.guid, lastToggle + 1 );

				// Make sure that clicks stop
				event.preventDefault();

				// and execute the function
				return args[ lastToggle ].apply( this, arguments ) || false;
			};

		// link all the functions, so any of them can unbind this click handler
		toggler.guid = guid;
		while ( i < args.length ) {
			args[ i++ ].guid = guid;
		}

		return this.click( toggler );
	},

	hover: function( fnOver, fnOut ) {
		return this.mouseenter( fnOver ).mouseleave( fnOut || fnOver );
	}
});

jQuery.each( ("blur focus focusin focusout load resize scroll unload click dblclick " +
	"mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave " +
	"change select submit keydown keypress keyup error contextmenu").split(" "), function( i, name ) {

	// Handle event binding
	jQuery.fn[ name ] = function( data, fn ) {
		if ( fn == null ) {
			fn = data;
			data = null;
		}

		return arguments.length > 0 ?
			this.on( name, null, data, fn ) :
			this.trigger( name );
	};

	if ( jQuery.attrFn ) {
		jQuery.attrFn[ name ] = true;
	}

	if ( rkeyEvent.test( name ) ) {
		jQuery.event.fixHooks[ name ] = jQuery.event.keyHooks;
	}

	if ( rmouseEvent.test( name ) ) {
		jQuery.event.fixHooks[ name ] = jQuery.event.mouseHooks;
	}
});



/*!
 * Sizzle CSS Selector Engine
 *  Copyright 2011, The Dojo Foundation
 *  Released under the MIT, BSD, and GPL Licenses.
 *  More information: http://sizzlejs.com/
 */
(function(){

var chunker = /((?:\((?:\([^()]+\)|[^()]+)+\)|\[(?:\[[^\[\]]*\]|['"][^'"]*['"]|[^\[\]'"]+)+\]|\\.|[^ >+~,(\[\\]+)+|[>+~])(\s*,\s*)?((?:.|\r|\n)*)/g,
	expando = "sizcache" + (Math.random() + '').replace('.', ''),
	done = 0,
	toString = Object.prototype.toString,
	hasDuplicate = false,
	baseHasDuplicate = true,
	rBackslash = /\\/g,
	rReturn = /\r\n/g,
	rNonWord = /\W/;

// Here we check if the JavaScript engine is using some sort of
// optimization where it does not always call our comparision
// function. If that is the case, discard the hasDuplicate value.
//   Thus far that includes Google Chrome.
[0, 0].sort(function() {
	baseHasDuplicate = false;
	return 0;
});

var Sizzle = function( selector, context, results, seed ) {
	results = results || [];
	context = context || document;

	var origContext = context;

	if ( context.nodeType !== 1 && context.nodeType !== 9 ) {
		return [];
	}
	
	if ( !selector || typeof selector !== "string" ) {
		return results;
	}

	var m, set, checkSet, extra, ret, cur, pop, i,
		prune = true,
		contextXML = Sizzle.isXML( context ),
		parts = [],
		soFar = selector;
	
	// Reset the position of the chunker regexp (start from head)
	do {
		chunker.exec( "" );
		m = chunker.exec( soFar );

		if ( m ) {
			soFar = m[3];
		
			parts.push( m[1] );
		
			if ( m[2] ) {
				extra = m[3];
				break;
			}
		}
	} while ( m );

	if ( parts.length > 1 && origPOS.exec( selector ) ) {

		if ( parts.length === 2 && Expr.relative[ parts[0] ] ) {
			set = posProcess( parts[0] + parts[1], context, seed );

		} else {
			set = Expr.relative[ parts[0] ] ?
				[ context ] :
				Sizzle( parts.shift(), context );

			while ( parts.length ) {
				selector = parts.shift();

				if ( Expr.relative[ selector ] ) {
					selector += parts.shift();
				}
				
				set = posProcess( selector, set, seed );
			}
		}

	} else {
		// Take a shortcut and set the context if the root selector is an ID
		// (but not if it'll be faster if the inner selector is an ID)
		if ( !seed && parts.length > 1 && context.nodeType === 9 && !contextXML &&
				Expr.match.ID.test(parts[0]) && !Expr.match.ID.test(parts[parts.length - 1]) ) {

			ret = Sizzle.find( parts.shift(), context, contextXML );
			context = ret.expr ?
				Sizzle.filter( ret.expr, ret.set )[0] :
				ret.set[0];
		}

		if ( context ) {
			ret = seed ?
				{ expr: parts.pop(), set: makeArray(seed) } :
				Sizzle.find( parts.pop(), parts.length === 1 && (parts[0] === "~" || parts[0] === "+") && context.parentNode ? context.parentNode : context, contextXML );

			set = ret.expr ?
				Sizzle.filter( ret.expr, ret.set ) :
				ret.set;

			if ( parts.length > 0 ) {
				checkSet = makeArray( set );

			} else {
				prune = false;
			}

			while ( parts.length ) {
				cur = parts.pop();
				pop = cur;

				if ( !Expr.relative[ cur ] ) {
					cur = "";
				} else {
					pop = parts.pop();
				}

				if ( pop == null ) {
					pop = context;
				}

				Expr.relative[ cur ]( checkSet, pop, contextXML );
			}

		} else {
			checkSet = parts = [];
		}
	}

	if ( !checkSet ) {
		checkSet = set;
	}

	if ( !checkSet ) {
		Sizzle.error( cur || selector );
	}

	if ( toString.call(checkSet) === "[object Array]" ) {
		if ( !prune ) {
			results.push.apply( results, checkSet );

		} else if ( context && context.nodeType === 1 ) {
			for ( i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && (checkSet[i] === true || checkSet[i].nodeType === 1 && Sizzle.contains(context, checkSet[i])) ) {
					results.push( set[i] );
				}
			}

		} else {
			for ( i = 0; checkSet[i] != null; i++ ) {
				if ( checkSet[i] && checkSet[i].nodeType === 1 ) {
					results.push( set[i] );
				}
			}
		}

	} else {
		makeArray( checkSet, results );
	}

	if ( extra ) {
		Sizzle( extra, origContext, results, seed );
		Sizzle.uniqueSort( results );
	}

	return results;
};

Sizzle.uniqueSort = function( results ) {
	if ( sortOrder ) {
		hasDuplicate = baseHasDuplicate;
		results.sort( sortOrder );

		if ( hasDuplicate ) {
			for ( var i = 1; i < results.length; i++ ) {
				if ( results[i] === results[ i - 1 ] ) {
					results.splice( i--, 1 );
				}
			}
		}
	}

	return results;
};

Sizzle.matches = function( expr, set ) {
	return Sizzle( expr, null, null, set );
};

Sizzle.matchesSelector = function( node, expr ) {
	return Sizzle( expr, null, null, [node] ).length > 0;
};

Sizzle.find = function( expr, context, isXML ) {
	var set, i, len, match, type, left;

	if ( !expr ) {
		return [];
	}

	for ( i = 0, len = Expr.order.length; i < len; i++ ) {
		type = Expr.order[i];
		
		if ( (match = Expr.leftMatch[ type ].exec( expr )) ) {
			left = match[1];
			match.splice( 1, 1 );

			if ( left.substr( left.length - 1 ) !== "\\" ) {
				match[1] = (match[1] || "").replace( rBackslash, "" );
				set = Expr.find[ type ]( match, context, isXML );

				if ( set != null ) {
					expr = expr.replace( Expr.match[ type ], "" );
					break;
				}
			}
		}
	}

	if ( !set ) {
		set = typeof context.getElementsByTagName !== "undefined" ?
			context.getElementsByTagName( "*" ) :
			[];
	}

	return { set: set, expr: expr };
};

Sizzle.filter = function( expr, set, inplace, not ) {
	var match, anyFound,
		type, found, item, filter, left,
		i, pass,
		old = expr,
		result = [],
		curLoop = set,
		isXMLFilter = set && set[0] && Sizzle.isXML( set[0] );

	while ( expr && set.length ) {
		for ( type in Expr.filter ) {
			if ( (match = Expr.leftMatch[ type ].exec( expr )) != null && match[2] ) {
				filter = Expr.filter[ type ];
				left = match[1];

				anyFound = false;

				match.splice(1,1);

				if ( left.substr( left.length - 1 ) === "\\" ) {
					continue;
				}

				if ( curLoop === result ) {
					result = [];
				}

				if ( Expr.preFilter[ type ] ) {
					match = Expr.preFilter[ type ]( match, curLoop, inplace, result, not, isXMLFilter );

					if ( !match ) {
						anyFound = found = true;

					} else if ( match === true ) {
						continue;
					}
				}

				if ( match ) {
					for ( i = 0; (item = curLoop[i]) != null; i++ ) {
						if ( item ) {
							found = filter( item, match, i, curLoop );
							pass = not ^ found;

							if ( inplace && found != null ) {
								if ( pass ) {
									anyFound = true;

								} else {
									curLoop[i] = false;
								}

							} else if ( pass ) {
								result.push( item );
								anyFound = true;
							}
						}
					}
				}

				if ( found !== undefined ) {
					if ( !inplace ) {
						curLoop = result;
					}

					expr = expr.replace( Expr.match[ type ], "" );

					if ( !anyFound ) {
						return [];
					}

					break;
				}
			}
		}

		// Improper expression
		if ( expr === old ) {
			if ( anyFound == null ) {
				Sizzle.error( expr );

			} else {
				break;
			}
		}

		old = expr;
	}

	return curLoop;
};

Sizzle.error = function( msg ) {
	throw new Error( "Syntax error, unrecognized expression: " + msg );
};

/**
 * Utility function for retreiving the text value of an array of DOM nodes
 * @param {Array|Element} elem
 */
var getText = Sizzle.getText = function( elem ) {
    var i, node,
		nodeType = elem.nodeType,
		ret = "";

	if ( nodeType ) {
		if ( nodeType === 1 || nodeType === 9 ) {
			// Use textContent || innerText for elements
			if ( typeof elem.textContent === 'string' ) {
				return elem.textContent;
			} else if ( typeof elem.innerText === 'string' ) {
				// Replace IE's carriage returns
				return elem.innerText.replace( rReturn, '' );
			} else {
				// Traverse it's children
				for ( elem = elem.firstChild; elem; elem = elem.nextSibling) {
					ret += getText( elem );
				}
			}
		} else if ( nodeType === 3 || nodeType === 4 ) {
			return elem.nodeValue;
		}
	} else {

		// If no nodeType, this is expected to be an array
		for ( i = 0; (node = elem[i]); i++ ) {
			// Do not traverse comment nodes
			if ( node.nodeType !== 8 ) {
				ret += getText( node );
			}
		}
	}
	return ret;
};

var Expr = Sizzle.selectors = {
	order: [ "ID", "NAME", "TAG" ],

	match: {
		ID: /#((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,
		CLASS: /\.((?:[\w\u00c0-\uFFFF\-]|\\.)+)/,
		NAME: /\[name=['"]*((?:[\w\u00c0-\uFFFF\-]|\\.)+)['"]*\]/,
		ATTR: /\[\s*((?:[\w\u00c0-\uFFFF\-]|\\.)+)\s*(?:(\S?=)\s*(?:(['"])(.*?)\3|(#?(?:[\w\u00c0-\uFFFF\-]|\\.)*)|)|)\s*\]/,
		TAG: /^((?:[\w\u00c0-\uFFFF\*\-]|\\.)+)/,
		CHILD: /:(only|nth|last|first)-child(?:\(\s*(even|odd|(?:[+\-]?\d+|(?:[+\-]?\d*)?n\s*(?:[+\-]\s*\d+)?))\s*\))?/,
		POS: /:(nth|eq|gt|lt|first|last|even|odd)(?:\((\d*)\))?(?=[^\-]|$)/,
		PSEUDO: /:((?:[\w\u00c0-\uFFFF\-]|\\.)+)(?:\((['"]?)((?:\([^\)]+\)|[^\(\)]*)+)\2\))?/
	},

	leftMatch: {},

	attrMap: {
		"class": "className",
		"for": "htmlFor"
	},

	attrHandle: {
		href: function( elem ) {
			return elem.getAttribute( "href" );
		},
		type: function( elem ) {
			return elem.getAttribute( "type" );
		}
	},

	relative: {
		"+": function(checkSet, part){
			var isPartStr = typeof part === "string",
				isTag = isPartStr && !rNonWord.test( part ),
				isPartStrNotTag = isPartStr && !isTag;

			if ( isTag ) {
				part = part.toLowerCase();
			}

			for ( var i = 0, l = checkSet.length, elem; i < l; i++ ) {
				if ( (elem = checkSet[i]) ) {
					while ( (elem = elem.previousSibling) && elem.nodeType !== 1 ) {}

					checkSet[i] = isPartStrNotTag || elem && elem.nodeName.toLowerCase() === part ?
						elem || false :
						elem === part;
				}
			}

			if ( isPartStrNotTag ) {
				Sizzle.filter( part, checkSet, true );
			}
		},

		">": function( checkSet, part ) {
			var elem,
				isPartStr = typeof part === "string",
				i = 0,
				l = checkSet.length;

			if ( isPartStr && !rNonWord.test( part ) ) {
				part = part.toLowerCase();

				for ( ; i < l; i++ ) {
					elem = checkSet[i];

					if ( elem ) {
						var parent = elem.parentNode;
						checkSet[i] = parent.nodeName.toLowerCase() === part ? parent : false;
					}
				}

			} else {
				for ( ; i < l; i++ ) {
					elem = checkSet[i];

					if ( elem ) {
						checkSet[i] = isPartStr ?
							elem.parentNode :
							elem.parentNode === part;
					}
				}

				if ( isPartStr ) {
					Sizzle.filter( part, checkSet, true );
				}
			}
		},

		"": function(checkSet, part, isXML){
			var nodeCheck,
				doneName = done++,
				checkFn = dirCheck;

			if ( typeof part === "string" && !rNonWord.test( part ) ) {
				part = part.toLowerCase();
				nodeCheck = part;
				checkFn = dirNodeCheck;
			}

			checkFn( "parentNode", part, doneName, checkSet, nodeCheck, isXML );
		},

		"~": function( checkSet, part, isXML ) {
			var nodeCheck,
				doneName = done++,
				checkFn = dirCheck;

			if ( typeof part === "string" && !rNonWord.test( part ) ) {
				part = part.toLowerCase();
				nodeCheck = part;
				checkFn = dirNodeCheck;
			}

			checkFn( "previousSibling", part, doneName, checkSet, nodeCheck, isXML );
		}
	},

	find: {
		ID: function( match, context, isXML ) {
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);
				// Check parentNode to catch when Blackberry 4.6 returns
				// nodes that are no longer in the document #6963
				return m && m.parentNode ? [m] : [];
			}
		},

		NAME: function( match, context ) {
			if ( typeof context.getElementsByName !== "undefined" ) {
				var ret = [],
					results = context.getElementsByName( match[1] );

				for ( var i = 0, l = results.length; i < l; i++ ) {
					if ( results[i].getAttribute("name") === match[1] ) {
						ret.push( results[i] );
					}
				}

				return ret.length === 0 ? null : ret;
			}
		},

		TAG: function( match, context ) {
			if ( typeof context.getElementsByTagName !== "undefined" ) {
				return context.getElementsByTagName( match[1] );
			}
		}
	},
	preFilter: {
		CLASS: function( match, curLoop, inplace, result, not, isXML ) {
			match = " " + match[1].replace( rBackslash, "" ) + " ";

			if ( isXML ) {
				return match;
			}

			for ( var i = 0, elem; (elem = curLoop[i]) != null; i++ ) {
				if ( elem ) {
					if ( not ^ (elem.className && (" " + elem.className + " ").replace(/[\t\n\r]/g, " ").indexOf(match) >= 0) ) {
						if ( !inplace ) {
							result.push( elem );
						}

					} else if ( inplace ) {
						curLoop[i] = false;
					}
				}
			}

			return false;
		},

		ID: function( match ) {
			return match[1].replace( rBackslash, "" );
		},

		TAG: function( match, curLoop ) {
			return match[1].replace( rBackslash, "" ).toLowerCase();
		},

		CHILD: function( match ) {
			if ( match[1] === "nth" ) {
				if ( !match[2] ) {
					Sizzle.error( match[0] );
				}

				match[2] = match[2].replace(/^\+|\s*/g, '');

				// parse equations like 'even', 'odd', '5', '2n', '3n+2', '4n-1', '-n+6'
				var test = /(-?)(\d*)(?:n([+\-]?\d*))?/.exec(
					match[2] === "even" && "2n" || match[2] === "odd" && "2n+1" ||
					!/\D/.test( match[2] ) && "0n+" + match[2] || match[2]);

				// calculate the numbers (first)n+(last) including if they are negative
				match[2] = (test[1] + (test[2] || 1)) - 0;
				match[3] = test[3] - 0;
			}
			else if ( match[2] ) {
				Sizzle.error( match[0] );
			}

			// TODO: Move to normal caching system
			match[0] = done++;

			return match;
		},

		ATTR: function( match, curLoop, inplace, result, not, isXML ) {
			var name = match[1] = match[1].replace( rBackslash, "" );
			
			if ( !isXML && Expr.attrMap[name] ) {
				match[1] = Expr.attrMap[name];
			}

			// Handle if an un-quoted value was used
			match[4] = ( match[4] || match[5] || "" ).replace( rBackslash, "" );

			if ( match[2] === "~=" ) {
				match[4] = " " + match[4] + " ";
			}

			return match;
		},

		PSEUDO: function( match, curLoop, inplace, result, not ) {
			if ( match[1] === "not" ) {
				// If we're dealing with a complex expression, or a simple one
				if ( ( chunker.exec(match[3]) || "" ).length > 1 || /^\w/.test(match[3]) ) {
					match[3] = Sizzle(match[3], null, null, curLoop);

				} else {
					var ret = Sizzle.filter(match[3], curLoop, inplace, true ^ not);

					if ( !inplace ) {
						result.push.apply( result, ret );
					}

					return false;
				}

			} else if ( Expr.match.POS.test( match[0] ) || Expr.match.CHILD.test( match[0] ) ) {
				return true;
			}
			
			return match;
		},

		POS: function( match ) {
			match.unshift( true );

			return match;
		}
	},
	
	filters: {
		enabled: function( elem ) {
			return elem.disabled === false && elem.type !== "hidden";
		},

		disabled: function( elem ) {
			return elem.disabled === true;
		},

		checked: function( elem ) {
			return elem.checked === true;
		},
		
		selected: function( elem ) {
			// Accessing this property makes selected-by-default
			// options in Safari work properly
			if ( elem.parentNode ) {
				elem.parentNode.selectedIndex;
			}
			
			return elem.selected === true;
		},

		parent: function( elem ) {
			return !!elem.firstChild;
		},

		empty: function( elem ) {
			return !elem.firstChild;
		},

		has: function( elem, i, match ) {
			return !!Sizzle( match[3], elem ).length;
		},

		header: function( elem ) {
			return (/h\d/i).test( elem.nodeName );
		},

		text: function( elem ) {
			var attr = elem.getAttribute( "type" ), type = elem.type;
			// IE6 and 7 will map elem.type to 'text' for new HTML5 types (search, etc) 
			// use getAttribute instead to test this case
			return elem.nodeName.toLowerCase() === "input" && "text" === type && ( attr === type || attr === null );
		},

		radio: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "radio" === elem.type;
		},

		checkbox: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "checkbox" === elem.type;
		},

		file: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "file" === elem.type;
		},

		password: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "password" === elem.type;
		},

		submit: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return (name === "input" || name === "button") && "submit" === elem.type;
		},

		image: function( elem ) {
			return elem.nodeName.toLowerCase() === "input" && "image" === elem.type;
		},

		reset: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return (name === "input" || name === "button") && "reset" === elem.type;
		},

		button: function( elem ) {
			var name = elem.nodeName.toLowerCase();
			return name === "input" && "button" === elem.type || name === "button";
		},

		input: function( elem ) {
			return (/input|select|textarea|button/i).test( elem.nodeName );
		},

		focus: function( elem ) {
			return elem === elem.ownerDocument.activeElement;
		}
	},
	setFilters: {
		first: function( elem, i ) {
			return i === 0;
		},

		last: function( elem, i, match, array ) {
			return i === array.length - 1;
		},

		even: function( elem, i ) {
			return i % 2 === 0;
		},

		odd: function( elem, i ) {
			return i % 2 === 1;
		},

		lt: function( elem, i, match ) {
			return i < match[3] - 0;
		},

		gt: function( elem, i, match ) {
			return i > match[3] - 0;
		},

		nth: function( elem, i, match ) {
			return match[3] - 0 === i;
		},

		eq: function( elem, i, match ) {
			return match[3] - 0 === i;
		}
	},
	filter: {
		PSEUDO: function( elem, match, i, array ) {
			var name = match[1],
				filter = Expr.filters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );

			} else if ( name === "contains" ) {
				return (elem.textContent || elem.innerText || getText([ elem ]) || "").indexOf(match[3]) >= 0;

			} else if ( name === "not" ) {
				var not = match[3];

				for ( var j = 0, l = not.length; j < l; j++ ) {
					if ( not[j] === elem ) {
						return false;
					}
				}

				return true;

			} else {
				Sizzle.error( name );
			}
		},

		CHILD: function( elem, match ) {
			var first, last,
				doneName, parent, cache,
				count, diff,
				type = match[1],
				node = elem;

			switch ( type ) {
				case "only":
				case "first":
					while ( (node = node.previousSibling) )	 {
						if ( node.nodeType === 1 ) { 
							return false; 
						}
					}

					if ( type === "first" ) { 
						return true; 
					}

					node = elem;

				case "last":
					while ( (node = node.nextSibling) )	 {
						if ( node.nodeType === 1 ) { 
							return false; 
						}
					}

					return true;

				case "nth":
					first = match[2];
					last = match[3];

					if ( first === 1 && last === 0 ) {
						return true;
					}
					
					doneName = match[0];
					parent = elem.parentNode;
	
					if ( parent && (parent[ expando ] !== doneName || !elem.nodeIndex) ) {
						count = 0;
						
						for ( node = parent.firstChild; node; node = node.nextSibling ) {
							if ( node.nodeType === 1 ) {
								node.nodeIndex = ++count;
							}
						} 

						parent[ expando ] = doneName;
					}
					
					diff = elem.nodeIndex - last;

					if ( first === 0 ) {
						return diff === 0;

					} else {
						return ( diff % first === 0 && diff / first >= 0 );
					}
			}
		},

		ID: function( elem, match ) {
			return elem.nodeType === 1 && elem.getAttribute("id") === match;
		},

		TAG: function( elem, match ) {
			return (match === "*" && elem.nodeType === 1) || !!elem.nodeName && elem.nodeName.toLowerCase() === match;
		},
		
		CLASS: function( elem, match ) {
			return (" " + (elem.className || elem.getAttribute("class")) + " ")
				.indexOf( match ) > -1;
		},

		ATTR: function( elem, match ) {
			var name = match[1],
				result = Sizzle.attr ?
					Sizzle.attr( elem, name ) :
					Expr.attrHandle[ name ] ?
					Expr.attrHandle[ name ]( elem ) :
					elem[ name ] != null ?
						elem[ name ] :
						elem.getAttribute( name ),
				value = result + "",
				type = match[2],
				check = match[4];

			return result == null ?
				type === "!=" :
				!type && Sizzle.attr ?
				result != null :
				type === "=" ?
				value === check :
				type === "*=" ?
				value.indexOf(check) >= 0 :
				type === "~=" ?
				(" " + value + " ").indexOf(check) >= 0 :
				!check ?
				value && result !== false :
				type === "!=" ?
				value !== check :
				type === "^=" ?
				value.indexOf(check) === 0 :
				type === "$=" ?
				value.substr(value.length - check.length) === check :
				type === "|=" ?
				value === check || value.substr(0, check.length + 1) === check + "-" :
				false;
		},

		POS: function( elem, match, i, array ) {
			var name = match[2],
				filter = Expr.setFilters[ name ];

			if ( filter ) {
				return filter( elem, i, match, array );
			}
		}
	}
};

var origPOS = Expr.match.POS,
	fescape = function(all, num){
		return "\\" + (num - 0 + 1);
	};

for ( var type in Expr.match ) {
	Expr.match[ type ] = new RegExp( Expr.match[ type ].source + (/(?![^\[]*\])(?![^\(]*\))/.source) );
	Expr.leftMatch[ type ] = new RegExp( /(^(?:.|\r|\n)*?)/.source + Expr.match[ type ].source.replace(/\\(\d+)/g, fescape) );
}

var makeArray = function( array, results ) {
	array = Array.prototype.slice.call( array, 0 );

	if ( results ) {
		results.push.apply( results, array );
		return results;
	}
	
	return array;
};

// Perform a simple check to determine if the browser is capable of
// converting a NodeList to an array using builtin methods.
// Also verifies that the returned array holds DOM nodes
// (which is not the case in the Blackberry browser)
try {
	Array.prototype.slice.call( document.documentElement.childNodes, 0 )[0].nodeType;

// Provide a fallback method if it does not work
} catch( e ) {
	makeArray = function( array, results ) {
		var i = 0,
			ret = results || [];

		if ( toString.call(array) === "[object Array]" ) {
			Array.prototype.push.apply( ret, array );

		} else {
			if ( typeof array.length === "number" ) {
				for ( var l = array.length; i < l; i++ ) {
					ret.push( array[i] );
				}

			} else {
				for ( ; array[i]; i++ ) {
					ret.push( array[i] );
				}
			}
		}

		return ret;
	};
}

var sortOrder, siblingCheck;

if ( document.documentElement.compareDocumentPosition ) {
	sortOrder = function( a, b ) {
		if ( a === b ) {
			hasDuplicate = true;
			return 0;
		}

		if ( !a.compareDocumentPosition || !b.compareDocumentPosition ) {
			return a.compareDocumentPosition ? -1 : 1;
		}

		return a.compareDocumentPosition(b) & 4 ? -1 : 1;
	};

} else {
	sortOrder = function( a, b ) {
		// The nodes are identical, we can exit early
		if ( a === b ) {
			hasDuplicate = true;
			return 0;

		// Fallback to using sourceIndex (in IE) if it's available on both nodes
		} else if ( a.sourceIndex && b.sourceIndex ) {
			return a.sourceIndex - b.sourceIndex;
		}

		var al, bl,
			ap = [],
			bp = [],
			aup = a.parentNode,
			bup = b.parentNode,
			cur = aup;

		// If the nodes are siblings (or identical) we can do a quick check
		if ( aup === bup ) {
			return siblingCheck( a, b );

		// If no parents were found then the nodes are disconnected
		} else if ( !aup ) {
			return -1;

		} else if ( !bup ) {
			return 1;
		}

		// Otherwise they're somewhere else in the tree so we need
		// to build up a full list of the parentNodes for comparison
		while ( cur ) {
			ap.unshift( cur );
			cur = cur.parentNode;
		}

		cur = bup;

		while ( cur ) {
			bp.unshift( cur );
			cur = cur.parentNode;
		}

		al = ap.length;
		bl = bp.length;

		// Start walking down the tree looking for a discrepancy
		for ( var i = 0; i < al && i < bl; i++ ) {
			if ( ap[i] !== bp[i] ) {
				return siblingCheck( ap[i], bp[i] );
			}
		}

		// We ended someplace up the tree so do a sibling check
		return i === al ?
			siblingCheck( a, bp[i], -1 ) :
			siblingCheck( ap[i], b, 1 );
	};

	siblingCheck = function( a, b, ret ) {
		if ( a === b ) {
			return ret;
		}

		var cur = a.nextSibling;

		while ( cur ) {
			if ( cur === b ) {
				return -1;
			}

			cur = cur.nextSibling;
		}

		return 1;
	};
}

// Check to see if the browser returns elements by name when
// querying by getElementById (and provide a workaround)
(function(){
	// We're going to inject a fake input element with a specified name
	var form = document.createElement("div"),
		id = "script" + (new Date()).getTime(),
		root = document.documentElement;

	form.innerHTML = "<a name='" + id + "'/>";

	// Inject it into the root element, check its status, and remove it quickly
	root.insertBefore( form, root.firstChild );

	// The workaround has to do additional checks after a getElementById
	// Which slows things down for other browsers (hence the branching)
	if ( document.getElementById( id ) ) {
		Expr.find.ID = function( match, context, isXML ) {
			if ( typeof context.getElementById !== "undefined" && !isXML ) {
				var m = context.getElementById(match[1]);

				return m ?
					m.id === match[1] || typeof m.getAttributeNode !== "undefined" && m.getAttributeNode("id").nodeValue === match[1] ?
						[m] :
						undefined :
					[];
			}
		};

		Expr.filter.ID = function( elem, match ) {
			var node = typeof elem.getAttributeNode !== "undefined" && elem.getAttributeNode("id");

			return elem.nodeType === 1 && node && node.nodeValue === match;
		};
	}

	root.removeChild( form );

	// release memory in IE
	root = form = null;
})();

(function(){
	// Check to see if the browser returns only elements
	// when doing getElementsByTagName("*")

	// Create a fake element
	var div = document.createElement("div");
	div.appendChild( document.createComment("") );

	// Make sure no comments are found
	if ( div.getElementsByTagName("*").length > 0 ) {
		Expr.find.TAG = function( match, context ) {
			var results = context.getElementsByTagName( match[1] );

			// Filter out possible comments
			if ( match[1] === "*" ) {
				var tmp = [];

				for ( var i = 0; results[i]; i++ ) {
					if ( results[i].nodeType === 1 ) {
						tmp.push( results[i] );
					}
				}

				results = tmp;
			}

			return results;
		};
	}

	// Check to see if an attribute returns normalized href attributes
	div.innerHTML = "<a href='#'></a>";

	if ( div.firstChild && typeof div.firstChild.getAttribute !== "undefined" &&
			div.firstChild.getAttribute("href") !== "#" ) {

		Expr.attrHandle.href = function( elem ) {
			return elem.getAttribute( "href", 2 );
		};
	}

	// release memory in IE
	div = null;
})();

if ( document.querySelectorAll ) {
	(function(){
		var oldSizzle = Sizzle,
			div = document.createElement("div"),
			id = "__sizzle__";

		div.innerHTML = "<p class='TEST'></p>";

		// Safari can't handle uppercase or unicode characters when
		// in quirks mode.
		if ( div.querySelectorAll && div.querySelectorAll(".TEST").length === 0 ) {
			return;
		}
	
		Sizzle = function( query, context, extra, seed ) {
			context = context || document;

			// Only use querySelectorAll on non-XML documents
			// (ID selectors don't work in non-HTML documents)
			if ( !seed && !Sizzle.isXML(context) ) {
				// See if we find a selector to speed up
				var match = /^(\w+$)|^\.([\w\-]+$)|^#([\w\-]+$)/.exec( query );
				
				if ( match && (context.nodeType === 1 || context.nodeType === 9) ) {
					// Speed-up: Sizzle("TAG")
					if ( match[1] ) {
						return makeArray( context.getElementsByTagName( query ), extra );
					
					// Speed-up: Sizzle(".CLASS")
					} else if ( match[2] && Expr.find.CLASS && context.getElementsByClassName ) {
						return makeArray( context.getElementsByClassName( match[2] ), extra );
					}
				}
				
				if ( context.nodeType === 9 ) {
					// Speed-up: Sizzle("body")
					// The body element only exists once, optimize finding it
					if ( query === "body" && context.body ) {
						return makeArray( [ context.body ], extra );
						
					// Speed-up: Sizzle("#ID")
					} else if ( match && match[3] ) {
						var elem = context.getElementById( match[3] );

						// Check parentNode to catch when Blackberry 4.6 returns
						// nodes that are no longer in the document #6963
						if ( elem && elem.parentNode ) {
							// Handle the case where IE and Opera return items
							// by name instead of ID
							if ( elem.id === match[3] ) {
								return makeArray( [ elem ], extra );
							}
							
						} else {
							return makeArray( [], extra );
						}
					}
					
					try {
						return makeArray( context.querySelectorAll(query), extra );
					} catch(qsaError) {}

				// qSA works strangely on Element-rooted queries
				// We can work around this by specifying an extra ID on the root
				// and working up from there (Thanks to Andrew Dupont for the technique)
				// IE 8 doesn't work on object elements
				} else if ( context.nodeType === 1 && context.nodeName.toLowerCase() !== "object" ) {
					var oldContext = context,
						old = context.getAttribute( "id" ),
						nid = old || id,
						hasParent = context.parentNode,
						relativeHierarchySelector = /^\s*[+~]/.test( query );

					if ( !old ) {
						context.setAttribute( "id", nid );
					} else {
						nid = nid.replace( /'/g, "\\$&" );
					}
					if ( relativeHierarchySelector && hasParent ) {
						context = context.parentNode;
					}

					try {
						if ( !relativeHierarchySelector || hasParent ) {
							return makeArray( context.querySelectorAll( "[id='" + nid + "'] " + query ), extra );
						}

					} catch(pseudoError) {
					} finally {
						if ( !old ) {
							oldContext.removeAttribute( "id" );
						}
					}
				}
			}
		
			return oldSizzle(query, context, extra, seed);
		};

		for ( var prop in oldSizzle ) {
			Sizzle[ prop ] = oldSizzle[ prop ];
		}

		// release memory in IE
		div = null;
	})();
}

(function(){
	var html = document.documentElement,
		matches = html.matchesSelector || html.mozMatchesSelector || html.webkitMatchesSelector || html.msMatchesSelector;

	if ( matches ) {
		// Check to see if it's possible to do matchesSelector
		// on a disconnected node (IE 9 fails this)
		var disconnectedMatch = !matches.call( document.createElement( "div" ), "div" ),
			pseudoWorks = false;

		try {
			// This should fail with an exception
			// Gecko does not error, returns false instead
			matches.call( document.documentElement, "[test!='']:sizzle" );
	
		} catch( pseudoError ) {
			pseudoWorks = true;
		}

		Sizzle.matchesSelector = function( node, expr ) {
			// Make sure that attribute selectors are quoted
			expr = expr.replace(/\=\s*([^'"\]]*)\s*\]/g, "='$1']");

			if ( !Sizzle.isXML( node ) ) {
				try { 
					if ( pseudoWorks || !Expr.match.PSEUDO.test( expr ) && !/!=/.test( expr ) ) {
						var ret = matches.call( node, expr );

						// IE 9's matchesSelector returns false on disconnected nodes
						if ( ret || !disconnectedMatch ||
								// As well, disconnected nodes are said to be in a document
								// fragment in IE 9, so check for that
								node.document && node.document.nodeType !== 11 ) {
							return ret;
						}
					}
				} catch(e) {}
			}

			return Sizzle(expr, null, null, [node]).length > 0;
		};
	}
})();

(function(){
	var div = document.createElement("div");

	div.innerHTML = "<div class='test e'></div><div class='test'></div>";

	// Opera can't find a second classname (in 9.6)
	// Also, make sure that getElementsByClassName actually exists
	if ( !div.getElementsByClassName || div.getElementsByClassName("e").length === 0 ) {
		return;
	}

	// Safari caches class attributes, doesn't catch changes (in 3.2)
	div.lastChild.className = "e";

	if ( div.getElementsByClassName("e").length === 1 ) {
		return;
	}
	
	Expr.order.splice(1, 0, "CLASS");
	Expr.find.CLASS = function( match, context, isXML ) {
		if ( typeof context.getElementsByClassName !== "undefined" && !isXML ) {
			return context.getElementsByClassName(match[1]);
		}
	};

	// release memory in IE
	div = null;
})();

function dirNodeCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];

		if ( elem ) {
			var match = false;

			elem = elem[dir];

			while ( elem ) {
				if ( elem[ expando ] === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 && !isXML ){
					elem[ expando ] = doneName;
					elem.sizset = i;
				}

				if ( elem.nodeName.toLowerCase() === cur ) {
					match = elem;
					break;
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

function dirCheck( dir, cur, doneName, checkSet, nodeCheck, isXML ) {
	for ( var i = 0, l = checkSet.length; i < l; i++ ) {
		var elem = checkSet[i];

		if ( elem ) {
			var match = false;
			
			elem = elem[dir];

			while ( elem ) {
				if ( elem[ expando ] === doneName ) {
					match = checkSet[elem.sizset];
					break;
				}

				if ( elem.nodeType === 1 ) {
					if ( !isXML ) {
						elem[ expando ] = doneName;
						elem.sizset = i;
					}

					if ( typeof cur !== "string" ) {
						if ( elem === cur ) {
							match = true;
							break;
						}

					} else if ( Sizzle.filter( cur, [elem] ).length > 0 ) {
						match = elem;
						break;
					}
				}

				elem = elem[dir];
			}

			checkSet[i] = match;
		}
	}
}

if ( document.documentElement.contains ) {
	Sizzle.contains = function( a, b ) {
		return a !== b && (a.contains ? a.contains(b) : true);
	};

} else if ( document.documentElement.compareDocumentPosition ) {
	Sizzle.contains = function( a, b ) {
		return !!(a.compareDocumentPosition(b) & 16);
	};

} else {
	Sizzle.contains = function() {
		return false;
	};
}

Sizzle.isXML = function( elem ) {
	// documentElement is verified for cases where it doesn't yet exist
	// (such as loading iframes in IE - #4833) 
	var documentElement = (elem ? elem.ownerDocument || elem : 0).documentElement;

	return documentElement ? documentElement.nodeName !== "HTML" : false;
};

var posProcess = function( selector, context, seed ) {
	var match,
		tmpSet = [],
		later = "",
		root = context.nodeType ? [context] : context;

	// Position selectors must be done after the filter
	// And so must :not(positional) so we move all PSEUDOs to the end
	while ( (match = Expr.match.PSEUDO.exec( selector )) ) {
		later += match[0];
		selector = selector.replace( Expr.match.PSEUDO, "" );
	}

	selector = Expr.relative[selector] ? selector + "*" : selector;

	for ( var i = 0, l = root.length; i < l; i++ ) {
		Sizzle( selector, root[i], tmpSet, seed );
	}

	return Sizzle.filter( later, tmpSet );
};

// EXPOSE
// Override sizzle attribute retrieval
Sizzle.attr = jQuery.attr;
Sizzle.selectors.attrMap = {};
jQuery.find = Sizzle;
jQuery.expr = Sizzle.selectors;
jQuery.expr[":"] = jQuery.expr.filters;
jQuery.unique = Sizzle.uniqueSort;
jQuery.text = Sizzle.getText;
jQuery.isXMLDoc = Sizzle.isXML;
jQuery.contains = Sizzle.contains;


})();


var runtil = /Until$/,
	rparentsprev = /^(?:parents|prevUntil|prevAll)/,
	// Note: This RegExp should be improved, or likely pulled from Sizzle
	rmultiselector = /,/,
	isSimple = /^.[^:#\[\.,]*$/,
	slice = Array.prototype.slice,
	POS = jQuery.expr.match.POS,
	// methods guaranteed to produce a unique set when starting from a unique set
	guaranteedUnique = {
		children: true,
		contents: true,
		next: true,
		prev: true
	};

jQuery.fn.extend({
	find: function( selector ) {
		var self = this,
			i, l;

		if ( typeof selector !== "string" ) {
			return jQuery( selector ).filter(function() {
				for ( i = 0, l = self.length; i < l; i++ ) {
					if ( jQuery.contains( self[ i ], this ) ) {
						return true;
					}
				}
			});
		}

		var ret = this.pushStack( "", "find", selector ),
			length, n, r;

		for ( i = 0, l = this.length; i < l; i++ ) {
			length = ret.length;
			jQuery.find( selector, this[i], ret );

			if ( i > 0 ) {
				// Make sure that the results are unique
				for ( n = length; n < ret.length; n++ ) {
					for ( r = 0; r < length; r++ ) {
						if ( ret[r] === ret[n] ) {
							ret.splice(n--, 1);
							break;
						}
					}
				}
			}
		}

		return ret;
	},

	has: function( target ) {
		var targets = jQuery( target );
		return this.filter(function() {
			for ( var i = 0, l = targets.length; i < l; i++ ) {
				if ( jQuery.contains( this, targets[i] ) ) {
					return true;
				}
			}
		});
	},

	not: function( selector ) {
		return this.pushStack( winnow(this, selector, false), "not", selector);
	},

	filter: function( selector ) {
		return this.pushStack( winnow(this, selector, true), "filter", selector );
	},

	is: function( selector ) {
		return !!selector && ( 
			typeof selector === "string" ?
				// If this is a positional selector, check membership in the returned set
				// so $("p:first").is("p:last") won't return true for a doc with two "p".
				POS.test( selector ) ? 
					jQuery( selector, this.context ).index( this[0] ) >= 0 :
					jQuery.filter( selector, this ).length > 0 :
				this.filter( selector ).length > 0 );
	},

	closest: function( selectors, context ) {
		var ret = [], i, l, cur = this[0];
		
		// Array (deprecated as of jQuery 1.7)
		if ( jQuery.isArray( selectors ) ) {
			var level = 1;

			while ( cur && cur.ownerDocument && cur !== context ) {
				for ( i = 0; i < selectors.length; i++ ) {

					if ( jQuery( cur ).is( selectors[ i ] ) ) {
						ret.push({ selector: selectors[ i ], elem: cur, level: level });
					}
				}

				cur = cur.parentNode;
				level++;
			}

			return ret;
		}

		// String
		var pos = POS.test( selectors ) || typeof selectors !== "string" ?
				jQuery( selectors, context || this.context ) :
				0;

		for ( i = 0, l = this.length; i < l; i++ ) {
			cur = this[i];

			while ( cur ) {
				if ( pos ? pos.index(cur) > -1 : jQuery.find.matchesSelector(cur, selectors) ) {
					ret.push( cur );
					break;

				} else {
					cur = cur.parentNode;
					if ( !cur || !cur.ownerDocument || cur === context || cur.nodeType === 11 ) {
						break;
					}
				}
			}
		}

		ret = ret.length > 1 ? jQuery.unique( ret ) : ret;

		return this.pushStack( ret, "closest", selectors );
	},

	// Determine the position of an element within
	// the matched set of elements
	index: function( elem ) {

		// No argument, return index in parent
		if ( !elem ) {
			return ( this[0] && this[0].parentNode ) ? this.prevAll().length : -1;
		}

		// index in selector
		if ( typeof elem === "string" ) {
			return jQuery.inArray( this[0], jQuery( elem ) );
		}

		// Locate the position of the desired element
		return jQuery.inArray(
			// If it receives a jQuery object, the first element is used
			elem.jquery ? elem[0] : elem, this );
	},

	add: function( selector, context ) {
		var set = typeof selector === "string" ?
				jQuery( selector, context ) :
				jQuery.makeArray( selector && selector.nodeType ? [ selector ] : selector ),
			all = jQuery.merge( this.get(), set );

		return this.pushStack( isDisconnected( set[0] ) || isDisconnected( all[0] ) ?
			all :
			jQuery.unique( all ) );
	},

	andSelf: function() {
		return this.add( this.prevObject );
	}
});

// A painfully simple check to see if an element is disconnected
// from a document (should be improved, where feasible).
function isDisconnected( node ) {
	return !node || !node.parentNode || node.parentNode.nodeType === 11;
}

jQuery.each({
	parent: function( elem ) {
		var parent = elem.parentNode;
		return parent && parent.nodeType !== 11 ? parent : null;
	},
	parents: function( elem ) {
		return jQuery.dir( elem, "parentNode" );
	},
	parentsUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "parentNode", until );
	},
	next: function( elem ) {
		return jQuery.nth( elem, 2, "nextSibling" );
	},
	prev: function( elem ) {
		return jQuery.nth( elem, 2, "previousSibling" );
	},
	nextAll: function( elem ) {
		return jQuery.dir( elem, "nextSibling" );
	},
	prevAll: function( elem ) {
		return jQuery.dir( elem, "previousSibling" );
	},
	nextUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "nextSibling", until );
	},
	prevUntil: function( elem, i, until ) {
		return jQuery.dir( elem, "previousSibling", until );
	},
	siblings: function( elem ) {
		return jQuery.sibling( elem.parentNode.firstChild, elem );
	},
	children: function( elem ) {
		return jQuery.sibling( elem.firstChild );
	},
	contents: function( elem ) {
		return jQuery.nodeName( elem, "iframe" ) ?
			elem.contentDocument || elem.contentWindow.document :
			jQuery.makeArray( elem.childNodes );
	}
}, function( name, fn ) {
	jQuery.fn[ name ] = function( until, selector ) {
		var ret = jQuery.map( this, fn, until );

		if ( !runtil.test( name ) ) {
			selector = until;
		}

		if ( selector && typeof selector === "string" ) {
			ret = jQuery.filter( selector, ret );
		}

		ret = this.length > 1 && !guaranteedUnique[ name ] ? jQuery.unique( ret ) : ret;

		if ( (this.length > 1 || rmultiselector.test( selector )) && rparentsprev.test( name ) ) {
			ret = ret.reverse();
		}

		return this.pushStack( ret, name, slice.call( arguments ).join(",") );
	};
});

jQuery.extend({
	filter: function( expr, elems, not ) {
		if ( not ) {
			expr = ":not(" + expr + ")";
		}

		return elems.length === 1 ?
			jQuery.find.matchesSelector(elems[0], expr) ? [ elems[0] ] : [] :
			jQuery.find.matches(expr, elems);
	},

	dir: function( elem, dir, until ) {
		var matched = [],
			cur = elem[ dir ];

		while ( cur && cur.nodeType !== 9 && (until === undefined || cur.nodeType !== 1 || !jQuery( cur ).is( until )) ) {
			if ( cur.nodeType === 1 ) {
				matched.push( cur );
			}
			cur = cur[dir];
		}
		return matched;
	},

	nth: function( cur, result, dir, elem ) {
		result = result || 1;
		var num = 0;

		for ( ; cur; cur = cur[dir] ) {
			if ( cur.nodeType === 1 && ++num === result ) {
				break;
			}
		}

		return cur;
	},

	sibling: function( n, elem ) {
		var r = [];

		for ( ; n; n = n.nextSibling ) {
			if ( n.nodeType === 1 && n !== elem ) {
				r.push( n );
			}
		}

		return r;
	}
});

// Implement the identical functionality for filter and not
function winnow( elements, qualifier, keep ) {

	// Can't pass null or undefined to indexOf in Firefox 4
	// Set to 0 to skip string check
	qualifier = qualifier || 0;

	if ( jQuery.isFunction( qualifier ) ) {
		return jQuery.grep(elements, function( elem, i ) {
			var retVal = !!qualifier.call( elem, i, elem );
			return retVal === keep;
		});

	} else if ( qualifier.nodeType ) {
		return jQuery.grep(elements, function( elem, i ) {
			return ( elem === qualifier ) === keep;
		});

	} else if ( typeof qualifier === "string" ) {
		var filtered = jQuery.grep(elements, function( elem ) {
			return elem.nodeType === 1;
		});

		if ( isSimple.test( qualifier ) ) {
			return jQuery.filter(qualifier, filtered, !keep);
		} else {
			qualifier = jQuery.filter( qualifier, filtered );
		}
	}

	return jQuery.grep(elements, function( elem, i ) {
		return ( jQuery.inArray( elem, qualifier ) >= 0 ) === keep;
	});
}




function createSafeFragment( document ) {
	var list = nodeNames.split( "|" ),
	safeFrag = document.createDocumentFragment();

	if ( safeFrag.createElement ) {
		while ( list.length ) {
			safeFrag.createElement(
				list.pop()
			);
		}
	}
	return safeFrag;
}

var nodeNames = "abbr|article|aside|audio|canvas|datalist|details|figcaption|figure|footer|" +
		"header|hgroup|mark|meter|nav|output|progress|section|summary|time|video",
	rinlinejQuery = / jQuery\d+="(?:\d+|null)"/g,
	rleadingWhitespace = /^\s+/,
	rxhtmlTag = /<(?!area|br|col|embed|hr|img|input|link|meta|param)(([\w:]+)[^>]*)\/>/ig,
	rtagName = /<([\w:]+)/,
	rtbody = /<tbody/i,
	rhtml = /<|&#?\w+;/,
	rnoInnerhtml = /<(?:script|style)/i,
	rnocache = /<(?:script|object|embed|option|style)/i,
	rnoshimcache = new RegExp("<(?:" + nodeNames + ")", "i"),
	// checked="checked" or checked
	rchecked = /checked\s*(?:[^=]|=\s*.checked.)/i,
	rscriptType = /\/(java|ecma)script/i,
	rcleanScript = /^\s*<!(?:\[CDATA\[|\-\-)/,
	wrapMap = {
		option: [ 1, "<select multiple='multiple'>", "</select>" ],
		legend: [ 1, "<fieldset>", "</fieldset>" ],
		thead: [ 1, "<table>", "</table>" ],
		tr: [ 2, "<table><tbody>", "</tbody></table>" ],
		td: [ 3, "<table><tbody><tr>", "</tr></tbody></table>" ],
		col: [ 2, "<table><tbody></tbody><colgroup>", "</colgroup></table>" ],
		area: [ 1, "<map>", "</map>" ],
		_default: [ 0, "", "" ]
	},
	safeFragment = createSafeFragment( document );

wrapMap.optgroup = wrapMap.option;
wrapMap.tbody = wrapMap.tfoot = wrapMap.colgroup = wrapMap.caption = wrapMap.thead;
wrapMap.th = wrapMap.td;

// IE can't serialize <link> and <script> tags normally
if ( !jQuery.support.htmlSerialize ) {
	wrapMap._default = [ 1, "div<div>", "</div>" ];
}

jQuery.fn.extend({
	text: function( text ) {
		if ( jQuery.isFunction(text) ) {
			return this.each(function(i) {
				var self = jQuery( this );

				self.text( text.call(this, i, self.text()) );
			});
		}

		if ( typeof text !== "object" && text !== undefined ) {
			return this.empty().append( (this[0] && this[0].ownerDocument || document).createTextNode( text ) );
		}

		return jQuery.text( this );
	},

	wrapAll: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function(i) {
				jQuery(this).wrapAll( html.call(this, i) );
			});
		}

		if ( this[0] ) {
			// The elements to wrap the target around
			var wrap = jQuery( html, this[0].ownerDocument ).eq(0).clone(true);

			if ( this[0].parentNode ) {
				wrap.insertBefore( this[0] );
			}

			wrap.map(function() {
				var elem = this;

				while ( elem.firstChild && elem.firstChild.nodeType === 1 ) {
					elem = elem.firstChild;
				}

				return elem;
			}).append( this );
		}

		return this;
	},

	wrapInner: function( html ) {
		if ( jQuery.isFunction( html ) ) {
			return this.each(function(i) {
				jQuery(this).wrapInner( html.call(this, i) );
			});
		}

		return this.each(function() {
			var self = jQuery( this ),
				contents = self.contents();

			if ( contents.length ) {
				contents.wrapAll( html );

			} else {
				self.append( html );
			}
		});
	},

	wrap: function( html ) {
		var isFunction = jQuery.isFunction( html );

		return this.each(function(i) {
			jQuery( this ).wrapAll( isFunction ? html.call(this, i) : html );
		});
	},

	unwrap: function() {
		return this.parent().each(function() {
			if ( !jQuery.nodeName( this, "body" ) ) {
				jQuery( this ).replaceWith( this.childNodes );
			}
		}).end();
	},

	append: function() {
		return this.domManip(arguments, true, function( elem ) {
			if ( this.nodeType === 1 ) {
				this.appendChild( elem );
			}
		});
	},

	prepend: function() {
		return this.domManip(arguments, true, function( elem ) {
			if ( this.nodeType === 1 ) {
				this.insertBefore( elem, this.firstChild );
			}
		});
	},

	before: function() {
		if ( this[0] && this[0].parentNode ) {
			return this.domManip(arguments, false, function( elem ) {
				this.parentNode.insertBefore( elem, this );
			});
		} else if ( arguments.length ) {
			var set = jQuery.clean( arguments );
			set.push.apply( set, this.toArray() );
			return this.pushStack( set, "before", arguments );
		}
	},

	after: function() {
		if ( this[0] && this[0].parentNode ) {
			return this.domManip(arguments, false, function( elem ) {
				this.parentNode.insertBefore( elem, this.nextSibling );
			});
		} else if ( arguments.length ) {
			var set = this.pushStack( this, "after", arguments );
			set.push.apply( set, jQuery.clean(arguments) );
			return set;
		}
	},

	// keepData is for internal use only--do not document
	remove: function( selector, keepData ) {
		for ( var i = 0, elem; (elem = this[i]) != null; i++ ) {
			if ( !selector || jQuery.filter( selector, [ elem ] ).length ) {
				if ( !keepData && elem.nodeType === 1 ) {
					jQuery.cleanData( elem.getElementsByTagName("*") );
					jQuery.cleanData( [ elem ] );
				}

				if ( elem.parentNode ) {
					elem.parentNode.removeChild( elem );
				}
			}
		}

		return this;
	},

	empty: function() {
		for ( var i = 0, elem; (elem = this[i]) != null; i++ ) {
			// Remove element nodes and prevent memory leaks
			if ( elem.nodeType === 1 ) {
				jQuery.cleanData( elem.getElementsByTagName("*") );
			}

			// Remove any remaining nodes
			while ( elem.firstChild ) {
				elem.removeChild( elem.firstChild );
			}
		}

		return this;
	},

	clone: function( dataAndEvents, deepDataAndEvents ) {
		dataAndEvents = dataAndEvents == null ? false : dataAndEvents;
		deepDataAndEvents = deepDataAndEvents == null ? dataAndEvents : deepDataAndEvents;

		return this.map( function () {
			return jQuery.clone( this, dataAndEvents, deepDataAndEvents );
		});
	},

	html: function( value ) {
		if ( value === undefined ) {
			return this[0] && this[0].nodeType === 1 ?
				this[0].innerHTML.replace(rinlinejQuery, "") :
				null;

		// See if we can take a shortcut and just use innerHTML
		} else if ( typeof value === "string" && !rnoInnerhtml.test( value ) &&
			(jQuery.support.leadingWhitespace || !rleadingWhitespace.test( value )) &&
			!wrapMap[ (rtagName.exec( value ) || ["", ""])[1].toLowerCase() ] ) {

			value = value.replace(rxhtmlTag, "<$1></$2>");

			try {
				for ( var i = 0, l = this.length; i < l; i++ ) {
					// Remove element nodes and prevent memory leaks
					if ( this[i].nodeType === 1 ) {
						jQuery.cleanData( this[i].getElementsByTagName("*") );
						this[i].innerHTML = value;
					}
				}

			// If using innerHTML throws an exception, use the fallback method
			} catch(e) {
				this.empty().append( value );
			}

		} else if ( jQuery.isFunction( value ) ) {
			this.each(function(i){
				var self = jQuery( this );

				self.html( value.call(this, i, self.html()) );
			});

		} else {
			this.empty().append( value );
		}

		return this;
	},

	replaceWith: function( value ) {
		if ( this[0] && this[0].parentNode ) {
			// Make sure that the elements are removed from the DOM before they are inserted
			// this can help fix replacing a parent with child elements
			if ( jQuery.isFunction( value ) ) {
				return this.each(function(i) {
					var self = jQuery(this), old = self.html();
					self.replaceWith( value.call( this, i, old ) );
				});
			}

			if ( typeof value !== "string" ) {
				value = jQuery( value ).detach();
			}

			return this.each(function() {
				var next = this.nextSibling,
					parent = this.parentNode;

				jQuery( this ).remove();

				if ( next ) {
					jQuery(next).before( value );
				} else {
					jQuery(parent).append( value );
				}
			});
		} else {
			return this.length ?
				this.pushStack( jQuery(jQuery.isFunction(value) ? value() : value), "replaceWith", value ) :
				this;
		}
	},

	detach: function( selector ) {
		return this.remove( selector, true );
	},

	domManip: function( args, table, callback ) {
		var results, first, fragment, parent,
			value = args[0],
			scripts = [];

		// We can't cloneNode fragments that contain checked, in WebKit
		if ( !jQuery.support.checkClone && arguments.length === 3 && typeof value === "string" && rchecked.test( value ) ) {
			return this.each(function() {
				jQuery(this).domManip( args, table, callback, true );
			});
		}

		if ( jQuery.isFunction(value) ) {
			return this.each(function(i) {
				var self = jQuery(this);
				args[0] = value.call(this, i, table ? self.html() : undefined);
				self.domManip( args, table, callback );
			});
		}

		if ( this[0] ) {
			parent = value && value.parentNode;

			// If we're in a fragment, just use that instead of building a new one
			if ( jQuery.support.parentNode && parent && parent.nodeType === 11 && parent.childNodes.length === this.length ) {
				results = { fragment: parent };

			} else {
				results = jQuery.buildFragment( args, this, scripts );
			}

			fragment = results.fragment;

			if ( fragment.childNodes.length === 1 ) {
				first = fragment = fragment.firstChild;
			} else {
				first = fragment.firstChild;
			}

			if ( first ) {
				table = table && jQuery.nodeName( first, "tr" );

				for ( var i = 0, l = this.length, lastIndex = l - 1; i < l; i++ ) {
					callback.call(
						table ?
							root(this[i], first) :
							this[i],
						// Make sure that we do not leak memory by inadvertently discarding
						// the original fragment (which might have attached data) instead of
						// using it; in addition, use the original fragment object for the last
						// item instead of first because it can end up being emptied incorrectly
						// in certain situations (Bug #8070).
						// Fragments from the fragment cache must always be cloned and never used
						// in place.
						results.cacheable || ( l > 1 && i < lastIndex ) ?
							jQuery.clone( fragment, true, true ) :
							fragment
					);
				}
			}

			if ( scripts.length ) {
				jQuery.each( scripts, evalScript );
			}
		}

		return this;
	}
});

function root( elem, cur ) {
	return jQuery.nodeName(elem, "table") ?
		(elem.getElementsByTagName("tbody")[0] ||
		elem.appendChild(elem.ownerDocument.createElement("tbody"))) :
		elem;
}

function cloneCopyEvent( src, dest ) {

	if ( dest.nodeType !== 1 || !jQuery.hasData( src ) ) {
		return;
	}

	var type, i, l,
		oldData = jQuery._data( src ),
		curData = jQuery._data( dest, oldData ),
		events = oldData.events;

	if ( events ) {
		delete curData.handle;
		curData.events = {};

		for ( type in events ) {
			for ( i = 0, l = events[ type ].length; i < l; i++ ) {
				jQuery.event.add( dest, type + ( events[ type ][ i ].namespace ? "." : "" ) + events[ type ][ i ].namespace, events[ type ][ i ], events[ type ][ i ].data );
			}
		}
	}

	// make the cloned public data object a copy from the original
	if ( curData.data ) {
		curData.data = jQuery.extend( {}, curData.data );
	}
}

function cloneFixAttributes( src, dest ) {
	var nodeName;

	// We do not need to do anything for non-Elements
	if ( dest.nodeType !== 1 ) {
		return;
	}

	// clearAttributes removes the attributes, which we don't want,
	// but also removes the attachEvent events, which we *do* want
	if ( dest.clearAttributes ) {
		dest.clearAttributes();
	}

	// mergeAttributes, in contrast, only merges back on the
	// original attributes, not the events
	if ( dest.mergeAttributes ) {
		dest.mergeAttributes( src );
	}

	nodeName = dest.nodeName.toLowerCase();

	// IE6-8 fail to clone children inside object elements that use
	// the proprietary classid attribute value (rather than the type
	// attribute) to identify the type of content to display
	if ( nodeName === "object" ) {
		dest.outerHTML = src.outerHTML;

	} else if ( nodeName === "input" && (src.type === "checkbox" || src.type === "radio") ) {
		// IE6-8 fails to persist the checked state of a cloned checkbox
		// or radio button. Worse, IE6-7 fail to give the cloned element
		// a checked appearance if the defaultChecked value isn't also set
		if ( src.checked ) {
			dest.defaultChecked = dest.checked = src.checked;
		}

		// IE6-7 get confused and end up setting the value of a cloned
		// checkbox/radio button to an empty string instead of "on"
		if ( dest.value !== src.value ) {
			dest.value = src.value;
		}

	// IE6-8 fails to return the selected option to the default selected
	// state when cloning options
	} else if ( nodeName === "option" ) {
		dest.selected = src.defaultSelected;

	// IE6-8 fails to set the defaultValue to the correct value when
	// cloning other types of input fields
	} else if ( nodeName === "input" || nodeName === "textarea" ) {
		dest.defaultValue = src.defaultValue;
	}

	// Event data gets referenced instead of copied if the expando
	// gets copied too
	dest.removeAttribute( jQuery.expando );
}

jQuery.buildFragment = function( args, nodes, scripts ) {
	var fragment, cacheable, cacheresults, doc,
	first = args[ 0 ];

	// nodes may contain either an explicit document object,
	// a jQuery collection or context object.
	// If nodes[0] contains a valid object to assign to doc
	if ( nodes && nodes[0] ) {
		doc = nodes[0].ownerDocument || nodes[0];
	}

	// Ensure that an attr object doesn't incorrectly stand in as a document object
	// Chrome and Firefox seem to allow this to occur and will throw exception
	// Fixes #8950
	if ( !doc.createDocumentFragment ) {
		doc = document;
	}

	// Only cache "small" (1/2 KB) HTML strings that are associated with the main document
	// Cloning options loses the selected state, so don't cache them
	// IE 6 doesn't like it when you put <object> or <embed> elements in a fragment
	// Also, WebKit does not clone 'checked' attributes on cloneNode, so don't cache
	// Lastly, IE6,7,8 will not correctly reuse cached fragments that were created from unknown elems #10501
	if ( args.length === 1 && typeof first === "string" && first.length < 512 && doc === document &&
		first.charAt(0) === "<" && !rnocache.test( first ) &&
		(jQuery.support.checkClone || !rchecked.test( first )) &&
		(jQuery.support.html5Clone || !rnoshimcache.test( first )) ) {

		cacheable = true;

		cacheresults = jQuery.fragments[ first ];
		if ( cacheresults && cacheresults !== 1 ) {
			fragment = cacheresults;
		}
	}

	if ( !fragment ) {
		fragment = doc.createDocumentFragment();
		jQuery.clean( args, doc, fragment, scripts );
	}

	if ( cacheable ) {
		jQuery.fragments[ first ] = cacheresults ? fragment : 1;
	}

	return { fragment: fragment, cacheable: cacheable };
};

jQuery.fragments = {};

jQuery.each({
	appendTo: "append",
	prependTo: "prepend",
	insertBefore: "before",
	insertAfter: "after",
	replaceAll: "replaceWith"
}, function( name, original ) {
	jQuery.fn[ name ] = function( selector ) {
		var ret = [],
			insert = jQuery( selector ),
			parent = this.length === 1 && this[0].parentNode;

		if ( parent && parent.nodeType === 11 && parent.childNodes.length === 1 && insert.length === 1 ) {
			insert[ original ]( this[0] );
			return this;

		} else {
			for ( var i = 0, l = insert.length; i < l; i++ ) {
				var elems = ( i > 0 ? this.clone(true) : this ).get();
				jQuery( insert[i] )[ original ]( elems );
				ret = ret.concat( elems );
			}

			return this.pushStack( ret, name, insert.selector );
		}
	};
});

function getAll( elem ) {
	if ( typeof elem.getElementsByTagName !== "undefined" ) {
		return elem.getElementsByTagName( "*" );

	} else if ( typeof elem.querySelectorAll !== "undefined" ) {
		return elem.querySelectorAll( "*" );

	} else {
		return [];
	}
}

// Used in clean, fixes the defaultChecked property
function fixDefaultChecked( elem ) {
	if ( elem.type === "checkbox" || elem.type === "radio" ) {
		elem.defaultChecked = elem.checked;
	}
}
// Finds all inputs and passes them to fixDefaultChecked
function findInputs( elem ) {
	var nodeName = ( elem.nodeName || "" ).toLowerCase();
	if ( nodeName === "input" ) {
		fixDefaultChecked( elem );
	// Skip scripts, get other children
	} else if ( nodeName !== "script" && typeof elem.getElementsByTagName !== "undefined" ) {
		jQuery.grep( elem.getElementsByTagName("input"), fixDefaultChecked );
	}
}

// Derived From: http://www.iecss.com/shimprove/javascript/shimprove.1-0-1.js
function shimCloneNode( elem ) {
	var div = document.createElement( "div" );
	safeFragment.appendChild( div );

	div.innerHTML = elem.outerHTML;
	return div.firstChild;
}

jQuery.extend({
	clone: function( elem, dataAndEvents, deepDataAndEvents ) {
		var srcElements,
			destElements,
			i,
			// IE<=8 does not properly clone detached, unknown element nodes
			clone = jQuery.support.html5Clone || !rnoshimcache.test( "<" + elem.nodeName ) ?
				elem.cloneNode( true ) :
				shimCloneNode( elem );

		if ( (!jQuery.support.noCloneEvent || !jQuery.support.noCloneChecked) &&
				(elem.nodeType === 1 || elem.nodeType === 11) && !jQuery.isXMLDoc(elem) ) {
			// IE copies events bound via attachEvent when using cloneNode.
			// Calling detachEvent on the clone will also remove the events
			// from the original. In order to get around this, we use some
			// proprietary methods to clear the events. Thanks to MooTools
			// guys for this hotness.

			cloneFixAttributes( elem, clone );

			// Using Sizzle here is crazy slow, so we use getElementsByTagName instead
			srcElements = getAll( elem );
			destElements = getAll( clone );

			// Weird iteration because IE will replace the length property
			// with an element if you are cloning the body and one of the
			// elements on the page has a name or id of "length"
			for ( i = 0; srcElements[i]; ++i ) {
				// Ensure that the destination node is not null; Fixes #9587
				if ( destElements[i] ) {
					cloneFixAttributes( srcElements[i], destElements[i] );
				}
			}
		}

		// Copy the events from the original to the clone
		if ( dataAndEvents ) {
			cloneCopyEvent( elem, clone );

			if ( deepDataAndEvents ) {
				srcElements = getAll( elem );
				destElements = getAll( clone );

				for ( i = 0; srcElements[i]; ++i ) {
					cloneCopyEvent( srcElements[i], destElements[i] );
				}
			}
		}

		srcElements = destElements = null;

		// Return the cloned set
		return clone;
	},

	clean: function( elems, context, fragment, scripts ) {
		var checkScriptType;

		context = context || document;

		// !context.createElement fails in IE with an error but returns typeof 'object'
		if ( typeof context.createElement === "undefined" ) {
			context = context.ownerDocument || context[0] && context[0].ownerDocument || document;
		}

		var ret = [], j;

		for ( var i = 0, elem; (elem = elems[i]) != null; i++ ) {
			if ( typeof elem === "number" ) {
				elem += "";
			}

			if ( !elem ) {
				continue;
			}

			// Convert html string into DOM nodes
			if ( typeof elem === "string" ) {
				if ( !rhtml.test( elem ) ) {
					elem = context.createTextNode( elem );
				} else {
					// Fix "XHTML"-style tags in all browsers
					elem = elem.replace(rxhtmlTag, "<$1></$2>");

					// Trim whitespace, otherwise indexOf won't work as expected
					var tag = ( rtagName.exec( elem ) || ["", ""] )[1].toLowerCase(),
						wrap = wrapMap[ tag ] || wrapMap._default,
						depth = wrap[0],
						div = context.createElement("div");

					// Append wrapper element to unknown element safe doc fragment
					if ( context === document ) {
						// Use the fragment we've already created for this document
						safeFragment.appendChild( div );
					} else {
						// Use a fragment created with the owner document
						createSafeFragment( context ).appendChild( div );
					}

					// Go to html and back, then peel off extra wrappers
					div.innerHTML = wrap[1] + elem + wrap[2];

					// Move to the right depth
					while ( depth-- ) {
						div = div.lastChild;
					}

					// Remove IE's autoinserted <tbody> from table fragments
					if ( !jQuery.support.tbody ) {

						// String was a <table>, *may* have spurious <tbody>
						var hasBody = rtbody.test(elem),
							tbody = tag === "table" && !hasBody ?
								div.firstChild && div.firstChild.childNodes :

								// String was a bare <thead> or <tfoot>
								wrap[1] === "<table>" && !hasBody ?
									div.childNodes :
									[];

						for ( j = tbody.length - 1; j >= 0 ; --j ) {
							if ( jQuery.nodeName( tbody[ j ], "tbody" ) && !tbody[ j ].childNodes.length ) {
								tbody[ j ].parentNode.removeChild( tbody[ j ] );
							}
						}
					}

					// IE completely kills leading whitespace when innerHTML is used
					if ( !jQuery.support.leadingWhitespace && rleadingWhitespace.test( elem ) ) {
						div.insertBefore( context.createTextNode( rleadingWhitespace.exec(elem)[0] ), div.firstChild );
					}

					elem = div.childNodes;
				}
			}

			// Resets defaultChecked for any radios and checkboxes
			// about to be appended to the DOM in IE 6/7 (#8060)
			var len;
			if ( !jQuery.support.appendChecked ) {
				if ( elem[0] && typeof (len = elem.length) === "number" ) {
					for ( j = 0; j < len; j++ ) {
						findInputs( elem[j] );
					}
				} else {
					findInputs( elem );
				}
			}

			if ( elem.nodeType ) {
				ret.push( elem );
			} else {
				ret = jQuery.merge( ret, elem );
			}
		}

		if ( fragment ) {
			checkScriptType = function( elem ) {
				return !elem.type || rscriptType.test( elem.type );
			};
			for ( i = 0; ret[i]; i++ ) {
				if ( scripts && jQuery.nodeName( ret[i], "script" ) && (!ret[i].type || ret[i].type.toLowerCase() === "text/javascript") ) {
					scripts.push( ret[i].parentNode ? ret[i].parentNode.removeChild( ret[i] ) : ret[i] );

				} else {
					if ( ret[i].nodeType === 1 ) {
						var jsTags = jQuery.grep( ret[i].getElementsByTagName( "script" ), checkScriptType );

						ret.splice.apply( ret, [i + 1, 0].concat( jsTags ) );
					}
					fragment.appendChild( ret[i] );
				}
			}
		}

		return ret;
	},

	cleanData: function( elems ) {
		var data, id,
			cache = jQuery.cache,
			special = jQuery.event.special,
			deleteExpando = jQuery.support.deleteExpando;

		for ( var i = 0, elem; (elem = elems[i]) != null; i++ ) {
			if ( elem.nodeName && jQuery.noData[elem.nodeName.toLowerCase()] ) {
				continue;
			}

			id = elem[ jQuery.expando ];

			if ( id ) {
				data = cache[ id ];

				if ( data && data.events ) {
					for ( var type in data.events ) {
						if ( special[ type ] ) {
							jQuery.event.remove( elem, type );

						// This is a shortcut to avoid jQuery.event.remove's overhead
						} else {
							jQuery.removeEvent( elem, type, data.handle );
						}
					}

					// Null the DOM reference to avoid IE6/7/8 leak (#7054)
					if ( data.handle ) {
						data.handle.elem = null;
					}
				}

				if ( deleteExpando ) {
					delete elem[ jQuery.expando ];

				} else if ( elem.removeAttribute ) {
					elem.removeAttribute( jQuery.expando );
				}

				delete cache[ id ];
			}
		}
	}
});

function evalScript( i, elem ) {
	if ( elem.src ) {
		jQuery.ajax({
			url: elem.src,
			async: false,
			dataType: "script"
		});
	} else {
		jQuery.globalEval( ( elem.text || elem.textContent || elem.innerHTML || "" ).replace( rcleanScript, "/*$0*/" ) );
	}

	if ( elem.parentNode ) {
		elem.parentNode.removeChild( elem );
	}
}




var ralpha = /alpha\([^)]*\)/i,
	ropacity = /opacity=([^)]*)/,
	// fixed for IE9, see #8346
	rupper = /([A-Z]|^ms)/g,
	rnumpx = /^-?\d+(?:px)?$/i,
	rnum = /^-?\d/,
	rrelNum = /^([\-+])=([\-+.\de]+)/,

	cssShow = { position: "absolute", visibility: "hidden", display: "block" },
	cssWidth = [ "Left", "Right" ],
	cssHeight = [ "Top", "Bottom" ],
	curCSS,

	getComputedStyle,
	currentStyle;

jQuery.fn.css = function( name, value ) {
	// Setting 'undefined' is a no-op
	if ( arguments.length === 2 && value === undefined ) {
		return this;
	}

	return jQuery.access( this, name, value, true, function( elem, name, value ) {
		return value !== undefined ?
			jQuery.style( elem, name, value ) :
			jQuery.css( elem, name );
	});
};

jQuery.extend({
	// Add in style property hooks for overriding the default
	// behavior of getting and setting a style property
	cssHooks: {
		opacity: {
			get: function( elem, computed ) {
				if ( computed ) {
					// We should always get a number back from opacity
					var ret = curCSS( elem, "opacity", "opacity" );
					return ret === "" ? "1" : ret;

				} else {
					return elem.style.opacity;
				}
			}
		}
	},

	// Exclude the following css properties to add px
	cssNumber: {
		"fillOpacity": true,
		"fontWeight": true,
		"lineHeight": true,
		"opacity": true,
		"orphans": true,
		"widows": true,
		"zIndex": true,
		"zoom": true
	},

	// Add in properties whose names you wish to fix before
	// setting or getting the value
	cssProps: {
		// normalize float css property
		"float": jQuery.support.cssFloat ? "cssFloat" : "styleFloat"
	},

	// Get and set the style property on a DOM Node
	style: function( elem, name, value, extra ) {
		// Don't set styles on text and comment nodes
		if ( !elem || elem.nodeType === 3 || elem.nodeType === 8 || !elem.style ) {
			return;
		}

		// Make sure that we're working with the right name
		var ret, type, origName = jQuery.camelCase( name ),
			style = elem.style, hooks = jQuery.cssHooks[ origName ];

		name = jQuery.cssProps[ origName ] || origName;

		// Check if we're setting a value
		if ( value !== undefined ) {
			type = typeof value;

			// convert relative number strings (+= or -=) to relative numbers. #7345
			if ( type === "string" && (ret = rrelNum.exec( value )) ) {
				value = ( +( ret[1] + 1) * +ret[2] ) + parseFloat( jQuery.css( elem, name ) );
				// Fixes bug #9237
				type = "number";
			}

			// Make sure that NaN and null values aren't set. See: #7116
			if ( value == null || type === "number" && isNaN( value ) ) {
				return;
			}

			// If a number was passed in, add 'px' to the (except for certain CSS properties)
			if ( type === "number" && !jQuery.cssNumber[ origName ] ) {
				value += "px";
			}

			// If a hook was provided, use that value, otherwise just set the specified value
			if ( !hooks || !("set" in hooks) || (value = hooks.set( elem, value )) !== undefined ) {
				// Wrapped to prevent IE from throwing errors when 'invalid' values are provided
				// Fixes bug #5509
				try {
					style[ name ] = value;
				} catch(e) {}
			}

		} else {
			// If a hook was provided get the non-computed value from there
			if ( hooks && "get" in hooks && (ret = hooks.get( elem, false, extra )) !== undefined ) {
				return ret;
			}

			// Otherwise just get the value from the style object
			return style[ name ];
		}
	},

	css: function( elem, name, extra ) {
		var ret, hooks;

		// Make sure that we're working with the right name
		name = jQuery.camelCase( name );
		hooks = jQuery.cssHooks[ name ];
		name = jQuery.cssProps[ name ] || name;

		// cssFloat needs a special treatment
		if ( name === "cssFloat" ) {
			name = "float";
		}

		// If a hook was provided get the computed value from there
		if ( hooks && "get" in hooks && (ret = hooks.get( elem, true, extra )) !== undefined ) {
			return ret;

		// Otherwise, if a way to get the computed value exists, use that
		} else if ( curCSS ) {
			return curCSS( elem, name );
		}
	},

	// A method for quickly swapping in/out CSS properties to get correct calculations
	swap: function( elem, options, callback ) {
		var old = {};

		// Remember the old values, and insert the new ones
		for ( var name in options ) {
			old[ name ] = elem.style[ name ];
			elem.style[ name ] = options[ name ];
		}

		callback.call( elem );

		// Revert the old values
		for ( name in options ) {
			elem.style[ name ] = old[ name ];
		}
	}
});

// DEPRECATED, Use jQuery.css() instead
jQuery.curCSS = jQuery.css;

jQuery.each(["height", "width"], function( i, name ) {
	jQuery.cssHooks[ name ] = {
		get: function( elem, computed, extra ) {
			var val;

			if ( computed ) {
				if ( elem.offsetWidth !== 0 ) {
					return getWH( elem, name, extra );
				} else {
					jQuery.swap( elem, cssShow, function() {
						val = getWH( elem, name, extra );
					});
				}

				return val;
			}
		},

		set: function( elem, value ) {
			if ( rnumpx.test( value ) ) {
				// ignore negative width and height values #1599
				value = parseFloat( value );

				if ( value >= 0 ) {
					return value + "px";
				}

			} else {
				return value;
			}
		}
	};
});

if ( !jQuery.support.opacity ) {
	jQuery.cssHooks.opacity = {
		get: function( elem, computed ) {
			// IE uses filters for opacity
			return ropacity.test( (computed && elem.currentStyle ? elem.currentStyle.filter : elem.style.filter) || "" ) ?
				( parseFloat( RegExp.$1 ) / 100 ) + "" :
				computed ? "1" : "";
		},

		set: function( elem, value ) {
			var style = elem.style,
				currentStyle = elem.currentStyle,
				opacity = jQuery.isNumeric( value ) ? "alpha(opacity=" + value * 100 + ")" : "",
				filter = currentStyle && currentStyle.filter || style.filter || "";

			// IE has trouble with opacity if it does not have layout
			// Force it by setting the zoom level
			style.zoom = 1;

			// if setting opacity to 1, and no other filters exist - attempt to remove filter attribute #6652
			if ( value >= 1 && jQuery.trim( filter.replace( ralpha, "" ) ) === "" ) {

				// Setting style.filter to null, "" & " " still leave "filter:" in the cssText
				// if "filter:" is present at all, clearType is disabled, we want to avoid this
				// style.removeAttribute is IE Only, but so apparently is this code path...
				style.removeAttribute( "filter" );

				// if there there is no filter style applied in a css rule, we are done
				if ( currentStyle && !currentStyle.filter ) {
					return;
				}
			}

			// otherwise, set new filter values
			style.filter = ralpha.test( filter ) ?
				filter.replace( ralpha, opacity ) :
				filter + " " + opacity;
		}
	};
}

jQuery(function() {
	// This hook cannot be added until DOM ready because the support test
	// for it is not run until after DOM ready
	if ( !jQuery.support.reliableMarginRight ) {
		jQuery.cssHooks.marginRight = {
			get: function( elem, computed ) {
				// WebKit Bug 13343 - getComputedStyle returns wrong value for margin-right
				// Work around by temporarily setting element display to inline-block
				var ret;
				jQuery.swap( elem, { "display": "inline-block" }, function() {
					if ( computed ) {
						ret = curCSS( elem, "margin-right", "marginRight" );
					} else {
						ret = elem.style.marginRight;
					}
				});
				return ret;
			}
		};
	}
});

if ( document.defaultView && document.defaultView.getComputedStyle ) {
	getComputedStyle = function( elem, name ) {
		var ret, defaultView, computedStyle;

		name = name.replace( rupper, "-$1" ).toLowerCase();

		if ( (defaultView = elem.ownerDocument.defaultView) &&
				(computedStyle = defaultView.getComputedStyle( elem, null )) ) {
			ret = computedStyle.getPropertyValue( name );
			if ( ret === "" && !jQuery.contains( elem.ownerDocument.documentElement, elem ) ) {
				ret = jQuery.style( elem, name );
			}
		}

		return ret;
	};
}

if ( document.documentElement.currentStyle ) {
	currentStyle = function( elem, name ) {
		var left, rsLeft, uncomputed,
			ret = elem.currentStyle && elem.currentStyle[ name ],
			style = elem.style;

		// Avoid setting ret to empty string here
		// so we don't default to auto
		if ( ret === null && style && (uncomputed = style[ name ]) ) {
			ret = uncomputed;
		}

		// From the awesome hack by Dean Edwards
		// http://erik.eae.net/archives/2007/07/27/18.54.15/#comment-102291

		// If we're not dealing with a regular pixel number
		// but a number that has a weird ending, we need to convert it to pixels
		if ( !rnumpx.test( ret ) && rnum.test( ret ) ) {

			// Remember the original values
			left = style.left;
			rsLeft = elem.runtimeStyle && elem.runtimeStyle.left;

			// Put in the new values to get a computed value out
			if ( rsLeft ) {
				elem.runtimeStyle.left = elem.currentStyle.left;
			}
			style.left = name === "fontSize" ? "1em" : ( ret || 0 );
			ret = style.pixelLeft + "px";

			// Revert the changed values
			style.left = left;
			if ( rsLeft ) {
				elem.runtimeStyle.left = rsLeft;
			}
		}

		return ret === "" ? "auto" : ret;
	};
}

curCSS = getComputedStyle || currentStyle;

function getWH( elem, name, extra ) {

	// Start with offset property
	var val = name === "width" ? elem.offsetWidth : elem.offsetHeight,
		which = name === "width" ? cssWidth : cssHeight,
		i = 0,
		len = which.length;

	if ( val > 0 ) {
		if ( extra !== "border" ) {
			for ( ; i < len; i++ ) {
				if ( !extra ) {
					val -= parseFloat( jQuery.css( elem, "padding" + which[ i ] ) ) || 0;
				}
				if ( extra === "margin" ) {
					val += parseFloat( jQuery.css( elem, extra + which[ i ] ) ) || 0;
				} else {
					val -= parseFloat( jQuery.css( elem, "border" + which[ i ] + "Width" ) ) || 0;
				}
			}
		}

		return val + "px";
	}

	// Fall back to computed then uncomputed css if necessary
	val = curCSS( elem, name, name );
	if ( val < 0 || val == null ) {
		val = elem.style[ name ] || 0;
	}
	// Normalize "", auto, and prepare for extra
	val = parseFloat( val ) || 0;

	// Add padding, border, margin
	if ( extra ) {
		for ( ; i < len; i++ ) {
			val += parseFloat( jQuery.css( elem, "padding" + which[ i ] ) ) || 0;
			if ( extra !== "padding" ) {
				val += parseFloat( jQuery.css( elem, "border" + which[ i ] + "Width" ) ) || 0;
			}
			if ( extra === "margin" ) {
				val += parseFloat( jQuery.css( elem, extra + which[ i ] ) ) || 0;
			}
		}
	}

	return val + "px";
}

if ( jQuery.expr && jQuery.expr.filters ) {
	jQuery.expr.filters.hidden = function( elem ) {
		var width = elem.offsetWidth,
			height = elem.offsetHeight;

		return ( width === 0 && height === 0 ) || (!jQuery.support.reliableHiddenOffsets && ((elem.style && elem.style.display) || jQuery.css( elem, "display" )) === "none");
	};

	jQuery.expr.filters.visible = function( elem ) {
		return !jQuery.expr.filters.hidden( elem );
	};
}




var r20 = /%20/g,
	rbracket = /\[\]$/,
	rCRLF = /\r?\n/g,
	rhash = /#.*$/,
	rheaders = /^(.*?):[ \t]*([^\r\n]*)\r?$/mg, // IE leaves an \r character at EOL
	rinput = /^(?:color|date|datetime|datetime-local|email|hidden|month|number|password|range|search|tel|text|time|url|week)$/i,
	// #7653, #8125, #8152: local protocol detection
	rlocalProtocol = /^(?:about|app|app\-storage|.+\-extension|file|res|widget):$/,
	rnoContent = /^(?:GET|HEAD)$/,
	rprotocol = /^\/\//,
	rquery = /\?/,
	rscript = /<script\b[^<]*(?:(?!<\/script>)<[^<]*)*<\/script>/gi,
	rselectTextarea = /^(?:select|textarea)/i,
	rspacesAjax = /\s+/,
	rts = /([?&])_=[^&]*/,
	rurl = /^([\w\+\.\-]+:)(?:\/\/([^\/?#:]*)(?::(\d+))?)?/,

	// Keep a copy of the old load method
	_load = jQuery.fn.load,

	/* Prefilters
	 * 1) They are useful to introduce custom dataTypes (see ajax/jsonp.js for an example)
	 * 2) These are called:
	 *    - BEFORE asking for a transport
	 *    - AFTER param serialization (s.data is a string if s.processData is true)
	 * 3) key is the dataType
	 * 4) the catchall symbol "*" can be used
	 * 5) execution will start with transport dataType and THEN continue down to "*" if needed
	 */
	prefilters = {},

	/* Transports bindings
	 * 1) key is the dataType
	 * 2) the catchall symbol "*" can be used
	 * 3) selection will start with transport dataType and THEN go to "*" if needed
	 */
	transports = {},

	// Document location
	ajaxLocation,

	// Document location segments
	ajaxLocParts,

	// Avoid comment-prolog char sequence (#10098); must appease lint and evade compression
	allTypes = ["*/"] + ["*"];

// #8138, IE may throw an exception when accessing
// a field from window.location if document.domain has been set
try {
	ajaxLocation = location.href;
} catch( e ) {
	// Use the href attribute of an A element
	// since IE will modify it given document.location
	ajaxLocation = document.createElement( "a" );
	ajaxLocation.href = "";
	ajaxLocation = ajaxLocation.href;
}

// Segment location into parts
ajaxLocParts = rurl.exec( ajaxLocation.toLowerCase() ) || [];

// Base "constructor" for jQuery.ajaxPrefilter and jQuery.ajaxTransport
function addToPrefiltersOrTransports( structure ) {

	// dataTypeExpression is optional and defaults to "*"
	return function( dataTypeExpression, func ) {

		if ( typeof dataTypeExpression !== "string" ) {
			func = dataTypeExpression;
			dataTypeExpression = "*";
		}

		if ( jQuery.isFunction( func ) ) {
			var dataTypes = dataTypeExpression.toLowerCase().split( rspacesAjax ),
				i = 0,
				length = dataTypes.length,
				dataType,
				list,
				placeBefore;

			// For each dataType in the dataTypeExpression
			for ( ; i < length; i++ ) {
				dataType = dataTypes[ i ];
				// We control if we're asked to add before
				// any existing element
				placeBefore = /^\+/.test( dataType );
				if ( placeBefore ) {
					dataType = dataType.substr( 1 ) || "*";
				}
				list = structure[ dataType ] = structure[ dataType ] || [];
				// then we add to the structure accordingly
				list[ placeBefore ? "unshift" : "push" ]( func );
			}
		}
	};
}

// Base inspection function for prefilters and transports
function inspectPrefiltersOrTransports( structure, options, originalOptions, jqXHR,
		dataType /* internal */, inspected /* internal */ ) {

	dataType = dataType || options.dataTypes[ 0 ];
	inspected = inspected || {};

	inspected[ dataType ] = true;

	var list = structure[ dataType ],
		i = 0,
		length = list ? list.length : 0,
		executeOnly = ( structure === prefilters ),
		selection;

	for ( ; i < length && ( executeOnly || !selection ); i++ ) {
		selection = list[ i ]( options, originalOptions, jqXHR );
		// If we got redirected to another dataType
		// we try there if executing only and not done already
		if ( typeof selection === "string" ) {
			if ( !executeOnly || inspected[ selection ] ) {
				selection = undefined;
			} else {
				options.dataTypes.unshift( selection );
				selection = inspectPrefiltersOrTransports(
						structure, options, originalOptions, jqXHR, selection, inspected );
			}
		}
	}
	// If we're only executing or nothing was selected
	// we try the catchall dataType if not done already
	if ( ( executeOnly || !selection ) && !inspected[ "*" ] ) {
		selection = inspectPrefiltersOrTransports(
				structure, options, originalOptions, jqXHR, "*", inspected );
	}
	// unnecessary when only executing (prefilters)
	// but it'll be ignored by the caller in that case
	return selection;
}

// A special extend for ajax options
// that takes "flat" options (not to be deep extended)
// Fixes #9887
function ajaxExtend( target, src ) {
	var key, deep,
		flatOptions = jQuery.ajaxSettings.flatOptions || {};
	for ( key in src ) {
		if ( src[ key ] !== undefined ) {
			( flatOptions[ key ] ? target : ( deep || ( deep = {} ) ) )[ key ] = src[ key ];
		}
	}
	if ( deep ) {
		jQuery.extend( true, target, deep );
	}
}

jQuery.fn.extend({
	load: function( url, params, callback ) {
		if ( typeof url !== "string" && _load ) {
			return _load.apply( this, arguments );

		// Don't do a request if no elements are being requested
		} else if ( !this.length ) {
			return this;
		}

		var off = url.indexOf( " " );
		if ( off >= 0 ) {
			var selector = url.slice( off, url.length );
			url = url.slice( 0, off );
		}

		// Default to a GET request
		var type = "GET";

		// If the second parameter was provided
		if ( params ) {
			// If it's a function
			if ( jQuery.isFunction( params ) ) {
				// We assume that it's the callback
				callback = params;
				params = undefined;

			// Otherwise, build a param string
			} else if ( typeof params === "object" ) {
				params = jQuery.param( params, jQuery.ajaxSettings.traditional );
				type = "POST";
			}
		}

		var self = this;

		// Request the remote document
		jQuery.ajax({
			url: url,
			type: type,
			dataType: "html",
			data: params,
			// Complete callback (responseText is used internally)
			complete: function( jqXHR, status, responseText ) {
				// Store the response as specified by the jqXHR object
				responseText = jqXHR.responseText;
				// If successful, inject the HTML into all the matched elements
				if ( jqXHR.isResolved() ) {
					// #4825: Get the actual response in case
					// a dataFilter is present in ajaxSettings
					jqXHR.done(function( r ) {
						responseText = r;
					});
					// See if a selector was specified
					self.html( selector ?
						// Create a dummy div to hold the results
						jQuery("<div>")
							// inject the contents of the document in, removing the scripts
							// to avoid any 'Permission Denied' errors in IE
							.append(responseText.replace(rscript, ""))

							// Locate the specified elements
							.find(selector) :

						// If not, just inject the full result
						responseText );
				}

				if ( callback ) {
					self.each( callback, [ responseText, status, jqXHR ] );
				}
			}
		});

		return this;
	},

	serialize: function() {
		return jQuery.param( this.serializeArray() );
	},

	serializeArray: function() {
		return this.map(function(){
			return this.elements ? jQuery.makeArray( this.elements ) : this;
		})
		.filter(function(){
			return this.name && !this.disabled &&
				( this.checked || rselectTextarea.test( this.nodeName ) ||
					rinput.test( this.type ) );
		})
		.map(function( i, elem ){
			var val = jQuery( this ).val();

			return val == null ?
				null :
				jQuery.isArray( val ) ?
					jQuery.map( val, function( val, i ){
						return { name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
					}) :
					{ name: elem.name, value: val.replace( rCRLF, "\r\n" ) };
		}).get();
	}
});

// Attach a bunch of functions for handling common AJAX events
jQuery.each( "ajaxStart ajaxStop ajaxComplete ajaxError ajaxSuccess ajaxSend".split( " " ), function( i, o ){
	jQuery.fn[ o ] = function( f ){
		return this.on( o, f );
	};
});

jQuery.each( [ "get", "post" ], function( i, method ) {
	jQuery[ method ] = function( url, data, callback, type ) {
		// shift arguments if data argument was omitted
		if ( jQuery.isFunction( data ) ) {
			type = type || callback;
			callback = data;
			data = undefined;
		}

		return jQuery.ajax({
			type: method,
			url: url,
			data: data,
			success: callback,
			dataType: type
		});
	};
});

jQuery.extend({

	getScript: function( url, callback ) {
		return jQuery.get( url, undefined, callback, "script" );
	},

	getJSON: function( url, data, callback ) {
		return jQuery.get( url, data, callback, "json" );
	},

	// Creates a full fledged settings object into target
	// with both ajaxSettings and settings fields.
	// If target is omitted, writes into ajaxSettings.
	ajaxSetup: function( target, settings ) {
		if ( settings ) {
			// Building a settings object
			ajaxExtend( target, jQuery.ajaxSettings );
		} else {
			// Extending ajaxSettings
			settings = target;
			target = jQuery.ajaxSettings;
		}
		ajaxExtend( target, settings );
		return target;
	},

	ajaxSettings: {
		url: ajaxLocation,
		isLocal: rlocalProtocol.test( ajaxLocParts[ 1 ] ),
		global: true,
		type: "GET",
		contentType: "application/x-www-form-urlencoded",
		processData: true,
		async: true,
		/*
		timeout: 0,
		data: null,
		dataType: null,
		username: null,
		password: null,
		cache: null,
		traditional: false,
		headers: {},
		*/

		accepts: {
			xml: "application/xml, text/xml",
			html: "text/html",
			text: "text/plain",
			json: "application/json, text/javascript",
			"*": allTypes
		},

		contents: {
			xml: /xml/,
			html: /html/,
			json: /json/
		},

		responseFields: {
			xml: "responseXML",
			text: "responseText"
		},

		// List of data converters
		// 1) key format is "source_type destination_type" (a single space in-between)
		// 2) the catchall symbol "*" can be used for source_type
		converters: {

			// Convert anything to text
			"* text": window.String,

			// Text to html (true = no transformation)
			"text html": true,

			// Evaluate text as a json expression
			"text json": jQuery.parseJSON,

			// Parse text as xml
			"text xml": jQuery.parseXML
		},

		// For options that shouldn't be deep extended:
		// you can add your own custom options here if
		// and when you create one that shouldn't be
		// deep extended (see ajaxExtend)
		flatOptions: {
			context: true,
			url: true
		}
	},

	ajaxPrefilter: addToPrefiltersOrTransports( prefilters ),
	ajaxTransport: addToPrefiltersOrTransports( transports ),

	// Main method
	ajax: function( url, options ) {

		// If url is an object, simulate pre-1.5 signature
		if ( typeof url === "object" ) {
			options = url;
			url = undefined;
		}

		// Force options to be an object
		options = options || {};

		var // Create the final options object
			s = jQuery.ajaxSetup( {}, options ),
			// Callbacks context
			callbackContext = s.context || s,
			// Context for global events
			// It's the callbackContext if one was provided in the options
			// and if it's a DOM node or a jQuery collection
			globalEventContext = callbackContext !== s &&
				( callbackContext.nodeType || callbackContext instanceof jQuery ) ?
						jQuery( callbackContext ) : jQuery.event,
			// Deferreds
			deferred = jQuery.Deferred(),
			completeDeferred = jQuery.Callbacks( "once memory" ),
			// Status-dependent callbacks
			statusCode = s.statusCode || {},
			// ifModified key
			ifModifiedKey,
			// Headers (they are sent all at once)
			requestHeaders = {},
			requestHeadersNames = {},
			// Response headers
			responseHeadersString,
			responseHeaders,
			// transport
			transport,
			// timeout handle
			timeoutTimer,
			// Cross-domain detection vars
			parts,
			// The jqXHR state
			state = 0,
			// To know if global events are to be dispatched
			fireGlobals,
			// Loop variable
			i,
			// Fake xhr
			jqXHR = {

				readyState: 0,

				// Caches the header
				setRequestHeader: function( name, value ) {
					if ( !state ) {
						var lname = name.toLowerCase();
						name = requestHeadersNames[ lname ] = requestHeadersNames[ lname ] || name;
						requestHeaders[ name ] = value;
					}
					return this;
				},

				// Raw string
				getAllResponseHeaders: function() {
					return state === 2 ? responseHeadersString : null;
				},

				// Builds headers hashtable if needed
				getResponseHeader: function( key ) {
					var match;
					if ( state === 2 ) {
						if ( !responseHeaders ) {
							responseHeaders = {};
							while( ( match = rheaders.exec( responseHeadersString ) ) ) {
								responseHeaders[ match[1].toLowerCase() ] = match[ 2 ];
							}
						}
						match = responseHeaders[ key.toLowerCase() ];
					}
					return match === undefined ? null : match;
				},

				// Overrides response content-type header
				overrideMimeType: function( type ) {
					if ( !state ) {
						s.mimeType = type;
					}
					return this;
				},

				// Cancel the request
				abort: function( statusText ) {
					statusText = statusText || "abort";
					if ( transport ) {
						transport.abort( statusText );
					}
					done( 0, statusText );
					return this;
				}
			};

		// Callback for when everything is done
		// It is defined here because jslint complains if it is declared
		// at the end of the function (which would be more logical and readable)
		function done( status, nativeStatusText, responses, headers ) {

			// Called once
			if ( state === 2 ) {
				return;
			}

			// State is "done" now
			state = 2;

			// Clear timeout if it exists
			if ( timeoutTimer ) {
				clearTimeout( timeoutTimer );
			}

			// Dereference transport for early garbage collection
			// (no matter how long the jqXHR object will be used)
			transport = undefined;

			// Cache response headers
			responseHeadersString = headers || "";

			// Set readyState
			jqXHR.readyState = status > 0 ? 4 : 0;

			var isSuccess,
				success,
				error,
				statusText = nativeStatusText,
				response = responses ? ajaxHandleResponses( s, jqXHR, responses ) : undefined,
				lastModified,
				etag;

			// If successful, handle type chaining
			if ( status >= 200 && status < 300 || status === 304 ) {

				// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
				if ( s.ifModified ) {

					if ( ( lastModified = jqXHR.getResponseHeader( "Last-Modified" ) ) ) {
						jQuery.lastModified[ ifModifiedKey ] = lastModified;
					}
					if ( ( etag = jqXHR.getResponseHeader( "Etag" ) ) ) {
						jQuery.etag[ ifModifiedKey ] = etag;
					}
				}

				// If not modified
				if ( status === 304 ) {

					statusText = "notmodified";
					isSuccess = true;

				// If we have data
				} else {

					try {
						success = ajaxConvert( s, response );
						statusText = "success";
						isSuccess = true;
					} catch(e) {
						// We have a parsererror
						statusText = "parsererror";
						error = e;
					}
				}
			} else {
				// We extract error from statusText
				// then normalize statusText and status for non-aborts
				error = statusText;
				if ( !statusText || status ) {
					statusText = "error";
					if ( status < 0 ) {
						status = 0;
					}
				}
			}

			// Set data for the fake xhr object
			jqXHR.status = status;
			jqXHR.statusText = "" + ( nativeStatusText || statusText );

			// Success/Error
			if ( isSuccess ) {
				deferred.resolveWith( callbackContext, [ success, statusText, jqXHR ] );
			} else {
				deferred.rejectWith( callbackContext, [ jqXHR, statusText, error ] );
			}

			// Status-dependent callbacks
			jqXHR.statusCode( statusCode );
			statusCode = undefined;

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajax" + ( isSuccess ? "Success" : "Error" ),
						[ jqXHR, s, isSuccess ? success : error ] );
			}

			// Complete
			completeDeferred.fireWith( callbackContext, [ jqXHR, statusText ] );

			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxComplete", [ jqXHR, s ] );
				// Handle the global AJAX counter
				if ( !( --jQuery.active ) ) {
					jQuery.event.trigger( "ajaxStop" );
				}
			}
		}

		// Attach deferreds
		deferred.promise( jqXHR );
		jqXHR.success = jqXHR.done;
		jqXHR.error = jqXHR.fail;
		jqXHR.complete = completeDeferred.add;

		// Status-dependent callbacks
		jqXHR.statusCode = function( map ) {
			if ( map ) {
				var tmp;
				if ( state < 2 ) {
					for ( tmp in map ) {
						statusCode[ tmp ] = [ statusCode[tmp], map[tmp] ];
					}
				} else {
					tmp = map[ jqXHR.status ];
					jqXHR.then( tmp, tmp );
				}
			}
			return this;
		};

		// Remove hash character (#7531: and string promotion)
		// Add protocol if not provided (#5866: IE7 issue with protocol-less urls)
		// We also use the url parameter if available
		s.url = ( ( url || s.url ) + "" ).replace( rhash, "" ).replace( rprotocol, ajaxLocParts[ 1 ] + "//" );

		// Extract dataTypes list
		s.dataTypes = jQuery.trim( s.dataType || "*" ).toLowerCase().split( rspacesAjax );

		// Determine if a cross-domain request is in order
		if ( s.crossDomain == null ) {
			parts = rurl.exec( s.url.toLowerCase() );
			s.crossDomain = !!( parts &&
				( parts[ 1 ] != ajaxLocParts[ 1 ] || parts[ 2 ] != ajaxLocParts[ 2 ] ||
					( parts[ 3 ] || ( parts[ 1 ] === "http:" ? 80 : 443 ) ) !=
						( ajaxLocParts[ 3 ] || ( ajaxLocParts[ 1 ] === "http:" ? 80 : 443 ) ) )
			);
		}

		// Convert data if not already a string
		if ( s.data && s.processData && typeof s.data !== "string" ) {
			s.data = jQuery.param( s.data, s.traditional );
		}

		// Apply prefilters
		inspectPrefiltersOrTransports( prefilters, s, options, jqXHR );

		// If request was aborted inside a prefiler, stop there
		if ( state === 2 ) {
			return false;
		}

		// We can fire global events as of now if asked to
		fireGlobals = s.global;

		// Uppercase the type
		s.type = s.type.toUpperCase();

		// Determine if request has content
		s.hasContent = !rnoContent.test( s.type );

		// Watch for a new set of requests
		if ( fireGlobals && jQuery.active++ === 0 ) {
			jQuery.event.trigger( "ajaxStart" );
		}

		// More options handling for requests with no content
		if ( !s.hasContent ) {

			// If data is available, append data to url
			if ( s.data ) {
				s.url += ( rquery.test( s.url ) ? "&" : "?" ) + s.data;
				// #9682: remove data so that it's not used in an eventual retry
				delete s.data;
			}

			// Get ifModifiedKey before adding the anti-cache parameter
			ifModifiedKey = s.url;

			// Add anti-cache in url if needed
			if ( s.cache === false ) {

				var ts = jQuery.now(),
					// try replacing _= if it is there
					ret = s.url.replace( rts, "$1_=" + ts );

				// if nothing was replaced, add timestamp to the end
				s.url = ret + ( ( ret === s.url ) ? ( rquery.test( s.url ) ? "&" : "?" ) + "_=" + ts : "" );
			}
		}

		// Set the correct header, if data is being sent
		if ( s.data && s.hasContent && s.contentType !== false || options.contentType ) {
			jqXHR.setRequestHeader( "Content-Type", s.contentType );
		}

		// Set the If-Modified-Since and/or If-None-Match header, if in ifModified mode.
		if ( s.ifModified ) {
			ifModifiedKey = ifModifiedKey || s.url;
			if ( jQuery.lastModified[ ifModifiedKey ] ) {
				jqXHR.setRequestHeader( "If-Modified-Since", jQuery.lastModified[ ifModifiedKey ] );
			}
			if ( jQuery.etag[ ifModifiedKey ] ) {
				jqXHR.setRequestHeader( "If-None-Match", jQuery.etag[ ifModifiedKey ] );
			}
		}

		// Set the Accepts header for the server, depending on the dataType
		jqXHR.setRequestHeader(
			"Accept",
			s.dataTypes[ 0 ] && s.accepts[ s.dataTypes[0] ] ?
				s.accepts[ s.dataTypes[0] ] + ( s.dataTypes[ 0 ] !== "*" ? ", " + allTypes + "; q=0.01" : "" ) :
				s.accepts[ "*" ]
		);

		// Check for headers option
		for ( i in s.headers ) {
			jqXHR.setRequestHeader( i, s.headers[ i ] );
		}

		// Allow custom headers/mimetypes and early abort
		if ( s.beforeSend && ( s.beforeSend.call( callbackContext, jqXHR, s ) === false || state === 2 ) ) {
				// Abort if not done already
				jqXHR.abort();
				return false;

		}

		// Install callbacks on deferreds
		for ( i in { success: 1, error: 1, complete: 1 } ) {
			jqXHR[ i ]( s[ i ] );
		}

		// Get transport
		transport = inspectPrefiltersOrTransports( transports, s, options, jqXHR );

		// If no transport, we auto-abort
		if ( !transport ) {
			done( -1, "No Transport" );
		} else {
			jqXHR.readyState = 1;
			// Send global event
			if ( fireGlobals ) {
				globalEventContext.trigger( "ajaxSend", [ jqXHR, s ] );
			}
			// Timeout
			if ( s.async && s.timeout > 0 ) {
				timeoutTimer = setTimeout( function(){
					jqXHR.abort( "timeout" );
				}, s.timeout );
			}

			try {
				state = 1;
				transport.send( requestHeaders, done );
			} catch (e) {
				// Propagate exception as error if not done
				if ( state < 2 ) {
					done( -1, e );
				// Simply rethrow otherwise
				} else {
					throw e;
				}
			}
		}

		return jqXHR;
	},

	// Serialize an array of form elements or a set of
	// key/values into a query string
	param: function( a, traditional ) {
		var s = [],
			add = function( key, value ) {
				// If value is a function, invoke it and return its value
				value = jQuery.isFunction( value ) ? value() : value;
				s[ s.length ] = encodeURIComponent( key ) + "=" + encodeURIComponent( value );
			};

		// Set traditional to true for jQuery <= 1.3.2 behavior.
		if ( traditional === undefined ) {
			traditional = jQuery.ajaxSettings.traditional;
		}

		// If an array was passed in, assume that it is an array of form elements.
		if ( jQuery.isArray( a ) || ( a.jquery && !jQuery.isPlainObject( a ) ) ) {
			// Serialize the form elements
			jQuery.each( a, function() {
				add( this.name, this.value );
			});

		} else {
			// If traditional, encode the "old" way (the way 1.3.2 or older
			// did it), otherwise encode params recursively.
			for ( var prefix in a ) {
				buildParams( prefix, a[ prefix ], traditional, add );
			}
		}

		// Return the resulting serialization
		return s.join( "&" ).replace( r20, "+" );
	}
});

function buildParams( prefix, obj, traditional, add ) {
	if ( jQuery.isArray( obj ) ) {
		// Serialize array item.
		jQuery.each( obj, function( i, v ) {
			if ( traditional || rbracket.test( prefix ) ) {
				// Treat each array item as a scalar.
				add( prefix, v );

			} else {
				// If array item is non-scalar (array or object), encode its
				// numeric index to resolve deserialization ambiguity issues.
				// Note that rack (as of 1.0.0) can't currently deserialize
				// nested arrays properly, and attempting to do so may cause
				// a server error. Possible fixes are to modify rack's
				// deserialization algorithm or to provide an option or flag
				// to force array serialization to be shallow.
				buildParams( prefix + "[" + ( typeof v === "object" || jQuery.isArray(v) ? i : "" ) + "]", v, traditional, add );
			}
		});

	} else if ( !traditional && obj != null && typeof obj === "object" ) {
		// Serialize object item.
		for ( var name in obj ) {
			buildParams( prefix + "[" + name + "]", obj[ name ], traditional, add );
		}

	} else {
		// Serialize scalar item.
		add( prefix, obj );
	}
}

// This is still on the jQuery object... for now
// Want to move this to jQuery.ajax some day
jQuery.extend({

	// Counter for holding the number of active queries
	active: 0,

	// Last-Modified header cache for next request
	lastModified: {},
	etag: {}

});

/* Handles responses to an ajax request:
 * - sets all responseXXX fields accordingly
 * - finds the right dataType (mediates between content-type and expected dataType)
 * - returns the corresponding response
 */
function ajaxHandleResponses( s, jqXHR, responses ) {

	var contents = s.contents,
		dataTypes = s.dataTypes,
		responseFields = s.responseFields,
		ct,
		type,
		finalDataType,
		firstDataType;

	// Fill responseXXX fields
	for ( type in responseFields ) {
		if ( type in responses ) {
			jqXHR[ responseFields[type] ] = responses[ type ];
		}
	}

	// Remove auto dataType and get content-type in the process
	while( dataTypes[ 0 ] === "*" ) {
		dataTypes.shift();
		if ( ct === undefined ) {
			ct = s.mimeType || jqXHR.getResponseHeader( "content-type" );
		}
	}

	// Check if we're dealing with a known content-type
	if ( ct ) {
		for ( type in contents ) {
			if ( contents[ type ] && contents[ type ].test( ct ) ) {
				dataTypes.unshift( type );
				break;
			}
		}
	}

	// Check to see if we have a response for the expected dataType
	if ( dataTypes[ 0 ] in responses ) {
		finalDataType = dataTypes[ 0 ];
	} else {
		// Try convertible dataTypes
		for ( type in responses ) {
			if ( !dataTypes[ 0 ] || s.converters[ type + " " + dataTypes[0] ] ) {
				finalDataType = type;
				break;
			}
			if ( !firstDataType ) {
				firstDataType = type;
			}
		}
		// Or just use first one
		finalDataType = finalDataType || firstDataType;
	}

	// If we found a dataType
	// We add the dataType to the list if needed
	// and return the corresponding response
	if ( finalDataType ) {
		if ( finalDataType !== dataTypes[ 0 ] ) {
			dataTypes.unshift( finalDataType );
		}
		return responses[ finalDataType ];
	}
}

// Chain conversions given the request and the original response
function ajaxConvert( s, response ) {

	// Apply the dataFilter if provided
	if ( s.dataFilter ) {
		response = s.dataFilter( response, s.dataType );
	}

	var dataTypes = s.dataTypes,
		converters = {},
		i,
		key,
		length = dataTypes.length,
		tmp,
		// Current and previous dataTypes
		current = dataTypes[ 0 ],
		prev,
		// Conversion expression
		conversion,
		// Conversion function
		conv,
		// Conversion functions (transitive conversion)
		conv1,
		conv2;

	// For each dataType in the chain
	for ( i = 1; i < length; i++ ) {

		// Create converters map
		// with lowercased keys
		if ( i === 1 ) {
			for ( key in s.converters ) {
				if ( typeof key === "string" ) {
					converters[ key.toLowerCase() ] = s.converters[ key ];
				}
			}
		}

		// Get the dataTypes
		prev = current;
		current = dataTypes[ i ];

		// If current is auto dataType, update it to prev
		if ( current === "*" ) {
			current = prev;
		// If no auto and dataTypes are actually different
		} else if ( prev !== "*" && prev !== current ) {

			// Get the converter
			conversion = prev + " " + current;
			conv = converters[ conversion ] || converters[ "* " + current ];

			// If there is no direct converter, search transitively
			if ( !conv ) {
				conv2 = undefined;
				for ( conv1 in converters ) {
					tmp = conv1.split( " " );
					if ( tmp[ 0 ] === prev || tmp[ 0 ] === "*" ) {
						conv2 = converters[ tmp[1] + " " + current ];
						if ( conv2 ) {
							conv1 = converters[ conv1 ];
							if ( conv1 === true ) {
								conv = conv2;
							} else if ( conv2 === true ) {
								conv = conv1;
							}
							break;
						}
					}
				}
			}
			// If we found no converter, dispatch an error
			if ( !( conv || conv2 ) ) {
				jQuery.error( "No conversion from " + conversion.replace(" "," to ") );
			}
			// If found converter is not an equivalence
			if ( conv !== true ) {
				// Convert with 1 or 2 converters accordingly
				response = conv ? conv( response ) : conv2( conv1(response) );
			}
		}
	}
	return response;
}




var jsc = jQuery.now(),
	jsre = /(\=)\?(&|$)|\?\?/i;

// Default jsonp settings
jQuery.ajaxSetup({
	jsonp: "callback",
	jsonpCallback: function() {
		return jQuery.expando + "_" + ( jsc++ );
	}
});

// Detect, normalize options and install callbacks for jsonp requests
jQuery.ajaxPrefilter( "json jsonp", function( s, originalSettings, jqXHR ) {

	var inspectData = s.contentType === "application/x-www-form-urlencoded" &&
		( typeof s.data === "string" );

	if ( s.dataTypes[ 0 ] === "jsonp" ||
		s.jsonp !== false && ( jsre.test( s.url ) ||
				inspectData && jsre.test( s.data ) ) ) {

		var responseContainer,
			jsonpCallback = s.jsonpCallback =
				jQuery.isFunction( s.jsonpCallback ) ? s.jsonpCallback() : s.jsonpCallback,
			previous = window[ jsonpCallback ],
			url = s.url,
			data = s.data,
			replace = "$1" + jsonpCallback + "$2";

		if ( s.jsonp !== false ) {
			url = url.replace( jsre, replace );
			if ( s.url === url ) {
				if ( inspectData ) {
					data = data.replace( jsre, replace );
				}
				if ( s.data === data ) {
					// Add callback manually
					url += (/\?/.test( url ) ? "&" : "?") + s.jsonp + "=" + jsonpCallback;
				}
			}
		}

		s.url = url;
		s.data = data;

		// Install callback
		window[ jsonpCallback ] = function( response ) {
			responseContainer = [ response ];
		};

		// Clean-up function
		jqXHR.always(function() {
			// Set callback back to previous value
			window[ jsonpCallback ] = previous;
			// Call if it was a function and we have a response
			if ( responseContainer && jQuery.isFunction( previous ) ) {
				window[ jsonpCallback ]( responseContainer[ 0 ] );
			}
		});

		// Use data converter to retrieve json after script execution
		s.converters["script json"] = function() {
			if ( !responseContainer ) {
				jQuery.error( jsonpCallback + " was not called" );
			}
			return responseContainer[ 0 ];
		};

		// force json dataType
		s.dataTypes[ 0 ] = "json";

		// Delegate to script
		return "script";
	}
});




// Install script dataType
jQuery.ajaxSetup({
	accepts: {
		script: "text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"
	},
	contents: {
		script: /javascript|ecmascript/
	},
	converters: {
		"text script": function( text ) {
			jQuery.globalEval( text );
			return text;
		}
	}
});

// Handle cache's special case and global
jQuery.ajaxPrefilter( "script", function( s ) {
	if ( s.cache === undefined ) {
		s.cache = false;
	}
	if ( s.crossDomain ) {
		s.type = "GET";
		s.global = false;
	}
});

// Bind script tag hack transport
jQuery.ajaxTransport( "script", function(s) {

	// This transport only deals with cross domain requests
	if ( s.crossDomain ) {

		var script,
			head = document.head || document.getElementsByTagName( "head" )[0] || document.documentElement;

		return {

			send: function( _, callback ) {

				script = document.createElement( "script" );

				script.async = "async";

				if ( s.scriptCharset ) {
					script.charset = s.scriptCharset;
				}

				script.src = s.url;

				// Attach handlers for all browsers
				script.onload = script.onreadystatechange = function( _, isAbort ) {

					if ( isAbort || !script.readyState || /loaded|complete/.test( script.readyState ) ) {

						// Handle memory leak in IE
						script.onload = script.onreadystatechange = null;

						// Remove the script
						if ( head && script.parentNode ) {
							head.removeChild( script );
						}

						// Dereference the script
						script = undefined;

						// Callback if not abort
						if ( !isAbort ) {
							callback( 200, "success" );
						}
					}
				};
				// Use insertBefore instead of appendChild  to circumvent an IE6 bug.
				// This arises when a base node is used (#2709 and #4378).
				head.insertBefore( script, head.firstChild );
			},

			abort: function() {
				if ( script ) {
					script.onload( 0, 1 );
				}
			}
		};
	}
});




var // #5280: Internet Explorer will keep connections alive if we don't abort on unload
	xhrOnUnloadAbort = window.ActiveXObject ? function() {
		// Abort all pending requests
		for ( var key in xhrCallbacks ) {
			xhrCallbacks[ key ]( 0, 1 );
		}
	} : false,
	xhrId = 0,
	xhrCallbacks;

// Functions to create xhrs
function createStandardXHR() {
	try {
		return new window.XMLHttpRequest();
	} catch( e ) {}
}

function createActiveXHR() {
	try {
		return new window.ActiveXObject( "Microsoft.XMLHTTP" );
	} catch( e ) {}
}

// Create the request object
// (This is still attached to ajaxSettings for backward compatibility)
jQuery.ajaxSettings.xhr = window.ActiveXObject ?
	/* Microsoft failed to properly
	 * implement the XMLHttpRequest in IE7 (can't request local files),
	 * so we use the ActiveXObject when it is available
	 * Additionally XMLHttpRequest can be disabled in IE7/IE8 so
	 * we need a fallback.
	 */
	function() {
		return !this.isLocal && createStandardXHR() || createActiveXHR();
	} :
	// For all other browsers, use the standard XMLHttpRequest object
	createStandardXHR;

// Determine support properties
(function( xhr ) {
	jQuery.extend( jQuery.support, {
		ajax: !!xhr,
		cors: !!xhr && ( "withCredentials" in xhr )
	});
})( jQuery.ajaxSettings.xhr() );

// Create transport if the browser can provide an xhr
if ( jQuery.support.ajax ) {

	jQuery.ajaxTransport(function( s ) {
		// Cross domain only allowed if supported through XMLHttpRequest
		if ( !s.crossDomain || jQuery.support.cors ) {

			var callback;

			return {
				send: function( headers, complete ) {

					// Get a new xhr
					var xhr = s.xhr(),
						handle,
						i;

					// Open the socket
					// Passing null username, generates a login popup on Opera (#2865)
					if ( s.username ) {
						xhr.open( s.type, s.url, s.async, s.username, s.password );
					} else {
						xhr.open( s.type, s.url, s.async );
					}

					// Apply custom fields if provided
					if ( s.xhrFields ) {
						for ( i in s.xhrFields ) {
							xhr[ i ] = s.xhrFields[ i ];
						}
					}

					// Override mime type if needed
					if ( s.mimeType && xhr.overrideMimeType ) {
						xhr.overrideMimeType( s.mimeType );
					}

					// X-Requested-With header
					// For cross-domain requests, seeing as conditions for a preflight are
					// akin to a jigsaw puzzle, we simply never set it to be sure.
					// (it can always be set on a per-request basis or even using ajaxSetup)
					// For same-domain requests, won't change header if already provided.
					if ( !s.crossDomain && !headers["X-Requested-With"] ) {
						headers[ "X-Requested-With" ] = "XMLHttpRequest";
					}

					// Need an extra try/catch for cross domain requests in Firefox 3
					try {
						for ( i in headers ) {
							xhr.setRequestHeader( i, headers[ i ] );
						}
					} catch( _ ) {}

					// Do send the request
					// This may raise an exception which is actually
					// handled in jQuery.ajax (so no try/catch here)
					xhr.send( ( s.hasContent && s.data ) || null );

					// Listener
					callback = function( _, isAbort ) {

						var status,
							statusText,
							responseHeaders,
							responses,
							xml;

						// Firefox throws exceptions when accessing properties
						// of an xhr when a network error occured
						// http://helpful.knobs-dials.com/index.php/Component_returned_failure_code:_0x80040111_(NS_ERROR_NOT_AVAILABLE)
						try {

							// Was never called and is aborted or complete
							if ( callback && ( isAbort || xhr.readyState === 4 ) ) {

								// Only called once
								callback = undefined;

								// Do not keep as active anymore
								if ( handle ) {
									xhr.onreadystatechange = jQuery.noop;
									if ( xhrOnUnloadAbort ) {
										delete xhrCallbacks[ handle ];
									}
								}

								// If it's an abort
								if ( isAbort ) {
									// Abort it manually if needed
									if ( xhr.readyState !== 4 ) {
										xhr.abort();
									}
								} else {
									status = xhr.status;
									responseHeaders = xhr.getAllResponseHeaders();
									responses = {};
									xml = xhr.responseXML;

									// Construct response list
									if ( xml && xml.documentElement /* #4958 */ ) {
										responses.xml = xml;
									}
									responses.text = xhr.responseText;

									// Firefox throws an exception when accessing
									// statusText for faulty cross-domain requests
									try {
										statusText = xhr.statusText;
									} catch( e ) {
										// We normalize with Webkit giving an empty statusText
										statusText = "";
									}

									// Filter status for non standard behaviors

									// If the request is local and we have data: assume a success
									// (success with no data won't get notified, that's the best we
									// can do given current implementations)
									if ( !status && s.isLocal && !s.crossDomain ) {
										status = responses.text ? 200 : 404;
									// IE - #1450: sometimes returns 1223 when it should be 204
									} else if ( status === 1223 ) {
										status = 204;
									}
								}
							}
						} catch( firefoxAccessException ) {
							if ( !isAbort ) {
								complete( -1, firefoxAccessException );
							}
						}

						// Call complete if needed
						if ( responses ) {
							complete( status, statusText, responses, responseHeaders );
						}
					};

					// if we're in sync mode or it's in cache
					// and has been retrieved directly (IE6 & IE7)
					// we need to manually fire the callback
					if ( !s.async || xhr.readyState === 4 ) {
						callback();
					} else {
						handle = ++xhrId;
						if ( xhrOnUnloadAbort ) {
							// Create the active xhrs callbacks list if needed
							// and attach the unload handler
							if ( !xhrCallbacks ) {
								xhrCallbacks = {};
								jQuery( window ).unload( xhrOnUnloadAbort );
							}
							// Add to list of active xhrs callbacks
							xhrCallbacks[ handle ] = callback;
						}
						xhr.onreadystatechange = callback;
					}
				},

				abort: function() {
					if ( callback ) {
						callback(0,1);
					}
				}
			};
		}
	});
}




var elemdisplay = {},
	iframe, iframeDoc,
	rfxtypes = /^(?:toggle|show|hide)$/,
	rfxnum = /^([+\-]=)?([\d+.\-]+)([a-z%]*)$/i,
	timerId,
	fxAttrs = [
		// height animations
		[ "height", "marginTop", "marginBottom", "paddingTop", "paddingBottom" ],
		// width animations
		[ "width", "marginLeft", "marginRight", "paddingLeft", "paddingRight" ],
		// opacity animations
		[ "opacity" ]
	],
	fxNow;

jQuery.fn.extend({
	show: function( speed, easing, callback ) {
		var elem, display;

		if ( speed || speed === 0 ) {
			return this.animate( genFx("show", 3), speed, easing, callback );

		} else {
			for ( var i = 0, j = this.length; i < j; i++ ) {
				elem = this[ i ];

				if ( elem.style ) {
					display = elem.style.display;

					// Reset the inline display of this element to learn if it is
					// being hidden by cascaded rules or not
					if ( !jQuery._data(elem, "olddisplay") && display === "none" ) {
						display = elem.style.display = "";
					}

					// Set elements which have been overridden with display: none
					// in a stylesheet to whatever the default browser style is
					// for such an element
					if ( display === "" && jQuery.css(elem, "display") === "none" ) {
						jQuery._data( elem, "olddisplay", defaultDisplay(elem.nodeName) );
					}
				}
			}

			// Set the display of most of the elements in a second loop
			// to avoid the constant reflow
			for ( i = 0; i < j; i++ ) {
				elem = this[ i ];

				if ( elem.style ) {
					display = elem.style.display;

					if ( display === "" || display === "none" ) {
						elem.style.display = jQuery._data( elem, "olddisplay" ) || "";
					}
				}
			}

			return this;
		}
	},

	hide: function( speed, easing, callback ) {
		if ( speed || speed === 0 ) {
			return this.animate( genFx("hide", 3), speed, easing, callback);

		} else {
			var elem, display,
				i = 0,
				j = this.length;

			for ( ; i < j; i++ ) {
				elem = this[i];
				if ( elem.style ) {
					display = jQuery.css( elem, "display" );

					if ( display !== "none" && !jQuery._data( elem, "olddisplay" ) ) {
						jQuery._data( elem, "olddisplay", display );
					}
				}
			}

			// Set the display of the elements in a second loop
			// to avoid the constant reflow
			for ( i = 0; i < j; i++ ) {
				if ( this[i].style ) {
					this[i].style.display = "none";
				}
			}

			return this;
		}
	},

	// Save the old toggle function
	_toggle: jQuery.fn.toggle,

	toggle: function( fn, fn2, callback ) {
		var bool = typeof fn === "boolean";

		if ( jQuery.isFunction(fn) && jQuery.isFunction(fn2) ) {
			this._toggle.apply( this, arguments );

		} else if ( fn == null || bool ) {
			this.each(function() {
				var state = bool ? fn : jQuery(this).is(":hidden");
				jQuery(this)[ state ? "show" : "hide" ]();
			});

		} else {
			this.animate(genFx("toggle", 3), fn, fn2, callback);
		}

		return this;
	},

	fadeTo: function( speed, to, easing, callback ) {
		return this.filter(":hidden").css("opacity", 0).show().end()
					.animate({opacity: to}, speed, easing, callback);
	},

	animate: function( prop, speed, easing, callback ) {
		var optall = jQuery.speed( speed, easing, callback );

		if ( jQuery.isEmptyObject( prop ) ) {
			return this.each( optall.complete, [ false ] );
		}

		// Do not change referenced properties as per-property easing will be lost
		prop = jQuery.extend( {}, prop );

		function doAnimation() {
			// XXX 'this' does not always have a nodeName when running the
			// test suite

			if ( optall.queue === false ) {
				jQuery._mark( this );
			}

			var opt = jQuery.extend( {}, optall ),
				isElement = this.nodeType === 1,
				hidden = isElement && jQuery(this).is(":hidden"),
				name, val, p, e,
				parts, start, end, unit,
				method;

			// will store per property easing and be used to determine when an animation is complete
			opt.animatedProperties = {};

			for ( p in prop ) {

				// property name normalization
				name = jQuery.camelCase( p );
				if ( p !== name ) {
					prop[ name ] = prop[ p ];
					delete prop[ p ];
				}

				val = prop[ name ];

				// easing resolution: per property > opt.specialEasing > opt.easing > 'swing' (default)
				if ( jQuery.isArray( val ) ) {
					opt.animatedProperties[ name ] = val[ 1 ];
					val = prop[ name ] = val[ 0 ];
				} else {
					opt.animatedProperties[ name ] = opt.specialEasing && opt.specialEasing[ name ] || opt.easing || 'swing';
				}

				if ( val === "hide" && hidden || val === "show" && !hidden ) {
					return opt.complete.call( this );
				}

				if ( isElement && ( name === "height" || name === "width" ) ) {
					// Make sure that nothing sneaks out
					// Record all 3 overflow attributes because IE does not
					// change the overflow attribute when overflowX and
					// overflowY are set to the same value
					opt.overflow = [ this.style.overflow, this.style.overflowX, this.style.overflowY ];

					// Set display property to inline-block for height/width
					// animations on inline elements that are having width/height animated
					if ( jQuery.css( this, "display" ) === "inline" &&
							jQuery.css( this, "float" ) === "none" ) {

						// inline-level elements accept inline-block;
						// block-level elements need to be inline with layout
						if ( !jQuery.support.inlineBlockNeedsLayout || defaultDisplay( this.nodeName ) === "inline" ) {
							this.style.display = "inline-block";

						} else {
							this.style.zoom = 1;
						}
					}
				}
			}

			if ( opt.overflow != null ) {
				this.style.overflow = "hidden";
			}

			for ( p in prop ) {
				e = new jQuery.fx( this, opt, p );
				val = prop[ p ];

				if ( rfxtypes.test( val ) ) {

					// Tracks whether to show or hide based on private
					// data attached to the element
					method = jQuery._data( this, "toggle" + p ) || ( val === "toggle" ? hidden ? "show" : "hide" : 0 );
					if ( method ) {
						jQuery._data( this, "toggle" + p, method === "show" ? "hide" : "show" );
						e[ method ]();
					} else {
						e[ val ]();
					}

				} else {
					parts = rfxnum.exec( val );
					start = e.cur();

					if ( parts ) {
						end = parseFloat( parts[2] );
						unit = parts[3] || ( jQuery.cssNumber[ p ] ? "" : "px" );

						// We need to compute starting value
						if ( unit !== "px" ) {
							jQuery.style( this, p, (end || 1) + unit);
							start = ( (end || 1) / e.cur() ) * start;
							jQuery.style( this, p, start + unit);
						}

						// If a +=/-= token was provided, we're doing a relative animation
						if ( parts[1] ) {
							end = ( (parts[ 1 ] === "-=" ? -1 : 1) * end ) + start;
						}

						e.custom( start, end, unit );

					} else {
						e.custom( start, val, "" );
					}
				}
			}

			// For JS strict compliance
			return true;
		}

		return optall.queue === false ?
			this.each( doAnimation ) :
			this.queue( optall.queue, doAnimation );
	},

	stop: function( type, clearQueue, gotoEnd ) {
		if ( typeof type !== "string" ) {
			gotoEnd = clearQueue;
			clearQueue = type;
			type = undefined;
		}
		if ( clearQueue && type !== false ) {
			this.queue( type || "fx", [] );
		}

		return this.each(function() {
			var index,
				hadTimers = false,
				timers = jQuery.timers,
				data = jQuery._data( this );

			// clear marker counters if we know they won't be
			if ( !gotoEnd ) {
				jQuery._unmark( true, this );
			}

			function stopQueue( elem, data, index ) {
				var hooks = data[ index ];
				jQuery.removeData( elem, index, true );
				hooks.stop( gotoEnd );
			}

			if ( type == null ) {
				for ( index in data ) {
					if ( data[ index ] && data[ index ].stop && index.indexOf(".run") === index.length - 4 ) {
						stopQueue( this, data, index );
					}
				}
			} else if ( data[ index = type + ".run" ] && data[ index ].stop ){
				stopQueue( this, data, index );
			}

			for ( index = timers.length; index--; ) {
				if ( timers[ index ].elem === this && (type == null || timers[ index ].queue === type) ) {
					if ( gotoEnd ) {

						// force the next step to be the last
						timers[ index ]( true );
					} else {
						timers[ index ].saveState();
					}
					hadTimers = true;
					timers.splice( index, 1 );
				}
			}

			// start the next in the queue if the last step wasn't forced
			// timers currently will call their complete callbacks, which will dequeue
			// but only if they were gotoEnd
			if ( !( gotoEnd && hadTimers ) ) {
				jQuery.dequeue( this, type );
			}
		});
	}

});

// Animations created synchronously will run synchronously
function createFxNow() {
	setTimeout( clearFxNow, 0 );
	return ( fxNow = jQuery.now() );
}

function clearFxNow() {
	fxNow = undefined;
}

// Generate parameters to create a standard animation
function genFx( type, num ) {
	var obj = {};

	jQuery.each( fxAttrs.concat.apply([], fxAttrs.slice( 0, num )), function() {
		obj[ this ] = type;
	});

	return obj;
}

// Generate shortcuts for custom animations
jQuery.each({
	slideDown: genFx( "show", 1 ),
	slideUp: genFx( "hide", 1 ),
	slideToggle: genFx( "toggle", 1 ),
	fadeIn: { opacity: "show" },
	fadeOut: { opacity: "hide" },
	fadeToggle: { opacity: "toggle" }
}, function( name, props ) {
	jQuery.fn[ name ] = function( speed, easing, callback ) {
		return this.animate( props, speed, easing, callback );
	};
});

jQuery.extend({
	speed: function( speed, easing, fn ) {
		var opt = speed && typeof speed === "object" ? jQuery.extend( {}, speed ) : {
			complete: fn || !fn && easing ||
				jQuery.isFunction( speed ) && speed,
			duration: speed,
			easing: fn && easing || easing && !jQuery.isFunction( easing ) && easing
		};

		opt.duration = jQuery.fx.off ? 0 : typeof opt.duration === "number" ? opt.duration :
			opt.duration in jQuery.fx.speeds ? jQuery.fx.speeds[ opt.duration ] : jQuery.fx.speeds._default;

		// normalize opt.queue - true/undefined/null -> "fx"
		if ( opt.queue == null || opt.queue === true ) {
			opt.queue = "fx";
		}

		// Queueing
		opt.old = opt.complete;

		opt.complete = function( noUnmark ) {
			if ( jQuery.isFunction( opt.old ) ) {
				opt.old.call( this );
			}

			if ( opt.queue ) {
				jQuery.dequeue( this, opt.queue );
			} else if ( noUnmark !== false ) {
				jQuery._unmark( this );
			}
		};

		return opt;
	},

	easing: {
		linear: function( p, n, firstNum, diff ) {
			return firstNum + diff * p;
		},
		swing: function( p, n, firstNum, diff ) {
			return ( ( -Math.cos( p*Math.PI ) / 2 ) + 0.5 ) * diff + firstNum;
		}
	},

	timers: [],

	fx: function( elem, options, prop ) {
		this.options = options;
		this.elem = elem;
		this.prop = prop;

		options.orig = options.orig || {};
	}

});

jQuery.fx.prototype = {
	// Simple function for setting a style value
	update: function() {
		if ( this.options.step ) {
			this.options.step.call( this.elem, this.now, this );
		}

		( jQuery.fx.step[ this.prop ] || jQuery.fx.step._default )( this );
	},

	// Get the current size
	cur: function() {
		if ( this.elem[ this.prop ] != null && (!this.elem.style || this.elem.style[ this.prop ] == null) ) {
			return this.elem[ this.prop ];
		}

		var parsed,
			r = jQuery.css( this.elem, this.prop );
		// Empty strings, null, undefined and "auto" are converted to 0,
		// complex values such as "rotate(1rad)" are returned as is,
		// simple values such as "10px" are parsed to Float.
		return isNaN( parsed = parseFloat( r ) ) ? !r || r === "auto" ? 0 : r : parsed;
	},

	// Start an animation from one number to another
	custom: function( from, to, unit ) {
		var self = this,
			fx = jQuery.fx;

		this.startTime = fxNow || createFxNow();
		this.end = to;
		this.now = this.start = from;
		this.pos = this.state = 0;
		this.unit = unit || this.unit || ( jQuery.cssNumber[ this.prop ] ? "" : "px" );

		function t( gotoEnd ) {
			return self.step( gotoEnd );
		}

		t.queue = this.options.queue;
		t.elem = this.elem;
		t.saveState = function() {
			if ( self.options.hide && jQuery._data( self.elem, "fxshow" + self.prop ) === undefined ) {
				jQuery._data( self.elem, "fxshow" + self.prop, self.start );
			}
		};

		if ( t() && jQuery.timers.push(t) && !timerId ) {
			timerId = setInterval( fx.tick, fx.interval );
		}
	},

	// Simple 'show' function
	show: function() {
		var dataShow = jQuery._data( this.elem, "fxshow" + this.prop );

		// Remember where we started, so that we can go back to it later
		this.options.orig[ this.prop ] = dataShow || jQuery.style( this.elem, this.prop );
		this.options.show = true;

		// Begin the animation
		// Make sure that we start at a small width/height to avoid any flash of content
		if ( dataShow !== undefined ) {
			// This show is picking up where a previous hide or show left off
			this.custom( this.cur(), dataShow );
		} else {
			this.custom( this.prop === "width" || this.prop === "height" ? 1 : 0, this.cur() );
		}

		// Start by showing the element
		jQuery( this.elem ).show();
	},

	// Simple 'hide' function
	hide: function() {
		// Remember where we started, so that we can go back to it later
		this.options.orig[ this.prop ] = jQuery._data( this.elem, "fxshow" + this.prop ) || jQuery.style( this.elem, this.prop );
		this.options.hide = true;

		// Begin the animation
		this.custom( this.cur(), 0 );
	},

	// Each step of an animation
	step: function( gotoEnd ) {
		var p, n, complete,
			t = fxNow || createFxNow(),
			done = true,
			elem = this.elem,
			options = this.options;

		if ( gotoEnd || t >= options.duration + this.startTime ) {
			this.now = this.end;
			this.pos = this.state = 1;
			this.update();

			options.animatedProperties[ this.prop ] = true;

			for ( p in options.animatedProperties ) {
				if ( options.animatedProperties[ p ] !== true ) {
					done = false;
				}
			}

			if ( done ) {
				// Reset the overflow
				if ( options.overflow != null && !jQuery.support.shrinkWrapBlocks ) {

					jQuery.each( [ "", "X", "Y" ], function( index, value ) {
						elem.style[ "overflow" + value ] = options.overflow[ index ];
					});
				}

				// Hide the element if the "hide" operation was done
				if ( options.hide ) {
					jQuery( elem ).hide();
				}

				// Reset the properties, if the item has been hidden or shown
				if ( options.hide || options.show ) {
					for ( p in options.animatedProperties ) {
						jQuery.style( elem, p, options.orig[ p ] );
						jQuery.removeData( elem, "fxshow" + p, true );
						// Toggle data is no longer needed
						jQuery.removeData( elem, "toggle" + p, true );
					}
				}

				// Execute the complete function
				// in the event that the complete function throws an exception
				// we must ensure it won't be called twice. #5684

				complete = options.complete;
				if ( complete ) {

					options.complete = false;
					complete.call( elem );
				}
			}

			return false;

		} else {
			// classical easing cannot be used with an Infinity duration
			if ( options.duration == Infinity ) {
				this.now = t;
			} else {
				n = t - this.startTime;
				this.state = n / options.duration;

				// Perform the easing function, defaults to swing
				this.pos = jQuery.easing[ options.animatedProperties[this.prop] ]( this.state, n, 0, 1, options.duration );
				this.now = this.start + ( (this.end - this.start) * this.pos );
			}
			// Perform the next step of the animation
			this.update();
		}

		return true;
	}
};

jQuery.extend( jQuery.fx, {
	tick: function() {
		var timer,
			timers = jQuery.timers,
			i = 0;

		for ( ; i < timers.length; i++ ) {
			timer = timers[ i ];
			// Checks the timer has not already been removed
			if ( !timer() && timers[ i ] === timer ) {
				timers.splice( i--, 1 );
			}
		}

		if ( !timers.length ) {
			jQuery.fx.stop();
		}
	},

	interval: 13,

	stop: function() {
		clearInterval( timerId );
		timerId = null;
	},

	speeds: {
		slow: 600,
		fast: 200,
		// Default speed
		_default: 400
	},

	step: {
		opacity: function( fx ) {
			jQuery.style( fx.elem, "opacity", fx.now );
		},

		_default: function( fx ) {
			if ( fx.elem.style && fx.elem.style[ fx.prop ] != null ) {
				fx.elem.style[ fx.prop ] = fx.now + fx.unit;
			} else {
				fx.elem[ fx.prop ] = fx.now;
			}
		}
	}
});

// Adds width/height step functions
// Do not set anything below 0
jQuery.each([ "width", "height" ], function( i, prop ) {
	jQuery.fx.step[ prop ] = function( fx ) {
		jQuery.style( fx.elem, prop, Math.max(0, fx.now) + fx.unit );
	};
});

if ( jQuery.expr && jQuery.expr.filters ) {
	jQuery.expr.filters.animated = function( elem ) {
		return jQuery.grep(jQuery.timers, function( fn ) {
			return elem === fn.elem;
		}).length;
	};
}

// Try to restore the default display value of an element
function defaultDisplay( nodeName ) {

	if ( !elemdisplay[ nodeName ] ) {

		var body = document.body,
			elem = jQuery( "<" + nodeName + ">" ).appendTo( body ),
			display = elem.css( "display" );
		elem.remove();

		// If the simple way fails,
		// get element's real default display by attaching it to a temp iframe
		if ( display === "none" || display === "" ) {
			// No iframe to use yet, so create it
			if ( !iframe ) {
				iframe = document.createElement( "iframe" );
				iframe.frameBorder = iframe.width = iframe.height = 0;
			}

			body.appendChild( iframe );

			// Create a cacheable copy of the iframe document on first call.
			// IE and Opera will allow us to reuse the iframeDoc without re-writing the fake HTML
			// document to it; WebKit & Firefox won't allow reusing the iframe document.
			if ( !iframeDoc || !iframe.createElement ) {
				iframeDoc = ( iframe.contentWindow || iframe.contentDocument ).document;
				iframeDoc.write( ( document.compatMode === "CSS1Compat" ? "<!doctype html>" : "" ) + "<html><body>" );
				iframeDoc.close();
			}

			elem = iframeDoc.createElement( nodeName );

			iframeDoc.body.appendChild( elem );

			display = jQuery.css( elem, "display" );
			body.removeChild( iframe );
		}

		// Store the correct default display
		elemdisplay[ nodeName ] = display;
	}

	return elemdisplay[ nodeName ];
}




var rtable = /^t(?:able|d|h)$/i,
	rroot = /^(?:body|html)$/i;

if ( "getBoundingClientRect" in document.documentElement ) {
	jQuery.fn.offset = function( options ) {
		var elem = this[0], box;

		if ( options ) {
			return this.each(function( i ) {
				jQuery.offset.setOffset( this, options, i );
			});
		}

		if ( !elem || !elem.ownerDocument ) {
			return null;
		}

		if ( elem === elem.ownerDocument.body ) {
			return jQuery.offset.bodyOffset( elem );
		}

		try {
			box = elem.getBoundingClientRect();
		} catch(e) {}

		var doc = elem.ownerDocument,
			docElem = doc.documentElement;

		// Make sure we're not dealing with a disconnected DOM node
		if ( !box || !jQuery.contains( docElem, elem ) ) {
			return box ? { top: box.top, left: box.left } : { top: 0, left: 0 };
		}

		var body = doc.body,
			win = getWindow(doc),
			clientTop  = docElem.clientTop  || body.clientTop  || 0,
			clientLeft = docElem.clientLeft || body.clientLeft || 0,
			scrollTop  = win.pageYOffset || jQuery.support.boxModel && docElem.scrollTop  || body.scrollTop,
			scrollLeft = win.pageXOffset || jQuery.support.boxModel && docElem.scrollLeft || body.scrollLeft,
			top  = box.top  + scrollTop  - clientTop,
			left = box.left + scrollLeft - clientLeft;

		return { top: top, left: left };
	};

} else {
	jQuery.fn.offset = function( options ) {
		var elem = this[0];

		if ( options ) {
			return this.each(function( i ) {
				jQuery.offset.setOffset( this, options, i );
			});
		}

		if ( !elem || !elem.ownerDocument ) {
			return null;
		}

		if ( elem === elem.ownerDocument.body ) {
			return jQuery.offset.bodyOffset( elem );
		}

		var computedStyle,
			offsetParent = elem.offsetParent,
			prevOffsetParent = elem,
			doc = elem.ownerDocument,
			docElem = doc.documentElement,
			body = doc.body,
			defaultView = doc.defaultView,
			prevComputedStyle = defaultView ? defaultView.getComputedStyle( elem, null ) : elem.currentStyle,
			top = elem.offsetTop,
			left = elem.offsetLeft;

		while ( (elem = elem.parentNode) && elem !== body && elem !== docElem ) {
			if ( jQuery.support.fixedPosition && prevComputedStyle.position === "fixed" ) {
				break;
			}

			computedStyle = defaultView ? defaultView.getComputedStyle(elem, null) : elem.currentStyle;
			top  -= elem.scrollTop;
			left -= elem.scrollLeft;

			if ( elem === offsetParent ) {
				top  += elem.offsetTop;
				left += elem.offsetLeft;

				if ( jQuery.support.doesNotAddBorder && !(jQuery.support.doesAddBorderForTableAndCells && rtable.test(elem.nodeName)) ) {
					top  += parseFloat( computedStyle.borderTopWidth  ) || 0;
					left += parseFloat( computedStyle.borderLeftWidth ) || 0;
				}

				prevOffsetParent = offsetParent;
				offsetParent = elem.offsetParent;
			}

			if ( jQuery.support.subtractsBorderForOverflowNotVisible && computedStyle.overflow !== "visible" ) {
				top  += parseFloat( computedStyle.borderTopWidth  ) || 0;
				left += parseFloat( computedStyle.borderLeftWidth ) || 0;
			}

			prevComputedStyle = computedStyle;
		}

		if ( prevComputedStyle.position === "relative" || prevComputedStyle.position === "static" ) {
			top  += body.offsetTop;
			left += body.offsetLeft;
		}

		if ( jQuery.support.fixedPosition && prevComputedStyle.position === "fixed" ) {
			top  += Math.max( docElem.scrollTop, body.scrollTop );
			left += Math.max( docElem.scrollLeft, body.scrollLeft );
		}

		return { top: top, left: left };
	};
}

jQuery.offset = {

	bodyOffset: function( body ) {
		var top = body.offsetTop,
			left = body.offsetLeft;

		if ( jQuery.support.doesNotIncludeMarginInBodyOffset ) {
			top  += parseFloat( jQuery.css(body, "marginTop") ) || 0;
			left += parseFloat( jQuery.css(body, "marginLeft") ) || 0;
		}

		return { top: top, left: left };
	},

	setOffset: function( elem, options, i ) {
		var position = jQuery.css( elem, "position" );

		// set position first, in-case top/left are set even on static elem
		if ( position === "static" ) {
			elem.style.position = "relative";
		}

		var curElem = jQuery( elem ),
			curOffset = curElem.offset(),
			curCSSTop = jQuery.css( elem, "top" ),
			curCSSLeft = jQuery.css( elem, "left" ),
			calculatePosition = ( position === "absolute" || position === "fixed" ) && jQuery.inArray("auto", [curCSSTop, curCSSLeft]) > -1,
			props = {}, curPosition = {}, curTop, curLeft;

		// need to be able to calculate position if either top or left is auto and position is either absolute or fixed
		if ( calculatePosition ) {
			curPosition = curElem.position();
			curTop = curPosition.top;
			curLeft = curPosition.left;
		} else {
			curTop = parseFloat( curCSSTop ) || 0;
			curLeft = parseFloat( curCSSLeft ) || 0;
		}

		if ( jQuery.isFunction( options ) ) {
			options = options.call( elem, i, curOffset );
		}

		if ( options.top != null ) {
			props.top = ( options.top - curOffset.top ) + curTop;
		}
		if ( options.left != null ) {
			props.left = ( options.left - curOffset.left ) + curLeft;
		}

		if ( "using" in options ) {
			options.using.call( elem, props );
		} else {
			curElem.css( props );
		}
	}
};


jQuery.fn.extend({

	position: function() {
		if ( !this[0] ) {
			return null;
		}

		var elem = this[0],

		// Get *real* offsetParent
		offsetParent = this.offsetParent(),

		// Get correct offsets
		offset       = this.offset(),
		parentOffset = rroot.test(offsetParent[0].nodeName) ? { top: 0, left: 0 } : offsetParent.offset();

		// Subtract element margins
		// note: when an element has margin: auto the offsetLeft and marginLeft
		// are the same in Safari causing offset.left to incorrectly be 0
		offset.top  -= parseFloat( jQuery.css(elem, "marginTop") ) || 0;
		offset.left -= parseFloat( jQuery.css(elem, "marginLeft") ) || 0;

		// Add offsetParent borders
		parentOffset.top  += parseFloat( jQuery.css(offsetParent[0], "borderTopWidth") ) || 0;
		parentOffset.left += parseFloat( jQuery.css(offsetParent[0], "borderLeftWidth") ) || 0;

		// Subtract the two offsets
		return {
			top:  offset.top  - parentOffset.top,
			left: offset.left - parentOffset.left
		};
	},

	offsetParent: function() {
		return this.map(function() {
			var offsetParent = this.offsetParent || document.body;
			while ( offsetParent && (!rroot.test(offsetParent.nodeName) && jQuery.css(offsetParent, "position") === "static") ) {
				offsetParent = offsetParent.offsetParent;
			}
			return offsetParent;
		});
	}
});


// Create scrollLeft and scrollTop methods
jQuery.each( ["Left", "Top"], function( i, name ) {
	var method = "scroll" + name;

	jQuery.fn[ method ] = function( val ) {
		var elem, win;

		if ( val === undefined ) {
			elem = this[ 0 ];

			if ( !elem ) {
				return null;
			}

			win = getWindow( elem );

			// Return the scroll offset
			return win ? ("pageXOffset" in win) ? win[ i ? "pageYOffset" : "pageXOffset" ] :
				jQuery.support.boxModel && win.document.documentElement[ method ] ||
					win.document.body[ method ] :
				elem[ method ];
		}

		// Set the scroll offset
		return this.each(function() {
			win = getWindow( this );

			if ( win ) {
				win.scrollTo(
					!i ? val : jQuery( win ).scrollLeft(),
					 i ? val : jQuery( win ).scrollTop()
				);

			} else {
				this[ method ] = val;
			}
		});
	};
});

function getWindow( elem ) {
	return jQuery.isWindow( elem ) ?
		elem :
		elem.nodeType === 9 ?
			elem.defaultView || elem.parentWindow :
			false;
}




// Create width, height, innerHeight, innerWidth, outerHeight and outerWidth methods
jQuery.each([ "Height", "Width" ], function( i, name ) {

	var type = name.toLowerCase();

	// innerHeight and innerWidth
	jQuery.fn[ "inner" + name ] = function() {
		var elem = this[0];
		return elem ?
			elem.style ?
			parseFloat( jQuery.css( elem, type, "padding" ) ) :
			this[ type ]() :
			null;
	};

	// outerHeight and outerWidth
	jQuery.fn[ "outer" + name ] = function( margin ) {
		var elem = this[0];
		return elem ?
			elem.style ?
			parseFloat( jQuery.css( elem, type, margin ? "margin" : "border" ) ) :
			this[ type ]() :
			null;
	};

	jQuery.fn[ type ] = function( size ) {
		// Get window width or height
		var elem = this[0];
		if ( !elem ) {
			return size == null ? null : this;
		}

		if ( jQuery.isFunction( size ) ) {
			return this.each(function( i ) {
				var self = jQuery( this );
				self[ type ]( size.call( this, i, self[ type ]() ) );
			});
		}

		if ( jQuery.isWindow( elem ) ) {
			// Everyone else use document.documentElement or document.body depending on Quirks vs Standards mode
			// 3rd condition allows Nokia support, as it supports the docElem prop but not CSS1Compat
			var docElemProp = elem.document.documentElement[ "client" + name ],
				body = elem.document.body;
			return elem.document.compatMode === "CSS1Compat" && docElemProp ||
				body && body[ "client" + name ] || docElemProp;

		// Get document width or height
		} else if ( elem.nodeType === 9 ) {
			// Either scroll[Width/Height] or offset[Width/Height], whichever is greater
			return Math.max(
				elem.documentElement["client" + name],
				elem.body["scroll" + name], elem.documentElement["scroll" + name],
				elem.body["offset" + name], elem.documentElement["offset" + name]
			);

		// Get or set width or height on the element
		} else if ( size === undefined ) {
			var orig = jQuery.css( elem, type ),
				ret = parseFloat( orig );

			return jQuery.isNumeric( ret ) ? ret : orig;

		// Set the width or height on the element (default to pixels if value is unitless)
		} else {
			return this.css( type, typeof size === "string" ? size : size + "px" );
		}
	};

});




// Expose jQuery to the global object
window.jQuery = window.$ = jQuery;

// Expose jQuery as an AMD module, but only for AMD loaders that
// understand the issues with loading multiple versions of jQuery
// in a page that all might call define(). The loader will indicate
// they have special allowances for multiple jQuery versions by
// specifying define.amd.jQuery = true. Register as a named module,
// since jQuery can be concatenated with other files that may use define,
// but not use a proper concatenation script that understands anonymous
// AMD modules. A named AMD is safest and most robust way to register.
// Lowercase jquery is used because AMD module names are derived from
// file names, and jQuery is normally delivered in a lowercase file name.
// Do this after creating the global so that if an AMD module wants to call
// noConflict to hide this version of jQuery, it will work.
if ( typeof define === "function" && define.amd && define.amd.jQuery ) {
	define( "jquery", [], function () { return jQuery; } );
}



})( window );
/*!
 * jQuery UI 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI
 */
(function(c,j){function k(a){return!c(a).parents().andSelf().filter(function(){return c.curCSS(this,"visibility")==="hidden"||c.expr.filters.hidden(this)}).length}c.ui=c.ui||{};if(!c.ui.version){c.extend(c.ui,{version:"1.8.10",keyCode:{ALT:18,BACKSPACE:8,CAPS_LOCK:20,COMMA:188,COMMAND:91,COMMAND_LEFT:91,COMMAND_RIGHT:93,CONTROL:17,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,INSERT:45,LEFT:37,MENU:93,NUMPAD_ADD:107,NUMPAD_DECIMAL:110,NUMPAD_DIVIDE:111,NUMPAD_ENTER:108,NUMPAD_MULTIPLY:106,
NUMPAD_SUBTRACT:109,PAGE_DOWN:34,PAGE_UP:33,PERIOD:190,RIGHT:39,SHIFT:16,SPACE:32,TAB:9,UP:38,WINDOWS:91}});c.fn.extend({_focus:c.fn.focus,focus:function(a,b){return typeof a==="number"?this.each(function(){var d=this;setTimeout(function(){c(d).focus();b&&b.call(d)},a)}):this._focus.apply(this,arguments)},scrollParent:function(){var a;a=c.browser.msie&&/(static|relative)/.test(this.css("position"))||/absolute/.test(this.css("position"))?this.parents().filter(function(){return/(relative|absolute|fixed)/.test(c.curCSS(this,
"position",1))&&/(auto|scroll)/.test(c.curCSS(this,"overflow",1)+c.curCSS(this,"overflow-y",1)+c.curCSS(this,"overflow-x",1))}).eq(0):this.parents().filter(function(){return/(auto|scroll)/.test(c.curCSS(this,"overflow",1)+c.curCSS(this,"overflow-y",1)+c.curCSS(this,"overflow-x",1))}).eq(0);return/fixed/.test(this.css("position"))||!a.length?c(document):a},zIndex:function(a){if(a!==j)return this.css("zIndex",a);if(this.length){a=c(this[0]);for(var b;a.length&&a[0]!==document;){b=a.css("position");
if(b==="absolute"||b==="relative"||b==="fixed"){b=parseInt(a.css("zIndex"),10);if(!isNaN(b)&&b!==0)return b}a=a.parent()}}return 0},disableSelection:function(){return this.bind((c.support.selectstart?"selectstart":"mousedown")+".ui-disableSelection",function(a){a.preventDefault()})},enableSelection:function(){return this.unbind(".ui-disableSelection")}});c.each(["Width","Height"],function(a,b){function d(f,g,l,m){c.each(e,function(){g-=parseFloat(c.curCSS(f,"padding"+this,true))||0;if(l)g-=parseFloat(c.curCSS(f,
"border"+this+"Width",true))||0;if(m)g-=parseFloat(c.curCSS(f,"margin"+this,true))||0});return g}var e=b==="Width"?["Left","Right"]:["Top","Bottom"],h=b.toLowerCase(),i={innerWidth:c.fn.innerWidth,innerHeight:c.fn.innerHeight,outerWidth:c.fn.outerWidth,outerHeight:c.fn.outerHeight};c.fn["inner"+b]=function(f){if(f===j)return i["inner"+b].call(this);return this.each(function(){c(this).css(h,d(this,f)+"px")})};c.fn["outer"+b]=function(f,g){if(typeof f!=="number")return i["outer"+b].call(this,f);return this.each(function(){c(this).css(h,
d(this,f,true,g)+"px")})}});c.extend(c.expr[":"],{data:function(a,b,d){return!!c.data(a,d[3])},focusable:function(a){var b=a.nodeName.toLowerCase(),d=c.attr(a,"tabindex");if("area"===b){b=a.parentNode;d=b.name;if(!a.href||!d||b.nodeName.toLowerCase()!=="map")return false;a=c("img[usemap=#"+d+"]")[0];return!!a&&k(a)}return(/input|select|textarea|button|object/.test(b)?!a.disabled:"a"==b?a.href||!isNaN(d):!isNaN(d))&&k(a)},tabbable:function(a){var b=c.attr(a,"tabindex");return(isNaN(b)||b>=0)&&c(a).is(":focusable")}});
c(function(){var a=document.body,b=a.appendChild(b=document.createElement("div"));c.extend(b.style,{minHeight:"100px",height:"auto",padding:0,borderWidth:0});c.support.minHeight=b.offsetHeight===100;c.support.selectstart="onselectstart"in b;a.removeChild(b).style.display="none"});c.extend(c.ui,{plugin:{add:function(a,b,d){a=c.ui[a].prototype;for(var e in d){a.plugins[e]=a.plugins[e]||[];a.plugins[e].push([b,d[e]])}},call:function(a,b,d){if((b=a.plugins[b])&&a.element[0].parentNode)for(var e=0;e<b.length;e++)a.options[b[e][0]]&&
b[e][1].apply(a.element,d)}},contains:function(a,b){return document.compareDocumentPosition?a.compareDocumentPosition(b)&16:a!==b&&a.contains(b)},hasScroll:function(a,b){if(c(a).css("overflow")==="hidden")return false;b=b&&b==="left"?"scrollLeft":"scrollTop";var d=false;if(a[b]>0)return true;a[b]=1;d=a[b]>0;a[b]=0;return d},isOverAxis:function(a,b,d){return a>b&&a<b+d},isOver:function(a,b,d,e,h,i){return c.ui.isOverAxis(a,d,h)&&c.ui.isOverAxis(b,e,i)}})}})(jQuery);
;/*!
 * jQuery UI Widget 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Widget
 */
(function(b,j){if(b.cleanData){var k=b.cleanData;b.cleanData=function(a){for(var c=0,d;(d=a[c])!=null;c++)b(d).triggerHandler("remove");k(a)}}else{var l=b.fn.remove;b.fn.remove=function(a,c){return this.each(function(){if(!c)if(!a||b.filter(a,[this]).length)b("*",this).add([this]).each(function(){b(this).triggerHandler("remove")});return l.call(b(this),a,c)})}}b.widget=function(a,c,d){var e=a.split(".")[0],f;a=a.split(".")[1];f=e+"-"+a;if(!d){d=c;c=b.Widget}b.expr[":"][f]=function(h){return!!b.data(h,
a)};b[e]=b[e]||{};b[e][a]=function(h,g){arguments.length&&this._createWidget(h,g)};c=new c;c.options=b.extend(true,{},c.options);b[e][a].prototype=b.extend(true,c,{namespace:e,widgetName:a,widgetEventPrefix:b[e][a].prototype.widgetEventPrefix||a,widgetBaseClass:f},d);b.widget.bridge(a,b[e][a])};b.widget.bridge=function(a,c){b.fn[a]=function(d){var e=typeof d==="string",f=Array.prototype.slice.call(arguments,1),h=this;d=!e&&f.length?b.extend.apply(null,[true,d].concat(f)):d;if(e&&d.charAt(0)==="_")return h;
e?this.each(function(){var g=b.data(this,a),i=g&&b.isFunction(g[d])?g[d].apply(g,f):g;if(i!==g&&i!==j){h=i;return false}}):this.each(function(){var g=b.data(this,a);g?g.option(d||{})._init():b.data(this,a,new c(d,this))});return h}};b.Widget=function(a,c){arguments.length&&this._createWidget(a,c)};b.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",options:{disabled:false},_createWidget:function(a,c){b.data(c,this.widgetName,this);this.element=b(c);this.options=b.extend(true,{},this.options,
this._getCreateOptions(),a);var d=this;this.element.bind("remove."+this.widgetName,function(){d.destroy()});this._create();this._trigger("create");this._init()},_getCreateOptions:function(){return b.metadata&&b.metadata.get(this.element[0])[this.widgetName]},_create:function(){},_init:function(){},destroy:function(){this.element.unbind("."+this.widgetName).removeData(this.widgetName);this.widget().unbind("."+this.widgetName).removeAttr("aria-disabled").removeClass(this.widgetBaseClass+"-disabled ui-state-disabled")},
widget:function(){return this.element},option:function(a,c){var d=a;if(arguments.length===0)return b.extend({},this.options);if(typeof a==="string"){if(c===j)return this.options[a];d={};d[a]=c}this._setOptions(d);return this},_setOptions:function(a){var c=this;b.each(a,function(d,e){c._setOption(d,e)});return this},_setOption:function(a,c){this.options[a]=c;if(a==="disabled")this.widget()[c?"addClass":"removeClass"](this.widgetBaseClass+"-disabled ui-state-disabled").attr("aria-disabled",c);return this},
enable:function(){return this._setOption("disabled",false)},disable:function(){return this._setOption("disabled",true)},_trigger:function(a,c,d){var e=this.options[a];c=b.Event(c);c.type=(a===this.widgetEventPrefix?a:this.widgetEventPrefix+a).toLowerCase();d=d||{};if(c.originalEvent){a=b.event.props.length;for(var f;a;){f=b.event.props[--a];c[f]=c.originalEvent[f]}}this.element.trigger(c,d);return!(b.isFunction(e)&&e.call(this.element[0],c,d)===false||c.isDefaultPrevented())}}})(jQuery);
;/*!
 * jQuery UI Mouse 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Mouse
 *
 * Depends:
 *	jquery.ui.widget.js
 */
(function(c){c.widget("ui.mouse",{options:{cancel:":input,option",distance:1,delay:0},_mouseInit:function(){var a=this;this.element.bind("mousedown."+this.widgetName,function(b){return a._mouseDown(b)}).bind("click."+this.widgetName,function(b){if(true===c.data(b.target,a.widgetName+".preventClickEvent")){c.removeData(b.target,a.widgetName+".preventClickEvent");b.stopImmediatePropagation();return false}});this.started=false},_mouseDestroy:function(){this.element.unbind("."+this.widgetName)},_mouseDown:function(a){a.originalEvent=
a.originalEvent||{};if(!a.originalEvent.mouseHandled){this._mouseStarted&&this._mouseUp(a);this._mouseDownEvent=a;var b=this,e=a.which==1,f=typeof this.options.cancel=="string"?c(a.target).parents().add(a.target).filter(this.options.cancel).length:false;if(!e||f||!this._mouseCapture(a))return true;this.mouseDelayMet=!this.options.delay;if(!this.mouseDelayMet)this._mouseDelayTimer=setTimeout(function(){b.mouseDelayMet=true},this.options.delay);if(this._mouseDistanceMet(a)&&this._mouseDelayMet(a)){this._mouseStarted=
this._mouseStart(a)!==false;if(!this._mouseStarted){a.preventDefault();return true}}this._mouseMoveDelegate=function(d){return b._mouseMove(d)};this._mouseUpDelegate=function(d){return b._mouseUp(d)};c(document).bind("mousemove."+this.widgetName,this._mouseMoveDelegate).bind("mouseup."+this.widgetName,this._mouseUpDelegate);a.preventDefault();return a.originalEvent.mouseHandled=true}},_mouseMove:function(a){if(c.browser.msie&&!(document.documentMode>=9)&&!a.button)return this._mouseUp(a);if(this._mouseStarted){this._mouseDrag(a);
return a.preventDefault()}if(this._mouseDistanceMet(a)&&this._mouseDelayMet(a))(this._mouseStarted=this._mouseStart(this._mouseDownEvent,a)!==false)?this._mouseDrag(a):this._mouseUp(a);return!this._mouseStarted},_mouseUp:function(a){c(document).unbind("mousemove."+this.widgetName,this._mouseMoveDelegate).unbind("mouseup."+this.widgetName,this._mouseUpDelegate);if(this._mouseStarted){this._mouseStarted=false;a.target==this._mouseDownEvent.target&&c.data(a.target,this.widgetName+".preventClickEvent",
true);this._mouseStop(a)}return false},_mouseDistanceMet:function(a){return Math.max(Math.abs(this._mouseDownEvent.pageX-a.pageX),Math.abs(this._mouseDownEvent.pageY-a.pageY))>=this.options.distance},_mouseDelayMet:function(){return this.mouseDelayMet},_mouseStart:function(){},_mouseDrag:function(){},_mouseStop:function(){},_mouseCapture:function(){return true}})})(jQuery);
;/*
 * jQuery UI Position 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Position
 */
(function(c){c.ui=c.ui||{};var n=/left|center|right/,o=/top|center|bottom/,t=c.fn.position,u=c.fn.offset;c.fn.position=function(b){if(!b||!b.of)return t.apply(this,arguments);b=c.extend({},b);var a=c(b.of),d=a[0],g=(b.collision||"flip").split(" "),e=b.offset?b.offset.split(" "):[0,0],h,k,j;if(d.nodeType===9){h=a.width();k=a.height();j={top:0,left:0}}else if(d.setTimeout){h=a.width();k=a.height();j={top:a.scrollTop(),left:a.scrollLeft()}}else if(d.preventDefault){b.at="left top";h=k=0;j={top:b.of.pageY,
left:b.of.pageX}}else{h=a.outerWidth();k=a.outerHeight();j=a.offset()}c.each(["my","at"],function(){var f=(b[this]||"").split(" ");if(f.length===1)f=n.test(f[0])?f.concat(["center"]):o.test(f[0])?["center"].concat(f):["center","center"];f[0]=n.test(f[0])?f[0]:"center";f[1]=o.test(f[1])?f[1]:"center";b[this]=f});if(g.length===1)g[1]=g[0];e[0]=parseInt(e[0],10)||0;if(e.length===1)e[1]=e[0];e[1]=parseInt(e[1],10)||0;if(b.at[0]==="right")j.left+=h;else if(b.at[0]==="center")j.left+=h/2;if(b.at[1]==="bottom")j.top+=
k;else if(b.at[1]==="center")j.top+=k/2;j.left+=e[0];j.top+=e[1];return this.each(function(){var f=c(this),l=f.outerWidth(),m=f.outerHeight(),p=parseInt(c.curCSS(this,"marginLeft",true))||0,q=parseInt(c.curCSS(this,"marginTop",true))||0,v=l+p+(parseInt(c.curCSS(this,"marginRight",true))||0),w=m+q+(parseInt(c.curCSS(this,"marginBottom",true))||0),i=c.extend({},j),r;if(b.my[0]==="right")i.left-=l;else if(b.my[0]==="center")i.left-=l/2;if(b.my[1]==="bottom")i.top-=m;else if(b.my[1]==="center")i.top-=
m/2;i.left=Math.round(i.left);i.top=Math.round(i.top);r={left:i.left-p,top:i.top-q};c.each(["left","top"],function(s,x){c.ui.position[g[s]]&&c.ui.position[g[s]][x](i,{targetWidth:h,targetHeight:k,elemWidth:l,elemHeight:m,collisionPosition:r,collisionWidth:v,collisionHeight:w,offset:e,my:b.my,at:b.at})});c.fn.bgiframe&&f.bgiframe();f.offset(c.extend(i,{using:b.using}))})};c.ui.position={fit:{left:function(b,a){var d=c(window);d=a.collisionPosition.left+a.collisionWidth-d.width()-d.scrollLeft();b.left=
d>0?b.left-d:Math.max(b.left-a.collisionPosition.left,b.left)},top:function(b,a){var d=c(window);d=a.collisionPosition.top+a.collisionHeight-d.height()-d.scrollTop();b.top=d>0?b.top-d:Math.max(b.top-a.collisionPosition.top,b.top)}},flip:{left:function(b,a){if(a.at[0]!=="center"){var d=c(window);d=a.collisionPosition.left+a.collisionWidth-d.width()-d.scrollLeft();var g=a.my[0]==="left"?-a.elemWidth:a.my[0]==="right"?a.elemWidth:0,e=a.at[0]==="left"?a.targetWidth:-a.targetWidth,h=-2*a.offset[0];b.left+=
a.collisionPosition.left<0?g+e+h:d>0?g+e+h:0}},top:function(b,a){if(a.at[1]!=="center"){var d=c(window);d=a.collisionPosition.top+a.collisionHeight-d.height()-d.scrollTop();var g=a.my[1]==="top"?-a.elemHeight:a.my[1]==="bottom"?a.elemHeight:0,e=a.at[1]==="top"?a.targetHeight:-a.targetHeight,h=-2*a.offset[1];b.top+=a.collisionPosition.top<0?g+e+h:d>0?g+e+h:0}}}};if(!c.offset.setOffset){c.offset.setOffset=function(b,a){if(/static/.test(c.curCSS(b,"position")))b.style.position="relative";var d=c(b),
g=d.offset(),e=parseInt(c.curCSS(b,"top",true),10)||0,h=parseInt(c.curCSS(b,"left",true),10)||0;g={top:a.top-g.top+e,left:a.left-g.left+h};"using"in a?a.using.call(b,g):d.css(g)};c.fn.offset=function(b){var a=this[0];if(!a||!a.ownerDocument)return null;if(b)return this.each(function(){c.offset.setOffset(this,b)});return u.call(this)}}})(jQuery);
;/*
 * jQuery UI Draggable 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Draggables
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.mouse.js
 *	jquery.ui.widget.js
 */
(function(d){d.widget("ui.draggable",d.ui.mouse,{widgetEventPrefix:"drag",options:{addClasses:true,appendTo:"parent",axis:false,connectToSortable:false,containment:false,cursor:"auto",cursorAt:false,grid:false,handle:false,helper:"original",iframeFix:false,opacity:false,refreshPositions:false,revert:false,revertDuration:500,scope:"default",scroll:true,scrollSensitivity:20,scrollSpeed:20,snap:false,snapMode:"both",snapTolerance:20,stack:false,zIndex:false},_create:function(){if(this.options.helper==
"original"&&!/^(?:r|a|f)/.test(this.element.css("position")))this.element[0].style.position="relative";this.options.addClasses&&this.element.addClass("ui-draggable");this.options.disabled&&this.element.addClass("ui-draggable-disabled");this._mouseInit()},destroy:function(){if(this.element.data("draggable")){this.element.removeData("draggable").unbind(".draggable").removeClass("ui-draggable ui-draggable-dragging ui-draggable-disabled");this._mouseDestroy();return this}},_mouseCapture:function(a){var b=
this.options;if(this.helper||b.disabled||d(a.target).is(".ui-resizable-handle"))return false;this.handle=this._getHandle(a);if(!this.handle)return false;return true},_mouseStart:function(a){var b=this.options;this.helper=this._createHelper(a);this._cacheHelperProportions();if(d.ui.ddmanager)d.ui.ddmanager.current=this;this._cacheMargins();this.cssPosition=this.helper.css("position");this.scrollParent=this.helper.scrollParent();this.offset=this.positionAbs=this.element.offset();this.offset={top:this.offset.top-
this.margins.top,left:this.offset.left-this.margins.left};d.extend(this.offset,{click:{left:a.pageX-this.offset.left,top:a.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()});this.originalPosition=this.position=this._generatePosition(a);this.originalPageX=a.pageX;this.originalPageY=a.pageY;b.cursorAt&&this._adjustOffsetFromHelper(b.cursorAt);b.containment&&this._setContainment();if(this._trigger("start",a)===false){this._clear();return false}this._cacheHelperProportions();
d.ui.ddmanager&&!b.dropBehaviour&&d.ui.ddmanager.prepareOffsets(this,a);this.helper.addClass("ui-draggable-dragging");this._mouseDrag(a,true);return true},_mouseDrag:function(a,b){this.position=this._generatePosition(a);this.positionAbs=this._convertPositionTo("absolute");if(!b){b=this._uiHash();if(this._trigger("drag",a,b)===false){this._mouseUp({});return false}this.position=b.position}if(!this.options.axis||this.options.axis!="y")this.helper[0].style.left=this.position.left+"px";if(!this.options.axis||
this.options.axis!="x")this.helper[0].style.top=this.position.top+"px";d.ui.ddmanager&&d.ui.ddmanager.drag(this,a);return false},_mouseStop:function(a){var b=false;if(d.ui.ddmanager&&!this.options.dropBehaviour)b=d.ui.ddmanager.drop(this,a);if(this.dropped){b=this.dropped;this.dropped=false}if((!this.element[0]||!this.element[0].parentNode)&&this.options.helper=="original")return false;if(this.options.revert=="invalid"&&!b||this.options.revert=="valid"&&b||this.options.revert===true||d.isFunction(this.options.revert)&&
this.options.revert.call(this.element,b)){var c=this;d(this.helper).animate(this.originalPosition,parseInt(this.options.revertDuration,10),function(){c._trigger("stop",a)!==false&&c._clear()})}else this._trigger("stop",a)!==false&&this._clear();return false},cancel:function(){this.helper.is(".ui-draggable-dragging")?this._mouseUp({}):this._clear();return this},_getHandle:function(a){var b=!this.options.handle||!d(this.options.handle,this.element).length?true:false;d(this.options.handle,this.element).find("*").andSelf().each(function(){if(this==
a.target)b=true});return b},_createHelper:function(a){var b=this.options;a=d.isFunction(b.helper)?d(b.helper.apply(this.element[0],[a])):b.helper=="clone"?this.element.clone():this.element;a.parents("body").length||a.appendTo(b.appendTo=="parent"?this.element[0].parentNode:b.appendTo);a[0]!=this.element[0]&&!/(fixed|absolute)/.test(a.css("position"))&&a.css("position","absolute");return a},_adjustOffsetFromHelper:function(a){if(typeof a=="string")a=a.split(" ");if(d.isArray(a))a={left:+a[0],top:+a[1]||
0};if("left"in a)this.offset.click.left=a.left+this.margins.left;if("right"in a)this.offset.click.left=this.helperProportions.width-a.right+this.margins.left;if("top"in a)this.offset.click.top=a.top+this.margins.top;if("bottom"in a)this.offset.click.top=this.helperProportions.height-a.bottom+this.margins.top},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var a=this.offsetParent.offset();if(this.cssPosition=="absolute"&&this.scrollParent[0]!=document&&d.ui.contains(this.scrollParent[0],
this.offsetParent[0])){a.left+=this.scrollParent.scrollLeft();a.top+=this.scrollParent.scrollTop()}if(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&this.offsetParent[0].tagName.toLowerCase()=="html"&&d.browser.msie)a={top:0,left:0};return{top:a.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:a.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if(this.cssPosition=="relative"){var a=this.element.position();return{top:a.top-
(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:a.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}else return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.element.css("marginLeft"),10)||0,top:parseInt(this.element.css("marginTop"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var a=this.options;if(a.containment==
"parent")a.containment=this.helper[0].parentNode;if(a.containment=="document"||a.containment=="window")this.containment=[(a.containment=="document"?0:d(window).scrollLeft())-this.offset.relative.left-this.offset.parent.left,(a.containment=="document"?0:d(window).scrollTop())-this.offset.relative.top-this.offset.parent.top,(a.containment=="document"?0:d(window).scrollLeft())+d(a.containment=="document"?document:window).width()-this.helperProportions.width-this.margins.left,(a.containment=="document"?
0:d(window).scrollTop())+(d(a.containment=="document"?document:window).height()||document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top];if(!/^(document|window|parent)$/.test(a.containment)&&a.containment.constructor!=Array){var b=d(a.containment)[0];if(b){a=d(a.containment).offset();var c=d(b).css("overflow")!="hidden";this.containment=[a.left+(parseInt(d(b).css("borderLeftWidth"),10)||0)+(parseInt(d(b).css("paddingLeft"),10)||0)-this.margins.left,a.top+(parseInt(d(b).css("borderTopWidth"),
10)||0)+(parseInt(d(b).css("paddingTop"),10)||0)-this.margins.top,a.left+(c?Math.max(b.scrollWidth,b.offsetWidth):b.offsetWidth)-(parseInt(d(b).css("borderLeftWidth"),10)||0)-(parseInt(d(b).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left,a.top+(c?Math.max(b.scrollHeight,b.offsetHeight):b.offsetHeight)-(parseInt(d(b).css("borderTopWidth"),10)||0)-(parseInt(d(b).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top]}}else if(a.containment.constructor==
Array)this.containment=a.containment},_convertPositionTo:function(a,b){if(!b)b=this.position;a=a=="absolute"?1:-1;var c=this.cssPosition=="absolute"&&!(this.scrollParent[0]!=document&&d.ui.contains(this.scrollParent[0],this.offsetParent[0]))?this.offsetParent:this.scrollParent,f=/(html|body)/i.test(c[0].tagName);return{top:b.top+this.offset.relative.top*a+this.offset.parent.top*a-(d.browser.safari&&d.browser.version<526&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollTop():
f?0:c.scrollTop())*a),left:b.left+this.offset.relative.left*a+this.offset.parent.left*a-(d.browser.safari&&d.browser.version<526&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():f?0:c.scrollLeft())*a)}},_generatePosition:function(a){var b=this.options,c=this.cssPosition=="absolute"&&!(this.scrollParent[0]!=document&&d.ui.contains(this.scrollParent[0],this.offsetParent[0]))?this.offsetParent:this.scrollParent,f=/(html|body)/i.test(c[0].tagName),e=a.pageX,g=a.pageY;
if(this.originalPosition){if(this.containment){if(a.pageX-this.offset.click.left<this.containment[0])e=this.containment[0]+this.offset.click.left;if(a.pageY-this.offset.click.top<this.containment[1])g=this.containment[1]+this.offset.click.top;if(a.pageX-this.offset.click.left>this.containment[2])e=this.containment[2]+this.offset.click.left;if(a.pageY-this.offset.click.top>this.containment[3])g=this.containment[3]+this.offset.click.top}if(b.grid){g=this.originalPageY+Math.round((g-this.originalPageY)/
b.grid[1])*b.grid[1];g=this.containment?!(g-this.offset.click.top<this.containment[1]||g-this.offset.click.top>this.containment[3])?g:!(g-this.offset.click.top<this.containment[1])?g-b.grid[1]:g+b.grid[1]:g;e=this.originalPageX+Math.round((e-this.originalPageX)/b.grid[0])*b.grid[0];e=this.containment?!(e-this.offset.click.left<this.containment[0]||e-this.offset.click.left>this.containment[2])?e:!(e-this.offset.click.left<this.containment[0])?e-b.grid[0]:e+b.grid[0]:e}}return{top:g-this.offset.click.top-
this.offset.relative.top-this.offset.parent.top+(d.browser.safari&&d.browser.version<526&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollTop():f?0:c.scrollTop()),left:e-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+(d.browser.safari&&d.browser.version<526&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():f?0:c.scrollLeft())}},_clear:function(){this.helper.removeClass("ui-draggable-dragging");this.helper[0]!=
this.element[0]&&!this.cancelHelperRemoval&&this.helper.remove();this.helper=null;this.cancelHelperRemoval=false},_trigger:function(a,b,c){c=c||this._uiHash();d.ui.plugin.call(this,a,[b,c]);if(a=="drag")this.positionAbs=this._convertPositionTo("absolute");return d.Widget.prototype._trigger.call(this,a,b,c)},plugins:{},_uiHash:function(){return{helper:this.helper,position:this.position,originalPosition:this.originalPosition,offset:this.positionAbs}}});d.extend(d.ui.draggable,{version:"1.8.10"});
d.ui.plugin.add("draggable","connectToSortable",{start:function(a,b){var c=d(this).data("draggable"),f=c.options,e=d.extend({},b,{item:c.element});c.sortables=[];d(f.connectToSortable).each(function(){var g=d.data(this,"sortable");if(g&&!g.options.disabled){c.sortables.push({instance:g,shouldRevert:g.options.revert});g._refreshItems();g._trigger("activate",a,e)}})},stop:function(a,b){var c=d(this).data("draggable"),f=d.extend({},b,{item:c.element});d.each(c.sortables,function(){if(this.instance.isOver){this.instance.isOver=
0;c.cancelHelperRemoval=true;this.instance.cancelHelperRemoval=false;if(this.shouldRevert)this.instance.options.revert=true;this.instance._mouseStop(a);this.instance.options.helper=this.instance.options._helper;c.options.helper=="original"&&this.instance.currentItem.css({top:"auto",left:"auto"})}else{this.instance.cancelHelperRemoval=false;this.instance._trigger("deactivate",a,f)}})},drag:function(a,b){var c=d(this).data("draggable"),f=this;d.each(c.sortables,function(){this.instance.positionAbs=
c.positionAbs;this.instance.helperProportions=c.helperProportions;this.instance.offset.click=c.offset.click;if(this.instance._intersectsWith(this.instance.containerCache)){if(!this.instance.isOver){this.instance.isOver=1;this.instance.currentItem=d(f).clone().appendTo(this.instance.element).data("sortable-item",true);this.instance.options._helper=this.instance.options.helper;this.instance.options.helper=function(){return b.helper[0]};a.target=this.instance.currentItem[0];this.instance._mouseCapture(a,
true);this.instance._mouseStart(a,true,true);this.instance.offset.click.top=c.offset.click.top;this.instance.offset.click.left=c.offset.click.left;this.instance.offset.parent.left-=c.offset.parent.left-this.instance.offset.parent.left;this.instance.offset.parent.top-=c.offset.parent.top-this.instance.offset.parent.top;c._trigger("toSortable",a);c.dropped=this.instance.element;c.currentItem=c.element;this.instance.fromOutside=c}this.instance.currentItem&&this.instance._mouseDrag(a)}else if(this.instance.isOver){this.instance.isOver=
0;this.instance.cancelHelperRemoval=true;this.instance.options.revert=false;this.instance._trigger("out",a,this.instance._uiHash(this.instance));this.instance._mouseStop(a,true);this.instance.options.helper=this.instance.options._helper;this.instance.currentItem.remove();this.instance.placeholder&&this.instance.placeholder.remove();c._trigger("fromSortable",a);c.dropped=false}})}});d.ui.plugin.add("draggable","cursor",{start:function(){var a=d("body"),b=d(this).data("draggable").options;if(a.css("cursor"))b._cursor=
a.css("cursor");a.css("cursor",b.cursor)},stop:function(){var a=d(this).data("draggable").options;a._cursor&&d("body").css("cursor",a._cursor)}});d.ui.plugin.add("draggable","iframeFix",{start:function(){var a=d(this).data("draggable").options;d(a.iframeFix===true?"iframe":a.iframeFix).each(function(){d('<div class="ui-draggable-iframeFix" style="background: #fff;"></div>').css({width:this.offsetWidth+"px",height:this.offsetHeight+"px",position:"absolute",opacity:"0.001",zIndex:1E3}).css(d(this).offset()).appendTo("body")})},
stop:function(){d("div.ui-draggable-iframeFix").each(function(){this.parentNode.removeChild(this)})}});d.ui.plugin.add("draggable","opacity",{start:function(a,b){a=d(b.helper);b=d(this).data("draggable").options;if(a.css("opacity"))b._opacity=a.css("opacity");a.css("opacity",b.opacity)},stop:function(a,b){a=d(this).data("draggable").options;a._opacity&&d(b.helper).css("opacity",a._opacity)}});d.ui.plugin.add("draggable","scroll",{start:function(){var a=d(this).data("draggable");if(a.scrollParent[0]!=
document&&a.scrollParent[0].tagName!="HTML")a.overflowOffset=a.scrollParent.offset()},drag:function(a){var b=d(this).data("draggable"),c=b.options,f=false;if(b.scrollParent[0]!=document&&b.scrollParent[0].tagName!="HTML"){if(!c.axis||c.axis!="x")if(b.overflowOffset.top+b.scrollParent[0].offsetHeight-a.pageY<c.scrollSensitivity)b.scrollParent[0].scrollTop=f=b.scrollParent[0].scrollTop+c.scrollSpeed;else if(a.pageY-b.overflowOffset.top<c.scrollSensitivity)b.scrollParent[0].scrollTop=f=b.scrollParent[0].scrollTop-
c.scrollSpeed;if(!c.axis||c.axis!="y")if(b.overflowOffset.left+b.scrollParent[0].offsetWidth-a.pageX<c.scrollSensitivity)b.scrollParent[0].scrollLeft=f=b.scrollParent[0].scrollLeft+c.scrollSpeed;else if(a.pageX-b.overflowOffset.left<c.scrollSensitivity)b.scrollParent[0].scrollLeft=f=b.scrollParent[0].scrollLeft-c.scrollSpeed}else{if(!c.axis||c.axis!="x")if(a.pageY-d(document).scrollTop()<c.scrollSensitivity)f=d(document).scrollTop(d(document).scrollTop()-c.scrollSpeed);else if(d(window).height()-
(a.pageY-d(document).scrollTop())<c.scrollSensitivity)f=d(document).scrollTop(d(document).scrollTop()+c.scrollSpeed);if(!c.axis||c.axis!="y")if(a.pageX-d(document).scrollLeft()<c.scrollSensitivity)f=d(document).scrollLeft(d(document).scrollLeft()-c.scrollSpeed);else if(d(window).width()-(a.pageX-d(document).scrollLeft())<c.scrollSensitivity)f=d(document).scrollLeft(d(document).scrollLeft()+c.scrollSpeed)}f!==false&&d.ui.ddmanager&&!c.dropBehaviour&&d.ui.ddmanager.prepareOffsets(b,a)}});d.ui.plugin.add("draggable",
"snap",{start:function(){var a=d(this).data("draggable"),b=a.options;a.snapElements=[];d(b.snap.constructor!=String?b.snap.items||":data(draggable)":b.snap).each(function(){var c=d(this),f=c.offset();this!=a.element[0]&&a.snapElements.push({item:this,width:c.outerWidth(),height:c.outerHeight(),top:f.top,left:f.left})})},drag:function(a,b){for(var c=d(this).data("draggable"),f=c.options,e=f.snapTolerance,g=b.offset.left,n=g+c.helperProportions.width,m=b.offset.top,o=m+c.helperProportions.height,h=
c.snapElements.length-1;h>=0;h--){var i=c.snapElements[h].left,k=i+c.snapElements[h].width,j=c.snapElements[h].top,l=j+c.snapElements[h].height;if(i-e<g&&g<k+e&&j-e<m&&m<l+e||i-e<g&&g<k+e&&j-e<o&&o<l+e||i-e<n&&n<k+e&&j-e<m&&m<l+e||i-e<n&&n<k+e&&j-e<o&&o<l+e){if(f.snapMode!="inner"){var p=Math.abs(j-o)<=e,q=Math.abs(l-m)<=e,r=Math.abs(i-n)<=e,s=Math.abs(k-g)<=e;if(p)b.position.top=c._convertPositionTo("relative",{top:j-c.helperProportions.height,left:0}).top-c.margins.top;if(q)b.position.top=c._convertPositionTo("relative",
{top:l,left:0}).top-c.margins.top;if(r)b.position.left=c._convertPositionTo("relative",{top:0,left:i-c.helperProportions.width}).left-c.margins.left;if(s)b.position.left=c._convertPositionTo("relative",{top:0,left:k}).left-c.margins.left}var t=p||q||r||s;if(f.snapMode!="outer"){p=Math.abs(j-m)<=e;q=Math.abs(l-o)<=e;r=Math.abs(i-g)<=e;s=Math.abs(k-n)<=e;if(p)b.position.top=c._convertPositionTo("relative",{top:j,left:0}).top-c.margins.top;if(q)b.position.top=c._convertPositionTo("relative",{top:l-c.helperProportions.height,
left:0}).top-c.margins.top;if(r)b.position.left=c._convertPositionTo("relative",{top:0,left:i}).left-c.margins.left;if(s)b.position.left=c._convertPositionTo("relative",{top:0,left:k-c.helperProportions.width}).left-c.margins.left}if(!c.snapElements[h].snapping&&(p||q||r||s||t))c.options.snap.snap&&c.options.snap.snap.call(c.element,a,d.extend(c._uiHash(),{snapItem:c.snapElements[h].item}));c.snapElements[h].snapping=p||q||r||s||t}else{c.snapElements[h].snapping&&c.options.snap.release&&c.options.snap.release.call(c.element,
a,d.extend(c._uiHash(),{snapItem:c.snapElements[h].item}));c.snapElements[h].snapping=false}}}});d.ui.plugin.add("draggable","stack",{start:function(){var a=d(this).data("draggable").options;a=d.makeArray(d(a.stack)).sort(function(c,f){return(parseInt(d(c).css("zIndex"),10)||0)-(parseInt(d(f).css("zIndex"),10)||0)});if(a.length){var b=parseInt(a[0].style.zIndex)||0;d(a).each(function(c){this.style.zIndex=b+c});this[0].style.zIndex=b+a.length}}});d.ui.plugin.add("draggable","zIndex",{start:function(a,
b){a=d(b.helper);b=d(this).data("draggable").options;if(a.css("zIndex"))b._zIndex=a.css("zIndex");a.css("zIndex",b.zIndex)},stop:function(a,b){a=d(this).data("draggable").options;a._zIndex&&d(b.helper).css("zIndex",a._zIndex)}})})(jQuery);
;/*
 * jQuery UI Droppable 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Droppables
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.widget.js
 *	jquery.ui.mouse.js
 *	jquery.ui.draggable.js
 */
(function(d){d.widget("ui.droppable",{widgetEventPrefix:"drop",options:{accept:"*",activeClass:false,addClasses:true,greedy:false,hoverClass:false,scope:"default",tolerance:"intersect"},_create:function(){var a=this.options,b=a.accept;this.isover=0;this.isout=1;this.accept=d.isFunction(b)?b:function(c){return c.is(b)};this.proportions={width:this.element[0].offsetWidth,height:this.element[0].offsetHeight};d.ui.ddmanager.droppables[a.scope]=d.ui.ddmanager.droppables[a.scope]||[];d.ui.ddmanager.droppables[a.scope].push(this);
a.addClasses&&this.element.addClass("ui-droppable")},destroy:function(){for(var a=d.ui.ddmanager.droppables[this.options.scope],b=0;b<a.length;b++)a[b]==this&&a.splice(b,1);this.element.removeClass("ui-droppable ui-droppable-disabled").removeData("droppable").unbind(".droppable");return this},_setOption:function(a,b){if(a=="accept")this.accept=d.isFunction(b)?b:function(c){return c.is(b)};d.Widget.prototype._setOption.apply(this,arguments)},_activate:function(a){var b=d.ui.ddmanager.current;this.options.activeClass&&
this.element.addClass(this.options.activeClass);b&&this._trigger("activate",a,this.ui(b))},_deactivate:function(a){var b=d.ui.ddmanager.current;this.options.activeClass&&this.element.removeClass(this.options.activeClass);b&&this._trigger("deactivate",a,this.ui(b))},_over:function(a){var b=d.ui.ddmanager.current;if(!(!b||(b.currentItem||b.element)[0]==this.element[0]))if(this.accept.call(this.element[0],b.currentItem||b.element)){this.options.hoverClass&&this.element.addClass(this.options.hoverClass);
this._trigger("over",a,this.ui(b))}},_out:function(a){var b=d.ui.ddmanager.current;if(!(!b||(b.currentItem||b.element)[0]==this.element[0]))if(this.accept.call(this.element[0],b.currentItem||b.element)){this.options.hoverClass&&this.element.removeClass(this.options.hoverClass);this._trigger("out",a,this.ui(b))}},_drop:function(a,b){var c=b||d.ui.ddmanager.current;if(!c||(c.currentItem||c.element)[0]==this.element[0])return false;var e=false;this.element.find(":data(droppable)").not(".ui-draggable-dragging").each(function(){var g=
d.data(this,"droppable");if(g.options.greedy&&!g.options.disabled&&g.options.scope==c.options.scope&&g.accept.call(g.element[0],c.currentItem||c.element)&&d.ui.intersect(c,d.extend(g,{offset:g.element.offset()}),g.options.tolerance)){e=true;return false}});if(e)return false;if(this.accept.call(this.element[0],c.currentItem||c.element)){this.options.activeClass&&this.element.removeClass(this.options.activeClass);this.options.hoverClass&&this.element.removeClass(this.options.hoverClass);this._trigger("drop",
a,this.ui(c));return this.element}return false},ui:function(a){return{draggable:a.currentItem||a.element,helper:a.helper,position:a.position,offset:a.positionAbs}}});d.extend(d.ui.droppable,{version:"1.8.10"});d.ui.intersect=function(a,b,c){if(!b.offset)return false;var e=(a.positionAbs||a.position.absolute).left,g=e+a.helperProportions.width,f=(a.positionAbs||a.position.absolute).top,h=f+a.helperProportions.height,i=b.offset.left,k=i+b.proportions.width,j=b.offset.top,l=j+b.proportions.height;
switch(c){case "fit":return i<=e&&g<=k&&j<=f&&h<=l;case "intersect":return i<e+a.helperProportions.width/2&&g-a.helperProportions.width/2<k&&j<f+a.helperProportions.height/2&&h-a.helperProportions.height/2<l;case "pointer":return d.ui.isOver((a.positionAbs||a.position.absolute).top+(a.clickOffset||a.offset.click).top,(a.positionAbs||a.position.absolute).left+(a.clickOffset||a.offset.click).left,j,i,b.proportions.height,b.proportions.width);case "touch":return(f>=j&&f<=l||h>=j&&h<=l||f<j&&h>l)&&(e>=
i&&e<=k||g>=i&&g<=k||e<i&&g>k);default:return false}};d.ui.ddmanager={current:null,droppables:{"default":[]},prepareOffsets:function(a,b){var c=d.ui.ddmanager.droppables[a.options.scope]||[],e=b?b.type:null,g=(a.currentItem||a.element).find(":data(droppable)").andSelf(),f=0;a:for(;f<c.length;f++)if(!(c[f].options.disabled||a&&!c[f].accept.call(c[f].element[0],a.currentItem||a.element))){for(var h=0;h<g.length;h++)if(g[h]==c[f].element[0]){c[f].proportions.height=0;continue a}c[f].visible=c[f].element.css("display")!=
"none";if(c[f].visible){c[f].offset=c[f].element.offset();c[f].proportions={width:c[f].element[0].offsetWidth,height:c[f].element[0].offsetHeight};e=="mousedown"&&c[f]._activate.call(c[f],b)}}},drop:function(a,b){var c=false;d.each(d.ui.ddmanager.droppables[a.options.scope]||[],function(){if(this.options){if(!this.options.disabled&&this.visible&&d.ui.intersect(a,this,this.options.tolerance))c=c||this._drop.call(this,b);if(!this.options.disabled&&this.visible&&this.accept.call(this.element[0],a.currentItem||
a.element)){this.isout=1;this.isover=0;this._deactivate.call(this,b)}}});return c},drag:function(a,b){a.options.refreshPositions&&d.ui.ddmanager.prepareOffsets(a,b);d.each(d.ui.ddmanager.droppables[a.options.scope]||[],function(){if(!(this.options.disabled||this.greedyChild||!this.visible)){var c=d.ui.intersect(a,this,this.options.tolerance);if(c=!c&&this.isover==1?"isout":c&&this.isover==0?"isover":null){var e;if(this.options.greedy){var g=this.element.parents(":data(droppable):eq(0)");if(g.length){e=
d.data(g[0],"droppable");e.greedyChild=c=="isover"?1:0}}if(e&&c=="isover"){e.isover=0;e.isout=1;e._out.call(e,b)}this[c]=1;this[c=="isout"?"isover":"isout"]=0;this[c=="isover"?"_over":"_out"].call(this,b);if(e&&c=="isout"){e.isout=0;e.isover=1;e._over.call(e,b)}}}})}}})(jQuery);
;/*
 * jQuery UI Resizable 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Resizables
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.mouse.js
 *	jquery.ui.widget.js
 */
(function(e){e.widget("ui.resizable",e.ui.mouse,{widgetEventPrefix:"resize",options:{alsoResize:false,animate:false,animateDuration:"slow",animateEasing:"swing",aspectRatio:false,autoHide:false,containment:false,ghost:false,grid:false,handles:"e,s,se",helper:false,maxHeight:null,maxWidth:null,minHeight:10,minWidth:10,zIndex:1E3},_create:function(){var b=this,a=this.options;this.element.addClass("ui-resizable");e.extend(this,{_aspectRatio:!!a.aspectRatio,aspectRatio:a.aspectRatio,originalElement:this.element,
_proportionallyResizeElements:[],_helper:a.helper||a.ghost||a.animate?a.helper||"ui-resizable-helper":null});if(this.element[0].nodeName.match(/canvas|textarea|input|select|button|img/i)){/relative/.test(this.element.css("position"))&&e.browser.opera&&this.element.css({position:"relative",top:"auto",left:"auto"});this.element.wrap(e('<div class="ui-wrapper" style="overflow: hidden;"></div>').css({position:this.element.css("position"),width:this.element.outerWidth(),height:this.element.outerHeight(),
top:this.element.css("top"),left:this.element.css("left")}));this.element=this.element.parent().data("resizable",this.element.data("resizable"));this.elementIsWrapper=true;this.element.css({marginLeft:this.originalElement.css("marginLeft"),marginTop:this.originalElement.css("marginTop"),marginRight:this.originalElement.css("marginRight"),marginBottom:this.originalElement.css("marginBottom")});this.originalElement.css({marginLeft:0,marginTop:0,marginRight:0,marginBottom:0});this.originalResizeStyle=
this.originalElement.css("resize");this.originalElement.css("resize","none");this._proportionallyResizeElements.push(this.originalElement.css({position:"static",zoom:1,display:"block"}));this.originalElement.css({margin:this.originalElement.css("margin")});this._proportionallyResize()}this.handles=a.handles||(!e(".ui-resizable-handle",this.element).length?"e,s,se":{n:".ui-resizable-n",e:".ui-resizable-e",s:".ui-resizable-s",w:".ui-resizable-w",se:".ui-resizable-se",sw:".ui-resizable-sw",ne:".ui-resizable-ne",
nw:".ui-resizable-nw"});if(this.handles.constructor==String){if(this.handles=="all")this.handles="n,e,s,w,se,sw,ne,nw";var c=this.handles.split(",");this.handles={};for(var d=0;d<c.length;d++){var f=e.trim(c[d]),g=e('<div class="ui-resizable-handle '+("ui-resizable-"+f)+'"></div>');/sw|se|ne|nw/.test(f)&&g.css({zIndex:++a.zIndex});"se"==f&&g.addClass("ui-icon ui-icon-gripsmall-diagonal-se");this.handles[f]=".ui-resizable-"+f;this.element.append(g)}}this._renderAxis=function(h){h=h||this.element;for(var i in this.handles){if(this.handles[i].constructor==
String)this.handles[i]=e(this.handles[i],this.element).show();if(this.elementIsWrapper&&this.originalElement[0].nodeName.match(/textarea|input|select|button/i)){var j=e(this.handles[i],this.element),k=0;k=/sw|ne|nw|se|n|s/.test(i)?j.outerHeight():j.outerWidth();j=["padding",/ne|nw|n/.test(i)?"Top":/se|sw|s/.test(i)?"Bottom":/^e$/.test(i)?"Right":"Left"].join("");h.css(j,k);this._proportionallyResize()}e(this.handles[i])}};this._renderAxis(this.element);this._handles=e(".ui-resizable-handle",this.element).disableSelection();
this._handles.mouseover(function(){if(!b.resizing){if(this.className)var h=this.className.match(/ui-resizable-(se|sw|ne|nw|n|e|s|w)/i);b.axis=h&&h[1]?h[1]:"se"}});if(a.autoHide){this._handles.hide();e(this.element).addClass("ui-resizable-autohide").hover(function(){e(this).removeClass("ui-resizable-autohide");b._handles.show()},function(){if(!b.resizing){e(this).addClass("ui-resizable-autohide");b._handles.hide()}})}this._mouseInit()},destroy:function(){this._mouseDestroy();var b=function(c){e(c).removeClass("ui-resizable ui-resizable-disabled ui-resizable-resizing").removeData("resizable").unbind(".resizable").find(".ui-resizable-handle").remove()};
if(this.elementIsWrapper){b(this.element);var a=this.element;a.after(this.originalElement.css({position:a.css("position"),width:a.outerWidth(),height:a.outerHeight(),top:a.css("top"),left:a.css("left")})).remove()}this.originalElement.css("resize",this.originalResizeStyle);b(this.originalElement);return this},_mouseCapture:function(b){var a=false;for(var c in this.handles)if(e(this.handles[c])[0]==b.target)a=true;return!this.options.disabled&&a},_mouseStart:function(b){var a=this.options,c=this.element.position(),
d=this.element;this.resizing=true;this.documentScroll={top:e(document).scrollTop(),left:e(document).scrollLeft()};if(d.is(".ui-draggable")||/absolute/.test(d.css("position")))d.css({position:"absolute",top:c.top,left:c.left});e.browser.opera&&/relative/.test(d.css("position"))&&d.css({position:"relative",top:"auto",left:"auto"});this._renderProxy();c=m(this.helper.css("left"));var f=m(this.helper.css("top"));if(a.containment){c+=e(a.containment).scrollLeft()||0;f+=e(a.containment).scrollTop()||0}this.offset=
this.helper.offset();this.position={left:c,top:f};this.size=this._helper?{width:d.outerWidth(),height:d.outerHeight()}:{width:d.width(),height:d.height()};this.originalSize=this._helper?{width:d.outerWidth(),height:d.outerHeight()}:{width:d.width(),height:d.height()};this.originalPosition={left:c,top:f};this.sizeDiff={width:d.outerWidth()-d.width(),height:d.outerHeight()-d.height()};this.originalMousePosition={left:b.pageX,top:b.pageY};this.aspectRatio=typeof a.aspectRatio=="number"?a.aspectRatio:
this.originalSize.width/this.originalSize.height||1;a=e(".ui-resizable-"+this.axis).css("cursor");e("body").css("cursor",a=="auto"?this.axis+"-resize":a);d.addClass("ui-resizable-resizing");this._propagate("start",b);return true},_mouseDrag:function(b){var a=this.helper,c=this.originalMousePosition,d=this._change[this.axis];if(!d)return false;c=d.apply(this,[b,b.pageX-c.left||0,b.pageY-c.top||0]);if(this._aspectRatio||b.shiftKey)c=this._updateRatio(c,b);c=this._respectSize(c,b);this._propagate("resize",
b);a.css({top:this.position.top+"px",left:this.position.left+"px",width:this.size.width+"px",height:this.size.height+"px"});!this._helper&&this._proportionallyResizeElements.length&&this._proportionallyResize();this._updateCache(c);this._trigger("resize",b,this.ui());return false},_mouseStop:function(b){this.resizing=false;var a=this.options,c=this;if(this._helper){var d=this._proportionallyResizeElements,f=d.length&&/textarea/i.test(d[0].nodeName);d=f&&e.ui.hasScroll(d[0],"left")?0:c.sizeDiff.height;
f=f?0:c.sizeDiff.width;f={width:c.helper.width()-f,height:c.helper.height()-d};d=parseInt(c.element.css("left"),10)+(c.position.left-c.originalPosition.left)||null;var g=parseInt(c.element.css("top"),10)+(c.position.top-c.originalPosition.top)||null;a.animate||this.element.css(e.extend(f,{top:g,left:d}));c.helper.height(c.size.height);c.helper.width(c.size.width);this._helper&&!a.animate&&this._proportionallyResize()}e("body").css("cursor","auto");this.element.removeClass("ui-resizable-resizing");
this._propagate("stop",b);this._helper&&this.helper.remove();return false},_updateCache:function(b){this.offset=this.helper.offset();if(l(b.left))this.position.left=b.left;if(l(b.top))this.position.top=b.top;if(l(b.height))this.size.height=b.height;if(l(b.width))this.size.width=b.width},_updateRatio:function(b){var a=this.position,c=this.size,d=this.axis;if(b.height)b.width=c.height*this.aspectRatio;else if(b.width)b.height=c.width/this.aspectRatio;if(d=="sw"){b.left=a.left+(c.width-b.width);b.top=
null}if(d=="nw"){b.top=a.top+(c.height-b.height);b.left=a.left+(c.width-b.width)}return b},_respectSize:function(b){var a=this.options,c=this.axis,d=l(b.width)&&a.maxWidth&&a.maxWidth<b.width,f=l(b.height)&&a.maxHeight&&a.maxHeight<b.height,g=l(b.width)&&a.minWidth&&a.minWidth>b.width,h=l(b.height)&&a.minHeight&&a.minHeight>b.height;if(g)b.width=a.minWidth;if(h)b.height=a.minHeight;if(d)b.width=a.maxWidth;if(f)b.height=a.maxHeight;var i=this.originalPosition.left+this.originalSize.width,j=this.position.top+
this.size.height,k=/sw|nw|w/.test(c);c=/nw|ne|n/.test(c);if(g&&k)b.left=i-a.minWidth;if(d&&k)b.left=i-a.maxWidth;if(h&&c)b.top=j-a.minHeight;if(f&&c)b.top=j-a.maxHeight;if((a=!b.width&&!b.height)&&!b.left&&b.top)b.top=null;else if(a&&!b.top&&b.left)b.left=null;return b},_proportionallyResize:function(){if(this._proportionallyResizeElements.length)for(var b=this.helper||this.element,a=0;a<this._proportionallyResizeElements.length;a++){var c=this._proportionallyResizeElements[a];if(!this.borderDif){var d=
[c.css("borderTopWidth"),c.css("borderRightWidth"),c.css("borderBottomWidth"),c.css("borderLeftWidth")],f=[c.css("paddingTop"),c.css("paddingRight"),c.css("paddingBottom"),c.css("paddingLeft")];this.borderDif=e.map(d,function(g,h){g=parseInt(g,10)||0;h=parseInt(f[h],10)||0;return g+h})}e.browser.msie&&(e(b).is(":hidden")||e(b).parents(":hidden").length)||c.css({height:b.height()-this.borderDif[0]-this.borderDif[2]||0,width:b.width()-this.borderDif[1]-this.borderDif[3]||0})}},_renderProxy:function(){var b=
this.options;this.elementOffset=this.element.offset();if(this._helper){this.helper=this.helper||e('<div style="overflow:hidden;"></div>');var a=e.browser.msie&&e.browser.version<7,c=a?1:0;a=a?2:-1;this.helper.addClass(this._helper).css({width:this.element.outerWidth()+a,height:this.element.outerHeight()+a,position:"absolute",left:this.elementOffset.left-c+"px",top:this.elementOffset.top-c+"px",zIndex:++b.zIndex});this.helper.appendTo("body").disableSelection()}else this.helper=this.element},_change:{e:function(b,
a){return{width:this.originalSize.width+a}},w:function(b,a){return{left:this.originalPosition.left+a,width:this.originalSize.width-a}},n:function(b,a,c){return{top:this.originalPosition.top+c,height:this.originalSize.height-c}},s:function(b,a,c){return{height:this.originalSize.height+c}},se:function(b,a,c){return e.extend(this._change.s.apply(this,arguments),this._change.e.apply(this,[b,a,c]))},sw:function(b,a,c){return e.extend(this._change.s.apply(this,arguments),this._change.w.apply(this,[b,a,
c]))},ne:function(b,a,c){return e.extend(this._change.n.apply(this,arguments),this._change.e.apply(this,[b,a,c]))},nw:function(b,a,c){return e.extend(this._change.n.apply(this,arguments),this._change.w.apply(this,[b,a,c]))}},_propagate:function(b,a){e.ui.plugin.call(this,b,[a,this.ui()]);b!="resize"&&this._trigger(b,a,this.ui())},plugins:{},ui:function(){return{originalElement:this.originalElement,element:this.element,helper:this.helper,position:this.position,size:this.size,originalSize:this.originalSize,
originalPosition:this.originalPosition}}});e.extend(e.ui.resizable,{version:"1.8.10"});e.ui.plugin.add("resizable","alsoResize",{start:function(){var b=e(this).data("resizable").options,a=function(c){e(c).each(function(){var d=e(this);d.data("resizable-alsoresize",{width:parseInt(d.width(),10),height:parseInt(d.height(),10),left:parseInt(d.css("left"),10),top:parseInt(d.css("top"),10),position:d.css("position")})})};if(typeof b.alsoResize=="object"&&!b.alsoResize.parentNode)if(b.alsoResize.length){b.alsoResize=
b.alsoResize[0];a(b.alsoResize)}else e.each(b.alsoResize,function(c){a(c)});else a(b.alsoResize)},resize:function(b,a){var c=e(this).data("resizable");b=c.options;var d=c.originalSize,f=c.originalPosition,g={height:c.size.height-d.height||0,width:c.size.width-d.width||0,top:c.position.top-f.top||0,left:c.position.left-f.left||0},h=function(i,j){e(i).each(function(){var k=e(this),q=e(this).data("resizable-alsoresize"),p={},r=j&&j.length?j:k.parents(a.originalElement[0]).length?["width","height"]:["width",
"height","top","left"];e.each(r,function(n,o){if((n=(q[o]||0)+(g[o]||0))&&n>=0)p[o]=n||null});if(e.browser.opera&&/relative/.test(k.css("position"))){c._revertToRelativePosition=true;k.css({position:"absolute",top:"auto",left:"auto"})}k.css(p)})};typeof b.alsoResize=="object"&&!b.alsoResize.nodeType?e.each(b.alsoResize,function(i,j){h(i,j)}):h(b.alsoResize)},stop:function(){var b=e(this).data("resizable"),a=b.options,c=function(d){e(d).each(function(){var f=e(this);f.css({position:f.data("resizable-alsoresize").position})})};
if(b._revertToRelativePosition){b._revertToRelativePosition=false;typeof a.alsoResize=="object"&&!a.alsoResize.nodeType?e.each(a.alsoResize,function(d){c(d)}):c(a.alsoResize)}e(this).removeData("resizable-alsoresize")}});e.ui.plugin.add("resizable","animate",{stop:function(b){var a=e(this).data("resizable"),c=a.options,d=a._proportionallyResizeElements,f=d.length&&/textarea/i.test(d[0].nodeName),g=f&&e.ui.hasScroll(d[0],"left")?0:a.sizeDiff.height;f={width:a.size.width-(f?0:a.sizeDiff.width),height:a.size.height-
g};g=parseInt(a.element.css("left"),10)+(a.position.left-a.originalPosition.left)||null;var h=parseInt(a.element.css("top"),10)+(a.position.top-a.originalPosition.top)||null;a.element.animate(e.extend(f,h&&g?{top:h,left:g}:{}),{duration:c.animateDuration,easing:c.animateEasing,step:function(){var i={width:parseInt(a.element.css("width"),10),height:parseInt(a.element.css("height"),10),top:parseInt(a.element.css("top"),10),left:parseInt(a.element.css("left"),10)};d&&d.length&&e(d[0]).css({width:i.width,
height:i.height});a._updateCache(i);a._propagate("resize",b)}})}});e.ui.plugin.add("resizable","containment",{start:function(){var b=e(this).data("resizable"),a=b.element,c=b.options.containment;if(a=c instanceof e?c.get(0):/parent/.test(c)?a.parent().get(0):c){b.containerElement=e(a);if(/document/.test(c)||c==document){b.containerOffset={left:0,top:0};b.containerPosition={left:0,top:0};b.parentData={element:e(document),left:0,top:0,width:e(document).width(),height:e(document).height()||document.body.parentNode.scrollHeight}}else{var d=
e(a),f=[];e(["Top","Right","Left","Bottom"]).each(function(i,j){f[i]=m(d.css("padding"+j))});b.containerOffset=d.offset();b.containerPosition=d.position();b.containerSize={height:d.innerHeight()-f[3],width:d.innerWidth()-f[1]};c=b.containerOffset;var g=b.containerSize.height,h=b.containerSize.width;h=e.ui.hasScroll(a,"left")?a.scrollWidth:h;g=e.ui.hasScroll(a)?a.scrollHeight:g;b.parentData={element:a,left:c.left,top:c.top,width:h,height:g}}}},resize:function(b){var a=e(this).data("resizable"),c=a.options,
d=a.containerOffset,f=a.position;b=a._aspectRatio||b.shiftKey;var g={top:0,left:0},h=a.containerElement;if(h[0]!=document&&/static/.test(h.css("position")))g=d;if(f.left<(a._helper?d.left:0)){a.size.width+=a._helper?a.position.left-d.left:a.position.left-g.left;if(b)a.size.height=a.size.width/c.aspectRatio;a.position.left=c.helper?d.left:0}if(f.top<(a._helper?d.top:0)){a.size.height+=a._helper?a.position.top-d.top:a.position.top;if(b)a.size.width=a.size.height*c.aspectRatio;a.position.top=a._helper?
d.top:0}a.offset.left=a.parentData.left+a.position.left;a.offset.top=a.parentData.top+a.position.top;c=Math.abs((a._helper?a.offset.left-g.left:a.offset.left-g.left)+a.sizeDiff.width);d=Math.abs((a._helper?a.offset.top-g.top:a.offset.top-d.top)+a.sizeDiff.height);f=a.containerElement.get(0)==a.element.parent().get(0);g=/relative|absolute/.test(a.containerElement.css("position"));if(f&&g)c-=a.parentData.left;if(c+a.size.width>=a.parentData.width){a.size.width=a.parentData.width-c;if(b)a.size.height=
a.size.width/a.aspectRatio}if(d+a.size.height>=a.parentData.height){a.size.height=a.parentData.height-d;if(b)a.size.width=a.size.height*a.aspectRatio}},stop:function(){var b=e(this).data("resizable"),a=b.options,c=b.containerOffset,d=b.containerPosition,f=b.containerElement,g=e(b.helper),h=g.offset(),i=g.outerWidth()-b.sizeDiff.width;g=g.outerHeight()-b.sizeDiff.height;b._helper&&!a.animate&&/relative/.test(f.css("position"))&&e(this).css({left:h.left-d.left-c.left,width:i,height:g});b._helper&&!a.animate&&
/static/.test(f.css("position"))&&e(this).css({left:h.left-d.left-c.left,width:i,height:g})}});e.ui.plugin.add("resizable","ghost",{start:function(){var b=e(this).data("resizable"),a=b.options,c=b.size;b.ghost=b.originalElement.clone();b.ghost.css({opacity:0.25,display:"block",position:"relative",height:c.height,width:c.width,margin:0,left:0,top:0}).addClass("ui-resizable-ghost").addClass(typeof a.ghost=="string"?a.ghost:"");b.ghost.appendTo(b.helper)},resize:function(){var b=e(this).data("resizable");
b.ghost&&b.ghost.css({position:"relative",height:b.size.height,width:b.size.width})},stop:function(){var b=e(this).data("resizable");b.ghost&&b.helper&&b.helper.get(0).removeChild(b.ghost.get(0))}});e.ui.plugin.add("resizable","grid",{resize:function(){var b=e(this).data("resizable"),a=b.options,c=b.size,d=b.originalSize,f=b.originalPosition,g=b.axis;a.grid=typeof a.grid=="number"?[a.grid,a.grid]:a.grid;var h=Math.round((c.width-d.width)/(a.grid[0]||1))*(a.grid[0]||1);a=Math.round((c.height-d.height)/
(a.grid[1]||1))*(a.grid[1]||1);if(/^(se|s|e)$/.test(g)){b.size.width=d.width+h;b.size.height=d.height+a}else if(/^(ne)$/.test(g)){b.size.width=d.width+h;b.size.height=d.height+a;b.position.top=f.top-a}else{if(/^(sw)$/.test(g)){b.size.width=d.width+h;b.size.height=d.height+a}else{b.size.width=d.width+h;b.size.height=d.height+a;b.position.top=f.top-a}b.position.left=f.left-h}}});var m=function(b){return parseInt(b,10)||0},l=function(b){return!isNaN(parseInt(b,10))}})(jQuery);
;/*
 * jQuery UI Selectable 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Selectables
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.mouse.js
 *	jquery.ui.widget.js
 */
(function(e){e.widget("ui.selectable",e.ui.mouse,{options:{appendTo:"body",autoRefresh:true,distance:0,filter:"*",tolerance:"touch"},_create:function(){var c=this;this.element.addClass("ui-selectable");this.dragged=false;var f;this.refresh=function(){f=e(c.options.filter,c.element[0]);f.each(function(){var d=e(this),b=d.offset();e.data(this,"selectable-item",{element:this,$element:d,left:b.left,top:b.top,right:b.left+d.outerWidth(),bottom:b.top+d.outerHeight(),startselected:false,selected:d.hasClass("ui-selected"),
selecting:d.hasClass("ui-selecting"),unselecting:d.hasClass("ui-unselecting")})})};this.refresh();this.selectees=f.addClass("ui-selectee");this._mouseInit();this.helper=e("<div class='ui-selectable-helper'></div>")},destroy:function(){this.selectees.removeClass("ui-selectee").removeData("selectable-item");this.element.removeClass("ui-selectable ui-selectable-disabled").removeData("selectable").unbind(".selectable");this._mouseDestroy();return this},_mouseStart:function(c){var f=this;this.opos=[c.pageX,
c.pageY];if(!this.options.disabled){var d=this.options;this.selectees=e(d.filter,this.element[0]);this._trigger("start",c);e(d.appendTo).append(this.helper);this.helper.css({left:c.clientX,top:c.clientY,width:0,height:0});d.autoRefresh&&this.refresh();this.selectees.filter(".ui-selected").each(function(){var b=e.data(this,"selectable-item");b.startselected=true;if(!c.metaKey){b.$element.removeClass("ui-selected");b.selected=false;b.$element.addClass("ui-unselecting");b.unselecting=true;f._trigger("unselecting",
c,{unselecting:b.element})}});e(c.target).parents().andSelf().each(function(){var b=e.data(this,"selectable-item");if(b){var g=!c.metaKey||!b.$element.hasClass("ui-selected");b.$element.removeClass(g?"ui-unselecting":"ui-selected").addClass(g?"ui-selecting":"ui-unselecting");b.unselecting=!g;b.selecting=g;(b.selected=g)?f._trigger("selecting",c,{selecting:b.element}):f._trigger("unselecting",c,{unselecting:b.element});return false}})}},_mouseDrag:function(c){var f=this;this.dragged=true;if(!this.options.disabled){var d=
this.options,b=this.opos[0],g=this.opos[1],h=c.pageX,i=c.pageY;if(b>h){var j=h;h=b;b=j}if(g>i){j=i;i=g;g=j}this.helper.css({left:b,top:g,width:h-b,height:i-g});this.selectees.each(function(){var a=e.data(this,"selectable-item");if(!(!a||a.element==f.element[0])){var k=false;if(d.tolerance=="touch")k=!(a.left>h||a.right<b||a.top>i||a.bottom<g);else if(d.tolerance=="fit")k=a.left>b&&a.right<h&&a.top>g&&a.bottom<i;if(k){if(a.selected){a.$element.removeClass("ui-selected");a.selected=false}if(a.unselecting){a.$element.removeClass("ui-unselecting");
a.unselecting=false}if(!a.selecting){a.$element.addClass("ui-selecting");a.selecting=true;f._trigger("selecting",c,{selecting:a.element})}}else{if(a.selecting)if(c.metaKey&&a.startselected){a.$element.removeClass("ui-selecting");a.selecting=false;a.$element.addClass("ui-selected");a.selected=true}else{a.$element.removeClass("ui-selecting");a.selecting=false;if(a.startselected){a.$element.addClass("ui-unselecting");a.unselecting=true}f._trigger("unselecting",c,{unselecting:a.element})}if(a.selected)if(!c.metaKey&&
!a.startselected){a.$element.removeClass("ui-selected");a.selected=false;a.$element.addClass("ui-unselecting");a.unselecting=true;f._trigger("unselecting",c,{unselecting:a.element})}}}});return false}},_mouseStop:function(c){var f=this;this.dragged=false;e(".ui-unselecting",this.element[0]).each(function(){var d=e.data(this,"selectable-item");d.$element.removeClass("ui-unselecting");d.unselecting=false;d.startselected=false;f._trigger("unselected",c,{unselected:d.element})});e(".ui-selecting",this.element[0]).each(function(){var d=
e.data(this,"selectable-item");d.$element.removeClass("ui-selecting").addClass("ui-selected");d.selecting=false;d.selected=true;d.startselected=true;f._trigger("selected",c,{selected:d.element})});this._trigger("stop",c);this.helper.remove();return false}});e.extend(e.ui.selectable,{version:"1.8.10"})})(jQuery);
;/*
 * jQuery UI Sortable 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Sortables
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.mouse.js
 *	jquery.ui.widget.js
 */
(function(d){d.widget("ui.sortable",d.ui.mouse,{widgetEventPrefix:"sort",options:{appendTo:"parent",axis:false,connectWith:false,containment:false,cursor:"auto",cursorAt:false,dropOnEmpty:true,forcePlaceholderSize:false,forceHelperSize:false,grid:false,handle:false,helper:"original",items:"> *",opacity:false,placeholder:false,revert:false,scroll:true,scrollSensitivity:20,scrollSpeed:20,scope:"default",tolerance:"intersect",zIndex:1E3},_create:function(){this.containerCache={};this.element.addClass("ui-sortable");
this.refresh();this.floating=this.items.length?/left|right/.test(this.items[0].item.css("float")):false;this.offset=this.element.offset();this._mouseInit()},destroy:function(){this.element.removeClass("ui-sortable ui-sortable-disabled").removeData("sortable").unbind(".sortable");this._mouseDestroy();for(var a=this.items.length-1;a>=0;a--)this.items[a].item.removeData("sortable-item");return this},_setOption:function(a,b){if(a==="disabled"){this.options[a]=b;this.widget()[b?"addClass":"removeClass"]("ui-sortable-disabled")}else d.Widget.prototype._setOption.apply(this,
arguments)},_mouseCapture:function(a,b){if(this.reverting)return false;if(this.options.disabled||this.options.type=="static")return false;this._refreshItems(a);var c=null,e=this;d(a.target).parents().each(function(){if(d.data(this,"sortable-item")==e){c=d(this);return false}});if(d.data(a.target,"sortable-item")==e)c=d(a.target);if(!c)return false;if(this.options.handle&&!b){var f=false;d(this.options.handle,c).find("*").andSelf().each(function(){if(this==a.target)f=true});if(!f)return false}this.currentItem=
c;this._removeCurrentsFromItems();return true},_mouseStart:function(a,b,c){b=this.options;var e=this;this.currentContainer=this;this.refreshPositions();this.helper=this._createHelper(a);this._cacheHelperProportions();this._cacheMargins();this.scrollParent=this.helper.scrollParent();this.offset=this.currentItem.offset();this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left};this.helper.css("position","absolute");this.cssPosition=this.helper.css("position");d.extend(this.offset,
{click:{left:a.pageX-this.offset.left,top:a.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()});this.originalPosition=this._generatePosition(a);this.originalPageX=a.pageX;this.originalPageY=a.pageY;b.cursorAt&&this._adjustOffsetFromHelper(b.cursorAt);this.domPosition={prev:this.currentItem.prev()[0],parent:this.currentItem.parent()[0]};this.helper[0]!=this.currentItem[0]&&this.currentItem.hide();this._createPlaceholder();b.containment&&this._setContainment();
if(b.cursor){if(d("body").css("cursor"))this._storedCursor=d("body").css("cursor");d("body").css("cursor",b.cursor)}if(b.opacity){if(this.helper.css("opacity"))this._storedOpacity=this.helper.css("opacity");this.helper.css("opacity",b.opacity)}if(b.zIndex){if(this.helper.css("zIndex"))this._storedZIndex=this.helper.css("zIndex");this.helper.css("zIndex",b.zIndex)}if(this.scrollParent[0]!=document&&this.scrollParent[0].tagName!="HTML")this.overflowOffset=this.scrollParent.offset();this._trigger("start",
a,this._uiHash());this._preserveHelperProportions||this._cacheHelperProportions();if(!c)for(c=this.containers.length-1;c>=0;c--)this.containers[c]._trigger("activate",a,e._uiHash(this));if(d.ui.ddmanager)d.ui.ddmanager.current=this;d.ui.ddmanager&&!b.dropBehaviour&&d.ui.ddmanager.prepareOffsets(this,a);this.dragging=true;this.helper.addClass("ui-sortable-helper");this._mouseDrag(a);return true},_mouseDrag:function(a){this.position=this._generatePosition(a);this.positionAbs=this._convertPositionTo("absolute");
if(!this.lastPositionAbs)this.lastPositionAbs=this.positionAbs;if(this.options.scroll){var b=this.options,c=false;if(this.scrollParent[0]!=document&&this.scrollParent[0].tagName!="HTML"){if(this.overflowOffset.top+this.scrollParent[0].offsetHeight-a.pageY<b.scrollSensitivity)this.scrollParent[0].scrollTop=c=this.scrollParent[0].scrollTop+b.scrollSpeed;else if(a.pageY-this.overflowOffset.top<b.scrollSensitivity)this.scrollParent[0].scrollTop=c=this.scrollParent[0].scrollTop-b.scrollSpeed;if(this.overflowOffset.left+
this.scrollParent[0].offsetWidth-a.pageX<b.scrollSensitivity)this.scrollParent[0].scrollLeft=c=this.scrollParent[0].scrollLeft+b.scrollSpeed;else if(a.pageX-this.overflowOffset.left<b.scrollSensitivity)this.scrollParent[0].scrollLeft=c=this.scrollParent[0].scrollLeft-b.scrollSpeed}else{if(a.pageY-d(document).scrollTop()<b.scrollSensitivity)c=d(document).scrollTop(d(document).scrollTop()-b.scrollSpeed);else if(d(window).height()-(a.pageY-d(document).scrollTop())<b.scrollSensitivity)c=d(document).scrollTop(d(document).scrollTop()+
b.scrollSpeed);if(a.pageX-d(document).scrollLeft()<b.scrollSensitivity)c=d(document).scrollLeft(d(document).scrollLeft()-b.scrollSpeed);else if(d(window).width()-(a.pageX-d(document).scrollLeft())<b.scrollSensitivity)c=d(document).scrollLeft(d(document).scrollLeft()+b.scrollSpeed)}c!==false&&d.ui.ddmanager&&!b.dropBehaviour&&d.ui.ddmanager.prepareOffsets(this,a)}this.positionAbs=this._convertPositionTo("absolute");if(!this.options.axis||this.options.axis!="y")this.helper[0].style.left=this.position.left+
"px";if(!this.options.axis||this.options.axis!="x")this.helper[0].style.top=this.position.top+"px";for(b=this.items.length-1;b>=0;b--){c=this.items[b];var e=c.item[0],f=this._intersectsWithPointer(c);if(f)if(e!=this.currentItem[0]&&this.placeholder[f==1?"next":"prev"]()[0]!=e&&!d.ui.contains(this.placeholder[0],e)&&(this.options.type=="semi-dynamic"?!d.ui.contains(this.element[0],e):true)){this.direction=f==1?"down":"up";if(this.options.tolerance=="pointer"||this._intersectsWithSides(c))this._rearrange(a,
c);else break;this._trigger("change",a,this._uiHash());break}}this._contactContainers(a);d.ui.ddmanager&&d.ui.ddmanager.drag(this,a);this._trigger("sort",a,this._uiHash());this.lastPositionAbs=this.positionAbs;return false},_mouseStop:function(a,b){if(a){d.ui.ddmanager&&!this.options.dropBehaviour&&d.ui.ddmanager.drop(this,a);if(this.options.revert){var c=this;b=c.placeholder.offset();c.reverting=true;d(this.helper).animate({left:b.left-this.offset.parent.left-c.margins.left+(this.offsetParent[0]==
document.body?0:this.offsetParent[0].scrollLeft),top:b.top-this.offset.parent.top-c.margins.top+(this.offsetParent[0]==document.body?0:this.offsetParent[0].scrollTop)},parseInt(this.options.revert,10)||500,function(){c._clear(a)})}else this._clear(a,b);return false}},cancel:function(){var a=this;if(this.dragging){this._mouseUp({target:null});this.options.helper=="original"?this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper"):this.currentItem.show();for(var b=this.containers.length-
1;b>=0;b--){this.containers[b]._trigger("deactivate",null,a._uiHash(this));if(this.containers[b].containerCache.over){this.containers[b]._trigger("out",null,a._uiHash(this));this.containers[b].containerCache.over=0}}}if(this.placeholder){this.placeholder[0].parentNode&&this.placeholder[0].parentNode.removeChild(this.placeholder[0]);this.options.helper!="original"&&this.helper&&this.helper[0].parentNode&&this.helper.remove();d.extend(this,{helper:null,dragging:false,reverting:false,_noFinalSort:null});
this.domPosition.prev?d(this.domPosition.prev).after(this.currentItem):d(this.domPosition.parent).prepend(this.currentItem)}return this},serialize:function(a){var b=this._getItemsAsjQuery(a&&a.connected),c=[];a=a||{};d(b).each(function(){var e=(d(a.item||this).attr(a.attribute||"id")||"").match(a.expression||/(.+)[-=_](.+)/);if(e)c.push((a.key||e[1]+"[]")+"="+(a.key&&a.expression?e[1]:e[2]))});!c.length&&a.key&&c.push(a.key+"=");return c.join("&")},toArray:function(a){var b=this._getItemsAsjQuery(a&&
a.connected),c=[];a=a||{};b.each(function(){c.push(d(a.item||this).attr(a.attribute||"id")||"")});return c},_intersectsWith:function(a){var b=this.positionAbs.left,c=b+this.helperProportions.width,e=this.positionAbs.top,f=e+this.helperProportions.height,g=a.left,h=g+a.width,i=a.top,k=i+a.height,j=this.offset.click.top,l=this.offset.click.left;j=e+j>i&&e+j<k&&b+l>g&&b+l<h;return this.options.tolerance=="pointer"||this.options.forcePointerForContainers||this.options.tolerance!="pointer"&&this.helperProportions[this.floating?
"width":"height"]>a[this.floating?"width":"height"]?j:g<b+this.helperProportions.width/2&&c-this.helperProportions.width/2<h&&i<e+this.helperProportions.height/2&&f-this.helperProportions.height/2<k},_intersectsWithPointer:function(a){var b=d.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,a.top,a.height);a=d.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,a.left,a.width);b=b&&a;a=this._getDragVerticalDirection();var c=this._getDragHorizontalDirection();if(!b)return false;return this.floating?
c&&c=="right"||a=="down"?2:1:a&&(a=="down"?2:1)},_intersectsWithSides:function(a){var b=d.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,a.top+a.height/2,a.height);a=d.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,a.left+a.width/2,a.width);var c=this._getDragVerticalDirection(),e=this._getDragHorizontalDirection();return this.floating&&e?e=="right"&&a||e=="left"&&!a:c&&(c=="down"&&b||c=="up"&&!b)},_getDragVerticalDirection:function(){var a=this.positionAbs.top-this.lastPositionAbs.top;
return a!=0&&(a>0?"down":"up")},_getDragHorizontalDirection:function(){var a=this.positionAbs.left-this.lastPositionAbs.left;return a!=0&&(a>0?"right":"left")},refresh:function(a){this._refreshItems(a);this.refreshPositions();return this},_connectWith:function(){var a=this.options;return a.connectWith.constructor==String?[a.connectWith]:a.connectWith},_getItemsAsjQuery:function(a){var b=[],c=[],e=this._connectWith();if(e&&a)for(a=e.length-1;a>=0;a--)for(var f=d(e[a]),g=f.length-1;g>=0;g--){var h=
d.data(f[g],"sortable");if(h&&h!=this&&!h.options.disabled)c.push([d.isFunction(h.options.items)?h.options.items.call(h.element):d(h.options.items,h.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),h])}c.push([d.isFunction(this.options.items)?this.options.items.call(this.element,null,{options:this.options,item:this.currentItem}):d(this.options.items,this.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),this]);for(a=c.length-1;a>=0;a--)c[a][0].each(function(){b.push(this)});
return d(b)},_removeCurrentsFromItems:function(){for(var a=this.currentItem.find(":data(sortable-item)"),b=0;b<this.items.length;b++)for(var c=0;c<a.length;c++)a[c]==this.items[b].item[0]&&this.items.splice(b,1)},_refreshItems:function(a){this.items=[];this.containers=[this];var b=this.items,c=[[d.isFunction(this.options.items)?this.options.items.call(this.element[0],a,{item:this.currentItem}):d(this.options.items,this.element),this]],e=this._connectWith();if(e)for(var f=e.length-1;f>=0;f--)for(var g=
d(e[f]),h=g.length-1;h>=0;h--){var i=d.data(g[h],"sortable");if(i&&i!=this&&!i.options.disabled){c.push([d.isFunction(i.options.items)?i.options.items.call(i.element[0],a,{item:this.currentItem}):d(i.options.items,i.element),i]);this.containers.push(i)}}for(f=c.length-1;f>=0;f--){a=c[f][1];e=c[f][0];h=0;for(g=e.length;h<g;h++){i=d(e[h]);i.data("sortable-item",a);b.push({item:i,instance:a,width:0,height:0,left:0,top:0})}}},refreshPositions:function(a){if(this.offsetParent&&this.helper)this.offset.parent=
this._getParentOffset();for(var b=this.items.length-1;b>=0;b--){var c=this.items[b],e=this.options.toleranceElement?d(this.options.toleranceElement,c.item):c.item;if(!a){c.width=e.outerWidth();c.height=e.outerHeight()}e=e.offset();c.left=e.left;c.top=e.top}if(this.options.custom&&this.options.custom.refreshContainers)this.options.custom.refreshContainers.call(this);else for(b=this.containers.length-1;b>=0;b--){e=this.containers[b].element.offset();this.containers[b].containerCache.left=e.left;this.containers[b].containerCache.top=
e.top;this.containers[b].containerCache.width=this.containers[b].element.outerWidth();this.containers[b].containerCache.height=this.containers[b].element.outerHeight()}return this},_createPlaceholder:function(a){var b=a||this,c=b.options;if(!c.placeholder||c.placeholder.constructor==String){var e=c.placeholder;c.placeholder={element:function(){var f=d(document.createElement(b.currentItem[0].nodeName)).addClass(e||b.currentItem[0].className+" ui-sortable-placeholder").removeClass("ui-sortable-helper")[0];
if(!e)f.style.visibility="hidden";return f},update:function(f,g){if(!(e&&!c.forcePlaceholderSize)){g.height()||g.height(b.currentItem.innerHeight()-parseInt(b.currentItem.css("paddingTop")||0,10)-parseInt(b.currentItem.css("paddingBottom")||0,10));g.width()||g.width(b.currentItem.innerWidth()-parseInt(b.currentItem.css("paddingLeft")||0,10)-parseInt(b.currentItem.css("paddingRight")||0,10))}}}}b.placeholder=d(c.placeholder.element.call(b.element,b.currentItem));b.currentItem.after(b.placeholder);
c.placeholder.update(b,b.placeholder)},_contactContainers:function(a){for(var b=null,c=null,e=this.containers.length-1;e>=0;e--)if(!d.ui.contains(this.currentItem[0],this.containers[e].element[0]))if(this._intersectsWith(this.containers[e].containerCache)){if(!(b&&d.ui.contains(this.containers[e].element[0],b.element[0]))){b=this.containers[e];c=e}}else if(this.containers[e].containerCache.over){this.containers[e]._trigger("out",a,this._uiHash(this));this.containers[e].containerCache.over=0}if(b)if(this.containers.length===
1){this.containers[c]._trigger("over",a,this._uiHash(this));this.containers[c].containerCache.over=1}else if(this.currentContainer!=this.containers[c]){b=1E4;e=null;for(var f=this.positionAbs[this.containers[c].floating?"left":"top"],g=this.items.length-1;g>=0;g--)if(d.ui.contains(this.containers[c].element[0],this.items[g].item[0])){var h=this.items[g][this.containers[c].floating?"left":"top"];if(Math.abs(h-f)<b){b=Math.abs(h-f);e=this.items[g]}}if(e||this.options.dropOnEmpty){this.currentContainer=
this.containers[c];e?this._rearrange(a,e,null,true):this._rearrange(a,null,this.containers[c].element,true);this._trigger("change",a,this._uiHash());this.containers[c]._trigger("change",a,this._uiHash(this));this.options.placeholder.update(this.currentContainer,this.placeholder);this.containers[c]._trigger("over",a,this._uiHash(this));this.containers[c].containerCache.over=1}}},_createHelper:function(a){var b=this.options;a=d.isFunction(b.helper)?d(b.helper.apply(this.element[0],[a,this.currentItem])):
b.helper=="clone"?this.currentItem.clone():this.currentItem;a.parents("body").length||d(b.appendTo!="parent"?b.appendTo:this.currentItem[0].parentNode)[0].appendChild(a[0]);if(a[0]==this.currentItem[0])this._storedCSS={width:this.currentItem[0].style.width,height:this.currentItem[0].style.height,position:this.currentItem.css("position"),top:this.currentItem.css("top"),left:this.currentItem.css("left")};if(a[0].style.width==""||b.forceHelperSize)a.width(this.currentItem.width());if(a[0].style.height==
""||b.forceHelperSize)a.height(this.currentItem.height());return a},_adjustOffsetFromHelper:function(a){if(typeof a=="string")a=a.split(" ");if(d.isArray(a))a={left:+a[0],top:+a[1]||0};if("left"in a)this.offset.click.left=a.left+this.margins.left;if("right"in a)this.offset.click.left=this.helperProportions.width-a.right+this.margins.left;if("top"in a)this.offset.click.top=a.top+this.margins.top;if("bottom"in a)this.offset.click.top=this.helperProportions.height-a.bottom+this.margins.top},_getParentOffset:function(){this.offsetParent=
this.helper.offsetParent();var a=this.offsetParent.offset();if(this.cssPosition=="absolute"&&this.scrollParent[0]!=document&&d.ui.contains(this.scrollParent[0],this.offsetParent[0])){a.left+=this.scrollParent.scrollLeft();a.top+=this.scrollParent.scrollTop()}if(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&this.offsetParent[0].tagName.toLowerCase()=="html"&&d.browser.msie)a={top:0,left:0};return{top:a.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:a.left+(parseInt(this.offsetParent.css("borderLeftWidth"),
10)||0)}},_getRelativeOffset:function(){if(this.cssPosition=="relative"){var a=this.currentItem.position();return{top:a.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:a.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}else return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.currentItem.css("marginLeft"),10)||0,top:parseInt(this.currentItem.css("marginTop"),10)||0}},_cacheHelperProportions:function(){this.helperProportions=
{width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var a=this.options;if(a.containment=="parent")a.containment=this.helper[0].parentNode;if(a.containment=="document"||a.containment=="window")this.containment=[0-this.offset.relative.left-this.offset.parent.left,0-this.offset.relative.top-this.offset.parent.top,d(a.containment=="document"?document:window).width()-this.helperProportions.width-this.margins.left,(d(a.containment=="document"?document:window).height()||
document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top];if(!/^(document|window|parent)$/.test(a.containment)){var b=d(a.containment)[0];a=d(a.containment).offset();var c=d(b).css("overflow")!="hidden";this.containment=[a.left+(parseInt(d(b).css("borderLeftWidth"),10)||0)+(parseInt(d(b).css("paddingLeft"),10)||0)-this.margins.left,a.top+(parseInt(d(b).css("borderTopWidth"),10)||0)+(parseInt(d(b).css("paddingTop"),10)||0)-this.margins.top,a.left+(c?Math.max(b.scrollWidth,
b.offsetWidth):b.offsetWidth)-(parseInt(d(b).css("borderLeftWidth"),10)||0)-(parseInt(d(b).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left,a.top+(c?Math.max(b.scrollHeight,b.offsetHeight):b.offsetHeight)-(parseInt(d(b).css("borderTopWidth"),10)||0)-(parseInt(d(b).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top]}},_convertPositionTo:function(a,b){if(!b)b=this.position;a=a=="absolute"?1:-1;var c=this.cssPosition=="absolute"&&!(this.scrollParent[0]!=
document&&d.ui.contains(this.scrollParent[0],this.offsetParent[0]))?this.offsetParent:this.scrollParent,e=/(html|body)/i.test(c[0].tagName);return{top:b.top+this.offset.relative.top*a+this.offset.parent.top*a-(d.browser.safari&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollTop():e?0:c.scrollTop())*a),left:b.left+this.offset.relative.left*a+this.offset.parent.left*a-(d.browser.safari&&this.cssPosition=="fixed"?0:(this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():
e?0:c.scrollLeft())*a)}},_generatePosition:function(a){var b=this.options,c=this.cssPosition=="absolute"&&!(this.scrollParent[0]!=document&&d.ui.contains(this.scrollParent[0],this.offsetParent[0]))?this.offsetParent:this.scrollParent,e=/(html|body)/i.test(c[0].tagName);if(this.cssPosition=="relative"&&!(this.scrollParent[0]!=document&&this.scrollParent[0]!=this.offsetParent[0]))this.offset.relative=this._getRelativeOffset();var f=a.pageX,g=a.pageY;if(this.originalPosition){if(this.containment){if(a.pageX-
this.offset.click.left<this.containment[0])f=this.containment[0]+this.offset.click.left;if(a.pageY-this.offset.click.top<this.containment[1])g=this.containment[1]+this.offset.click.top;if(a.pageX-this.offset.click.left>this.containment[2])f=this.containment[2]+this.offset.click.left;if(a.pageY-this.offset.click.top>this.containment[3])g=this.containment[3]+this.offset.click.top}if(b.grid){g=this.originalPageY+Math.round((g-this.originalPageY)/b.grid[1])*b.grid[1];g=this.containment?!(g-this.offset.click.top<
this.containment[1]||g-this.offset.click.top>this.containment[3])?g:!(g-this.offset.click.top<this.containment[1])?g-b.grid[1]:g+b.grid[1]:g;f=this.originalPageX+Math.round((f-this.originalPageX)/b.grid[0])*b.grid[0];f=this.containment?!(f-this.offset.click.left<this.containment[0]||f-this.offset.click.left>this.containment[2])?f:!(f-this.offset.click.left<this.containment[0])?f-b.grid[0]:f+b.grid[0]:f}}return{top:g-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+(d.browser.safari&&
this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollTop():e?0:c.scrollTop()),left:f-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+(d.browser.safari&&this.cssPosition=="fixed"?0:this.cssPosition=="fixed"?-this.scrollParent.scrollLeft():e?0:c.scrollLeft())}},_rearrange:function(a,b,c,e){c?c[0].appendChild(this.placeholder[0]):b.item[0].parentNode.insertBefore(this.placeholder[0],this.direction=="down"?b.item[0]:b.item[0].nextSibling);this.counter=
this.counter?++this.counter:1;var f=this,g=this.counter;window.setTimeout(function(){g==f.counter&&f.refreshPositions(!e)},0)},_clear:function(a,b){this.reverting=false;var c=[];!this._noFinalSort&&this.currentItem[0].parentNode&&this.placeholder.before(this.currentItem);this._noFinalSort=null;if(this.helper[0]==this.currentItem[0]){for(var e in this._storedCSS)if(this._storedCSS[e]=="auto"||this._storedCSS[e]=="static")this._storedCSS[e]="";this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper")}else this.currentItem.show();
this.fromOutside&&!b&&c.push(function(f){this._trigger("receive",f,this._uiHash(this.fromOutside))});if((this.fromOutside||this.domPosition.prev!=this.currentItem.prev().not(".ui-sortable-helper")[0]||this.domPosition.parent!=this.currentItem.parent()[0])&&!b)c.push(function(f){this._trigger("update",f,this._uiHash())});if(!d.ui.contains(this.element[0],this.currentItem[0])){b||c.push(function(f){this._trigger("remove",f,this._uiHash())});for(e=this.containers.length-1;e>=0;e--)if(d.ui.contains(this.containers[e].element[0],
this.currentItem[0])&&!b){c.push(function(f){return function(g){f._trigger("receive",g,this._uiHash(this))}}.call(this,this.containers[e]));c.push(function(f){return function(g){f._trigger("update",g,this._uiHash(this))}}.call(this,this.containers[e]))}}for(e=this.containers.length-1;e>=0;e--){b||c.push(function(f){return function(g){f._trigger("deactivate",g,this._uiHash(this))}}.call(this,this.containers[e]));if(this.containers[e].containerCache.over){c.push(function(f){return function(g){f._trigger("out",
g,this._uiHash(this))}}.call(this,this.containers[e]));this.containers[e].containerCache.over=0}}this._storedCursor&&d("body").css("cursor",this._storedCursor);this._storedOpacity&&this.helper.css("opacity",this._storedOpacity);if(this._storedZIndex)this.helper.css("zIndex",this._storedZIndex=="auto"?"":this._storedZIndex);this.dragging=false;if(this.cancelHelperRemoval){if(!b){this._trigger("beforeStop",a,this._uiHash());for(e=0;e<c.length;e++)c[e].call(this,a);this._trigger("stop",a,this._uiHash())}return false}b||
this._trigger("beforeStop",a,this._uiHash());this.placeholder[0].parentNode.removeChild(this.placeholder[0]);this.helper[0]!=this.currentItem[0]&&this.helper.remove();this.helper=null;if(!b){for(e=0;e<c.length;e++)c[e].call(this,a);this._trigger("stop",a,this._uiHash())}this.fromOutside=false;return true},_trigger:function(){d.Widget.prototype._trigger.apply(this,arguments)===false&&this.cancel()},_uiHash:function(a){var b=a||this;return{helper:b.helper,placeholder:b.placeholder||d([]),position:b.position,
originalPosition:b.originalPosition,offset:b.positionAbs,item:b.currentItem,sender:a?a.element:null}}});d.extend(d.ui.sortable,{version:"1.8.10"})})(jQuery);
;/*
 * jQuery UI Button 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Button
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.widget.js
 */
(function(a){var g,i=function(b){a(":ui-button",b.target.form).each(function(){var c=a(this).data("button");setTimeout(function(){c.refresh()},1)})},h=function(b){var c=b.name,d=b.form,f=a([]);if(c)f=d?a(d).find("[name='"+c+"']"):a("[name='"+c+"']",b.ownerDocument).filter(function(){return!this.form});return f};a.widget("ui.button",{options:{disabled:null,text:true,label:null,icons:{primary:null,secondary:null}},_create:function(){this.element.closest("form").unbind("reset.button").bind("reset.button",
i);if(typeof this.options.disabled!=="boolean")this.options.disabled=this.element.attr("disabled");this._determineButtonType();this.hasTitle=!!this.buttonElement.attr("title");var b=this,c=this.options,d=this.type==="checkbox"||this.type==="radio",f="ui-state-hover"+(!d?" ui-state-active":"");if(c.label===null)c.label=this.buttonElement.html();if(this.element.is(":disabled"))c.disabled=true;this.buttonElement.addClass("ui-button ui-widget ui-state-default ui-corner-all").attr("role","button").bind("mouseenter.button",
function(){if(!c.disabled){a(this).addClass("ui-state-hover");this===g&&a(this).addClass("ui-state-active")}}).bind("mouseleave.button",function(){c.disabled||a(this).removeClass(f)}).bind("focus.button",function(){a(this).addClass("ui-state-focus")}).bind("blur.button",function(){a(this).removeClass("ui-state-focus")});d&&this.element.bind("change.button",function(){b.refresh()});if(this.type==="checkbox")this.buttonElement.bind("click.button",function(){if(c.disabled)return false;a(this).toggleClass("ui-state-active");
b.buttonElement.attr("aria-pressed",b.element[0].checked)});else if(this.type==="radio")this.buttonElement.bind("click.button",function(){if(c.disabled)return false;a(this).addClass("ui-state-active");b.buttonElement.attr("aria-pressed",true);var e=b.element[0];h(e).not(e).map(function(){return a(this).button("widget")[0]}).removeClass("ui-state-active").attr("aria-pressed",false)});else{this.buttonElement.bind("mousedown.button",function(){if(c.disabled)return false;a(this).addClass("ui-state-active");
g=this;a(document).one("mouseup",function(){g=null})}).bind("mouseup.button",function(){if(c.disabled)return false;a(this).removeClass("ui-state-active")}).bind("keydown.button",function(e){if(c.disabled)return false;if(e.keyCode==a.ui.keyCode.SPACE||e.keyCode==a.ui.keyCode.ENTER)a(this).addClass("ui-state-active")}).bind("keyup.button",function(){a(this).removeClass("ui-state-active")});this.buttonElement.is("a")&&this.buttonElement.keyup(function(e){e.keyCode===a.ui.keyCode.SPACE&&a(this).click()})}this._setOption("disabled",
c.disabled)},_determineButtonType:function(){this.type=this.element.is(":checkbox")?"checkbox":this.element.is(":radio")?"radio":this.element.is("input")?"input":"button";if(this.type==="checkbox"||this.type==="radio"){this.buttonElement=this.element.parents().last().find("label[for="+this.element.attr("id")+"]");this.element.addClass("ui-helper-hidden-accessible");var b=this.element.is(":checked");b&&this.buttonElement.addClass("ui-state-active");this.buttonElement.attr("aria-pressed",b)}else this.buttonElement=
this.element},widget:function(){return this.buttonElement},destroy:function(){this.element.removeClass("ui-helper-hidden-accessible");this.buttonElement.removeClass("ui-button ui-widget ui-state-default ui-corner-all ui-state-hover ui-state-active  ui-button-icons-only ui-button-icon-only ui-button-text-icons ui-button-text-icon-primary ui-button-text-icon-secondary ui-button-text-only").removeAttr("role").removeAttr("aria-pressed").html(this.buttonElement.find(".ui-button-text").html());this.hasTitle||
this.buttonElement.removeAttr("title");a.Widget.prototype.destroy.call(this)},_setOption:function(b,c){a.Widget.prototype._setOption.apply(this,arguments);if(b==="disabled")c?this.element.attr("disabled",true):this.element.removeAttr("disabled");this._resetButton()},refresh:function(){var b=this.element.is(":disabled");b!==this.options.disabled&&this._setOption("disabled",b);if(this.type==="radio")h(this.element[0]).each(function(){a(this).is(":checked")?a(this).button("widget").addClass("ui-state-active").attr("aria-pressed",
true):a(this).button("widget").removeClass("ui-state-active").attr("aria-pressed",false)});else if(this.type==="checkbox")this.element.is(":checked")?this.buttonElement.addClass("ui-state-active").attr("aria-pressed",true):this.buttonElement.removeClass("ui-state-active").attr("aria-pressed",false)},_resetButton:function(){if(this.type==="input")this.options.label&&this.element.val(this.options.label);else{var b=this.buttonElement.removeClass("ui-button-icons-only ui-button-icon-only ui-button-text-icons ui-button-text-icon-primary ui-button-text-icon-secondary ui-button-text-only"),
c=a("<span></span>").addClass("ui-button-text").html(this.options.label).appendTo(b.empty()).text(),d=this.options.icons,f=d.primary&&d.secondary,e=[];if(d.primary||d.secondary){e.push("ui-button-text-icon"+(f?"s":d.primary?"-primary":"-secondary"));d.primary&&b.prepend("<span class='ui-button-icon-primary ui-icon "+d.primary+"'></span>");d.secondary&&b.append("<span class='ui-button-icon-secondary ui-icon "+d.secondary+"'></span>");if(!this.options.text){e.push(f?"ui-button-icons-only":"ui-button-icon-only");
b.removeClass("ui-button-text-icons ui-button-text-icon-primary ui-button-text-icon-secondary");this.hasTitle||b.attr("title",c)}}else e.push("ui-button-text-only");b.addClass(e.join(" "))}}});a.widget("ui.buttonset",{options:{items:":button, :submit, :reset, :checkbox, :radio, a, :data(button)"},_create:function(){this.element.addClass("ui-buttonset")},_init:function(){this.refresh()},_setOption:function(b,c){b==="disabled"&&this.buttons.button("option",b,c);a.Widget.prototype._setOption.apply(this,
arguments)},refresh:function(){this.buttons=this.element.find(this.options.items).filter(":ui-button").button("refresh").end().not(":ui-button").button().end().map(function(){return a(this).button("widget")[0]}).removeClass("ui-corner-all ui-corner-left ui-corner-right").filter(":first").addClass("ui-corner-left").end().filter(":last").addClass("ui-corner-right").end().end()},destroy:function(){this.element.removeClass("ui-buttonset");this.buttons.map(function(){return a(this).button("widget")[0]}).removeClass("ui-corner-left ui-corner-right").end().button("destroy");
a.Widget.prototype.destroy.call(this)}})})(jQuery);
;/*
 * jQuery UI Dialog 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Dialog
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.widget.js
 *  jquery.ui.button.js
 *	jquery.ui.draggable.js
 *	jquery.ui.mouse.js
 *	jquery.ui.position.js
 *	jquery.ui.resizable.js
 */
(function(c,j){var k={buttons:true,height:true,maxHeight:true,maxWidth:true,minHeight:true,minWidth:true,width:true},l={maxHeight:true,maxWidth:true,minHeight:true,minWidth:true};c.widget("ui.dialog",{options:{autoOpen:true,buttons:{},closeOnEscape:true,closeText:"close",dialogClass:"",draggable:true,hide:null,height:"auto",maxHeight:false,maxWidth:false,minHeight:150,minWidth:150,modal:false,position:{my:"center",at:"center",collision:"fit",using:function(a){var b=c(this).css(a).offset().top;b<0&&
c(this).css("top",a.top-b)}},resizable:true,show:null,stack:true,title:"",width:300,zIndex:1E3},_create:function(){this.originalTitle=this.element.attr("title");if(typeof this.originalTitle!=="string")this.originalTitle="";this.options.title=this.options.title||this.originalTitle;var a=this,b=a.options,d=b.title||"&#160;",e=c.ui.dialog.getTitleId(a.element),g=(a.uiDialog=c("<div></div>")).appendTo(document.body).hide().addClass("ui-dialog ui-widget ui-widget-content ui-corner-all "+b.dialogClass).css({zIndex:b.zIndex}).attr("tabIndex",
-1).css("outline",0).keydown(function(i){if(b.closeOnEscape&&i.keyCode&&i.keyCode===c.ui.keyCode.ESCAPE){a.close(i);i.preventDefault()}}).attr({role:"dialog","aria-labelledby":e}).mousedown(function(i){a.moveToTop(false,i)});a.element.show().removeAttr("title").addClass("ui-dialog-content ui-widget-content").appendTo(g);var f=(a.uiDialogTitlebar=c("<div></div>")).addClass("ui-dialog-titlebar ui-widget-header ui-corner-all ui-helper-clearfix").prependTo(g),h=c('<a href="#"></a>').addClass("ui-dialog-titlebar-close ui-corner-all").attr("role",
"button").hover(function(){h.addClass("ui-state-hover")},function(){h.removeClass("ui-state-hover")}).focus(function(){h.addClass("ui-state-focus")}).blur(function(){h.removeClass("ui-state-focus")}).click(function(i){a.close(i);return false}).appendTo(f);(a.uiDialogTitlebarCloseText=c("<span></span>")).addClass("ui-icon ui-icon-closethick").text(b.closeText).appendTo(h);c("<span></span>").addClass("ui-dialog-title").attr("id",e).html(d).prependTo(f);if(c.isFunction(b.beforeclose)&&!c.isFunction(b.beforeClose))b.beforeClose=
b.beforeclose;f.find("*").add(f).disableSelection();b.draggable&&c.fn.draggable&&a._makeDraggable();b.resizable&&c.fn.resizable&&a._makeResizable();a._createButtons(b.buttons);a._isOpen=false;c.fn.bgiframe&&g.bgiframe()},_init:function(){this.options.autoOpen&&this.open()},destroy:function(){var a=this;a.overlay&&a.overlay.destroy();a.uiDialog.hide();a.element.unbind(".dialog").removeData("dialog").removeClass("ui-dialog-content ui-widget-content").hide().appendTo("body");a.uiDialog.remove();a.originalTitle&&
a.element.attr("title",a.originalTitle);return a},widget:function(){return this.uiDialog},close:function(a){var b=this,d,e;if(false!==b._trigger("beforeClose",a)){b.overlay&&b.overlay.destroy();b.uiDialog.unbind("keypress.ui-dialog");b._isOpen=false;if(b.options.hide)b.uiDialog.hide(b.options.hide,function(){b._trigger("close",a)});else{b.uiDialog.hide();b._trigger("close",a)}c.ui.dialog.overlay.resize();if(b.options.modal){d=0;c(".ui-dialog").each(function(){if(this!==b.uiDialog[0]){e=c(this).css("z-index");
isNaN(e)||(d=Math.max(d,e))}});c.ui.dialog.maxZ=d}return b}},isOpen:function(){return this._isOpen},moveToTop:function(a,b){var d=this,e=d.options;if(e.modal&&!a||!e.stack&&!e.modal)return d._trigger("focus",b);if(e.zIndex>c.ui.dialog.maxZ)c.ui.dialog.maxZ=e.zIndex;if(d.overlay){c.ui.dialog.maxZ+=1;d.overlay.$el.css("z-index",c.ui.dialog.overlay.maxZ=c.ui.dialog.maxZ)}a={scrollTop:d.element.attr("scrollTop"),scrollLeft:d.element.attr("scrollLeft")};c.ui.dialog.maxZ+=1;d.uiDialog.css("z-index",c.ui.dialog.maxZ);
d.element.attr(a);d._trigger("focus",b);return d},open:function(){if(!this._isOpen){var a=this,b=a.options,d=a.uiDialog;a.overlay=b.modal?new c.ui.dialog.overlay(a):null;a._size();a._position(b.position);d.show(b.show);a.moveToTop(true);b.modal&&d.bind("keypress.ui-dialog",function(e){if(e.keyCode===c.ui.keyCode.TAB){var g=c(":tabbable",this),f=g.filter(":first");g=g.filter(":last");if(e.target===g[0]&&!e.shiftKey){f.focus(1);return false}else if(e.target===f[0]&&e.shiftKey){g.focus(1);return false}}});
c(a.element.find(":tabbable").get().concat(d.find(".ui-dialog-buttonpane :tabbable").get().concat(d.get()))).eq(0).focus();a._isOpen=true;a._trigger("open");return a}},_createButtons:function(a){var b=this,d=false,e=c("<div></div>").addClass("ui-dialog-buttonpane ui-widget-content ui-helper-clearfix"),g=c("<div></div>").addClass("ui-dialog-buttonset").appendTo(e);b.uiDialog.find(".ui-dialog-buttonpane").remove();typeof a==="object"&&a!==null&&c.each(a,function(){return!(d=true)});if(d){c.each(a,function(f,
h){h=c.isFunction(h)?{click:h,text:f}:h;f=c('<button type="button"></button>').attr(h,true).unbind("click").click(function(){h.click.apply(b.element[0],arguments)}).appendTo(g);c.fn.button&&f.button()});e.appendTo(b.uiDialog)}},_makeDraggable:function(){function a(f){return{position:f.position,offset:f.offset}}var b=this,d=b.options,e=c(document),g;b.uiDialog.draggable({cancel:".ui-dialog-content, .ui-dialog-titlebar-close",handle:".ui-dialog-titlebar",containment:"document",start:function(f,h){g=
d.height==="auto"?"auto":c(this).height();c(this).height(c(this).height()).addClass("ui-dialog-dragging");b._trigger("dragStart",f,a(h))},drag:function(f,h){b._trigger("drag",f,a(h))},stop:function(f,h){d.position=[h.position.left-e.scrollLeft(),h.position.top-e.scrollTop()];c(this).removeClass("ui-dialog-dragging").height(g);b._trigger("dragStop",f,a(h));c.ui.dialog.overlay.resize()}})},_makeResizable:function(a){function b(f){return{originalPosition:f.originalPosition,originalSize:f.originalSize,
position:f.position,size:f.size}}a=a===j?this.options.resizable:a;var d=this,e=d.options,g=d.uiDialog.css("position");a=typeof a==="string"?a:"n,e,s,w,se,sw,ne,nw";d.uiDialog.resizable({cancel:".ui-dialog-content",containment:"document",alsoResize:d.element,maxWidth:e.maxWidth,maxHeight:e.maxHeight,minWidth:e.minWidth,minHeight:d._minHeight(),handles:a,start:function(f,h){c(this).addClass("ui-dialog-resizing");d._trigger("resizeStart",f,b(h))},resize:function(f,h){d._trigger("resize",f,b(h))},stop:function(f,
h){c(this).removeClass("ui-dialog-resizing");e.height=c(this).height();e.width=c(this).width();d._trigger("resizeStop",f,b(h));c.ui.dialog.overlay.resize()}}).css("position",g).find(".ui-resizable-se").addClass("ui-icon ui-icon-grip-diagonal-se")},_minHeight:function(){var a=this.options;return a.height==="auto"?a.minHeight:Math.min(a.minHeight,a.height)},_position:function(a){var b=[],d=[0,0],e;if(a){if(typeof a==="string"||typeof a==="object"&&"0"in a){b=a.split?a.split(" "):[a[0],a[1]];if(b.length===
1)b[1]=b[0];c.each(["left","top"],function(g,f){if(+b[g]===b[g]){d[g]=b[g];b[g]=f}});a={my:b.join(" "),at:b.join(" "),offset:d.join(" ")}}a=c.extend({},c.ui.dialog.prototype.options.position,a)}else a=c.ui.dialog.prototype.options.position;(e=this.uiDialog.is(":visible"))||this.uiDialog.show();this.uiDialog.css({top:0,left:0}).position(c.extend({of:window},a));e||this.uiDialog.hide()},_setOptions:function(a){var b=this,d={},e=false;c.each(a,function(g,f){b._setOption(g,f);if(g in k)e=true;if(g in
l)d[g]=f});e&&this._size();this.uiDialog.is(":data(resizable)")&&this.uiDialog.resizable("option",d)},_setOption:function(a,b){var d=this,e=d.uiDialog;switch(a){case "beforeclose":a="beforeClose";break;case "buttons":d._createButtons(b);break;case "closeText":d.uiDialogTitlebarCloseText.text(""+b);break;case "dialogClass":e.removeClass(d.options.dialogClass).addClass("ui-dialog ui-widget ui-widget-content ui-corner-all "+b);break;case "disabled":b?e.addClass("ui-dialog-disabled"):e.removeClass("ui-dialog-disabled");
break;case "draggable":var g=e.is(":data(draggable)");g&&!b&&e.draggable("destroy");!g&&b&&d._makeDraggable();break;case "position":d._position(b);break;case "resizable":(g=e.is(":data(resizable)"))&&!b&&e.resizable("destroy");g&&typeof b==="string"&&e.resizable("option","handles",b);!g&&b!==false&&d._makeResizable(b);break;case "title":c(".ui-dialog-title",d.uiDialogTitlebar).html(""+(b||"&#160;"));break}c.Widget.prototype._setOption.apply(d,arguments)},_size:function(){var a=this.options,b,d,e=
this.uiDialog.is(":visible");this.element.show().css({width:"auto",minHeight:0,height:0});if(a.minWidth>a.width)a.width=a.minWidth;b=this.uiDialog.css({height:"auto",width:a.width}).height();d=Math.max(0,a.minHeight-b);if(a.height==="auto")if(c.support.minHeight)this.element.css({minHeight:d,height:"auto"});else{this.uiDialog.show();a=this.element.css("height","auto").height();e||this.uiDialog.hide();this.element.height(Math.max(a,d))}else this.element.height(Math.max(a.height-b,0));this.uiDialog.is(":data(resizable)")&&
this.uiDialog.resizable("option","minHeight",this._minHeight())}});c.extend(c.ui.dialog,{version:"1.8.10",uuid:0,maxZ:0,getTitleId:function(a){a=a.attr("id");if(!a){this.uuid+=1;a=this.uuid}return"ui-dialog-title-"+a},overlay:function(a){this.$el=c.ui.dialog.overlay.create(a)}});c.extend(c.ui.dialog.overlay,{instances:[],oldInstances:[],maxZ:0,events:c.map("focus,mousedown,mouseup,keydown,keypress,click".split(","),function(a){return a+".dialog-overlay"}).join(" "),create:function(a){if(this.instances.length===
0){setTimeout(function(){c.ui.dialog.overlay.instances.length&&c(document).bind(c.ui.dialog.overlay.events,function(d){if(c(d.target).zIndex()<c.ui.dialog.overlay.maxZ)return false})},1);c(document).bind("keydown.dialog-overlay",function(d){if(a.options.closeOnEscape&&d.keyCode&&d.keyCode===c.ui.keyCode.ESCAPE){a.close(d);d.preventDefault()}});c(window).bind("resize.dialog-overlay",c.ui.dialog.overlay.resize)}var b=(this.oldInstances.pop()||c("<div></div>").addClass("ui-widget-overlay")).appendTo(document.body).css({width:this.width(),
height:this.height()});c.fn.bgiframe&&b.bgiframe();this.instances.push(b);return b},destroy:function(a){var b=c.inArray(a,this.instances);b!=-1&&this.oldInstances.push(this.instances.splice(b,1)[0]);this.instances.length===0&&c([document,window]).unbind(".dialog-overlay");a.remove();var d=0;c.each(this.instances,function(){d=Math.max(d,this.css("z-index"))});this.maxZ=d},height:function(){var a,b;if(c.browser.msie&&c.browser.version<7){a=Math.max(document.documentElement.scrollHeight,document.body.scrollHeight);
b=Math.max(document.documentElement.offsetHeight,document.body.offsetHeight);return a<b?c(window).height()+"px":a+"px"}else return c(document).height()+"px"},width:function(){var a,b;if(c.browser.msie&&c.browser.version<7){a=Math.max(document.documentElement.scrollWidth,document.body.scrollWidth);b=Math.max(document.documentElement.offsetWidth,document.body.offsetWidth);return a<b?c(window).width()+"px":a+"px"}else return c(document).width()+"px"},resize:function(){var a=c([]);c.each(c.ui.dialog.overlay.instances,
function(){a=a.add(this)});a.css({width:0,height:0}).css({width:c.ui.dialog.overlay.width(),height:c.ui.dialog.overlay.height()})}});c.extend(c.ui.dialog.overlay.prototype,{destroy:function(){c.ui.dialog.overlay.destroy(this.$el)}})})(jQuery);
;/*
 * jQuery UI Tabs 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Tabs
 *
 * Depends:
 *	jquery.ui.core.js
 *	jquery.ui.widget.js
 */
(function(d,p){function u(){return++v}function w(){return++x}var v=0,x=0;d.widget("ui.tabs",{options:{add:null,ajaxOptions:null,cache:false,cookie:null,collapsible:false,disable:null,disabled:[],enable:null,event:"click",fx:null,idPrefix:"ui-tabs-",load:null,panelTemplate:"<div></div>",remove:null,select:null,show:null,spinner:"<em>Loading&#8230;</em>",tabTemplate:"<li><a href='#{href}'><span>#{label}</span></a></li>"},_create:function(){this._tabify(true)},_setOption:function(b,e){if(b=="selected")this.options.collapsible&&
e==this.options.selected||this.select(e);else{this.options[b]=e;this._tabify()}},_tabId:function(b){return b.title&&b.title.replace(/\s/g,"_").replace(/[^\w\u00c0-\uFFFF-]/g,"")||this.options.idPrefix+u()},_sanitizeSelector:function(b){return b.replace(/:/g,"\\:")},_cookie:function(){var b=this.cookie||(this.cookie=this.options.cookie.name||"ui-tabs-"+w());return d.cookie.apply(null,[b].concat(d.makeArray(arguments)))},_ui:function(b,e){return{tab:b,panel:e,index:this.anchors.index(b)}},_cleanup:function(){this.lis.filter(".ui-state-processing").removeClass("ui-state-processing").find("span:data(label.tabs)").each(function(){var b=
d(this);b.html(b.data("label.tabs")).removeData("label.tabs")})},_tabify:function(b){function e(g,f){g.css("display","");!d.support.opacity&&f.opacity&&g[0].style.removeAttribute("filter")}var a=this,c=this.options,h=/^#.+/;this.list=this.element.find("ol,ul").eq(0);this.lis=d(" > li:has(a[href])",this.list);this.anchors=this.lis.map(function(){return d("a",this)[0]});this.panels=d([]);this.anchors.each(function(g,f){var i=d(f).attr("href"),l=i.split("#")[0],q;if(l&&(l===location.toString().split("#")[0]||
(q=d("base")[0])&&l===q.href)){i=f.hash;f.href=i}if(h.test(i))a.panels=a.panels.add(a.element.find(a._sanitizeSelector(i)));else if(i&&i!=="#"){d.data(f,"href.tabs",i);d.data(f,"load.tabs",i.replace(/#.*$/,""));i=a._tabId(f);f.href="#"+i;f=a.element.find("#"+i);if(!f.length){f=d(c.panelTemplate).attr("id",i).addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").insertAfter(a.panels[g-1]||a.list);f.data("destroy.tabs",true)}a.panels=a.panels.add(f)}else c.disabled.push(g)});if(b){this.element.addClass("ui-tabs ui-widget ui-widget-content ui-corner-all");
this.list.addClass("ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all");this.lis.addClass("ui-state-default ui-corner-top");this.panels.addClass("ui-tabs-panel ui-widget-content ui-corner-bottom");if(c.selected===p){location.hash&&this.anchors.each(function(g,f){if(f.hash==location.hash){c.selected=g;return false}});if(typeof c.selected!=="number"&&c.cookie)c.selected=parseInt(a._cookie(),10);if(typeof c.selected!=="number"&&this.lis.filter(".ui-tabs-selected").length)c.selected=
this.lis.index(this.lis.filter(".ui-tabs-selected"));c.selected=c.selected||(this.lis.length?0:-1)}else if(c.selected===null)c.selected=-1;c.selected=c.selected>=0&&this.anchors[c.selected]||c.selected<0?c.selected:0;c.disabled=d.unique(c.disabled.concat(d.map(this.lis.filter(".ui-state-disabled"),function(g){return a.lis.index(g)}))).sort();d.inArray(c.selected,c.disabled)!=-1&&c.disabled.splice(d.inArray(c.selected,c.disabled),1);this.panels.addClass("ui-tabs-hide");this.lis.removeClass("ui-tabs-selected ui-state-active");
if(c.selected>=0&&this.anchors.length){a.element.find(a._sanitizeSelector(a.anchors[c.selected].hash)).removeClass("ui-tabs-hide");this.lis.eq(c.selected).addClass("ui-tabs-selected ui-state-active");a.element.queue("tabs",function(){a._trigger("show",null,a._ui(a.anchors[c.selected],a.element.find(a._sanitizeSelector(a.anchors[c.selected].hash))[0]))});this.load(c.selected)}d(window).bind("unload",function(){a.lis.add(a.anchors).unbind(".tabs");a.lis=a.anchors=a.panels=null})}else c.selected=this.lis.index(this.lis.filter(".ui-tabs-selected"));
this.element[c.collapsible?"addClass":"removeClass"]("ui-tabs-collapsible");c.cookie&&this._cookie(c.selected,c.cookie);b=0;for(var j;j=this.lis[b];b++)d(j)[d.inArray(b,c.disabled)!=-1&&!d(j).hasClass("ui-tabs-selected")?"addClass":"removeClass"]("ui-state-disabled");c.cache===false&&this.anchors.removeData("cache.tabs");this.lis.add(this.anchors).unbind(".tabs");if(c.event!=="mouseover"){var k=function(g,f){f.is(":not(.ui-state-disabled)")&&f.addClass("ui-state-"+g)},n=function(g,f){f.removeClass("ui-state-"+
g)};this.lis.bind("mouseover.tabs",function(){k("hover",d(this))});this.lis.bind("mouseout.tabs",function(){n("hover",d(this))});this.anchors.bind("focus.tabs",function(){k("focus",d(this).closest("li"))});this.anchors.bind("blur.tabs",function(){n("focus",d(this).closest("li"))})}var m,o;if(c.fx)if(d.isArray(c.fx)){m=c.fx[0];o=c.fx[1]}else m=o=c.fx;var r=o?function(g,f){d(g).closest("li").addClass("ui-tabs-selected ui-state-active");f.hide().removeClass("ui-tabs-hide").animate(o,o.duration||"normal",
function(){e(f,o);a._trigger("show",null,a._ui(g,f[0]))})}:function(g,f){d(g).closest("li").addClass("ui-tabs-selected ui-state-active");f.removeClass("ui-tabs-hide");a._trigger("show",null,a._ui(g,f[0]))},s=m?function(g,f){f.animate(m,m.duration||"normal",function(){a.lis.removeClass("ui-tabs-selected ui-state-active");f.addClass("ui-tabs-hide");e(f,m);a.element.dequeue("tabs")})}:function(g,f){a.lis.removeClass("ui-tabs-selected ui-state-active");f.addClass("ui-tabs-hide");a.element.dequeue("tabs")};
this.anchors.bind(c.event+".tabs",function(){var g=this,f=d(g).closest("li"),i=a.panels.filter(":not(.ui-tabs-hide)"),l=a.element.find(a._sanitizeSelector(g.hash));if(f.hasClass("ui-tabs-selected")&&!c.collapsible||f.hasClass("ui-state-disabled")||f.hasClass("ui-state-processing")||a.panels.filter(":animated").length||a._trigger("select",null,a._ui(this,l[0]))===false){this.blur();return false}c.selected=a.anchors.index(this);a.abort();if(c.collapsible)if(f.hasClass("ui-tabs-selected")){c.selected=
-1;c.cookie&&a._cookie(c.selected,c.cookie);a.element.queue("tabs",function(){s(g,i)}).dequeue("tabs");this.blur();return false}else if(!i.length){c.cookie&&a._cookie(c.selected,c.cookie);a.element.queue("tabs",function(){r(g,l)});a.load(a.anchors.index(this));this.blur();return false}c.cookie&&a._cookie(c.selected,c.cookie);if(l.length){i.length&&a.element.queue("tabs",function(){s(g,i)});a.element.queue("tabs",function(){r(g,l)});a.load(a.anchors.index(this))}else throw"jQuery UI Tabs: Mismatching fragment identifier.";
d.browser.msie&&this.blur()});this.anchors.bind("click.tabs",function(){return false})},_getIndex:function(b){if(typeof b=="string")b=this.anchors.index(this.anchors.filter("[href$="+b+"]"));return b},destroy:function(){var b=this.options;this.abort();this.element.unbind(".tabs").removeClass("ui-tabs ui-widget ui-widget-content ui-corner-all ui-tabs-collapsible").removeData("tabs");this.list.removeClass("ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all");this.anchors.each(function(){var e=
d.data(this,"href.tabs");if(e)this.href=e;var a=d(this).unbind(".tabs");d.each(["href","load","cache"],function(c,h){a.removeData(h+".tabs")})});this.lis.unbind(".tabs").add(this.panels).each(function(){d.data(this,"destroy.tabs")?d(this).remove():d(this).removeClass("ui-state-default ui-corner-top ui-tabs-selected ui-state-active ui-state-hover ui-state-focus ui-state-disabled ui-tabs-panel ui-widget-content ui-corner-bottom ui-tabs-hide")});b.cookie&&this._cookie(null,b.cookie);return this},add:function(b,
e,a){if(a===p)a=this.anchors.length;var c=this,h=this.options;e=d(h.tabTemplate.replace(/#\{href\}/g,b).replace(/#\{label\}/g,e));b=!b.indexOf("#")?b.replace("#",""):this._tabId(d("a",e)[0]);e.addClass("ui-state-default ui-corner-top").data("destroy.tabs",true);var j=c.element.find("#"+b);j.length||(j=d(h.panelTemplate).attr("id",b).data("destroy.tabs",true));j.addClass("ui-tabs-panel ui-widget-content ui-corner-bottom ui-tabs-hide");if(a>=this.lis.length){e.appendTo(this.list);j.appendTo(this.list[0].parentNode)}else{e.insertBefore(this.lis[a]);
j.insertBefore(this.panels[a])}h.disabled=d.map(h.disabled,function(k){return k>=a?++k:k});this._tabify();if(this.anchors.length==1){h.selected=0;e.addClass("ui-tabs-selected ui-state-active");j.removeClass("ui-tabs-hide");this.element.queue("tabs",function(){c._trigger("show",null,c._ui(c.anchors[0],c.panels[0]))});this.load(0)}this._trigger("add",null,this._ui(this.anchors[a],this.panels[a]));return this},remove:function(b){b=this._getIndex(b);var e=this.options,a=this.lis.eq(b).remove(),c=this.panels.eq(b).remove();
if(a.hasClass("ui-tabs-selected")&&this.anchors.length>1)this.select(b+(b+1<this.anchors.length?1:-1));e.disabled=d.map(d.grep(e.disabled,function(h){return h!=b}),function(h){return h>=b?--h:h});this._tabify();this._trigger("remove",null,this._ui(a.find("a")[0],c[0]));return this},enable:function(b){b=this._getIndex(b);var e=this.options;if(d.inArray(b,e.disabled)!=-1){this.lis.eq(b).removeClass("ui-state-disabled");e.disabled=d.grep(e.disabled,function(a){return a!=b});this._trigger("enable",null,
this._ui(this.anchors[b],this.panels[b]));return this}},disable:function(b){b=this._getIndex(b);var e=this.options;if(b!=e.selected){this.lis.eq(b).addClass("ui-state-disabled");e.disabled.push(b);e.disabled.sort();this._trigger("disable",null,this._ui(this.anchors[b],this.panels[b]))}return this},select:function(b){b=this._getIndex(b);if(b==-1)if(this.options.collapsible&&this.options.selected!=-1)b=this.options.selected;else return this;this.anchors.eq(b).trigger(this.options.event+".tabs");return this},
load:function(b){b=this._getIndex(b);var e=this,a=this.options,c=this.anchors.eq(b)[0],h=d.data(c,"load.tabs");this.abort();if(!h||this.element.queue("tabs").length!==0&&d.data(c,"cache.tabs"))this.element.dequeue("tabs");else{this.lis.eq(b).addClass("ui-state-processing");if(a.spinner){var j=d("span",c);j.data("label.tabs",j.html()).html(a.spinner)}this.xhr=d.ajax(d.extend({},a.ajaxOptions,{url:h,success:function(k,n){e.element.find(e._sanitizeSelector(c.hash)).html(k);e._cleanup();a.cache&&d.data(c,
"cache.tabs",true);e._trigger("load",null,e._ui(e.anchors[b],e.panels[b]));try{a.ajaxOptions.success(k,n)}catch(m){}},error:function(k,n){e._cleanup();e._trigger("load",null,e._ui(e.anchors[b],e.panels[b]));try{a.ajaxOptions.error(k,n,b,c)}catch(m){}}}));e.element.dequeue("tabs");return this}},abort:function(){this.element.queue([]);this.panels.stop(false,true);this.element.queue("tabs",this.element.queue("tabs").splice(-2,2));if(this.xhr){this.xhr.abort();delete this.xhr}this._cleanup();return this},
url:function(b,e){this.anchors.eq(b).removeData("cache.tabs").data("load.tabs",e);return this},length:function(){return this.anchors.length}});d.extend(d.ui.tabs,{version:"1.8.10"});d.extend(d.ui.tabs.prototype,{rotation:null,rotate:function(b,e){var a=this,c=this.options,h=a._rotate||(a._rotate=function(j){clearTimeout(a.rotation);a.rotation=setTimeout(function(){var k=c.selected;a.select(++k<a.anchors.length?k:0)},b);j&&j.stopPropagation()});e=a._unrotate||(a._unrotate=!e?function(j){j.clientX&&
a.rotate(null)}:function(){t=c.selected;h()});if(b){this.element.bind("tabsshow",h);this.anchors.bind(c.event+".tabs",e);h()}else{clearTimeout(a.rotation);this.element.unbind("tabsshow",h);this.anchors.unbind(c.event+".tabs",e);delete this._rotate;delete this._unrotate}return this}})})(jQuery);
;/*
 * jQuery UI Effects 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Effects/
 */
jQuery.effects||function(f,j){function n(c){var a;if(c&&c.constructor==Array&&c.length==3)return c;if(a=/rgb\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*\)/.exec(c))return[parseInt(a[1],10),parseInt(a[2],10),parseInt(a[3],10)];if(a=/rgb\(\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*\)/.exec(c))return[parseFloat(a[1])*2.55,parseFloat(a[2])*2.55,parseFloat(a[3])*2.55];if(a=/#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})/.exec(c))return[parseInt(a[1],
16),parseInt(a[2],16),parseInt(a[3],16)];if(a=/#([a-fA-F0-9])([a-fA-F0-9])([a-fA-F0-9])/.exec(c))return[parseInt(a[1]+a[1],16),parseInt(a[2]+a[2],16),parseInt(a[3]+a[3],16)];if(/rgba\(0, 0, 0, 0\)/.exec(c))return o.transparent;return o[f.trim(c).toLowerCase()]}function s(c,a){var b;do{b=f.curCSS(c,a);if(b!=""&&b!="transparent"||f.nodeName(c,"body"))break;a="backgroundColor"}while(c=c.parentNode);return n(b)}function p(){var c=document.defaultView?document.defaultView.getComputedStyle(this,null):this.currentStyle,
a={},b,d;if(c&&c.length&&c[0]&&c[c[0]])for(var e=c.length;e--;){b=c[e];if(typeof c[b]=="string"){d=b.replace(/\-(\w)/g,function(g,h){return h.toUpperCase()});a[d]=c[b]}}else for(b in c)if(typeof c[b]==="string")a[b]=c[b];return a}function q(c){var a,b;for(a in c){b=c[a];if(b==null||f.isFunction(b)||a in t||/scrollbar/.test(a)||!/color/i.test(a)&&isNaN(parseFloat(b)))delete c[a]}return c}function u(c,a){var b={_:0},d;for(d in a)if(c[d]!=a[d])b[d]=a[d];return b}function k(c,a,b,d){if(typeof c=="object"){d=
a;b=null;a=c;c=a.effect}if(f.isFunction(a)){d=a;b=null;a={}}if(typeof a=="number"||f.fx.speeds[a]){d=b;b=a;a={}}if(f.isFunction(b)){d=b;b=null}a=a||{};b=b||a.duration;b=f.fx.off?0:typeof b=="number"?b:b in f.fx.speeds?f.fx.speeds[b]:f.fx.speeds._default;d=d||a.complete;return[c,a,b,d]}function m(c){if(!c||typeof c==="number"||f.fx.speeds[c])return true;if(typeof c==="string"&&!f.effects[c])return true;return false}f.effects={};f.each(["backgroundColor","borderBottomColor","borderLeftColor","borderRightColor",
"borderTopColor","borderColor","color","outlineColor"],function(c,a){f.fx.step[a]=function(b){if(!b.colorInit){b.start=s(b.elem,a);b.end=n(b.end);b.colorInit=true}b.elem.style[a]="rgb("+Math.max(Math.min(parseInt(b.pos*(b.end[0]-b.start[0])+b.start[0],10),255),0)+","+Math.max(Math.min(parseInt(b.pos*(b.end[1]-b.start[1])+b.start[1],10),255),0)+","+Math.max(Math.min(parseInt(b.pos*(b.end[2]-b.start[2])+b.start[2],10),255),0)+")"}});var o={aqua:[0,255,255],azure:[240,255,255],beige:[245,245,220],black:[0,
0,0],blue:[0,0,255],brown:[165,42,42],cyan:[0,255,255],darkblue:[0,0,139],darkcyan:[0,139,139],darkgrey:[169,169,169],darkgreen:[0,100,0],darkkhaki:[189,183,107],darkmagenta:[139,0,139],darkolivegreen:[85,107,47],darkorange:[255,140,0],darkorchid:[153,50,204],darkred:[139,0,0],darksalmon:[233,150,122],darkviolet:[148,0,211],fuchsia:[255,0,255],gold:[255,215,0],green:[0,128,0],indigo:[75,0,130],khaki:[240,230,140],lightblue:[173,216,230],lightcyan:[224,255,255],lightgreen:[144,238,144],lightgrey:[211,
211,211],lightpink:[255,182,193],lightyellow:[255,255,224],lime:[0,255,0],magenta:[255,0,255],maroon:[128,0,0],navy:[0,0,128],olive:[128,128,0],orange:[255,165,0],pink:[255,192,203],purple:[128,0,128],violet:[128,0,128],red:[255,0,0],silver:[192,192,192],white:[255,255,255],yellow:[255,255,0],transparent:[255,255,255]},r=["add","remove","toggle"],t={border:1,borderBottom:1,borderColor:1,borderLeft:1,borderRight:1,borderTop:1,borderWidth:1,margin:1,padding:1};f.effects.animateClass=function(c,a,b,
d){if(f.isFunction(b)){d=b;b=null}return this.queue("fx",function(){var e=f(this),g=e.attr("style")||" ",h=q(p.call(this)),l,v=e.attr("className");f.each(r,function(w,i){c[i]&&e[i+"Class"](c[i])});l=q(p.call(this));e.attr("className",v);e.animate(u(h,l),a,b,function(){f.each(r,function(w,i){c[i]&&e[i+"Class"](c[i])});if(typeof e.attr("style")=="object"){e.attr("style").cssText="";e.attr("style").cssText=g}else e.attr("style",g);d&&d.apply(this,arguments)});h=f.queue(this);l=h.splice(h.length-1,1)[0];
h.splice(1,0,l);f.dequeue(this)})};f.fn.extend({_addClass:f.fn.addClass,addClass:function(c,a,b,d){return a?f.effects.animateClass.apply(this,[{add:c},a,b,d]):this._addClass(c)},_removeClass:f.fn.removeClass,removeClass:function(c,a,b,d){return a?f.effects.animateClass.apply(this,[{remove:c},a,b,d]):this._removeClass(c)},_toggleClass:f.fn.toggleClass,toggleClass:function(c,a,b,d,e){return typeof a=="boolean"||a===j?b?f.effects.animateClass.apply(this,[a?{add:c}:{remove:c},b,d,e]):this._toggleClass(c,
a):f.effects.animateClass.apply(this,[{toggle:c},a,b,d])},switchClass:function(c,a,b,d,e){return f.effects.animateClass.apply(this,[{add:a,remove:c},b,d,e])}});f.extend(f.effects,{version:"1.8.10",save:function(c,a){for(var b=0;b<a.length;b++)a[b]!==null&&c.data("ec.storage."+a[b],c[0].style[a[b]])},restore:function(c,a){for(var b=0;b<a.length;b++)a[b]!==null&&c.css(a[b],c.data("ec.storage."+a[b]))},setMode:function(c,a){if(a=="toggle")a=c.is(":hidden")?"show":"hide";return a},getBaseline:function(c,
a){var b;switch(c[0]){case "top":b=0;break;case "middle":b=0.5;break;case "bottom":b=1;break;default:b=c[0]/a.height}switch(c[1]){case "left":c=0;break;case "center":c=0.5;break;case "right":c=1;break;default:c=c[1]/a.width}return{x:c,y:b}},createWrapper:function(c){if(c.parent().is(".ui-effects-wrapper"))return c.parent();var a={width:c.outerWidth(true),height:c.outerHeight(true),"float":c.css("float")},b=f("<div></div>").addClass("ui-effects-wrapper").css({fontSize:"100%",background:"transparent",
border:"none",margin:0,padding:0});c.wrap(b);b=c.parent();if(c.css("position")=="static"){b.css({position:"relative"});c.css({position:"relative"})}else{f.extend(a,{position:c.css("position"),zIndex:c.css("z-index")});f.each(["top","left","bottom","right"],function(d,e){a[e]=c.css(e);if(isNaN(parseInt(a[e],10)))a[e]="auto"});c.css({position:"relative",top:0,left:0,right:"auto",bottom:"auto"})}return b.css(a).show()},removeWrapper:function(c){if(c.parent().is(".ui-effects-wrapper"))return c.parent().replaceWith(c);
return c},setTransition:function(c,a,b,d){d=d||{};f.each(a,function(e,g){unit=c.cssUnit(g);if(unit[0]>0)d[g]=unit[0]*b+unit[1]});return d}});f.fn.extend({effect:function(c){var a=k.apply(this,arguments),b={options:a[1],duration:a[2],callback:a[3]};a=b.options.mode;var d=f.effects[c];if(f.fx.off||!d)return a?this[a](b.duration,b.callback):this.each(function(){b.callback&&b.callback.call(this)});return d.call(this,b)},_show:f.fn.show,show:function(c){if(m(c))return this._show.apply(this,arguments);
else{var a=k.apply(this,arguments);a[1].mode="show";return this.effect.apply(this,a)}},_hide:f.fn.hide,hide:function(c){if(m(c))return this._hide.apply(this,arguments);else{var a=k.apply(this,arguments);a[1].mode="hide";return this.effect.apply(this,a)}},__toggle:f.fn.toggle,toggle:function(c){if(m(c)||typeof c==="boolean"||f.isFunction(c))return this.__toggle.apply(this,arguments);else{var a=k.apply(this,arguments);a[1].mode="toggle";return this.effect.apply(this,a)}},cssUnit:function(c){var a=this.css(c),
b=[];f.each(["em","px","%","pt"],function(d,e){if(a.indexOf(e)>0)b=[parseFloat(a),e]});return b}});f.easing.jswing=f.easing.swing;f.extend(f.easing,{def:"easeOutQuad",swing:function(c,a,b,d,e){return f.easing[f.easing.def](c,a,b,d,e)},easeInQuad:function(c,a,b,d,e){return d*(a/=e)*a+b},easeOutQuad:function(c,a,b,d,e){return-d*(a/=e)*(a-2)+b},easeInOutQuad:function(c,a,b,d,e){if((a/=e/2)<1)return d/2*a*a+b;return-d/2*(--a*(a-2)-1)+b},easeInCubic:function(c,a,b,d,e){return d*(a/=e)*a*a+b},easeOutCubic:function(c,
a,b,d,e){return d*((a=a/e-1)*a*a+1)+b},easeInOutCubic:function(c,a,b,d,e){if((a/=e/2)<1)return d/2*a*a*a+b;return d/2*((a-=2)*a*a+2)+b},easeInQuart:function(c,a,b,d,e){return d*(a/=e)*a*a*a+b},easeOutQuart:function(c,a,b,d,e){return-d*((a=a/e-1)*a*a*a-1)+b},easeInOutQuart:function(c,a,b,d,e){if((a/=e/2)<1)return d/2*a*a*a*a+b;return-d/2*((a-=2)*a*a*a-2)+b},easeInQuint:function(c,a,b,d,e){return d*(a/=e)*a*a*a*a+b},easeOutQuint:function(c,a,b,d,e){return d*((a=a/e-1)*a*a*a*a+1)+b},easeInOutQuint:function(c,
a,b,d,e){if((a/=e/2)<1)return d/2*a*a*a*a*a+b;return d/2*((a-=2)*a*a*a*a+2)+b},easeInSine:function(c,a,b,d,e){return-d*Math.cos(a/e*(Math.PI/2))+d+b},easeOutSine:function(c,a,b,d,e){return d*Math.sin(a/e*(Math.PI/2))+b},easeInOutSine:function(c,a,b,d,e){return-d/2*(Math.cos(Math.PI*a/e)-1)+b},easeInExpo:function(c,a,b,d,e){return a==0?b:d*Math.pow(2,10*(a/e-1))+b},easeOutExpo:function(c,a,b,d,e){return a==e?b+d:d*(-Math.pow(2,-10*a/e)+1)+b},easeInOutExpo:function(c,a,b,d,e){if(a==0)return b;if(a==
e)return b+d;if((a/=e/2)<1)return d/2*Math.pow(2,10*(a-1))+b;return d/2*(-Math.pow(2,-10*--a)+2)+b},easeInCirc:function(c,a,b,d,e){return-d*(Math.sqrt(1-(a/=e)*a)-1)+b},easeOutCirc:function(c,a,b,d,e){return d*Math.sqrt(1-(a=a/e-1)*a)+b},easeInOutCirc:function(c,a,b,d,e){if((a/=e/2)<1)return-d/2*(Math.sqrt(1-a*a)-1)+b;return d/2*(Math.sqrt(1-(a-=2)*a)+1)+b},easeInElastic:function(c,a,b,d,e){c=1.70158;var g=0,h=d;if(a==0)return b;if((a/=e)==1)return b+d;g||(g=e*0.3);if(h<Math.abs(d)){h=d;c=g/4}else c=
g/(2*Math.PI)*Math.asin(d/h);return-(h*Math.pow(2,10*(a-=1))*Math.sin((a*e-c)*2*Math.PI/g))+b},easeOutElastic:function(c,a,b,d,e){c=1.70158;var g=0,h=d;if(a==0)return b;if((a/=e)==1)return b+d;g||(g=e*0.3);if(h<Math.abs(d)){h=d;c=g/4}else c=g/(2*Math.PI)*Math.asin(d/h);return h*Math.pow(2,-10*a)*Math.sin((a*e-c)*2*Math.PI/g)+d+b},easeInOutElastic:function(c,a,b,d,e){c=1.70158;var g=0,h=d;if(a==0)return b;if((a/=e/2)==2)return b+d;g||(g=e*0.3*1.5);if(h<Math.abs(d)){h=d;c=g/4}else c=g/(2*Math.PI)*Math.asin(d/
h);if(a<1)return-0.5*h*Math.pow(2,10*(a-=1))*Math.sin((a*e-c)*2*Math.PI/g)+b;return h*Math.pow(2,-10*(a-=1))*Math.sin((a*e-c)*2*Math.PI/g)*0.5+d+b},easeInBack:function(c,a,b,d,e,g){if(g==j)g=1.70158;return d*(a/=e)*a*((g+1)*a-g)+b},easeOutBack:function(c,a,b,d,e,g){if(g==j)g=1.70158;return d*((a=a/e-1)*a*((g+1)*a+g)+1)+b},easeInOutBack:function(c,a,b,d,e,g){if(g==j)g=1.70158;if((a/=e/2)<1)return d/2*a*a*(((g*=1.525)+1)*a-g)+b;return d/2*((a-=2)*a*(((g*=1.525)+1)*a+g)+2)+b},easeInBounce:function(c,
a,b,d,e){return d-f.easing.easeOutBounce(c,e-a,0,d,e)+b},easeOutBounce:function(c,a,b,d,e){return(a/=e)<1/2.75?d*7.5625*a*a+b:a<2/2.75?d*(7.5625*(a-=1.5/2.75)*a+0.75)+b:a<2.5/2.75?d*(7.5625*(a-=2.25/2.75)*a+0.9375)+b:d*(7.5625*(a-=2.625/2.75)*a+0.984375)+b},easeInOutBounce:function(c,a,b,d,e){if(a<e/2)return f.easing.easeInBounce(c,a*2,0,d,e)*0.5+b;return f.easing.easeOutBounce(c,a*2-e,0,d,e)*0.5+d*0.5+b}})}(jQuery);
;/*
 * jQuery UI Effects Fade 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Effects/Fade
 *
 * Depends:
 *	jquery.effects.core.js
 */
(function(b){b.effects.fade=function(a){return this.queue(function(){var c=b(this),d=b.effects.setMode(c,a.options.mode||"hide");c.animate({opacity:d},{queue:false,duration:a.duration,easing:a.options.easing,complete:function(){a.callback&&a.callback.apply(this,arguments);c.dequeue()}})})}})(jQuery);
;/*
 * jQuery UI Effects Highlight 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Effects/Highlight
 *
 * Depends:
 *	jquery.effects.core.js
 */
(function(b){b.effects.highlight=function(c){return this.queue(function(){var a=b(this),e=["backgroundImage","backgroundColor","opacity"],d=b.effects.setMode(a,c.options.mode||"show"),f={backgroundColor:a.css("backgroundColor")};if(d=="hide")f.opacity=0;b.effects.save(a,e);a.show().css({backgroundImage:"none",backgroundColor:c.options.color||"#ffff99"}).animate(f,{queue:false,duration:c.duration,easing:c.options.easing,complete:function(){d=="hide"&&a.hide();b.effects.restore(a,e);d=="show"&&!b.support.opacity&&
this.style.removeAttribute("filter");c.callback&&c.callback.apply(this,arguments);a.dequeue()}})})}})(jQuery);
;/*
 * jQuery UI Effects Slide 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Effects/Slide
 *
 * Depends:
 *	jquery.effects.core.js
 */
(function(c){c.effects.slide=function(d){return this.queue(function(){var a=c(this),h=["position","top","bottom","left","right"],f=c.effects.setMode(a,d.options.mode||"show"),b=d.options.direction||"left";c.effects.save(a,h);a.show();c.effects.createWrapper(a).css({overflow:"hidden"});var g=b=="up"||b=="down"?"top":"left";b=b=="up"||b=="left"?"pos":"neg";var e=d.options.distance||(g=="top"?a.outerHeight({margin:true}):a.outerWidth({margin:true}));if(f=="show")a.css(g,b=="pos"?isNaN(e)?"-"+e:-e:e);
var i={};i[g]=(f=="show"?b=="pos"?"+=":"-=":b=="pos"?"-=":"+=")+e;a.animate(i,{queue:false,duration:d.duration,easing:d.options.easing,complete:function(){f=="hide"&&a.hide();c.effects.restore(a,h);c.effects.removeWrapper(a);d.callback&&d.callback.apply(this,arguments);a.dequeue()}})})}})(jQuery);
;/*
 * jQuery UI Effects Transfer 1.8.10
 *
 * Copyright 2011, AUTHORS.txt (http://jqueryui.com/about)
 * Dual licensed under the MIT or GPL Version 2 licenses.
 * http://jquery.org/license
 *
 * http://docs.jquery.com/UI/Effects/Transfer
 *
 * Depends:
 *	jquery.effects.core.js
 */
(function(e){e.effects.transfer=function(a){return this.queue(function(){var b=e(this),c=e(a.options.to),d=c.offset();c={top:d.top,left:d.left,height:c.innerHeight(),width:c.innerWidth()};d=b.offset();var f=e('<div class="ui-effects-transfer"></div>').appendTo(document.body).addClass(a.options.className).css({top:d.top,left:d.left,height:b.innerHeight(),width:b.innerWidth(),position:"absolute"}).animate(c,a.duration,a.options.easing,function(){f.remove();a.callback&&a.callback.apply(b[0],arguments);
b.dequeue()})})}})(jQuery);
;/*
  This file is part of Cytoscape Web.
  Copyright (c) 2009, The Cytoscape Consortium (www.cytoscape.org)

  The Cytoscape Consortium is:
    - Agilent Technologies
    - Institut Pasteur
    - Institute for Systems Biology
    - Memorial Sloan-Kettering Cancer Center
    - National Center for Integrative Biomedical Informatics
    - Unilever
    - University of California San Diego
    - University of California San Francisco
    - University of Toronto

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

// ===[ namespaces ]================================================================================

// Create namespaces if not already defined:
(function () {
    "use strict";
    
    if (typeof(window['org']) === 'undefined') {
    	window['org'] = {};
    }
    if (typeof(window.org['cytoscapeweb']) === 'undefined') {
    	window.org['cytoscapeweb'] = {};
    }
    if (typeof(window.org.cytoscapeweb['demo']) === 'undefined') {
    	window.org.cytoscapeweb['demo'] = {};
    }

    // ===[ Exporter ]==============================================================================

    // Create a global map to store all instances:
    window._cytoscapeWebExporterInstances = { index: 0 };
    	
    org.cytoscapeweb.demo.Exporter = function (containerId, options) {
        this.containerId = containerId;

        if (!options) { options = {}; }
        this.options = options;

        // Part of the embed or object tag id:
        this.idToken = options.idToken ? options.idToken : "exporter";
        // The .swf path, including its name, but without the file extension:
        this.swfPath = options.swfPath ? options.swfPath : "Exporter";
        // The path of the .swf file that updates the Flash player version:
        this.flashInstallerPath = options.flashInstallerPath ? options.flashInstallerPath : "playerProductInstall";
        // Alternate content to be displayed in case user does not have Flash installed:
        this.flashAlternateContent = options.flashAlternateContent ? options.flashAlternateContent : 'This content requires the Adobe Flash Player. ' +
                                                                                                     '<a href=http://www.adobe.com/go/getflash/>Get Flash</a>';
        _cytoscapeWebExporterInstances.index++;

        this.id = this.idToken + _cytoscapeWebExporterInstances.index;
        _cytoscapeWebExporterInstances[this.id] = this;
        _embedSWF(this.containerId, this.id, this.swfPath, this.flashInstallerPath);
    };

    org.cytoscapeweb.demo.Exporter.prototype = {
    	
    	swf: function () {
    		return _swf(this.id);
    	},
    		
        _onReady: function () {
    		if (this.swf().fileName) {
    			this.swf().fileName(this.options.fileName());
    		}
    		if (this.swf().base64) {
    			this.swf().base64(this.options.base64);
    		}
            this.options.ready();
        },
        
        _onClick: function () {
        	return this.options.data();
        }
    };
    
    // ===[ Importer ]==============================================================================
    
    // Create a global map to store all instances:
    window._cytoscapeWebImporterInstances = { index: 0 };
    	
    org.cytoscapeweb.demo.Importer = function (containerId, options) {
        this.containerId = containerId;

        if (!options) { options = {}; }
        this.options = options;

        // Part of the embed or object tag id:
        this.idToken = options.idToken ? options.idToken : "importer";
        // The .swf path, including its name, but without the file extension:
        this.swfPath = options.swfPath ? options.swfPath : "Importer";
        // The path of the .swf file that updates the Flash player version:
        this.flashInstallerPath = options.flashInstallerPath ? options.flashInstallerPath : "playerProductInstall";
        // Alternate content to be displayed in case user does not have Flash installed:
        this.flashAlternateContent = options.flashAlternateContent ? options.flashAlternateContent : 'This content requires the Adobe Flash Player. ' +
                                                                                                     '<a href=http://www.adobe.com/go/getflash/>Get Flash</a>';
        _cytoscapeWebImporterInstances.index++;

        this.id = this.idToken + _cytoscapeWebImporterInstances.index;
        _cytoscapeWebImporterInstances[this.id] = this;
        _embedSWF(this.containerId, this.id, this.swfPath, this.flashInstallerPath);
    };
    
    org.cytoscapeweb.demo.Importer.prototype = {
    	swf: function () {
			return _swf(this.id);
		},
    		
    	_onReady: function () {
			if (this.swf().typeFilter) {
				this.swf().typeFilter(this.options.typeFilter());
			}
			if (this.swf().typeDescription) {
				this.swf().typeDescription(this.options.typeDescription());
			}
            this.options.ready();
        },
        
		_onStart: function(metadata){
        	// when data starts opening
        	if (this.options.binary) {
        		this.swf().binary(this.options.binary(metadata));
        	}
        	if (this.options.start) {
        		this.options.start(metadata);
        	}
		},
		
		_onComplete: function(data){
			// when data finishes loading
			this.options.data(data);
		},
		
		_onCancel: function(){
			// when the user closes the dialog
			if (this.options.cancel) {
				this.options.cancel();
			}
		},
		
		_onError: function(msg){
			// when the file can not be read
			if (this.options.error) {
				this.options.error(msg);
			}
		}
    };
    
    // ===[ Functions ]=============================================================================
    
    function _swf (id) {
        if (navigator.appName.indexOf("Microsoft") !== -1) {
            return window[id];
        } else {
            return document[id];
        }
    }

    function _embedSWF (containerId, id, swfPath, flashInstallerPath) {
        //Major version of Flash required
        var requiredMajorVersion = 10;
        //Minor version of Flash required
        var requiredMinorVersion = 0;
        //Minor version of Flash required
        var requiredRevision = 0;

        // Let's redefine the default AC_OETags function, because we don't necessarily want
        // to replace the whole HTML page with the swf object:
        AC_Generateobj = function (objAttrs, params, embedAttrs) {
            var str = '';
            var i;
            if (isIE && isWin && !isOpera) {
                str += '<object ';
                for (i in objAttrs) {
                    if (Object.hasOwnProperty.call(objAttrs, i)) {
                        str += i + '="' + objAttrs[i] + '" ';
                    }
                }
                str += '>';
                for (i in params) {
                    if (Object.hasOwnProperty.call(params, i)) {
                        str += '<param name="' + i + '" value="' + params[i] + '" /> ';
                    }
                }
                str += '</object>';
            } else {
                str += '<embed ';
                for (i in embedAttrs) {
                    if (Object.hasOwnProperty.call(embedAttrs, i)) {
                        str += i + '="' + embedAttrs[i] + '" ';
                    }
                }
                str += '> </embed>';
            }
            // Replace only the indicated DOM element:
            document.getElementById(containerId).innerHTML = str;
        };

        // Version check for the Flash Player that has the ability to start Player Product Install (6.0r65)
        var hasProductInstall = DetectFlashVer(6, 0, 65);

        // Version check based upon the values defined in globals
        var hasRequestedVersion = DetectFlashVer(requiredMajorVersion, requiredMinorVersion, requiredRevision);

        if (hasProductInstall && !hasRequestedVersion) {
            // DO NOT MODIFY THE FOLLOWING FOUR LINES
            // Location visited after installation is complete if installation is required
            var MMPlayerType = (isIE === true) ? "ActiveX" : "PlugIn";
            var MMredirectURL = window.location;
            document.title = document.title.slice(0, 47) + " - Flash Player Installation";
            var MMdoctitle = document.title;

            AC_FL_RunContent(
                "src", flashInstallerPath,
                "FlashVars", "MMredirectURL="+MMredirectURL+'&MMplayerType='+MMPlayerType+'&MMdoctitle='+MMdoctitle+"",
                "width", "100%",
                "height", "100%",
                "align", "middle",
                "id", id,
                "quality", "high",
                "bgcolor", "#ffffff",
                "name", id,
                "allowScriptAccess","sameDomain",
                "type", "application/x-shockwave-flash",
                "pluginspage", "http://www.adobe.com/go/getflashplayer"
            );
        } else if (hasRequestedVersion) {
            // if we've detected an acceptable version
            // embed the Flash Content SWF when all tests are passed
            AC_FL_RunContent(
                    "src", swfPath,
                    "class", "swf",
                    "width", "100%",
                    "height", "100%",
                    "align", "middle",
                    "id", id,
                    "quality", "high",
                    "name", id,
                    "allowScriptAccess", "always",
                    "type", "application/x-shockwave-flash",
                    "pluginspage", "http://www.adobe.com/go/getflashplayer",
                    "wmode", "transparent",
                    "flashVars", "id="+id
            );
        } else { // flash is too old or we can't detect the plugin
            // Insert non-flash content:
            document.getElementById(containerId).innerHTML = 'This content requires the Adobe Flash Player. '
               	+ '<a href=http://www.adobe.com/go/getflash/>Get Flash</a>';
        }
        return this;
    }
})();
//http://www.featureblend.com/license.txt
var FlashDetect=new function(){var self=this;self.installed=false;self.raw="";self.major=-1;self.minor=-1;self.revision=-1;self.revisionStr="";var activeXDetectRules=[{"name":"ShockwaveFlash.ShockwaveFlash.7","version":function(obj){return getActiveXVersion(obj);}},{"name":"ShockwaveFlash.ShockwaveFlash.6","version":function(obj){var version="6,0,21";try{obj.AllowScriptAccess="always";version=getActiveXVersion(obj);}catch(err){}
return version;}},{"name":"ShockwaveFlash.ShockwaveFlash","version":function(obj){return getActiveXVersion(obj);}}];var getActiveXVersion=function(activeXObj){var version=-1;try{version=activeXObj.GetVariable("$version");}catch(err){}
return version;};var getActiveXObject=function(name){var obj=-1;try{obj=new ActiveXObject(name);}catch(err){obj={activeXError:true};}
return obj;};var parseActiveXVersion=function(str){var versionArray=str.split(",");return{"raw":str,"major":parseInt(versionArray[0].split(" ")[1],10),"minor":parseInt(versionArray[1],10),"revision":parseInt(versionArray[2],10),"revisionStr":versionArray[2]};};var parseStandardVersion=function(str){var descParts=str.split(/ +/);var majorMinor=descParts[2].split(/\./);var revisionStr=descParts[3];return{"raw":str,"major":parseInt(majorMinor[0],10),"minor":parseInt(majorMinor[1],10),"revisionStr":revisionStr,"revision":parseRevisionStrToInt(revisionStr)};};var parseRevisionStrToInt=function(str){return parseInt(str.replace(/[a-zA-Z]/g,""),10)||self.revision;};self.majorAtLeast=function(version){return self.major>=version;};self.minorAtLeast=function(version){return self.minor>=version;};self.revisionAtLeast=function(version){return self.revision>=version;};self.versionAtLeast=function(major){var properties=[self.major,self.minor,self.revision];var len=Math.min(properties.length,arguments.length);for(i=0;i<len;i++){if(properties[i]>=arguments[i]){if(i+1<len&&properties[i]==arguments[i]){continue;}else{return true;}}else{return false;}}};self.FlashDetect=function(){if(navigator.plugins&&navigator.plugins.length>0){var type='application/x-shockwave-flash';var mimeTypes=navigator.mimeTypes;if(mimeTypes&&mimeTypes[type]&&mimeTypes[type].enabledPlugin&&mimeTypes[type].enabledPlugin.description){var version=mimeTypes[type].enabledPlugin.description;var versionObj=parseStandardVersion(version);self.raw=versionObj.raw;self.major=versionObj.major;self.minor=versionObj.minor;self.revisionStr=versionObj.revisionStr;self.revision=versionObj.revision;self.installed=true;}}else if(navigator.appVersion.indexOf("Mac")==-1&&window.execScript){var version=-1;for(var i=0;i<activeXDetectRules.length&&version==-1;i++){var obj=getActiveXObject(activeXDetectRules[i].name);if(!obj.activeXError){self.installed=true;version=activeXDetectRules[i].version(obj);if(version!=-1){var versionObj=parseActiveXVersion(version);self.raw=versionObj.raw;self.major=versionObj.major;self.minor=versionObj.minor;self.revision=versionObj.revision;self.revisionStr=versionObj.revisionStr;}}}}}();};FlashDetect.JS_RELEASE="1.0.4";/*
jQuery.ThreeDots.min

Author Jeremy Horn
Version 1.0.10
Date: 1/25/2009
More: http://tpgblog.com/ThreeDots/
compiled by http://yui.2clics.net/
*/

(function(e){e.fn.ThreeDots=function(h){var g=this;if((typeof h=="object")||(h==undefined)){e.fn.ThreeDots.the_selected=this;var g=e.fn.ThreeDots.update(h)}return g};e.fn.ThreeDots.update=function(u){var k,t=null;var m,j,s,q,o;var l,i;var r,h,n;if((typeof u=="object")||(u==undefined)){e.fn.ThreeDots.c_settings=e.extend({},e.fn.ThreeDots.settings,u);var p=e.fn.ThreeDots.c_settings.max_rows;if(p<1){return e.fn.ThreeDots.the_selected}var g=false;jQuery.each(e.fn.ThreeDots.c_settings.valid_delimiters,function(v,w){if(((new String(w)).length==1)){g=true}});if(g==false){return e.fn.ThreeDots.the_selected}e.fn.ThreeDots.the_selected.each(function(){k=e(this);if(e(k).children("."+e.fn.ThreeDots.c_settings.text_span_class).length==0){return true}l=e(k).children("."+e.fn.ThreeDots.c_settings.text_span_class).get(0);var y=a(k,true);var x=e(l).text();d(k,l,y);var v=e(l).text();if((h=e(k).attr("threedots"))!=undefined){e(l).text(h);e(k).children("."+e.fn.ThreeDots.c_settings.e_span_class).remove()}r=e(l).text();if(r.length<=0){r=""}e(k).attr("threedots",x);if(a(k,y)>p){curr_ellipsis=e(k).append('<span style="white-space:nowrap" class="'+e.fn.ThreeDots.c_settings.e_span_class+'">'+e.fn.ThreeDots.c_settings.ellipsis_string+"</span>");while(a(k,y)>p){i=b(e(l).text());e(l).text(i.updated_string);t=i.word;n=i.del;if(n==null){break}}if(t!=null){var w=c(k,y);if((a(k,y)<=p-1)||(w)||(!e.fn.ThreeDots.c_settings.whole_word)){r=e(l).text();if(i.del!=null){e(l).text(r+n)}if(a(k,y)>p){e(l).text(r)}else{e(l).text(e(l).text()+t);if((a(k,y)>p+1)||(!e.fn.ThreeDots.c_settings.whole_word)||(v==t)||w){while((a(k,y)>p)){if(e(l).text().length>0){e(l).text(e(l).text().substr(0,e(l).text().length-1))}else{break}}}}}}}if(x==e(e(k).children("."+e.fn.ThreeDots.c_settings.text_span_class).get(0)).text()){e(k).children("."+e.fn.ThreeDots.c_settings.e_span_class).remove()}else{if((e(k).children("."+e.fn.ThreeDots.c_settings.e_span_class)).length>0){if(e.fn.ThreeDots.c_settings.alt_text_t){e(k).children("."+e.fn.ThreeDots.c_settings.text_span_class).attr("title",x)}if(e.fn.ThreeDots.c_settings.alt_text_e){e(k).children("."+e.fn.ThreeDots.c_settings.e_span_class).attr("title",x)}}}})}return e.fn.ThreeDots.the_selected};e.fn.ThreeDots.settings={valid_delimiters:[" ",",","."],ellipsis_string:"...",max_rows:2,text_span_class:"ellipsis_text",e_span_class:"threedots_ellipsis",whole_word:true,allow_dangle:false,alt_text_e:false,alt_text_t:false};function c(k,h){if(e.fn.ThreeDots.c_settings.allow_dangle==true){return false}var l=e(k).children("."+e.fn.ThreeDots.c_settings.e_span_class).get(0);var g=e(l).css("display");var i=a(k,h);e(l).css("display","none");var j=a(k,h);e(l).css("display",g);if(i>j){return true}else{return false}}function a(i,j){var g=typeof j;if((g=="object")||(g==undefined)){return e(i).height()/j.lh}else{if(g=="boolean"){var h=f(e(i));return{lh:h}}}}function b(k){var j;var i=e.fn.ThreeDots.c_settings.valid_delimiters;k=jQuery.trim(k);var g=-1;var h=null;var l=null;jQuery.each(i,function(m,o){if(((new String(o)).length!=1)||(o==null)){return false}var n=k.lastIndexOf(o);if(n!=-1){if(n>g){g=n;h=k.substring(g+1);l=o}}});if(g>0){return{updated_string:jQuery.trim(k.substring(0,g)),word:h,del:l}}else{return{updated_string:"",word:jQuery.trim(k),del:null}}}function f(h){e(h).append("<div id='temp_ellipsis_div' style='position:absolute; visibility:hidden'>H</div>");var g=e("#temp_ellipsis_div").height();e("#temp_ellipsis_div").remove();return g}function d(k,l,m){var q=e(l).text();var i=q;var o=e.fn.ThreeDots.c_settings.max_rows;var h,g,n,r,j;var p;if(a(k,m)<=o){return}else{p=0;curr_length=i.length;curr_middle=Math.floor((curr_length-p)/2);h=q.substring(p,p+curr_middle);g=q.substring(p+curr_middle);while(curr_middle!=0){e(l).text(h);if(a(k,m)<=(o)){j=Math.floor(g.length/2);n=g.substring(0,j);p=h.length;i=h+n;curr_length=i.length;e(l).text(i)}else{i=h;curr_length=i.length}curr_middle=Math.floor((curr_length-p)/2);h=q.substring(0,p+curr_middle);g=q.substring(p+curr_middle)}}}})(jQuery);/*
 * Autocomplete - jQuery plugin 1.0.2
 *
 * Copyright (c) 2009 Dylan Verheul, Dan G. Switzer, Anjesh Tuladhar, Jörn Zaefferer, Max Franz
 *
 * Dual licensed under the MIT and GPL licenses:
 *   http://www.opensource.org/licenses/mit-license.php
 *   http://www.gnu.org/licenses/gpl.html
 *
 * Revision: $Id: jquery.autocomplete.js 5747 2008-06-25 18:30:55Z joern.zaefferer $
 *
 */

;(function($) {
	
$.fn.extend({
	autocomplete: function(urlOrData, options) {
		var isUrl = typeof urlOrData == "string";
		options = $.extend({}, $.Autocompleter.defaults, {
			url: isUrl ? urlOrData : null,
			data: isUrl ? null : urlOrData,
			delay: isUrl ? $.Autocompleter.defaults.delay : 10,
			max: options && !options.scroll ? 10 : 150
		}, options);
		
		// if highlight is set to false, replace it with a do-nothing function
		options.highlight = options.highlight || function(value) { return value; };
		
		// if the formatMatch option is not specified, then use formatItem for backwards compatibility
		options.formatMatch = options.formatMatch || options.formatItem;
		
		return this.each(function() {
			new $.Autocompleter(this, options);
		});
	},
	result: function(handler) {
		return this.bind("result", handler);
	},
	search: function(handler) {
		return this.trigger("search", [handler]);
	},
	flushCache: function() {
		return this.trigger("flushCache");
	},
	setOptions: function(options){
		return this.trigger("setOptions", [options]);
	},
	unautocomplete: function() {
		return this.trigger("unautocomplete");
	}
});

$.Autocompleter = function(input, options) {

	var KEY = {
		UP: 38,
		DOWN: 40,
		DEL: 46,
		TAB: 9,
		RETURN: 13,
		ESC: 27,
		COMMA: 188,
		PAGEUP: 33,
		PAGEDOWN: 34,
		BACKSPACE: 8
	};

	// Create $ object for input element
	var $input = $(input).attr("autocomplete", "off").addClass(options.inputClass);

	var timeout;
	var previousValue = "";
	var cache = $.Autocompleter.Cache(options);
	var hasFocus = 0;
	var lastKeyPressCode;
	var config = {
		mouseDownOnSelect: false
	};
	var select = $.Autocompleter.Select(options, input, selectCurrent, config);
	
	var blockSubmit;
	
	// prevent form submit in opera when selecting with return key
	$.browser.opera && $(input.form).bind("submit.autocomplete", function() {
		if (blockSubmit) {
			blockSubmit = false;
			return false;
		}
	});
	
	// only opera doesn't trigger keydown multiple times while pressed, others don't work with keypress at all
	$input.bind(($.browser.opera ? "keypress" : "keydown") + ".autocomplete", function(event) {
		// track last key pressed
		lastKeyPressCode = event.keyCode;
		switch(event.keyCode) {
		
			case KEY.UP:
				event.preventDefault();
				if ( select.visible() ) {
					select.prev();
				} else {
					onChange(0, true);
				}
				break;
				
			case KEY.DOWN:
				event.preventDefault();
				if ( select.visible() ) {
					select.next();
				} else {
					onChange(0, true);
				}
				break;
				
			case KEY.PAGEUP:
				event.preventDefault();
				if ( select.visible() ) {
					select.pageUp();
				} else {
					onChange(0, true);
				}
				break;
				
			case KEY.PAGEDOWN:
				event.preventDefault();
				if ( select.visible() ) {
					select.pageDown();
				} else {
					onChange(0, true);
				}
				break;
			
			// matches also semicolon
			case options.multiple && $.trim(options.multipleSeparator) == "," && KEY.COMMA:
			case KEY.TAB:
			case KEY.RETURN:
				if( selectCurrent() ) {
					// stop default to prevent a form submit, Opera needs special handling
					event.preventDefault();
					blockSubmit = true;
					return false;
				}
				break;
				
			case KEY.ESC:
				select.hide();
				break;
				
			default:
				clearTimeout(timeout);
				timeout = setTimeout(onChange, options.delay);
				break;
		}
	}).focus(function(){
		// track whether the field has focus, we shouldn't process any
		// results if the field no longer has focus
		hasFocus++;
	}).blur(function() {
		hasFocus = 0;
		if (!config.mouseDownOnSelect) {
			hideResults();
		}
	}).click(function() {
		// show select when clicking in a focused field
		if ( hasFocus++ > 1 && !select.visible() ) {
			onChange(0, true);
		}
	}).bind("search", function() {
		// TODO why not just specifying both arguments?
		var fn = (arguments.length > 1) ? arguments[1] : null;
		function findValueCallback(q, data) {
			var result;
			if( data && data.length ) {
				for (var i=0; i < data.length; i++) {
					if( data[i].result.toLowerCase() == q.toLowerCase() ) {
						result = data[i];
						break;
					}
				}
			}
			if( typeof fn == "function" ) fn(result);
			else $input.trigger("result", result && [result.data, result.value]);
		}
		$.each(trimWords($input.val()), function(i, value) {
			request(value, findValueCallback, findValueCallback);
		});
	}).bind("flushCache", function() {
		cache.flush();
	}).bind("setOptions", function() {
		$.extend(options, arguments[1]);
		// if we've updated the data, repopulate
		if ( "data" in arguments[1] )
			cache.populate();
	}).bind("unautocomplete", function() {
		select.unbind();
		$input.unbind();
		$(input.form).unbind(".autocomplete");
	});
	
	
	function selectCurrent() {
		var selected = select.selected();
		if( !selected )
			return false;
		
		var v = selected.result;
		previousValue = v;
		
		if ( options.multiple ) {
			var words = trimWords($input.val());
			if ( words.length > 1 ) {
				v = words.slice(0, words.length - 1).join( options.multipleSeparator ) + options.multipleSeparator + v;
			}
			v += options.multipleSeparator;
		}
		
		$input.val(v);
		hideResultsNow();
		$input.trigger("result", [selected.data, selected.value]);
		options.onSetValue();
		return true;
	}
	
	function onChange(crap, skipPrevCheck) {
		if( lastKeyPressCode == KEY.DEL ) {
			select.hide();
			return;
		}
		
		var currentValue = $input.val();
		
		if ( !skipPrevCheck && currentValue == previousValue )
			return;
		
		previousValue = currentValue;
		
		currentValue = lastWord(currentValue);
		if ( currentValue.length >= options.minChars) {
			$input.addClass(options.loadingClass);
			if (!options.matchCase)
				currentValue = currentValue.toLowerCase();
			request(currentValue, receiveData, hideResultsNow);
		} else {
			stopLoading();
			select.hide();
		}
	};
	
	function trimWords(value) {
		if ( !value ) {
			return [""];
		}
		var words = value.split( options.multipleSeparator );
		var result = [];
		$.each(words, function(i, value) {
			if ( $.trim(value) )
				result[i] = $.trim(value);
		});
		return result;
	}
	
	function lastWord(value) {
		if ( !options.multiple )
			return value;
		var words = trimWords(value);
		return words[words.length - 1];
	}
	
	// fills in the input box w/the first match (assumed to be the best match)
	// q: the term entered
	// sValue: the first matching result
	function autoFill(q, sValue){
		// autofill in the complete box w/the first match as long as the user hasn't entered in more data
		// if the last user key pressed was backspace, don't autofill
		if( options.autoFill && (lastWord($input.val()).toLowerCase() == q.toLowerCase()) && lastKeyPressCode != KEY.BACKSPACE ) {
			// fill in the value (keep the case the user has typed)
			$input.val($input.val() + sValue.substring(lastWord(previousValue).length));
			// select the portion of the value not typed by the user (so the next character will erase)
			$.Autocompleter.Selection(options, input, previousValue.length, previousValue.length + sValue.length);
		}
	};

	function hideResults() {
		clearTimeout(timeout);
		timeout = setTimeout(hideResultsNow, 200);
	};

	function hideResultsNow() {
		var wasVisible = select.visible();
		select.hide();
		clearTimeout(timeout);
		stopLoading();
		if (options.mustMatch) {
			// call search and run callback
			$input.search(
				function (result){
					// if no value found, clear the input box
					if( !result ) {
						if (options.multiple) {
							var words = trimWords($input.val()).slice(0, -1);
							$input.val( words.join(options.multipleSeparator) + (words.length ? options.multipleSeparator : "") );
						}
						else
							$input.val( "" );
					}
				}
			);
		}
		if (wasVisible) {
			// position cursor at end of input field
			// *** Removed this line, because it cleans the selected value on Safari and IE ********
			//$.Autocompleter.Selection(options, input, input.value.length, input.value.length);
		}
	};

	function receiveData(q, data) {
		if ( data && data.length && hasFocus ) {
			stopLoading();
			select.display(data, q);
			autoFill(q, data[0].value);
			select.show();
		} else {
			hideResultsNow();
		}
	};

	function request(term, success, failure) {
		if (!options.matchCase)
			term = term.toLowerCase();
		var data = cache.load(term);
		// recieve the cached data
		if (data && data.length) {
			success(term, data);
		// if an AJAX url has been supplied, try loading the data now
		} else if( (typeof options.url == "string") && (options.url.length > 0) ){
			
			var extraParams = {
				timestamp: +new Date()
			};
			$.each(options.extraParams, function(key, param) {
				extraParams[key] = typeof param == "function" ? param() : param;
			});
			
			$.ajax({
				// try to leverage ajaxQueue plugin to abort previous requests
				mode: "abort",
				// limit abortion to this input
				port: "autocomplete" + input.name,
				dataType: options.dataType,
				url: options.url,
				data: $.extend({
					q: lastWord(term),
					limit: options.max
				}, extraParams),
				success: function(data) {
					var parsed = options.parse && options.parse(data) || parse(data);
					cache.add(term, parsed);
					success(term, parsed);
				}
			});
		} else {
			// if we have a failure, we need to empty the list -- this prevents the the [TAB] key from selecting the last successful match
			select.emptyList();
			failure(term);
		}
	};
	
	function parse(data) {
		var parsed = [];
		var rows = data.split("\n");
		for (var i=0; i < rows.length; i++) {
			var row = $.trim(rows[i]);
			if (row) {
				row = row.split("|");
				parsed[parsed.length] = {
					data: row,
					value: row[0],
					result: options.formatResult && options.formatResult(row, row[0]) || row[0]
				};
			}
		}
		return parsed;
	};

	function stopLoading() {
		$input.removeClass(options.loadingClass);
	};

};

$.Autocompleter.defaults = {
	inputClass: "ac_input",
	resultsClass: "ac_results",
	loadingClass: "ac_loading",
	minChars: 1,
	delay: 400,
	matchCase: false,
	matchSubset: true,
	matchContains: false,
	cacheLength: 10,
	max: 100,
	mustMatch: false,
	extraParams: {},
	selectFirst: true,
	formatItem: function(row) { return row[0]; },
	formatMatch: null,
	autoFill: false,
	width: 0,
	multiple: false,
	multipleSeparator: ", ",
	highlight: function(value, term) {
		return value.replace(new RegExp("(?![^&;]+;)(?!<[^<>]*)(" + term.replace(/([\^\$\(\)\[\]\{\}\*\.\+\?\|\\])/gi, "\\$1") + ")(?![^<>]*>)(?![^&;]+;)", "gi"), "<strong>$1</strong>");
	},
    scroll: true,
    scrollHeight: 180,
    focusFieldAfterSelect: true,
    focusFieldAfterCancel: true,
    onSetValue : function(){}
};

$.Autocompleter.Cache = function(options) {

	var data = {};
	var length = 0;
	
	function matchSubset(s, sub) {
		if (!options.matchCase) 
			s = s.toLowerCase();
		var i = s.indexOf(sub);
		if (i == -1) return false;
		return i == 0 || options.matchContains;
	};
	
	function add(q, value) {
		if (length > options.cacheLength){
			flush();
		}
		if (!data[q]){ 
			length++;
		}
		data[q] = value;
	}
	
	function populate(){
		if( !options.data ) return false;
		// track the matches
		var stMatchSets = {},
			nullData = 0;

		// no url was specified, we need to adjust the cache length to make sure it fits the local data store
		if( !options.url ) options.cacheLength = 1;
		
		// track all options for minChars = 0
		stMatchSets[""] = [];
		
		// loop through the array and create a lookup structure
		for ( var i = 0, ol = options.data.length; i < ol; i++ ) {
			var rawValue = options.data[i];
			// if rawValue is a string, make an array otherwise just reference the array
			rawValue = (typeof rawValue == "string") ? [rawValue] : rawValue;
			
			var value = options.formatMatch(rawValue, i+1, options.data.length);
			if ( value === false )
				continue;
			
			var firstChar = "";
			if(value) {
				firstChar = value.charAt(0).toLowerCase();
			}
			// if no lookup array for this character exists, look it up now
			if( !stMatchSets[firstChar] ) 
				stMatchSets[firstChar] = [];

			// if the match is a string
			var row = {
				value: value,
				data: rawValue,
				result: options.formatResult && options.formatResult(rawValue) || value
			};
			
			// push the current match into the set list
			stMatchSets[firstChar].push(row);

			// keep track of minChars zero items
			if ( nullData++ < options.max ) {
				stMatchSets[""].push(row);
			}
		};

		// add the data items to the cache
		$.each(stMatchSets, function(i, value) {
			// increase the cache size
			options.cacheLength++;
			// add to the cache
			add(i, value);
		});
	}
	
	// populate any existing data
	setTimeout(populate, 25);
	
	function flush(){
		data = {};
		length = 0;
	}
	
	return {
		flush: flush,
		add: add,
		populate: populate,
		load: function(q) {
			if (!options.cacheLength || !length)
				return null;
			/* 
			 * if dealing w/local data and matchContains than we must make sure
			 * to loop through all the data collections looking for matches
			 */
			if( !options.url && options.matchContains ){
				// track all matches
				var csub = [];
				// loop through all the data grids for matches
				for( var k in data ){
					// don't search through the stMatchSets[""] (minChars: 0) cache
					// this prevents duplicates
					if( k.length > 0 ){
						var c = data[k];
						$.each(c, function(i, x) {
							// if we've got a match, add it to the array
							if (matchSubset(x.value, q)) {
								csub.push(x);
							}
						});
					}
				}				
				return csub;
			} else 
			// if the exact item exists, use it
			if (data[q]){
				return data[q];
			} else
			if (options.matchSubset) {
				for (var i = q.length - 1; i >= options.minChars; i--) {
					var c = data[q.substr(0, i)];
					if (c) {
						var csub = [];
						$.each(c, function(i, x) {
							if (matchSubset(x.value, q)) {
								csub[csub.length] = x;
							}
						});
						return csub;
					}
				}
			}
			return null;
		}
	};
};

$.Autocompleter.Select = function (options, input, select, config) {
	var CLASSES = {
		ACTIVE: "ac_over"
	};
	
	var listItems,
		active = -1,
		data,
		term = "",
		needsInit = true,
		element,
		list;
	
	// Create results
	function init() {
		if (!needsInit)
			return;
		element = $("<div/>")
		.hide()
		.addClass(options.resultsClass)
		.css("position", "absolute")
		.appendTo(document.body);
	
		list = $("<ul/>").appendTo(element).mouseover( function(event) {
			if(target(event).nodeName && target(event).nodeName.toUpperCase() == 'LI') {
	            active = $("li", list).removeClass(CLASSES.ACTIVE).index(target(event));
			    $(target(event)).addClass(CLASSES.ACTIVE);            
	        }
		}).click(function(event) {
			$(target(event)).addClass(CLASSES.ACTIVE);
			select();
			if ( options.focusFieldAfterSelect ) {
			    input.focus();
			}
			return false;
		}).mousedown(function() {
			config.mouseDownOnSelect = true;
		}).mouseup(function() {
			config.mouseDownOnSelect = false;
		});
		
		if( options.width > 0 )
			element.css("width", options.width);
			
		needsInit = false;
	} 
	
	function target(event) {
		var element = event.target;
		while(element && element.tagName != "LI")
			element = element.parentNode;
		// more fun with IE, sometimes event.target is empty, just ignore it then
		if(!element)
			return [];
		return element;
	}

	function moveSelect(step) {
		listItems.slice(active, active + 1).removeClass(CLASSES.ACTIVE);
		movePosition(step);
        var activeItem = listItems.slice(active, active + 1).addClass(CLASSES.ACTIVE);
        if(options.scroll) {
            var offset = 0;
            listItems.slice(0, active).each(function() {
				offset += this.offsetHeight;
			});
            if((offset + activeItem[0].offsetHeight - list.scrollTop()) > list[0].clientHeight) {
                list.scrollTop(offset + activeItem[0].offsetHeight - list.innerHeight());
            } else if(offset < list.scrollTop()) {
                list.scrollTop(offset);
            }
        }
	};
	
	function movePosition(step) {
		active += step;
		if (active < 0) {
			active = listItems.size() - 1;
		} else if (active >= listItems.size()) {
			active = 0;
		}
	}
	
	function limitNumberOfItems(available) {
		return options.max && options.max < available
			? options.max
			: available;
	}
	
	function fillList() {
		list.empty();
		var max = limitNumberOfItems(data.length);
		for (var i=0; i < max; i++) {
			if (!data[i])
				continue;
			var formatted = options.formatItem(data[i].data, i+1, max, data[i].value, term);
			if ( formatted === false )
				continue;
			var li = $("<li/>").html( options.highlight(formatted, term) ).addClass(i%2 == 0 ? "ac_even" : "ac_odd").appendTo(list)[0];
			$.data(li, "ac_data", data[i]);
		}
		listItems = list.find("li");
		if ( options.selectFirst ) {
			listItems.slice(0, 1).addClass(CLASSES.ACTIVE);
			active = 0;
		}
		// apply bgiframe if available
		if ( $.fn.bgiframe )
			list.bgiframe();
	}
	
	return {
		display: function(d, q) {
			init();
			data = d;
			term = q;
			fillList();
		},
		next: function() {
			moveSelect(1);
		},
		prev: function() {
			moveSelect(-1);
		},
		pageUp: function() {
			if (active != 0 && active - 8 < 0) {
				moveSelect( -active );
			} else {
				moveSelect(-8);
			}
		},
		pageDown: function() {
			if (active != listItems.size() - 1 && active + 8 > listItems.size()) {
				moveSelect( listItems.size() - 1 - active );
			} else {
				moveSelect(8);
			}
		},
		hide: function() {
			element && element.hide();
			listItems && listItems.removeClass(CLASSES.ACTIVE);
			active = -1;
		},
		visible : function() {
			return element && element.is(":visible");
		},
		current: function() {
			return this.visible() && (listItems.filter("." + CLASSES.ACTIVE)[0] || options.selectFirst && listItems[0]);
		},
		show: function() {
			var offset = $(input).offset();
			element.css({
				width: typeof options.width == "string" || options.width > 0 ? options.width : $(input).width(),
				top: offset.top + input.offsetHeight,
				left: offset.left
			}).show();
            if(options.scroll) {
                list.scrollTop(0);
                list.css({
					maxHeight: options.scrollHeight,
					overflow: 'auto'
				});
				
                if($.browser.msie && typeof document.body.style.maxHeight === "undefined") {
					var listHeight = 0;
					listItems.each(function() {
						listHeight += this.offsetHeight;
					});
					var scrollbarsVisible = listHeight > options.scrollHeight;
                    list.css('height', scrollbarsVisible ? options.scrollHeight : listHeight );
					if (!scrollbarsVisible) {
						// IE doesn't recalculate width when scrollbar disappears
						listItems.width( list.width() - parseInt(listItems.css("padding-left")) - parseInt(listItems.css("padding-right")) );
					}
                }
                
            }
		},
		selected: function() {
			var selected = listItems && listItems.filter("." + CLASSES.ACTIVE).removeClass(CLASSES.ACTIVE);
			return selected && selected.length && $.data(selected[0], "ac_data");
		},
		emptyList: function (){
			list && list.empty();
		},
		unbind: function() {
			element && element.remove();
		}
	};
};

$.Autocompleter.Selection = function(options, field, start, end) {
	if( field.createTextRange ){
		var selRange = field.createTextRange();
		selRange.collapse(true);
		selRange.moveStart("character", start);
		selRange.moveEnd("character", end);
		selRange.select();
	} else if( field.setSelectionRange ){
		field.setSelectionRange(start, end);
	} else {
		if( field.selectionStart ){
			field.selectionStart = start;
			field.selectionEnd = end;
		}
	}
	if( options.focusFieldAfterCancel ) {
	    field.focus();
    }    
};

})(jQuery);
(function($){

    $.fn.autoexpand = function(options) {  
        var defaults = {
            minLines: undefined,
            maxLines: undefined,
            timeBetweenSizeChanges: 100, // once one event is fired to change the text box size, other events are ignored for this No. of ms, and then the text box is resized
            canShrink: true
        };
    
        options = $.extend({}, defaults, options);
        
        return $(this).each(function(){
            
            var target = $(this);
            var timeout = undefined;

            var div = $("<div>&nbsp;</div>").css({
                "font" : target.css("font"),
                "font-size" : target.css("font-size"),
                "position" : "absolute",
                "left" : 0,
                "top" : 0
            });
            
            if( $.browser.msie && parseFloat($.browser.version) <= 8 ){
            	// ie8 does this for garbage and returns 1px
            } else {
            	div.css({
            		"line-height": target.css("line-height")
            	});
            }
            
            $("body").append(div);
            var lineHeight = parseFloat( div.css("height") );
            
            if( $.browser.msie || isNaN(lineHeight) ){
            	lineHeight = div.height();
            }
      
            div.remove();
            
            var clone = $(this).clone(false);
            
            $("body").append(clone);
            
            $(clone).attr("id", "").css({
                "position" : "absolute",
                "width" : target.width() || target.css("width"),
                "height" : lineHeight,
                "overflow-x" : target.css("overflow-x"),
                "overflow-y" : "scroll",
                "font" : target.css("font"),
                "white-space": target.css("white-space"),
                "font-size" : target.css("font-size")
            });
            
            if( $.browser.msie && parseFloat($.browser.version) <= 8 ){
            	// ie8 does this for garbage and returns 1px
            } else {
            	$(clone).css({
            		"line-height": target.css("line-height")
            	});
            }
            
            clone.css({
                "left" : -2 * clone.width(),
                "top" : -2 * clone.height()
            });
            
            var widthWithoutScroll = 2 * clone.width() - clone.attr("scrollWidth");
            var widthWithScroll = clone.width();
            
            function scrollMax(obj) {
                var prevScroll = 0;
                obj.scrollTop(1);
                while( obj.scrollTop() != prevScroll ) {
                    prevScroll = obj.scrollTop();
                    obj.scrollTop(prevScroll * 2);
                }
                
                if(obj.scrollTop() == 1) {
                    obj.scrollTop(0);
                }
            }
            
            function getCloneHeight() {
                return clone.scrollTop() + clone.innerHeight();
            }
           
            function fix() {           
                
                if( target.css("overflow-y") == "scroll" ) {
                    clone.width( widthWithScroll );
                } else {
                    clone.width( widthWithoutScroll );
                }
    
                clone.val( target.val() );
                
                
                // safari 3 fix
                // TODO remove on safari 4 main release
                if( $.browser.safari && parseInt( $.browser.version ) <= 525 ) {
                    var text = target.val();
                    
                    if( text.charAt( text.length - 1 ) == '\n' ) {
                        text = text.substr( 0, text.length - 1 );
                        clone.val( text );
                    }
                }
                
                var minHeight = (options.minLines * lineHeight) || parseFloat( target.css("min-height") ) || lineHeight;
                var maxHeight = (options.maxLines * lineHeight) || parseFloat( target.css("max-height") ) || target.height();
                
                // use scroll top, since scroll height can be buggy in some browsers
                // increase until can't be bigger
                
                scrollMax(clone);
                var newHeight = getCloneHeight();
                var oldHeight = target.height();
                
                if( ! options.canShrink && newHeight < target.height() ) {
                    newHeight = target.height();
                }
                
                if( newHeight < minHeight ) {
                    target.css("overflow-y", "hidden");
                    target.css( "height", minHeight );
                } else if( newHeight > maxHeight ) {
                    if( target.css("overflow-y") != "scroll" ) {
                        target.css("overflow-y", "scroll");
                    }
                    if( target.height() < maxHeight ) {
                        target.css("overflow-y", "hidden");
                        target.css("height", maxHeight);
                    }
                } else {
                    target.css("overflow-y", "hidden");
                    target.css( "height", newHeight );
                }

                if (oldHeight != newHeight && options.onResize) {
                	options.onResize();
                }
            }
            
            $(this).bind("click mouseup blur keyup input keydown keypress focus paste change", function(e){

//            	console.log(e.type);
//            	console.log($(this).val());
            	
            	clearTimeout(timeout);
            	timeout = null;
            	
                if( ! timeout ) {
//                	console.log("fix");
//                	console.log($(this).val());
                    fix();
                    
                    timeout = setTimeout(function(){
                        fix();
                        timeout = undefined;
                    }, options.timeBetweenSizeChanges);
                }
            });
            
            fix(); // initial fix for current text
        });
    }

})(jQuery); /*

jQuery Browser Plugin
	* Version 2.3
	* 2008-09-17 19:27:05
	* URL: http://jquery.thewikies.com/browser
	* Description: jQuery Browser Plugin extends browser detection capabilities and can assign browser selectors to CSS classes.
	* Author: Nate Cavanaugh, Minhchau Dang, & Jonathan Neal
	* Copyright: Copyright (c) 2008 Jonathan Neal under dual MIT/GPL license.
	* JSLint: This javascript file passes JSLint verification.
*//*jslint
		bitwise: true,
		browser: true,
		eqeqeq: true,
		forin: true,
		nomen: true,
		plusplus: true,
		undef: true,
		white: true
*//*global
		jQuery
*/

(function ($) {
	$.browserTest = function (a, z) {
		var u = 'unknown', x = 'X', m = function (r, h) {
			for (var i = 0; i < h.length; i = i + 1) {
				r = r.replace(h[i][0], h[i][1]);
			}

			return r;
		}, c = function (i, a, b, c) {
			var r = {
				name: m((a.exec(i) || [u, u])[1], b)
			};

			r[r.name] = true;

			r.version = (c.exec(i) || [x, x, x, x])[3];

			if (r.name.match(/safari/) && r.version > 400) {
				r.version = '2.0';
			}

			if (r.name === 'presto') {
				r.version = ($.browser.version > 9.27) ? 'futhark' : 'linear_b';
			}
			r.versionNumber = parseFloat(r.version, 10) || 0;
			r.versionX = (r.version !== x) ? (r.version + '').substr(0, 1) : x;
			r.className = r.name + r.versionX;

			return r;
		};

		a = (a.match(/Opera|Navigator|Minefield|KHTML|Chrome/) ? m(a, [
			[/(Firefox|MSIE|KHTML,\slike\sGecko|Konqueror)/, ''],
			['Chrome Safari', 'Chrome'],
			['KHTML', 'Konqueror'],
			['Minefield', 'Firefox'],
			['Navigator', 'Netscape']
		]) : a).toLowerCase();

		$.browser = $.extend((!z) ? $.browser : {}, c(a, /(camino|chrome|firefox|netscape|konqueror|lynx|msie|opera|safari)/, [], /(camino|chrome|firefox|netscape|netscape6|opera|version|konqueror|lynx|msie|safari)(\/|\s)([a-z0-9\.\+]*?)(\;|dev|rel|\s|$)/));

		$.layout = c(a, /(gecko|konqueror|msie|opera|webkit)/, [
			['konqueror', 'khtml'],
			['msie', 'trident'],
			['opera', 'presto']
		], /(applewebkit|rv|konqueror|msie)(\:|\/|\s)([a-z0-9\.]*?)(\;|\)|\s)/);

		$.os = {
			name: (/(win|mac|linux|sunos|solaris|iphone)/.exec(navigator.platform.toLowerCase()) || [u])[0].replace('sunos', 'solaris')
		};

		if (!z) {
			$('html').addClass([$.os.name, $.browser.name, $.browser.className, $.layout.name, $.layout.className].join(' '));
		}
	};

	$.browserTest(navigator.userAgent);
})(jQuery);(function($){
	
	var defaults = {
		title: "Your browser needs to be updated",
		siteName: "this site",
		outOfDateText: "Your browser is out-of-date and needs to be updated to work on {name} properly.",
		updateText: "Update {name}.",
		unsupportedText: "Your browser is not supported by {name}.  Please upgrade to one of the supported browsers below.",
		alternativeText: "These browsers are also supported by {name}.",
		recommendText: "<strong>Internet Explorer is slower and buggier than other browsers.  We recommend you use one of these alternative browsers with {name}.</strong>",
		browsers: {
		}
	};
	
	function version(str){
		str = "" + str;
		var parts = str.split(".");
		var version = {};
		
		version.major = parseInt(parts[0]);
		if( parts.length > 1 ){
			version.minor = parseInt(parts[1]);
		}
		
		version.olderThan = function(needed){
			if( needed.major > version.major ){
				return true;
			} else if( needed.major == version.major ){
				if( needed.minor > version.minor ){
					return true;
				}
			}
			
			return false;
		};
		
		return version;
	}
	
	$.browserupdate = function(opts){
		var options = $.extend(true, {}, defaults, opts);
		
		var browser = options.browsers[ $.browser.name ];
		
		var $root = $('<div class="ui-browserupdate-notification"></div>');
		$("body").append($root);
		
		var $bg = $('<div class="ui-browserupdate-background"></div>');
		$root.append($bg);
		
		var $msg = $('<div class="ui-browserupdate-message"></div>');
		$root.append($msg);
		
		$msg.append('<h2>' + options.title + '</h2>');
		
		if( browser == null ){
			$msg.append('<p class="ui-browserupdate-unsupported">' + options.unsupportedText.replace("{name}", options.siteName) + '</p>');
		} else {
			var current = version($.browser.version);
			var needed = version(browser.version);
			
			if( current.olderThan(needed) ) {
				$msg.append('<p class="ui-browserupdate-out-of-date">' + options.outOfDateText.replace("{name}", options.siteName) + '</p>');
				
				$msg.append('<a href="' + browser.url + '">\
							<div class="ui-browserupdate-icon '+ $.browser.name +'"></div>\
							<span class="ui-browserupdate-update-message">' + options.updateText.replace("{name}", browser.name) + '</span>\
							<span class="ui-browserupdate-versions">Current version: <em>' + $.browser.version + '</em></span>\
							<span class="ui-browserupdate-versions">Minimum required version: <em>' + browser.version + '</em></span>\
						</a>');
				
				if( $.browser.msie ){
					$msg.append('<p class="ui-browserupdate-alternative">' + options.recommendText.replace("{name}", options.siteName) + '</p>');
				} else {
					$msg.append('<p class="ui-browserupdate-alternative">' + options.alternativeText.replace("{name}", options.siteName) + '</p>');
				}
				
			} else {
				$root.hide();
			}
			
			$.each(options.browsers, function(name, browser){
				if( name == $.browser.name || name == "msie" ){
					return;
				}
				
				$msg.append('<a href="'+ browser.url +'" target="_blank">\
								<div class="ui-browserupdate-icon '+ name +'"><span class="ui-browserupdate-name">'+ browser.name +'</span></div>\
							</a>');
			});
		}
		
		
		
		$msg.css({
			position: "absolute",
			left: "50%",
			top: "50%"
		});
		
		$msg.css({
			marginLeft: -1*( parseInt($msg.outerWidth())/2 ),
			marginTop: -1*( parseInt($msg.outerHeight())/2 )
		});
		
		
	};
	
})(jQuery);/*
 * jQuery Caret Range plugin
 * Copyright (c) 2009 Matt Zabriskie
 * Released under the MIT and GPL licenses.
 */
(function($) {
	$.extend($.fn, {
		caret: function (start, end) {
			var elem = this[0];

			if (elem) {							
				// get caret range
				if (typeof start == "undefined") {
					if (elem.selectionStart) {
						start = elem.selectionStart;
						end = elem.selectionEnd;
					}
					else if (document.selection) {
						var val = this.val();
						var range = document.selection.createRange().duplicate();
						range.moveEnd("character", val.length)
						start = (range.text == "" ? val.length : val.lastIndexOf(range.text));

						range = document.selection.createRange().duplicate();
						range.moveStart("character", -val.length);
						end = range.text.length;
					}
				}
				// set caret range
				else {
					var val = this.val();

					if (typeof start != "number") start = -1;
					if (typeof end != "number") end = -1;
					if (start < 0) start = 0;
					if (end > val.length) end = val.length;
					if (end < start) end = start;
					if (start > end) start = end;

					elem.focus();

					if (elem.selectionStart) {
						elem.selectionStart = start;
						elem.selectionEnd = end;
					}
					else if (document.selection) {
						var range = elem.createTextRange();
						range.collapse(true);
						range.moveStart("character", start);
						range.moveEnd("character", end - start);
						range.select();
					}
				}

				return {start:start, end:end};
			}
		}
	});
})(jQuery);/**
    Project: CheckTree jQuery Plugin
    Version: 0.21
    Project Website: http://static.geewax.org/checktree/
    Author: JJ Geewax <jj@geewax.org>
    
    License:
        The CheckTree jQuery plugin is currently available for use in all personal or 
        commercial projects under both MIT and GPL licenses. This means that you can choose 
        the license that best suits your project, and use it accordingly.
*/
(function(jQuery) {

jQuery.fn.modifyCheckTree = function(settings) {
    settings = jQuery.extend({
        expandAll: false,
        expandTopLevel: false,
        expandNone: false,
        checkAll: false,
        checkNone: false
    }, settings);
    
    if(settings.expandAll) {
        jQuery(this).children("li").find(".arrow").each(function() {
            if( ! jQuery(this).hasClass("expanded") ) {
                jQuery(this).click();
            }
        });
    } else {
        if(settings.expandNone || settings.expandTopLevel) {
            // must be children first, then parent (reverse is a specific case of that scenario)
            jQuery(this).children("li").find(".arrow").reverse().each(function() {
                if( jQuery(this).hasClass("expanded") ) {
                    jQuery(this).click();
                }
            });
        }
        
        if(settings.expandTopLevel) {
            jQuery(this).children("li").find(".arrow:first").each(function() {
                if( ! jQuery(this).hasClass("expanded") ) {
                    jQuery(this).click();
                }
            });
        }
    }

    if(settings.checkAll) {
    	// On Firefox, sometimes children("li") returns zero elements after rebuilding the tree:
    	// Using a pseudo-class instead of "li" seems to work (don't ask me why!!!):
    	// *****************************************************************************************
    	//jQuery(this).children("li").find(".checkbox:first").each(function() {
    	jQuery(this).children(".checktree_top_level").find(".checkbox:first").each(function() {
            if( ! jQuery(this).hasClass("checked") ) {
                jQuery(this).click();
            }
        });
    } else if(settings.checkNone) {
    	// On Firefox, sometimes children("li") returns zero elements after rebuilding the tree:
    	// Using a pseudo-class instead of "li" seems to work (don't ask me why!!!):
    	// *****************************************************************************************
    	//jQuery(this).children("li").find(".checkbox:first").each(function() {
    	jQuery(this).children(".checktree_top_level").find(".checkbox:first").each(function() {
    		if (! jQuery(this).find("input:first").attr("disabled")) {
	    		if( jQuery(this).hasClass("checked") ) {
	                jQuery(this).click();
	            } else if ( jQuery(this).hasClass("half_checked") ) {
	                jQuery(this).click().click();
	            }
    		}
        });
    }
};


jQuery.fn.checkTree = function(settings) {


    settings = jQuery.extend({
        /* Callbacks
            The callbacks should be functions that take one argument. The checkbox tree
            will return the jQuery wrapped LI element of the item that was checked/expanded.
        */
        onExpand: null,
        onCollapse: null,
        onCheck: null,
        onUnCheck: null,
        onHalfCheck: null,
        onLabelHoverOver: null,
        onLabelHoverOut: null,
        onSelect: null,
        onUnSelect: null,
        
        checkedText: "&nbsp;",
        uncheckedText: "&nbsp;",
        halfCheckedText: "&nbsp",
        
        expandedText: "&nbsp",
        collapsedText: "&nbsp;",
        
        /* Valid choices: 'expand', 'check' */
        labelAction: "expand",
        
        // Debug (currently does nothing)
        debug: false
    }, settings);

    function updateNamedCheckMarks(li) {
        var name = li.children(":checkbox").attr("name");
        var checked = li.find(".checkbox").hasClass("checked");
        
        if( ! name ) {   
            return;
        }
       

        li.parents("ul").find(":checkbox[name=" + name +"]").each(function(){
            jQuery(this).prev(".checkbox:first").each(function(){
                if( checked ) {
                    if( ! jQuery(this).hasClass("checked") ) {
                        jQuery(this).click();
                    }
                } else {
                    if( jQuery(this).hasClass("checked") ) {
                        jQuery(this).click();
                    } else if ( jQuery(this).hasClass("half_checked") ) {
                        jQuery(this).click().click();
                    }
                }
            });
        });
        
    }

    // update view of arrows
    function updateCheckTreeArrow(li, type) {
        li.find(".arrow:first").each(function() {
            if( type == "expanded" || ( ! type && jQuery(this).hasClass("expanded") ) ) {
                jQuery(this).html(settings.expandedText);
            } else if ( type == "collapsed" || ( ! type && jQuery(this).hasClass("collapsed") ) ) {
                jQuery(this).html(settings.collapsedText);
            } else {
                jQuery(this).html("&nbsp;");
            }
        });
    }
    
    // update check box views
    function updateCheckTreeCheck(li, type, change) {
    	
    	if( li.size() == 0 ){
    		return;
    	}
    	
//    	console.log(li);
//    	console.log(type);
    	
    	function update_parent(check){
    		var parent = check.parents("li:first").parents("li:first");
    		var sibs = check.parents("ul:first").find(".checkbox");
    		var checked_sibs = sibs.filter(".checked");
    		
    		if( checked_sibs.size() == sibs.size() ){
    			updateCheckTreeCheck(parent, "checked");
    		} else if( checked_sibs.size() == 0 ){
    			updateCheckTreeCheck(parent, "unchecked");
    		} else {
    			updateCheckTreeCheck(parent, "half_checked");
    		}
    	}
    	
        li.find(".checkbox:first").each(function() {
            if( type == "checked" || ( ! type && jQuery(this).hasClass("checked") ) ) {
            	jQuery(this).siblings(":checkbox").attr("checked", "checked");
            	jQuery(this).removeClass("half_checked").addClass("checked");
                jQuery(this).html(settings.checkedText);
                
                update_parent( jQuery(this) );
                
                if( change ){
                	jQuery(this).trigger("change");
                	if (settings.onCheck) settings.onCheck(li);
                }
            } else if ( type == "half_checked" || ( ! type && jQuery(this).hasClass("half_checked") ) ) {
            	jQuery(this).siblings(":checkbox").attr("checked", null);
            	jQuery(this).removeClass("checked").addClass("half_checked");
                jQuery(this).html(settings.halfCheckedText);
                
                update_parent( jQuery(this) );
                
                if( change ){
                	jQuery(this).trigger("change");
                	if (settings.onHalfCheck) settings.onHalfCheck(li);
                }
            } else {
            	jQuery(this).siblings(":checkbox").attr("checked", null);
            	jQuery(this).removeClass("checked").removeClass("half_checked");
                jQuery(this).html(settings.uncheckedText);
                
                update_parent( jQuery(this) );
                
                if( change ){
                	jQuery(this).trigger("change");
                	if (settings.onUnCheck) settings.onUnCheck(li);
                }
            }
        });
    }

    var $tree = this;
    
    $tree.find("li")
        // Hide all of the sub-trees
        .find("ul")
            .hide()
        .end()
        
        // Hide all checkbox inputs
        .find(":checkbox")
        	.hide()
        .end()
        
        
        .find(".label")
            // Clicking the labels should expand the children
            .click(function() {
                var action = settings.labelAction;
                switch(settings.labelAction) {
                    case 'expand':
                        jQuery(this).siblings(".arrow").click();
                        break;
                    case 'check':
                        jQuery(this).siblings(".checkbox").click();
                        break;
                    case 'select':
                    	if( jQuery(this).hasClass("selected") ) {
                    		jQuery(this).removeClass("selected").removeClass("select_hover");
                    		if(settings.onSelect != null) {
                    			settings.onUnSelect( jQuery(this).parent() );
                        	}
                    	} else {
                    		jQuery(this).addClass("selected").removeClass("select_hover");
                    		if(settings.onSelect != null) {
                    			settings.onSelect( jQuery(this).parent() );
                        	}
                    	}
                    	break;
                }
            })
        .mouseover(function(){
        	if( settings.labelAction == "select" ) {
        		jQuery(this).addClass("select_hover");
        	}
        	if(settings.onLabelHoverOver != null) {
    			settings.onLabelHoverOver( jQuery(this).parent() );
        	}
        })
        .mouseout(function(){
        	if( settings.labelAction == "select" ) {
        		jQuery(this).removeClass("select_hover");
        	}
        	if(settings.onLabelHoverOut != null) {
    			settings.onLabelHoverOut( jQuery(this).parent() );
        	}
        })
        .end()
        
        .each(function() {
            // Create the image for the arrow (to expand and collapse the hidden trees)
            var $arrow = jQuery('<div class="arrow">&nbsp;</div>');
            
            // If it has children:
            if (jQuery(this).is(":has(ul)")) {
                $arrow.addClass("collapsed"); // Should start collapsed
                
                // When you click the image, toggle the child list
                $arrow.click(function() {
                    jQuery(this).siblings("ul").toggle();
                    
                    if (jQuery(this).hasClass("collapsed")) {
                        //toggled = settings.expandedarrow;
                        jQuery(this)
                            .addClass("expanded")
                            .removeClass("collapsed")
                        ;
                        if (settings.onExpand) settings.onExpand(jQuery(this).parent());
                        updateCheckTreeArrow( jQuery(this).parent(), "expanded" );
                    }
                    else {
                        //toggled = settings.collapsedarrow;
                        jQuery(this)
                            .addClass("collapsed")
                            .removeClass("expanded")
                        ;
                        if (settings.onCollapse) settings.onCollapse(jQuery(this).parent());
                        updateCheckTreeArrow( jQuery(this).parent(), "collapsed" );
                    }
                });
            }
            
            // Create the image for the checkbox next to the label
            var $checkbox = jQuery('<div class="checkbox">&nbsp;</div>');
            
            // Don't create check box if there is no input
            if ( ! jQuery(this).is(":has(input)") ) {
                $checkbox = jQuery('<div class="checkbox_spacer">&nbsp;</div>');
            }
            
            // When you click the checkbox, it should do the checking/unchecking
            $checkbox.click(function() {
                
                if( $(this).hasClass("checkbox_spacer") ) {
                    return;
                }
                
                // Make the current class checked
                if( jQuery(this).hasClass("half_checked") ){
                	updateCheckTreeCheck(jQuery(this).parent(), "checked", true);
                } else if( jQuery(this).hasClass("checked") ){
                	updateCheckTreeCheck(jQuery(this).parent(), "unchecked", true);
                } else {
                	updateCheckTreeCheck(jQuery(this).parent(), "checked", true);
                }
                
                // Check/uncheck children depending on our status.
                if (jQuery(this).hasClass("checked")) {
                    // Fire the check callback for this parent
//                    updateCheckTreeCheck(jQuery(this).parent(), "checked");
//                    updateNamedCheckMarks(jQuery(this).parent());
//                    if (settings.onCheck) settings.onCheck(jQuery(this).parent());
                    
                    
                    jQuery(this).siblings("ul").find(".checkbox").not(".checked")
                        .each(function() {
                            
                            updateCheckTreeCheck(jQuery(this).parent(), "checked");
                            updateNamedCheckMarks(jQuery(this).parent());
//                            if (settings.onCheck) settings.onCheck(jQuery(this).parent());
                            
                        })
                    ;

                    // Moved check action to here, so it's not called once per input:
                    // #################################################################
					// Tell our parent checkbox that we've changed
                	//jQuery(this).parents("ul").siblings(".checkbox").change();
//                    if (settings.onCheck) settings.onCheck(jQuery(this).parent());
                    // #################################################################
                }
                else {
                    // Fire the uncheck callback for this parent
//                    updateCheckTreeCheck(jQuery(this).parent(), "unchecked");
//                    updateNamedCheckMarks(jQuery(this).parent());
                    //if (settings.onUnCheck) settings.onUnCheck(jQuery(this).parent());
                    
                    jQuery(this).siblings("ul").find(".checkbox").filter(".checked")
                        .each(function() {
                            
                            updateCheckTreeCheck(jQuery(this).parent(), "unchecked");
                            updateNamedCheckMarks(jQuery(this).parent());
                            //if (settings.onUnCheck) settings.onUnCheck(jQuery(this).parent());
                            
                        })

                    ;

                    // Moved uncheck action to here, so it's not called once per input:
                    // #################################################################
					// Tell our parent checkbox that we've changed
                	//jQuery(this).parents("ul").siblings(".checkbox").change();
//                    if (settings.onUnCheck) settings.onUnCheck(jQuery(this).parent());
                    // #################################################################
                }
                // Tell our parent checkbox that we've changed
                //jQuery(this).parents("ul").siblings(":checkbox").change();
                if (settings.onCheck) settings.onCheck(jQuery(this).parent());
            });
            
            // Prepend the arrow and checkbox images to the front of the LI
            jQuery(this)
                .prepend($checkbox)
                .prepend($arrow)    
            ;
            
            updateCheckTreeCheck( jQuery(this) );
            updateCheckTreeArrow( jQuery(this) );
            
            
        });
        

    return $tree;
};
})(jQuery);/*! Build-0.6: color.core.js color.rgb.js color.huebased.js color.hsv.js color.hsl.js color.object.js color.parse.js color.related.js color.palette.css3.js color.css.js */
/*!
 * jQuery Colour 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Author: Mark Gibson (jollytoad at gmail dot com)
 *
 * http://www.adaptavist.com/display/jQuery/Colour+Library
 */
(jQuery.color || (function($) {

$.color = {

	// Compare two colour tuples (must be of the same colour space)
	isEqual: function ( tupleA, tupleB ) {
		if ( tupleA.length !== tupleB.length ) { return false; }
		
		var i = tupleA.length;
		while (i--) {
			if ( tupleA[i] !== tupleB[i] ) { return false; }
		}
		
		return true;
	},
	
	// Fix the values in a colour tuple
	fix: function ( tuple, format ) {
		var i = format.length;
		while (i--) {
			if ( typeof tuple[i] === 'number' ) {
				switch(format.charAt(i)) {
					case 'i': // integer
						tuple[i] = Math.round(tuple[i]);
						break;
					case 'o': // octet; integer 0..255
						tuple[i] = Math.min(255, Math.max(0, Math.round(tuple[i])));
						break;
					case '1': // one: float, 0..1
						tuple[i] = Math.min(1, Math.max(0, tuple[i]));
						break;
				}
			}
		}
		return tuple;
	},
	
	self: function( tuple ) {
		return tuple;
	},
	
	// Common alpha channel retrieval, defaults to 1
	alpha: function( val ) {
		return val === undefined ? 1 : val;
	},
	
	// A collection of colour palettes
	palette: {},
	
	// Registered colour functions
	fns: []
};

})(jQuery)
);

/*
 * jQuery UI Colour Red-Green-Blue 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *	color.core.js
 */
(jQuery.color && (function($) {

$.color.RGB = {

	fix: function ( rgb ) {
		rgb = $.color.fix(rgb, 'ooo1');
		return rgb;
	},
	
	toRGB: $.color.self,

	// RGB values must be integers in the range 0-255
	toHEX: function ( rgb ) {
		return '#' + (0x1000000 + rgb[0]*0x10000 + rgb[1]*0x100 + rgb[2]).toString(16).slice(-6);
	},

	toCSS: function ( rgb ) {
		if ( $.color.alpha(rgb[3]) === 0 ) {
			// Completely transparent, use the universally supported name
			return 'transparent';
		}
		if ( $.color.alpha(rgb[3]) < 1 ) {
			// Color is not opaque - according to the CSS3 working draft we should
			// not simply treat an RGBA value as an RGB value with opacity ignored.
			return 'rgba(' + rgb.join(',') + ')';
		}
		return 'rgb(' + Array.prototype.slice.call(rgb,0,3).join(',') + ')';
	},
	
	red: function ( rgb ) {
		return rgb[0];
	},
	
	green: function ( rgb ) {
		return rgb[1];
	},
	
	blue: function ( rgb ) {
		return rgb[2];
	},
	
	alpha: function ( rgb ) {
		return $.color.alpha(rgb[3]);
	}
};

$.color.RGB.toString = $.color.RGB.toHEX;

// Register the colour space methods
$.color.fns.push(
	'RGB.toRGB', 'RGB.toHEX', 'RGB.toCSS',
	'RGB.red', 'RGB.green', 'RGB.blue', 'RGB.alpha'
);

})(jQuery)
);

/*
 * jQuery Colour - Common functions for HSV & HSL colour spaces 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *	color.core.js
 *  color.rgb.js
 */
(jQuery.color && (function($) {

$.color.HueBased = {

	fix: function ( hx ) {
		hx[0] = (hx[0] + 1) % 1;
		return $.color.fix(hx, '1111');
	},

	complementary: function ( hx, offset ) {
		return [ (hx[0] + 0.5 + (offset || 0)) % 1.0, hx[1], hx[2], hx[3] ];
	},

	analogous: function ( hx, offset ) {
		return [ (hx[0] + 1.0 + (offset || 0)) % 1.0, hx[1], hx[2], hx[3] ];
	},

	hue: function ( hx ) {
		return hx[0];
	},

	alpha: function ( hx ) {
		return $.color.alpha(hx[3]);
	}
};

})(jQuery)
);

/*
 * jQuery Colour Hue-Saturation-Value 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *	color.core.js
 *  color.rgb.js
 *  color.huebased.js
 */
(jQuery.color && (function($) {

$.color.HSV = $.extend({

	toHSV: $.color.self,

	// HSV values are normalized to the range 0..1
	toRGB: function ( hsv ) {
		var ha = hsv[0]*6,
			hb = Math.floor( ha ),
			f = ha - hb,
			s = hsv[1],
			v = hsv[2] * 255,
			a = hsv[3],
			p = Math.round(v * ( 1 - s )),
			q = Math.round(v * ( 1 - f * s)),
			t = Math.round(v * ( 1 - ( 1 - f ) * s ));
		v = Math.round(v);
		switch (hb % 6) {
			case 0: return [v,t,p,a];
			case 1: return [q,v,p,a];
			case 2: return [p,v,t,a];
			case 3: return [p,q,v,a];
			case 4: return [t,p,v,a];
			case 5: return [v,p,q,a];
		}
	},

	// NOTE: the 'V' this is to distingush HSV from HSL which has a differing view of saturation
	saturationV: function ( hsv ) {
		return hsv[1];
	},

	value: function ( hsv ) {
		return hsv[2];
	}

}, $.color.HueBased);

$.color.RGB.toHSV = function ( rgb ) {
	var r = rgb[0]/255,
		g = rgb[1]/255,
		b = rgb[2]/255,
		min = Math.min(r,g,b),
		max = Math.max(r,g,b),
		d = max - min;

	return [
		d === 0 ? 0 :
		(g === max ? (b-r)/d/6 + (1/3) :
		 b === max ? (r-g)/d/6 + (2/3) :
		         (g-b)/d/6 + 1) % 1,
		d === 0 ? 0 : d/max,
		max,
		rgb[3]
	];
};

// Register the colour space methods
$.color.fns.push(
	'HSV.toHSV', 'HSV.toRGB', 'RGB.toHSV',
	'HSV.complementary', 'HSV.analogous',
	'HSV.hue', 'HSV.saturationV', 'HSV.value', 'HSV.alpha'
);

})(jQuery)
);

/*
 * jQuery Colour Hue-Saturation-Lightness 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *	color.core.js
 *  color.rgb.js
 *  color.huebased.js
 */
(jQuery.color && (function($) {

$.color.HSL = $.extend({

	toHSL: $.color.self,

	toRGB: function ( hsl ) {
		var h = hsl[0],
			s = hsl[1],
			l = hsl[2],
			q = l < 0.5 ? l*(1+s) : l+s-(l*s),
			p = 2*l-q;

		function c(x) {
			var t = x < 0 ? x+1 : x > 1 ? x-1 : x;
			return t < 1/6 ? p + (q-p) * 6 * t :
			       t < 1/2 ? q :
			       t < 2/3 ? p + (q-p) * 6 * (2/3 - t) :
			                 p;
		}

		return [
			Math.round(255 * c(h + 1/3)),
			Math.round(255 * c(h)),
			Math.round(255 * c(h - 1/3)),
			hsl[3]
		];
	},

	// NOTE: the 'L' this is to distingush HSL from HSV which has a differing view of saturation
	saturationL: function ( hsl ) {
		return hsl[1];
	},

	lightness: function ( hsl ) {
		return hsl[2];
	}

}, $.color.HueBased);

$.color.RGB.toHSL = function ( rgb ) {
	var r = rgb[0]/255,
		g = rgb[1]/255,
		b = rgb[2]/255,
		min = Math.min(r,g,b),
		max = Math.max(r,g,b),
		d = max - min,
		p = max + min;

	return [
		d === 0 ? 0 :
		(g === max ? (b-r)/d/6 + (1/3) :
		 b === max ? (r-g)/d/6 + (2/3) :
		             (g-b)/d/6 + 1) % 1,

		d === 0 ? 0 :
		p > 1 ? d / (2 - max - min) :
		        d / p,

		p/2,
		rgb[3]
	];
};

$.color.fns.push(
	'HSL.toHSL', 'HSL.toRGB', 'RGB.toHSL',
	'HSL.complementary', 'HSL.analogous',
	'HSL.hue', 'HSL.saturationL', 'HSL.lightness', 'HSL.alpha'
);

})(jQuery)
);

/*
 * jQuery Colour Object 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  color.core.js
 *  color.rgb.js
 */
(jQuery.color && jQuery.Color || (function($) {

// Construct a colour object of a given space (eg. 'RGB', 'HSV')
$.Color = function ( color, space, name ) {
	if ( typeof this === 'function' ) {
		return new $.Color(color, space, name);
	}
	
	if ( typeof color === 'string' && $.color.parse ) {
		if (!name) {
			name = color;
		}
		// Attempt to parse the string if the parser is available
		color = $.color.parse(color);
	}
	
	if ( color && color.length ) {
		// Copy channel values
		var i;
		i = this.length = color.length;
		while( i-- ) {
			this[i] = color[i];
		}
	}
	
	if ( color ) {
		this.space = space || color.space || 'RGB';
		this.name = name || color.name;
	}
};

function modify( tuple, relative ) {
	// Ensure the color to be modified is the same space as the argument
	var color = $.Color.isInstance(tuple) && tuple.space !== this.space ?
				this.to(tuple.space) :
				new $.Color(this),
		i = color.length,
		mod = false;
	
	while( i-- ) {
		if ( typeof tuple[i] === 'number' ) {
			var v = relative ? color[i] + tuple[i] : tuple[i];
			if ( v !== color[i] ) {
				color[i] = v;
				mod = true;
			}
		}
	}
	
	return mod ? color.setName() : this;
}

$.Color.fn = $.Color.prototype = {

	color: "0.6",
	
	// Get the utility functions for the colour space
	util: function() {
		return $.color[this.space];
	},
	
	// Convert the colour to a different colour space
	to: function( space ) {
		return this['to'+space]();
	},

	// Ensure colour channels values are within the valid limits
	fix: function() {
		return this.util().fix(this);
	},
	
	// Modify the individual colour channels, returning a new color object
	modify: function( tuple ) {
		return modify.call(this, tuple);
	},
	
	// Adjust the colour channels relative to current values
	adjust: function( tuple ) {
		return modify.call(this, tuple, true);
	},
	
	setName: function( newName ) {
		this.name = newName;
		return this;
	},

	toString: function() {
		if ( !this.space ) { return ''; }
		var util = this.util();
		return util.hasOwnProperty('toString') ? util.toString(this) : this.to('RGB').toString();
	},
	
	join: [].join,
	push: [].push
};

// Check whether the given argument is a valid color object
$.Color.isInstance = function( color ) {
	return color && typeof color === 'object' && color.color === $.Color.fn.color && color.space;
};

// Hold the default colour space for each method
$.Color.fnspace = {};

// Generate the wrapper for colour methods calls
function wrapper( color, subject, fn, space, copyName ) {
	return function() {
		var args = [color];
		Array.prototype.push.apply(args, arguments);
		var result = fn.apply(subject, args);
		return $.isArray(result) ? new $.Color(result, space, copyName ? color.name : undefined) : result;
	};
}

// Generate the prototype for method calls
function method( color, name ) {
	var toSpace = /^to/.test(name) ? name.substring(2) : false;
	
	return function() {
		var color = this,
			util = color.util();
		
		if ( !util[name] ) {
			// Convert to the appropriate colour space
			color = color.to($.Color.fnspace[name]);
			util = color.util();
		}
		
		var fn = wrapper(color, util, util[name], toSpace || color.space, !!toSpace),
			result = fn.apply(color, arguments);
		
		// Override the function for this instance so it can be reused
		// without the overhead of another lookup or conversion.
		if ( toSpace ) {
			// The function will return the same result every time, so cache the result
			this[name] = function() {
				return result;
			};
			if ( $.Color.isInstance(result) ) {
				color = this;
				result['to'+this.space] = function() {
					return color;
				};
			}
		} else {
			this[name] = fn;
		}
		
		return result;
	};
}

// Add colour function to the prototype
function addfn() {
	var s = this.split('.'),
		name = s[1],
		space = s[0];
	
	// Ensure the colour space conversion function isn't associated with it's own space
	if ( !$.Color.fnspace[name] && name !== 'to'+space ) {
		$.Color.fnspace[name] = space;
	}
	
	if ( !$.Color.fn[name] ) {
		$.Color.fn[name] = method(this, name);
	}
}

// Add existing functions
$.each($.color.fns, addfn);

// Override push to catch new functions
$.color.fns.push = function() {
	$.each(arguments, addfn);
	return Array.prototype.push.apply(this, arguments);
};

})(jQuery)
);

/*
 * jQuery Colour Parsing 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  color.core.js
 */
(jQuery.color && (function($) {

$.extend($.color, {

	// Color string parsing taken from effects.core.js
	parse: function ( color ) {
		var m;

		if ( typeof color === 'string' ) {

			// Look for rgb(int,int,int) or rgba(int,int,int,float)
			if ( (m = /^\s*rgb(a)?\(\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*,\s*([0-9]{1,3})\s*(?:,\s*([0-9]+(?:\.[0-9]+)?)\s*)?\)\s*$/.exec(color)) && !m[1] === !m[5] ) {
				return [parseInt(m[2],10), parseInt(m[3],10), parseInt(m[4],10), m[5] ? parseFloat(m[5]) : 1];
			}

			// Look for rgb(float%,float%,float%) or rgba(float%,float%,float%,float)
			if ( (m = /^\s*rgb(a)?\(\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*(?:,\s*([0-9]+(?:\.[0-9]+)?)\s*)?\)\s*$/.exec(color)) && !m[1] === !m[5] ) {
				return [parseFloat(m[2])*255/100, parseFloat(m[3])*255/100, parseFloat(m[4])*255/100, m[5] ? parseFloat(m[5]) : 1];
			}

			// Look for #a0b1c2
			if ( (m = /^\s*#([a-fA-F0-9]{2})([a-fA-F0-9]{2})([a-fA-F0-9]{2})\s*$/.exec(color)) ) {
				return [parseInt(m[1],16), parseInt(m[2],16), parseInt(m[3],16), 1];
			}

			// Look for #fff
			if ( (m = /^\s*#([a-fA-F0-9])([a-fA-F0-9])([a-fA-F0-9])\s*$/.exec(color)) ) {
				return [parseInt(m[1]+m[1],16), parseInt(m[2]+m[2],16), parseInt(m[3]+m[3],16), 1];
			}

			// Look for hsl(int,float%,float%) or hsla(int,float%,float%,float)
			if ( (m = /^\s*hsl(a)?\(\s*([0-9]{1,3})\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*,\s*([0-9]+(?:\.[0-9]+)?)\%\s*(?:,\s*([0-9]+(?:\.[0-9]+)?)\s*)?\)\s*$/.exec(color)) && !m[1] === !m[5] ) {
				return [parseInt(m[2],10)/360, parseFloat(m[3])/100, parseFloat(m[4])/100, m[5] ? parseFloat(m[5]) : 1];
			}

			// Otherwise, we're most likely dealing with a named color
			return $.color.named(color);
		}

		// Check if we're already dealing with a color tuple
		if ( color && ( color.length === 3 || color.length === 4 ) ) {
			if ( color.length === 3 ) {
				color.push( 1 );
			}
			return color;
		}
	},

	named: function ( color ) {
		var result;
		color = $.trim(color.toLowerCase());

		// Check for transparent
		if ( color === "transparent" ) {
			return [0, 0, 0, 0];
		}

		$.each($.color.palette, function(n, palette) {
			if (palette[color]) {
				result = palette[color];
				return false;
			}
		});
		return result;
	}

});

})(jQuery)
);

/*
 * jQuery Colour Related Palette Generator 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  color.object.js
 */
(jQuery.Color && (function($) {

// Generate a palette of related colours
$.Color.fn.related = function( offset ) {
	var i18n = $.Color.fn.related.i18n,
		off = offset || $.Color.fn.related.offset,
		offD = Math.round(off * 360) + i18n.deg;
	
	return {
		'anal-': this.analogous(-off).setName(i18n.anal + ' -' + offD),
		'anal0': this.analogous().setName(i18n.orig),
		'anal+': this.analogous(off).setName(i18n.anal + ' +' + offD),
		
		'comp-': this.complementary(-off).setName(i18n.split + ' -' + offD),
		'comp0': this.complementary().setName(i18n.comp),
		'comp+': this.complementary(off).setName(i18n.split + ' +' + offD),
		
		'triad-': this.analogous(-1/3).setName(i18n.triad + ' -120' + i18n.deg),
		'triad0': this.analogous().setName(i18n.orig),
		'triad+': this.analogous(1/3).setName(i18n.triad + ' +120' + i18n.deg)
	};
};

$.Color.fn.related.offset = 30/360;

$.Color.fn.related.i18n = {
	'deg': '°',
	'anal': 'Analogous',
	'orig': 'Original',
	'split': 'Split Complementary',
	'comp': 'Complementary',
	'triad': 'Triadic'
};

})(jQuery)
);

/*
 * jQuery Colour SVG/X11/CSS3 Palette 0.6
 *
 * Copyright (c) 2009 Adaptavist.com
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  color.core.js
 */
(jQuery.color && (function($) {

$.color.palette.css3 = {
	aliceblue:				[240,248,255],
	antiquewhite:			[250,235,215],
	aqua:					[0,255,255],
	aquamarine:				[127,255,212],
	azure:					[240,255,255],
	beige:					[245,245,220],
	bisque:					[255,228,196],
	black:					[0,0,0],
	blanchedalmond:			[255,235,205],
	blue:					[0,0,255],
	blueviolet:				[138,43,226],
	brown:					[165,42,42],
	burlywood:				[222,184,135],
	cadetblue:				[95,158,160],
	chartreuse:				[127,255,0],
	chocolate:				[210,105,30],
	coral:					[255,127,80],
	cornflowerblue:			[100,149,237],
	cornsilk:				[255,248,220],
	crimson:				[220,20,60],
	cyan:					[0,255,255],
	darkblue:				[0,0,139],
	darkcyan:				[0,139,139],
	darkgoldenrod:			[184,134,11],
	darkgray:				[169,169,169],
	darkgreen:				[0,100,0],
	darkgrey:				[169,169,169],
	darkkhaki:				[189,183,107],
	darkmagenta:			[139,0,139],
	darkolivegreen:			[85,107,47],
	darkorange:				[255,140,0],
	darkorchid:				[153,50,204],
	darkred:				[139,0,0],
	darksalmon:				[233,150,122],
	darkseagreen:			[143,188,143],
	darkslateblue:			[72,61,139],
	darkslategray:			[47,79,79],
	darkslategrey:			[47,79,79],
	darkturquoise:			[0,206,209],
	darkviolet:				[148,0,211],
	deeppink:				[255,20,147],
	deepskyblue:			[0,191,255],
	dimgray:				[105,105,105],
	dimgrey:				[105,105,105],
	dodgerblue:				[30,144,255],
	firebrick:				[178,34,34],
	floralwhite:			[255,250,240],
	forestgreen:			[34,139,34],
	fuchsia:				[255,0,255],
	gainsboro:				[220,220,220],
	ghostwhite:				[248,248,255],
	gold:					[255,215,0],
	goldenrod:				[218,165,32],
	gray:					[128,128,128],
	grey:					[128,128,128],
	green:					[0,128,0],
	greenyellow:			[173,255,47],
	honeydew:				[240,255,240],
	hotpink:				[255,105,180],
	indianred:				[205,92,92],
	indigo:					[75,0,130],
	ivory:					[255,255,240],
	khaki:					[240,230,140],
	lavender:				[230,230,250],
	lavenderblush:			[255,240,245],
	lawngreen:				[124,252,0],
	lemonchiffon:			[255,250,205],
	lightblue:				[173,216,230],
	lightcoral:				[240,128,128],
	lightcyan:				[224,255,255],
	lightgoldenrodyellow:	[250,250,210],
	lightgray:				[211,211,211],
	lightgreen:				[144,238,144],
	lightgrey:				[211,211,211],
	lightpink:				[255,182,193],
	lightsalmon:			[255,160,122],
	lightseagreen:			[32,178,170],
	lightskyblue:			[135,206,250],
	lightslategray:			[119,136,153],
	lightslategrey:			[119,136,153],
	lightsteelblue:			[176,196,222],
	lightyellow:			[255,255,224],
	lime:					[0,255,0],
	limegreen:				[50,205,50],
	linen:					[250,240,230],
	magenta:				[255,0,255],
	maroon:					[128,0,0],
	mediumaquamarine:		[102,205,170],
	mediumblue:				[0,0,205],
	mediumorchid:			[186,85,211],
	mediumpurple:			[147,112,219],
	mediumseagreen:			[60,179,113],
	mediumslateblue:		[123,104,238],
	mediumspringgreen:		[0,250,154],
	mediumturquoise:		[72,209,204],
	mediumvioletred:		[199,21,133],
	midnightblue:			[25,25,112],
	mintcream:				[245,255,250],
	mistyrose:				[255,228,225],
	moccasin:				[255,228,181],
	navajowhite:			[255,222,173],
	navy:					[0,0,128],
	oldlace:				[253,245,230],
	olive:					[128,128,0],
	olivedrab:				[107,142,35],
	orange:					[255,165,0],
	orangered:				[255,69,0],
	orchid:					[218,112,214],
	palegoldenrod:			[238,232,170],
	palegreen:				[152,251,152],
	paleturquoise:			[175,238,238],
	palevioletred:			[219,112,147],
	papayawhip:				[255,239,213],
	peachpuff:				[255,218,185],
	peru:					[205,133,63],
	pink:					[255,192,203],
	plum:					[221,160,221],
	powderblue:				[176,224,230],
	purple:					[128,0,128],
	red:					[255,0,0],
	rosybrown:				[188,143,143],
	royalblue:				[65,105,225],
	saddlebrown:			[139,69,19],
	salmon:					[250,128,114],
	sandybrown:				[244,164,96],
	seagreen:				[46,139,87],
	seashell:				[255,245,238],
	sienna:					[160,82,45],
	silver:					[192,192,192],
	skyblue:				[135,206,235],
	slateblue:				[106,90,205],
	slategray:				[112,128,144],
	slategrey:				[112,128,144],
	snow:					[255,250,250],
	springgreen:			[0,255,127],
	steelblue:				[70,130,180],
	tan:					[210,180,140],
	teal:					[0,128,128],
	thistle:				[216,191,216],
	tomato:					[255,99,71],
	turquoise:				[64,224,208],
	violet:					[238,130,238],
	wheat:					[245,222,179],
	white:					[255,255,255],
	whitesmoke:				[245,245,245],
	yellow:					[255,255,0],
	yellowgreen:			[154,205,50]
};

})(jQuery)
);

/*
 * jQuery CSS Colour Manipulation 0.6
 *
 * Copyright (c) 2010 Mark Gibson
 * Dual licensed under the MIT (MIT-LICENSE.txt)
 * and GPL (GPL-LICENSE.txt) licenses.
 *
 * Depends:
 *  color.core.js
 *  color.object.js
 */
(function($) {

	// Extract a CSS colour property as a Color object from the selection
	$.fn.cssColor = function(prop) {
		return $.Color(this.css(prop));
	};

	// Apply the colour to a CSS property on the selection
	$.Color.fn.applyCSS = function(selector, prop) {
		$(selector).css(prop, this.toCSS());
		return this;
	};

})(jQuery);

/*jslint browser: true */ /*global jQuery: true */

/**
 * jQuery Cookie plugin
 *
 * Copyright (c) 2010 Klaus Hartl (stilbuero.de)
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 *
 */

// TODO JsDoc

/**
 * Create a cookie with the given key and value and other optional parameters.
 *
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Set the value of a cookie.
 * @example $.cookie('the_cookie', 'the_value', { expires: 7, path: '/', domain: 'jquery.com', secure: true });
 * @desc Create a cookie with all available options.
 * @example $.cookie('the_cookie', 'the_value');
 * @desc Create a session cookie.
 * @example $.cookie('the_cookie', null);
 * @desc Delete a cookie by passing null as value. Keep in mind that you have to use the same path and domain
 *       used when the cookie was set.
 *
 * @param String key The key of the cookie.
 * @param String value The value of the cookie.
 * @param Object options An object literal containing key/value pairs to provide optional cookie attributes.
 * @option Number|Date expires Either an integer specifying the expiration date from now on in days or a Date object.
 *                             If a negative value is specified (e.g. a date in the past), the cookie will be deleted.
 *                             If set to null or omitted, the cookie will be a session cookie and will not be retained
 *                             when the the browser exits.
 * @option String path The value of the path atribute of the cookie (default: path of page that created the cookie).
 * @option String domain The value of the domain attribute of the cookie (default: domain of page that created the cookie).
 * @option Boolean secure If true, the secure attribute of the cookie will be set and the cookie transmission will
 *                        require a secure protocol (like HTTPS).
 * @type undefined
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */

/**
 * Get the value of a cookie with the given key.
 *
 * @example $.cookie('the_cookie');
 * @desc Get the value of a cookie.
 *
 * @param String key The key of the cookie.
 * @return The value of the cookie.
 * @type String
 *
 * @name $.cookie
 * @cat Plugins/Cookie
 * @author Klaus Hartl/klaus.hartl@stilbuero.de
 */
jQuery.cookie = function (key, value, options) {
    
    // key and at least value given, set cookie...
    if (arguments.length > 1 && String(value) !== "[object Object]") {
        options = jQuery.extend({}, options);

        if (value === null || value === undefined) {
            options.expires = -1;
        }

        if (typeof options.expires === 'number') {
            var days = options.expires, t = options.expires = new Date();
            t.setDate(t.getDate() + days);
        }
        
        value = String(value);
        
        return (document.cookie = [
            encodeURIComponent(key), '=',
            options.raw ? value : encodeURIComponent(value),
            options.expires ? '; expires=' + options.expires.toUTCString() : '', // use expires attribute, max-age is not supported by IE
            options.path ? '; path=' + options.path : '',
            options.domain ? '; domain=' + options.domain : '',
            options.secure ? '; secure' : ''
        ].join(''));
    }

    // key and possibly options given, get cookie...
    options = value || {};
    var result, decode = options.raw ? function (s) { return s; } : decodeURIComponent;
    return (result = new RegExp('(?:^|; )' + encodeURIComponent(key) + '=([^;]*)').exec(document.cookie)) ? decode(result[1]) : null;
};
(function(jQuery) {
	
	$.cookiesenabled = function(){
		$.cookie("cookiesenabled", "true");
		
		if( $.cookie("cookiesenabled") == "true" ){
			$.cookie("cookiesenabled", null);
			return true;
		} else {
			return false;
		}
	};
	
})(jQuery);/*
 * jquery.layout 1.2.0
 *
 * Copyright (c) 2008 
 *   Fabrizio Balliano (http://www.fabrizioballiano.net)
 *   Kevin Dalman (http://allpro.net)
 *
 * Dual licensed under the GPL (http://www.gnu.org/licenses/gpl.html)
 * and MIT (http://www.opensource.org/licenses/mit-license.php) licenses.
 *
 * $Date: 2008-12-27 02:17:22 +0100 (sab, 27 dic 2008) $
 * $Rev: 203 $
 * 
 * NOTE: For best code readability, view this with a fixed-space font and tabs equal to 4-chars
 */
(function($) {

$.fn.layout = function (opts) {

/*
 * ###########################
 *   WIDGET CONFIG & OPTIONS
 * ###########################
 */

	// DEFAULTS for options
	var 
		prefix = "ui-layout-" // prefix for ALL selectors and classNames
	,	defaults = { //	misc default values
			paneClass:				prefix+"pane"		// ui-layout-pane
		,	resizerClass:			prefix+"resizer"	// ui-layout-resizer
		,	togglerClass:			prefix+"toggler"	// ui-layout-toggler
		,	togglerInnerClass:		prefix+""			// ui-layout-open / ui-layout-closed
		,	buttonClass:			prefix+"button"		// ui-layout-button
		,	contentSelector:		"."+prefix+"content"// ui-layout-content
		,	contentIgnoreSelector:	"."+prefix+"ignore"	// ui-layout-mask 
		}
	;

	// DEFAULT PANEL OPTIONS - CHANGE IF DESIRED
	var options = {
		name:						""			// FUTURE REFERENCE - not used right now
	,	scrollToBookmarkOnLoad:		true		// after creating a layout, scroll to bookmark in URL (.../page.htm#myBookmark)
	,	defaults: { // default options for 'all panes' - will be overridden by 'per-pane settings'
			applyDefaultStyles: 	false		// apply basic styles directly to resizers & buttons? If not, then stylesheet must handle it
		,	closable:				true		// pane can open & close
		,	resizable:				true		// when open, pane can be resized 
		,	slidable:				true		// when closed, pane can 'slide' open over other panes - closes on mouse-out
		//,	paneSelector:			[ ]			// MUST be pane-specific!
		,	contentSelector:		defaults.contentSelector	// INNER div/element to auto-size so only it scrolls, not the entire pane!
		,	contentIgnoreSelector:	defaults.contentIgnoreSelector	// elem(s) to 'ignore' when measuring 'content'
		,	paneClass:				defaults.paneClass		// border-Pane - default: 'ui-layout-pane'
		,	resizerClass:			defaults.resizerClass	// Resizer Bar		- default: 'ui-layout-resizer'
		,	togglerClass:			defaults.togglerClass	// Toggler Button	- default: 'ui-layout-toggler'
		,	buttonClass:			defaults.buttonClass	// CUSTOM Buttons	- default: 'ui-layout-button-toggle/-open/-close/-pin'
		,	resizerDragOpacity:		1			// option for ui.draggable
		//,	resizerCursor:			""			// MUST be pane-specific - cursor when over resizer-bar
		,	maskIframesOnResize:	true		// true = all iframes OR = iframe-selector(s) - adds masking-div during resizing/dragging
		//,	size:					100			// inital size of pane - defaults are set 'per pane'
		,	minSize:				0			// when manually resizing a pane
		,	maxSize:				0			// ditto, 0 = no limit
		,	spacing_open:			6			// space between pane and adjacent panes - when pane is 'open'
		,	spacing_closed:			6			// ditto - when pane is 'closed'
		,	togglerLength_open:		50			// Length = WIDTH of toggler button on north/south edges - HEIGHT on east/west edges
		,	togglerLength_closed: 	50			// 100% OR -1 means 'full height/width of resizer bar' - 0 means 'hidden'
		,	togglerAlign_open:		"center"	// top/left, bottom/right, center, OR...
		,	togglerAlign_closed:	"center"	// 1 => nn = offset from top/left, -1 => -nn == offset from bottom/right
		,	togglerTip_open:		"Close"		// Toggler tool-tip (title)
		,	togglerTip_closed:		"Open"		// ditto
		,	resizerTip:				"Resize"	// Resizer tool-tip (title)
		,	sliderTip:				"Slide Open" // resizer-bar triggers 'sliding' when pane is closed
		,	sliderCursor:			"pointer"	// cursor when resizer-bar will trigger 'sliding'
		,	slideTrigger_open:		"click"		// click, dblclick, mouseover
		,	slideTrigger_close:		"mouseout"	// click, mouseout
		,	hideTogglerOnSlide:		false		// when pane is slid-open, should the toggler show?
		,	togglerContent_open:	""			// text or HTML to put INSIDE the toggler
		,	togglerContent_closed:	""			// ditto
		,	showOverflowOnHover:	false		// will bind allowOverflow() utility to pane.onMouseOver
		,	enableCursorHotkey:		true		// enabled 'cursor' hotkeys
		//,	customHotkey:			""			// MUST be pane-specific - EITHER a charCode OR a character
		,	customHotkeyModifier:	"SHIFT"		// either 'SHIFT', 'CTRL' or 'CTRL+SHIFT' - NOT 'ALT'
		//	NOTE: fxSss_open & fxSss_close options (eg: fxName_open) are auto-generated if not passed
		,	fxName:					"slide" 	// ('none' or blank), slide, drop, scale
		,	fxSpeed:				null		// slow, normal, fast, 200, nnn - if passed, will OVERRIDE fxSettings.duration
		,	fxSettings:				{}			// can be passed, eg: { easing: "easeOutBounce", duration: 1500 }
		,	initClosed:				false		// true = init pane as 'closed'
		,	initHidden: 			false 		// true = init pane as 'hidden' - no resizer or spacing
		
		/*	callback options do not have to be set - listed here for reference only
		,	onshow_start:			""			// CALLBACK when pane STARTS to Show	- BEFORE onopen/onhide_start
		,	onshow_end:				""			// CALLBACK when pane ENDS being Shown	- AFTER  onopen/onhide_end
		,	onhide_start:			""			// CALLBACK when pane STARTS to Close	- BEFORE onclose_start
		,	onhide_end:				""			// CALLBACK when pane ENDS being Closed	- AFTER  onclose_end
		,	onopen_start:			""			// CALLBACK when pane STARTS to Open
		,	onopen_end:				""			// CALLBACK when pane ENDS being Opened
		,	onclose_start:			""			// CALLBACK when pane STARTS to Close
		,	onclose_end:			""			// CALLBACK when pane ENDS being Closed
		,	onresize_start:			""			// CALLBACK when pane STARTS to be ***MANUALLY*** Resized
		,	onresize_end:			""			// CALLBACK when pane ENDS being Resized ***FOR ANY REASON***
		*/
		}
	,	north: {
			paneSelector:			"."+prefix+"north" // default = .ui-layout-north
		,	size:					"auto"
		,	resizerCursor:			"n-resize"
		}
	,	south: {
			paneSelector:			"."+prefix+"south" // default = .ui-layout-south
		,	size:					"auto"
		,	resizerCursor:			"s-resize"
		}
	,	east: {
			paneSelector:			"."+prefix+"east" // default = .ui-layout-east
		,	size:					200
		,	resizerCursor:			"e-resize"
		}
	,	west: {
			paneSelector:			"."+prefix+"west" // default = .ui-layout-west
		,	size:					200
		,	resizerCursor:			"w-resize"
		}
	,	center: {
			paneSelector:			"."+prefix+"center" // default = .ui-layout-center
		}

	};


	var effects = { // LIST *PREDEFINED EFFECTS* HERE, even if effect has no settings
		slide:	{
			all:	{ duration:  "fast"	} // eg: duration: 1000, easing: "easeOutBounce"
		,	north:	{ direction: "up"	}
		,	south:	{ direction: "down"	}
		,	east:	{ direction: "right"}
		,	west:	{ direction: "left"	}
		}
	,	drop:	{
			all:	{ duration:  "slow"	} // eg: duration: 1000, easing: "easeOutQuint"
		,	north:	{ direction: "up"	}
		,	south:	{ direction: "down"	}
		,	east:	{ direction: "right"}
		,	west:	{ direction: "left"	}
		}
	,	scale:	{
			all:	{ duration:  "fast"	}
		}
	};


	// STATIC, INTERNAL CONFIG - DO NOT CHANGE THIS!
	var config = {
		allPanes:		"north,south,east,west,center"
	,	borderPanes:	"north,south,east,west"
	,	zIndex: { // set z-index values here
			resizer_normal:	1		// normal z-index for resizer-bars
		,	pane_normal:	2		// normal z-index for panes
		,	mask:			4		// overlay div used to mask pane(s) during resizing
		,	sliding:		100		// applied to both the pane and its resizer when a pane is 'slid open'
		,	resizing:		10000	// applied to the CLONED resizer-bar when being 'dragged'
		,	animation:		10000	// applied to the pane when being animated - not applied to the resizer
		}
	,	resizers: {
			cssReq: {
				position: 	"absolute"
			,	padding: 	0
			,	margin: 	0
			,	fontSize:	"1px"
			,	textAlign:	"left" // to counter-act "center" alignment!
			,	overflow: 	"hidden" // keep toggler button from overflowing
			,	zIndex: 	1
			}
		,	cssDef: { // DEFAULT CSS - applied if: options.PANE.applyDefaultStyles=true
				background: "#DDD"
			,	border:		"none"
			}
		}
	,	togglers: {
			cssReq: {
				position: 	"absolute"
			,	display: 	"block"
			,	padding: 	0
			,	margin: 	0
			,	overflow:	"hidden"
			,	textAlign:	"center"
			,	fontSize:	"1px"
			,	cursor: 	"pointer"
			,	zIndex: 	1
			}
		,	cssDef: { // DEFAULT CSS - applied if: options.PANE.applyDefaultStyles=true
				background: "#AAA"
			}
		}
	,	content: {
			cssReq: {
				overflow:	"auto"
			}
		,	cssDef: {}
		}
	,	defaults: { // defaults for ALL panes - overridden by 'per-pane settings' below
			cssReq: {
				position: 	"absolute"
			,	margin:		0
			,	zIndex: 	2
			}
		,	cssDef: {
				padding:	"10px"
			,	background:	"#FFF"
			,	border:		"1px solid #BBB"
			,	overflow:	"auto"
			}
		}
	,	north: {
			edge:			"top"
		,	sizeType:		"height"
		,	dir:			"horz"
		,	cssReq: {
				top: 		0
			,	bottom: 	"auto"
			,	left: 		0
			,	right: 		0
			,	width: 		"auto"
			//	height: 	DYNAMIC
			}
		}
	,	south: {
			edge:			"bottom"
		,	sizeType:		"height"
		,	dir:			"horz"
		,	cssReq: {
				top: 		"auto"
			,	bottom: 	0
			,	left: 		0
			,	right: 		0
			,	width: 		"auto"
			//	height: 	DYNAMIC
			}
		}
	,	east: {
			edge:			"right"
		,	sizeType:		"width"
		,	dir:			"vert"
		,	cssReq: {
				left: 		"auto"
			,	right: 		0
			,	top: 		"auto" // DYNAMIC
			,	bottom: 	"auto" // DYNAMIC
			,	height: 	"auto"
			//	width: 		DYNAMIC
			}
		}
	,	west: {
			edge:			"left"
		,	sizeType:		"width"
		,	dir:			"vert"
		,	cssReq: {
				left: 		0
			,	right: 		"auto"
			,	top: 		"auto" // DYNAMIC
			,	bottom: 	"auto" // DYNAMIC
			,	height: 	"auto"
			//	width: 		DYNAMIC
			}
		}
	,	center: {
			dir:			"center"
		,	cssReq: {
				left: 		"auto" // DYNAMIC
			,	right: 		"auto" // DYNAMIC
			,	top: 		"auto" // DYNAMIC
			,	bottom: 	"auto" // DYNAMIC
			,	height: 	"auto"
			,	width: 		"auto"
			}
		}
	};


	// DYNAMIC DATA
	var state = {
		// generate random 'ID#' to identify layout - used to create global namespace for timers
		id:			Math.floor(Math.random() * 10000)
	,	container:	{}
	,	north:		{}
	,	south:		{}
	,	east:		{}
	,	west:		{}
	,	center:		{}
	};


	var 
		altEdge = {
			top:	"bottom"
		,	bottom: "top"
		,	left:	"right"
		,	right:	"left"
		}
	,	altSide = {
			north:	"south"
		,	south:	"north"
		,	east: 	"west"
		,	west: 	"east"
		}
	;


/*
 * ###########################
 *  INTERNAL HELPER FUNCTIONS
 * ###########################
 */

	/**
	 * isStr
	 *
	 * Returns true if passed param is EITHER a simple string OR a 'string object' - otherwise returns false
	 */
	var isStr = function (o) {
		if (typeof o == "string")
			return true;
		else if (typeof o == "object") {
			try {
				var match = o.constructor.toString().match(/string/i); 
				return (match !== null);
			} catch (e) {} 
		}
		return false;
	};

	/**
	 * str
	 *
	 * Returns a simple string if the passed param is EITHER a simple string OR a 'string object',
	 *  else returns the original object
	 */
	var str = function (o) {
		if (typeof o == "string" || isStr(o)) return $.trim(o); // trim converts 'String object' to a simple string
		else return o;
	};

	/**
	 * min / max
	 *
	 * Alias for Math.min/.max to simplify coding
	 */
	var min = function (x,y) { return Math.min(x,y); };
	var max = function (x,y) { return Math.max(x,y); };

	/**
	 * transformData
	 *
	 * Processes the options passed in and transforms them into the format used by layout()
	 * Missing keys are added, and converts the data if passed in 'flat-format' (no sub-keys)
	 * In flat-format, pane-specific-settings are prefixed like: north__optName  (2-underscores)
	 * To update effects, options MUST use nested-keys format, with an effects key
	 *
	 * @callers  initOptions()
	 * @params  JSON  d  Data/options passed by user - may be a single level or nested levels
	 * @returns JSON  Creates a data struture that perfectly matches 'options', ready to be imported
	 */
	var transformData = function (d) {
		var json = { defaults:{fxSettings:{}}, north:{fxSettings:{}}, south:{fxSettings:{}}, east:{fxSettings:{}}, west:{fxSettings:{}}, center:{fxSettings:{}} };
		d = d || {};
		if (d.effects || d.defaults || d.north || d.south || d.west || d.east || d.center)
			json = $.extend( json, d ); // already in json format - add to base keys
		else
			// convert 'flat' to 'nest-keys' format - also handles 'empty' user-options
			$.each( d, function (key,val) {
				a = key.split("__");
				json[ a[1] ? a[0] : "defaults" ][ a[1] ? a[1] : a[0] ] = val;
			});
		return json;
	};

	/**
	 * setFlowCallback
	 *
	 * Set an INTERNAL callback to avoid simultaneous animation
	 * Runs only if needed and only if all callbacks are not 'already set'!
	 *
	 * @param String   action  Either 'open' or 'close'
	 * @pane  String   pane    A valid border-pane name, eg 'west'
	 * @pane  Boolean  param   Extra param for callback (optional)
	 */
	var setFlowCallback = function (action, pane, param) {
		var
			cb = action +","+ pane +","+ (param ? 1 : 0)
		,	cP, cbPane
		;
		$.each(c.borderPanes.split(","), function (i,p) {
			if (c[p].isMoving) {
				bindCallback(p); // TRY to bind a callback
				return false; // BREAK
			}
		});

		function bindCallback (p, test) {
			cP = c[p];
			if (!cP.doCallback) {
				cP.doCallback = true;
				cP.callback = cb;
			}
			else { // try to 'chain' this callback
				cpPane = cP.callback.split(",")[1]; // 2nd param is 'pane'
				if (cpPane != p && cpPane != pane) // callback target NOT 'itself' and NOT 'this pane'
					bindCallback (cpPane, true); // RECURSE
			}
		}
	};

	/**
	 * execFlowCallback
	 *
	 * RUN the INTERNAL callback for this pane - if one exists
	 *
	 * @param String   action  Either 'open' or 'close'
	 * @pane  String   pane    A valid border-pane name, eg 'west'
	 * @pane  Boolean  param   Extra param for callback (optional)
	 */
	var execFlowCallback = function (pane) {
		var cP = c[pane];

		// RESET flow-control flaGs
		c.isLayoutBusy = false;
		delete cP.isMoving;
		if (!cP.doCallback || !cP.callback) return;

		cP.doCallback = false; // RESET logic flag

		// EXECUTE the callback
		var
			cb = cP.callback.split(",")
		,	param = (cb[2] > 0 ? true : false)
		;
		if (cb[0] == "open")
			open( cb[1], param  );
		else if (cb[0] == "close")
			close( cb[1], param );

		if (!cP.doCallback) cP.callback = null; // RESET - unless callback above enabled it again!
	};

	/**
	 * execUserCallback
	 *
	 * Executes a Callback function after a trigger event, like resize, open or close
	 *
	 * @param String  pane   This is passed only so we can pass the 'pane object' to the callback
	 * @param String  v_fn  Accepts a function name, OR a comma-delimited array: [0]=function name, [1]=argument
	 */
	var execUserCallback = function (pane, v_fn) {
		if (!v_fn) return;
		var fn;
		try {
			if (typeof v_fn == "function")
				fn = v_fn;	
			else if (typeof v_fn != "string")
				return;
			else if (v_fn.indexOf(",") > 0) {
				// function name cannot contain a comma, so must be a function name AND a 'name' parameter
				var
					args = v_fn.split(",")
				,	fn = eval(args[0])
				;
				if (typeof fn=="function" && args.length > 1)
					return fn(args[1]); // pass the argument parsed from 'list'
			}
			else // just the name of an external function?
				fn = eval(v_fn);

			if (typeof fn=="function")
				// pass data: pane-name, pane-element, pane-state, pane-options, and layout-name
				return fn( pane, $Ps[pane], $.extend({},state[pane]), $.extend({},options[pane]), options.name );
		}
		catch (ex) {}
	};

	/**
	 * cssNum
	 *
	 * Returns the 'current CSS value' for an element - returns 0 if property does not exist
	 *
	 * @callers  Called by many methods
	 * @param jQuery  $Elem  Must pass a jQuery object - first element is processed
	 * @param String  property  The name of the CSS property, eg: top, width, etc.
	 * @returns Variant  Usually is used to get an integer value for position (top, left) or size (height, width)
	 */
	var cssNum = function ($E, prop) {
		var
			val = 0
		,	hidden = false
		,	visibility = ""
		;
		if (!$.browser.msie) { // IE CAN read dimensions of 'hidden' elements - FF CANNOT
			if ($.curCSS($E[0], "display", true) == "none") {
				hidden = true;
				visibility = $.curCSS($E[0], "visibility", true); // SAVE current setting
				$E.css({ display: "block", visibility: "hidden" }); // show element 'invisibly' so we can measure it
			}
		}

		val = parseInt($.curCSS($E[0], prop, true), 10) || 0;

		if (hidden) { // WAS hidden, so put back the way it was
			$E.css({ display: "none" });
			if (visibility && visibility != "hidden")
				$E.css({ visibility: visibility }); // reset 'visibility'
		}

		return val;
	};

	/**
	 * cssW / cssH / cssSize
	 *
	 * Contains logic to check boxModel & browser, and return the correct width/height for the current browser/doctype
	 *
	 * @callers  initPanes(), sizeMidPanes(), initHandles(), sizeHandles()
	 * @param Variant  elem  Can accept a 'pane' (east, west, etc) OR a DOM object OR a jQuery object
	 * @param Integer  outerWidth/outerHeight  (optional) Can pass a width, allowing calculations BEFORE element is resized
	 * @returns Integer  Returns the innerHeight of the elem by subtracting padding and borders
	 *
	 * @TODO  May need to add additional logic to handle more browser/doctype variations?
	 */
	var cssW = function (e, outerWidth) {
		var $E;
		if (isStr(e)) {
			e = str(e);
			$E = $Ps[e];
		}
		else
			$E = $(e);

		// a 'calculated' outerHeight can be passed so borders and/or padding are removed if needed
		if (outerWidth <= 0)
			return 0;
		else if (!(outerWidth>0))
			outerWidth = isStr(e) ? getPaneSize(e) : $E.outerWidth();

		if (!$.boxModel)
			return outerWidth;

		else // strip border and padding size from outerWidth to get CSS Width
			return outerWidth
				- cssNum($E, "paddingLeft")		
				- cssNum($E, "paddingRight")
				- ($.curCSS($E[0], "borderLeftStyle", true) == "none" ? 0 : cssNum($E, "borderLeftWidth"))
				- ($.curCSS($E[0], "borderRightStyle", true) == "none" ? 0 : cssNum($E, "borderRightWidth"))
			;
	};
	var cssH = function (e, outerHeight) {
		var $E;
		if (isStr(e)) {
			e = str(e);
			$E = $Ps[e];
		}
		else
			$E = $(e);

		// a 'calculated' outerHeight can be passed so borders and/or padding are removed if needed
		if (outerHeight <= 0)
			return 0;
		else if (!(outerHeight>0))
			outerHeight = (isStr(e)) ? getPaneSize(e) : $E.outerHeight();

		if (!$.boxModel)
			return outerHeight;

		else // strip border and padding size from outerHeight to get CSS Height
			return outerHeight
				- cssNum($E, "paddingTop")
				- cssNum($E, "paddingBottom")
				- ($.curCSS($E[0], "borderTopStyle", true) == "none" ? 0 : cssNum($E, "borderTopWidth"))
				- ($.curCSS($E[0], "borderBottomStyle", true) == "none" ? 0 : cssNum($E, "borderBottomWidth"))
			;
	};
	var cssSize = function (pane, outerSize) {
		if (c[pane].dir=="horz") // pane = north or south
			return cssH(pane, outerSize);
		else // pane = east or west
			return cssW(pane, outerSize);
	};

	/**
	 * getPaneSize
	 *
	 * Calculates the current 'size' (width or height) of a border-pane - optionally with 'pane spacing' added
	 *
	 * @returns Integer  Returns EITHER Width for east/west panes OR Height for north/south panes - adjusted for boxModel & browser
	 */
	var getPaneSize = function (pane, inclSpace) {
		var 
			$P	= $Ps[pane]
		,	o	= options[pane]
		,	s	= state[pane]
		,	oSp	= (inclSpace ? o.spacing_open : 0)
		,	cSp	= (inclSpace ? o.spacing_closed : 0)
		;
		if (!$P || s.isHidden)
			return 0;
		else if (s.isClosed || (s.isSliding && inclSpace))
			return cSp;
		else if (c[pane].dir == "horz")
			return $P.outerHeight() + oSp;
		else // dir == "vert"
			return $P.outerWidth() + oSp;
	};

	var setPaneMinMaxSizes = function (pane) {
		var 
			d				= cDims
		,	edge			= c[pane].edge
		,	dir				= c[pane].dir
		,	o				= options[pane]
		,	s				= state[pane]
		,	$P				= $Ps[pane]
		,	$altPane		= $Ps[ altSide[pane] ]
		,	paneSpacing		= o.spacing_open
		,	altPaneSpacing	= options[ altSide[pane] ].spacing_open
		,	altPaneSize		= (!$altPane ? 0 : (dir=="horz" ? $altPane.outerHeight() : $altPane.outerWidth()))
		,	containerSize	= (dir=="horz" ? d.innerHeight : d.innerWidth)
		//	limitSize prevents this pane from 'overlapping' opposite pane - even if opposite pane is currently closed
		,	limitSize		= containerSize - paneSpacing - altPaneSize - altPaneSpacing
		,	minSize			= s.minSize || 0
		,	maxSize			= Math.min(s.maxSize || 9999, limitSize)
		,	minPos, maxPos	// used to set resizing limits
		;
		switch (pane) {
			case "north":	minPos = d.offsetTop + minSize;
							maxPos = d.offsetTop + maxSize;
							break;
			case "west":	minPos = d.offsetLeft + minSize;
							maxPos = d.offsetLeft + maxSize;
							break;
			case "south":	minPos = d.offsetTop + d.innerHeight - maxSize;
							maxPos = d.offsetTop + d.innerHeight - minSize;
							break;
			case "east":	minPos = d.offsetLeft + d.innerWidth - maxSize;
							maxPos = d.offsetLeft + d.innerWidth - minSize;
							break;
		}
		// save data to pane-state
		$.extend(s, { minSize: minSize, maxSize: maxSize, minPosition: minPos, maxPosition: maxPos });
	};

	/**
	 * getPaneDims
	 *
	 * Returns data for setting the size/position of center pane. Date is also used to set Height for east/west panes
	 *
	 * @returns JSON  Returns a hash of all dimensions: top, bottom, left, right, (outer) width and (outer) height
	 */
	var getPaneDims = function () {
		var d = {
			top:	getPaneSize("north", true) // true = include 'spacing' value for p
		,	bottom:	getPaneSize("south", true)
		,	left:	getPaneSize("west", true)
		,	right:	getPaneSize("east", true)
		,	width:	0
		,	height:	0
		};

		with (d) {
			width 	= cDims.innerWidth - left - right;
			height 	= cDims.innerHeight - bottom - top;
			// now add the 'container border/padding' to get final positions - relative to the container
			top		+= cDims.top;
			bottom	+= cDims.bottom;
			left	+= cDims.left;
			right	+= cDims.right;
		}

		return d;
	};


	/**
	 * getElemDims
	 *
	 * Returns data for setting size of an element (container or a pane).
	 *
	 * @callers  create(), onWindowResize() for container, plus others for pane
	 * @returns JSON  Returns a hash of all dimensions: top, bottom, left, right, outerWidth, innerHeight, etc
	 */
	var getElemDims = function ($E) {
		var
			d = {} // dimensions hash
		,	e, b, p // edge, border, padding
		;

		$.each("Left,Right,Top,Bottom".split(","), function () {
			e = str(this);
			b = d["border" +e] = cssNum($E, "border"+e+"Width");
			p = d["padding"+e] = cssNum($E, "padding"+e);
			d["offset" +e] = b + p; // total offset of content from outer edge
			// if BOX MODEL, then 'position' = PADDING (ignore borderWidth)
			if ($E == $Container)
				d[e.toLowerCase()] = ($.boxModel ? p : 0); 
		});

		d.innerWidth  = d.outerWidth  = $E.outerWidth();
		d.innerHeight = d.outerHeight = $E.outerHeight();
		if ($.boxModel) {
			d.innerWidth  -= (d.offsetLeft + d.offsetRight);
			d.innerHeight -= (d.offsetTop  + d.offsetBottom);
		}

		return d;
	};


	var setTimer = function (pane, action, fn, ms) {
		var
			Layout = window.layout = window.layout || {}
		,	Timers = Layout.timers = Layout.timers || {}
		,	name = "layout_"+ state.id +"_"+ pane +"_"+ action // UNIQUE NAME for every layout-pane-action
		;
		if (Timers[name]) return; // timer already set!
		else Timers[name] = setTimeout(fn, ms);
	};

	var clearTimer = function (pane, action) {
		var
			Layout = window.layout = window.layout || {}
		,	Timers = Layout.timers = Layout.timers || {}
		,	name = "layout_"+ state.id +"_"+ pane +"_"+ action // UNIQUE NAME for every layout-pane-action
		;
		if (Timers[name]) {
			clearTimeout( Timers[name] );
			delete Timers[name];
			return true;
		}
		else
			return false;
	};


/*
 * ###########################
 *   INITIALIZATION METHODS
 * ###########################
 */

	/**
	 * create
	 *
	 * Initialize the layout - called automatically whenever an instance of layout is created
	 *
	 * @callers  NEVER explicity called
	 * @returns  An object pointer to the instance created
	 */
	var create = function () {
		// initialize config/options
		initOptions();

		// initialize all objects
		initContainer();	// set CSS as needed and init state.container dimensions
		initPanes();		// size & position all panes
		initHandles();		// create and position all resize bars & togglers buttons
		initResizable();	// activate resizing on all panes where resizable=true
		sizeContent("all");	// AFTER panes & handles have been initialized, size 'content' divs

		if (options.scrollToBookmarkOnLoad)
			with (self.location) if (hash) replace( hash ); // scrollTo Bookmark

		// bind hotkey function - keyDown - if required
		initHotkeys();

		// bind resizeAll() for 'this layout instance' to window.resize event
		$(window).resize(function () {
			var timerID = "timerLayout_"+state.id;
			if (window[timerID]) clearTimeout(window[timerID]);
			window[timerID] = null;
			if (true || $.browser.msie) // use a delay for IE because the resize event fires repeatly
				window[timerID] = setTimeout(resizeAll, 100);
			else // most other browsers have a built-in delay before firing the resize event
				resizeAll(); // resize all layout elements NOW!
		});
	};

	/**
	 * initContainer
	 *
	 * Validate and initialize container CSS and events
	 *
	 * @callers  create()
	 */
	var initContainer = function () {
		try { // format html/body if this is a full page layout
			if ($Container[0].tagName == "BODY") {
				$("html").css({
					height:		"100%"
				,	overflow:	"hidden"
				});
				$("body").css({
					position:	"relative"
				,	height:		"100%"
				,	overflow:	"hidden"
				,	margin:		0
				,	padding:	0		// TODO: test whether body-padding could be handled?
				,	border:		"none"	// a body-border creates problems because it cannot be measured!
				});
			}
			else { // set required CSS - overflow and position
				var
					CSS	= { overflow: "hidden" } // make sure container will not 'scroll'
				,	p	= $Container.css("position")
				,	h	= $Container.css("height")
				;
				// if this is a NESTED layout, then outer-pane ALREADY has position and height
				if (!$Container.hasClass("ui-layout-pane")) {
					if (!p || "fixed,absolute,relative".indexOf(p) < 0)
						CSS.position = "relative"; // container MUST have a 'position'
					if (!h || h=="auto")
						CSS.height = "100%"; // container MUST have a 'height'
				}
				$Container.css( CSS );
			}
		} catch (ex) {}

		// get layout-container dimensions (updated when necessary)
		cDims = state.container = getElemDims( $Container ); // update data-pointer too
	};

	/**
	 * initHotkeys
	 *
	 * Bind layout hotkeys - if options enabled
	 *
	 * @callers  create()
	 */
	var initHotkeys = function () {
		// bind keyDown to capture hotkeys, if option enabled for ANY pane
		$.each(c.borderPanes.split(","), function (i,pane) {
			var o = options[pane];
			if (o.enableCursorHotkey || o.customHotkey) {
				$(document).keydown( keyDown ); // only need to bind this ONCE
				return false; // BREAK - binding was done
			}
		});
	};

	/**
	 * initOptions
	 *
	 * Build final CONFIG and OPTIONS data
	 *
	 * @callers  create()
	 */
	var initOptions = function () {
		// simplify logic by making sure passed 'opts' var has basic keys
		opts = transformData( opts );

		// update default effects, if case user passed key
		if (opts.effects) {
			$.extend( effects, opts.effects );
			delete opts.effects;
		}

		// see if any 'global options' were specified
		$.each("name,scrollToBookmarkOnLoad".split(","), function (idx,key) {
			if (opts[key] !== undefined)
				options[key] = opts[key];
			else if (opts.defaults[key] !== undefined) {
				options[key] = opts.defaults[key];
				delete opts.defaults[key];
			}
		});

		// remove any 'defaults' that MUST be set 'per-pane'
		$.each("paneSelector,resizerCursor,customHotkey".split(","),
			function (idx,key) { delete opts.defaults[key]; } // is OK if key does not exist
		);

		// now update options.defaults
		$.extend( options.defaults, opts.defaults );
		// make sure required sub-keys exist
		//if (typeof options.defaults.fxSettings != "object") options.defaults.fxSettings = {};

		// merge all config & options for the 'center' pane
		c.center = $.extend( true, {}, c.defaults, c.center );
		$.extend( options.center, opts.center );
		// Most 'default options' do not apply to 'center', so add only those that DO
		var o_Center = $.extend( true, {}, options.defaults, opts.defaults, options.center ); // TEMP data
		$.each("paneClass,contentSelector,contentIgnoreSelector,applyDefaultStyles,showOverflowOnHover".split(","),
			function (idx,key) { options.center[key] = o_Center[key]; }
		);

		var defs = options.defaults;

		// create a COMPLETE set of options for EACH border-pane
		$.each(c.borderPanes.split(","), function(i,pane) {
			// apply 'pane-defaults' to CONFIG.PANE
			c[pane] = $.extend( true, {}, c.defaults, c[pane] );
			// apply 'pane-defaults' +  user-options to OPTIONS.PANE
			o = options[pane] = $.extend( true, {}, options.defaults, options[pane], opts.defaults, opts[pane] );

			// make sure we have base-classes
			if (!o.paneClass)		o.paneClass		= defaults.paneClass;
			if (!o.resizerClass)	o.resizerClass	= defaults.resizerClass;
			if (!o.togglerClass)	o.togglerClass	= defaults.togglerClass;

			// create FINAL fx options for each pane, ie: options.PANE.fxName/fxSpeed/fxSettings[_open|_close]
			$.each(["_open","_close",""], function (i,n) { 
				var
					sName		= "fxName"+n
				,	sSpeed		= "fxSpeed"+n
				,	sSettings	= "fxSettings"+n
				;
				// recalculate fxName according to specificity rules
				o[sName] =
					opts[pane][sName]		// opts.west.fxName_open
				||	opts[pane].fxName		// opts.west.fxName
				||	opts.defaults[sName]	// opts.defaults.fxName_open
				||	opts.defaults.fxName	// opts.defaults.fxName
				||	o[sName]				// options.west.fxName_open
				||	o.fxName				// options.west.fxName
				||	defs[sName]				// options.defaults.fxName_open
				||	defs.fxName				// options.defaults.fxName
				||	"none"
				;
				// validate fxName to be sure is a valid effect
				var fxName = o[sName];
				if (fxName == "none" || !$.effects || !$.effects[fxName] || (!effects[fxName] && !o[sSettings] && !o.fxSettings))
					fxName = o[sName] = "none"; // effect not loaded, OR undefined FX AND fxSettings not passed
				// set vars for effects subkeys to simplify logic
				var
					fx = effects[fxName]	|| {} // effects.slide
				,	fx_all	= fx.all		|| {} // effects.slide.all
				,	fx_pane	= fx[pane]		|| {} // effects.slide.west
				;
				// RECREATE the fxSettings[_open|_close] keys using specificity rules
				o[sSettings] = $.extend(
					{}
				,	fx_all						// effects.slide.all
				,	fx_pane						// effects.slide.west
				,	defs.fxSettings || {}		// options.defaults.fxSettings
				,	defs[sSettings] || {}		// options.defaults.fxSettings_open
				,	o.fxSettings				// options.west.fxSettings
				,	o[sSettings]				// options.west.fxSettings_open
				,	opts.defaults.fxSettings	// opts.defaults.fxSettings
				,	opts.defaults[sSettings] || {} // opts.defaults.fxSettings_open
				,	opts[pane].fxSettings		// opts.west.fxSettings
				,	opts[pane][sSettings] || {}	// opts.west.fxSettings_open
				);
				// recalculate fxSpeed according to specificity rules
				o[sSpeed] =
					opts[pane][sSpeed]		// opts.west.fxSpeed_open
				||	opts[pane].fxSpeed		// opts.west.fxSpeed (pane-default)
				||	opts.defaults[sSpeed]	// opts.defaults.fxSpeed_open
				||	opts.defaults.fxSpeed	// opts.defaults.fxSpeed
				||	o[sSpeed]				// options.west.fxSpeed_open
				||	o[sSettings].duration	// options.west.fxSettings_open.duration
				||	o.fxSpeed				// options.west.fxSpeed
				||	o.fxSettings.duration	// options.west.fxSettings.duration
				||	defs.fxSpeed			// options.defaults.fxSpeed
				||	defs.fxSettings.duration// options.defaults.fxSettings.duration
				||	fx_pane.duration		// effects.slide.west.duration
				||	fx_all.duration			// effects.slide.all.duration
				||	"normal"				// DEFAULT
				;
				// DEBUG: if (pane=="east") debugData( $.extend({}, {speed: o[sSpeed], fxSettings_duration: o[sSettings].duration}, o[sSettings]), pane+"."+sName+" = "+fxName );
			});
		});
	};

	/**
	 * initPanes
	 *
	 * Initialize module objects, styling, size and position for all panes
	 *
	 * @callers  create()
	 */
	var initPanes = function () {
		// NOTE: do north & south FIRST so we can measure their height - do center LAST
		$.each(c.allPanes.split(","), function() {
			var 
				pane	= str(this)
			,	o		= options[pane]
			,	s		= state[pane]
			,	fx		= s.fx
			,	dir		= c[pane].dir
			//	if o.size is not > 0, then we will use MEASURE the pane and use that as it's 'size'
			,	size	= o.size=="auto" || isNaN(o.size) ? 0 : o.size
			,	minSize	= o.minSize || 1
			,	maxSize	= o.maxSize || 9999
			,	spacing	= o.spacing_open || 0
			,	sel		= o.paneSelector
			,	isIE6	= ($.browser.msie && $.browser.version < 7)
			,	CSS		= {}
			,	$P, $C
			;
			$Cs[pane] = false; // init

			if (sel.substr(0,1)==="#") // ID selector
				// NOTE: elements selected 'by ID' DO NOT have to be 'children'
				$P = $Ps[pane] = $Container.find(sel+":first");
			else { // class or other selector
				$P = $Ps[pane] = $Container.children(sel+":first");
				// look for the pane nested inside a 'form' element
				if (!$P.length) $P = $Ps[pane] = $Container.children("form:first").children(sel+":first");
			}

			if (!$P.length) {
				$Ps[pane] = false; // logic
				return true; // SKIP to next
			}

			// add basic classes & attributes
			$P
				.attr("pane", pane) // add pane-identifier
				.addClass( o.paneClass +" "+ o.paneClass+"-"+pane ) // default = "ui-layout-pane ui-layout-pane-west" - may be a dupe of 'paneSelector'
			;

			// init pane-logic vars, etc.
			if (pane != "center") {
				s.isClosed  = false; // true = pane is closed
				s.isSliding = false; // true = pane is currently open by 'sliding' over adjacent panes
				s.isResizing= false; // true = pane is in process of being resized
				s.isHidden	= false; // true = pane is hidden - no spacing, resizer or toggler is visible!
				s.noRoom	= false; // true = pane 'automatically' hidden due to insufficient room - will unhide automatically
				// create special keys for internal use
				c[pane].pins = [];   // used to track and sync 'pin-buttons' for border-panes
			}

			CSS = $.extend({ visibility: "visible", display: "block" }, c.defaults.cssReq, c[pane].cssReq );
			if (o.applyDefaultStyles) $.extend( CSS, c.defaults.cssDef, c[pane].cssDef ); // cosmetic defaults
			$P.css(CSS); // add base-css BEFORE 'measuring' to calc size & position
			CSS = {};	// reset var

			// set css-position to account for container borders & padding
			switch (pane) {
				case "north": 	CSS.top 	= cDims.top;
								CSS.left 	= cDims.left;
								CSS.right	= cDims.right;
								break;
				case "south": 	CSS.bottom	= cDims.bottom;
								CSS.left 	= cDims.left;
								CSS.right 	= cDims.right;
								break;
				case "west": 	CSS.left 	= cDims.left; // top, bottom & height set by sizeMidPanes()
								break;
				case "east": 	CSS.right 	= cDims.right; // ditto
								break;
				case "center":	// top, left, width & height set by sizeMidPanes()
			}

			if (dir == "horz") { // north or south pane
				if (size === 0 || size == "auto") {
					$P.css({ height: "auto" });
					size = $P.outerHeight();
				}
				size = max(size, minSize);
				size = min(size, maxSize);
				size = min(size, cDims.innerHeight - spacing);
				CSS.height = max(1, cssH(pane, size));
				s.size = size; // update state
				// make sure minSize is sufficient to avoid errors
				s.maxSize = maxSize; // init value
				s.minSize = max(minSize, size - CSS.height + 1); // = pane.outerHeight when css.height = 1px
				// handle IE6
				//if (isIE6) CSS.width = cssW($P, cDims.innerWidth);
				$P.css(CSS); // apply size & position
			}
			else if (dir == "vert") { // east or west pane
				if (size === 0 || size == "auto") {
					$P.css({ "width": "auto", "float": "left" }); // float = FORCE pane to auto-size
					size = $P.outerWidth();
					$P.css({ "float": "none" }); // RESET
				}
				size = max(size, minSize);
				size = min(size, maxSize);
				size = min(size, cDims.innerWidth - spacing);
				CSS.width = max(1, cssW(pane, size));
				s.size = size; // update state
				s.maxSize = maxSize; // init value
				// make sure minSize is sufficient to avoid errors
				s.minSize = max(minSize, size - CSS.width + 1); // = pane.outerWidth when css.width = 1px
				$P.css(CSS); // apply size - top, bottom & height set by sizeMidPanes
				sizeMidPanes(pane, null, true); // true = onInit
			}
			else if (pane == "center") {
				$P.css(CSS); // top, left, width & height set by sizeMidPanes...
				sizeMidPanes("center", null, true); // true = onInit
			}

			// close or hide the pane if specified in settings
			if (o.initClosed && o.closable) {
				$P.hide().addClass("closed");
				s.isClosed = true;
			}
			else if (o.initHidden || o.initClosed) {
				hide(pane, true); // will be completely invisible - no resizer or spacing
				s.isHidden = true;
			}
			else
				$P.addClass("open");

			// check option for auto-handling of pop-ups & drop-downs
			if (o.showOverflowOnHover)
				$P.hover( allowOverflow, resetOverflow );

			/*
			 *	see if this pane has a 'content element' that we need to auto-size
			 */
			if (o.contentSelector) {
				$C = $Cs[pane] = $P.children(o.contentSelector+":first"); // match 1-element only
				if (!$C.length) {
					$Cs[pane] = false;
					return true; // SKIP to next
				}
				$C.css( c.content.cssReq );
				if (o.applyDefaultStyles) $C.css( c.content.cssDef ); // cosmetic defaults
				// NO PANE-SCROLLING when there is a content-div
				$P.css({ overflow: "hidden" });
			}
		});
	};

	/**
	 * initHandles
	 *
	 * Initialize module objects, styling, size and position for all resize bars and toggler buttons
	 *
	 * @callers  create()
	 */
	var initHandles = function () {
		// create toggler DIVs for each pane, and set object pointers for them, eg: $R.north = north toggler DIV
		$.each(c.borderPanes.split(","), function() {
			var 
				pane	= str(this)
			,	o		= options[pane]
			,	s		= state[pane]
			,	rClass	= o.resizerClass
			,	tClass	= o.togglerClass
			,	$P		= $Ps[pane]
			;
			$Rs[pane] = false; // INIT
			$Ts[pane] = false;

			if (!$P || (!o.closable && !o.resizable)) return; // pane does not exist - skip

			var 
				edge	= c[pane].edge
			,	isOpen	= $P.is(":visible")
			,	spacing	= (isOpen ? o.spacing_open : o.spacing_closed)
			,	_pane	= "-"+ pane // used for classNames
			,	_state	= (isOpen ? "-open" : "-closed") // used for classNames
			,	$R, $T
			;
			// INIT RESIZER BAR
			$R = $Rs[pane] = $("<span></span>");
	
			if (isOpen && o.resizable)
				; // this is handled by initResizable
			else if (!isOpen && o.slidable)
				$R.attr("title", o.sliderTip).css("cursor", o.sliderCursor);
	
			$R
				// if paneSelector is an ID, then create a matching ID for the resizer, eg: "#paneLeft" => "paneLeft-resizer"
				.attr("id", (o.paneSelector.substr(0,1)=="#" ? o.paneSelector.substr(1) + "-resizer" : ""))
				.attr("resizer", pane) // so we can read this from the resizer
				.css(c.resizers.cssReq) // add base/required styles
				// POSITION of resizer bar - allow for container border & padding
				.css(edge, cDims[edge] + getPaneSize(pane))
				// ADD CLASSNAMES - eg: class="resizer resizer-west resizer-open"
				.addClass( rClass +" "+ rClass+_pane +" "+ rClass+_state +" "+ rClass+_pane+_state )
				.appendTo($Container) // append DIV to container
			;
			 // ADD VISUAL STYLES
			if (o.applyDefaultStyles)
				$R.css(c.resizers.cssDef);

			if (o.closable) {
				// INIT COLLAPSER BUTTON
				$T = $Ts[pane] = $("<div></div>");
				$T
					// if paneSelector is an ID, then create a matching ID for the resizer, eg: "#paneLeft" => "paneLeft-toggler"
					.attr("id", (o.paneSelector.substr(0,1)=="#" ? o.paneSelector.substr(1) + "-toggler" : ""))
					.css(c.togglers.cssReq) // add base/required styles
					.attr("title", (isOpen ? o.togglerTip_open : o.togglerTip_closed))
					.click(function(evt){ toggle(pane); evt.stopPropagation(); })
					.mouseover(function(evt){ evt.stopPropagation(); }) // prevent resizer event
					// ADD CLASSNAMES - eg: class="toggler toggler-west toggler-west-open"
					.addClass( tClass +" "+ tClass+_pane +" "+ tClass+_state +" "+ tClass+_pane+_state )
					.appendTo($R) // append SPAN to resizer DIV
				;

				// ADD INNER-SPANS TO TOGGLER
				if (o.togglerContent_open) // ui-layout-open
					$("<span>"+ o.togglerContent_open +"</span>")
						.addClass("content content-open")
						.css("display", s.isClosed ? "none" : "block")
						.appendTo( $T )
					;
				if (o.togglerContent_closed) // ui-layout-closed
					$("<span>"+ o.togglerContent_closed +"</span>")
						.addClass("content content-closed")
						.css("display", s.isClosed ? "block" : "none")
						.appendTo( $T )
					;

				 // ADD BASIC VISUAL STYLES
				if (o.applyDefaultStyles)
					$T.css(c.togglers.cssDef);

				if (!isOpen) bindStartSlidingEvent(pane, true); // will enable if state.PANE.isSliding = true
			}

		});

		// SET ALL HANDLE SIZES & LENGTHS
		sizeHandles("all", true); // true = onInit
	};

	/**
	 * initResizable
	 *
	 * Add resize-bars to all panes that specify it in options
	 *
	 * @dependancies  $.fn.resizable - will abort if not found
	 * @callers  create()
	 */
	var initResizable = function () {
		var
			draggingAvailable = (typeof $.fn.draggable == "function")
		,	minPosition, maxPosition, edge // set in start()
		;

		$.each(c.borderPanes.split(","), function() {
			var 
				pane	= str(this)
			,	o		= options[pane]
			,	s		= state[pane]
			;
			if (!draggingAvailable || !$Ps[pane] || !o.resizable) {
				o.resizable = false;
				return true; // skip to next
			}

			var 
				rClass				= o.resizerClass
			//	'drag' classes are applied to the ORIGINAL resizer-bar while dragging is in process
			,	dragClass			= rClass+"-drag"			// resizer-drag
			,	dragPaneClass		= rClass+"-"+pane+"-drag"	// resizer-north-drag
			//	'dragging' class is applied to the CLONED resizer-bar while it is being dragged
			,	draggingClass		= rClass+"-dragging"		// resizer-dragging
			,	draggingPaneClass	= rClass+"-"+pane+"-dragging" // resizer-north-dragging
			,	draggingClassSet	= false 					// logic var
			,	$P 					= $Ps[pane]
			,	$R					= $Rs[pane]
			;

			if (!s.isClosed)
				$R
					.attr("title", o.resizerTip)
					.css("cursor", o.resizerCursor) // n-resize, s-resize, etc
				;

			$R.draggable({
				containment:	$Container[0] // limit resizing to layout container
			,	axis:			(c[pane].dir=="horz" ? "y" : "x") // limit resizing to horz or vert axis
			,	delay:			200
			,	distance:		1
			//	basic format for helper - style it using class: .ui-draggable-dragging
			,	helper:			"clone"
			,	opacity:		o.resizerDragOpacity
			//,	iframeFix:		o.draggableIframeFix // TODO: consider using when bug is fixed
			,	zIndex:			c.zIndex.resizing

			,	start: function (e, ui) {
					// onresize_start callback - will CANCEL hide if returns false
					// TODO: CONFIRM that dragging can be cancelled like this???
					if (false === execUserCallback(pane, o.onresize_start)) return false;

					s.isResizing = true; // prevent pane from closing while resizing
					clearTimer(pane, "closeSlider"); // just in case already triggered

					$R.addClass( dragClass +" "+ dragPaneClass ); // add drag classes
					draggingClassSet = false; // reset logic var - see drag()

					// SET RESIZING LIMITS - used in drag()
					var resizerWidth = (pane=="east" || pane=="south" ? o.spacing_open : 0);
					setPaneMinMaxSizes(pane); // update pane-state
					s.minPosition -= resizerWidth;
					s.maxPosition -= resizerWidth;
					edge = (c[pane].dir=="horz" ? "top" : "left");

					// MASK PANES WITH IFRAMES OR OTHER TROUBLESOME ELEMENTS
					$(o.maskIframesOnResize === true ? "iframe" : o.maskIframesOnResize).each(function() {					
						$('<div class="ui-layout-mask"/>')
							.css({
								background:	"#fff"
							,	opacity:	"0.001"
							,	zIndex:		9
							,	position:	"absolute"
							,	width:		this.offsetWidth+"px"
							,	height:		this.offsetHeight+"px"
							})
							.css($(this).offset()) // top & left
							.appendTo(this.parentNode) // put div INSIDE pane to avoid zIndex issues
						;
					});
				}

			,	drag: function (e, ui) {
					if (!draggingClassSet) { // can only add classes after clone has been added to the DOM
						$(".ui-draggable-dragging")
							.addClass( draggingClass +" "+ draggingPaneClass ) // add dragging classes
							.children().css("visibility","hidden") // hide toggler inside dragged resizer-bar
						;
						draggingClassSet = true;
						// draggable bug!? RE-SET zIndex to prevent E/W resize-bar showing through N/S pane!
						if (s.isSliding) $Ps[pane].css("zIndex", c.zIndex.sliding);
					}
					// CONTAIN RESIZER-BAR TO RESIZING LIMITS
					if		(ui.position[edge] < s.minPosition) ui.position[edge] = s.minPosition;
					else if (ui.position[edge] > s.maxPosition) ui.position[edge] = s.maxPosition;
				}

			,	stop: function (e, ui) {
					var 
						dragPos	= ui.position
					,	resizerPos
					,	newSize
					;
					$R.removeClass( dragClass +" "+ dragPaneClass ); // remove drag classes
	
					switch (pane) {
						case "north":	resizerPos = dragPos.top; break;
						case "west":	resizerPos = dragPos.left; break;
						case "south":	resizerPos = cDims.outerHeight - dragPos.top - $R.outerHeight(); break;
						case "east":	resizerPos = cDims.outerWidth - dragPos.left - $R.outerWidth(); break;
					}
					// remove container margin from resizer position to get the pane size
					newSize = resizerPos - cDims[ c[pane].edge ];

					sizePane(pane, newSize);

					// UN-MASK PANES MASKED IN drag.start
					$("div.ui-layout-mask").remove(); // Remove iframe masks	

					s.isResizing = false;
				}

			});
		});
	};



/*
 * ###########################
 *       ACTION METHODS
 * ###########################
 */

	/**
	 * hide / show
	 *
	 * Completely 'hides' a pane, including its spacing - as if it does not exist
	 * The pane is not actually 'removed' from the source, so can use 'show' to un-hide it
	 *
	 * @param String  pane   The pane being hidden, ie: north, south, east, or west
	 */
	var hide = function (pane, onInit) {
		var
			o	= options[pane]
		,	s	= state[pane]
		,	$P	= $Ps[pane]
		,	$R	= $Rs[pane]
		;
		if (!$P || s.isHidden) return; // pane does not exist OR is already hidden

		// onhide_start callback - will CANCEL hide if returns false
		if (false === execUserCallback(pane, o.onhide_start)) return;

		s.isSliding = false; // just in case

		// now hide the elements
		if ($R) $R.hide(); // hide resizer-bar
		if (onInit || s.isClosed) {
			s.isClosed = true; // to trigger open-animation on show()
			s.isHidden  = true;
			$P.hide(); // no animation when loading page
			sizeMidPanes(c[pane].dir == "horz" ? "all" : "center");
			execUserCallback(pane, o.onhide_end || o.onhide);
		}
		else {
			s.isHiding = true; // used by onclose
			close(pane, false); // adjust all panes to fit
			//s.isHidden  = true; - will be set by close - if not cancelled
		}
	};

	var show = function (pane, openPane) {
		var
			o	= options[pane]
		,	s	= state[pane]
		,	$P	= $Ps[pane]
		,	$R	= $Rs[pane]
		;
		if (!$P || !s.isHidden) return; // pane does not exist OR is not hidden

		// onhide_start callback - will CANCEL hide if returns false
		if (false === execUserCallback(pane, o.onshow_start)) return;

		s.isSliding = false; // just in case
		s.isShowing = true; // used by onopen/onclose
		//s.isHidden  = false; - will be set by open/close - if not cancelled

		// now show the elements
		if ($R && o.spacing_open > 0) $R.show();
		if (openPane === false)
			close(pane, true); // true = force
		else
			open(pane); // adjust all panes to fit
	};


	/**
	 * toggle
	 *
	 * Toggles a pane open/closed by calling either open or close
	 *
	 * @param String  pane   The pane being toggled, ie: north, south, east, or west
	 */
	var toggle = function (pane) {
		var s = state[pane];
		if (s.isHidden)
			show(pane); // will call 'open' after unhiding it
		else if (s.isClosed)
			open(pane);
		else
			close(pane);
	};

	/**
	 * close
	 *
	 * Close the specified pane (animation optional), and resize all other panes as needed
	 *
	 * @param String  pane   The pane being closed, ie: north, south, east, or west
	 */
	var close = function (pane, force, noAnimation) {
		var 
			$P		= $Ps[pane]
		,	$R		= $Rs[pane]
		,	$T		= $Ts[pane]
		,	o		= options[pane]
		,	s		= state[pane]
		,	doFX	= !noAnimation && !s.isClosed && (o.fxName_close != "none")
		,	edge	= c[pane].edge
		,	rClass	= o.resizerClass
		,	tClass	= o.togglerClass
		,	_pane	= "-"+ pane // used for classNames
		,	_open	= "-open"
		,	_sliding= "-sliding"
		,	_closed	= "-closed"
		// 	transfer logic vars to temp vars
		,	isShowing = s.isShowing
		,	isHiding = s.isHiding
		;
		// now clear the logic vars
		delete s.isShowing;
		delete s.isHiding;

		if (!$P || (!o.resizable && !o.closable)) return; // invalid request
		else if (!force && s.isClosed && !isShowing) return; // already closed

		if (c.isLayoutBusy) { // layout is 'busy' - probably with an animation
			setFlowCallback("close", pane, force); // set a callback for this action, if possible
			return; // ABORT 
		}

		// onclose_start callback - will CANCEL hide if returns false
		// SKIP if just 'showing' a hidden pane as 'closed'
		if (!isShowing && false === execUserCallback(pane, o.onclose_start)) return;

		// SET flow-control flags
		c[pane].isMoving = true;
		c.isLayoutBusy = true;

		s.isClosed = true;
		// update isHidden BEFORE sizing panes
		if (isHiding) s.isHidden = true;
		else if (isShowing) s.isHidden = false;

		// sync any 'pin buttons'
		syncPinBtns(pane, false);

		// resize panes adjacent to this one
		if (!s.isSliding) sizeMidPanes(c[pane].dir == "horz" ? "all" : "center");

		// if this pane has a resizer bar, move it now
		if ($R) {
			$R
				.css(edge, cDims[edge]) // move the resizer bar
				.removeClass( rClass+_open +" "+ rClass+_pane+_open )
				.removeClass( rClass+_sliding +" "+ rClass+_pane+_sliding )
				.addClass( rClass+_closed +" "+ rClass+_pane+_closed )
			;
			// DISABLE 'resizing' when closed - do this BEFORE bindStartSlidingEvent
			if (o.resizable)
				$R
					.draggable("disable")
					.css("cursor", "default")
					.attr("title","")
				;
			// if pane has a toggler button, adjust that too
			if ($T) {
				$T
					.removeClass( tClass+_open +" "+ tClass+_pane+_open )
					.addClass( tClass+_closed +" "+ tClass+_pane+_closed )
					.attr("title", o.togglerTip_closed) // may be blank
				;
			}
			sizeHandles(); // resize 'length' and position togglers for adjacent panes
		}

		// ANIMATE 'CLOSE' - if no animation, then was ALREADY shown above
		if (doFX) {
			lockPaneForFX(pane, true); // need to set left/top so animation will work
			$P.hide( o.fxName_close, o.fxSettings_close, o.fxSpeed_close, function () {
				lockPaneForFX(pane, false); // undo
				if (!s.isClosed) return; // pane was opened before animation finished!
				close_2();
			});
		}
		else {
			$P.hide(); // just hide pane NOW
			close_2();
		}

		// SUBROUTINE
		function close_2 () {
			bindStartSlidingEvent(pane, true); // will enable if state.PANE.isSliding = true

			// onclose callback - UNLESS just 'showing' a hidden pane as 'closed'
			if (!isShowing)	execUserCallback(pane, o.onclose_end || o.onclose);
			// onhide OR onshow callback
			if (isShowing)	execUserCallback(pane, o.onshow_end || o.onshow);
			if (isHiding)	execUserCallback(pane, o.onhide_end || o.onhide);

			// internal flow-control callback
			execFlowCallback(pane);
		}
	};

	/**
	 * open
	 *
	 * Open the specified pane (animation optional), and resize all other panes as needed
	 *
	 * @param String  pane   The pane being opened, ie: north, south, east, or west
	 */
	var open = function (pane, slide, noAnimation) {
		var 
			$P		= $Ps[pane]
		,	$R		= $Rs[pane]
		,	$T		= $Ts[pane]
		,	o		= options[pane]
		,	s		= state[pane]
		,	doFX	= !noAnimation && s.isClosed && (o.fxName_open != "none")
		,	edge	= c[pane].edge
		,	rClass	= o.resizerClass
		,	tClass	= o.togglerClass
		,	_pane	= "-"+ pane // used for classNames
		,	_open	= "-open"
		,	_closed	= "-closed"
		,	_sliding= "-sliding"
		// 	transfer logic var to temp var
		,	isShowing = s.isShowing
		;
		// now clear the logic var
		delete s.isShowing;

		if (!$P || (!o.resizable && !o.closable)) return; // invalid request
		else if (!s.isClosed && !s.isSliding) return; // already open

		// pane can ALSO be unhidden by just calling show(), so handle this scenario
		if (s.isHidden && !isShowing) {
			show(pane, true);
			return;
		}

		if (c.isLayoutBusy) { // layout is 'busy' - probably with an animation
			setFlowCallback("open", pane, slide); // set a callback for this action, if possible
			return; // ABORT
		}

		// onopen_start callback - will CANCEL hide if returns false
		if (false === execUserCallback(pane, o.onopen_start)) return;

		// SET flow-control flags
		c[pane].isMoving = true;
		c.isLayoutBusy = true;

		// 'PIN PANE' - stop sliding
		if (s.isSliding && !slide) // !slide = 'open pane normally' - NOT sliding
			bindStopSlidingEvents(pane, false); // will set isSliding=false

		s.isClosed = false;
		// update isHidden BEFORE sizing panes
		if (isShowing) s.isHidden = false;

		// Container size may have changed - shrink the pane if now 'too big'
		setPaneMinMaxSizes(pane); // update pane-state
		if (s.size > s.maxSize) // pane is too big! resize it before opening
			$P.css( c[pane].sizeType, max(1, cssSize(pane, s.maxSize)) );

		bindStartSlidingEvent(pane, false); // remove trigger event from resizer-bar

		if (doFX) { // ANIMATE
			lockPaneForFX(pane, true); // need to set left/top so animation will work
			$P.show( o.fxName_open, o.fxSettings_open, o.fxSpeed_open, function() {
				lockPaneForFX(pane, false); // undo
				if (s.isClosed) return; // pane was closed before animation finished!
				open_2(); // continue
			});
		}
		else {// no animation
			$P.show();	// just show pane and...
			open_2();	// continue
		}

		// SUBROUTINE
		function open_2 () {
			// NOTE: if isSliding, then other panes are NOT 'resized'
			if (!s.isSliding) // resize all panes adjacent to this one
				sizeMidPanes(c[pane].dir=="vert" ? "center" : "all");

			// if this pane has a toggler, move it now
			if ($R) {
				$R
					.css(edge, cDims[edge] + getPaneSize(pane)) // move the toggler
					.removeClass( rClass+_closed +" "+ rClass+_pane+_closed )
					.addClass( rClass+_open +" "+ rClass+_pane+_open )
					.addClass( !s.isSliding ? "" : rClass+_sliding +" "+ rClass+_pane+_sliding )
				;
				if (o.resizable)
					$R
						.draggable("enable")
						.css("cursor", o.resizerCursor)
						.attr("title", o.resizerTip)
					;
				else
					$R.css("cursor", "default"); // n-resize, s-resize, etc
				// if pane also has a toggler button, adjust that too
				if ($T) {
					$T
						.removeClass( tClass+_closed +" "+ tClass+_pane+_closed )
						.addClass( tClass+_open +" "+ tClass+_pane+_open )
						.attr("title", o.togglerTip_open) // may be blank
					;
				}
				sizeHandles("all"); // resize resizer & toggler sizes for all panes
			}

			// resize content every time pane opens - to be sure
			sizeContent(pane);

			// sync any 'pin buttons'
			syncPinBtns(pane, !s.isSliding);

			// onopen callback
			execUserCallback(pane, o.onopen_end || o.onopen);

			// onshow callback
			if (isShowing) execUserCallback(pane, o.onshow_end || o.onshow);

			// internal flow-control callback
			execFlowCallback(pane);
		}
	};
	

	/**
	 * lockPaneForFX
	 *
	 * Must set left/top on East/South panes so animation will work properly
	 *
	 * @param String  pane  The pane to lock, 'east' or 'south' - any other is ignored!
	 * @param Boolean  doLock  true = set left/top, false = remove
	 */
	var lockPaneForFX = function (pane, doLock) {
		var $P = $Ps[pane];
		if (doLock) {
			$P.css({ zIndex: c.zIndex.animation }); // overlay all elements during animation
			if (pane=="south")
				$P.css({ top: cDims.top + cDims.innerHeight - $P.outerHeight() });
			else if (pane=="east")
				$P.css({ left: cDims.left + cDims.innerWidth - $P.outerWidth() });
		}
		else {
			if (!state[pane].isSliding) $P.css({ zIndex: c.zIndex.pane_normal });
			if (pane=="south")
				$P.css({ top: "auto" });
			else if (pane=="east")
				$P.css({ left: "auto" });
		}
	};


	/**
	 * bindStartSlidingEvent
	 *
	 * Toggle sliding functionality of a specific pane on/off by adding removing 'slide open' trigger
	 *
	 * @callers  open(), close()
	 * @param String  pane  The pane to enable/disable, 'north', 'south', etc.
	 * @param Boolean  enable  Enable or Disable sliding?
	 */
	var bindStartSlidingEvent = function (pane, enable) {
		var 
			o		= options[pane]
		,	$R		= $Rs[pane]
		,	trigger	= o.slideTrigger_open
		;
		if (!$R || !o.slidable) return;
		// make sure we have a valid event
		if (trigger != "click" && trigger != "dblclick" && trigger != "mouseover") trigger = "click";
		$R
			// add or remove trigger event
			[enable ? "bind" : "unbind"](trigger, slideOpen)
			// set the appropriate cursor & title/tip
			.css("cursor", (enable ? o.sliderCursor: "default"))
			.attr("title", (enable ? o.sliderTip : ""))
		;
	};

	/**
	 * bindStopSlidingEvents
	 *
	 * Add or remove 'mouseout' events to 'slide close' when pane is 'sliding' open or closed
	 * Also increases zIndex when pane is sliding open
	 * See bindStartSlidingEvent for code to control 'slide open'
	 *
	 * @callers  slideOpen(), slideClosed()
	 * @param String  pane  The pane to process, 'north', 'south', etc.
	 * @param Boolean  isOpen  Is pane open or closed?
	 */
	var bindStopSlidingEvents = function (pane, enable) {
		var 
			o		= options[pane]
		,	s		= state[pane]
		,	trigger	= o.slideTrigger_close
		,	action	= (enable ? "bind" : "unbind") // can't make 'unbind' work! - see disabled code below
		,	$P		= $Ps[pane]
		,	$R		= $Rs[pane]
		;

		s.isSliding = enable; // logic
		clearTimer(pane, "closeSlider"); // just in case

		// raise z-index when sliding
		$P.css({ zIndex: (enable ? c.zIndex.sliding : c.zIndex.pane_normal) });
		$R.css({ zIndex: (enable ? c.zIndex.sliding : c.zIndex.resizer_normal) });

		// make sure we have a valid event
		if (trigger != "click" && trigger != "mouseout") trigger = "mouseout";

		// when trigger is 'mouseout', must cancel timer when mouse moves between 'pane' and 'resizer'
		if (enable) { // BIND trigger events
			$P.bind(trigger, slideClosed );
			$R.bind(trigger, slideClosed );
			if (trigger = "mouseout") {
				$P.bind("mouseover", cancelMouseOut );
				$R.bind("mouseover", cancelMouseOut );
			}
		}
		else { // UNBIND trigger events
			// TODO: why does unbind of a 'single function' not work reliably?
			//$P[action](trigger, slideClosed );
			$P.unbind(trigger);
			$R.unbind(trigger);
			if (trigger = "mouseout") {
				//$P[action]("mouseover", cancelMouseOut );
				$P.unbind("mouseover");
				$R.unbind("mouseover");
				clearTimer(pane, "closeSlider");
			}
		}

		// SUBROUTINE for mouseout timer clearing
		function cancelMouseOut (evt) {
			clearTimer(pane, "closeSlider");
			evt.stopPropagation();
		}
	};

	var slideOpen = function () {
		var pane = $(this).attr("resizer"); // attr added by initHandles
		if (state[pane].isClosed) { // skip if already open!
			bindStopSlidingEvents(pane, true); // pane is opening, so BIND trigger events to close it
			open(pane, true); // true = slide - ie, called from here!
		}
	};

	var slideClosed = function () {
		var
			$E = $(this)
		,	pane = $E.attr("pane") || $E.attr("resizer")
		,	o = options[pane]
		,	s = state[pane]
		;
		if (s.isClosed || s.isResizing)
			return; // skip if already closed OR in process of resizing
		else if (o.slideTrigger_close == "click")
			close_NOW(); // close immediately onClick
		else // trigger = mouseout - use a delay
			setTimer(pane, "closeSlider", close_NOW, 300); // .3 sec delay

		// SUBROUTINE for timed close
		function close_NOW () {
			bindStopSlidingEvents(pane, false); // pane is being closed, so UNBIND trigger events
			if (!s.isClosed) close(pane); // skip if already closed!
		}
	};


	/**
	 * sizePane
	 *
	 * @callers  initResizable.stop()
	 * @param String  pane   The pane being resized - usually west or east, but potentially north or south
	 * @param Integer  newSize  The new size for this pane - will be validated
	 */
	var sizePane = function (pane, size) {
		// TODO: accept "auto" as size, and size-to-fit pane content
		var 
			edge	= c[pane].edge
		,	dir		= c[pane].dir
		,	o		= options[pane]
		,	s		= state[pane]
		,	$P		= $Ps[pane]
		,	$R		= $Rs[pane]
		;
		// calculate 'current' min/max sizes
		setPaneMinMaxSizes(pane); // update pane-state
		// compare/update calculated min/max to user-options
		s.minSize = max(s.minSize, o.minSize);
		if (o.maxSize > 0) s.maxSize = min(s.maxSize, o.maxSize);
		// validate passed size
		size = max(size, s.minSize);
		size = min(size, s.maxSize);
		s.size = size; // update state

		// move the resizer bar and resize the pane
		$R.css( edge, size + cDims[edge] );
		$P.css( c[pane].sizeType, max(1, cssSize(pane, size)) );

		// resize all the adjacent panes, and adjust their toggler buttons
		if (!s.isSliding) sizeMidPanes(dir=="horz" ? "all" : "center");
		sizeHandles();
		sizeContent(pane);
		execUserCallback(pane, o.onresize_end || o.onresize);
	};

	/**
	 * sizeMidPanes
	 *
	 * @callers  create(), open(), close(), onWindowResize()
	 */
	var sizeMidPanes = function (panes, overrideDims, onInit) {
		if (!panes || panes == "all") panes = "east,west,center";

		var d = getPaneDims();
		if (overrideDims) $.extend( d, overrideDims );

		$.each(panes.split(","), function() {
			if (!$Ps[this]) return; // NO PANE - skip
			var 
				pane	= str(this)
			,	o		= options[pane]
			,	s		= state[pane]
			,	$P		= $Ps[pane]
			,	$R		= $Rs[pane]
			,	hasRoom	= true
			,	CSS		= {}
			;

			if (pane == "center") {
				d = getPaneDims(); // REFRESH Dims because may have just 'unhidden' East or West pane after a 'resize'
				CSS = $.extend( {}, d ); // COPY ALL of the paneDims
				CSS.width  = max(1, cssW(pane, CSS.width));
				CSS.height = max(1, cssH(pane, CSS.height));
				hasRoom = (CSS.width > 1 && CSS.height > 1);
				/*
				 * Extra CSS for IE6 or IE7 in Quirks-mode - add 'width' to NORTH/SOUTH panes
				 * Normally these panes have only 'left' & 'right' positions so pane auto-sizes
				 */
				if ($.browser.msie && (!$.boxModel || $.browser.version < 7)) {
					if ($Ps.north) $Ps.north.css({ width: cssW($Ps.north, cDims.innerWidth) });
					if ($Ps.south) $Ps.south.css({ width: cssW($Ps.south, cDims.innerWidth) });
				}
			}
			else { // for east and west, set only the height
				CSS.top = d.top;
				CSS.bottom = d.bottom;
				
				CSS.height = max(1, cssH(pane, d.height));
				hasRoom = (CSS.height > 1);
			}

			if (hasRoom) {
				$P.css(CSS);
				if (s.noRoom) {
					s.noRoom = false;
					if (s.isHidden) return;
					else show(pane, !s.isClosed);
					/* OLD CODE - keep until sure line above works right!
					if (!s.isClosed) $P.show(); // in case was previously hidden due to NOT hasRoom
					if ($R) $R.show();
					*/
				}
				if (!onInit) {
					sizeContent(pane);
					execUserCallback(pane, o.onresize_end || o.onresize);
				}
			}
			else if (!s.noRoom) { // no room for pane, so just hide it (if not already)
				s.noRoom = true; // update state
				if (s.isHidden) return;
				if (onInit) { // skip onhide callback and other logic onLoad
					$P.hide();
					if ($R) $R.hide();
				}
				else hide(pane);
			}
		});
	};


	var sizeContent = function (panes) {
		if (!panes || panes == "all") panes = c.allPanes;

		$.each(panes.split(","), function() {
			if (!$Cs[this]) return; // NO CONTENT - skip
			var 
				pane	= str(this)
			,	ignore	= options[pane].contentIgnoreSelector
			,	$P		= $Ps[pane]
			,	$C		= $Cs[pane]
			,	e_C		= $C[0]		// DOM element
			,	height	= cssH($P);	// init to pane.innerHeight
			;
			$P.children().each(function() {
				if (this == e_C) return; // Content elem - skip
				var $E = $(this);
				if (!ignore || !$E.is(ignore))
					height -= $E.outerHeight();
			});
			if (height > 0)
				height = cssH($C, height);
			if (height < 1)
				$C.hide(); // no room for content!
			else
				$C.css({ height: height }).show();
		});
	};


	/**
	 * sizeHandles
	 *
	 * Called every time a pane is opened, closed, or resized to slide the togglers to 'center' and adjust their length if necessary
	 *
	 * @callers  initHandles(), open(), close(), resizeAll()
	 */
	var sizeHandles = function (panes, onInit) {
		if (!panes || panes == "all") panes = c.borderPanes;

		$.each(panes.split(","), function() {
			var 
				pane	= str(this)
			,	o		= options[pane]
			,	s		= state[pane]
			,	$P		= $Ps[pane]
			,	$R		= $Rs[pane]
			,	$T		= $Ts[pane]
			;
			if (!$P || !$R || (!o.resizable && !o.closable)) return; // skip

			var 
				dir			= c[pane].dir
			,	_state		= (s.isClosed ? "_closed" : "_open")
			,	spacing		= o["spacing"+ _state]
			,	togAlign	= o["togglerAlign"+ _state]
			,	togLen		= o["togglerLength"+ _state]
			,	paneLen
			,	offset
			,	CSS = {}
			;
			if (spacing == 0) {
				$R.hide();
				return;
			}
			else if (!s.noRoom && !s.isHidden) // skip if resizer was hidden for any reason
				$R.show(); // in case was previously hidden

			// Resizer Bar is ALWAYS same width/height of pane it is attached to
			if (dir == "horz") { // north/south
				paneLen = $P.outerWidth();
				$R.css({
					width:	max(1, cssW($R, paneLen)) // account for borders & padding
				,	height:	max(1, cssH($R, spacing)) // ditto
				,	left:	cssNum($P, "left")
				});
			}
			else { // east/west
				paneLen = $P.outerHeight();
				$R.css({
					height:	max(1, cssH($R, paneLen)) // account for borders & padding
				,	width:	max(1, cssW($R, spacing)) // ditto
				,	top:	cDims.top + getPaneSize("north", true)
				//,	top:	cssNum($Ps["center"], "top")
				});
				
			}

			if ($T) {
				if (togLen == 0 || (s.isSliding && o.hideTogglerOnSlide)) {
					$T.hide(); // always HIDE the toggler when 'sliding'
					return;
				}
				else
					$T.show(); // in case was previously hidden

				if (!(togLen > 0) || togLen == "100%" || togLen > paneLen) {
					togLen = paneLen;
					offset = 0;
				}
				else { // calculate 'offset' based on options.PANE.togglerAlign_open/closed
					if (typeof togAlign == "string") {
						switch (togAlign) {
							case "top":
							case "left":	offset = 0;
											break;
							case "bottom":
							case "right":	offset = paneLen - togLen;
											break;
							case "middle":
							case "center":
							default:		offset = Math.floor((paneLen - togLen) / 2); // 'default' catches typos
						}
					}
					else { // togAlign = number
						var x = parseInt(togAlign); //
						if (togAlign >= 0) offset = x;
						else offset = paneLen - togLen + x; // NOTE: x is negative!
					}
				}

				var
					$TC_o = (o.togglerContent_open   ? $T.children(".content-open") : false)
				,	$TC_c = (o.togglerContent_closed ? $T.children(".content-closed")   : false)
				,	$TC   = (s.isClosed ? $TC_c : $TC_o)
				;
				if ($TC_o) $TC_o.css("display", s.isClosed ? "none" : "block");
				if ($TC_c) $TC_c.css("display", s.isClosed ? "block" : "none");

				if (dir == "horz") { // north/south
					var width = cssW($T, togLen);
					$T.css({
						width:	max(0, width)  // account for borders & padding
					,	height:	max(1, cssH($T, spacing)) // ditto
					,	left:	offset // TODO: VERIFY that toggler  positions correctly for ALL values
					});
					if ($TC) // CENTER the toggler content SPAN
						$TC.css("marginLeft", Math.floor((width-$TC.outerWidth())/2)); // could be negative
				}
				else { // east/west
					var height = cssH($T, togLen);
					$T.css({
						height:	max(0, height)  // account for borders & padding
					,	width:	max(1, cssW($T, spacing)) // ditto
					,	top:	offset // POSITION the toggler
					});
					if ($TC) // CENTER the toggler content SPAN
						$TC.css("marginTop", Math.floor((height-$TC.outerHeight())/2)); // could be negative
				}


			}

			// DONE measuring and sizing this resizer/toggler, so can be 'hidden' now
			if (onInit && o.initHidden) {
				$R.hide();
				if ($T) $T.hide();
			}
		});
	};


	/**
	 * resizeAll
	 *
	 * @callers  window.onresize(), callbacks or custom code
	 */
	var resizeAll = function () {
		var
			oldW	= cDims.innerWidth
		,	oldH	= cDims.innerHeight
		;
		cDims = state.container = getElemDims($Container); // UPDATE container dimensions

		var
			checkH	= (cDims.innerHeight < oldH)
		,	checkW	= (cDims.innerWidth < oldW)
		,	s, dir
		;

		if (checkH || checkW)
			// NOTE special order for sizing: S-N-E-W
			$.each(["south","north","east","west"], function(i,pane) {
				s = state[pane];
				dir = c[pane].dir;
				if (!s.isClosed && ((checkH && dir=="horz") || (checkW && dir=="vert"))) {
					setPaneMinMaxSizes(pane); // update pane-state
					// shrink pane if 'too big' to fit
					if (s.size > s.maxSize)
						sizePane(pane, s.maxSize);
				}
			});

		sizeMidPanes("all");
		sizeHandles("all"); // reposition the toggler elements
	};


	/**
	 * keyDown
	 *
	 * Capture keys when enableCursorHotkey - toggle pane if hotkey pressed
	 *
	 * @callers  document.keydown()
	 */
	function keyDown (evt) {
		if (!evt) return true;
		var code = evt.keyCode;
		if (code < 33) return true; // ignore special keys: ENTER, TAB, etc

		var
			PANE = {
				38: "north" // Up Cursor
			,	40: "south" // Down Cursor
			,	37: "west"  // Left Cursor
			,	39: "east"  // Right Cursor
			}
		,	isCursorKey = (code >= 37 && code <= 40)
		,	ALT = evt.altKey // no worky!
		,	SHIFT = evt.shiftKey
		,	CTRL = evt.ctrlKey
		,	pane = false
		,	s, o, k, m, el
		;

		if (!CTRL && !SHIFT)
			return true; // no modifier key - abort
		else if (isCursorKey && options[PANE[code]].enableCursorHotkey) // valid cursor-hotkey
			pane = PANE[code];
		else // check to see if this matches a custom-hotkey
			$.each(c.borderPanes.split(","), function(i,p) { // loop each pane to check its hotkey
				o = options[p];
				k = o.customHotkey;
				m = o.customHotkeyModifier; // if missing or invalid, treated as "CTRL+SHIFT"
				if ((SHIFT && m=="SHIFT") || (CTRL && m=="CTRL") || (CTRL && SHIFT)) { // Modifier matches
					if (k && code == (isNaN(k) || k <= 9 ? k.toUpperCase().charCodeAt(0) : k)) { // Key matches
						pane = p;
						return false; // BREAK
					}
				}
			});

		if (!pane) return true; // no hotkey - abort

		// validate pane
		o = options[pane]; // get pane options
		s = state[pane]; // get pane options
		if (!o.enableCursorHotkey || s.isHidden || !$Ps[pane]) return true;

		// see if user is in a 'form field' because may be 'selecting text'!
		el = evt.target || evt.srcElement;
		if (el && SHIFT && isCursorKey && (el.tagName=="TEXTAREA" || (el.tagName=="INPUT" && (code==37 || code==39))))
			return true; // allow text-selection

		// SYNTAX NOTES
		// use "returnValue=false" to abort keystroke but NOT abort function - can run another command afterwards
		// use "return false" to abort keystroke AND abort function
		toggle(pane);
		evt.stopPropagation();
		evt.returnValue = false; // CANCEL key
		return false;
	};


/*
 * ###########################
 *     UTILITY METHODS
 *   called externally only
 * ###########################
 */

	function allowOverflow (elem) {
		if (this && this.tagName) elem = this; // BOUND to element
		var $P;
		if (typeof elem=="string")
			$P = $Ps[elem];
		else {
			if ($(elem).attr("pane")) $P = $(elem);
			else $P = $(elem).parents("div[pane]:first");
		}
		if (!$P.length) return; // INVALID

		var
			pane	= $P.attr("pane")
		,	s		= state[pane]
		;

		// if pane is already raised, then reset it before doing it again!
		// this would happen if allowOverflow is attached to BOTH the pane and an element 
		if (s.cssSaved)
			resetOverflow(pane); // reset previous CSS before continuing

		// if pane is raised by sliding or resizing, or it's closed, then abort
		if (s.isSliding || s.isResizing || s.isClosed) {
			s.cssSaved = false;
			return;
		}

		var
			newCSS	= { zIndex: (c.zIndex.pane_normal + 1) }
		,	curCSS	= {}
		,	of		= $P.css("overflow")
		,	ofX		= $P.css("overflowX")
		,	ofY		= $P.css("overflowY")
		;
		// determine which, if any, overflow settings need to be changed
		if (of != "visible") {
			curCSS.overflow = of;
			newCSS.overflow = "visible";
		}
		if (ofX && ofX != "visible" && ofX != "auto") {
			curCSS.overflowX = ofX;
			newCSS.overflowX = "visible";
		}
		if (ofY && ofY != "visible" && ofY != "auto") {
			curCSS.overflowY = ofX;
			newCSS.overflowY = "visible";
		}

		// save the current overflow settings - even if blank!
		s.cssSaved = curCSS;

		// apply new CSS to raise zIndex and, if necessary, make overflow 'visible'
		$P.css( newCSS );

		// make sure the zIndex of all other panes is normal
		$.each(c.allPanes.split(","), function(i, p) {
			if (p != pane) resetOverflow(p);
		});

	};

	function resetOverflow (elem) {
		if (this && this.tagName) elem = this; // BOUND to element
		var $P;
		if (typeof elem=="string")
			$P = $Ps[elem];
		else {
			if ($(elem).hasClass("ui-layout-pane")) $P = $(elem);
			else $P = $(elem).parents("div[pane]:first");
		}
		if (!$P.length) return; // INVALID

		var
			pane	= $P.attr("pane")
		,	s		= state[pane]
		,	CSS		= s.cssSaved || {}
		;
		// reset the zIndex
		if (!s.isSliding && !s.isResizing)
			$P.css("zIndex", c.zIndex.pane_normal);

		// reset Overflow - if necessary
		$P.css( CSS );

		// clear var
		s.cssSaved = false;
	};


	/**
	* getBtn
	*
	* Helper function to validate params received by addButton utilities
	*
	* @param String   selector 	jQuery selector for button, eg: ".ui-layout-north .toggle-button"
	* @param String   pane 		Name of the pane the button is for: 'north', 'south', etc.
	* @returns  If both params valid, the element matching 'selector' in a jQuery wrapper - otherwise 'false'
	*/
	function getBtn(selector, pane, action) {
		var
			$E = $(selector)
		,	err = "Error Adding Button \n\nInvalid "
		;
		if (!$E.length) // element not found
			alert(err+"selector: "+ selector);
		else if (c.borderPanes.indexOf(pane) == -1) // invalid 'pane' sepecified
			alert(err+"pane: "+ pane);
		else { // VALID
			var btn = options[pane].buttonClass +"-"+ action;
			$E.addClass( btn +" "+ btn +"-"+ pane );
			return $E;
		}
		return false;  // INVALID
	};


	/**
	* addToggleBtn
	*
	* Add a custom Toggler button for a pane
	*
	* @param String   selector 	jQuery selector for button, eg: ".ui-layout-north .toggle-button"
	* @param String   pane 		Name of the pane the button is for: 'north', 'south', etc.
	*/
	function addToggleBtn (selector, pane) {
		var $E = getBtn(selector, pane, "toggle");
		if ($E)
			$E
				.attr("title", state[pane].isClosed ? "Open" : "Close")
				.click(function (evt) {
					toggle(pane);
					evt.stopPropagation();
				})
			;
	};

	/**
	* addOpenBtn
	*
	* Add a custom Open button for a pane
	*
	* @param String   selector 	jQuery selector for button, eg: ".ui-layout-north .open-button"
	* @param String   pane 		Name of the pane the button is for: 'north', 'south', etc.
	*/
	function addOpenBtn (selector, pane) {
		var $E = getBtn(selector, pane, "open");
		if ($E)
			$E
				.attr("title", "Open")
				.click(function (evt) {
					open(pane);
					evt.stopPropagation();
				})
			;
	};

	/**
	* addCloseBtn
	*
	* Add a custom Close button for a pane
	*
	* @param String   selector 	jQuery selector for button, eg: ".ui-layout-north .close-button"
	* @param String   pane 		Name of the pane the button is for: 'north', 'south', etc.
	*/
	function addCloseBtn (selector, pane) {
		var $E = getBtn(selector, pane, "close");
		if ($E)
			$E
				.attr("title", "Close")
				.click(function (evt) {
					close(pane);
					evt.stopPropagation();
				})
			;
	};

	/**
	* addPinBtn
	*
	* Add a custom Pin button for a pane
	*
	* Four classes are added to the element, based on the paneClass for the associated pane...
	* Assuming the default paneClass and the pin is 'up', these classes are added for a west-pane pin:
	*  - ui-layout-pane-pin
	*  - ui-layout-pane-west-pin
	*  - ui-layout-pane-pin-up
	*  - ui-layout-pane-west-pin-up
	*
	* @param String   selector 	jQuery selector for button, eg: ".ui-layout-north .ui-layout-pin"
	* @param String   pane 		Name of the pane the pin is for: 'north', 'south', etc.
	*/
	function addPinBtn (selector, pane) {
		var $E = getBtn(selector, pane, "pin");
		if ($E) {
			var s = state[pane];
			$E.click(function (evt) {
				setPinState($(this), pane, (s.isSliding || s.isClosed));
				if (s.isSliding || s.isClosed) open( pane ); // change from sliding to open
				else close( pane ); // slide-closed
				evt.stopPropagation();
			});
			// add up/down pin attributes and classes
			setPinState ($E, pane, (!s.isClosed && !s.isSliding));
			// add this pin to the pane data so we can 'sync it' automatically
			// PANE.pins key is an array so we can store multiple pins for each pane
			c[pane].pins.push( selector ); // just save the selector string
		}
	};

	/**
	* syncPinBtns
	*
	* INTERNAL function to sync 'pin buttons' when pane is opened or closed
	* Unpinned means the pane is 'sliding' - ie, over-top of the adjacent panes
	*
	* @callers  open(), close()
	* @params  pane   These are the params returned to callbacks by layout()
	* @params  doPin  True means set the pin 'down', False means 'up'
	*/
	function syncPinBtns (pane, doPin) {
		$.each(c[pane].pins, function (i, selector) {
			setPinState($(selector), pane, doPin);
		});
	};

	/**
	* setPinState
	*
	* Change the class of the pin button to make it look 'up' or 'down'
	*
	* @callers  addPinBtn(), syncPinBtns()
	* @param Element  $Pin		The pin-span element in a jQuery wrapper
	* @param Boolean  doPin		True = set the pin 'down', False = set it 'up'
	* @param String   pinClass	The root classname for pins - will add '-up' or '-down' suffix
	*/
	function setPinState ($Pin, pane, doPin) {
		var updown = $Pin.attr("pin");
		if (updown && doPin == (updown=="down")) return; // already in correct state
		var
			root	= options[pane].buttonClass
		,	class1	= root +"-pin"
		,	class2	= class1 +"-"+ pane
		,	UP1		= class1 + "-up"
		,	UP2		= class2 + "-up"
		,	DN1		= class1 + "-down"
		,	DN2		= class2 + "-down"
		;
		$Pin
			.attr("pin", doPin ? "down" : "up") // logic
			.attr("title", doPin ? "Un-Pin" : "Pin")
			.removeClass( doPin ? UP1 : DN1 ) 
			.removeClass( doPin ? UP2 : DN2 ) 
			.addClass( doPin ? DN1 : UP1 ) 
			.addClass( doPin ? DN2 : UP2 ) 
		;
	};


/*
 * ###########################
 * CREATE/RETURN BORDER-LAYOUT
 * ###########################
 */

	// init global vars
	var 
		$Container = $(this).css({ overflow: "hidden" }) // Container elem
	,	$Ps		= {} // Panes x4	- set in initPanes()
	,	$Cs		= {} // Content x4	- set in initPanes()
	,	$Rs		= {} // Resizers x4	- set in initHandles()
	,	$Ts		= {} // Togglers x4	- set in initHandles()
	//	object aliases
	,	c		= config // alias for config hash
	,	cDims	= state.container // alias for easy access to 'container dimensions'
	;

	// create the border layout NOW
	create();

	// return object pointers to expose data & option Properties, and primary action Methods
	return {
		options:		options			// property - options hash
	,	state:			state			// property - dimensions hash
	,	panes:			$Ps				// property - object pointers for ALL panes: panes.north, panes.center
	,	toggle:			toggle			// method - pass a 'pane' ("north", "west", etc)
	,	open:			open			// method - ditto
	,	close:			close			// method - ditto
	,	hide:			hide			// method - ditto
	,	show:			show			// method - ditto
	,	resizeContent:	sizeContent		// method - ditto
	,	sizePane:		sizePane		// method - pass a 'pane' AND a 'size' in pixels
	,	resizeAll:		resizeAll		// method - no parameters
	,	addToggleBtn:	addToggleBtn	// utility - pass element selector and 'pane'
	,	addOpenBtn:		addOpenBtn		// utility - ditto
	,	addCloseBtn:	addCloseBtn		// utility - ditto
	,	addPinBtn:		addPinBtn		// utility - ditto
	,	allowOverflow:	allowOverflow	// utility - pass calling element
	,	resetOverflow:	resetOverflow	// utility - ditto
	,	cssWidth:		cssW
	,	cssHeight:		cssH
	};

}
})( jQuery );(function($){  
    $.fn.listSort = function(options) {  
        
        var defaults = {
            value: "label",
            descending: true,
            attribute: undefined
        };
        
        var options = $.extend(defaults, options); 
        
        function sortList(ul, child, descending, attribute) {
            
            var items = ul.children("li");
    
            items.sort(function(x, y){
                var ret;
                var xVal, yVal;

                if( attribute != null ){
                	xVal = $(x).find(child).attr( attribute );
                	yVal = $(y).find(child).attr( attribute );
                } else {
                	xVal = $(x).find(child).text() || "";
                    yVal = $(y).find(child).text() || "";
                }
                
                // use integer values if possible
                var xFloat = parseFloat(xVal);
                var yFloat = parseFloat(yVal);
                
                if( isFinite(xFloat) && isFinite(yFloat) ) {
                    xVal = xFloat;
                    yVal = yFloat;
                } else {               
                    xVal = $.trim(xVal).toLowerCase();
                    yVal = $.trim(yVal).toLowerCase();
                }
                
                if( xVal < yVal ) {
                    ret = -1;
                } else if ( xVal > yVal ) {
                    ret =  1;
                } else {
                    ret = 0;
                }
                
                if( ! descending ) {
                    return ret;
                } else {
                    return -1 * ret;
                }
            });
            
            $.each(items, function(i, li){
                ul.append(li);
            });
            
            items.children("ul").each(function(){
                sortList( $(this), child, descending, attribute);
            });
    
        }
        
        return this.each(function() {  
            sortList( $(this), options.value, options.descending, options.attribute );
        });  
    };  
})(jQuery);  /* (c) 2009
   AUTHORS: Max Franz
   LICENSE: TODO
*/

;(function($){
   
    $.fn.menu = function(options) {  
        
        var defaults = {
            menuTitleClass: "ui-menu-title",
            menuItemClass: "ui-menu-item",
            selectedClass: "ui-state-active",
            togglableClass: "ui-menu-togglable",
            menuBarClass: "ui-menu-bar",
            topMenuClass: "ui-top-menu",
            subMenuClass: "ui-sub-menu",
            menuItemFirstClass: "ui-menu-item-first",
            menuItemLastClass: "ui-menu-item-last",
            menuItemParentClass: "ui-menu-item-parent",
            parentIconClass: "ui-menu-parent-icon",
            checkableClass: "ui-menu-checkable",
            checkIconClass: "ui-menu-check-icon",
            checkedClass: "ui-menu-checked",
            addArrow: true,
            titleArrowText: "<small>&nbsp;&#9660;</small>",
            menuZIndex: 9999,
            menuOpenDelay: 200, // in ms
            onMenuItemSelect: function(li){},
            onMenuItemDeselect: function(li){},
            onMenuItemOpen: function(li){},
            onMenuItemClose: function(li){},
            onMenuItemClick: function(li){},
            onMenuItemCheck: function(li){},
            onMenuItemUnCheck: function(li){}
        };
        
        var options = $.extend(defaults, options); 
        
        var timeout;
        
        function openMenu(li) {
            // add selected style
            li.addClass(options.selectedClass);
            
            // clicking one menu item (toggle or otherwise) closes siblings
            li.siblings("li").each(function(){
                if( $(this).hasClass(options.selectedClass) && ! $(this).hasClass(options.togglableClass) ) {
                    $(this).click();
                }
            });
            
            li.find("." + options.parentIconClass).addClass(options.selectedClass);
            li.children("ul").find("." + options.parentIconClass).removeClass(options.selectedClass);
            
            li.find("." + options.checkIconClass).addClass(options.selectedClass);
            li.children("ul").find("." + options.checkIconClass).removeClass(options.selectedClass);
        
            (! options.onMenuItemSelect) || options.onMenuItemSelect(li);
            
            // opening sub menu is delayed
            timeout = setTimeout(function(){
                var children = false;
                
                // open sub menus
                li.children("ul").show().each(function(){
                    
                    var maxWidth = 0;
                    
                    $(this).css("width", "100%");
                    
                    $(this).children("li").each(function(){
                        $(this).css("display", "block").css("width", "auto");
                        maxWidth = Math.max( maxWidth, $(this).outerWidth() );
                    }).width(maxWidth);
                    
                    $(this).css("height", $(this).height());
                    
                    if( ! $(this).parent().hasClass(options.menuTitleClass) ) {
                        $(this).css( "left", Math.ceil( $(this).parent().outerWidth() ) );
                        $(this).css( "top", Math.ceil( $(this).parent().position().top ) );
                    } else {
                        $(this).css( "top", Math.ceil( $(this).parent().outerHeight() + $(this).parent().position().top ) );
                        $(this).css( "left", Math.ceil( $(this).parent().offset().left ) );
                    }
                    
                    // icon offsets
                    $(this).children("li").children("." + options.parentIconClass + ", ." + options.checkIconClass).each(function(){
                        var offsetY = ( ( $(this).closest("li").height() - $(this).height() ) / 2 );
                        
                        if( parseInt( $(this).css("margin-top") ) == 0 ) {
                            $(this).css( "margin-top", offsetY );
                        }
                        
                        if( $(this).hasClass(options.parentIconClass) ) {
                            var offsetX = $(this).closest("li").width() - $(this).width();
                            $(this).css("margin-left", offsetX);
                        }
                    });
                    
                    children = true;
                });
                
                if(children) {
                    (! options.onMenuItemOpen) || options.onMenuItemOpen(li);
                }
            }, (li.hasClass(options.menuTitleClass)) ? (0) : (options.menuOpenDelay) );
        }
        
        function closeMenu(li) {
            clearTimeout(timeout);
        
            var children = li.children("ul").length > 0;
        
            li.children("ul").hide();
            
            li.removeClass(options.selectedClass);
            
            li.find("." + options.parentIconClass).removeClass(options.selectedClass);
            li.find("." + options.checkIconClass).removeClass(options.selectedClass);
            
            if( li.hasClass(options.togglableClass) ) {
                li.siblings("li").each(function(){
                    if( $(this).hasClass(options.selectedClass) && ! $(this).hasClass(options.togglableClass) ) {
                        $(this).click();
                    }
                });
            }
            
            (! options.onMenuItemDeselect) || options.onMenuItemDeselect(li);
            if(children) {
                (! options.onMenuItemClose) || options.onMenuItemClose(li);
            }
        }
        
        return this.each(function() {
            var ul = $(this).children("ul:first");
        
            // add style classes
            $(this).addClass("ui-corner-all outline ui-menu ui-widget ui-widget-content");
            ul.addClass("ui-menu-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-top");
            ul.addClass(options.menuBarClass);
            ul.children("li").addClass("ui-state-default ui-corner-all");
            ul.find("ul").each(function(){                
                $(this).children("li:first").addClass("ui-corner-top").addClass(options.menuItemFirstClass);
                $(this).children("li:last").addClass("ui-corner-bottom").addClass(options.menuItemLastClass);
                
                $(this).children("li").each(function(){
                    if( $(this).children("ul").length > 0 ) {
                        $(this).addClass(options.menuItemParentClass);
                        $(this).prepend("<div style=\"position: absolute\" class=\"" + options.parentIconClass + "\"></div>");
                    }
                    
                    if( $(this).hasClass(options.checkableClass) ) {
                        $(this).prepend("<div style=\"position: absolute\" class=\"" + options.checkIconClass + "\"></div>");
                        
                        $(this).click(function(){
                            var check = $(this).children("." + options.checkIconClass);
                            
                            check.toggleClass(options.checkedClass);
                            
                            if( check.hasClass(options.checkedClass) ) {
                                (! options.onMenuItemCheck) || options.onMenuItemCheck( $(this) );
                            } else {
                                (! options.onMenuItemCheck) || options.onMenuItemUncheck( $(this) );
                            }
                        });
                    }
                });
            });
            
            // only top level are menu items
            ul.find("li").addClass(options.menuItemClass);
            ul.children("li").removeClass(options.menuItemClass).addClass(options.menuTitleClass);
            ul.children("li").each(function(){
                if( $(this).children("ul").length > 0 ) {
                    $(this).find("label:first").append(options.titleArrowText);
                } else {
                    $(this).addClass(options.togglableClass);
                }
            });
            
            // differentiate top level and sub menus
            ul.find("ul").each(function(){
                if( $(this).parent().hasClass(options.menuTitleClass) ) {
                    $(this).addClass(options.topMenuClass);
                } else {
                    $(this).addClass(options.subMenuClass);
                }
            });
            
            // remove corner from top level menus
            ul.find("." + options.topMenuClass).children("." + options.menuItemFirstClass);
            
            // set position and index
            ul.find("ul").css("position", "absolute").css("z-index", options.menuZIndex).hide();

            ul.children("li").toggle(function(){
                openMenu( $(this) );
            }, function(){
                closeMenu( $(this) );
            });
            
            ul.find("." + options.menuItemClass).mouseenter(function(){
                openMenu( $(this) );
            }).mouseleave(function(){
                closeMenu( $(this) );
            });
            
            ul.find("li").click(function(){
                // don't send parent clicks
                if( $(this).find("." + options.menuItemClass + "." + options.selectedClass).length > 0 ) {
                    return;
                }
            
                // if it's not selected, we've clicked it, not the user
                // BUT always send toggles
                if( $(this).hasClass(options.selectedClass) || $(this).hasClass(options.togglableClass) ) {
                    (! options.onMenuItemClick) || options.onMenuItemClick( $(this) );
                }
            });
            
            function closeAll() {
                ul.find("." + options.selectedClass).reverse().each(function(){
                    if( ! $(this).hasClass(options.togglableClass) ) {
                    	closeMenu( $(this) );
                    }
                });
            }
            
            $("html").click(function(){
                closeAll();
            });
        });  
    };  
})(jQuery);  /*
 * jquery.qtip. The jQuery tooltip plugin
 *
 * Copyright (c) 2009 Craig Thompson
 * http://craigsworks.com
 *
 * Licensed under MIT
 * http://www.opensource.org/licenses/mit-license.php
 *
 * Launch  : February 2009
 * Version : 1.0.0-rc3
 * Released: Tuesday 12th May, 2009 - 00:00
 * Debug: jquery.qtip.debug.js
 */
(function(f){f.fn.qtip=function(B,u){var y,t,A,s,x,w,v,z;if(typeof B=="string"){if(typeof f(this).data("qtip")!=="object"){f.fn.qtip.log.error.call(self,1,f.fn.qtip.constants.NO_TOOLTIP_PRESENT,false)}if(B=="api"){return f(this).data("qtip").interfaces[f(this).data("qtip").current]}else{if(B=="interfaces"){return f(this).data("qtip").interfaces}}}else{if(!B){B={}}if(typeof B.content!=="object"||(B.content.jquery&&B.content.length>0)){B.content={text:B.content}}if(typeof B.content.title!=="object"){B.content.title={text:B.content.title}}if(typeof B.position!=="object"){B.position={corner:B.position}}if(typeof B.position.corner!=="object"){B.position.corner={target:B.position.corner,tooltip:B.position.corner}}if(typeof B.show!=="object"){B.show={when:B.show}}if(typeof B.show.when!=="object"){B.show.when={event:B.show.when}}if(typeof B.show.effect!=="object"){B.show.effect={type:B.show.effect}}if(typeof B.hide!=="object"){B.hide={when:B.hide}}if(typeof B.hide.when!=="object"){B.hide.when={event:B.hide.when}}if(typeof B.hide.effect!=="object"){B.hide.effect={type:B.hide.effect}}if(typeof B.style!=="object"){B.style={name:B.style}}B.style=c(B.style);s=f.extend(true,{},f.fn.qtip.defaults,B);s.style=a.call({options:s},s.style);s.user=f.extend(true,{},B)}return f(this).each(function(){if(typeof B=="string"){w=B.toLowerCase();A=f(this).qtip("interfaces");if(typeof A=="object"){if(u===true&&w=="destroy"){while(A.length>0){A[A.length-1].destroy()}}else{if(u!==true){A=[f(this).qtip("api")]}for(y=0;y<A.length;y++){if(w=="destroy"){A[y].destroy()}else{if(A[y].status.rendered===true){if(w=="show"){A[y].show()}else{if(w=="hide"){A[y].hide()}else{if(w=="focus"){A[y].focus()}else{if(w=="disable"){A[y].disable(true)}else{if(w=="enable"){A[y].disable(false)}}}}}}}}}}}else{v=f.extend(true,{},s);v.hide.effect.length=s.hide.effect.length;v.show.effect.length=s.show.effect.length;if(v.position.container===false){v.position.container=f(document.body)}if(v.position.target===false){v.position.target=f(this)}if(v.show.when.target===false){v.show.when.target=f(this)}if(v.hide.when.target===false){v.hide.when.target=f(this)}t=f.fn.qtip.interfaces.length;for(y=0;y<t;y++){if(typeof f.fn.qtip.interfaces[y]=="undefined"){t=y;break}}x=new d(f(this),v,t);f.fn.qtip.interfaces[t]=x;if(typeof f(this).data("qtip")=="object"){if(typeof f(this).attr("qtip")==="undefined"){f(this).data("qtip").current=f(this).data("qtip").interfaces.length}f(this).data("qtip").interfaces.push(x)}else{f(this).data("qtip",{current:0,interfaces:[x]})}if(v.content.prerender===false&&v.show.when.event!==false&&v.show.ready!==true){v.show.when.target.bind(v.show.when.event+".qtip-"+t+"-create",{qtip:t},function(C){z=f.fn.qtip.interfaces[C.data.qtip];z.options.show.when.target.unbind(z.options.show.when.event+".qtip-"+C.data.qtip+"-create");z.cache.mouse={x:C.pageX,y:C.pageY};p.call(z);z.options.show.when.target.trigger(z.options.show.when.event)})}else{x.cache.mouse={x:v.show.when.target.offset().left,y:v.show.when.target.offset().top};p.call(x)}}})};function d(u,t,v){var s=this;s.id=v;s.options=t;s.status={animated:false,rendered:false,disabled:false,focused:false};s.elements={target:u.addClass(s.options.style.classes.target),tooltip:null,wrapper:null,content:null,contentWrapper:null,title:null,button:null,tip:null,bgiframe:null};s.cache={mouse:{},position:{},toggle:0};s.timers={};f.extend(s,s.options.api,{show:function(y){var x,z;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"show")}if(s.elements.tooltip.css("display")!=="none"){return s}s.elements.tooltip.stop(true,false);x=s.beforeShow.call(s,y);if(x===false){return s}function w(){if(s.options.position.type!=="static"){s.focus()}s.onShow.call(s,y);if(f.browser.msie){s.elements.tooltip.get(0).style.removeAttribute("filter")}}s.cache.toggle=1;if(s.options.position.type!=="static"){s.updatePosition(y,(s.options.show.effect.length>0))}if(typeof s.options.show.solo=="object"){z=f(s.options.show.solo)}else{if(s.options.show.solo===true){z=f("div.qtip").not(s.elements.tooltip)}}if(z){z.each(function(){if(f(this).qtip("api").status.rendered===true){f(this).qtip("api").hide()}})}if(typeof s.options.show.effect.type=="function"){s.options.show.effect.type.call(s.elements.tooltip,s.options.show.effect.length);s.elements.tooltip.queue(function(){w();f(this).dequeue()})}else{switch(s.options.show.effect.type.toLowerCase()){case"fade":s.elements.tooltip.fadeIn(s.options.show.effect.length,w);break;case"slide":s.elements.tooltip.slideDown(s.options.show.effect.length,function(){w();if(s.options.position.type!=="static"){s.updatePosition(y,true)}});break;case"grow":s.elements.tooltip.show(s.options.show.effect.length,w);break;default:s.elements.tooltip.show(null,w);break}s.elements.tooltip.addClass(s.options.style.classes.active)}return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_SHOWN,"show")},hide:function(y){var x;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"hide")}else{if(s.elements.tooltip.css("display")==="none"){return s}}clearTimeout(s.timers.show);s.elements.tooltip.stop(true,false);x=s.beforeHide.call(s,y);if(x===false){return s}function w(){s.onHide.call(s,y)}s.cache.toggle=0;if(typeof s.options.hide.effect.type=="function"){s.options.hide.effect.type.call(s.elements.tooltip,s.options.hide.effect.length);s.elements.tooltip.queue(function(){w();f(this).dequeue()})}else{switch(s.options.hide.effect.type.toLowerCase()){case"fade":s.elements.tooltip.fadeOut(s.options.hide.effect.length,w);break;case"slide":s.elements.tooltip.slideUp(s.options.hide.effect.length,w);break;case"grow":s.elements.tooltip.hide(s.options.hide.effect.length,w);break;default:s.elements.tooltip.hide(null,w);break}s.elements.tooltip.removeClass(s.options.style.classes.active)}return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_HIDDEN,"hide")},updatePosition:function(w,x){var C,G,L,J,H,E,y,I,B,D,K,A,F,z;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"updatePosition")}else{if(s.options.position.type=="static"){return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.CANNOT_POSITION_STATIC,"updatePosition")}}G={position:{left:0,top:0},dimensions:{height:0,width:0},corner:s.options.position.corner.target};L={position:s.getPosition(),dimensions:s.getDimensions(),corner:s.options.position.corner.tooltip};if(s.options.position.target!=="mouse"){if(s.options.position.target.get(0).nodeName.toLowerCase()=="area"){J=s.options.position.target.attr("coords").split(",");for(C=0;C<J.length;C++){J[C]=parseInt(J[C])}H=s.options.position.target.parent("map").attr("name");E=f('img[usemap="#'+H+'"]:first').offset();G.position={left:Math.floor(E.left+J[0]),top:Math.floor(E.top+J[1])};switch(s.options.position.target.attr("shape").toLowerCase()){case"rect":G.dimensions={width:Math.ceil(Math.abs(J[2]-J[0])),height:Math.ceil(Math.abs(J[3]-J[1]))};break;case"circle":G.dimensions={width:J[2]+1,height:J[2]+1};break;case"poly":G.dimensions={width:J[0],height:J[1]};for(C=0;C<J.length;C++){if(C%2==0){if(J[C]>G.dimensions.width){G.dimensions.width=J[C]}if(J[C]<J[0]){G.position.left=Math.floor(E.left+J[C])}}else{if(J[C]>G.dimensions.height){G.dimensions.height=J[C]}if(J[C]<J[1]){G.position.top=Math.floor(E.top+J[C])}}}G.dimensions.width=G.dimensions.width-(G.position.left-E.left);G.dimensions.height=G.dimensions.height-(G.position.top-E.top);break;default:return f.fn.qtip.log.error.call(s,4,f.fn.qtip.constants.INVALID_AREA_SHAPE,"updatePosition");break}G.dimensions.width-=2;G.dimensions.height-=2}else{if(s.options.position.target.add(document.body).length===1){G.position={left:f(document).scrollLeft(),top:f(document).scrollTop()};G.dimensions={height:f(window).height(),width:f(window).width()}}else{if(typeof s.options.position.target.attr("qtip")!=="undefined"){G.position=s.options.position.target.qtip("api").cache.position}else{G.position=s.options.position.target.offset()}G.dimensions={height:s.options.position.target.outerHeight(),width:s.options.position.target.outerWidth()}}}y=f.extend({},G.position);if(G.corner.search(/right/i)!==-1){y.left+=G.dimensions.width}if(G.corner.search(/bottom/i)!==-1){y.top+=G.dimensions.height}if(G.corner.search(/((top|bottom)Middle)|center/)!==-1){y.left+=(G.dimensions.width/2)}if(G.corner.search(/((left|right)Middle)|center/)!==-1){y.top+=(G.dimensions.height/2)}}else{G.position=y={left:s.cache.mouse.x,top:s.cache.mouse.y};G.dimensions={height:1,width:1}}if(L.corner.search(/right/i)!==-1){y.left-=L.dimensions.width}if(L.corner.search(/bottom/i)!==-1){y.top-=L.dimensions.height}if(L.corner.search(/((top|bottom)Middle)|center/)!==-1){y.left-=(L.dimensions.width/2)}if(L.corner.search(/((left|right)Middle)|center/)!==-1){y.top-=(L.dimensions.height/2)}I=(f.browser.msie)?1:0;B=(f.browser.msie&&parseInt(f.browser.version.charAt(0))===6)?1:0;if(s.options.style.border.radius>0){if(L.corner.search(/Left/)!==-1){y.left-=s.options.style.border.radius}else{if(L.corner.search(/Right/)!==-1){y.left+=s.options.style.border.radius}}if(L.corner.search(/Top/)!==-1){y.top-=s.options.style.border.radius}else{if(L.corner.search(/Bottom/)!==-1){y.top+=s.options.style.border.radius}}}if(I){if(L.corner.search(/top/)!==-1){y.top-=I}else{if(L.corner.search(/bottom/)!==-1){y.top+=I}}if(L.corner.search(/left/)!==-1){y.left-=I}else{if(L.corner.search(/right/)!==-1){y.left+=I}}if(L.corner.search(/leftMiddle|rightMiddle/)!==-1){y.top-=1}}if(s.options.position.adjust.screen===true){y=o.call(s,y,G,L)}if(s.options.position.target==="mouse"&&s.options.position.adjust.mouse===true){if(s.options.position.adjust.screen===true&&s.elements.tip){K=s.elements.tip.attr("rel")}else{K=s.options.position.corner.tooltip}y.left+=(K.search(/right/i)!==-1)?-6:6;y.top+=(K.search(/bottom/i)!==-1)?-6:6}if(!s.elements.bgiframe&&f.browser.msie&&parseInt(f.browser.version.charAt(0))==6){f("select, object").each(function(){A=f(this).offset();A.bottom=A.top+f(this).height();A.right=A.left+f(this).width();if(y.top+L.dimensions.height>=A.top&&y.left+L.dimensions.width>=A.left){k.call(s)}})}y.left+=s.options.position.adjust.x;y.top+=s.options.position.adjust.y;F=s.getPosition();if(y.left!=F.left||y.top!=F.top){z=s.beforePositionUpdate.call(s,w);if(z===false){return s}s.cache.position=y;if(x===true){s.status.animated=true;s.elements.tooltip.animate(y,200,"swing",function(){s.status.animated=false})}else{s.elements.tooltip.css(y)}s.onPositionUpdate.call(s,w);if(typeof w!=="undefined"&&w.type&&w.type!=="mousemove"){f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_POSITION_UPDATED,"updatePosition")}}return s},updateWidth:function(w){var x;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"updateWidth")}else{if(w&&typeof w!=="number"){return f.fn.qtip.log.error.call(s,2,"newWidth must be of type number","updateWidth")}}x=s.elements.contentWrapper.siblings().add(s.elements.tip).add(s.elements.button);if(!w){if(typeof s.options.style.width.value=="number"){w=s.options.style.width.value}else{s.elements.tooltip.css({width:"auto"});x.hide();if(f.browser.msie){s.elements.wrapper.add(s.elements.contentWrapper.children()).css({zoom:"normal"})}w=s.getDimensions().width+1;if(!s.options.style.width.value){if(w>s.options.style.width.max){w=s.options.style.width.max}if(w<s.options.style.width.min){w=s.options.style.width.min}}}}if(w%2!==0){w-=1}s.elements.tooltip.width(w);x.show();if(s.options.style.border.radius){s.elements.tooltip.find(".qtip-betweenCorners").each(function(y){f(this).width(w-(s.options.style.border.radius*2))})}if(f.browser.msie){s.elements.wrapper.add(s.elements.contentWrapper.children()).css({zoom:"1"});s.elements.wrapper.width(w);if(s.elements.bgiframe){s.elements.bgiframe.width(w).height(s.getDimensions.height)}}return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_WIDTH_UPDATED,"updateWidth")},updateStyle:function(w){var z,A,x,y,B;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"updateStyle")}else{if(typeof w!=="string"||!f.fn.qtip.styles[w]){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.STYLE_NOT_DEFINED,"updateStyle")}}s.options.style=a.call(s,f.fn.qtip.styles[w],s.options.user.style);s.elements.content.css(q(s.options.style));if(s.options.content.title.text!==false){s.elements.title.css(q(s.options.style.title,true))}s.elements.contentWrapper.css({borderColor:s.options.style.border.color});if(s.options.style.tip.corner!==false){if(f("<canvas>").get(0).getContext){z=s.elements.tooltip.find(".qtip-tip canvas:first");x=z.get(0).getContext("2d");x.clearRect(0,0,300,300);y=z.parent("div[rel]:first").attr("rel");B=b(y,s.options.style.tip.size.width,s.options.style.tip.size.height);h.call(s,z,B,s.options.style.tip.color||s.options.style.border.color)}else{if(f.browser.msie){z=s.elements.tooltip.find('.qtip-tip [nodeName="shape"]');z.attr("fillcolor",s.options.style.tip.color||s.options.style.border.color)}}}if(s.options.style.border.radius>0){s.elements.tooltip.find(".qtip-betweenCorners").css({backgroundColor:s.options.style.border.color});if(f("<canvas>").get(0).getContext){A=g(s.options.style.border.radius);s.elements.tooltip.find(".qtip-wrapper canvas").each(function(){x=f(this).get(0).getContext("2d");x.clearRect(0,0,300,300);y=f(this).parent("div[rel]:first").attr("rel");r.call(s,f(this),A[y],s.options.style.border.radius,s.options.style.border.color)})}else{if(f.browser.msie){s.elements.tooltip.find('.qtip-wrapper [nodeName="arc"]').each(function(){f(this).attr("fillcolor",s.options.style.border.color)})}}}return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_STYLE_UPDATED,"updateStyle")},updateContent:function(A,y){var z,x,w;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"updateContent")}else{if(!A){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.NO_CONTENT_PROVIDED,"updateContent")}}z=s.beforeContentUpdate.call(s,A);if(typeof z=="string"){A=z}else{if(z===false){return}}if(f.browser.msie){s.elements.contentWrapper.children().css({zoom:"normal"})}if(A.jquery&&A.length>0){A.clone(true).appendTo(s.elements.content).show()}else{s.elements.content.html(A)}x=s.elements.content.find("img[complete=false]");if(x.length>0){w=0;x.each(function(C){f('<img src="'+f(this).attr("src")+'" />').load(function(){if(++w==x.length){B()}})})}else{B()}function B(){s.updateWidth();if(y!==false){if(s.options.position.type!=="static"){s.updatePosition(s.elements.tooltip.is(":visible"),true)}if(s.options.style.tip.corner!==false){n.call(s)}}}s.onContentUpdate.call(s);return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_CONTENT_UPDATED,"loadContent")},loadContent:function(w,z,A){var y;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"loadContent")}y=s.beforeContentLoad.call(s);if(y===false){return s}if(A=="post"){f.post(w,z,x)}else{f.get(w,z,x)}function x(B){s.onContentLoad.call(s);f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_CONTENT_LOADED,"loadContent");s.updateContent(B)}return s},updateTitle:function(w){if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"updateTitle")}else{if(!w){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.NO_CONTENT_PROVIDED,"updateTitle")}}returned=s.beforeTitleUpdate.call(s);if(returned===false){return s}if(s.elements.button){s.elements.button=s.elements.button.clone(true)}s.elements.title.html(w);if(s.elements.button){s.elements.title.prepend(s.elements.button)}s.onTitleUpdate.call(s);return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_TITLE_UPDATED,"updateTitle")},focus:function(A){var y,x,w,z;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"focus")}else{if(s.options.position.type=="static"){return f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.CANNOT_FOCUS_STATIC,"focus")}}y=parseInt(s.elements.tooltip.css("z-index"));x=6000+f("div.qtip[qtip]").length-1;if(!s.status.focused&&y!==x){z=s.beforeFocus.call(s,A);if(z===false){return s}f("div.qtip[qtip]").not(s.elements.tooltip).each(function(){if(f(this).qtip("api").status.rendered===true){w=parseInt(f(this).css("z-index"));if(typeof w=="number"&&w>-1){f(this).css({zIndex:parseInt(f(this).css("z-index"))-1})}f(this).qtip("api").status.focused=false}});s.elements.tooltip.css({zIndex:x});s.status.focused=true;s.onFocus.call(s,A);f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_FOCUSED,"focus")}return s},disable:function(w){if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"disable")}if(w){if(!s.status.disabled){s.status.disabled=true;f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_DISABLED,"disable")}else{f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.TOOLTIP_ALREADY_DISABLED,"disable")}}else{if(s.status.disabled){s.status.disabled=false;f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_ENABLED,"disable")}else{f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.TOOLTIP_ALREADY_ENABLED,"disable")}}return s},destroy:function(){var w,x,y;x=s.beforeDestroy.call(s);if(x===false){return s}if(s.status.rendered){s.options.show.when.target.unbind("mousemove.qtip",s.updatePosition);s.options.show.when.target.unbind("mouseout.qtip",s.hide);s.options.show.when.target.unbind(s.options.show.when.event+".qtip");s.options.hide.when.target.unbind(s.options.hide.when.event+".qtip");s.elements.tooltip.unbind(s.options.hide.when.event+".qtip");s.elements.tooltip.unbind("mouseover.qtip",s.focus);s.elements.tooltip.remove()}else{s.options.show.when.target.unbind(s.options.show.when.event+".qtip-create")}if(typeof s.elements.target.data("qtip")=="object"){y=s.elements.target.data("qtip").interfaces;if(typeof y=="object"&&y.length>0){for(w=0;w<y.length-1;w++){if(y[w].id==s.id){y.splice(w,1)}}}}delete f.fn.qtip.interfaces[s.id];if(typeof y=="object"&&y.length>0){s.elements.target.data("qtip").current=y.length-1}else{s.elements.target.removeData("qtip")}s.onDestroy.call(s);f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_DESTROYED,"destroy");return s.elements.target},getPosition:function(){var w,x;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"getPosition")}w=(s.elements.tooltip.css("display")!=="none")?false:true;if(w){s.elements.tooltip.css({visiblity:"hidden"}).show()}x=s.elements.tooltip.offset();if(w){s.elements.tooltip.css({visiblity:"visible"}).hide()}return x},getDimensions:function(){var w,x;if(!s.status.rendered){return f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.TOOLTIP_NOT_RENDERED,"getDimensions")}w=(!s.elements.tooltip.is(":visible"))?true:false;if(w){s.elements.tooltip.css({visiblity:"hidden"}).show()}x={height:s.elements.tooltip.outerHeight(),width:s.elements.tooltip.outerWidth()};if(w){s.elements.tooltip.css({visiblity:"visible"}).hide()}return x}})}function p(){var s,w,u,t,v,y,x;s=this;s.beforeRender.call(s);s.status.rendered=true;s.elements.tooltip='<div qtip="'+s.id+'" class="qtip '+(s.options.style.classes.tooltip||s.options.style)+'"style="display:none; -moz-border-radius:0; -webkit-border-radius:0; border-radius:0;position:'+s.options.position.type+';">  <div class="qtip-wrapper" style="position:relative; overflow:hidden; text-align:left;">    <div class="qtip-contentWrapper" style="overflow:hidden;">       <div class="qtip-content '+s.options.style.classes.content+'"></div></div></div></div>';s.elements.tooltip=f(s.elements.tooltip);s.elements.tooltip.appendTo(s.options.position.container);s.elements.tooltip.data("qtip",{current:0,interfaces:[s]});s.elements.wrapper=s.elements.tooltip.children("div:first");s.elements.contentWrapper=s.elements.wrapper.children("div:first").css({background:s.options.style.background});s.elements.content=s.elements.contentWrapper.children("div:first").css(q(s.options.style));if(f.browser.msie){s.elements.wrapper.add(s.elements.content).css({zoom:1})}if(s.options.hide.when.event=="unfocus"){s.elements.tooltip.attr("unfocus",true)}if(typeof s.options.style.width.value=="number"){s.updateWidth()}if(f("<canvas>").get(0).getContext||f.browser.msie){if(s.options.style.border.radius>0){m.call(s)}else{s.elements.contentWrapper.css({border:s.options.style.border.width+"px solid "+s.options.style.border.color})}if(s.options.style.tip.corner!==false){e.call(s)}}else{s.elements.contentWrapper.css({border:s.options.style.border.width+"px solid "+s.options.style.border.color});s.options.style.border.radius=0;s.options.style.tip.corner=false;f.fn.qtip.log.error.call(s,2,f.fn.qtip.constants.CANVAS_VML_NOT_SUPPORTED,"render")}if((typeof s.options.content.text=="string"&&s.options.content.text.length>0)||(s.options.content.text.jquery&&s.options.content.text.length>0)){u=s.options.content.text}else{if(typeof s.elements.target.attr("title")=="string"&&s.elements.target.attr("title").length>0){u=s.elements.target.attr("title").replace("\\n","<br />");s.elements.target.attr("title","")}else{if(typeof s.elements.target.attr("alt")=="string"&&s.elements.target.attr("alt").length>0){u=s.elements.target.attr("alt").replace("\\n","<br />");s.elements.target.attr("alt","")}else{u=" ";f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.NO_VALID_CONTENT,"render")}}}if(s.options.content.title.text!==false){j.call(s)}s.updateContent(u);l.call(s);if(s.options.show.ready===true){s.show()}if(s.options.content.url!==false){t=s.options.content.url;v=s.options.content.data;y=s.options.content.method||"get";s.loadContent(t,v,y)}s.onRender.call(s);f.fn.qtip.log.error.call(s,1,f.fn.qtip.constants.EVENT_RENDERED,"render")}function m(){var F,z,t,B,x,E,u,G,D,y,w,C,A,s,v;F=this;F.elements.wrapper.find(".qtip-borderBottom, .qtip-borderTop").remove();t=F.options.style.border.width;B=F.options.style.border.radius;x=F.options.style.border.color||F.options.style.tip.color;E=g(B);u={};for(z in E){u[z]='<div rel="'+z+'" style="'+((z.search(/Left/)!==-1)?"left":"right")+":0; position:absolute; height:"+B+"px; width:"+B+'px; overflow:hidden; line-height:0.1px; font-size:1px">';if(f("<canvas>").get(0).getContext){u[z]+='<canvas height="'+B+'" width="'+B+'" style="vertical-align: top"></canvas>'}else{if(f.browser.msie){G=B*2+3;u[z]+='<v:arc stroked="false" fillcolor="'+x+'" startangle="'+E[z][0]+'" endangle="'+E[z][1]+'" style="width:'+G+"px; height:"+G+"px; margin-top:"+((z.search(/bottom/)!==-1)?-2:-1)+"px; margin-left:"+((z.search(/Right/)!==-1)?E[z][2]-3.5:-1)+'px; vertical-align:top; display:inline-block; behavior:url(#default#VML)"></v:arc>'}}u[z]+="</div>"}D=F.getDimensions().width-(Math.max(t,B)*2);y='<div class="qtip-betweenCorners" style="height:'+B+"px; width:"+D+"px; overflow:hidden; background-color:"+x+'; line-height:0.1px; font-size:1px;">';w='<div class="qtip-borderTop" dir="ltr" style="height:'+B+"px; margin-left:"+B+'px; line-height:0.1px; font-size:1px; padding:0;">'+u.topLeft+u.topRight+y;F.elements.wrapper.prepend(w);C='<div class="qtip-borderBottom" dir="ltr" style="height:'+B+"px; margin-left:"+B+'px; line-height:0.1px; font-size:1px; padding:0;">'+u.bottomLeft+u.bottomRight+y;F.elements.wrapper.append(C);if(f("<canvas>").get(0).getContext){F.elements.wrapper.find("canvas").each(function(){A=E[f(this).parent("[rel]:first").attr("rel")];r.call(F,f(this),A,B,x)})}else{if(f.browser.msie){F.elements.tooltip.append('<v:image style="behavior:url(#default#VML);"></v:image>')}}s=Math.max(B,(B+(t-B)));v=Math.max(t-B,0);F.elements.contentWrapper.css({border:"0px solid "+x,borderWidth:v+"px "+s+"px"})}function r(u,w,s,t){var v=u.get(0).getContext("2d");v.fillStyle=t;v.beginPath();v.arc(w[0],w[1],s,0,Math.PI*2,false);v.fill()}function e(v){var t,s,x,u,w;t=this;if(t.elements.tip!==null){t.elements.tip.remove()}s=t.options.style.tip.color||t.options.style.border.color;if(t.options.style.tip.corner===false){return}else{if(!v){v=t.options.style.tip.corner}}x=b(v,t.options.style.tip.size.width,t.options.style.tip.size.height);t.elements.tip='<div class="'+t.options.style.classes.tip+'" dir="ltr" rel="'+v+'" style="position:absolute; height:'+t.options.style.tip.size.height+"px; width:"+t.options.style.tip.size.width+'px; margin:0 auto; line-height:0.1px; font-size:1px;">';if(f("<canvas>").get(0).getContext){t.elements.tip+='<canvas height="'+t.options.style.tip.size.height+'" width="'+t.options.style.tip.size.width+'"></canvas>'}else{if(f.browser.msie){u=t.options.style.tip.size.width+","+t.options.style.tip.size.height;w="m"+x[0][0]+","+x[0][1];w+=" l"+x[1][0]+","+x[1][1];w+=" "+x[2][0]+","+x[2][1];w+=" xe";t.elements.tip+='<v:shape fillcolor="'+s+'" stroked="false" filled="true" path="'+w+'" coordsize="'+u+'" style="width:'+t.options.style.tip.size.width+"px; height:"+t.options.style.tip.size.height+"px; line-height:0.1px; display:inline-block; behavior:url(#default#VML); vertical-align:"+((v.search(/top/)!==-1)?"bottom":"top")+'"></v:shape>';t.elements.tip+='<v:image style="behavior:url(#default#VML);"></v:image>';t.elements.contentWrapper.css("position","relative")}}t.elements.tooltip.prepend(t.elements.tip+"</div>");t.elements.tip=t.elements.tooltip.find("."+t.options.style.classes.tip).eq(0);if(f("<canvas>").get(0).getContext){h.call(t,t.elements.tip.find("canvas:first"),x,s)}if(v.search(/top/)!==-1&&f.browser.msie&&parseInt(f.browser.version.charAt(0))===6){t.elements.tip.css({marginTop:-4})}n.call(t,v)}function h(t,v,s){var u=t.get(0).getContext("2d");u.fillStyle=s;u.beginPath();u.moveTo(v[0][0],v[0][1]);u.lineTo(v[1][0],v[1][1]);u.lineTo(v[2][0],v[2][1]);u.fill()}function n(u){var t,w,s,x,v;t=this;if(t.options.style.tip.corner===false||!t.elements.tip){return}if(!u){u=t.elements.tip.attr("rel")}w=positionAdjust=(f.browser.msie)?1:0;t.elements.tip.css(u.match(/left|right|top|bottom/)[0],0);if(u.search(/top|bottom/)!==-1){if(f.browser.msie){if(parseInt(f.browser.version.charAt(0))===6){positionAdjust=(u.search(/top/)!==-1)?-3:1}else{positionAdjust=(u.search(/top/)!==-1)?1:2}}if(u.search(/Middle/)!==-1){t.elements.tip.css({left:"50%",marginLeft:-(t.options.style.tip.size.width/2)})}else{if(u.search(/Left/)!==-1){t.elements.tip.css({left:t.options.style.border.radius-w})}else{if(u.search(/Right/)!==-1){t.elements.tip.css({right:t.options.style.border.radius+w})}}}if(u.search(/top/)!==-1){t.elements.tip.css({top:-positionAdjust})}else{t.elements.tip.css({bottom:positionAdjust})}}else{if(u.search(/left|right/)!==-1){if(f.browser.msie){positionAdjust=(parseInt(f.browser.version.charAt(0))===6)?1:((u.search(/left/)!==-1)?1:2)}if(u.search(/Middle/)!==-1){t.elements.tip.css({top:"50%",marginTop:-(t.options.style.tip.size.height/2)})}else{if(u.search(/Top/)!==-1){t.elements.tip.css({top:t.options.style.border.radius-w})}else{if(u.search(/Bottom/)!==-1){t.elements.tip.css({bottom:t.options.style.border.radius+w})}}}if(u.search(/left/)!==-1){t.elements.tip.css({left:-positionAdjust})}else{t.elements.tip.css({right:positionAdjust})}}}s="padding-"+u.match(/left|right|top|bottom/)[0];x=t.options.style.tip.size[(s.search(/left|right/)!==-1)?"width":"height"];t.elements.tooltip.css("padding",0);t.elements.tooltip.css(s,x);if(f.browser.msie&&parseInt(f.browser.version.charAt(0))==6){v=parseInt(t.elements.tip.css("margin-top"))||0;v+=parseInt(t.elements.content.css("margin-top"))||0;t.elements.tip.css({marginTop:v})}}function j(){var s=this;if(s.elements.title!==null){s.elements.title.remove()}s.elements.title=f('<div class="'+s.options.style.classes.title+'">').css(q(s.options.style.title,true)).css({zoom:(f.browser.msie)?1:0}).prependTo(s.elements.contentWrapper);if(s.options.content.title.text){s.updateTitle.call(s,s.options.content.title.text)}if(s.options.content.title.button!==false&&typeof s.options.content.title.button=="string"){s.elements.button=f('<a class="'+s.options.style.classes.button+'" style="float:right; position: relative"></a>').css(q(s.options.style.button,true)).html(s.options.content.title.button).prependTo(s.elements.title).click(function(t){if(!s.status.disabled){s.hide(t)}})}}function l(){var t,v,u,s;t=this;v=t.options.show.when.target;u=t.options.hide.when.target;if(t.options.hide.fixed){u=u.add(t.elements.tooltip)}if(t.options.hide.when.event=="inactive"){s=["click","dblclick","mousedown","mouseup","mousemove","mouseout","mouseenter","mouseleave","mouseover"];function y(z){if(t.status.disabled===true){return}clearTimeout(t.timers.inactive);t.timers.inactive=setTimeout(function(){f(s).each(function(){u.unbind(this+".qtip-inactive");t.elements.content.unbind(this+".qtip-inactive")});t.hide(z)},t.options.hide.delay)}}else{if(t.options.hide.fixed===true){t.elements.tooltip.bind("mouseover.qtip",function(){if(t.status.disabled===true){return}clearTimeout(t.timers.hide)})}}function x(z){if(t.status.disabled===true){return}if(t.options.hide.when.event=="inactive"){f(s).each(function(){u.bind(this+".qtip-inactive",y);t.elements.content.bind(this+".qtip-inactive",y)});y()}clearTimeout(t.timers.show);clearTimeout(t.timers.hide);t.timers.show=setTimeout(function(){t.show(z)},t.options.show.delay)}function w(z){if(t.status.disabled===true){return}if(t.options.hide.fixed===true&&t.options.hide.when.event.search(/mouse(out|leave)/i)!==-1&&f(z.relatedTarget).parents("div.qtip[qtip]").length>0){z.stopPropagation();z.preventDefault();clearTimeout(t.timers.hide);return false}clearTimeout(t.timers.show);clearTimeout(t.timers.hide);t.elements.tooltip.stop(true,true);t.timers.hide=setTimeout(function(){t.hide(z)},t.options.hide.delay)}if((t.options.show.when.target.add(t.options.hide.when.target).length===1&&t.options.show.when.event==t.options.hide.when.event&&t.options.hide.when.event!=="inactive")||t.options.hide.when.event=="unfocus"){t.cache.toggle=0;v.bind(t.options.show.when.event+".qtip",function(z){if(t.cache.toggle==0){x(z)}else{w(z)}})}else{v.bind(t.options.show.when.event+".qtip",x);if(t.options.hide.when.event!=="inactive"){u.bind(t.options.hide.when.event+".qtip",w)}}if(t.options.position.type.search(/(fixed|absolute)/)!==-1){t.elements.tooltip.bind("mouseover.qtip",t.focus)}if(t.options.position.target==="mouse"&&t.options.position.type!=="static"){v.bind("mousemove.qtip",function(z){t.cache.mouse={x:z.pageX,y:z.pageY};if(t.status.disabled===false&&t.options.position.adjust.mouse===true&&t.options.position.type!=="static"&&t.elements.tooltip.css("display")!=="none"){t.updatePosition(z)}})}}function o(u,v,A){var z,s,x,y,t,w;z=this;if(A.corner=="center"){return v.position}s=f.extend({},u);y={x:false,y:false};t={left:(s.left<f.fn.qtip.cache.screen.scroll.left),right:(s.left+A.dimensions.width+2>=f.fn.qtip.cache.screen.width+f.fn.qtip.cache.screen.scroll.left),top:(s.top<f.fn.qtip.cache.screen.scroll.top),bottom:(s.top+A.dimensions.height+2>=f.fn.qtip.cache.screen.height+f.fn.qtip.cache.screen.scroll.top)};x={left:(t.left&&(A.corner.search(/right/i)!=-1||(A.corner.search(/right/i)==-1&&!t.right))),right:(t.right&&(A.corner.search(/left/i)!=-1||(A.corner.search(/left/i)==-1&&!t.left))),top:(t.top&&A.corner.search(/top/i)==-1),bottom:(t.bottom&&A.corner.search(/bottom/i)==-1)};if(x.left){if(z.options.position.target!=="mouse"){s.left=v.position.left+v.dimensions.width}else{s.left=z.cache.mouse.x}y.x="Left"}else{if(x.right){if(z.options.position.target!=="mouse"){s.left=v.position.left-A.dimensions.width}else{s.left=z.cache.mouse.x-A.dimensions.width}y.x="Right"}}if(x.top){if(z.options.position.target!=="mouse"){s.top=v.position.top+v.dimensions.height}else{s.top=z.cache.mouse.y}y.y="top"}else{if(x.bottom){if(z.options.position.target!=="mouse"){s.top=v.position.top-A.dimensions.height}else{s.top=z.cache.mouse.y-A.dimensions.height}y.y="bottom"}}if(s.left<0){s.left=u.left;y.x=false}if(s.top<0){s.top=u.top;y.y=false}if(z.options.style.tip.corner!==false){s.corner=new String(A.corner);if(y.x!==false){s.corner=s.corner.replace(/Left|Right|Middle/,y.x)}if(y.y!==false){s.corner=s.corner.replace(/top|bottom/,y.y)}if(s.corner!==z.elements.tip.attr("rel")){e.call(z,s.corner)}}return s}function q(u,t){var v,s;v=f.extend(true,{},u);for(s in v){if(t===true&&s.search(/(tip|classes)/i)!==-1){delete v[s]}else{if(!t&&s.search(/(width|border|tip|title|classes|user)/i)!==-1){delete v[s]}}}return v}function c(s){if(typeof s.tip!=="object"){s.tip={corner:s.tip}}if(typeof s.tip.size!=="object"){s.tip.size={width:s.tip.size,height:s.tip.size}}if(typeof s.border!=="object"){s.border={width:s.border}}if(typeof s.width!=="object"){s.width={value:s.width}}if(typeof s.width.max=="string"){s.width.max=parseInt(s.width.max.replace(/([0-9]+)/i,"$1"))}if(typeof s.width.min=="string"){s.width.min=parseInt(s.width.min.replace(/([0-9]+)/i,"$1"))}if(typeof s.tip.size.x=="number"){s.tip.size.width=s.tip.size.x;delete s.tip.size.x}if(typeof s.tip.size.y=="number"){s.tip.size.height=s.tip.size.y;delete s.tip.size.y}return s}function a(){var s,t,u,x,v,w;s=this;u=[true,{}];for(t=0;t<arguments.length;t++){u.push(arguments[t])}x=[f.extend.apply(f,u)];while(typeof x[0].name=="string"){x.unshift(c(f.fn.qtip.styles[x[0].name]))}x.unshift(true,{classes:{tooltip:"qtip-"+(arguments[0].name||"defaults")}},f.fn.qtip.styles.defaults);v=f.extend.apply(f,x);w=(f.browser.msie)?1:0;v.tip.size.width+=w;v.tip.size.height+=w;if(v.tip.size.width%2>0){v.tip.size.width+=1}if(v.tip.size.height%2>0){v.tip.size.height+=1}if(v.tip.corner===true){v.tip.corner=(s.options.position.corner.tooltip==="center")?false:s.options.position.corner.tooltip}return v}function b(v,u,t){var s={bottomRight:[[0,0],[u,t],[u,0]],bottomLeft:[[0,0],[u,0],[0,t]],topRight:[[0,t],[u,0],[u,t]],topLeft:[[0,0],[0,t],[u,t]],topMiddle:[[0,t],[u/2,0],[u,t]],bottomMiddle:[[0,0],[u,0],[u/2,t]],rightMiddle:[[0,0],[u,t/2],[0,t]],leftMiddle:[[u,0],[u,t],[0,t/2]]};s.leftTop=s.bottomRight;s.rightTop=s.bottomLeft;s.leftBottom=s.topRight;s.rightBottom=s.topLeft;return s[v]}function g(s){var t;if(f("<canvas>").get(0).getContext){t={topLeft:[s,s],topRight:[0,s],bottomLeft:[s,0],bottomRight:[0,0]}}else{if(f.browser.msie){t={topLeft:[-90,90,0],topRight:[-90,90,-s],bottomLeft:[90,270,0],bottomRight:[90,270,-s]}}}return t}function k(){var s,t,u;s=this;u=s.getDimensions();t='<iframe class="qtip-bgiframe" frameborder="0" tabindex="-1" src="javascript:false" style="display:block; position:absolute; z-index:-1; filter:alpha(opacity=\'0\'); border: 1px solid red; height:'+u.height+"px; width:"+u.width+'px" />';s.elements.bgiframe=s.elements.wrapper.prepend(t).children(".qtip-bgiframe:first")}f(document).ready(function(){f.fn.qtip.cache={screen:{scroll:{left:f(window).scrollLeft(),top:f(window).scrollTop()},width:f(window).width(),height:f(window).height()}};var s;f(window).bind("resize scroll",function(t){clearTimeout(s);s=setTimeout(function(){if(t.type==="scroll"){f.fn.qtip.cache.screen.scroll={left:f(window).scrollLeft(),top:f(window).scrollTop()}}else{f.fn.qtip.cache.screen.width=f(window).width();f.fn.qtip.cache.screen.height=f(window).height()}for(i=0;i<f.fn.qtip.interfaces.length;i++){var u=f.fn.qtip.interfaces[i];if(u.status.rendered===true&&(u.options.position.type!=="static"||u.options.position.adjust.scroll&&t.type==="scroll"||u.options.position.adjust.resize&&t.type==="resize")){u.updatePosition(t,true)}}},100)});f(document).bind("mousedown.qtip",function(t){if(f(t.target).parents("div.qtip").length===0){f(".qtip[unfocus]").each(function(){var u=f(this).qtip("api");if(f(this).is(":visible")&&!u.status.disabled&&f(t.target).add(u.elements.target).length>1){u.hide(t)}})}})});f.fn.qtip.interfaces=[];f.fn.qtip.log={error:function(){return this}};f.fn.qtip.constants={};f.fn.qtip.defaults={content:{prerender:false,text:false,url:false,data:null,title:{text:false,button:false}},position:{target:false,corner:{target:"bottomRight",tooltip:"topLeft"},adjust:{x:0,y:0,mouse:true,screen:false,scroll:true,resize:true},type:"absolute",container:false},show:{when:{target:false,event:"mouseover"},effect:{type:"fade",length:100},delay:140,solo:false,ready:false},hide:{when:{target:false,event:"mouseout"},effect:{type:"fade",length:100},delay:0,fixed:false},api:{beforeRender:function(){},onRender:function(){},beforePositionUpdate:function(){},onPositionUpdate:function(){},beforeShow:function(){},onShow:function(){},beforeHide:function(){},onHide:function(){},beforeContentUpdate:function(){},onContentUpdate:function(){},beforeContentLoad:function(){},onContentLoad:function(){},beforeTitleUpdate:function(){},onTitleUpdate:function(){},beforeDestroy:function(){},onDestroy:function(){},beforeFocus:function(){},onFocus:function(){}}};f.fn.qtip.styles={defaults:{background:"white",color:"#111",overflow:"hidden",textAlign:"left",width:{min:0,max:250},padding:"5px 9px",border:{width:1,radius:0,color:"#d3d3d3"},tip:{corner:false,color:false,size:{width:13,height:13},opacity:1},title:{background:"#e1e1e1",fontWeight:"bold",padding:"7px 12px"},button:{cursor:"pointer"},classes:{target:"",tip:"qtip-tip",title:"qtip-title",button:"qtip-button",content:"qtip-content",active:"qtip-active"}},cream:{border:{width:3,radius:0,color:"#F9E98E"},title:{background:"#F0DE7D",color:"#A27D35"},background:"#FBF7AA",color:"#A27D35",classes:{tooltip:"qtip-cream"}},light:{border:{width:3,radius:0,color:"#E2E2E2"},title:{background:"#f1f1f1",color:"#454545"},background:"white",color:"#454545",classes:{tooltip:"qtip-light"}},dark:{border:{width:3,radius:0,color:"#303030"},title:{background:"#404040",color:"#f3f3f3"},background:"#505050",color:"#f3f3f3",classes:{tooltip:"qtip-dark"}},red:{border:{width:3,radius:0,color:"#CE6F6F"},title:{background:"#f28279",color:"#9C2F2F"},background:"#F79992",color:"#9C2F2F",classes:{tooltip:"qtip-red"}},green:{border:{width:3,radius:0,color:"#A9DB66"},title:{background:"#b9db8c",color:"#58792E"},background:"#CDE6AC",color:"#58792E",classes:{tooltip:"qtip-green"}},blue:{border:{width:3,radius:0,color:"#ADD9ED"},title:{background:"#D0E9F5",color:"#5E99BD"},background:"#E5F6FE",color:"#4D9FBF",classes:{tooltip:"qtip-blue"}}}})(jQuery);;(function($){

	// much faster than native jquery $(ele).eq(index)
	// about 100x faster which improves dom manipulation performance significantly
	
	$.fn.quickeq = function(index){
		
		var set = $(this);
		var elem = $([1]);
		var size = set.size();
		
		if( index < 0 ){
			index = size - 1 + index;
		} 
		
		if( index < 0 || index >= size ){
			throw "quickeq can not go out of bounds with index " + index + " on array size " + size;
		}
		
		(elem.context = elem[0] = set[index]);
		
		return elem;
	}
	
})(jQuery); jQuery.fn.reverse = [].reverse;/**
 * jQuery.ScrollTo - Easy element scrolling using jQuery.
 * Copyright (c) 2007-2009 Ariel Flesler - aflesler(at)gmail(dot)com | http://flesler.blogspot.com
 * Dual licensed under MIT and GPL.
 * Date: 5/25/2009
 * @author Ariel Flesler
 * @version 1.4.2
 *
 * http://flesler.blogspot.com/2007/10/jqueryscrollto.html
 */
;(function(d){var k=d.scrollTo=function(a,i,e){d(window).scrollTo(a,i,e)};k.defaults={axis:'xy',duration:parseFloat(d.fn.jquery)>=1.3?0:1};k.window=function(a){return d(window)._scrollable()};d.fn._scrollable=function(){return this.map(function(){var a=this,i=!a.nodeName||d.inArray(a.nodeName.toLowerCase(),['iframe','#document','html','body'])!=-1;if(!i)return a;var e=(a.contentWindow||a).document||a.ownerDocument||a;return d.browser.safari||e.compatMode=='BackCompat'?e.body:e.documentElement})};d.fn.scrollTo=function(n,j,b){if(typeof j=='object'){b=j;j=0}if(typeof b=='function')b={onAfter:b};if(n=='max')n=9e9;b=d.extend({},k.defaults,b);j=j||b.speed||b.duration;b.queue=b.queue&&b.axis.length>1;if(b.queue)j/=2;b.offset=p(b.offset);b.over=p(b.over);return this._scrollable().each(function(){var q=this,r=d(q),f=n,s,g={},u=r.is('html,body');switch(typeof f){case'number':case'string':if(/^([+-]=)?\d+(\.\d+)?(px|%)?$/.test(f)){f=p(f);break}f=d(f,this);case'object':if(f.is||f.style)s=(f=d(f)).offset()}d.each(b.axis.split(''),function(a,i){var e=i=='x'?'Left':'Top',h=e.toLowerCase(),c='scroll'+e,l=q[c],m=k.max(q,i);if(s){g[c]=s[h]+(u?0:l-r.offset()[h]);if(b.margin){g[c]-=parseInt(f.css('margin'+e))||0;g[c]-=parseInt(f.css('border'+e+'Width'))||0}g[c]+=b.offset[h]||0;if(b.over[h])g[c]+=f[i=='x'?'width':'height']()*b.over[h]}else{var o=f[h];g[c]=o.slice&&o.slice(-1)=='%'?parseFloat(o)/100*m:o}if(/^\d+$/.test(g[c]))g[c]=g[c]<=0?0:Math.min(g[c],m);if(!a&&b.queue){if(l!=g[c])t(b.onAfterFirst);delete g[c]}});t(b.onAfter);function t(a){r.animate(g,j,b.easing,a&&function(){a.call(this,n,b)})}}).end()};k.max=function(a,i){var e=i=='x'?'Width':'Height',h='scroll'+e;if(!d(a).is('html,body'))return a[h]-d(a)[e.toLowerCase()]();var c='client'+e,l=a.ownerDocument.documentElement,m=a.ownerDocument.body;return Math.max(l[h],m[h])-Math.min(l[c],m[c])};function p(a){return typeof a=='object'?a:{top:a,left:a}}})(jQuery);jQuery.scrollBarSize = function(){
	
	function getScrollBarWidth () {
		var inner = document.createElement('p');
		inner.style.width = "100%";
		inner.style.height = "200px";

		var outer = document.createElement('div');
		outer.style.position = "absolute";
		outer.style.top = "0px";
		outer.style.left = "0px";
		outer.style.visibility = "hidden";
		outer.style.width = "200px";
		outer.style.height = "150px";
		outer.style.overflow = "hidden";
		outer.appendChild (inner);

		document.body.appendChild (outer);
		var w1 = inner.offsetWidth;
		outer.style.overflow = 'scroll';
		var w2 = inner.offsetWidth;
		if (w1 == w2) w2 = outer.clientWidth;

		document.body.removeChild (outer);

		return (w1 - w2);
	};
	
	
	return getScrollBarWidth();
};(function($){
    
	var options;
	var map = {};
	
    $.i18n = function(opts){
    	var defaults = {
            url: null,
            success: function(){}
        };
        
    	if( typeof(opts) == "string" ){ 
    		var message = opts;
    		
    		var ret = "" + map[message.toLowerCase()];
    		
    		for(var i = 1; i < arguments.length; i++){
    			var arg = arguments[i];

    			while( ret.match("\\{" + (i - 1) + "\\}") ){
    				ret = ret.replace("{" + (i - 1) + "}", arg);	
    			}
    		}
    		
    		return ret;
    	} else {
    		options = $.extend(defaults, opts);
    		
    		$.ajax({
    			url: options.url,
    			cache: false, // never cache the file to avoid sync issues
    			async: false,
    			success: function(data){
    				
    				$.each(data, function(i, v){
    					map[ i.toLowerCase() ] = v;
    				});
    				
    				
    				typeof options.success == typeof function(){} && options.success();
    			}
    		});    		
    	}
    }
   
    
})(jQuery);  /*
 * 
 * TableSorter 2.0 - Client-side table sorting with ease!
 * Version 2.0.5b
 * @requires jQuery v1.2.3
 * 
 * Copyright (c) 2007 Christian Bach
 * Examples and docs at: http://tablesorter.com
 * Dual licensed under the MIT and GPL licenses:
 * http://www.opensource.org/licenses/mit-license.php
 * http://www.gnu.org/licenses/gpl.html
 * 
 */
/**
 * 
 * @description Create a sortable table with multi-column sorting capabilitys
 * 
 * @example $('table').tablesorter();
 * @desc Create a simple tablesorter interface.
 * 
 * @example $('table').tablesorter({ sortList:[[0,0],[1,0]] });
 * @desc Create a tablesorter interface and sort on the first and secound column column headers.
 * 
 * @example $('table').tablesorter({ headers: { 0: { sorter: false}, 1: {sorter: false} } });
 *          
 * @desc Create a tablesorter interface and disableing the first and second  column headers.
 *      
 * 
 * @example $('table').tablesorter({ headers: { 0: {sorter:"integer"}, 1: {sorter:"currency"} } });
 * 
 * @desc Create a tablesorter interface and set a column parser for the first
 *       and second column.
 * 
 * 
 * @param Object
 *            settings An object literal containing key/value pairs to provide
 *            optional settings.
 * 
 * 
 * @option String cssHeader (optional) A string of the class name to be appended
 *         to sortable tr elements in the thead of the table. Default value:
 *         "header"
 * 
 * @option String cssAsc (optional) A string of the class name to be appended to
 *         sortable tr elements in the thead on a ascending sort. Default value:
 *         "headerSortUp"
 * 
 * @option String cssDesc (optional) A string of the class name to be appended
 *         to sortable tr elements in the thead on a descending sort. Default
 *         value: "headerSortDown"
 * 
 * @option String sortInitialOrder (optional) A string of the inital sorting
 *         order can be asc or desc. Default value: "asc"
 * 
 * @option String sortMultisortKey (optional) A string of the multi-column sort
 *         key. Default value: "shiftKey"
 * 
 * @option String textExtraction (optional) A string of the text-extraction
 *         method to use. For complex html structures inside td cell set this
 *         option to "complex", on large tables the complex option can be slow.
 *         Default value: "simple"
 * 
 * @option Object headers (optional) An array containing the forces sorting
 *         rules. This option let's you specify a default sorting rule. Default
 *         value: null
 * 
 * @option Array sortList (optional) An array containing the forces sorting
 *         rules. This option let's you specify a default sorting rule. Default
 *         value: null
 * 
 * @option Array sortForce (optional) An array containing forced sorting rules.
 *         This option let's you specify a default sorting rule, which is
 *         prepended to user-selected rules. Default value: null
 * 
 * @option Boolean sortLocaleCompare (optional) Boolean flag indicating whatever
 *         to use String.localeCampare method or not. Default set to true.
 * 
 * 
 * @option Array sortAppend (optional) An array containing forced sorting rules.
 *         This option let's you specify a default sorting rule, which is
 *         appended to user-selected rules. Default value: null
 * 
 * @option Boolean widthFixed (optional) Boolean flag indicating if tablesorter
 *         should apply fixed widths to the table columns. This is usefull when
 *         using the pager companion plugin. This options requires the dimension
 *         jquery plugin. Default value: false
 * 
 * @option Boolean cancelSelection (optional) Boolean flag indicating if
 *         tablesorter should cancel selection of the table headers text.
 *         Default value: true
 * 
 * @option Boolean debug (optional) Boolean flag indicating if tablesorter
 *         should display debuging information usefull for development.
 * 
 * @type jQuery
 * 
 * @name tablesorter
 * 
 * @cat Plugins/Tablesorter
 * 
 * @author Christian Bach/christian.bach@polyester.se
 */

(function ($) {
    $.extend({
        tablesorter: new
        function () {

            var parsers = [],
                widgets = [];

            this.defaults = {
                cssHeader: "header",
                cssAsc: "headerSortUp",
                cssDesc: "headerSortDown",
                cssChildRow: "expand-child",
                sortInitialOrder: "asc",
                sortMultiSortKey: "shiftKey",
                sortForce: null,
                sortAppend: null,
                sortLocaleCompare: true,
                textExtraction: "simple",
                parsers: {}, widgets: [],
                widgetZebra: {
                    css: ["even", "odd"]
                }, headers: {}, widthFixed: false,
                cancelSelection: true,
                sortList: [],
                headerList: [],
                dateFormat: "us",
                decimal: '/\.|\,/g',
                onRenderHeader: null,
                selectorHeaders: 'thead th',
                debug: false
            };

            /* debuging utils */

            function benchmark(s, d) {
                log(s + "," + (new Date().getTime() - d.getTime()) + "ms");
            }

            this.benchmark = benchmark;

            function log(s) {
                if (typeof console != "undefined" && typeof console.debug != "undefined") {
                    console.log(s);
                } else {
                    alert(s);
                }
            }

            /* parsers utils */

            function buildParserCache(table, $headers) {

                if (table.config.debug) {
                    var parsersDebug = "";
                }

                if (table.tBodies.length == 0) return; // In the case of empty tables
                var rows = table.tBodies[0].rows;

                if (rows[0]) {

                    var list = [],
                        cells = rows[0].cells,
                        l = cells.length;

                    for (var i = 0; i < l; i++) {

                        var p = false;

                        if ($.metadata && ($($headers[i]).metadata() && $($headers[i]).metadata().sorter)) {

                            p = getParserById($($headers[i]).metadata().sorter);

                        } else if ((table.config.headers[i] && table.config.headers[i].sorter)) {

                            p = getParserById(table.config.headers[i].sorter);
                        }
                        if (!p) {

                            p = detectParserForColumn(table, rows, -1, i);
                        }

                        if (table.config.debug) {
                            parsersDebug += "column:" + i + " parser:" + p.id + "\n";
                        }

                        list.push(p);
                    }
                }

                if (table.config.debug) {
                    log(parsersDebug);
                }

                return list;
            };

            function detectParserForColumn(table, rows, rowIndex, cellIndex) {
                var l = parsers.length,
                    node = false,
                    nodeValue = false,
                    keepLooking = true;
                while (nodeValue == '' && keepLooking) {
                    rowIndex++;
                    if (rows[rowIndex]) {
                        node = getNodeFromRowAndCellIndex(rows, rowIndex, cellIndex);
                        nodeValue = trimAndGetNodeText(table.config, node);
                        if (table.config.debug) {
                            log('Checking if value was empty on row:' + rowIndex);
                        }
                    } else {
                        keepLooking = false;
                    }
                }
                for (var i = 1; i < l; i++) {
                    if (parsers[i].is(nodeValue, table, node)) {
                        return parsers[i];
                    }
                }
                // 0 is always the generic parser (text)
                return parsers[0];
            }

            function getNodeFromRowAndCellIndex(rows, rowIndex, cellIndex) {
                return rows[rowIndex].cells[cellIndex];
            }

            function trimAndGetNodeText(config, node) {
                return $.trim(getElementText(config, node));
            }

            function getParserById(name) {
                var l = parsers.length;
                for (var i = 0; i < l; i++) {
                    if (parsers[i].id.toLowerCase() == name.toLowerCase()) {
                        return parsers[i];
                    }
                }
                return false;
            }

            /* utils */

            function buildCache(table) {

                if (table.config.debug) {
                    var cacheTime = new Date();
                }

                var totalRows = (table.tBodies[0] && table.tBodies[0].rows.length) || 0,
                    totalCells = (table.tBodies[0].rows[0] && table.tBodies[0].rows[0].cells.length) || 0,
                    parsers = table.config.parsers,
                    cache = {
                        row: [],
                        normalized: []
                    };

                for (var i = 0; i < totalRows; ++i) {

                    /** Add the table data to main data array */
                    var c = $(table.tBodies[0].rows[i]),
                        cols = [];

                    // if this is a child row, add it to the last row's children and
                    // continue to the next row
                    if (c.hasClass(table.config.cssChildRow)) {
                        cache.row[cache.row.length - 1] = cache.row[cache.row.length - 1].add(c);
                        // go to the next for loop
                        continue;
                    }

                    cache.row.push(c);

                    for (var j = 0; j < totalCells; ++j) {
                        cols.push(parsers[j].format(getElementText(table.config, c[0].cells[j]), table, c[0].cells[j]));
                    }

                    cols.push(cache.normalized.length); // add position for rowCache
                    cache.normalized.push(cols);
                    cols = null;
                };

                if (table.config.debug) {
                    benchmark("Building cache for " + totalRows + " rows:", cacheTime);
                }

                return cache;
            };

            function getElementText(config, node) {

                var text = "";

                if (!node) return "";

                if (!config.supportsTextContent) config.supportsTextContent = node.textContent || false;

                if (config.textExtraction == "simple") {
                    if (config.supportsTextContent) {
                        text = node.textContent;
                    } else {
                        if (node.childNodes[0] && node.childNodes[0].hasChildNodes()) {
                            text = node.childNodes[0].innerHTML;
                        } else {
                            text = node.innerHTML;
                        }
                    }
                } else {
                    if (typeof(config.textExtraction) == "function") {
                        text = config.textExtraction(node);
                    } else {
                        text = $(node).text();
                    }
                }
                return text;
            }

            function appendToTable(table, cache) {

                if (table.config.debug) {
                    var appendTime = new Date()
                }

                var c = cache,
                    r = c.row,
                    n = c.normalized,
                    totalRows = n.length,
                    checkCell = (n[0].length - 1),
                    tableBody = $(table.tBodies[0]),
                    rows = [];


                for (var i = 0; i < totalRows; i++) {
                    var pos = n[i][checkCell];

                    rows.push(r[pos]);

                    if (!table.config.appender) {

                        //var o = ;
                        var l = r[pos].length;
                        for (var j = 0; j < l; j++) {
                            tableBody[0].appendChild(r[pos][j]);
                        }

                        // 
                    }
                }



                if (table.config.appender) {

                    table.config.appender(table, rows);
                }

                rows = null;

                if (table.config.debug) {
                    benchmark("Rebuilt table:", appendTime);
                }

                // apply table widgets
                applyWidget(table);

                // trigger sortend
                setTimeout(function () {
                    $(table).trigger("sortEnd");
                }, 0);

            };

            function buildHeaders(table) {

                if (table.config.debug) {
                    var time = new Date();
                }

                var meta = ($.metadata) ? true : false;
                
                var header_index = computeTableHeaderCellIndexes(table);

                $tableHeaders = $(table.config.selectorHeaders, table).each(function (index) {

                    this.column = header_index[this.parentNode.rowIndex + "-" + this.cellIndex];
                    // this.column = index;
                    this.order = formatSortingOrder(table.config.sortInitialOrder);
                    
					
					this.count = this.order;

                    if (checkHeaderMetadata(this) || checkHeaderOptions(table, index)) this.sortDisabled = true;
					if (checkHeaderOptionsSortingLocked(table, index)) this.order = this.lockedOrder = checkHeaderOptionsSortingLocked(table, index);

                    if (!this.sortDisabled) {
                        var $th = $(this).addClass(table.config.cssHeader);
                        if (table.config.onRenderHeader) table.config.onRenderHeader.apply($th);
                    }

                    // add cell to headerList
                    table.config.headerList[index] = this;
                });

                if (table.config.debug) {
                    benchmark("Built headers:", time);
                    log($tableHeaders);
                }

                return $tableHeaders;

            };

            // from:
            // http://www.javascripttoolbox.com/lib/table/examples.php
            // http://www.javascripttoolbox.com/temp/table_cellindex.html


            function computeTableHeaderCellIndexes(t) {
                var matrix = [];
                var lookup = {};
                var thead = t.getElementsByTagName('THEAD')[0];
                var trs = thead.getElementsByTagName('TR');

                for (var i = 0; i < trs.length; i++) {
                    var cells = trs[i].cells;
                    for (var j = 0; j < cells.length; j++) {
                        var c = cells[j];

                        var rowIndex = c.parentNode.rowIndex;
                        var cellId = rowIndex + "-" + c.cellIndex;
                        var rowSpan = c.rowSpan || 1;
                        var colSpan = c.colSpan || 1
                        var firstAvailCol;
                        if (typeof(matrix[rowIndex]) == "undefined") {
                            matrix[rowIndex] = [];
                        }
                        // Find first available column in the first row
                        for (var k = 0; k < matrix[rowIndex].length + 1; k++) {
                            if (typeof(matrix[rowIndex][k]) == "undefined") {
                                firstAvailCol = k;
                                break;
                            }
                        }
                        lookup[cellId] = firstAvailCol;
                        for (var k = rowIndex; k < rowIndex + rowSpan; k++) {
                            if (typeof(matrix[k]) == "undefined") {
                                matrix[k] = [];
                            }
                            var matrixrow = matrix[k];
                            for (var l = firstAvailCol; l < firstAvailCol + colSpan; l++) {
                                matrixrow[l] = "x";
                            }
                        }
                    }
                }
                return lookup;
            }

            function checkCellColSpan(table, rows, row) {
                var arr = [],
                    r = table.tHead.rows,
                    c = r[row].cells;

                for (var i = 0; i < c.length; i++) {
                    var cell = c[i];

                    if (cell.colSpan > 1) {
                        arr = arr.concat(checkCellColSpan(table, headerArr, row++));
                    } else {
                        if (table.tHead.length == 1 || (cell.rowSpan > 1 || !r[row + 1])) {
                            arr.push(cell);
                        }
                        // headerArr[row] = (i+row);
                    }
                }
                return arr;
            };

            function checkHeaderMetadata(cell) {
                if (($.metadata) && ($(cell).metadata().sorter === false)) {
                    return true;
                };
                return false;
            }

            function checkHeaderOptions(table, i) {
                if ((table.config.headers[i]) && (table.config.headers[i].sorter === false)) {
                    return true;
                };
                return false;
            }
			
			 function checkHeaderOptionsSortingLocked(table, i) {
                if ((table.config.headers[i]) && (table.config.headers[i].lockedOrder)) return table.config.headers[i].lockedOrder;
                return false;
            }
			
            function applyWidget(table) {
                var c = table.config.widgets;
                var l = c.length;
                for (var i = 0; i < l; i++) {

                    getWidgetById(c[i]).format(table);
                }

            }

            function getWidgetById(name) {
                var l = widgets.length;
                for (var i = 0; i < l; i++) {
                    if (widgets[i].id.toLowerCase() == name.toLowerCase()) {
                        return widgets[i];
                    }
                }
            };

            function formatSortingOrder(v) {
                if (typeof(v) != "Number") {
                    return (v.toLowerCase() == "desc") ? 1 : 0;
                } else {
                    return (v == 1) ? 1 : 0;
                }
            }

            function isValueInArray(v, a) {
                var l = a.length;
                for (var i = 0; i < l; i++) {
                    if (a[i][0] == v) {
                        return true;
                    }
                }
                return false;
            }

            function setHeadersCss(table, $headers, list, css) {
                // remove all header information
                $headers.removeClass(css[0]).removeClass(css[1]);

                var h = [];
                $headers.each(function (offset) {
                    if (!this.sortDisabled) {
                        h[this.column] = $(this);
                    }
                });

                var l = list.length;
                for (var i = 0; i < l; i++) {
                    h[list[i][0]].addClass(css[list[i][1]]);
                }
            }

            function fixColumnWidth(table, $headers) {
                var c = table.config;
                if (c.widthFixed) {
                    var colgroup = $('<colgroup>');
                    $("tr:first td", table.tBodies[0]).each(function () {
                        colgroup.append($('<col>').css('width', $(this).width()));
                    });
                    $(table).prepend(colgroup);
                };
            }

            function updateHeaderSortCount(table, sortList) {
                var c = table.config,
                    l = sortList.length;
                for (var i = 0; i < l; i++) {
                    var s = sortList[i],
                        o = c.headerList[s[0]];
                    o.count = s[1];
                    o.count++;
                }
            }

            /* sorting methods */

            function multisort(table, sortList, cache) {

                if (table.config.debug) {
                    var sortTime = new Date();
                }

                var dynamicExp = "var sortWrapper = function(a,b) {",
                    l = sortList.length;

                // TODO: inline functions.
                for (var i = 0; i < l; i++) {

                    var c = sortList[i][0];
                    var order = sortList[i][1];
                    // var s = (getCachedSortType(table.config.parsers,c) == "text") ?
                    // ((order == 0) ? "sortText" : "sortTextDesc") : ((order == 0) ?
                    // "sortNumeric" : "sortNumericDesc");
                    // var s = (table.config.parsers[c].type == "text") ? ((order == 0)
                    // ? makeSortText(c) : makeSortTextDesc(c)) : ((order == 0) ?
                    // makeSortNumeric(c) : makeSortNumericDesc(c));
                    var s = (table.config.parsers[c].type == "text") ? ((order == 0) ? makeSortFunction("text", "asc", c) : makeSortFunction("text", "desc", c)) : ((order == 0) ? makeSortFunction("numeric", "asc", c) : makeSortFunction("numeric", "desc", c));
                    var e = "e" + i;

                    dynamicExp += "var " + e + " = " + s; // + "(a[" + c + "],b[" + c
                    // + "]); ";
                    dynamicExp += "if(" + e + ") { return " + e + "; } ";
                    dynamicExp += "else { ";

                }

                // if value is the same keep orignal order
                var orgOrderCol = cache.normalized[0].length - 1;
                dynamicExp += "return a[" + orgOrderCol + "]-b[" + orgOrderCol + "];";

                for (var i = 0; i < l; i++) {
                    dynamicExp += "}; ";
                }

                dynamicExp += "return 0; ";
                dynamicExp += "}; ";

                if (table.config.debug) {
                    benchmark("Evaling expression:" + dynamicExp, new Date());
                }

                eval(dynamicExp);

                cache.normalized.sort(sortWrapper);

                if (table.config.debug) {
                    benchmark("Sorting on " + sortList.toString() + " and dir " + order + " time:", sortTime);
                }

                return cache;
            };

            function makeSortFunction(type, direction, index) {
                var a = "a[" + index + "]",
                    b = "b[" + index + "]";
                if (type == 'text' && direction == 'asc') {
                    return "(" + a + " == " + b + " ? 0 : (" + a + " === null ? Number.POSITIVE_INFINITY : (" + b + " === null ? Number.NEGATIVE_INFINITY : (" + a + " < " + b + ") ? -1 : 1 )));";
                } else if (type == 'text' && direction == 'desc') {
                    return "(" + a + " == " + b + " ? 0 : (" + a + " === null ? Number.POSITIVE_INFINITY : (" + b + " === null ? Number.NEGATIVE_INFINITY : (" + b + " < " + a + ") ? -1 : 1 )));";
                } else if (type == 'numeric' && direction == 'asc') {
                    return "(" + a + " === null && " + b + " === null) ? 0 :(" + a + " === null ? Number.POSITIVE_INFINITY : (" + b + " === null ? Number.NEGATIVE_INFINITY : " + a + " - " + b + "));";
                } else if (type == 'numeric' && direction == 'desc') {
                    return "(" + a + " === null && " + b + " === null) ? 0 :(" + a + " === null ? Number.POSITIVE_INFINITY : (" + b + " === null ? Number.NEGATIVE_INFINITY : " + b + " - " + a + "));";
                }
            };

            function makeSortText(i) {
                return "((a[" + i + "] < b[" + i + "]) ? -1 : ((a[" + i + "] > b[" + i + "]) ? 1 : 0));";
            };

            function makeSortTextDesc(i) {
                return "((b[" + i + "] < a[" + i + "]) ? -1 : ((b[" + i + "] > a[" + i + "]) ? 1 : 0));";
            };

            function makeSortNumeric(i) {
                return "a[" + i + "]-b[" + i + "];";
            };

            function makeSortNumericDesc(i) {
                return "b[" + i + "]-a[" + i + "];";
            };

            function sortText(a, b) {
                if (table.config.sortLocaleCompare) return a.localeCompare(b);
                return ((a < b) ? -1 : ((a > b) ? 1 : 0));
            };

            function sortTextDesc(a, b) {
                if (table.config.sortLocaleCompare) return b.localeCompare(a);
                return ((b < a) ? -1 : ((b > a) ? 1 : 0));
            };

            function sortNumeric(a, b) {
                return a - b;
            };

            function sortNumericDesc(a, b) {
                return b - a;
            };

            function getCachedSortType(parsers, i) {
                return parsers[i].type;
            }; /* public methods */
            this.construct = function (settings) {
                return this.each(function () {
                    // if no thead or tbody quit.
                    if (!this.tHead || !this.tBodies) return;
                    // declare
                    var $this, $document, $headers, cache, config, shiftDown = 0,
                        sortOrder;
                    // new blank config object
                    this.config = {};
                    // merge and extend.
                    config = $.extend(this.config, $.tablesorter.defaults, settings);
                    // store common expression for speed
                    $this = $(this);
                    // save the settings where they read
                    $.data(this, "tablesorter", config);
                    // build headers
                    $headers = buildHeaders(this);
                    // try to auto detect column type, and store in tables config
                    this.config.parsers = buildParserCache(this, $headers);
                    // build the cache for the tbody cells
                    cache = buildCache(this);
                    // get the css class names, could be done else where.
                    var sortCSS = [config.cssDesc, config.cssAsc];
                    // fixate columns if the users supplies the fixedWidth option
                    fixColumnWidth(this);
                    // apply event handling to headers
                    // this is to big, perhaps break it out?
                    $headers.click(

                    function (e) {
                        var totalRows = ($this[0].tBodies[0] && $this[0].tBodies[0].rows.length) || 0;
                        if (!this.sortDisabled && totalRows > 0) {
                            // Only call sortStart if sorting is
                            // enabled.
                            $this.trigger("sortStart");
                            // store exp, for speed
                            var $cell = $(this);
                            // get current column index
                            var i = this.column;
                            // get current column sort order
                            this.order = this.count++ % 2;
							// always sort on the locked order.
							if(this.lockedOrder) this.order = this.lockedOrder;
							
							// user only whants to sort on one
                            // column
                            if (!e[config.sortMultiSortKey]) {
                                // flush the sort list
                                config.sortList = [];
                                if (config.sortForce != null) {
                                    var a = config.sortForce;
                                    for (var j = 0; j < a.length; j++) {
                                        if (a[j][0] != i) {
                                            config.sortList.push(a[j]);
                                        }
                                    }
                                }
                                // add column to sort list
                                config.sortList.push([i, this.order]);
                                // multi column sorting
                            } else {
                                // the user has clicked on an all
                                // ready sortet column.
                                if (isValueInArray(i, config.sortList)) {
                                    // revers the sorting direction
                                    // for all tables.
                                    for (var j = 0; j < config.sortList.length; j++) {
                                        var s = config.sortList[j],
                                            o = config.headerList[s[0]];
                                        if (s[0] == i) {
                                            o.count = s[1];
                                            o.count++;
                                            s[1] = o.count % 2;
                                        }
                                    }
                                } else {
                                    // add column to sort list array
                                    config.sortList.push([i, this.order]);
                                }
                            };
                            setTimeout(function () {
                                // set css for headers
                                setHeadersCss($this[0], $headers, config.sortList, sortCSS);
                                appendToTable(
	                                $this[0], multisort(
	                                $this[0], config.sortList, cache)
								);
                            }, 1);
                            // stop normal event by returning false
                            return false;
                        }
                        // cancel selection
                    }).mousedown(function () {
                        if (config.cancelSelection) {
                            this.onselectstart = function () {
                                return false
                            };
                            return false;
                        }
                    });
                    // apply easy methods that trigger binded events
                    $this.bind("update", function () {
                        var me = this;
                        setTimeout(function () {
                            // rebuild parsers.
                            me.config.parsers = buildParserCache(
                            me, $headers);
                            // rebuild the cache map
                            cache = buildCache(me);
                        }, 1);
                    }).bind("updateCell", function (e, cell) {
                        var config = this.config;
                        // get position from the dom.
                        var pos = [(cell.parentNode.rowIndex - 1), cell.cellIndex];
                        // update cache
                        cache.normalized[pos[0]][pos[1]] = config.parsers[pos[1]].format(
                        getElementText(config, cell), cell);
                    }).bind("sorton", function (e, list) {
                        $(this).trigger("sortStart");
                        config.sortList = list;
                        // update and store the sortlist
                        var sortList = config.sortList;
                        // update header count index
                        updateHeaderSortCount(this, sortList);
                        // set css for headers
                        setHeadersCss(this, $headers, sortList, sortCSS);
                        // sort the table and append it to the dom
                        appendToTable(this, multisort(this, sortList, cache));
                    }).bind("appendCache", function () {
                        appendToTable(this, cache);
                    }).bind("applyWidgetId", function (e, id) {
                        getWidgetById(id).format(this);
                    }).bind("applyWidgets", function () {
                        // apply widgets
                        applyWidget(this);
                    });
                    if ($.metadata && ($(this).metadata() && $(this).metadata().sortlist)) {
                        config.sortList = $(this).metadata().sortlist;
                    }
                    // if user has supplied a sort list to constructor.
                    if (config.sortList.length > 0) {
                        $this.trigger("sorton", [config.sortList]);
                    }
                    // apply widgets
                    applyWidget(this);
                });
            };
            this.addParser = function (parser) {
                var l = parsers.length,
                    a = true;
                for (var i = 0; i < l; i++) {
                    if (parsers[i].id.toLowerCase() == parser.id.toLowerCase()) {
                        a = false;
                    }
                }
                if (a) {
                    parsers.push(parser);
                };
            };
            this.addWidget = function (widget) {
                widgets.push(widget);
            };
            this.formatFloat = function (s) {
                var i = parseFloat(s);
                return (isNaN(i)) ? 0 : i;
            };
            this.formatInt = function (s) {
                var i = parseInt(s);
                return (isNaN(i)) ? 0 : i;
            };
            this.isDigit = function (s, config) {
                // replace all an wanted chars and match.
                return /^[-+]?\d*$/.test($.trim(s.replace(/[,.']/g, '')));
            };
            this.clearTableBody = function (table) {
                if ($.browser.msie) {
                    function empty() {
                        while (this.firstChild)
                        this.removeChild(this.firstChild);
                    }
                    empty.apply(table.tBodies[0]);
                } else {
                    table.tBodies[0].innerHTML = "";
                }
            };
        }
    });

    // extend plugin scope
    $.fn.extend({
        tablesorter: $.tablesorter.construct
    });

    // make shortcut
    var ts = $.tablesorter;

    // add default parsers
    ts.addParser({
        id: "text",
        is: function (s) {
            return true;
        }, format: function (s) {
            return $.trim(s.toLocaleLowerCase());
        }, type: "text"
    });

    ts.addParser({
        id: "digit",
        is: function (s, table) {
            var c = table.config;
            return $.tablesorter.isDigit(s, c);
        }, format: function (s) {
            return $.tablesorter.formatFloat(s);
        }, type: "numeric"
    });

    ts.addParser({
        id: "currency",
        is: function (s) {
            return /^[$?.]/.test(s);
        }, format: function (s) {
            return $.tablesorter.formatFloat(s.replace(new RegExp(/[$]/g), ""));
        }, type: "numeric"
    });

    ts.addParser({
        id: "ipAddress",
        is: function (s) {
            return /^\d{2,3}[\.]\d{2,3}[\.]\d{2,3}[\.]\d{2,3}$/.test(s);
        }, format: function (s) {
            var a = s.split("."),
                r = "",
                l = a.length;
            for (var i = 0; i < l; i++) {
                var item = a[i];
                if (item.length == 2) {
                    r += "0" + item;
                } else {
                    r += item;
                }
            }
            return $.tablesorter.formatFloat(r);
        }, type: "numeric"
    });

    ts.addParser({
        id: "url",
        is: function (s) {
            return /^(https?|ftp|file):\/\/$/.test(s);
        }, format: function (s) {
            return jQuery.trim(s.replace(new RegExp(/(https?|ftp|file):\/\//), ''));
        }, type: "text"
    });

    ts.addParser({
        id: "isoDate",
        is: function (s) {
            return /^\d{4}[\/-]\d{1,2}[\/-]\d{1,2}$/.test(s);
        }, format: function (s) {
            return $.tablesorter.formatFloat((s != "") ? new Date(s.replace(
            new RegExp(/-/g), "/")).getTime() : "0");
        }, type: "numeric"
    });

    ts.addParser({
        id: "percent",
        is: function (s) {
            return /\%$/.test($.trim(s));
        }, format: function (s) {
            return $.tablesorter.formatFloat(s.replace(new RegExp(/%/g), ""));
        }, type: "numeric"
    });

    ts.addParser({
        id: "usLongDate",
        is: function (s) {
            return s.match(new RegExp(/^[A-Za-z]{3,10}\.? [0-9]{1,2}, ([0-9]{4}|'?[0-9]{2}) (([0-2]?[0-9]:[0-5][0-9])|([0-1]?[0-9]:[0-5][0-9]\s(AM|PM)))$/));
        }, format: function (s) {
            return $.tablesorter.formatFloat(new Date(s).getTime());
        }, type: "numeric"
    });

    ts.addParser({
        id: "shortDate",
        is: function (s) {
            return /\d{1,2}[\/\-]\d{1,2}[\/\-]\d{2,4}/.test(s);
        }, format: function (s, table) {
            var c = table.config;
            s = s.replace(/\-/g, "/");
            if (c.dateFormat == "us") {
                // reformat the string in ISO format
                s = s.replace(/(\d{1,2})[\/\-](\d{1,2})[\/\-](\d{4})/, "$3/$1/$2");
            } else if (c.dateFormat == "uk") {
                // reformat the string in ISO format
                s = s.replace(/(\d{1,2})[\/\-](\d{1,2})[\/\-](\d{4})/, "$3/$2/$1");
            } else if (c.dateFormat == "dd/mm/yy" || c.dateFormat == "dd-mm-yy") {
                s = s.replace(/(\d{1,2})[\/\-](\d{1,2})[\/\-](\d{2})/, "$1/$2/$3");
            }
            return $.tablesorter.formatFloat(new Date(s).getTime());
        }, type: "numeric"
    });
    ts.addParser({
        id: "time",
        is: function (s) {
            return /^(([0-2]?[0-9]:[0-5][0-9])|([0-1]?[0-9]:[0-5][0-9]\s(am|pm)))$/.test(s);
        }, format: function (s) {
            return $.tablesorter.formatFloat(new Date("2000/01/01 " + s).getTime());
        }, type: "numeric"
    });
    ts.addParser({
        id: "metadata",
        is: function (s) {
            return false;
        }, format: function (s, table, cell) {
            var c = table.config,
                p = (!c.parserMetadataName) ? 'sortValue' : c.parserMetadataName;
            return $(cell).metadata()[p];
        }, type: "numeric"
    });
    // add default widgets
    ts.addWidget({
        id: "zebra",
        format: function (table) {
            if (table.config.debug) {
                var time = new Date();
            }
            var $tr, row = -1,
                odd;
            // loop through the visible rows
            $("tr:visible", table.tBodies[0]).each(function (i) {
                $tr = $(this);
                // style children rows the same way the parent
                // row was styled
                if (!$tr.hasClass(table.config.cssChildRow)) row++;
                odd = (row % 2 == 0);
                $tr.removeClass(
                table.config.widgetZebra.css[odd ? 0 : 1]).addClass(
                table.config.widgetZebra.css[odd ? 1 : 0])
            });
            if (table.config.debug) {
                $.tablesorter.benchmark("Applying Zebra widget", time);
            }
        }
    });
})(jQuery);/* (c) 2009
   AUTHORS: Max Franz
   LICENSE: TODO
*/

;(function($){
    
	var visibleTooltips = {};
	
    $.tooltip = new Object();
    
    $.tooltip.defaults = {
        toolTipId: "tool_tip",
        toolTipClass: "tool_tip",
        toolTipTopArrowId: "tool_tip_top",
        toolTipBottomArrowId: "tool_tip_bottom",
        toolTipPointerClass: "tool_tip_pointer", // must have width set in css
        toolTipTextId: "tool_tip_text",
        toolTipDirectionUp: "points_to_top",
        toolTipDirectionDown: "points_to_bottom",
        toolTipAttr: "tooltip",
        showEvents: new Array("mouseover"), // list of target events to show tool tip
        hideEvents: new Array("mouseout", "mousedown"), // list of target events to hide tool tip
        minToolTipWidth: 200, // min width of entire tool tip bubble; used if target is smaller than minToolTipWidth and text overflows, otherwise, fits to text or target size
        delayAfterHover: 500, // ms delay from mouse over to showing tool tip; do not show tool tip on negative value
        minArrowMargin: 7, // min margin (left or right) from edge of tool tip bubble for the tool tip arrow
        opacity: 1,
        anchor: "bottom", // top | bottom
        left: null, // where arrow points NOT BOUNDING BOX LEFT
        top: null, // where arrow points NOT BOUNDING BOX TOP
        positionAtCursor: false, // overrides left and top, tooltip at cursor position after timeout
        object: null, // target object OVERRIDE
        message: null, // message to print OVERRIDE
        arrowAlign: "cursor", // OVERRIDE: one of "left", "right", "center", or "cursor",
        fadeInSpeed: 0,
        fadeOutSpeed: 0,
        showLength: null // how long to show before auto hide
    };
    
    $.tooltip.timeoutList = []; // timeoutList[toolTipId] == timeout for toolTipId 
    $.tooltip.autohideTimeoutList = []; // autohideTimeoutList[toolTipId] == autohide timeout for toolTipId 
    
    $.tooltip.cursor = {x:0, y:0};

    $.tooltip.cursor.getPosition = function(e) {
        e = e || window.event;
        var cursor = {x:0, y:0};
        if (e.pageX || e.pageY) {
            cursor.x = e.pageX;
            cursor.y = e.pageY;
        } 
        else {
            var de = document.documentElement;
            var b = document.body;
            cursor.x = e.clientX + 
                (de.scrollLeft || b.scrollLeft) - (de.clientLeft || 0);
            cursor.y = e.clientY + 
                (de.scrollTop || b.scrollTop) - (de.clientTop || 0);
        }
        return cursor;
    }
    
    $(document).ready(function(){
        $("html").mousemove(function(e){
            $.tooltip.cursor.x = $.tooltip.cursor.getPosition(e).x;
            $.tooltip.cursor.y = $.tooltip.cursor.getPosition(e).y;
        });
    });
    
    // attribute used to see if tool tips are enabled
    $.tooltip.attr = "tooltipsEnabled";
    
    // TODO replace show hide with this
    $.tooltip.show = function(options) {
    
        options = $.extend({}, $.tooltip.defaults, options);
    
        if( options.toolTipSpeed < 0 ) {
            return;
        }
    
        clearTimeout( $.tooltip.autohideTimeoutList[options.toolTipId] );
    
        if( $("#" + options.toolTipId).length <= 0 ) {  
            $("body").append("<div id=\"" + options.toolTipId + "\" class=\"" + options.toolTipClass + "\">"
                    + "<div class=\"" + options.toolTipTopArrowId + "\"> <div class='" + options.toolTipPointerClass + "'></div> </div>"
                    + "<div class=\"" + options.toolTipTextId + "\">&nbsp;</div>"
                    + "<div class=\"" + options.toolTipBottomArrowId + "\"> <div class='" + options.toolTipPointerClass + "'></div> </div>"
                    + "</div>");
        }
        
        var toolTip = $("#" + options.toolTipId);       
        var toolTipTop = $("#" + options.toolTipId + " ." + options.toolTipTopArrowId);
        var toolTipBottom = $("#" + options.toolTipId + " ." + options.toolTipBottomArrowId);
        var toolTipText = $("#" + options.toolTipId + " ." + options.toolTipTextId);
        
        // show offscreen so dimensions calculations always work
        if( jQuery.browser.opera ) {
            // opera uses 16 bit int for page dimensions
            // -2^15 <= pageX, pageY <= 2^15
            toolTip.css("left", -30000).css("top", -30000).show(); 
        } else {
            // decent browsers use at least a 32 bit int for page dimensions
            // -2^31 <= pageX, pageY <= 2^31
            toolTip.css("left", -20000000).css("top", -20000000).show();
        }
        
        $.tooltip.timeoutList[options.toolTipId] = setTimeout(function(){
        
            var target = options.object;
            var message = options.message;
            var screenWidth = $("html").width();
            var screenHeight = $("html").height();
            var offsetLeft = 0;
            var width = options.minToolTipWidth;
            var offsetTop = 0;
            var offsetBottom = 0;
            var enabled = true;
            var imgWidth = parseInt( $("." + options.toolTipTopArrowId).css("width") );
            var targetShown = target.is(":visible") && target.css("display") != "none";
            var absMinWidth = imgWidth + options.minArrowMargin * 2;
            var cursorX = $.tooltip.cursor.x;
            var cursorY = $.tooltip.cursor.y;
            var useTargetWidth;
            
            if( target ) {
                offsetLeft = target.offset().left;
                
                if( target.outerWidth() && target.outerWidth() > options.minToolTipWidth ) {
                    width = target.outerWidth();
                    useTargetWidth = true;
                }
                
                offsetTop = target.offset().top + target.outerHeight();
                offsetBottom = target.offset().top;
                message = message || target.attr(options.toolTipAttr);
                enabled = target.tooltipsEnabled();
            }
            
            // fix these or they could throw off the auto width calc
            toolTipTop.css("margin-left", 0);
            toolTipBottom.css("margin-left", 0);
            
            // set text and fix width in case the text is smaller than min
            toolTipText.html( message ); 
            toolTip.css("width", "auto");
            if( toolTip.width() > width ) {
                toolTip.css("width", width);
            } else {
                if( toolTip.width() < absMinWidth ) {
                    toolTip.css("width", absMinWidth);
                }
                
                width = toolTip.width() + 1; // + 1 to fix in case width is a decimal
                toolTip.css("width", width);
                useTargetWidth = false;
                
                offsetLeft = Math.min(cursorX - imgWidth/2 - options.minArrowMargin, target.offset().left + target.width() - width);
                offsetLeft = Math.max(offsetLeft, target.offset().left);
            }
            
            if( message == null || message == "" ){
            	return;
            }
                        
            // override position using options
            if( options.left != 0 && ! options.left ) {
                // value is undefined, not zero
            } else {
                offsetLeft = options.left - imgWidth/2 - options.minArrowMargin;
            }
            
            if( options.top != 0 && ! options.top ) {
                // value is undefined, not zero
            } else {
                offsetTop = options.top;
            }
           
            // correct x bounds
            if(screenWidth < offsetLeft + width) {
                offsetLeft -= offsetLeft + width - screenWidth;
            }
            
            if(options.positionAtCursor) {
                offsetTop = cursorY;
                offsetLeft = cursorX - imgWidth/2 - options.minArrowMargin;
            }
            
            toolTip.css("top",  offsetTop).css("left", offsetLeft);
            
            // do cursor calculations in timeout; otherwise, it's out of date
            var offset = Math.max(cursorX - offsetLeft, 0); // must be > 0
            var left = offset;
            var right = width - offset;
            var minMargin = options.minArrowMargin;
            var height = toolTip.height();
            
            if( options.arrowAlign ) {
                switch( options.arrowAlign.toLowerCase() ){
                case "left":
                    left = 0;
                    break;
                case "right":
                    left = width;
                    break;
                case "center":
                    left = width / 2;
                    break;
                case "cursor":
                    break;
                }
            }
            
            function adjustMargin(arrow) {
                arrow.css( "margin-left", Math.max( Math.min(left - imgWidth/2, width - imgWidth - minMargin), minMargin) );
            }
            
            adjustMargin(toolTipTop);
            adjustMargin(toolTipBottom);
            
            // on small targets, align the arrow with the cursor rather than the lhs of the box
            var cursorArrowDiff = cursorX - toolTipTop.offset().left - imgWidth/2;
            if(options.arrowAlign == "cursor" && ! useTargetWidth && Math.round(cursorArrowDiff) != 0 && target.outerWidth() <= imgWidth + minMargin) {
                toolTip.css("left", offsetLeft- imgWidth/2 - minMargin + target.outerWidth()/2);
            }

            if(options.anchor == "top" || screenHeight < offsetTop + height) {
                toolTip.css( "top", offsetBottom - toolTip.height() );
                toolTipTop.css("visibility", "hidden");
                toolTipBottom.css("visibility", "visible");
                toolTipText.addClass(options.toolTipDirectionDown).removeClass(options.toolTipDirectionUp);
            } else {
                toolTipTop.css("visibility", "visible");
                toolTipBottom.css("visibility", "hidden");
                toolTipText.addClass(options.toolTipDirectionUp).removeClass(options.toolTipDirectionDown);
            }

            if( enabled && (! target || targetShown) ) {
                toolTip.show();
                currentTooltip = toolTip;
                
                if(options.opacity != 1.0) {
                    toolTip.fadeTo(options.fadeInSpeed, options.opacity);
                }
                
                if(options.showLength) {
                    $.tooltip.autohideTimeoutList[options.toolTipId] = setTimeout(function(){
                        $.tooltip.hide(options);
                    }, options.showLength);
                }
            }
        }, options.delayAfterHover);
    };
    
    $.tooltip.hide = function(options) {
        options = $.extend({}, $.tooltip.defaults, options);

        clearTimeout( $.tooltip.timeoutList[options.toolTipId] );
        clearTimeout( $.tooltip.autohideTimeoutList[options.toolTipId] );
    
        $("#" + options.toolTipId).fadeOut(options.fadeOutSpeed);
        currentTooltip = null;
    };
    
    $.fn.disableTooltips = function() {
        return this.each(function() {
            $(this).attr($.tooltip.attr, "false");
        });
    };
    
    $.fn.enableTooltips = function() {
        return this.each(function() {
            $(this).attr($.tooltip.attr, "true");
        });
    };
    
    // tool tip enabling is recursive (e.g. if parent has tool tips enabled, then the
    // children do)
    
    // HOWEVER, children can override parents
    
    // body[enabled] > #foo > #bar[disabled]
    // here #bar is disabled but #foo is enabled by body  
    
    $.fn.tooltipsEnabled = function() {
        if( "true" == $(this).attr($.tooltip.attr) ) {
            return true; // true if self true
        } else if ( "true" == $(this).closest("[" + $.tooltip.attr + "]").attr($.tooltip.attr) ) {
            return true; // true if nearest set parent true
        } else {
            return false;
        }
    };
    
    $.fn.hideTooltip = function(options) {
	    options = $.extend({}, $.tooltip.defaults, options);
	    options.object = $(this);
	    if ( options.toolTipId == visibleTooltips[$(this).attr("id")] ) {
	        $.tooltip.hide(options);
	        visibleTooltips[$(this).attr("id")] = null;
        }
    };
    
    $.fn.showTooltip = function(options) {
        options = $.extend({}, $.tooltip.defaults, options);
        visibleTooltips[$(this).attr("id")] = options.toolTipId;
        options.object = $(this);
        $.tooltip.show(options);
    };
    
    $.fn.tooltip = function(options) {  
        
        options = $.extend({}, $.tooltip.defaults, options);
        
        return this.each(function() { 
            $(this).enableTooltips();
            	
        	for(i in options.showEvents) {
        		var event = options.showEvents[i];
        		
        		$(this).find("[" + options.toolTipAttr + "]").live(event, function(){
        			// If the same HTML element already has a displayed tooltip, does not show another one: 
                    if (visibleTooltips[$(this).attr("id")] == null) {
                    	$(this).showTooltip(options);
                    }
        		});
        	}
        	
        	for(i in options.hideEvents) {
        		var event = options.hideEvents[i];
        		
        		$(this).find("[" + options.toolTipAttr + "]").live(event, function(){
        			$(this).hideTooltip(options);
        		});
        	}

        });  
    };  
})(jQuery);  var isIE=(navigator.appVersion.indexOf("MSIE")!=-1)?true:false;var isWin=(navigator.appVersion.toLowerCase().indexOf("win")!=-1)?true:false;var isOpera=(navigator.userAgent.indexOf("Opera")!=-1)?true:false;function ControlVersion(){var a;var b;var c;try{b=new ActiveXObject("ShockwaveFlash.ShockwaveFlash.7");a=b.GetVariable("$version")}catch(c){}if(!a){try{b=new ActiveXObject("ShockwaveFlash.ShockwaveFlash.6");a="WIN 6,0,21,0";b.AllowScriptAccess="always";a=b.GetVariable("$version")}catch(c){}}if(!a){try{b=new ActiveXObject("ShockwaveFlash.ShockwaveFlash.3");a=b.GetVariable("$version")}catch(c){}}if(!a){try{b=new ActiveXObject("ShockwaveFlash.ShockwaveFlash.3");a="WIN 3,0,18,0"}catch(c){}}if(!a){try{b=new ActiveXObject("ShockwaveFlash.ShockwaveFlash");a="WIN 2,0,0,11"}catch(c){a=-1}}return a}function GetSwfVer(){var g=-1;if(navigator.plugins!=null&&navigator.plugins.length>0){if(navigator.plugins["Shockwave Flash 2.0"]||navigator.plugins["Shockwave Flash"]){var f=navigator.plugins["Shockwave Flash 2.0"]?" 2.0":"";var a=navigator.plugins["Shockwave Flash"+f].description;var e=a.split(" ");var c=e[2].split(".");var h=c[0];var b=c[1];var d=e[3];if(d==""){d=e[4]}if(d[0]=="d"){d=d.substring(1)}else{if(d[0]=="r"){d=d.substring(1);if(d.indexOf("d")>0){d=d.substring(0,d.indexOf("d"))}}else{if(d[0]=="b"){d=d.substring(1)}}}var g=h+"."+b+"."+d}}else{if(navigator.userAgent.toLowerCase().indexOf("webtv/2.6")!=-1){g=4}else{if(navigator.userAgent.toLowerCase().indexOf("webtv/2.5")!=-1){g=3}else{if(navigator.userAgent.toLowerCase().indexOf("webtv")!=-1){g=2}else{if(isIE&&isWin&&!isOpera){g=ControlVersion()}}}}}return g}function DetectFlashVer(f,d,c){versionStr=GetSwfVer();if(versionStr==-1){return false}else{if(versionStr!=0){if(isIE&&isWin&&!isOpera){tempArray=versionStr.split(" ");tempString=tempArray[1];versionArray=tempString.split(",")}else{versionArray=versionStr.split(".")}var e=versionArray[0];var a=versionArray[1];var b=versionArray[2];if(e>parseFloat(f)){return true}else{if(e==parseFloat(f)){if(a>parseFloat(d)){return true}else{if(a==parseFloat(d)){if(b>=parseFloat(c)){return true}}}}}return false}}}function AC_AddExtension(b,a){if(b.indexOf("?")!=-1){return b.replace(/\?/,a+"?")}else{return b+a}}function AC_Generateobj(e,d,a){var c="";if(isIE&&isWin&&!isOpera){c+="<object ";for(var b in e){c+=b+'="'+e[b]+'" '}c+=">";for(var b in d){c+='<param name="'+b+'" value="'+d[b]+'" /> '}c+="</object>"}else{c+="<embed ";for(var b in a){c+=b+'="'+a[b]+'" '}c+="> </embed>"}document.write(c)}function AC_FL_RunContent(){var a=AC_GetArgs(arguments,".swf","movie","clsid:d27cdb6e-ae6d-11cf-96b8-444553540000","application/x-shockwave-flash");AC_Generateobj(a.objAttrs,a.params,a.embedAttrs)}function AC_GetArgs(b,e,g,d,h){var a=new Object();a.embedAttrs=new Object();a.params=new Object();a.objAttrs=new Object();for(var c=0;c<b.length;c=c+2){var f=b[c].toLowerCase();switch(f){case"classid":break;case"pluginspage":a.embedAttrs[b[c]]=b[c+1];break;case"src":case"movie":b[c+1]=AC_AddExtension(b[c+1],e);a.embedAttrs.src=b[c+1];a.params[g]=b[c+1];break;case"onafterupdate":case"onbeforeupdate":case"onblur":case"oncellchange":case"onclick":case"ondblClick":case"ondrag":case"ondragend":case"ondragenter":case"ondragleave":case"ondragover":case"ondrop":case"onfinish":case"onfocus":case"onhelp":case"onmousedown":case"onmouseup":case"onmouseover":case"onmousemove":case"onmouseout":case"onkeypress":case"onkeydown":case"onkeyup":case"onload":case"onlosecapture":case"onpropertychange":case"onreadystatechange":case"onrowsdelete":case"onrowenter":case"onrowexit":case"onrowsinserted":case"onstart":case"onscroll":case"onbeforeeditfocus":case"onactivate":case"onbeforedeactivate":case"ondeactivate":case"type":case"codebase":a.objAttrs[b[c]]=b[c+1];break;case"id":case"width":case"height":case"align":case"vspace":case"hspace":case"class":case"title":case"accesskey":case"name":case"tabindex":a.embedAttrs[b[c]]=a.objAttrs[b[c]]=b[c+1];break;default:a.embedAttrs[b[c]]=a.params[b[c]]=b[c+1]}}a.objAttrs.classid=d;if(h){a.embedAttrs.type=h}return a};/*
  This file is part of Cytoscape Web.
  Copyright (c) 2009, The Cytoscape Consortium (www.cytoscape.org)

  The Cytoscape Consortium is:
    - Agilent Technologies
    - Institut Pasteur
    - Institute for Systems Biology
    - Memorial Sloan-Kettering Cancer Center
    - National Center for Integrative Biomedical Informatics
    - Unilever
    - University of California San Diego
    - University of California San Francisco
    - University of Toronto

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
*/

// ===[ namespaces ]================================================================================

// Create namespaces if not already defined:
(function () {
	
    if (typeof(window['org']) === 'undefined') {
    	/**
    	 * @namespace
    	 * @name org
    	 */
    	window['org'] = {};
    }
    if (typeof(window.org['cytoscapeweb']) === 'undefined') {
    	/**
    	 * @namespace
    	 * @name org.cytoscapeweb
    	 */
    	org['cytoscapeweb'] = {};
    }
    
    // Create a global map to store all instances of Cytoscape Web:
    window._cytoscapeWebInstances = { index: 0 };

    // ===[ Visualization ]=========================================================================
    
    /**
     * <p>Initialise Cytoscape Web. It does not draw the network yet.</p>
     * <p>The {@link org.cytoscapeweb.Visualization#draw} method must be called when 
     * you want the network to be displayed.</p>
     * @example
     * &lt;html&gt;
     * &lt;head&gt; &lt;/head&gt;
     * &lt;body&gt;
     * 
     * &lt;h1&gt;Sample&lt;/h1&gt;
     * &lt;div id="cytoWebContent" style="width: 600px;height: 400px;"&gt;&lt;/div&gt;
     * 
     * &lt;script type="text/javascript"&gt;
     *     var options = { swfPath: "path/to/swf/CytoscapeWeb",
     *                     flashInstallerPath: "path/to/swf/playerProductInstall",
     *                     flashAlternateContent: "Le Flash Player est n&eacute;cessaire." };
     *                     
     *     var vis = new org.cytoscapeweb.Visualization("cytoWebContent", options);
     *     
     *     vis.draw({ network: '&lt;graphml&gt;...&lt;/graphml&gt;' });
     * &lt;/script&gt;
     *
     * &lt;/body&gt;
     * &lt;html&gt;
     * @class
     * @param {String} containerId The id of the HTML element (containing your alternative content)
     *                             you would like to have replaced by the Flash object.
     * @param {Object} [options] Cytoscape Web parameters:
     *                <ul class="options">
     *                    <li><code>swfPath</code>: The path of the compiled Cytoscape Web SWF file, but without the
     *                                              <code>.swf</code> extension. If you use the provided <code>CytoscapeWeb.swf</code>
     *                                              file and put it in the root path of the web application, this option does not need
     *                                              to be specified. But, for example, if you deploy the swf file at <code>/plugin/flash</code>,
     *                                              the <code>swfPath</code> value must be "/plugin/flash/CytoscapeWeb".</li>
     *                    <li><code>flashInstallerPath</code>: The path to the compiled Flash video that should be displayed in case
     *                                                         the browser does not have the Flash Player version required by Cytoscape Web.
     *                                                         The default value is "playerProductInstall" and, if this option is not changed,
     *                                                         the <code>playerProductInstall.swf</code> file must be deployed in the
     *                                                         web site's root path. Otherwise, just specify the new path without the
     *                                                         <code>.swf</code> extension.</li>
     *                    <li><code>flashAlternateContent</code>: The text message that should be displayed if the browser does not have
     *                                                            the Flash Player plugin. If none is provided, Cytoscape Web will show
     *                                                            a default message and a link to the "Get Flash" page.</li>
     *                    <li><code>resourceBundleUrl</code>: An optional resource bundle path. Usually a <code>.properties</code> file
     *                                                        that redefines the default labels and messages used by Cytoscape Web.
     *                                                        Example of a valid file with all the available keys:
     * <pre>
     * global.wait = Please wait...
     * error.title = Error
     * pan.tooltip = Grab to pan
     * pan.up.tooltip = Pan up
     * pan.down.tooltip = Pan down
     * pan.left.tooltip = Pan left
     * pan.right.tooltip = Pan right
     * zoom.out.tooltip = Zoom out
     * zoom.in.tooltip = Zoom in
     * zoom.fit.tooltip = Fit to screen
     * zoom.slider.tooltip = {0}%
     * </pre></li>
     *                    <li><code>idToken</code>: A string used to create the embedded Flash video id
     *                                              (usually an HTML <code>embed</code> or <code>object</code> tag).
     *                                              The default token is "cytoscapeWeb" and the final id will be the token followed
     *                                              by a number, so if the application has two instances of the Visualization in the same page,
     *                                              their id's will be "cytoscapeWeb1" and "cytoscapeWeb2".
     *                                              This token does not usually need to be changed.</li>
     *                    <li><code>mouseDownToDragDelay</code>: If the user clicks (mouse-down) the background and hold if for a while,
     *                                                           Cytoscape Web temporarily changes to grab-to-pan mode, so the user can drag the whole network.
     *                                                           If the user do the same over a node, the grab-to-pan mode can be activated as well,
     *                                                           but dragging the node will drag the node's disconnected component only,
     *                                                           not necessarily the whole network--if there are other disconnected parts, they will not be dragged.
     *                                                           This parameter lets you set the time (in milliseconds) Cytoscape Web should wait before changing modes.
     *                                                           The default value is <code>400</code>.
     *                                                           If this parameter is set to <code>-1</code>, Cytoscape Web will never activate the grab-to-pan mode
     *                                                           automatically, but users can still use the grab-to-pan option from the pan-zoom control.</li>
     *                </ul>
     * @return {org.cytoscapeweb.Visualization} The Visualization instance.
     * @see org.cytoscapeweb.Visualization#draw
     * @see org.cytoscapeweb.Visualization#ready
     */
    org.cytoscapeweb.Visualization = function (containerId, options) {
        this.containerId = containerId;

        if (!options) { options = {}; }
        this.options = options;

        // Part of the embed or object tag id:
        this.idToken = options.idToken ? options.idToken : "cytoscapeWeb";
        // The .swf path, including its name, but without the file extension:
        this.swfPath = options.swfPath ? options.swfPath : "CytoscapeWeb";
        // The path of the .swf file that updates the Flash player version:
        this.flashInstallerPath = options.flashInstallerPath ? options.flashInstallerPath : "playerProductInstall";
        // Alternate content to be displayed in case user does not have Flash installed:
        this.flashAlternateContent = options.flashAlternateContent ? options.flashAlternateContent : 'This content requires the Adobe Flash Player. ' +
                                                                                                     '<a href=http://www.adobe.com/go/getflash/>Get Flash</a>';
        _cytoscapeWebInstances.index++;

        this.id = this.idToken + _cytoscapeWebInstances.index;
        _cytoscapeWebInstances[this.id] = this;
    };

    org.cytoscapeweb.Visualization.prototype = {

        // PUBLIC METHODS:
        // -----------------------------------------------------------------------------------------

        /**
         * <p>Start Cytoscape Web by drawing the network.
         * At least the <code>network</code> option must be specified.</p>
         * @example
         * var vis = new org.cytoscapeweb.Visualization("container-id");
         * vis.draw({ network: '&lt;graphml&gt;...&lt;/graphml&gt;',
         *            edgeLabelsVisible: false,
         *            layout: 'Circle',
         *            panZoomControlPosition: 'bottomLeft',
         *            visualStyle: {
         *                global: {
         *                    backgroundColor: "#000033",
         *                    nodeSelectionColor: "#ffce81"
         *                },
         *                nodes: {
         *                    shape: "diamond"
         *                },
         *                edges: {
         *                    width: 2
         *                }
         *            }
         *         });
         *
         * @description
         * <p>Just remember that you probably want to register a callback function with {@link org.cytoscapeweb.Visualization#ready}
         * before calling <code>draw()</code>.</p>
         * 
         * @example
         * var vis = new org.cytoscapeweb.Visualization("container-id");
         * vis.ready(function () {
         *     // Start interaction with the network here...
         * });
         * vis.draw({ network: '&lt;graphml&gt;...&lt;/graphml&gt;' });
         *
         * @param {Object} options
         *               <ul class="options">
         *                    <li><code>network</code>: The model that describes the network. Only this option is mandatory. It can be one of the following formats:
         *                                              <ul><li>{@link org.cytoscapeweb.NetworkModel}: A simple JavaScript object that defines the raw data from which to build a network.</li>
         *                                                  <li><a href="http://graphml.graphdrawing.org/primer/graphml-primer.html" rel="external">GraphML</a>: An XML format for graphs.</li>
         *                                                  <li><a href="http://www.cs.rpi.edu/~puninj/XGMML/" rel="external">XGMML</a>: This XML format allows you to define
         *                                                      visual properties (e.g. colors and shapes) and nodes positioning, if you want to,
         *                                                      although using the <code>visualStyle</code> and <code>layout</code> options is usually better.</li>
         *                                                  <li><a href="http://cytoscape.wodaklab.org/wiki/Cytoscape_User_Manual/Network_Formats/" rel="external">SIF</a>: A simpler text format
         *                                                      that can be very useful if you do not need to set custom nodes/edges attributes.</li>
         *                                              </ul></li>
         *                    <li><code>visualStyle</code>: an optional {@link org.cytoscapeweb.VisualStyle} object to be applied on this network.</li>
         *                    <li><code>layout</code>: an optional {@link org.cytoscapeweb.Layout} object, or just the layout name.
         *                                             The default is "ForceDirected", unless the network data is an 
         *                                             <a href="http://www.cs.rpi.edu/~puninj/XGMML/" rel="external">XGMML</a>, whose 
         *                                             <code><a href="http://www.cs.rpi.edu/research/groups/pb/punin/public_html/XGMML/draft-xgmml-20010628.html#NodeE" rel="external">node</a></code>
         *                                             elements contain
         *                                             <code><a href="http://www.cs.rpi.edu/research/groups/pb/punin/public_html/XGMML/draft-xgmml-20010628.html#GLCPE" rel="external">graphics</a></code>
         *                                             tags with defined <code>x</code> and <code>y</code> attributes. In that case, the "Preset" layout is applied by default.</li>
         *                    <li><code>nodeLabelsVisible</code>: Boolean that defines whether or not the node labels will be visible.
         *                                                        The default value is <code>true</code>.
         *                                                        You can call {@link org.cytoscapeweb.Visualization#nodeLabelsVisible} 
         *                                                        later (after the network is ready) to change it.</li>
         *                    <li><code>edgeLabelsVisible</code>: Boolean that defines whether or not the edge labels will be visible.
         *                                                        The default value is <code>false</code>.
         *                                                        You can use {@link org.cytoscapeweb.Visualization#edgeLabelsVisible} later to change it.</li>
         *                    <li><code>nodeTooltipsEnabled</code>: Boolean value that enables or disables the node tooltips.
         *                                                          The default value is <code>true</code>.
         *                                                          You can call {@link org.cytoscapeweb.Visualization#nodeTooltipsEnabled} later to change it.</li>
         *                    <li><code>edgeTooltipsEnabled</code>: Boolean that enables or disables the edge tooltips.
         *                                                          The default value is <code>true</code>.
         *                                                          You can use {@link org.cytoscapeweb.Visualization#edgeTooltipsEnabled} later to change it.</li>
         *                    <li><code>edgesMerged</code>: Boolean that defines whether or not the network will be initially
         *                                                  rendered with merged edges. The default value is <code>false</code>.
         *                                                  You can call {@link org.cytoscapeweb.Visualization#edgesMerged} after the network is ready to change it.</li>
         *                    <li><code>panZoomControlVisible</code>: Boolean value that sets whether or not the built-in control
         *                                                            will be visible. The default value is <code>true</code>.
         *                                                            The visibility of the control can be changed later with
         *                                                            {@link org.cytoscapeweb.Visualization#panZoomControlVisible}.</li>
         *                    <li><code>panZoomControlPosition</code>: String value that sets the initial position of the built-in control.
         *                                                             The allowed values are:
         *                                                             <ul><li><code>topLeft</code></li>
         *                                                                 <li><code>topCenter</code></li>
         *                                                                 <li><code>topRight</code></li>
         *                                                                 <li><code>middleLeft</code></li>
         *                                                                 <li><code>middleCenter</code></li>
         *                                                                 <li><code>middleRight</code></li>
         *                                                                 <li><code>bottomLeft</code></li>
         *                                                                 <li><code>bottomCenter</code></li>
         *                                                                 <li><code>bottomRight</code></li></ul>
         *                                                             The default value is <code>"bottomRight"</code>.</li>
         *                    <li><code>preloadImages</code>: Boolean that defines whether or not to load all images before rendering the network.
         *                                                    If <code>true</code>, all images from a 
         *                                                    {@link org.cytoscapeweb.VisualStyle} or {@link org.cytoscapeweb.VisualStyleBypass}
         *                                                    will be loaded before the network is drawn or before a visual style (or bypass) is applied.
         *                                                    The default value is <code>true</code>.</li>
         *                </ul>
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#ready
         * @see org.cytoscapeweb.VisualStyle
         * @see org.cytoscapeweb.Layout
         */
        draw: function (options) {
            if (!options) { options = {}; }
            this.drawOptions = options;
            // Start the Flash video:
            this.embedSWF();
            return this;
        },

        /**
         * <p>Register a function to be called after a {@link org.cytoscapeweb.Visualization#draw} method is executed and the visualization
         * is ready to receive requests, such as getting or selecting nodes, zooming, etc.</p>
         * <p>If the application wants to interact with the rendered network, this function must be used
         * before calling the <code>draw</code> method.</p>
         *
         * @example
         * // 1. Create the visualization instance:
         * var vis = new org.cytoscapeweb.Visualization("container-id");
         *
         * // 2. Register a callback function for the ready event:
         * vis.ready(function () {
         *     // Write code to interact with Cytoscape Web, e.g:
         *     var nodes = vis.nodes();
         *     // and so on...
         * });
         *
         * // 3. And then call draw:
         * vis.draw({ network: '&lt;graphml&gt;...&lt;/graphml&gt;' });
         *
         * @param {Function} fn The callback function that will be invoked after the network has been drawn
         *                      and the visualization is ready.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#draw
         */
        ready: function (fn) {
            if (!fn) { this._onReady = function () {/*do nothing*/}; }
            else { this._onReady = fn; }
            return this;
        },

        /**
         * <p>If the <code>layout</code> argument is passed, it applies the layout to the network.
         * Otherwise it just returns the the current layout object.</p>
         * <p>In order to set a layout, you can send a layout object or just the layout name, 
         * if you want to use the default options.</p>
         * <p>See {@link org.cytoscapeweb.Layout} for the available options.</p>
         *
         * @example
         * // 1. Initialise Cytoscape Web with a Circle layout (default layout options):
         * var vis = new org.cytoscapeweb.Visualization("container-id");
         * vis.draw({ network: '&lt;graphml&gt;...&lt;/graphml&gt;', layout: 'Circle' });
         *
         * // 2. Get the current layout:
         * var layout = vis.layout(); // returns: { name: 'Circle', options: { angleWidth: 360 } };
         * 
         * // 3. Apply a new layout, using default options:
         * vis.layout('ForceDirected');
         *
         * // 4. Apply a new layout with custom options:
         * var options = { 
         *      drag:          0.2,
         *      gravitation:   -200,
         *      minDistance:   1,
         *      maxDistance:   400,
         *      mass:          2,
         *      tension:       0.2,
         *      weightAttr:    "weight",
         *      restLength:    100,
         *      iterations:    200,
         *      maxTime:       10000,
         *      autoStabilize: false
         * };
         * vis.layout({ name: 'ForceDirected', options: options });
         *
         * @param {Object} [layout] The {@link org.cytoscapeweb.Layout} object or the layout name.
         * @return <ul><li>The current {@link org.cytoscapeweb.Layout} object for <code>layout()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>layout({Object})</code>.</li></ul>
         * @see org.cytoscapeweb.Layout
         */
        layout: function (/*layout*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.applyLayout(arguments[0]); return this; }
            else { return swf.getLayout(); }
        },

        /**
         * <p>If the <code>style</code> argument is passed, it applies that visual style to the network.
         * Otherwise it just returns the current visual style object.</p>
         * @param {org.cytoscapeweb.VisualStyle} [style] An object that contains the desired visual properties and attribute mappings.
         * @example
         * var style = {
         *         global: {
         *             backgroundColor: "#000000"
         *         },
         *         nodes: {
         *             color: "#ffffff",
         *             size: 40
         *         }
         * };
         * 
         * vis.visualStyle(style);
         * 
         * @return <ul><li>The {@link org.cytoscapeweb.VisualStyle} object for <code>visualStyle()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>visualStyle({Object})</code>.</li></ul>
         * @see org.cytoscapeweb.VisualStyle
         * @see org.cytoscapeweb.Visualization#visualStyleBypass
         */
        visualStyle: function (/*style*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.setVisualStyle(arguments[0]); return this; }
            else { return swf.getVisualStyle(); }
        },

        /**
         * <p>If the <code>bypass</code> argument is passed, it sets a visual style bypass on top of the regular styles.
         * Otherwise it just returns the current bypass object.</p>
         * <p>It allows you to override the visual styles (including the ones set by mappers) for individual nodes and edges,
         * which is very useful when the default visual style mechanism is not enough to create the desired effect.</p>
         * @example
         * // Change the labels of selected nodes and edges:
         * var selected = vis.selected();
         * 
         * var bypass = { nodes: { }, edges: { } };
         * var props = { 
         *         labelFontSize: 16,
         *         labelFontColor: "#ff0000",
         *         labelFontWeight: "bold"
         * };
         * 
         * for (var i=0; i < selected.length; i++) {
         *     var obj = selected[i];
         *     
         *     // obj.group is either "nodes" or "edges"...
         *     bypass[obj.group][obj.data.id] = props;
         * }
         * 
         * vis.visualStyleBypass(bypass);
         * 
         * @example
         * // To remove a bypass, just set <code>null</code> or an empty object:
         * vis.visualStyleBypass(null);
         *
         * @param {org.cytoscapeweb.VisualStyleBypass} bypass The visual properties for nodes and edges. Must be a map that has nodes/edges
         *                                                    ids as keys and the desired visual properties as values.
         *                                                    The visual properties are the same ones used by the VisualStyle objects, except that
         *                                                    <code>global</code> properties cannot be bypassed and are just ignored. Another difference is that you
         *                                                    cannot set visual mappers, but only static values.
         * @return <ul><li>The {@link org.cytoscapeweb.VisualStyleBypass} object for <code>visualStyleBypass()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} instance for <code>visualStyleBypass({Object})</code>.</li></ul>
         * @see org.cytoscapeweb.VisualStyleBypass
         * @see org.cytoscapeweb.Visualization#visualStyle
         */
        visualStyleBypass: function (/*bypass*/) {
            var swf = this.swf();
            var json;
            if (arguments.length > 0) {
            	json = JSON.stringify(arguments[0]); // to avoid errors with special characters in node IDs
            	swf.setVisualStyleBypass(json);
            	return this;
            } else {
            	json = swf.getVisualStyleBypass();
            	return this._parseJSON(json);
            }
        },

        /**
         * <p>If the boolean argument is passed, it shows or hides the built-in pan-zoom control.</p>
         * <p>If not, it just returns a boolean value indicating whether or not the control is visible.</p>
         * @param {Boolean} [visible] true to show it and false to hide it.
         * @return <ul><li>A boolean value for <code>panZoomControlVisible()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>panZoomControlVisible({Boolean})</code>.</li></ul>
         */
        panZoomControlVisible: function (/*visible*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.showPanZoomControl(arguments[0]); return this; }
            else { return swf.isPanZoomControlVisible(); }
        },

        /**
         * <p>If the boolean argument is passed, it merges or unmerge all the edges and returns the Visualization object.</p>
         * <p>If not, it returns a boolean value indicating whether or not the edges are merged.</p>
         * @param {Boolean} [merged] true to merge the edges or false to unmerge them.
         * @return <ul><li>A boolean value for <code>edgesMerged()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>edgesMerged({Boolean})</code>.</li></ul>
         */
        edgesMerged: function (/*merged*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.mergeEdges(arguments[0]); return this; }
            else { return swf.isEdgesMerged(); }
        },

        /**
         * <p>If the boolean argument is passed, it shows or hides all the node labels and returns the Visualization object.</p>
         * <p>If not, it returns a boolean value indicating whether or not the node labels are visible.</p>
         * @param {Boolean} [visible] true to show the labels or false to hide them.
         * @return <ul><li>A boolean value for <code>nodeLabelsVisible()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>nodeLabelsVisible({Boolean})</code>.</li></ul>
         */
        nodeLabelsVisible: function (/*visible*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.showNodeLabels(arguments[0]); return this; }
            else { return swf.isNodeLabelsVisible(); }
        },

        /**
         * <p>If the boolean argument is passed, it shows or hides all the edge labels and returns the Visualization object.</p>
         * <p>If not, it returns a boolean value indicating whether or not the edge labels are visible.</p>
         * @param {Boolean} [visible] true to show the labels or false to hide them.
         * @return <ul><li>A boolean value for <code>edgeLabelsVisible()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>edgeLabelsVisible({Boolean})</code>.</li></ul>
         */
        edgeLabelsVisible: function (/*visible*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.showEdgeLabels(arguments[0]); return this; }
            else { return swf.isEdgeLabelsVisible(); }
        },

        /**
         * <p>If the boolean argument is passed, it enables or disables the node tooltips.</p>
         * <p>If not, it returns a boolean value indicating whether or not the node tooltips are enabled.</p>
         * @param {Boolean} [enabled] true to enable the tooltips or false to disable them.
         * @return <ul><li>A boolean value for <code>nodeTooltipsEnabled()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>nodeTooltipsEnabled({Boolean})</code>.</li></ul>
         */
        nodeTooltipsEnabled: function (/*enabled*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.enableNodeTooltips(arguments[0]); return this; }
            else { return swf.isNodeTooltipsEnabled(); }
        },

        /**
         * <p>If the boolean argument is passed, it enables or disables the edge tooltips.</p>
         * <p>If not, it returns a boolean value indicating whether or not the edge tooltips are enabled.</p>
         * @param {Boolean} [enabled] true to enable the tooltips or false to disable them.
         * @return <ul><li>A boolean value for <code>edgeTooltipsEnabled()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>edgeTooltipsEnabled({Boolean})</code>.</li></ul>
         */
        edgeTooltipsEnabled: function (/*enabled*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.enableEdgeTooltips(arguments[0]); return this; }
            else { return swf.isEdgeTooltipsEnabled(); }
        },
        
        /**
         * <p>If the boolean argument is passed in, it enables or disables custom mouse cursors, such as the hand icon used when panning the network.</p>
         * <p>If no argument is passed in, it returns a boolean value indicating whether or not custom cursors are enabled.</p>
         * @param {Boolean} [enabled] If <code>true</code>, custom (Flash) cursors can be used in some actions.
         *                            If <code>false</code>, Cytoscape Web will never replace the operating system's cursors.
         * @return <ul><li>A boolean value for <code>customCursorsEnabled()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>panEnabled({Boolean})</code>.</li></ul>
         */
        customCursorsEnabled: function (/*enabled*/) {
        	if (arguments.length > 0) { this.swf().enableCustomCursors(arguments[0]); return this; }
        	else { return this.swf().isCustomCursorsEnabled(); }
        },

        /**
         * <p>If the boolean argument is passed, it enables or disables the "grab to pan" mode.
         * It's like clicking the "grab to pan" button in the pan-zoom control.</p>
         * <p>If no argument is passed, it returns a boolean value indicating whether or not the pan mode is enabled.</p>
         * @param {Boolean} [enabled] If <code>true</code>, clicking and dragging the background will pan the network.
         *                            If <code>false</code>, the pan mode is turned off - clicking and dragging the background
         *                            will start a drag-selection action.
         * @return <ul><li>A boolean value for <code>panEnabled()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>panEnabled({Boolean})</code>.</li></ul>
         * @see org.cytoscapeweb.Visualization#panBy
         * @see org.cytoscapeweb.Visualization#panToCenter
         */
        panEnabled: function (/*enabled*/) {
            if (arguments.length > 0) { this.swf().enableGrabToPan(arguments[0]); return this; }
            else { return this.swf().isGrabToPanEnabled(); }
        },
        
        /**
         * <p>Pan the "camera" by the specified amount, in pixels.</p>
         * @param {Number} amountX If negative, pan left (the network moves to the right side).
         * @param {Number} amountY If negative, pan up (the network moves down).
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#panEnabled
         * @see org.cytoscapeweb.Visualization#panToCenter
         */
        panBy: function (amountX, amountY) {
            this.swf().panBy(amountX, amountY);
            return this;
        },

        /**
         * <p>Center the network in the canvas area.</p>
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#panEnabled
         * @see org.cytoscapeweb.Visualization#panBy
         */
        panToCenter: function () {
            this.swf().panToCenter();
            return this;
        },

        /**
         * <p>If the scale argument is passed, it changes the zoom level of the network.
         * Otherwise it gets the current zoom value.</p>
         * @param {Number} [scale] Value between 0 and 1.
         * @return <ul><li>A number for <code>zoom()</code>.</li>
         *             <li>The {@link org.cytoscapeweb.Visualization} object for <code>zoom({Number})</code>.</li></ul>
         * @see org.cytoscapeweb.Visualization#zoomToFit
         */
        zoom: function (/*scale*/) {
            var swf = this.swf();
            if (arguments.length > 0) { swf.zoomTo(arguments[0]); return this; }
            else { return swf.getZoom(); }
        },

        /**
         * <p>Change the scale of the network until it fits the screen.</p>
         * <p>If the network scale is or reaches 1 (100%) and it's not cropped, it is not zoomed in to more than that.
         * It also centers the network, even if the scale was not changed.</p>
         * <p>It does not return the result scale.
         * If you want to get the applied zoom level, add an event listener before calling <code>zoomToFit</code>.</p>
         * @example
         * var scale;
         * vis.addListener("zoom", function(evt) {
         *     scale = evt.value;
         * });
         * vis.zoomToFit();
         *
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#zoom
         */
        zoomToFit: function () {
            this.swf().zoomToFit();
            return this;
        },

        /**
         * <p>Get one node by its unique ID.</p>
         * @example
         * var node = vis.node("n4");
         * 
         * @param {String} id The node id.
         * @return {org.cytoscapeweb.Node} The node object or <code>null</code>, if there is no node with the specified id.
         * @see org.cytoscapeweb.Visualization#edge
         * @see org.cytoscapeweb.Visualization#nodes
         */
        node: function (id) {
            var str = this.swf().getNodeById(id);
            return this._parseJSON(str);
        },
        
        /**
         * <p>Get all nodes from the network.</p>
         * <p>If the <code>topLevelOnly</code> parameter is <code>true</code>, child nodes are not returned in the list.</p>
         * In order to retrieve the children of a compound node, simply pass the parent node object or its <code>id</code>.</p>
         * @param {Boolean} [topLevelOnly] It is optional and the default value is <code>false</code>, which means that all existing nodes
                                           are returned as a flat list, no matter whether or not they are regular, child or parent ones.
                                           If <code>false</code>, nodes that are children of other nodes are not included in the list. 
         * @return {Array} List of {@link org.cytoscapeweb.Node} objects.
         * @see org.cytoscapeweb.Visualization#edges
         * @see org.cytoscapeweb.Visualization#node
         * @see org.cytoscapeweb.Visualization#parentNodes
         * @see org.cytoscapeweb.Visualization#childNodes
         */
        nodes: function (topLevelOnly) {
            var str = this.swf().getNodes(topLevelOnly);
            return this._parseJSON(str);
        },
        
        /**
         * <p>Get all nodes that belong to a compound node.</p>
         * @param {Object} parent The parent node object or the parent ID (String).
         * @return {Array} List of {@link org.cytoscapeweb.Node} objects.
         * @see org.cytoscapeweb.Visualization#edges
         * @see org.cytoscapeweb.Visualization#nodes
         * @see org.cytoscapeweb.Visualization#parentNodes
         * @see org.cytoscapeweb.Visualization#node
         */
        childNodes: function (parent) {
        	if (parent == null) { throw("The 'parent' parameter is mandatory."); }
        	if (typeof parent === "object" && parent.hasOwnProperty("data")) {
        		if (typeof parent.data === "object") { parent = parent.data.id; }
        	}
        	var str = this.swf().getChildNodes(parent);
        	return this._parseJSON(str);
        },
        
        /**
         * <p>Get only the compound nodes from the network, if there is any.</p>
         * @return {Array} List of {@link org.cytoscapeweb.Node} objects that contain one or more child nodes.
         * @see org.cytoscapeweb.Visualization#edges
         * @see org.cytoscapeweb.Visualization#nodes
         * @see org.cytoscapeweb.Visualization#childNodes
         * @see org.cytoscapeweb.Visualization#node
         */
        parentNodes: function () {
        	var str = this.swf().getParentNodes();
        	return this._parseJSON(str);
        },

        /**
         * <p>Get one edge, including any merged edge, by its unique ID.</p>
         * @example
         * var edge = vis.edge("e10");
         * 
         * @param {String} id The edge id.
         * @return {org.cytoscapeweb.Edge} The edge object or <code>null</code>, if there is no edge with the specified id.
         * @see org.cytoscapeweb.Visualization#node
         * @see org.cytoscapeweb.Visualization#edges
         */
        edge: function (id) {
            var str = this.swf().getEdgeById(id);
            return this._parseJSON(str);
        },
        
        /**
         * <p>Get all the regular edges from the network. Merged edges are not included.</p>
         * @return {Array} List of edges.
         * @see org.cytoscapeweb.Visualization#mergedEdges
         * @see org.cytoscapeweb.Visualization#nodes
         * @see org.cytoscapeweb.Visualization#edge
         * @see org.cytoscapeweb.Edge
         */
        edges: function () {
            var str = this.swf().getEdges();
            return this._parseJSON(str);
        },
        
        /**
         * <p>Get all merged edges from the network.</p>
         * @return {Array} List of merged edges.
         * @see org.cytoscapeweb.Visualization#edges
         * @see org.cytoscapeweb.Edge
         */
        mergedEdges: function () {
            var str = this.swf().getMergedEdges();
            return this._parseJSON(str);
        },
        
        /**
         * <p>Add new nodes and/or edges to the network.</p>
         * <p>The rules described in {@link org.cytoscapeweb.Visualization#addNode} and {@link org.cytoscapeweb.Visualization#addEdge}
         * are still valid here.</p>
         * <p>You can add nodes and edges together and, when doing so, the order of the elements in
         * the array do not matter, because Cytoscape Web will first add all the nodes, and then
         * the edges.</p>
         * <p>The new element must have a <code>group</code> field set to either <code>"nodes"</code> or <code>"edges"</code>,
         * because {@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} objects are untype objects and Cytoscape Web
         * has no other safe way of making the distinction between the two types.</p>
         * <p>If the new elements contain visual properties (e.g. color, opacity), they are simply ignored and Cytoscape Web applies
         * new visual property values according to the current {@link org.cytoscapeweb.VisualStyle} and {@link org.cytoscapeweb.VisualStyleBypass}.
         * The node positions (x, y), if specified, are respected, though.</p>
         * @example
         * // 1. Add two nodes with no data (IDs will be created automatically):
         * var nodesArray = [ { group: "nodes", x: 10, y: 35 },
         *                    { group: "nodes", x: 20, y: 70 } ];
         * var nodes = vis.addElements(nodesArray, true);
         * 
         * // 2. Add edges and nodes altogether:
         * var array = [ { group: "nodes", x: 10, y: 35, data: { id: "n01" } },
         *               { group: "nodes", x: 20, y: 70, data: { id: "n02" } },
         *               { group: "edges", data: { source: "n01", target: "n02" } } ];
         * var elements = vis.addElements(array, true);
         * @param {Array} [items] The nodes and edges to be added to the network.
         *                        The array must contain {@link org.cytoscapeweb.Node} and/or {@link org.cytoscapeweb.Edge} objects.
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to reapply the visual mappers
         *                                        to the network view after removing the elements.
         * @return {Array} The new created elements ({@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} objects).
         * @see org.cytoscapeweb.Visualization#removeElements
         * @see org.cytoscapeweb.Visualization#addNode
         * @see org.cytoscapeweb.Visualization#addEdge
         */
        addElements: function(/*items, updateVisualMappers*/) {
            var items, updateVisualMappers = false;
            if (arguments.length > 0 && this._typeof(arguments[0]) === "array") {
            	items = arguments[0];
            }
        	if (items == null) { throw("The 'items' object is mandatory."); }
            if (arguments.length > 1 && typeof arguments[1] === "boolean") {
            	updateVisualMappers = arguments[1];
            }
            var s = this.swf().addElements(items, updateVisualMappers);
            return this._parseJSON(s);
        },
        
        /**
         * <p>Create a new node and add it to the network view.<p>
         * <p>You can also add a node as a child of another node, by setting the parent node ID to the "parent" data attribute.<p>
         * <p>If the node <code>id</code> is not specified, Cytoscape Web creates a new one automatically.</p>
         * <p>If the data contains attributes that have not been previously defined in the {@link org.cytoscapeweb.DataSchema},
         * Cytoscape Web will throw an error. To prevent that, you simply add the new fields to the schema first, 
         * by calling {@link org.cytoscapeweb.Visualization#addDataField}.</p>
         * <p>Keep in mind that {@link org.cytoscapeweb.Visualization#addElements} is much faster if you have to
         * add more than one element at once.</p>
         * @example
         * // 1. Add a new node to the network (here we assume that the data fields for
         * // "label" and "weight" already exists in the node schema):
         * var data = { id: "n4",
         *              label: "MYO2 (Yeast)",
         *              weight: 0.54 };
         * var node1 = vis.addNode(240, 360, data, true);
         * 
         * // 2. Add a new node as a child of another (compound) node:
         * var node2 = vis.addNode(node1.x, node1.y, { parent: "n4" }, true);
         *
         * @param {Object} x The horizontal coordinate of the node.
         * @param {Object} y The vertical coordinate of the node.
         * @param {Object} [data] The object that contains the node attributes.
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to update and reapply the visual mappers
         *                                        to the network view after adding the node.
         *                                        The default value is <code>false</code>.
         * @return {org.cytoscapeweb.Node} The new created node object.
         * @see org.cytoscapeweb.Visualization#addEdge
         * @see org.cytoscapeweb.Visualization#addElements
         * @see org.cytoscapeweb.Visualization#removeElements
         */
        addNode: function (x, y/*, data, updateVisualMappers*/) {
            var data;
            var updateVisualMappers = false;
            var parentId = null;
            var i = 2;
            if (arguments.length > i && (typeof arguments[i] === "object" || arguments[i] == null)) {
            	data = arguments[i++];
            }
            if (arguments.length > i && typeof arguments[i] === "boolean") {
            	updateVisualMappers = arguments[i++];
            }
            var n = this.swf().addNode(x, y, data, updateVisualMappers);
            return this._parseJSON(n);
        },
         
        /**
         * <p>Create a new edge linking two nodes and add it to the network view.<p>
         * <p>If the edge <code>id</code> is not specified, Cytoscape Web creates a new one automatically.</p>
         * <p>However the <code>source</code> and <code>target</code> data fields are mandatory, and Cytoscape Web 
         * throws an error if any of them is missing.</p>
         * <p>If the data contains attributes that have not been previously defined in the {@link org.cytoscapeweb.DataSchema},
         * Cytoscape Web will throw an error. To prevent that, just add the new fields to the schema first, 
         * by calling {@link org.cytoscapeweb.Visualization#addDataField}.</p>
         * <p>You might also want to take a look at {@link org.cytoscapeweb.Visualization#addElements}, which is much faster
         * when adding more than one element at once.</p>
         * @example
         * var data = { id: "e10",
         *              source: "n1",
         *              target: "n4",
         *              directed: false,
         *              label: "Co-expression",
         *              weight: 0.88 };
         * 
         * var edge = vis.addEdge(data, true);
         * 
         * @param {Object} data The object that contains the edge attributes.
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to update and reapply the visual mappers
         *                                        to the network view after adding the edge.
         * @return {org.cytoscapeweb.Edge} The new created edge object.
         * @see org.cytoscapeweb.Visualization#addNode
         * @see org.cytoscapeweb.Visualization#addElements
         * @see org.cytoscapeweb.Visualization#removeElements
         */
        addEdge: function (data/*, updateVisualMappers*/) {
            var updateVisualMappers = false;
            if (data == null) { throw("The 'data' object is mandatory."); }
            if (data.source == null) { throw("The 'source' node ID mandatory."); }
            if (data.target == null) { throw("The 'target' node ID mandatory."); }
            if (arguments.length > 1) { updateVisualMappers = arguments[1]; }
            var e = this.swf().addEdge(data, updateVisualMappers);
            return this._parseJSON(e);
        },
        
        /**
         * <p>Permanently delete the specified node and its associated edges from the network.</p>
         * <p>If a node is deleted, all of its connected edges will be removed as well.</p>
         * @example
         * // 1. Pass the whole Node object:
         * var node = vis.nodes()[0];
         * vis.removeNode(node);
         * 
         * // 2. Or just specify the node id:
         * vis.removeNode("n3");
         * 
         * @param {Object} node The node to be removed from the network. It can be a {@link org.cytoscapeweb.Node}
         *                      object or just its <code>id</code> (String).
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to reapply the visual mappers
         *                                        to the network view after removing the element.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#removeEdge
         * @see org.cytoscapeweb.Visualization#removeElements
         * @see org.cytoscapeweb.Visualization#addNode
         * @see org.cytoscapeweb.Visualization#addEdge
         */
        removeNode: function(node, updateVisualMappers) {
            this.swf().removeElements("nodes", [node], updateVisualMappers);
            return this;
        },
        
        /**
         * <p>Permanently delete the specified edge from the network.</p>
         * <p>If the specified edge is a merged one, all of its "regular" edges are deleted as well.</p>
         * @example
         * // 1. Pass the whole Edge object:
         * var edge = vis.edges()[0];
         * vis.removeEdge(edge);
         * 
         * // 2. Or just pass the edge id:
         * vis.removeEdge("e101");
         * 
         * @param {Object} edge The edge to be removed from the network. It can be an {@link org.cytoscapeweb.Edge}
         *                      object or just its <code>id</code> (String).
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to reapply the visual mappers
         *                                        to the network view after removing the element.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#removeNode
         * @see org.cytoscapeweb.Visualization#removeElements
         * @see org.cytoscapeweb.Visualization#addNode
         * @see org.cytoscapeweb.Visualization#addEdge
         */
        removeEdge: function(edge, updateVisualMappers) {
            this.swf().removeElements("edges", [edge], updateVisualMappers);
            return this;
        },
        
        /**
         * <p>Permanently delete nodes and/or edges from the network.</p>
         * <p>If a node is deleted, all of its connected edges will be removed as well.</p>
         * @example
         * // 1. Remove edges by ID:
         * vis.removeElements("edges", ["1", "2", "5"]);
         * 
         * // 2. Remove edges and nodes altogether, by passing the objects to be deleted:
         * var nodes = vis.nodes();
         * var edges = vis.edges();
         * vis.removeElements([nodes[0], nodes[1], edges[0]], true);
         * 
         * // 3. Remove all edges:
         * vis.removeElements("edges");
         * 
         * // 4. Remove everything (nodes and edges):
         * vis.removeElements();
         * 
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements.
         * @param {Array} [items] The items to be removed from the network. The array can contain node/edge objects or only
         *                        their <code>id</code> values. Remember that, if you pass only the id
         *                        and do not pass the group argument, if an edge and a node have the same id value,
         *                        both will be removed.
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to reapply the visual mappers
         *                                        to the network view after removing the elements.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#removeNode
         * @see org.cytoscapeweb.Visualization#removeEdge
         * @see org.cytoscapeweb.Visualization#addNode
         * @see org.cytoscapeweb.Visualization#addEdge
         */
        removeElements: function(/*gr, items, updateVisualMappers*/) {
            var gr, items, updateVisualMappers = false;
            if (arguments.length >= 1) {
                if (typeof arguments[0] === "string") { gr = arguments[0]; }
                else if (this._typeof(arguments[0]) === "array") { items = arguments[0]; }
                else if (typeof arguments[0] === "boolean") { updateVisualMappers = arguments[0]; }
            }
            if (arguments.length >= 2) {
                if (this._typeof(arguments[1]) === "array") { items = arguments[1]; }
                else if (typeof arguments[1] === "boolean") { updateVisualMappers = arguments[1]; }
            }
            if (arguments.length > 2) { updateVisualMappers = arguments[2]; }
            gr = this._normalizeGroup(gr);
            this.swf().removeElements(gr, items, updateVisualMappers);
            return this;
        },
        
        /**
         * <p>Get the network data schema, which contains all the nodes and edges data fields.</p>
         * @example
         * var schema = vis.dataSchema();
         * var nodeFields = schema.nodes;
         * var edgeFields = schema.edges;
         * 
         * @return {org.cytoscapeweb.DataSchema} The data schema object.
         * @see org.cytoscapeweb.Visualization#addDataField
         * @see org.cytoscapeweb.Visualization#removeDataField
         * @see org.cytoscapeweb.Visualization#updateData
         */
        dataSchema: function () {
            return this._parseJSON(this.swf().getDataSchema());
        },
        
        /**
         * <p>Add a custom attribute definition to the current node or edge data schema.</p>
         * <p>If an attribute with the same name is already defined for the same group,
         * the attribute will not be added again or replace the previous definitions.</p>
         * @example
         * // 1: Add the same new field to nodes and edges data:
         * var field = { name: "url", type: "string", defValue: "http://cytoscapeweb.cytoscape.org/" };
         * vis.addDataField(field);
         * 
         * // 2: Add new field to nodes only:
         * var field = { name: "score", type: "number", defValue: 0.15 };
         * vis.addDataField("nodes", field);
         * 
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements. If no group is passed,
         *                                      Cytoscape Web will try to add the new field to both nodes and edges data schema.
         * @param {org.cytoscapeweb.DataField} dataField An object that contains the attribute definitions.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#removeDataField
         * @see org.cytoscapeweb.Visualization#updateData
         * @see org.cytoscapeweb.Visualization#dataSchema
         */
        addDataField: function (/*gr, dataField*/) {
        	var gr, dataField, i = 0;
        	if (arguments.length > 1) { gr = arguments[i++]; }
        	dataField = arguments[i];
        	if (dataField == null) { throw("The 'dataField' object is mandatory."); }
        	if (dataField.name == null) { throw("The 'name' of the data field is mandatory."); }
        	if (dataField.type == null)  { throw("The 'type' of the data field is mandatory."); }
        	gr = this._normalizeGroup(gr);
        	this.swf().addDataField(gr, dataField);
        	return this;
        },
        
        /**
         * <p>Remove a custom attribute definition from the data schema.</p>
         * <p>Remember that only custom metadata can be removed. Any attempt to remove the following data fields will be ignored:</p>
         * <ul>
         *     <li><code>id</code> (nodes and edges)</li>
         *     <li><code>source</code> (edges)</li>
         *     <li><code>target</code> (edges)</li>
         *     <li><code>directed</code> (edges)</li>
         * </ul>
         * @example
         * // 1: Remove a data field from nodes and edges:
         * vis.removeDataField("url");
         * 
         * // 2: Remove a data field from edges only:
         * vis.removeDataField("edges", "url");
         *
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements. If no group is passed,
         *                                      Cytoscape Web will try to remove the field from both nodes and edges data schema.
         * @param {String} name The name of the custom data field that will be removed.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#addDataField
         * @see org.cytoscapeweb.Visualization#updateData
         * @see org.cytoscapeweb.Visualization#dataSchema
         */
        removeDataField: function (/*gr, name*/) {
            var gr, name, i = 0;
            if (arguments.length > 1) { gr = arguments[i++]; }
            name = arguments[i];
            if (name == null) { throw("The 'name' of the data field is mandatory."); }
            gr = this._normalizeGroup(gr);
            this.swf().removeDataField(gr, name);
            return this;
        },
        
        /**
         * <p>This method updates nodes and edges <code>data</code> attributes. You can use it to
         * change the value of any existing data attribute, except:</p>
         * <ul>
         *     <li><code>id</code> (nodes and edges)</li>
         *     <li><code>source</code> (edges)</li>
         *     <li><code>target</code> (edges)</li>
         * </ul>
         * <p>You can only update <code>data</code> attributes. Visual properties such as <code>color</code>
         * and <code>width</code> cannot be updated with this method. In order to change visual properties,
         * use {@link org.cytoscapeweb.Visualization#visualStyle} or {@link org.cytoscapeweb.Visualization#visualStyleBypass}.</p>
         * <p>If you try to change an attribute that has not been previously defined, Cytoscape Web will throw an {@link org.cytoscapeweb.Error}.
         * In this case, you have to add the attribute definition first, by calling {@link org.cytoscapeweb.Visualization#addDataField}.</p>
         * <p>Another important thing to remember is that you cannot update merged edges data, since they are derived attributes.</p>
         * <p>Finally, all the continuous and custom mappers - defined by the current visual style - will be automatically recomputed after
         * updating the data.</p>
         * 
         * @example
         * // 1: Update only one node or edge:
         * var n = vis.nodes()[0];
         * n.data.label = "New Label...";
         * n.data.weight *= 2;
         * vis.updateData([n]);
         * 
         * // 2: Update more than one object at once:
         * var nodes = vis.nodes();
         * var n1 = nodes[0];
         * var n2 = nodes[1];
         * n1.data.label = "New Label for N1";
         * n2.data.label = "New Label for N2";
         * 
         * var e = vis.edges[0];
         * e.data.weight = 0.8;
         * 
         * vis.updateData([n1, n2, e]);
         * 
         * // 3: Update more than one object from the same group at once,
         * //    setting the same values to all of them:
         * var edge_ids = ["1","3","7"];
         * var data = { weight: 0.5, interaction: "pp" };
         * vis.updateData("edges", edge_ids, data);
         * 
         * // 4: Update more than one node and edge at once,
         * //    setting the same values to all of them:
         * var ids = ["n1","n2","e7","e10"];
         * var data = { weight: 0 };
         * vis.updateData(ids, data);
         * 
         * // 5: Update all nodes and edges with the same attribute values:
         * var data = { weight: 0 };
         * vis.updateData(data);
         * 
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements.
         * @param {Array} [items] The items to be updated. The array can contain node/edge objects or only
         *                        their <code>id</code> values. Notice however that, if you specify only the id
         *                        and do not pass the group argument, if an edge and a node have the same id value,
         *                        both will be updated.
         * @param {Object} [data] The data object that contains the attributes with the new values to be applied
         *                        to all the elements of the specified group or to the passed <code>items</code> only.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#addDataField
         * @see org.cytoscapeweb.Visualization#removeDataField
         * @see org.cytoscapeweb.Visualization#dataSchema
         */
        updateData: function (/*gr, items, data*/) {
            var gr, items, data;
            if (arguments.length === 1) {
                if (typeof arguments[0] === "string") { gr = arguments[0]; }
                else if (this._typeof(arguments[0]) === "array") { items = arguments[0]; }
                else { data = arguments[0]; }
            } else if (arguments.length === 2) {
                if (typeof arguments[0] === "string") {
                    gr = arguments[0];
                    if (this._typeof(arguments[1]) === "array") { items = arguments[1]; }
                    else { data = arguments[1]; }
                } else {
                    items = arguments[0];
                    data = arguments[1];
                }
            } else if (arguments.length > 2) {
                gr = arguments[0];
                items = arguments[1];
                data = arguments[2];
            }
            gr = this._normalizeGroup(gr);
            this.swf().updateData(gr, items, data);
            return this;
        },
        
        /**
         * <p>Select the indicated nodes and edges.</p>
         * <p>The same method can also be used to select all nodes/edges.
         * To do that, just omit the <code>items</code> argument and specify the group of elements to be selected.</p>
         * <p>If you send repeated or invalid elements, they will be ignored.</p>
         * @example
         * // a) Select nodes by id:
         * var ids = [1,3,5,10];
         * vis.select("nodes", ids);
         *
         * // b) Select one node:
         * // Notice that the group parameter ("nodes") is optional here,
         * // because it's sending a node object and not only its id.
         * var n = vis.nodes()[0];
         * vis.select([n]);
         *
         * // c) Select nodes and edges at the same time:
         * var n = vis.nodes()[0];
         * var e = vis.edges()[0];
         * vis.select([n,e]);
         *
         * // d) Select all nodes:
         * vis.select("nodes");
         * 
         * // e) Select all edges:
         * vis.select("edges");
         *
         * // f) Select all nodes and all edges:
         * vis.select();
         *
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements.
         * @param {Array} [items] The items to be selected. The array can contain node/edge objects or only
         *                        their <code>id</code> values. Notice however that, if you specify only the id
         *                        and do not pass the group argument, if an edge and a node have the same id value,
         *                        both will be selected.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#deselect
         * @see org.cytoscapeweb.Visualization#selected
         */
        select: function (/*gr, items*/) {
            var gr, items;
            if (arguments.length === 1) {
                if (typeof arguments[0] === "string") { gr = arguments[0]; }
                else { items = arguments[0]; }
            } else if (arguments.length > 1) {
                 gr = arguments[0];
                 items = arguments[1];
            }
            gr = this._normalizeGroup(gr);
            this.swf().select(gr, items);
            return this;
        },

        /**
         * <p>Get all selected nodes or edges from the network.</p>
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements.
         * @return {Array} List of node or edge objects. If the group is not passed or is <code>null</code>,
         *                 the returned array may contain both nodes and edges.
         * @see org.cytoscapeweb.Visualization#select
         * @see org.cytoscapeweb.Visualization#deselect
         */
        selected: function (gr) {
            return this._nodesAndEdges(gr, "getSelectedNodes", "getSelectedEdges");
        },

        /**
         * <p>Deselect the indicated nodes and edges, if they are selected.</p>
         * <p>The same method can also be used to deselect all nodes/edges.
         * To do that, just omit the <code>items</code> argument and specify the group of elements to be deselected.</p>
         * <p>If you send repeated or invalid elements, they will be ignored.</p>
         * @example
         * // a) Deselect edges by id:
         * var ids = [4,6,21];
         * vis.deselect("edges", ids);
         *
         * // b) Deselect one edge only:
         * // Notice that the group parameter ("edges") is optional here,
         * // because it's sending an edge object and not only its id.
         * var e = vis.selected("edges")[0]; // assuming there is at least one selected edge!
         * vis.deselect([e]);
         *
         * // c) Deselect nodes and edges at the same time:
         * var n = vis.selected("nodes")[0];
         * var e = vis.selected("edges")[0];
         * vis.deselect([n,e]);
         *
         * // d) Deselect all nodes:
         * vis.deselect("nodes");
         * 
         * // e) Deselect all edges:
         * vis.deselect("edges");
         *
         * // f) Deselect all nodes and all edges:
         * vis.deselect();
         *
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements.
         *                                      If not specified, it will try to deselect elements from both <code>node</code>
         *                                      and <code>edge</code> groups.
         * @param {Array} [items] The items to be deselected. The array can contain node/edge objects or only
         *                        their <code>id</code> values. Notice however that, if you specify only the id
         *                        and do not pass the group argument, and if an edge and a node have the same id value,
         *                        both will be deselected.<br>
         *                        If this argument is <code>null</code>, <code>undefined</code> 
         *                        or omitted, it will deselect all selected items that belong to the indicated group.<br>
         *                        If you send an empty array, no action will be performed.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#select
         * @see org.cytoscapeweb.Visualization#selected
         */
        deselect: function (/*gr, items*/) {
            var gr, items;
            if (arguments.length === 1) {
                if (typeof arguments[0] === "string") { gr = arguments[0]; }
                else { items = arguments[0]; }
            } else if (arguments.length > 1) {
                gr = arguments[0];
                items = arguments[1];
            }
            gr = this._normalizeGroup(gr);
            this.swf().deselect(gr, items);
            return this;
        },

        /**
         * <p>Filter nodes or edges. The filtered out elements will be hidden.</p>
         * @example
         * // 1. Hide all edges that have a weight that is lower than 0.4:
         * vis.filter("edges", function(edge) {
         *     return edge.data.weight >= 0.4;
         * }, true);
         *
         * // 2. Hide all nodes, except two of them, by id:
         * vis.filter("nodes", ['n01', 'n02']);
         *
         * // 3. Hide all nodes (and edges, of course):
         * vis.filter("nodes", []);
         *
         * // 4. This also hides everything:
         * vis.filter();
         *
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements to filter.
         *                                       If <code>null</code>, filter both nodes and edges.
         * @param {Object} filter A filter function or an array of elements to be filtered.
         *                        If a function is passed, it will receive a node or edge as argument and must
         *                        return a boolean value indicating the visibility of that element.
         *                        So, if it returns false, that node or edge will be hidden.
         *                        If the argument is an array, it must contain the IDs or the Node/Edge objects
         *                        you want to make or keep visible.
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to update and reapply the visual mappers
         *                                        to the network view after the filtering action is done.
         *                                        Remember that continuous mappers ignore filtered out elements
         *                                        when interpolating the results.
         *                                        The default value is <code>false</code>.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#removeFilter
         * @see org.cytoscapeweb.ContinuousMapper
         */
        filter: function (/*gr, filter, updateVisualMappers*/) {
            var gr, filter, arr = [], updateVisualMappers = false;
            if (arguments.length > 2) {
                gr = arguments[0];
                filter = arguments[1];
                updateVisualMappers = arguments[2];
            } else if (arguments.length === 2) {
                if (typeof arguments[0] === 'string') {
                    gr = arguments[0];
                    filter = arguments[1];
                } else {
                    filter = arguments[0];
                    updateVisualMappers = arguments[1];
                }
            } else if (arguments.length === 1) {
            	filter = arguments[0];
            }
            gr = this._normalizeGroup(gr);
            if (typeof filter === "function") {
            	var list = this._nodesAndEdges(gr, "getNodes", "getEdges");
            	if (list.length > 0) {
	                for (var i = 0; i < list.length; i++) {
	                    var obj = list[i];
	                	if (filter(obj)) { arr.push(obj.data.id); }
	                }
                }
            } else if (this._typeof(filter) === "array") {
            	arr = filter;
            }
            this.swf().filter(gr, arr, updateVisualMappers);
            return this;
        },

        /**
         * <p>Remove a nodes or edges filter.</p>
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements to remove the filter from.
         *                                       If <code>null</code>, remove any existing filters from both nodes and edges.
         * @param {Boolean} [updateVisualMappers] It tells Cytoscape Web to update and reapply the visual mappers
         *                                        to the network view after the filtering action is done.
         *                                        Remember that continuous mappers ignore filtered out elements
         *                                        when interpolating the results.
         *                                        The default value is <code>false</code>.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#filter
         * @see org.cytoscapeweb.ContinuousMapper
         */
        removeFilter: function (gr, updateVisualMappers) {
            gr = this._normalizeGroup(gr);
            this.swf().removeFilter(gr, updateVisualMappers);
            return this;
        },

        /**
         * <p>Return the first neighbors of one or more nodes.</p>
         * @param {Array} nodes Array of node objects or node IDs.
         * @param {Boolean} [ignoreFilteredOut] If <code>true</code>, the algorithm will ignore any filtered out node and edge.
         *                                      The default value is <code>false</code>.
         * @return An object that contains the following properties: 
         *         <ul class="options"><li><code>rootNodes</code> {Array}: the node objects that were passed as the function parameter.</li>
         *             <li><code>neighbors</code> {Array}: the node objects that are neighbors of the root ones.</li>
         *             <li><code>edges</code> {Array}: the edge objects that connects the root and the neighbor nodes.</li>
         *             <li><code>mergedEdges</code> {Array}: the merged edge objects that connect the returned nodes.</li></ul>.
         */
        firstNeighbors: function (nodes, ignoreFilteredOut) {
            var str = this.swf().firstNeighbors(nodes, ignoreFilteredOut);
            return this._parseJSON(str);
        },

        /**
         * <p>Return the network model as an object.</p>
         * @return {org.cytoscapeweb.NetworkModel} The network model as a JavaScript object.
         * @see org.cytoscapeweb.Visualization#graphml
         * @see org.cytoscapeweb.Visualization#xgmml
         * @see org.cytoscapeweb.Visualization#sif
         */
        networkModel: function () {
        	var json = this.swf().getNetworkModel();
            return this._parseJSON(json);
        },
        
        /**
         * <p>Return the network data as <a href="http://graphml.graphdrawing.org/primer/graphml-primer.html" rel="external">GraphML</a>.</p>
         * @return {String} The XML text.
         * @see org.cytoscapeweb.Visualization#xgmml
         * @see org.cytoscapeweb.Visualization#sif
         * @see org.cytoscapeweb.Visualization#networkModel
         */
        graphml: function () {
        	return this.swf().getNetworkAsText("graphml");
        },

        /**
         * <p>Return the network data as <a href="http://www.cs.rpi.edu/~puninj/XGMML/" rel="external">XGMML</a>.</p>
         * @return {String} The XML text.
         * @see org.cytoscapeweb.Visualization#graphml
         * @see org.cytoscapeweb.Visualization#sif
         * @see org.cytoscapeweb.Visualization#networkModel
         */
        xgmml: function () {
            return this.swf().getNetworkAsText("xgmml");
        },
        
        /**
         * <p>Return the network data as <a href="http://cytoscape.wodaklab.org/wiki/Cytoscape_User_Manual/Network_Formats/" rel="external">Simple Interaction Format (SIF)</a>.</p>
         * <p>Cytoscape Web uses tab characters to delimit the fields, because the node and interaction names may contain spaces.</p>
         * <p>By default, the node name in the SIF text is taken from the node's <code>data.id</code> attribute.
         * You can choose any other node attribute to be the node name by passing the <code>nodeAttr</code> option. 
         * Of course the custom node field should have unique values.</p>
         * <p>Cytoscape Web tries to get the interaction name from the edge's <code>data.interaction</code> attribute.
         * You can choose any other edge attribute to be the interaction name by passing the <code>interactionAttr</code> option.
         * If the edge data does not have the defined interaction field, Cytoscape Web just uses the edge <code>id</code>.</p>
         * @example
         * var xml = '&lt;graphml&gt;' +
         *               // Create a custom node "label" attribute:
         *               '&lt;key id="label" for="node" attr.name="label" attr.type="string"/&gt;' +
         *               // Create a custom  edge "type" attribute:
         *               '&lt;key id="type" for="edge" attr.name="type" attr.type="string"/&gt;' +
         *               '&lt;graph&gt;' +
         *                   '&lt;node id="1"&gt;' +
         *                       '&lt;data key="label"&gt;Node 1&lt;/data&gt;' +
         *                   '&lt;/node&gt;' +
         *                   '&lt;node id="2"&gt;' +
         *                       '&lt;data key="label"&gt;Node 2&lt;/data&gt;' +
         *                   '&lt;/node&gt;' +
         *                   '&lt;edge source="1" target="2"&gt;' +
         *                       '&lt;data key="type"&gt;co-expression&lt;/data&gt;' +
         *                   '&lt;/edge&gt;' +
         *                   '&lt;edge source="2" target="1"&gt;' +
         *                       '&lt;data key="type"&gt;co-localization&lt;/data&gt;' +
         *                   '&lt;/edge&gt;' +
         *               '&lt;/graph&gt;' +
         *           '&lt;/graphml&gt;';
         *           
         * var vis = new org.cytoscapeweb.Visualization("container_id");
         * 
         * vis.ready(function() {
         *     // Export to SIF, using the "label" as node ID and edge "type" as edge interaction:
         *     var text = vis.sif({ nodeAttr: 'label', interactionAttr: 'type'});
         *     
         *     // text ==  'Node 1\tco-expression\tNode 2\n' +
         *     //          'Node 2\tco-localization\Node 1\n'
         * });
         * 
         * vis.draw({ network: xml });
         * 
         * @param {Object} [options] Non-mandatory settings:
         *                           <ul class="options">
         *                               <li><code>nodeAttr</code>:</strong> Optional node attribute name to be used as node name.</li>
         *                               <li><code>interactionAttr</code>:</strong> Optional edge attribute name to be used as interaction name.</li>
         *                           </ul>
         * @return {String} The SIF text.
         * @see org.cytoscapeweb.Visualization#graphml
         * @see org.cytoscapeweb.Visualization#xgmml
         * @see org.cytoscapeweb.Visualization#networkModel
         */
        sif: function (options) {
            return this.swf().getNetworkAsText("sif", options);
        },

        /**
         * <p>Return a PDF with the network vector image.</p>
         * @param {Object} [options] Additional options:
         *                           <ul class="options">
         *                               <li><code>width</code>:</strong> The desired width of the image in pixels.</li>
         *                               <li><code>height</code>:</strong> The desired height of the image in pixels.</li>
         *                           </ul>
         * @return {String} The PDF binary data encoded to a Base64 string.
         */
        pdf: function (options) {
            return this.swf().getNetworkAsImage("pdf", options);
        },
        
        /**
         * <p>Return an SVG image.</p>
         * @param {Object} [options] Additional options:
         *                           <ul class="options">
         *                               <li><code>width</code>:</strong> The desired width of the image in pixels.</li>
         *                               <li><code>height</code>:</strong> The desired height of the image in pixels.</li>
         *                           </ul>
         * @return {String} The SVG image.
         */
        svg: function (options) {
        	return this.swf().getNetworkAsImage("svg", options);
        },
        
        /**
         * <p>Return the network as a PNG image.</p>
         * @return {String} The PNG binary data encoded to a Base64 string.
         */
        png: function () {
            return this.swf().getNetworkAsImage("png");
        },

        /**
         * <p>Export the network to a URL.
         * It's useful when you want to download the network as an image or xml, for example.</p>
         * <p>This method requires a server-side part (e.g. Java, PHP, etc.) to receive the raw data from Cytoscape Web.
         * That server-side code should send the data back to the browser.</p>
         * @example
         * // The JavaScript code
         * vis.exportNetwork('xgmml', 'export.php?type=xml');
         * 
         * @example
         * &lt;?php
         *     # ##### The server-side code in PHP ####
         * 
         *     # Type sent as part of the URL:
         *     &#36;type = &#36;_GET['type'];
         *     # Get the raw POST data:
         *     &#36;data = file_get_contents('php://input');
         *
         *     # Set the content type accordingly:
         *     if (&#36;type == 'png') {
         *         header('Content-type: image/png');
         *     } elseif (&#36;type == 'pdf') {
         *         header('Content-type: application/pdf');
         *     } elseif (&#36;type == 'svg') {
         *        header('Content-type: image/svg+xml');
         *     } elseif (&#36;type == 'xml') {
         *         header('Content-type: text/xml');
         *     } elseif (&#36;type == 'txt') {
    	 *         header('Content-type: text/plain');
    	 *     }
         * 
         *     # To force the browser to download the file:
         *     header('Content-disposition: attachment; filename="network.' . &#36;type . '"');
         *     # Send the data to the browser:
         *     print &#36;data;
         * ?&gt;
         * 
         * @param {String} format One of: <code>"png"</code>, <code>"svg"</code>, <code>"pdf"</code>, <code>"xgmml"</code>, <code>"graphml"</code>, <code>"sif"</code>.
         * @param {String} url The url that will receive the exported image (bytes) or xml (text).
         * @param {Object} [options] Additional options:
         *                              <ul class="options">
         *                                  <li><code>width</code>:</strong> The desired width of the image in pixels (only for 'pdf' format).</li>
         *                                  <li><code>height</code>:</strong> The desired height of the image in pixels (only for 'pdf' format).</li>
         *                                  <li><code>nodeAttr</code>:</strong> Optional node attribute name to be used as the exported SIF node (only for 'sif' format).</li>
         *                                  <li><code>interactionAttr</code>:</strong> Optional edge attribute name to be used as the exported SIF interaction (only for 'sif' format).</li>
         *                                  <li><code>window</code>:</strong> The browser window or HTML frame in which to display the exported image or xml.
         *                                                  You can enter the name of a specific window or use one of the following values:
         *                                                  <ul><li><code>_self</code>: the current frame in the current window.</li>
         *                                                      <li><code>_blank</code>: a new window.</li>
         *                                                      <li><code>_parent</code>: the parent of the current frame.</li>
         *                                                      <li><code>_top</code>: the top-level frame in the current window.</li></ul>
         *                                                  The default is <code>_self</code>.</li>
         *                              </ul>
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#png
         * @see org.cytoscapeweb.Visualization#pdf
         * @see org.cytoscapeweb.Visualization#sif
         * @see org.cytoscapeweb.Visualization#graphml
         * @see org.cytoscapeweb.Visualization#svg
         * @see org.cytoscapeweb.Visualization#xgmml
         */
        exportNetwork: function (format, url, options) {
            format = format.toLowerCase().trim();
            this.swf().exportNetwork(format, url, options);
            return this;
        },

        /**
         * <p>Appends an event listener to the network.</p>
         * <p>Listeners can be added or removed at any time, even before the graph is rendered, which means that you do not
         * need to wait for the {@link org.cytoscapeweb.Visualization#ready} function to be called.</p>
         * 
         * @example
         * // 1. Create the visualization instance:
         * var vis = new org.cytoscapeweb.Visualization("container-id");
         * 
         * // 2. Add listeners at any time:
         * vis.addListener("zoom", function(evt) {
         *     var zoom = evt.value;
         *     alert("New zoom value is " + (zoom * 100) + "%");
         * })
         * .addListener("click", "edges", function(evt) {
         *     var edge = evt.target;
         *     alert("Edge " + edge.data.id + " was clicked");
         * })
         * .addListener("select", "nodes", function(evt) {
         *     var nodes = evt.target;
         *     alert(nodes.length + " node(s) selected");
         * });
         * 
         * // 3. Draw the network:
         * vis.draw({ network: '&lt;graphml&gt;...&lt;/graphml&gt;' });
         *  
         * @param {org.cytoscapeweb.EventType} evt The event type.
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements to assign the listener to (optional for some events).
         * @param {Function} fn The callback function the event invokes.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Event
         * @see org.cytoscapeweb.Visualization#hasListener
         * @see org.cytoscapeweb.Visualization#removeListener
         */
        addListener: function (evt, /*gr, */fn) {
            var gr;
            if (arguments.length > 2) {
                gr = arguments[1];
                fn = arguments[2];
            }
            evt = this._normalizeEvent(evt);
            gr = this._normalizeGroup(gr);

            if (!this._listeners) { this._listeners = {/* group: { event: listeners[] } */}; }
            if (!this._listeners[gr]) { this._listeners[gr] = {}; }

            var fnList = this._listeners[gr][evt];
            if (!fnList) {
                fnList = [];
                this._listeners[gr][evt] = fnList;
            }
            var duplicated = false;
            for (var i = 0; i < fnList.length; i++) {
                if (fn === fnList[i]) {
                    duplicated = true;
                    break;
                }
            }
            if (!duplicated) { fnList.push(fn); }
            return this;
        },

        /**
         * <p>Removes an event listener.</p>
         * @param {org.cytoscapeweb.EventType} evt The event type.
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements to assign the listener to (optional for some events).
         * @param {Function} [fn] The function the event invokes. If undefined, all registered functions
         *                        for the specified event are removed.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Event
         * @see org.cytoscapeweb.Visualization#addListener
         * @see org.cytoscapeweb.Visualization#hasListener
         */
        removeListener: function (evt/*, gr, fn*/) {
            var gr; var fn;

            if (arguments.length > 2) {
                gr = arguments[1];
                fn = arguments[2];
            } else if (arguments.length === 2) {
                if (typeof arguments[1] === 'function') { fn = arguments[1]; }
                else { gr = arguments[1]; }
            }
            evt = this._normalizeEvent(evt);
            gr = this._normalizeGroup(gr);

            var evtList;
            if (this._listeners) { evtList = this._listeners[gr] };

            if (evtList) {
                if (!fn) {
                    // Remove all of the event's functions:
                    delete evtList[evt];
                } else {
                    // Remove only the specified function:
                    var fnList = evtList[evt];
                    if (fnList) {
                        for (var i = 0; i < fnList.length; i++) {
                            if (fn === fnList[i]) {
                                fnList.splice(i, 1);
                                break;
                            }
                        }
                    }
                }
            }
            return this;
        },

        /**
         * <p>Tells whether or not there are listeners to an event type.</p>
         * @param {org.cytoscapeweb.EventType} evt The event type.
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements the listener was assigned to (optional for some events).
         * @return {Boolean} True if there is at least one listener to the event, false otherwise.
         * @see org.cytoscapeweb.Event
         * @see org.cytoscapeweb.Visualization#addListener
         * @see org.cytoscapeweb.Visualization#removeListener
         */
        hasListener: function (evt/*, gr*/) {
            var has = false;
            var gr;

            if (arguments.length > 1) { gr = arguments[1]; }
            evt = this._normalizeEvent(evt);
            gr = this._normalizeGroup(gr);

            if (this._listeners) {
                var evtList = this._listeners[gr];
                if (evtList) {
                    var fnList = evtList[evt];
                    has = fnList && fnList.length > 0;
                }
            }
            return has;
        },

        /**
         * <p>Adds a custom menu item to the right-click context menu.</p>
         * <p>This method can only be used after a network has been drawn, so it is better to use it after the
         * <code>ready</code> callback function is called (see {@link org.cytoscapeweb.Visualization#ready}).</p>
         * <p>If an item with the same label has already been set to the same group, it will not add another
         * callback function to that menu item. In that case, the previous function will be replaced by
         * the new one and only one menu item will be displayed.</p>
         * <p>It is possible to add more than one menu item with the same label, but only if they are added to
         * different groups.</p>
         * <p><b>Important:</b> Since the context menu is rendered by Flash, be aware of these 
         * <a href="http://help.adobe.com/en_US/FlashPlatform/reference/actionscript/3/flash/ui/ContextMenuItem.html" rel="external">restrictions</a>.</p>
         * 
         * @example
         * // We will use the context menu to select the first neighbors of the
         * // right-clicked node.
         * 
         * // 1. Assuming that you have created a visualization object:
         * var vis = new org.cytoscapeweb.Visualization("container-id");
         * 
         * // 2. Add a context menu item any time after the network is ready:
         * vis.ready(function () {
         *     vis.addContextMenuItem("Select first neighbors", "nodes", 
         *         function (evt) {
         *             // Get the right-clicked node:
         *             var rootNode = evt.target;
         *         
         *             // Get the first neighbors of that node:
         *             var fNeighbors = vis.firstNeighbors([rootNode]);
         *             var neighborNodes = fNeighbors.neighbors;
         *         
         *             // Select the root node and its neighbors:
         *             vis.select([rootNode]).select(neighborNodes);
         *         }
         *     );
         * });
         * @param {String} lbl The context menu item label to be displayed.
         * @param {org.cytoscapeweb.Group} [gr] The group of network elements the menu item will be assigned to.
         *                                       If <code>"nodes"</code>, the menu item will be visible only on right-clicks
         *                                       when the cursor is over a node. If <code>"edges"</code>, only when its over an edge.
         *                                       If <code>"none"</code> or no group is provided, the menu item will be available after a right-click
         *                                       over any network element, including the canvas background.
         * @param {Function} fn The callback function that is invoked after the user selects the injected menu item.
         *                      That function always receives an event object as argument. The event type is always <code>"contextmenu"</code>.
         *                      If the context menu was added to the <code>nodes</code> or <code>edges</code> group, you might want to
         *                      get the right-clicked node or edge object by using the event's <code>target</code> property.
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#removeContextMenuItem
         * @see org.cytoscapeweb.Visualization#removeAllContextMenuItems
         */
        addContextMenuItem: function (lbl, /*gr, */fn) {
            if (lbl && fn) {
                var gr;
                if (arguments.length > 2) {
                    gr = arguments[1];
                    fn = arguments[2];
                }
                gr = this._normalizeGroup(gr);

                if (!this._contextMenuItems) {
                    this._contextMenuItems = {/* group: {label: fn} */};
                }
                var grItems = this._contextMenuItems[gr];
                if (!grItems) {
                    grItems = {};
                    this._contextMenuItems[gr] = grItems;
                }
                grItems[lbl] = fn;
                this.swf().addContextMenuItem(lbl, gr);
            }
            return this;
        },

        /**
         * <p>Removes a menu item from the right-click context menu.</p>
         * @param {String} lbl The menu item label.
         * @param {org.cytoscapeweb.Group} [gr] <p>The related group. If <code>null</code>, and there is a menu item with the same label
         *                                        associated with a <code>"nodes"</code> or <code>"edges"</code> group, that item will not be removed.
         *                                        In that case, you need to call this function again with the other groups.</p>
         *                                        </p>For example, <code>removeContextMenuItem("Select")</code> does not remove the menu item
         *                                        added with <code>addContextMenuItem("Select", "edge")</code>, but only the the one added with
         *                                        <code>addContextMenuItem("Select")</code>.<p>
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#addContextMenuItem
         * @see org.cytoscapeweb.Visualization#removeAllContextMenuItems
         */
        removeContextMenuItem: function (lbl/*, gr*/) {
            if (lbl) {
                var gr;
                if (arguments.length > 1) { gr = arguments[1]; }
                gr = this._normalizeGroup(gr);
                if (this._contextMenuItems) {
                    var grItems = this._contextMenuItems[gr];
                    if (grItems) {
                        if (grItems[lbl]) {
                            this.swf().removeContextMenuItem(lbl, gr);
                            delete grItems[lbl];
                        }
                    }
                }
            }
            return this;
        },

        /**
         * <p>Removes all preset menu items from the right-click context menu.</p>
         * @return {org.cytoscapeweb.Visualization} The Visualization instance.
         * @see org.cytoscapeweb.Visualization#addContextMenuItem
         * @see org.cytoscapeweb.Visualization#removeContextMenuItem
         */
        removeAllContextMenuItems: function () {
            if (this._contextMenuItems) {
                for (var gr in this._contextMenuItems) {
                    if (this._contextMenuItems.hasOwnProperty(gr)) {
                        var grItems = this._contextMenuItems[gr];
                        if (grItems) {
                            for (var lbl in grItems) {
                                if (grItems.hasOwnProperty(lbl)) { this.removeContextMenuItem(lbl, gr); }
                            }
                        }
                    }
                }
            }
            return this;
        },

        /**
         * <p>Get Cytoscape Web's Flash object.</p>
         * @return {Object} The appropriate reference to the Flash object.
         */
        swf: function () {
            if (navigator.appName.indexOf("Microsoft") !== -1) {
                return window[this.id];
            } else {
                return document[this.id];
            }
        },

        /**
         * <p>Redefine this function if you want to use another method to detect the Flash Player version
         * and embed the SWF file (e.g. SWFObject).</p>
         * <p>By default, Adobe's <a href="http://www.adobe.com/products/flashplayer/download/detection_kit/" rel="external">Flash Player Detection Kit</a>
         * is used.</p>
         * @requires <code>AC_OETags.js</code> and <code>playerProductInstall.swf</code>
         */
        embedSWF: function () {
            //Major version of Flash required
            var requiredMajorVersion = 10;
            //Minor version of Flash required
            var requiredMinorVersion = 0;
            //Minor version of Flash required
            var requiredRevision = 0;

            var containerId = this.containerId;

            // Let's redefine the default AC_OETags function, because we don't necessarily want
            // to replace the whole HTML page with the swf object:
            AC_Generateobj = function (objAttrs, params, embedAttrs) {
                var str = '';
                var i;
                if (isIE && isWin && !isOpera) {
                    str += '<object ';
                    for (i in objAttrs) {
                        if (Object.hasOwnProperty.call(objAttrs, i)) {
                            str += i + '="' + objAttrs[i] + '" ';
                        }
                    }
                    str += '>';
                    for (i in params) {
                        if (Object.hasOwnProperty.call(params, i)) {
                            str += '<param name="' + i + '" value="' + params[i] + '" /> ';
                        }
                    }
                    str += '</object>';
                } else {
                    str += '<embed ';
                    for (i in embedAttrs) {
                        if (Object.hasOwnProperty.call(embedAttrs, i)) {
                            str += i + '="' + embedAttrs[i] + '" ';
                        }
                    }
                    str += '> </embed>';
                }
                // Replace only the indicated DOM element:
                document.getElementById(containerId).innerHTML = str;
            };

            // Version check for the Flash Player that has the ability to start Player Product Install (6.0r65)
            var hasProductInstall = DetectFlashVer(6, 0, 65);

            // Version check based upon the values defined in globals
            var hasRequestedVersion = DetectFlashVer(requiredMajorVersion, requiredMinorVersion, requiredRevision);

            if (hasProductInstall && !hasRequestedVersion) {
                // DO NOT MODIFY THE FOLLOWING FOUR LINES
                // Location visited after installation is complete if installation is required
                var MMPlayerType = (isIE === true) ? "ActiveX" : "PlugIn";
                var MMredirectURL = window.location;
                document.title = document.title.slice(0, 47) + " - Flash Player Installation";
                var MMdoctitle = document.title;

                AC_FL_RunContent(
                    "src", this.flashInstallerPath,
                    "FlashVars", "MMredirectURL="+MMredirectURL+'&MMplayerType='+MMPlayerType+'&MMdoctitle='+MMdoctitle+"",
                    "width", "100%",
                    "height", "100%",
                    "align", "middle",
                    "id", this.id,
                    "quality", "high",
                    "bgcolor", "#ffffff",
                    "name", this.id,
                    "allowScriptAccess","always",
                    "type", "application/x-shockwave-flash",
                    "pluginspage", "http://www.adobe.com/go/getflashplayer"
                );
            } else if (hasRequestedVersion) {
                var optionKeys = ["resourceBundleUrl"];
                var flashVars = "";
                if (this.options) {
                    for (var i in optionKeys) {
                        if (Object.hasOwnProperty.call(optionKeys, i)) {
                            var key = optionKeys[i];
                            if (this.options[key] !== undefined) {
                                flashVars += key + "=" + this.options[key] + "&";
                            }
                        }
                    }
                    flashVars += "id=" + this.id;
                }

                // if we've detected an acceptable version
                // embed the Flash Content SWF when all tests are passed
                AC_FL_RunContent(
                        "src", this.swfPath,
                        "width", "100%",
                        "height", "100%",
                        "align", "middle",
                        "id", this.id,
                        "quality", "high",
                        "bgcolor", "#ffffff",
                        "name", this.id,
                        "allowScriptAccess", "always",
                        "type", "application/x-shockwave-flash",
                        "pluginspage", "http://www.adobe.com/go/getflashplayer",
                        "wmode", "opaque", // DO NOT set it to "transparent", because it may crash FireFox and IE on Windows!
                        "flashVars", flashVars
                );
            } else { // flash is too old or we can't detect the plugin
                // Insert non-flash content:
                document.getElementById(containerId).innerHTML = this.flashAlternateContent;
            }
            return this;
        },

        // PRIVATE METHODS:
        // -----------------------------------------------------------------------------------------

        // --------------------------------------------
        // Used by the ActionScript External Interface:
        // --------------------------------------------

        /**
         * Workaround for a problem with Internet Explorer.
         * @ignore
         */
        _onBeforeComplete: function() {
            var backup1 = window.__flash__addCallback;
            window.__flash__addCallback = function (instance, name) { try {backup1(instance, name);} catch (x){} };
            var backup2 = window.__flash__removeCallback;
            window.__flash__removeCallback = function (instance, name) { try {backup2(instance, name);} catch (x){} };
        },

        /**
         * Callback for when the Flash object is completely loaded.
         * @ignore
         */
        _onComplete: function() {
            this.swf().draw(this.drawOptions);
        },

        _onReady: function () {
            // Do nothing.
        },

        /**
         * Proxy function called by the Flash side when the object must be sent as JSON format, usually
         * to avoid compatibility problems when converting complex Flash objects to JavaScript.
         * The JSON argument is converted to an Object and the destination function is called.
         * Do NOT redefine this function!!!
         * @ignore
         */
        _dispatch: function (functionName, jsonArg) {
            var arg = null;
            if (jsonArg != null) {
            	arg = this._parseJSON(jsonArg);
            }
            var ret = this[functionName](arg);
            return ret;
        },

        /**
         * Just a proxy to hasListener.
         * @ignore
         */
        _hasListener: function (evt) {
            return this.hasListener(evt.type, evt.group);
        },

        /**
         * Invokes each listener that was registered to a given event type.
         * @ignore
         */
        _invokeListeners: function (evt) {
            if (this._listeners) {
                var gr = this._normalizeGroup(evt.group);
                var evtList = this._listeners[gr];
                if (evtList) {
                    var type = this._normalizeEvent(evt.type);
                    var fnList = evtList[type];
                    for (var i = 0; i < fnList.length; i++) {
                        fnList[i](evt);
                    }
                }
            }
        },

        /**
         * Invokes a registered context menu callback function.
         * @ignore
         */
        _invokeContextMenuCallback: function (evt) {
            if (this._contextMenuItems) {
                var gr = this._normalizeGroup(evt.group);
                var grItems = this._contextMenuItems[gr];
                if (grItems) {
                    evt = new org.cytoscapeweb.Event(evt);
                    var fn = grItems[evt.value];
                    if (fn) { fn(evt); }
                }
            }
        },

        // --------------------------------------------
        // Utility functions:
        // --------------------------------------------

        _normalizeEvent: function (evt) {
            if (evt) { evt = evt.toLowerCase().trim(); }
            return evt;
        },

        _normalizeGroup: function (gr) {
            if (gr) { gr = gr.toLowerCase().trim(); }
            if (gr !== "nodes" && gr !== "edges") { gr = "none"; }
            return gr;
        },

        _nodesAndEdges: function (gr, fnNodes, fnEdges) {
            var list = [];
            gr = this._normalizeGroup(gr);
            if (gr === "nodes" || gr === "none") {
                var nodes = this._parseJSON(this.swf()[fnNodes]());
                list = list.concat(nodes);
            }
            if (gr === "edges" || gr === "none") {
                var edges = this._parseJSON(this.swf()[fnEdges]());
                list = list.concat(edges);
            }
            return list;
        },
        
        _typeof: function(v) {
            if (typeof(v) == "object") {
                if (v === null) return "null";
                if (v.constructor == (new Array).constructor) return "array";
                if (v.constructor == (new Date).constructor) return "date";
                if (v.constructor == (new RegExp).constructor) return "regex";
                return "object";
            }
            return typeof(v);
        },
        
        _parseJSON: function(s) {
        	return JSON.parse(s);
        }
    };

    if (!String.prototype.trim) {
        String.prototype.trim = function () {
            return this.replace(/(?:(?:^|\n)\s+|\s+(?:$|\n))/g,"");
        };
    }
    
    // ===[ Events ]================================================================================

    /**
     * <p>This object represents an Event.</p>
     * <p>Events are objects passed as arguments to listeners when an event occurs.</p>
     * <p>All event objects have at least the following fields:</p>
     *    <ul><li><code>type</code></li><li><code>group</code></li></ul>
     * <p>The following tables lists the possible properties for each event type.</p>
     * <p><label><strong>click:</strong></label> Fired when the user clicks an element that belongs to the <code>group</code> you registered. 
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired when the background of the network visualization is clicked.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>{@link org.cytoscapeweb.Node}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>{@link org.cytoscapeweb.Edge}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code>: clicking the visualization background</td><td><code>undefined</code></td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>dblclick:</strong></label> Fired when the user double clicks an element that belongs to the <code>group</code> you registered. 
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired when the background of the network visualization is double-clicked.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>{@link org.cytoscapeweb.Node}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>{@link org.cytoscapeweb.Edge}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code>: double-clicking the visualization background</td><td><code>undefined</code></td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>dragstart:</strong></label> Fired when the user starts dragging a node.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>{@link org.cytoscapeweb.Node}</td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>dragstop:</strong></label> Fired when the user stops dragging a node.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>{@link org.cytoscapeweb.Node}</td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>mouseover:</strong></label> Fired when the user moves the mouse over an element that belongs to the <code>group</code> you registered. 
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired any time the cursor enters the visualization rectangle.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>{@link org.cytoscapeweb.Node}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>{@link org.cytoscapeweb.Edge}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code>: mouse enters the visualization area</td><td><code>undefined</code></td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>mouseout:</strong></label> Fired when the user moves the mouse out of an element that belongs to the <code>group</code> you registered. 
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired when the cursor leaves the visualization area.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>{@link org.cytoscapeweb.Node}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>{@link org.cytoscapeweb.Edge}</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code>: mouse leaves the visualization area</td><td><code>undefined</code></td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>select:</strong></label> Fired when an element that belongs to the <code>group</code> you registered is selected.
     * Nodes and edges can be selected by three possible ways:
     * directly clicking it; using the drag-rectangle (the select event is dispatched only after the the mouse button is released); programmatically, with {@link org.cytoscapeweb.Visualization#select}. 
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired after selecting any nodes or edges.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>Array of selected {@link org.cytoscapeweb.Node} objects</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>Array of selected {@link org.cytoscapeweb.Edge} objects</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code></td><td>Array of selected {@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} objects</td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>deselect:</strong></label> Fired when an element that belongs to the <code>group</code> you registered is deselected.
     * Nodes and edges can be deselected by the user or programmatically, with {@link org.cytoscapeweb.Visualization#deselect}. 
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired after deselecting any nodes or edges.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>Array of deselected {@link org.cytoscapeweb.Node} objects</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>Array of deselected {@link org.cytoscapeweb.Edge} objects</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code></td><td>Array of deselected {@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} objects</td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>filter:</strong></label> Fired when the <code>group</code> you registered is filtered.
     * Nodes and edges can be filtered with {@link org.cytoscapeweb.Visualization#filter}.  
     * If you don't specify any group or if the group is <code>none</code>, the event will be fired after filtering nodes or edges elements.
     * It is important to be aware that if no element of the specified <code>group</code> is filtered (no filter applied), 
     * the event's <code>target</code> property will be <code>null</code>.
     * But if all the elements of that <code>group</code> is filtered out, <code>target</code> will be an empty array.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>Array of filtered {@link org.cytoscapeweb.Node} objects or <code>null</code></td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>Array of filtered {@link org.cytoscapeweb.Edge} objects or <code>null</code></td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code></td><td>Array of filtered {@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} objects or <code>null</code></td><td><code>undefined</code></td></tr>
     * </Table>
     * <p><label><strong>layout:</strong></label> Fired after a layout is applied (see {@link org.cytoscapeweb.Visualization#layout}.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>none</code></td><td><code>undefined</code></td><td><code>The applied {@link org.cytoscapeweb.Layout} object</code></td></tr>
     * </Table>
     * <p><label><strong>zoom:</strong></label> Fired after the network is rescaled, either by calling {@link org.cytoscapeweb.Visualization#zoom} or 
     * when the user interacts with the visualization's pan-zoom control.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>none</code></td><td><code>undefined</code></td><td>The zoom value (float number from 0 to 1)</td></tr>
     * </Table>
     * <p><label><strong>error:</strong></label> Fired when an exception is thrown.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>none</code></td><td><code>undefined</code></td><td>The {@link org.cytoscapeweb.Error} object</td></tr>
     * </Table>
     * <p><label><strong>contextmenu:</strong></label> Events of this type are only passed to the callback functions that are registered with {@link org.cytoscapeweb.Visualization#addContextMenuItem}.
     * You cannot add listeners to this event.</p>
     * <table>
     *     <tr><th>group</th><th>target</th><th>value</th></tr>
     *     <tr><td><code>nodes</code></td><td>The related {@link org.cytoscapeweb.Node} object</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>edges</code></td><td>The related  {@link org.cytoscapeweb.Edge} object</td><td><code>undefined</code></td></tr>
     *     <tr><td><code>none</code></td><td>The {@link org.cytoscapeweb.Node} or {@link org.cytoscapeweb.Edge} object, if a node or edge was right-clicked. Or <code>undefined</code>, if the right click was done on an empty background area.</td><td><code>undefined</code></td></tr>
     * </Table>
     * 
     * @class
     * @see org.cytoscapeweb.EventType
     * @see org.cytoscapeweb.Visualization#addListener
     * @see org.cytoscapeweb.Visualization#hasListener
     * @see org.cytoscapeweb.Visualization#removeListener
     */
    org.cytoscapeweb.Event = function (options) {
        /**
         * The event type name.
         * @type org.cytoscapeweb.EventType
         */
        this.type = options.type;
        /**
         * The group of network elements the event is related to.
         * @type org.cytoscapeweb.Group
         */
        this.group = options.group;
        /**
         * The event target. For example, if one or more nodes are selected, the target of the 
         * <code>"select"</code> event will be an array of node objects.
         * But if a node is clicked, the target of the <code>"click"</code> event will be just a node object.
         * This property is available only for event types that are related to actions performed on nodes or edges.
         * For the other events it is <code>undefined</code>.
         * @type Object
         */
        this.target = options.target;
        /**
         * This property is a very generic one and is usually used to send back any important value that
         * is not defined as <code>target</code>. For example, for <code>"zoom"</code> events, value is
         * the new scale, but for <code>"error"</code> events it is an error object.
         */
        this.value = options.value;
        /**
         * The local x coordinate of the mouse position, in pixels.
         * Available only when the event type is 'click', 'dblclick', 'mouseover', 'mouseout' or 'contextmenu'.
         */
        this.mouseX = options.mouseX;
        /**
         * The local y coordinate of the mouse position, in pixels.
         * Available only when the event type is 'click', 'dblclick', 'mouseover', 'mouseout' or 'contextmenu'.
         */
        this.mouseY = options.mouseY;
    };
    
    // ===[ NetworkModel ]==========================================================================
    
    /**
     * <p>This object represents a NetworkModel type, but is actually just an untyped object.</p>
     * <p>It defines the raw data (nodes and edges data values) and the data schema for a network.
     * It is important to notice that the network model does <b>not</b> contain {@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} objects, 
     * as it is not supposed to describe visual attributes such as colors, shapes and x/y coordinates.
     * Visual styles must be defined separately, through {@link org.cytoscapeweb.VisualStyle} or {@link org.cytoscapeweb.VisualStyleBypass}.
     * Nodes positioning are done by {@link org.cytoscapeweb.Layout} objects.</p>
     * <p>A NetworkModel object has only two fields:</p>
     * <ul class="options">
      *     <li><code>dataSchema</code> {{@link org.cytoscapeweb.DataSchema}}: It defines the nodes/edges data fields.
      *                                 You do not need to specify these essential fields: 
      *                                 <code>id</code> (nodes or edges), <code>source</code> (edges), <code>target</code> (edges), <code>directed</code> (edges).
      *                                 Actually, trying to modify these fields in the schema might throw an {@link org.cytoscapeweb.Error}.</li>
      *     <li><code>data</code> {Object}: The actual nodes/edges data values used to create {@link org.cytoscapeweb.Node} and {@link org.cytoscapeweb.Edge} elements.
      *                                     It contains two fields (<code>nodes</code> and <code>edges</code>), which are arrays of nodes/edges data objects.
      *                                     Note: data attributes of type <code>int</code> or <code>boolean</code> (see {@link org.cytoscapeweb.DataField}) 
      *                                     do NOT accept <code>null</code> values.</li>
      *</ul>
     * @example
     * var network = {
     * 
     *     dataSchema: {
     *         nodes: [ { name: "label", type: "string" },
     *                  { name: "score", type: "number" } ],
     *                  
     *         edges: [ { name: "label", type: "string" },
     *                  { name: "weight", type: "number" },
     *                  { name: "directed", type: "boolean", defValue: true} ]
     *     },
     *     
     *     data: {
     *         nodes: [ { id: "n1", label: "Node 1", score: 1.0 },
     *                  { id: "n2", label: "Node 2", score: 2.2 },
     *                  { id: "n3", label: "Node 3", score: 3.5 } ],
     *                  
     *         edges: [ { id: "e1", label: "Edge 1", weight: 1.1, source: "n1", target: "n3" },
     *                  { id: "e2", label: "Edge 2", weight: 3.3, source:"n2", target:"n1"} ]
     *     }
     * };
     * @class
     * @name NetworkModel
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.Visualization#draw
     * @see org.cytoscapeweb.Visualization#networkModel
     * @see org.cytoscapeweb.Visualization#dataSchema
     * @see org.cytoscapeweb.DataSchema
     */
    
    // ===[ Node ]==================================================================================
     
    /**
     * <p>This object represents a Node type, but is actually just an untyped object.</p>
     * <p>So never do:</p>
     * <p><code>var node = new org.cytoscapeweb.Node(); // Wrong!!!</code></p>
     * <p>In order to create a node, just create an object with the expected fields.
     * Notice that the attribute <code>group</code> must always be <code>"nodes"</code>, 
     * because that is what really defines this type.</p>
     * @example
     * var node = {
     *     group: "nodes",
     *     shape: "TRIANGLE",
     *     size: 20,
     *     color: "0000ff",
     *     // etc...
     *     data: {
     *         id: 1
     *     }
     * };
     * @class
     * @name Node
     * @type Object
     * @memberOf org.cytoscapeweb
     */
    /**
     * The group name that defines this Data type (always <code>"nodes"</code>).
     * @property
     * @name group
     * @type org.cytoscapeweb.Group
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The object that stores the custom node attributes.
     * It should have at least the <code>id</code> property.
     * @property
     * @name data
     * @type Object
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
      * The shape name.
      * @property
      * @name shape
      * @type org.cytoscapeweb.NodeShape
      * @memberOf org.cytoscapeweb.Node#
      */  
    /**
     * The node fill color, in hexadecimal code (e.g. <code>"#ff3333"</code>).
     * @property
     * @name color
     * @type String
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The node opacity, from <code>0</code> to <code>1.0</code> (100% opaque).
     * @property
     * @name opacity
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The border color, in hexadecimal code (e.g. <code>"#000000"</code>).
     * @property
     * @name borderColor
     * @type String
     * @memberOf org.cytoscapeweb.Node#
     */    
    /**
     * The border width, in pixels.
     * @property
     * @name borderWidth
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */ 
    /**
     * The absolute node size (in pixels), when the zoom level is 100%. It is the highest value of height and width.
     * Notice that this value is not scaled, so if you want the real visualized size, you need to multiply
     * this value by the current network scale, which is provided by {@link org.cytoscapeweb.Visualization#zoom}.
     * If you set the size to <code>"auto"</code>, the node will be automatically resized to enclose its label text.
     * @property
     * @name size
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The absolute node width (in pixels), when the zoom level is 100%.
     * This value is not scaled, so if you want the real rendered width, you need to multiply
     * this value by the current network scale, which is provided by {@link org.cytoscapeweb.Visualization#zoom}.
     * @property
     * @name width
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The absolute node height (in pixels), when the zoom level is 100%.
     * This value is not scaled, so if you want the real rendered height, you need to multiply
     * this value by the current network scale, which is provided by {@link org.cytoscapeweb.Visualization#zoom}.
     * @property
     * @name height
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The number of child nodes.
     * @property
     * @name nodesCount
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * A boolean value that indicates whether or not the node is set to visible.
     * @property
     * @name visible
     * @type Boolean
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The x coordinate value that indicates where the center of the node is positioned in
     * the horizontal axis of the Visualization rectangle.
     * If <code>x == 0</code>, the middle point of the node is located exactly at the left border of the network view.
     * @property
     * @name x
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The y coordinate value that indicates where the center of the node is positioned in
     * the vertical axis of the Visualization rectangle.
     * If <code>y == 0</code>, the middle point of the node is located exactly at the top border of the network view.
     * @property
     * @name y
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    /**
     * The stacking order of the element.
     * @property
     * @name zIndex
     * @type Number
     * @memberOf org.cytoscapeweb.Node#
     */
    
     // ===[ Edge ]=================================================================================
    
    /**
     * <p>This object represents an Edge type, but is just an untyped object.</p>
     * <p>So never do:</p>
     * <p><code>var edge = new org.cytoscapeweb.Edge(); // Wrong!!!</code></p>
     * <p>In order to create an edge, just create an object with the expected fields.
     * Notice that the attribute <code>group</code> must always be <code>"edges"</code>, 
     * because that is what really defines this type.</p>
     * @example
     * var edge = {
     *     group: "edges",
     *     merged: false,
     *     opacity: 0.8,
     *     color: "333333",
     *     width: 2,
     *     // etc...
     *     data: {
     *         id: 1,
     *         source: 1,
     *         target: 3,
     *         weight: 0.5
     *     }
     * };
     * @class
     * @name Edge
     * @memberOf org.cytoscapeweb
     * @type Object
     */
    /**
     * The group name that defines this Data type (always <code>"edges"</code>).
     * @property
     * @name group
     * @type org.cytoscapeweb.Group
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
      * The object that stores the custom edge attributes.
      * It should have at least the following properties:
      * <ul class="options">
      *     <li><code>id</code> {String}: the edge id.</li>
      *     <li><code>source</code> {String}: the source node id.</li>
      *     <li><code>target</code> {String}: the target node id.</li>
      *     <li><code>directed</code> {Boolean}: a directed edge has a default arrow pointed to the target node.</li></ul>
      * When the network was created from a SIF data format, the edge's data object will also have the <code>interaction</code>
      * attribute (String type).
      * @property
      * @name data
      * @type Object
      * @memberOf org.cytoscapeweb.Edge#
      */
    /**
     * Indicate whether or not the edge is a merged one. Merged edges are used to simplify the 
     * network visualization by just showing that two nodes are connected to each other, without 
     * displaying all the real edges that link them together.
     * @property
     * @name merged
     * @type Boolean
     * @memberOf org.cytoscapeweb.Edge#
     * @see org.cytoscapeweb.Visualization#edgesMerged
     */
    /**
     * If the edge is a merged one, this property provides the regular parallel edges that were merged together.
     * If the edge is already a regular non-merged type, this property is undefined.
     * @property
     * @name edges
     * @type Array
     * @memberOf org.cytoscapeweb.Edge#
     * @see org.cytoscapeweb.Edge#merged
     */
    /**
     * The edge opacity, from <code>0</code> to <code>1.0</code> (100% opaque).
     * @property
     * @name opacity
     * @type Number
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The edge color, in hexadecimal code (e.g. <code>"#666666"</code>).
     * @property
     * @name color
     * @type String
     * @memberOf org.cytoscapeweb.Edge#
     */    
    /**
     * The edge line width, in pixels.
     * @property
     * @name width
     * @type Number
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The shape name of the edge's source arrow.
     * @default <code>"NONE"</code>, unless the current visual style sets a different value.
     * @property
     * @name sourceArrowShape
     * @type org.cytoscapeweb.ArrowShape
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The shape name of the edge's target arrow.
     * @default <ul><li><code>"NONE"</code>, if the edge is undirected</li>
     *              <li><code>"DELTA"</code>, if the edge is directed</li>
     * @property
     * @name targetArrowShape
     * @type org.cytoscapeweb.ArrowShape
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The color code of the source arrow.
     * @property
     * @name sourceArrowColor
     * @type String
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The color code of the target arrow.
     * @property
     * @name targetArrowColor
     * @type String
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The value that defines the curvature rate of curved edges. Higher values create more curved edges.
     * @default 18
     * @property
     * @name curvature
     * @type Number
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * A boolean value that indicates whether or not the edge is set to visible.
     * @property
     * @name visible
     * @type Boolean
     * @memberOf org.cytoscapeweb.Edge#
     */
    /**
     * The stacking order of the element.
     * @property
     * @name zIndex
     * @type Number
     * @memberOf org.cytoscapeweb.Edge#
     */
     
    // ===[ Layout ]================================================================================
    
    /**
     * <p>Layouts are just untyped objects.</p>
     * @example
     * var layout = {
     *     name:    "Radial",
     *     options: { angleWidth: 180, radius: 80 }
     * };
     * @class
     * @name Layout
     * @type String
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.Visualization#layout
     */
    /**
     * <p>The layout name. This field is mandatory and must be one of:</p>
     * <ul class="options"><li><code>ForceDirected</code>
     *     <li><code>Circle</code></li>
     *     <li><code>Radial</code></li>
     *     <li><code>Tree</code></li>
     *     <li><code>Preset</code></li>
     *     <li><code>CompoundSpringEmbedder</code>: use this option when the network is a compound graph</li></ul>
     * @property
     * @name name
     * @type String
     * @memberOf org.cytoscapeweb.Layout#
     */
    /**
     * <p>The available options for each layout type are:</p>
     * <ol class="options">
     * <li><b>ForceDirected:</b></li>
     *     <ul class="options">
     *         <li><code>mass</code> {Number}: The default mass value for nodes.</li>
     *         <li><code>gravitation</code> {Number}: The gravitational attraction (or repulsion, for
     *                                                negative values) between nodes.</li>
     *         <li><code>tension</code> {Number}: The default spring tension for edges.</li>
     *         <li><code>restLength</code> {Number}: The default spring rest length for edges.</li>
     *         <li><code>drag</code> {Number}: The co-efficient for frictional drag forces.</li>
     *         <li><code>iterations</code> {Number}: The number of iterations to run the simulation.</li>
     *         <li><code>maxTime</code> {Number}: The maximum time to run the simulation, in milliseconds.</li>
     *         <li><code>minDistance</code> {Number}: The minimum effective distance over which forces are exerted.
     *                                                Any lesser distances will be treated as the minimum.</li>
     *         <li><code>maxDistance</code> {Number}: The maximum distance over which forces are exerted. 
     *                                                Any greater distances will be ignored.</li>
     *         <li><code>seed</code> {Number}: Optional positive integer which is used to set the random seed for generating the initial node positions.
     *                                         Force-directed layouts are non-deterministic by nature, but this option can be used to reproduce the same topology.
     *                                         Just leave this property <code>undefined</code> or set <code>0</code> if you want to keep the layout non-deterministic (i.e. a random seed is used).</li>
     *         <li><code>autoStabilize</code> {Boolean}: A common problem with force-directed layouts is that they can be highly unstable.
     *                                                   If this parameter is <code>true</code> and the edges are being stretched too much
     *                                                   between each iteration, Cytoscape Web automatically tries to stabilize 
     *                                                   the network. The stabilization attempt is executed after the determined number
     *                                                   of <code>iterations</code>, until each edge length seems constant or until the 
     *                                                   <code>maxTime</code> is reached. Set <code>false</code> if you think the results
     *                                                   look worse than expected, or if the layout is taking too long to execute.</li>
     *         <li><code>weightAttr</code> {String}: The name of the edge attribute that contains the weights.
     *                                               The default value is <code>null</code>, which means that the layout is unweighted with respect to edges.
     *                                               If you want to generate an edge-weighted layout, you just need to provide the name of the data attribute that should be used as weight.</li>
     *         <li><code>weightNorm</code> {String}: The normalization method that is applied to the weight values when using a weighted layout (i.e. <code>weightAttr != null</code>).
     *                                               Possible values are: <code>"linear"</code>, <code>"invlinear"</code> and <code>"log"</code>.
     *                                               The default value is <code>"linear"</code>.</li>
     *         <li><code>minWeight</code> {Number}: The minimum edge weight to consider, if the layout is set to be weighted.
     *                                              Do not specify any value if you want the layout to get the minimum weight from the rendered edges data (filtered-out edges are ignored).
     *                                              Any edge with a weight bellow the minimum will be laid out the same as an edge with the minimum weight.</li>
     *         <li><code>maxWeight</code> {Number}: The maximum edge weight to consider, if the layout is set to be weighted.
     *                                              Do not specify any value if you want the layout to get the maximum weight from the rendered edges data (filtered-out edges are ignored).
     *                                              Any edge with a weight above the maximum will be laid out the same as an edge with maximum weight.</li>
     *     </ul>
     * <li><b>Circle:</b></li>
     *     <ul class="options">
     *         <li><code>angleWidth</code> {Number}: The angular width of the layout, in degrees.</li>
     *         <li><code>tree</code> {Boolean}: Flag indicating if any tree-structure in the data should be used to inform the layout. The default value is <code>false</code>.</li>
     *     </ul>
     * <li><b>Radial:</b></li>
     *     <ul class="options">
     *         <li><code>angleWidth</code> {Number}: The angular width of the layout, in degrees.</li>
     *         <li><code>radius</code> {Number}: The radius increment between depth levels.</li>
     *     </ul>
     * <li><b>Tree:</b></li>
     *     <ul class="options">
     *         <li><code>orientation</code> {String}: The orientation of the tree. One of: 
     *                                                <code>"leftToRight"</code>,
     *                                                <code>"rightToLeft"</code>,
     *                                                <code>"topToBottom"</code>,
     *                                                <code>"bottomToTop"</code>.</li>
     *         <li><code>depthSpace</code> {Number}: The space between depth levels in the tree.</li>
     *         <li><code>breadthSpace</code> {Number}: The space between siblings in the tree.</li>
     *         <li><code>subtreeSpace</code> {Number}: The space between different sub-trees.</li>
     *     </ul>
     * <li><b>CompoundSpringEmbedder:</b></li>
     *     <ul class="options">
     *         <li><code>layoutQuality</code> {String}: Can be one of the following: <code>"default"</code>, <code>"draft"</code>, <code>"proof"</code>. A better quality layout requires more iterations, taking longer.</li>
     *         <li><code>incremental</code> {Boolean}: If <code>true</code>, layout is applied incrementally by taking current positions of nodes into account.</li>
     *         <li><code>tension</code> {Number}: The default spring tension (spring constant) for edges.</li>
     *         <li><code>restLength</code> {Number}: The default spring rest length (desired length) for edges.</li>
     *         <li><code>smartRestLength</code> {Boolean}: Whether or not smart calculation of ideal rest length should be performed for inter-graph edges. When this is enabled, we calculate the number of nesting levels each edge spans and calculate the desired edge length accordingly.</li>
     *         <li><code>gravitation</code> {Number}: The gravitational attraction (or repulsion, for negative values) force between nodes.</li>
     *         <li><code>smartDistance</code> {Boolean}: If <code>true</code>, gravitational repulsion forces are calculated only when node pairs are in a certain range, resulting in faster layout at the relatively minimal cost of layout quality.</li>
     *         <li><code>centralGravitation</code> {Number}: All nodes are assumed to be pulled slightly towards the center of the network by a central gravitational force (gravitational constant) during layout. This is done to avoid arbitrary separation of disconnected parts of a network.</li>
     *         <li><code>centralGravityDistance</code> {Number}: The radius of the region in the center of the drawing, in which central gravitation is not exerted.</li>
     *         <li><code>compoundCentralGravitation</code> {Number}: The central gravitational constant for compound nodes. Contents of each compound node have a separate center of gravity.</li>
     *         <li><code>compoundCentralGravityDistance</code> {Number}: The radius of the region in the center of a compound node, over which central gravity force is exerted.</li>
     *         <li><code>multiLevelScaling</code> {Boolean}: If <code>true</code>, multi-level scaling algorithm is applied both to better capture the overall structure of the network and to save time on large networks.</li>
     *         <li><code>uniformLeafNodeSizes</code> {Boolean}: If <code>true</code>, leaf (non-compound or simple) node dimensions are assumed to be uniform, resulting in faster layout.</li>
     *     </ul>
     * <li><b>Preset:</b></li>
     *     <ul class="options">
     *         <li><code>fitToScreen</code> {Boolean}: If <code>true</code>, the network is centered, and can be zoomed out to fit the screen.</li>
     *         <li><code>points</code> {Array}: A list of plain objects containing the node <code>id</code> and the <code>x</code>/<code>y</code>
     *                                          coordinates. Example:<br>
     * <pre class="example ln-"><code class="js"
     * >var options = {
     *     fitToScreen: false,
     *     points: [ { id: "1", x:  10, y:  60 },
     *               { id: "2", x: -54, y:  32 },
     *               { id: "3", x: 120, y: -12 } ]
     * };</code></pre></li>
     *     </ul>
     * </ol>
     * @property
     * @name options
     * @type Object
     * @memberOf org.cytoscapeweb.Layout#
     */

    // ===[ VisualStyle ]===========================================================================
    
    /**
     * <p>This object represents a Visual Style type, but it is actually just an untyped object.</p>
     * <p>A visual style may have three attributes:</p>
     * <ul class="options">
     *     <li><code>global</code></li>
     *     <li><code>nodes</code></li>
     *     <li><code>edges</code></li></ul>
     * <p>Each one is an object that defines a set of visual properties.</p>
     * 
     * <p>For each visual property, you can specify a default value or define a dynamic visual mapping.
     * Cytoscape Web currently supports four different types of visual mappers:</p>
     * <ul class="options">
     *     <li><code>continuousMapper</code></li>
     *     <li><code>discreteMapper</code></li>
     *     <li><code>passthroughMapper</code></li>
     *     <li><code>customMapper</code></li></ul> 
     * 
     * <p>In order to create a visual style, just create an object with the expected fields.</p>
     * <p>Never do:</p>
     * <p><code>var style = new org.cytoscapeweb.VisualStyle(); // Wrong!!!</code></p>
     * @example
     * var style = {
     *         global: {
     *             backgroundColor: "#ffffff",
     *             tooltipDelay: 1000
     *         },
     *         nodes: {
     *             shape: "ELLIPSE",
     *             color: "#333333",
     *             opacity: 1,
     *             size: { defaultValue: 12, 
     *                     continuousMapper: { attrName: "weight", 
     *                                         minValue: 12, 
     *                                         maxValue: 36 } },
     *             borderColor: "#000000",
     *             tooltipText: "&lt;b&gt;&#36{label}&lt;/b&gt;: &#36{weight}"
     *         },
     *         edges: {
     *             color: "#999999",
     *             width: 2,
     *             mergeWidth: 2,
     *             opacity: 1,
     *             label: { passthroughMapper: { attrName: "id" } },
     *             labelFontSize: 10,
     *             labelFontWeight: "bold"
     *          }
     * };
     * @class
     * @name VisualStyle
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.ContinuousMapper
     * @see org.cytoscapeweb.DiscreteMapper
     * @see org.cytoscapeweb.PassthroughMapper
     * @see org.cytoscapeweb.CustomMapper
     * @see org.cytoscapeweb.Visualization#visualStyle
     * @see org.cytoscapeweb.VisualStyleBypass
     */
    /**
     * <p>An object that defines global visual properties.</p>
     * <p>Remember that global properties do not accept visual mappers, because they cannot be associated with nodes/edges data attributes.
     * If you try to set a mapper to a global property, the mapper is simply ignored.</p>
     * <p>The possible global properties are:</p>
     * <ul class="options"><li><code>backgroundColor</code> {String}: Background color of the network view (hexadecimal code).
     *                                                                 The default value is "#ffffff".</li>
     *     <li><code>tooltipDelay</code>  {Number}: Number of milliseconds to delay before displaying the tooltip, when the cursor is over a node or edge.
     *                                                               The default value is 800 milliseconds.</li>
     *     <li><code>selectionFillColor</code> {String}: Fill color of the drag-selection rectangle. The default value is "#8888ff".</li>
     *     <li><code>selectionLineColor</code> {String}: Line color of the drag-selection border. The default value is "#8888ff".</li>
     *     <li><code>selectionFillOpacity</code> {Number}: Fill opacity of the drag-selection rectangle (0 to 1). The default value is 0.1.</li>
     *     <li><code>selectionLineOpacity</code> {Number}: Line opacity of the drag-selection border (0 to 1). The default value is 0.8.</li>
     *     <li><code>selectionLineWidth</code> {Number}: Line width of the drag-selection border. The default value is 1.</li></ul>
     * @property
     * @name global
     * @type Object
     * @memberOf org.cytoscapeweb.VisualStyle#
     */
    /**
     * <p>An object that defines visual styles for nodes.</p>
     * <p>The possible node properties are as follows.  Each property can be prefixed by "compound"
     * and the first letter of the term is correspondingly capitalized (e.g. "color" becomes "compoundColor") for compound nodes.</p>
     * <ul class="options">
     *     <li><code>shape</code> {{@link org.cytoscapeweb.NodeShape}}: Node shape name. The default value for regular nodes is <code>"ELLIPSE"</code>.
     *         The <code>compoundShape</code> property accepts only <code>"RECTANGLE"</code> (default), <code>"ROUNDRECT"</code> or <code>"ELLIPSE"</code>.</li>
     *     <li><code>size</code> {Number}: Node size, in pixels.
     *                                     It has the same effect of setting the same value (or mapper) to both <code>width</code> and <code>height</code>.
     *                                     The default value is 24.
     *                                     You can also set <code>"auto"</code> if you want to let Cytoscape Web calculate the size of the node automatically so it encloses its label.</li>
     *     <li><code>compoundSize</code> {String}: The size of compound nodes is always automatically calculated, so the actual size cannot be set.
     *                                             This property accepts two values:
     *                                             <ul><li><code>auto</code> (default): compound nodes are sized to fit their child labels inside.</li>
     *                                                 <li><code>ignoreLabels</code>: child labels are ignored when calculating the compound node size.</li>
     *                                             </ul></li>
     *     <li><code>width</code> {Number}: Node width, in pixels. It is not set by default.
     *                                      If set, it overrides the value returned by <code>size</code>.
     *                                      Because the size of compound nodes is automatically calculated, setting <code>compoundWidth</code> has no effect.</li>
     *     <li><code>height</code> {Number}: Node height, in pixels. It is not set by default.
     *                                       If set, it overrides the value returned by <code>size</code>.
     *                                       Because the size of compound nodes is automatically calculated, setting <code>compoundHeight</code> has no effect.</li>
     *     <li><code>color</code> {String}: Fill color code of nodes. The default value is "#f5f5f5".</li>
     *     <li><code>image</code> {String}: The URL of the image to be used as the node background. No image is used by default.
     *                                      If you specify a cross-domain address, then the image might not be loaded by Flash, unless
     *                                      the host provides a cross-domain XML file that allows you to do so.  We recommend you use
     *                                      a server-side proxy on your web host machine if you have this issue. See an
     *                                      <a href="http://www.abdulqabiz.com/blog/archives/2007/05/31/php-proxy-script-for-cross-domain-requests/" rel="external">example in PHP</a>
     *                                      to understand the process of writing your own proxy.</li>
     *     <li><code>borderColor</code> {String}: Border color of nodes. The default value is "#666666".</li>
     *     <li><code>borderWidth</code> {Number}: Border width of nodes. The default value is 1.</li>
     *     <li><code>opacity</code> {Number}: The node opacity (0 to 1). The default value is 0.8.</li>
     *     <li><code>selectionColor</code> {String}: The fill color of selected nodes.
     *                                                                The default value is the same one set to <code>color</code>.</li>
     *     <li><code>selectionBorderColor</code> {String}: The border color of selected nodes.
     *                                                                      The default value is the same one set to <code>borderColor</code>.</li>
     *     <li><code>selectionOpacity</code> {Number}: The opacity of selected nodes (0 to 1).
     *                                                                  The default value is the same one set to <code>opacity</code>.</li>
     *     <li><code>selectionBorderWidth</code> {Number}: The border width of selected nodes (0 to 1).
     *                                                                      The default value is the same one set to <code>borderWidth</code>.</li>
     *     <li><code>selectionGlowColor</code> {String}: The glow color of selected nodes.The default value is "#ffff33".</li>
     *     <li><code>selectionGlowOpacity</code> {Number}: The glow transparency of selected nodes. Valid values are 0 to 1.
     *                                                                      The default value is 0.6 (60% opaque).</li>
     *     <li><code>selectionGlowBlur</code> {Number}: The amount of blur for the selection glow. Valid values are 0 to 255 (floating point).
     *                                                                   The default value is 8. Values that are a power of 2 (such as 2, 4, 8, 16, and 32) 
     *                                                                   are optimized to render more quickly.</li>
     *     <li><code>selectionGlowStrength</code> {Number}: The strength of the glow color imprint or spread when the node is selected.
     *                                                                       The higher the value, the more color is imprinted and the stronger the contrast
     *                                                                       between the glow and the background.
     *                                                                       Valid values are 0 to 255. The default is 6.</li>
     *     <li><code>hoverOpacity</code> {Number}: The opacity of the node when the mouse is over it (0 to 1).
     *                                                              The default value is the same one set to <code>opacity</code>.</li>
     *     <li><code>hoverBorderColor</code> {String}: The border color when the mouse is over a node.
     *                                                                  The default value is the same one set to <code>borderColor</code>.</li>
     *     <li><code>hoverBorderWidth</code> {Number}: The node border width on mouse over.
     *                                                                  The default value is the same one set to <code>borderWidth</code>.</li>
     *     <li><code>hoverGlowColor</code> {String}: The node glow color on mouse over.
     *                                                                The default value is "#aae6ff".</li>
     *     <li><code>hoverGlowOpacity</code> {Number}: The node glow opacity on mouse over (0 to 1).
     *                                                                  The default value is 0, which means that there is no visible glow on mouse over.</li>
     *     <li><code>hoverGlowBlur</code> {Number}: The amount of blur for the mouse over glow. Valid values are 0 to 255 (floating point).
     *                                                               The default value is 8. Values that are a power of 2 (such as 2, 4, 8, 16, and 32) 
     *                                                               are optimized to render more quickly.</li>
     *     <li><code>hoverGlowStrength</code> {Number}: The strength of the glow color imprint or spread on mouse over.
     *                                                                   The higher the value, the more color is imprinted and the stronger the contrast
     *                                                                   between the glow and the background.
     *                                                                   Valid values are 0 to 255. The default is 6.</li>
     *     <li><code>label</code> {String}: The text to be displayed as node label. A Passthrough Mapper is created by default, 
     *                                                       and it displays the node <code>data.label</code> attribute value.</li>
     *     <li><code>labelFontName</code> {String}: Font name of node labels. The default is "Arial".</li>
     *     <li><code>labelFontSize</code> {Number}: The point size of node labels. The default size is 11.</li>
     *     <li><code>labelFontColor</code> {String}: Font color of node labels. The default value "#000000".</li>
     *     <li><code>labelFontWeight</code> {String}: <code>normal</code> or <code>bold</code>. The default is "normal".</li>
     *     <li><code>labelFontStyle</code> {String}: <code>normal</code> or <code>italic</code>. The default is "normal".</li>
     *     <li><code>labelHorizontalAnchor</code> {String}: The horizontal label anchor: 
     *                                                                       <code>left</code>, <code>center</code> or <code>right</code></li>
     *     <li><code>labelVerticalAnchor</code> {String}: The vertical label anchor: 
     *                                                                     <code>top</code>, <code>middle</code> or <code>bottom</code></li>
     *     <li><code>labelXOffset</code> {Number}: Horizontal distance of the label from the node border. 
     *                                                              If <code>labelHorizontalAnchor</code> is "right",
     *                                                              the distance is measured from the left side of the node, and
     *                                                              a negative offset displaces the label towards left.</li>
     *     <li><code>labelYOffset</code> {Number}: Vertical distance of the label from the node border. 
     *                                                              If <code>labelVerticalAnchor</code> is "bottom", 
     *                                                              the distance is measured from the top side of the node, and
     *                                                              a negative offset moves the label upper.</li>
     *     <li><code>labelGlowColor</code> {String}: The color of the label glow. The default value is "#ffffff".</li>
     *     <li><code>labelGlowOpacity</code> {Number}: The alpha transparency of the label glow. Valid values are 0 to 1.
     *                                                                  The default value is 0 (totally transparent).</li>
     *     <li><code>labelGlowBlur</code> {Number}: The amount of blur for the label glow. Valid values are 0 to 255 (floating point).
     *                                                               The default value is 8. Values that are a power of 2 (such as 2, 4, 8, 16, and 32) 
     *                                                               are optimized to render more quickly.</li>
     *     <li><code>labelGlowStrength</code> {Number}: The strength of the imprint or spread. The higher the value, the more color 
     *                                                                   is imprinted and the stronger the contrast between the glow and the background.
     *                                                                   Valid values are 0 to 255. The default is 20.</li>
     *     <li><code>tooltipText</code> {String}: Static text or a text formatter for node tool tips. 
     *                                                             A list with all the node <code>data</code> attributes is displayed by default.</li>
     *     <li><code>tooltipFont</code> {String}: Font name of node tool tips. The default font is "Arial".</li>
     *     <li><code>tooltipFontSize</code> {Number}: The point size of node tool tips. The default value is 11.</li>
     *     <li><code>tooltipFontColor</code> {String}: Font color of node tool tips. The default value is "#000000".</li>
     *     <li><code>tooltipBackgroundColor</code> {String}: Background color of node tool tips. The default value is "#f5f5cc".</li>
     *     <li><code>tooltipBorderColor</code> {String}: Border color of node tool tips. The default value is "#000000".</li></ul>
     * @property
     * @name nodes
     * @type Object
     * @memberOf org.cytoscapeweb.VisualStyle#
     */    
    /**
     * <p>An object that defines visual styles for edges.</p>
     * <p>The possible edge properties are:</p>
     * <ul class="options"><li><code>color</code> {String}: Color of edges. The default value is "#999999".</li>
     *     <li><code>width</code> {Number}: Line width of edges. The default value is 1 pixel.</li>
     *     <li><code>opacity</code> {Number}: The edge opacity (0 to 1). The default value is 0.8.</li>
     *     <li><code>style</code> {String}: The edge line style.
     *                                      One of: <code>"SOLID"</code>, <code>"DOT"</code>, <code>"LONG_DASH"</code>, <code>"EQUAL_DASH"</code>.
     *                                      The default value is <code>"SOLID"</code>.</li>
     *     <li><code>mergeStyle</code> {String}: The line style for merged edges.
     *                                           One of: <code>"SOLID"</code>, <code>"DOT"</code>, <code>"LONG_DASH"</code>, <code>"EQUAL_DASH"</code>.
     *                                           The default value is <code>"SOLID"</code>.</li>
     *     <li><code>mergeColor</code> {String}: Line color for merged edges. The default value is "#666666".</li>
     *     <li><code>mergeWidth</code> {Number}: Line width for merged edges. The default value is 1 pixel.</li>
     *     <li><code>mergeOpacity</code> {Number}: Opacity of merged edges (0 to 1). The default value is 0.8.</li>
     *     <li><code>selectionColor</code> {String}: The fill color of selected edges.
     *                                                                The default value is the same one set to <code>color</code>
     *                                                                (or <code>mergeColor</code>, when edges are merged).</li>
     *     <li><code>selectionOpacity</code> {Number}: The opacity of selected edges (0 to 1).
     *                                                                  The default value is the same one set to <code>opacity</code>.</li>
     *     <li><code>selectionGlowColor</code> {String}: The glow color of selected edges.The default value is "#ffff33".</li>
     *     <li><code>selectionGlowOpacity</code> {Number}: The glow transparency of selected edges. Valid values are 0 to 1.
     *                                                                      The default value is 0.6 (60% opaque).</li>
     *     <li><code>selectionGlowBlur</code> {Number}: The amount of blur for the selection glow. Valid values are 0 to 255 (floating point).
     *                                                                   The default value is 4. Values that are a power of 2 (such as 2, 4, 8, 16, and 32) 
     *                                                                   are optimized to render more quickly.</li>
     *     <li><code>selectionGlowStrength</code> {Number}: The strength of the glow color imprint or spread when the edge is selected.
     *                                                                       The higher the value, the more color is imprinted and the stronger the contrast
     *                                                                       between the glow and the background.
     *                                                                       Valid values are 0 to 255. The default is 10.</li>
     *     <li><code>hoverOpacity</code> {Number}: The opacity of the edge when the mouse is over it (0 to 1).
     *                                                              The default value is the same one set to <code>opacity</code>.</li>
     *     <li><code>curvature</code> {Number}: The curvature amount of curved edges. The default value is 18.</li>
     *     <li><code>sourceArrowShape</code> {{@link org.cytoscapeweb.ArrowShape}}: Shape name of source arrows. The default value is "NONE".</li>
     *     <li><code>targetArrowShape</code> {{@link org.cytoscapeweb.ArrowShape}}: Shape name of target arrows.
     *                                                                                                For directed edges, the default value is "DELTA".
     *                                                                                                For undirected ones, the default value is "NONE".</li>
     *     <li><code>sourceArrowColor</code> {String}: Color code of source arrows.
     *                                                                  The default value is the same one set to the edge <code>color</code> property.</li>
     *     <li><code>targetArrowColor</code> {String}: Color code of target arrows.
     *                                                                  The default value is the same one set to the edge <code>color</code> property.</li>
     *     <li><code>label</code> {String}: The text to be displayed as edge label. There is no default value or mapper for edge labels.</li>
     *     <li><code>labelFontName</code> {String}: Font name of edge labels. The default is "Arial".</li>
     *     <li><code>labelFontSize</code> {Number}: The point size of edge labels. The default size is 11.</li>
     *     <li><code>labelFontColor</code> {String}: Font color of edge labels. The default value "#000000".</li>
     *     <li><code>labelFontWeight</code> {String}: <code>normal</code> or <code>bold</code>. The default is "normal".</li>
     *     <li><code>labelFontStyle</code> {String}: <code>normal</code> or <code>italic</code>. The default is "normal".</li>
     *     <li><code>labelGlowColor</code> {String}: The color of the label glow. The default value is "#ffffff".</li>
     *     <li><code>labelGlowOpacity</code> {Number}: The alpha transparency of the label glow. Valid values are 0 to 1.
     *                                                                  The default value is 0 (totally transparent).</li>
     *     <li><code>labelGlowBlur</code> {Number}: The amount of blur for the label glow. Valid values are 0 to 255 (floating point).
     *                                                               The default value is 2. Values that are a power of 2 (such as 2, 4, 8, 16, and 32) 
     *                                                               are optimized to render more quickly.</li>
     *     <li><code>labelGlowStrength</code> {Number}: The strength of the imprint or spread. The higher the value, the more color 
     *                                                                   is imprinted and the stronger the contrast between the glow and the background.
     *                                                                   Valid values are 0 to 255. The default is 20.</li>
     *     <li><code>tooltipText</code> {String}: Static text or a text formatter for regular edge tool tips. 
     *                                                             A list with all the edge <code>data</code> attributes is displayed by default.</li>
     *     <li><code>mergeTooltipText</code> {String}: Static text or a text formatter for merged edge tool tips.
     *                                                                  A list with all the merged edge <code>data</code> attributes is displayed by default.</li>
     *     <li><code>tooltipFont</code> {String}: Font name of edge tool tips. The default font is "Arial".</li>
     *     <li><code>tooltipFontSize</code> {Number}: The point size of edge tool tips. The default value is 11.</li>
     *     <li><code>tooltipFontColor</code> {String}: Font color of edge tool tips. The default value is "#000000".</li>
     *     <li><code>tooltipBackgroundColor</code> {String}: Background color of edge tool tips. The default value is "#f5f5cc".</li>
     *     <li><code>tooltipBorderColor</code> {String}: Border color of edge tool tips. The default value is "#000000".</li></ul>
     * @property
     * @name edges
     * @type Object
     * @memberOf org.cytoscapeweb.VisualStyle#
     */

    // ===[ VisualStyleBypass ]===========================================================================
    
    /**
     * <p>This object represents a Visual Style Bypass type, but it is actually just an untyped object.</p>
     * <p>A visual style bypass may have two attributes:</p>
     * <ul class="options">
     *     <li><code>nodes</code></li>
     *     <li><code>edges</code></li></ul>
     * <p>Each one is an object that redefines a set of visual properties. They are dictionaries
     * that have edges and nodes <code>id</code> values as keys, and objects that contain the visual styles as values.</p>
     * <p>Notice that you cannot bypass <code>global</code> properties, and it is not possible to set visual mappings either.</p>
     * <p>You can bypass any of the nodes or edges visual properties. Just use the same names listed at 
     * {@link org.cytoscapeweb.VisualStyle}.</p>
     * @example
     * var bypass = {
     *         nodes: {
     *             "1": { color: "#ff0000", opacity: 0.5, size: 32 },
     *             "3": { color: "#ffff00", opacity: 0.9 },
     *             "7": { color: "#ffff00", opacity: 0.2 }
     *         },
     *         edges: {
     *             "22": { width: 4, opacity: 0.2 },
     *             "23": { width: 4, opacity: 0.2 }
     *          }
     * };
     * @class
     * @name VisualStyleBypass
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.VisualStyle
     * @see org.cytoscapeweb.Visualization#visualStyleBypass
     */
    
    // ===[ Mappers ]===============================================================================
    
    /**
     * <p>This object represents a Continuous Mapper type, although it is just an untyped object.</p>
     * <p>Depending on the visual attribute, there are two kinds of continuous mappers:</p>
     * <ol><li><strong>Continuous-to-Continuous Mapper:</strong> for example, you can map a continuous numerical value to a node size.</li>
     *     <li><strong>Color Gradient Mapper:</strong> This is a special case of continuous-to-continuous mapping. 
     *         Continuous numerical values are mapped to a color gradient.</li></ol>
     * <p>Notice that:
     * <ul>
     *     <li><strong>Continuous-to-Discrete</strong> mappers are not supported yet (e.g. all values below 0 are mapped to square nodes, 
     * and all values above 0 are mapped to circular nodes).</li>
     *     <li>Only numerical attributes and colors can be mapped with continuous mappers. For example,
     * there is no way to smoothly morph between circular nodes and square nodes.</il>
     *     <li>The mapping algorithm uses a linear interpolation to calculate the values.</li>
     *     <li>Continuous mappers ignore filtered out elements.</li>
     * </ul>
     * 
     * @example
     * // A mapper that could be used to set the sizes of the nodes between 12 and 36 pixels:
     * var sizeMapper = { attrName: "weight",  minValue: 12, maxValue: 36 };
     * 
     * // This one could be used to create a color range from yellow to green:
     * var colorMapper = { attrName: "score",  minValue: "#ffff00", maxValue: "#00ff00" };
     * 
     * // This edge width mapper specifies the minimum and maximum data values for the scale.
     * // Weights lower than 0.1 are given a width of 1, and weights higher than 1.0 are given a width of 4.
     * var widthMapper = { attrName: "weight",  minValue: 1, maxValue: 4, minAttrValue: 0.1, maxAttrValue: 1.0 };
     * @class
     * @name ContinuousMapper
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.DiscreteMapper
     * @see org.cytoscapeweb.PassthroughMapper
     * @see org.cytoscapeweb.CustomMapper
     * @see org.cytoscapeweb.VisualStyle
     */
    /**
     * The name of the data attribute that will be mapped to a visual style's property.
     * @property
     * @name attrName
     * @type String
     * @memberOf org.cytoscapeweb.ContinuousMapper#
     */
    /**
     * The minimum value of the visual style's property. It is usually a number (e.g. edge width),
     * but accepts strings if the visual property is a color.
     * @property
     * @name minValue
     * @memberOf org.cytoscapeweb.ContinuousMapper#
     */
    /**
     * The maximum value of the visual style's property. It is usually a number (e.g. edge width),
     * but accepts strings if the visual property is a color.
     * @property
     * @name maxValue
     * @memberOf org.cytoscapeweb.ContinuousMapper#
     */
    /**
     * An optional minimum value for the linear scale. If you don't specify it,
     * Cytoscape Web gets the lowest attribute value from the rendered nodes or edges (filtered-out elements are ignored).
     * And if an element's data value is lower than the specified minimum, that element's visual property is simply scaled up to the minimum value.
     * @property
     * @name minAttrValue
     * @type Number
     * @memberOf org.cytoscapeweb.ContinuousMapper#
     */
    /**
     * An optional maximum value for the linear scale. If you don't specify it,
     * Cytoscape Web gets the highest attribute value from the rendered nodes or edges (filtered-out elements are ignored).
     * And if an element's data value is higher than the specified maximum, that element's visual property is simply scaled down to the maximum value.
     * @property
     * @name maxAttrValue
     * @type Number
     * @memberOf org.cytoscapeweb.ContinuousMapper#
     */
    
    /**
     * <p>This object represents a Discrete Mapper type, but is just an untyped object.</p>
     * <p>Discrete network attributes are mapped to discrete visual attributes.</p>
     * <p>For example, a discrete mapper can map node colors to gene annotations.</p>
     * @example
     * // Create the mapper:
     * var colorMapper = {
     *         attrName: "molecular_function",
     *         entries: [ { attrValue: "catalytic", value: "#ff0000" },
     *                    { attrValue: "transporter", value: "#00ff00" },
     *                    { attrValue: "binding", value: "#0000ff" } ]
     * };
     * 
     * // Set the mapper to a Visual Style;
     * var style = {
     *         nodes: {
     *             color: { discreteMapper: colorMapper }
     *         }
     * };
     * 
     * // Set the new style to the Visualization:
     * vis.visualStyle(style);
     * 
     * // Now, if ( node.data["molecular_function"] == "binding" ),
     * // then the node will be blue
     * 
     * @class
     * @name DiscreteMapper
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.ContinuousMapper
     * @see org.cytoscapeweb.PassthroughMapper
     * @see org.cytoscapeweb.CustomMapper
     * @see org.cytoscapeweb.VisualStyle
     */
    /**
     * The name of the data attribute that will be mapped to a visual style's property.
     * @property
     * @name attrName
     * @type String
     * @memberOf org.cytoscapeweb.DiscreteMapper#
     */
    /**
     * An array of objects used to map data attributes to visual style values.
     * Each entry object must define:
     * <ul class="options"><li><code>attrValue</code>: The edge or node data attribute value.</li>
     *     <li><code>value</code>: The visual style value (e.g. a color code).</li></ul>
     * @property
     * @name entries
     * @type Array
     * @memberOf org.cytoscapeweb.DiscreteMapper#
     */
    
    /**
     * <p>This is an untyped object that represents a Passthrough Mapper type.</p>
     * <p>The values of network attributes are passed directly through to visual attributes.</p>
     * <p>The most common use case is using this mapper to specify node/edge labels.
     * For example, a passthrough mapper can label all nodes with their gene symbols.</p>
     * <p>When defining a passthrough mapper, you just need to specify the name of the node or edge
     * data attribute that contains the visual style values.</p>
     * @example
     * // Create the mapper and set it to a Visual Style's nodes.label property;
     * var style = {
     *         nodes: {
     *             label: { passthroughMapper: { attrName: "symbol" } }
     *         }
     * };
     * 
     * // Set the new style to the Visualization:
     * vis.visualStyle(style);
     * 
     * @class
     * @name PassthroughMapper
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.ContinuousMapper
     * @see org.cytoscapeweb.DiscreteMapper
     * @see org.cytoscapeweb.CustomMapper
     * @see org.cytoscapeweb.VisualStyle
     */
    
    /**
     * <p>This is a special type of mapper that allows you to register a callback function
     * that will be called for each associated element (nodes or edges). 
     * The function will then be responsible for returning the desired property value.</p>
     * <p>The callback function should expect a <code>data</code> object as argument.</p>
     * <p>You could, for example, use a custom mapper to create a better tooltip text.</p>
     * 
     * @example
     * // 1. First, create a function and add it to the Visualization object.
     * vis["customTooltip"] = function (data) {
     *     var value = Math.round(100 * data["weight"]) + "%";
     *     return 'The confidence level of this link is: ' +
     *            '&lt;font color="#000099" face="Courier" size="14"&gt;' + value + '&lt;/font&gt;';
     * };
     * 
     * // 2. Now create a new visual style (or get the current one) and register
     * //    the custom mapper to one or more visual properties:
     * var style = vis.visualStyle();
     * style.edges.tooltipText = { customMapper: { functionName: "customTooltip" } },
     * 
     * // 3. Finally set the visual style again:
     * vis.visualStyle(style);
     * 
     * @class
     * @name CustomMapper
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.ContinuousMapper
     * @see org.cytoscapeweb.DiscreteMapper
     * @see org.cytoscapeweb.PassthroughMapper
     * @see org.cytoscapeweb.VisualStyle
     */
    /**
     * The name of the JavaScript function that will return the visual style value for each node or edge.
     * The callback function always receives the node's or edge's <code>data</code> object as argument.
     * @property
     * @name functionName
     * @type String
     * @memberOf org.cytoscapeweb.CustomMapper#
     */

     // ===[ Error ]================================================================================
    
    /**
     * <p>This object represents an Error type, but is just an untyped object.</p>
     * <p>It is returned by <code>"error"</code> type events.</p>
     * @class
     * @name Error
     * @memberOf org.cytoscapeweb
     * @type Object
     * @see org.cytoscapeweb.EventType
     * @see org.cytoscapeweb.Event
     * @see org.cytoscapeweb.Visualization#addListener
     */
    /**
     * The error message.
     * @property
     * @name msg
     * @type String
     * @memberOf org.cytoscapeweb.Error#
     */
    /**
     * The error id.
     * @property
     * @name id
     * @type String
     * @memberOf org.cytoscapeweb.Error#
     */
    /**
     * The error name.
     * @property
     * @name name
     * @type String
     * @memberOf org.cytoscapeweb.Error#
     */
    /**
     * The stack trace of the error.
     * @property
     * @name stackTrace
     * @type String
     * @memberOf org.cytoscapeweb.Error#
     */

    // ===[ Data Schema ]==================================================================================
    
    /**
     * <p>This is an untyped object that represents a Data Schema type.</p>
     * <p>A data schema is automatically created when a network is loaded into Cytoscape Web,
     *    and cannot be created programatically through the API.
     *    However you can use the {@link org.cytoscapeweb.Visualization#addDataField} and
     *    {@link org.cytoscapeweb.Visualization#removeDataField} methods to change the current schema.</p>
     * <p>A data schema has two attributes:</p>
     * <ul class="options">
     *     <li><code>nodes</code> {Array}</li>
     *     <li><code>edges</code> {Array}</li></ul>
     * <p>Those are arrays of {@link org.cytoscapeweb.DataField} objects:</p>
     * 
     * @example
     * var schema = {
     *     nodes: [
     *         { name: "id",    type: "string" },
     *         { name: "label", type: "string" }
     *     ],
     *     edges: [
     *         { name: "id",     type: "string" },
     *         { name: "weight", type: "number", defValue: 0.5 }
     *     ]
     * };
     * @class
     * @name DataSchema
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.Visualization#dataSchema
     * @see org.cytoscapeweb.DataField
     */
    
    /**
     * <p>This untyped object represents a Data Field, which is a node or edge attribute definition.</p>
     * <p>A data field object contains the following properties:</p>
     * <ul class="options">
	 * 	   <li><code>name</code>: The name of the data attribute.</li>
	 *     <li><code>type</code>: The data type of the attribute. One of:
	 *         <code>"string"</code>, <code>"boolean"</code>, <code>"number"</code>, <code>"int"</code>, <code>"object"</code>.</li>
	 *     <li><code>defValue</code>: An optional default value.</li>
	 * </ul>
     * @class
     * @name DataField
     * @type Object
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.Visualization#addDataField
     * @see org.cytoscapeweb.Visualization#removeDataField
     * @see org.cytoscapeweb.Visualization#dataSchema
     * @see org.cytoscapeweb.DataSchema
     */
    
    // ===[ Fake Enum Types ]=======================================================================
    
    /**
     * <p>This object represents a Group type. In actuality, it is a string.</p>
     * <p>However, its value must be one of:</p>
     * <ul class="options"><li><code>nodes</code></li><li><code>edges</code></li><li><code>none</code> (same as <code>null</code>)</li></ul>
     * @class
     * @name Group
     * @type String
     * @memberOf org.cytoscapeweb
     */
    /**
     * <p>This object represents an event type. In actuality, it is a string.</p>
     * <p>All of them, but <code>"contextmenu"</code> can be used with the listener methods 
     * ({@link org.cytoscapeweb.Visualization#addListener}, {@link org.cytoscapeweb.Visualization#hasListener} and
     * {@link org.cytoscapeweb.Visualization#removeListener}).</p>
     * <p>Its value must be one of:</p>
     *     <ul class="options">
     *         <li><code>click</code>:</strong> For mouse click events on nodes, edges or the visualization background.</li>
     *         <li><code>dblclick</code>:</strong> For double-click events on nodes, edges or the visualization background.</li>
     *         <li><code>dragstart</code>:</strong> For drag-start events on nodes.</li>
     *         <li><code>dragstop</code>:</strong> For drag-stop events on nodes.</li>
     *         <li><code>mouseover</code>:</strong> For mouse-over events on nodes, edges or the visualization background.</li>
     *         <li><code>mouseout</code>:</strong> For mouse-out events on nodes, edges or the visualization background.</li>
     *         <li><code>select</code>:</strong> For events dispatched after nodes or edges are selected (e.g. by direct mouse clicking or by drag-selecting).</li>
     *         <li><code>deselect</code>:</strong> For events dispatched after nodes or edges are unselected.</li>
     *         <li><code>filter</code>:</strong> For events dispatched after nodes or edges are filtered.</li>
     *         <li><code>zoom</code>:</strong> For events dispatched after the network is rescaled.</li>
     *         <li><code>layout</code>:</strong> For events dispatched after a new layout is applied or the current one is recomputed.</li>
     *         <li><code>contextmenu</code>:</strong> For events dispatched after a right-click context menu item is selected.
     *                                                        You cannot use this type with the listener methods (e.g. {@link org.cytoscapeweb.Visualization#addListener}).
     *                                                        Events of this type are only dispatched to the callback functions that are registered with
     *                                                        {@link org.cytoscapeweb.Visualization#addContextMenuItem}.</li>
     *         <li><code>error</code>:</strong> For events dispatched when an internal error or exception occurs.</li></ul>
     * @class
     * @name EventType
     * @type String
     * @memberOf org.cytoscapeweb
     * @see org.cytoscapeweb.Visualization#addListener
     * @see org.cytoscapeweb.Visualization#hasListener
     * @see org.cytoscapeweb.Visualization#removeListener
     * @see org.cytoscapeweb.Visualization#addContextMenuItem
     */
    /**
     * <p>This object represents node shapes. In actuality, it is just a string.</p>
     * <p>Possible values:</p>
     * <ul class="options">
     *    <li><code>ELLIPSE</code></li>
     *    <li><code>RECTANGLE</code></li>
     *    <li><code>TRIANGLE</code></li>
     *    <li><code>DIAMOND</code></li>
     *    <li><code>HEXAGON</code></li>
     *    <li><code>OCTAGON</code></li>
     *    <li><code>PARALLELOGRAM</code></li>
     *    <li><code>ROUNDRECT</code></li>
     *    <li><code>VEE</code></li></ul>
     * @class
     * @name NodeShape
     * @type String
     * @memberOf org.cytoscapeweb
     */
    /**
     * <p>This object represents edge arrow shapes. In actuality, it is just a string.</p>
     * <p>Its value must be one of:</p>
     * <ul class="options">
     *      <li><code>NONE</code></li>
     *      <li><code>DELTA</code></li>
     *      <li><code>ARROW</code></li>
     *      <li><code>DIAMOND</code></li>
     *      <li><code>CIRCLE</code></li>
     *      <li><code>T</code></li></ul>
     * @class
     * @name ArrowShape
     * @type String
     * @memberOf org.cytoscapeweb
     */
})();
if(!this.JSON){this.JSON={}}(function(){function f(n){return n<10?"0"+n:n}if(typeof Date.prototype.toJSON!=="function"){Date.prototype.toJSON=function(key){return isFinite(this.valueOf())?this.getUTCFullYear()+"-"+f(this.getUTCMonth()+1)+"-"+f(this.getUTCDate())+"T"+f(this.getUTCHours())+":"+f(this.getUTCMinutes())+":"+f(this.getUTCSeconds())+"Z":null};String.prototype.toJSON=Number.prototype.toJSON=Boolean.prototype.toJSON=function(key){return this.valueOf()}}var cx=/[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,escapable=/[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,gap,indent,meta={"\b":"\\b","\t":"\\t","\n":"\\n","\f":"\\f","\r":"\\r",'"':'\\"',"\\":"\\\\"},rep;function quote(string){escapable.lastIndex=0;return escapable.test(string)?'"'+string.replace(escapable,function(a){var c=meta[a];return typeof c==="string"?c:"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4)})+'"':'"'+string+'"'}function str(key,holder){var i,k,v,length,mind=gap,partial,value=holder[key];if(value&&typeof value==="object"&&typeof value.toJSON==="function"){value=value.toJSON(key)}if(typeof rep==="function"){value=rep.call(holder,key,value)}switch(typeof value){case"string":return quote(value);case"number":return isFinite(value)?String(value):"null";case"boolean":case"null":return String(value);case"object":if(!value){return"null"}gap+=indent;partial=[];if(Object.prototype.toString.apply(value)==="[object Array]"){length=value.length;for(i=0;i<length;i+=1){partial[i]=str(i,value)||"null"}v=partial.length===0?"[]":gap?"[\n"+gap+partial.join(",\n"+gap)+"\n"+mind+"]":"["+partial.join(",")+"]";gap=mind;return v}if(rep&&typeof rep==="object"){length=rep.length;for(i=0;i<length;i+=1){k=rep[i];if(typeof k==="string"){v=str(k,value);if(v){partial.push(quote(k)+(gap?": ":":")+v)}}}}else{for(k in value){if(Object.hasOwnProperty.call(value,k)){v=str(k,value);if(v){partial.push(quote(k)+(gap?": ":":")+v)}}}}v=partial.length===0?"{}":gap?"{\n"+gap+partial.join(",\n"+gap)+"\n"+mind+"}":"{"+partial.join(",")+"}";gap=mind;return v}}if(typeof JSON.stringify!=="function"){JSON.stringify=function(value,replacer,space){var i;gap="";indent="";if(typeof space==="number"){for(i=0;i<space;i+=1){indent+=" "}}else{if(typeof space==="string"){indent=space}}rep=replacer;if(replacer&&typeof replacer!=="function"&&(typeof replacer!=="object"||typeof replacer.length!=="number")){throw new Error("JSON.stringify")}return str("",{"":value})}}if(typeof JSON.parse!=="function"){JSON.parse=function(text,reviver){var j;function walk(holder,key){var k,v,value=holder[key];if(value&&typeof value==="object"){for(k in value){if(Object.hasOwnProperty.call(value,k)){v=walk(value,k);if(v!==undefined){value[k]=v}else{delete value[k]}}}}return reviver.call(holder,key,value)}cx.lastIndex=0;if(cx.test(text)){text=text.replace(cx,function(a){return"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4)})}if(/^[\],:{}\s]*$/.test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g,"@").replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g,"]").replace(/(?:^|:|,)(?:\s*\[)+/g,""))){j=eval("("+text+")");return typeof reviver==="function"?walk({"":j},""):j}throw new SyntaxError("JSON.parse")}}}());// animation speeds
// in ms or "fast", "slow"
var ANI_SPD = "fast";

// speed to show help tool tip on first query page
// in ms
var HELP_SPD = 1500;

// ms to wait before showing loading screen
var LOADING_WAIT = 1000;

var MIN_FLASH_VERSION = 10;

var MIN_FLASH_MINOR_VERSION = 1;

// whether to show debug messages
window.DEBUG = true;
$(function(){
	if( $("#search_page").size() != 0 || $("#results_page").size() != 0 ){
		// load i18n
		$.i18n({
			url: absoluteUrl("json/i18n")
		});
	}
});
// need to set this or jquery adds "[]" to array variables in posts
// bad for networks -- spring can pick up "networks" not "networks[]"
jQuery.ajaxSettings.traditional = true;


CytowebUtil = {};

$(function(){

	/**********************************************
	 * Loader
	 **********************************************/
	
    $.fn.showLoader = function(options) {
	    var options = $.extend({
	        message: "Loading..."
	    }, options);

	    var loader = $('<div class="loader">' +
                           '<div>' +
                               '<div class="icon"></div>' +
                               '<div class="message">' + (options.message || '') + '</div>' +
                           '</div>' +
                       '</div>');
        $(this).append(loader);
    };
    
    $.fn.hideLoader = function() {
        $(this).find(".loader").fadeOut(ANI_SPD, function(){
            $(this).remove();
        });
    };
 

});


function rgb2hex(rgb) {
    rgb = rgb.match(/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/);
    function hex(x) {
        return parseInt(x).toString(16);
    }
    return "#" + hex(rgb[1]) + hex(rgb[2]) + hex(rgb[3]);
}

if( !window.console || !console || !console.log ){
	console = { log: function(){} };
}

function absoluteUrl(relativeUrl){
	return $("html").attr("contextpath") + "/" + relativeUrl;
}

function track(object, action, label, value){
	// don't track while progress bar is going
	if( !$("#progress").is(":visible") ){
//		console.log("track (" + object + ", " + action + ", " + label + ", " + value + ")");
		
		_gaq.push(['_trackEvent', object, action, label, value]);
	}
}
CytowebUtil._addListeners = function() {
    
    var node_hover_timeout;

    function handle_click(evt){
        if(!CytowebUtil._shift_down){
        	
        	if( evt.target.group == "edges" && evt.target.data.attributeId != null ){
        		evt.target = _vis.node(evt.target.data.source);
    			evt.group = "nodes";
    			
    			_vis.deselect();
    			setTimeout(function(){
    				_vis.select([ evt.target.data.id ]);
    			}, 10);
    			
        	}
        	
            CytowebUtil._showTooltip(evt);
        }
    }

    _vis.addListener("select", "nodes", function(evt) {
    	setTimeout(function(){
    		var nodesArray = evt.target;
	    	var already_selected = false;
	    	$.each(nodesArray, function(i, node) {
	    		if (!$("#gene"+node.data.id + " .label").hasClass("selected")) {
	    			// hilight node
	    			$("#gene"+node.data.id + " .label").addClass("selected");
	    			already_selected = true;
	    			
	    	    	// Expand the gene row content:
	    	    	if(false && $("#gene"+node.data.id + " .arrow").hasClass("collapsed")) {
	    				$("#gene"+node.data.id + " .arrow").click();
	    			}
	    		}
	    	});
	    	// scroll to only first gene
	    	if (false && already_selected) {
				var first_node = $("#genes_widget").find(".label.selected").parent();
				$("#genes_tab .content").scrollTo($(first_node), CytowebUtil.SCROLL_DELAY);
	    	}
	    	CytowebUtil._onSelectNodesChanged();
    	}, 100);
    	
    })
    .addListener("deselect", "nodes", function(evt) {
    	setTimeout(function(){
	    	var nodesArray = evt.target;
	    	$.each(nodesArray, function(i, node) {
	    		$("#gene"+node.data.id + " .label").removeClass("selected");
	    	});
	    	CytowebUtil._onSelectNodesChanged();
    	}, 100);
    })
    .addListener("select", "edges", function(evt) {
    	setTimeout(function(){
    		CytowebUtil._onSelectEdgesChanged();
    	}, 100);
    })
    .addListener("deselect", "edges", function(evt) {
    	setTimeout(function(){
    		CytowebUtil._onSelectEdgesChanged();
    	}, 100);
    })
    .addListener("click", "nodes", function(evt) {
        handle_click(evt);
        track("Node", "Click");
    })
    .addListener("dblclick", "nodes", function(evt) {
        handle_click(evt);
    })
    .addListener("click", "edges", function(evt) {
        handle_click(evt);
        track("Edge", "Click");
    })
    .addListener("dblclick", "edges", function(evt) {
        handle_click(evt);
    })
    .addListener("layout", function(evt) {
    	
    })
	.addListener("error", function(error) {
		//console.log('Error drawing network: ' + evt.value.msg);
		CytowebUtil.error(error.value);
	})
	.addListener("click", function(evt) {
    	CytowebUtil.body_click();
    });
};CytowebUtil.updateBypass = function(){
	if(_vis){
		var bypass = {};
		
		var highlightNodeOpacity = 1;
		var highlightEdgeOpacity = 1;
		var unhighlightNodeOpacity = 0.125;
		var halfHighlightNodeOpacity = 1;
		var unhighlightEdgeOpacity = 0.125;
		var highlightBorderColor = "#aae6ff";
		var highlightBorderWidth = 3;
		
		
		function assign(group, id, property, value){
			if( bypass[group] == null ){
				bypass[group] = {};
			}
			
			if( bypass[group][id] == null ){
				bypass[group][id] = {};
			}
			
			if( value != null ){
				bypass[group][id][property] = value;
				
				if( group == "edges" && property == "opacity" ){
					bypass[group][id]["mergeOpacity"] = value;
				}
			} else {
				delete bypass[group][id][property];
				
				if( group == "edges" && property == "opacity" ){
					delete bypass[group][id]["mergeOpacity"];
				} 
			}
		}
		
		$.each(CytowebUtil.goColor, function(id, color){
			assign("nodes", id, "color", color);
			assign("nodes", id, "labelGlowColor", color);
			assign("nodes", id, "borderColor", color);
		});
		
		$.each(CytowebUtil.goEdgesColor, function(id, color){
			assign("edges", id, "color", color);
		});
		
		if( CytowebUtil._highlight ){
			$.each(CytowebUtil.goHighlight, function(id, highlight){
				assign("nodes", id, "opacity", highlight ? highlightNodeOpacity : unhighlightNodeOpacity);
			});
			
			$.each(CytowebUtil.goEdgesHighlight, function(id, highlight){
				assign("edges", id, "opacity", highlight ? highlightEdgeOpacity : unhighlightEdgeOpacity);
			});
		} else if( CytowebUtil.geneHighlighted ){
			var edges = CytowebUtil.edges();
			$.each(edges, function(i, edge){
				assign("edges", edge.data.id, "opacity", unhighlightEdgeOpacity);
			});
			
			var nodes = CytowebUtil.nodes();
			$.each(nodes, function(i, node){
				assign("nodes", node.data.id, "opacity", unhighlightNodeOpacity);
			});
			
			$.each(CytowebUtil.highlightedGenes, function(id, highlight){
				assign("nodes", id, "opacity", highlight ? highlightNodeOpacity : unhighlightNodeOpacity);
				assign("nodes", id, "borderColor", highlight ? highlightBorderColor : null);
				assign("nodes", id, "borderWidth", highlight ? highlightBorderWidth : null);
				assign("nodes", id, "selectionBorderColor", highlight ? highlightBorderColor : null);
			});
			$.each(CytowebUtil.highlightedGeneEdges, function(id, highlight){
				assign("edges", id, "opacity", highlight ? highlightEdgeOpacity : unhighlightEdgeOpacity);
			});
			$.each(CytowebUtil.halfHighlightedGenes, function(id, highlight){
				assign("nodes", id, "opacity", highlight ? halfHighlightNodeOpacity : unhighlightNodeOpacity);
			});
		} else if( CytowebUtil.neighborsHighlighted ){
			var edges = CytowebUtil.edges();
			$.each(edges, function(i, edge){
				assign("edges", edge.data.id, "opacity", unhighlightEdgeOpacity);
			});
			
			var nodes = CytowebUtil.nodes();
			$.each(nodes, function(i, node){
				assign("nodes", node.data.id, "opacity", unhighlightNodeOpacity);
			});
			
			$.each(CytowebUtil.firstNeighborsNodes, function(id, highlight){
				assign("nodes", id, "opacity", highlight ? highlightNodeOpacity : unhighlightNodeOpacity);
			});
			$.each(CytowebUtil.firstNeighborsEdges, function(id, highlight){
				assign("edges", id, "opacity", highlight ? highlightEdgeOpacity : unhighlightEdgeOpacity);
			});
		}
		
		_vis.visualStyleBypass(bypass);
	}
};CytowebUtil.checkVersionFor = function(name){
		var version = "" + $("html").attr("webversion");
	var dbVersion = "" + $("html").attr("dbversion");
	var newVersion;
	var newDbVersion;
	
	var error = false;
	
	$.ajax({
		async: false,
		type: "GET",
		data: {},
		dataType: "json",
		url: absoluteUrl("json/version"),
		error: function(request, status, error){
			warning();
			error = true;
		},
		success: function(data, status, request){
			newVersion = "" + data.webappVersion;
			newDbVersion = "" + data.dbVersion;
			
		}
	});
	
	if( error ){
		return false;
	}
	
	function warning(){
		$('<p>GeneMANIA has been updated since you performed your search.  To make sure that your ' + name + ' is consistent ' +
				'with results from the new version of GeneMANIA, please resubmit your search and try to generate the ' +
				name + ' again.</p>').dialog({
			title: "The " + name + " can not be generated",
			buttons: {
				"OK, resubmit the search for me.": function(){ $(this).dialog("close"); $("#reloader").show(); $("#resubmit_form").submit(); },
				"Nevermind, I will resubmit myself later.": function(){ $(this).dialog("close"); }
			},
			modal: true,
			closeOnEscape: true,
	    	resizable: false,
	    	width: 500,
	    	minHeight: 0
		});
	}
	

	// ok to print if versions match or local debug version
	if( version.search("antdebug") < 0 && ( newVersion != version || newDbVersion != dbVersion ) ){
//			console.log("version: " + version);
//			console.log("new version: " + newVersion);
//			console.log("dbversion: " + dbVersion);
//			console.log("new dbversion: " + newDbVersion);
//			console.log("different versions -- can't print");
		
		warning();
		
		return false;
	} else {
//			console.log("ok to make report");
		
		return true;
	}
};_edgesCache = null;

CytowebUtil.refresh = function() {
	// Create a Cytoscape Lite instance:
	// ---------------------------------------------------------------------------------------------
	var options = {
			swfPath: absoluteUrl("swf/CytoscapeWeb_1.0.4_01"),
			flashInstallerPath: absoluteUrl("swf/playerProductInstall"),
			flashAlternateContent: '<div class="ui-state-error ui-corner-all"><p>GeneMANIA requires the Adobe Flash Player to use all of its features.</p>' +
			                       '<p><a href="http://get.adobe.com/flashplayer/"><img width="160" height="41" border="0" alt="Get Adobe Flash Player" src="http://www.adobe.com/macromedia/style_guide/images/160x41_Get_Flash_Player.jpg"></a></p></div>'
	};
	_vis = new org.cytoscapeweb.Visualization("graphBox", options);
	_vis.embedSWF = this._embedSWF;
	
	_vis.shapeMapper = CytowebUtil.shapeMapper;
	_vis.imageMapper = CytowebUtil.imageMapper;
	
	// Overwrite callback functions of interest:
	// ---------------------------------------------------------------------------------------------
	_vis.ready(function() {
		CytowebUtil.recomputeLayout();

		// Update the page:
		_vis.swf().focus(); // So ctrl-click, etc. work without having to click the CW area first!

		_edgesCache = CytowebUtil.edges();
		
		CytowebUtil.updateNetworksTab();
    	if (onCytoscapeWebLoaded) { onCytoscapeWebLoaded(); }
    	
    	// Context menu items:
	    _vis.addContextMenuItem("About Cytoscape Web...", function(evt) {
    		window.open("http://cytoscapeweb.cytoscape.org/");	
    	});

    	// Workaround to prevent the cursor from disappearing when CytoWeb is using a custom cursor
    	// and the mouse is over an HTML container that overlaps the Flash area:
    	$(".over_flash, .qtip").live("mouseover", function() {
			if (_vis) { _vis.customCursorsEnabled(false); }
		}).live("mouseout", function() {
			if (_vis) { _vis.customCursorsEnabled(true); }
		});
		
		// enable query genes grey colour by default
        $("#go_tab .content table tr.query .add_button").children(":first").click();
        
        // no tooltips open at start
        $("#menu_close_tooltips").addClass("ui-state-disabled");
        
        progress("cytolite");
        $("body").trigger("cytoweb"); // added to help the QUnit test recognize when the results page is loaded
        //console.log(_vis.xgmml());
		
    });

	_vis.onEdgeTooltip = function(data) {
		var w = Math.round(data.weight*1000)/10;
		w = w < 0.1 ? "&lt; 0.1" : w;
		return "<b>weight:</b> " + w;
	}
	
	CytowebUtil._addListeners();
    
    // Draw the network:
    // ---------------------------------------------------------------------------------------------
	
	CytowebUtil.loadNetwork();
};


CytowebUtil.loadNetwork = function(){
	var selected_networks = [];
	var selected_attrgroups = [];
	
	$("#selected_networks").children().filter("[network]").each(function(){
		var id = parseInt( $(this).attr("network") );
		selected_networks.push( id );
	});
		
	$("#selected_networks").children().filter("[attrgroup]").each(function(){
		var id = parseInt( $(this).attr("attrgroup") );
		selected_attrgroups.push( id );
	});
	
	//console.log(selected_networks);
	$.ajax({
	    type: "POST",
	    url: absoluteUrl("json/visualization?" + Math.random()),
	    dataType:"json",
	    data: {
			networks: selected_networks,
			organism: $("[name=organism]").val(),
			genes: $("[name=genes]").val(),
		 	weighting: $("[name=weighting]:checked").val(),
			threshold: $("[name=threshold]").val(),
			attrThreshold: $("[name=attrThreshold]").val(),
			attrgroups: selected_attrgroups
		},
	    success: function (response) {
			CytowebUtil.OPTIONS.network = response;
//				console.log("response");
//				console.log(response);
			
			_vis.draw(CytowebUtil.OPTIONS);
	    },
	    error: function (xhr, ajaxOptions, thrownError){
	    	//console.log('Error loading network : ' + thrownError);
	    },
	    cache: false
	});
};
	




CytowebUtil.error = function(error){
	
	var msg = "An exception occurred in Cytoscape Web and was caught by the GeneMANIA web application.  It is logged below.\n";
	
	msg += "\n====\nID\n" + error.id;
	
	msg += "\n====\nName\n" + error.name;
	
	msg += "\n====\nMessage\n" + error.msg;
	
	msg += "\n====\nStack trace\n" + error.stackTrace;
	
	msg += "\n\n\n======== Debug info ========\n\n";
	
	var i = 0;
	$("#debug_info_values").children().each(function(){
		
		if( i % 2 == 0){
			msg += "====\n";
		}
		
		msg += $(this).text() + "\n";
		i++;
	});
	
	$("#cytoweb_error").show();
	
	$.ajax({
		type: "POST",
		url: absoluteUrl("json/mail"),
		data: { 
			subject: "GeneMANIA exception: " + error.name + " : " + error.msg,
			message: msg
		},
		success: function(){
			$("#cytoweb_error .sending").hide();
			$("#cytoweb_error .sent").show();
		},
		error: function(){
			$("#cytoweb_error .sending").hide();
			$("#cytoweb_error .error").show();
		}
	});
};


CytowebUtil.edges = function(){
	if( CytowebUtil._edges == null ){
		CytowebUtil._edges = _vis.edges();
	}
	
	return CytowebUtil._edges;
};

CytowebUtil.nodes = function(){
	if( CytowebUtil._nodes == null ){
		CytowebUtil._nodes = _vis.nodes();
	}
	
	return CytowebUtil._nodes;
};

CytowebUtil.genes = function(){
	if( CytowebUtil._genes == null ){
		CytowebUtil._genes = $.grep( CytowebUtil.nodes(), function(node){
			return !node.data.attribute;
		} );
	}
	
	return CytowebUtil._genes;
};

CytowebUtil.node = function(id){
	if( CytowebUtil._nodesById == null ){
		var map = CytowebUtil._nodesById = [];
		
		$.each(CytowebUtil.nodes(), function(i, node){
			map[ node.data.id ] = node;
		});
	}
	
	return CytowebUtil._nodesById[id];
};

CytowebUtil.edge = function(id){
	if( CytowebUtil._edgesById == null ){
		var map = CytowebUtil._edgesById = [];
		
		$.each(CytowebUtil.edges(), function(i, edge){
			map[ edge.data.id ] = edge;
		});
	}
	
	return CytowebUtil._edgesById[id];
};


CytowebUtil._embedSWF = function() {
    //Major version of Flash required
    var requiredMajorVersion = MIN_FLASH_VERSION;
    //Minor version of Flash required
    var requiredMinorVersion = MIN_FLASH_MINOR_VERSION;
    //Minor version of Flash required
    var requiredRevision = 0;

    var containerId = this.containerId;

    // Let's redefine the default AC_OETags function, because we don't necessarily want
    // to replace the whole HTML page with the swf object:
    AC_Generateobj = function (objAttrs, params, embedAttrs) {
        var str = '';
        var i;
        if (isIE && isWin && !isOpera) {
            str += '<object ';
            for (i in objAttrs) {
                if (Object.hasOwnProperty.call(objAttrs, i)) {
                    str += i + '="' + objAttrs[i] + '" ';
                }
            }
            str += '>';
            for (i in params) {
                if (Object.hasOwnProperty.call(params, i)) {
                    str += '<param name="' + i + '" value="' + params[i] + '" /> ';
                }
            }
            str += '</object>';
        } else {
            str += '<embed ';
            for (i in embedAttrs) {
                if (Object.hasOwnProperty.call(embedAttrs, i)) {
                    str += i + '="' + embedAttrs[i] + '" ';
                }
            }
            str += '> </embed>';
        }
        // Replace only the indicated DOM element:
        document.getElementById(containerId).innerHTML = str;
    };

    // Version check for the Flash Player that has the ability to start Player Product Install (6.0r65)
    var hasProductInstall = DetectFlashVer(6, 0, 65);

    // Version check based upon the values defined in globals
    var hasRequestedVersion = DetectFlashVer(requiredMajorVersion, requiredMinorVersion, requiredRevision);

    if (hasProductInstall && !hasRequestedVersion) {
        // DO NOT MODIFY THE FOLLOWING FOUR LINES
        // Location visited after installation is complete if installation is required
        var MMPlayerType = (isIE === true) ? "ActiveX" : "PlugIn";
        var MMredirectURL = window.location;
        document.title = document.title.slice(0, 47) + " - Flash Player Installation";
        var MMdoctitle = document.title;

        AC_FL_RunContent(
            "src", this.flashInstallerPath,
            "FlashVars", "MMredirectURL="+MMredirectURL+'&MMplayerType='+MMPlayerType+'&MMdoctitle='+MMdoctitle+"",
            "width", "100%",
            "height", "100%",
            "align", "middle",
            "id", this.id,
            "quality", "high",
            "bgcolor", "#ffffff",
            "name", this.id,
            "allowScriptAccess","sameDomain",
            "type", "application/x-shockwave-flash",
            "pluginspage", "http://www.adobe.com/go/getflashplayer"
        );
        
        if (onCytoscapeWebLoaded) { onCytoscapeWebLoaded(true); progress("cytolite"); }
    } else if (hasRequestedVersion) {
        var optionKeys = ["resourceBundleUrl"];
        var flashVars = "";
        if (this.options) {
            for (var i in optionKeys) {
                if (Object.hasOwnProperty.call(optionKeys, i)) {
                    var key = optionKeys[i];
                    if (this.options[key] !== undefined) {
                        flashVars += key + "=" + this.options[key] + "&";
                    }
                }
            }
            flashVars += "id=" + this.id;
        }

        // if we've detected an acceptable version
        // embed the Flash Content SWF when all tests are passed
        AC_FL_RunContent(
                "src", this.swfPath,
                "width", "100%",
                "height", "100%",
                "align", "middle",
                "id", this.id,
                "quality", "high",
                "bgcolor", "#ffffff",
                "name", this.id,
                "allowScriptAccess", "always",
                "type", "application/x-shockwave-flash",
                "pluginspage", "http://www.adobe.com/go/getflashplayer",
                "wmode", "opaque", // DO NOT set it to "transparent", because it may crash FireFox and IE on Windows!
                "flashVars", flashVars
        );
    } else { // flash is too old or we can't detect the plugin
        // Insert non-flash content:
        document.getElementById(containerId).innerHTML = this.flashAlternateContent;
    	
        if (onCytoscapeWebLoaded) { onCytoscapeWebLoaded(true); progress("cytolite"); }
    }
    return this;
};
CytowebUtil.exportNetwork = function() {
		if( !CytowebUtil.checkVersionFor("text network") ){
		return;
	}
	
	$("#text_form").submit();
};

CytowebUtil.exportParams = function() {
	$("#params_form").submit();
};

CytowebUtil.exportParamsJson = function() {
	$("#params_json_form").submit();
};

CytowebUtil.exportNetworks = function() {
	if( !CytowebUtil.checkVersionFor("networks") ){
		return;
	}
	
	$("#networks_form").submit();
};

CytowebUtil.exportGenes = function() {
	if( !CytowebUtil.checkVersionFor("genes") ){
		return;
	}
	
	$("#genes_form").submit();
};

CytowebUtil.exportGo = function() {
	if( !CytowebUtil.checkVersionFor("functions") ){
		return;
	}
	
	$("#go_form").submit();
};

CytowebUtil.exportInteractions = function() {
	if( !CytowebUtil.checkVersionFor("interactions") ){
		return;
	}
	
	$("#interactions_form").submit();
};

CytowebUtil.exportSvg = function(use_svg_from_cytoweb) {
	if( use_svg_from_cytoweb == undefined || use_svg_from_cytoweb ){
		$("#svg_form [name=content]").val( _vis.svg() );
	}
	$("#svg_form").submit();
};

CytowebUtil.exportAttributes = function() {
	if( !CytowebUtil.checkVersionFor("attributes") ){
		return;
	}
	
	$("#attributes_form").submit();
};
CytowebUtil._lastFilter = null; // filter function

var _filterTimeout = null;
CytowebUtil.filterNetworks = function() {

	clearTimeout(_filterTimeout);
	_filterTimeout = setTimeout(function(){
		filter();
	}, 25);
	
	function filter(){
		if (_vis) {
			// Get checked network groups:
			var checkedGr = { netid: {}, attrid: {} };
			var halfcheckedGr = { netid: {}, attrid: {} };
			var count = 0;
			var edges = CytowebUtil.edges();
			var nodes = CytowebUtil.nodes();
			
			$("#networks_tab .checktree_top_level").each(function() {
				var $this = $(this);
				var id = $this.attr("netid") || $this.attr("attrid");
				var idAttr = $this.attr("netid") ? "netid" : "attrid";
				
				if( $this.hasClass("checktree_attr_group") ){
					id = $this.attr("attrid");
				}
				
				var $checkbox = $this.find(".checkbox:first");
				if ( $checkbox.hasClass("checked") ) {
					checkedGr[idAttr][id] = true;
				}
				if ( $checkbox.hasClass("half_checked") ) {
					halfcheckedGr[idAttr][id] = true;
				}
				if ( $checkbox.size() > 0 ) {
					count++;
				}
			});
	
			// Get checked individual networks: 
			var checkedNet = { netid: {}, attrid: {} };
			for(var grId in halfcheckedGr['netid']){			
				$("#networks_tab .checktree_network_group[netid=" + grId + "]").find(".checktree_network").each(function() {
					var id = $(this).attr("netid");
					if ($(this).find(".checked").length > 0) {
						checkedNet['netid'][id] = true;
					}
				});
			}
			for(var grId in halfcheckedGr['attrid']){	
				$("#networks_tab .checktree_attr_group[attrid=" + grId + "]").find(".checktree_attr").each(function() {
					var id = $(this).attr("attrid");
					if ($(this).find(".checked").length > 0) {
						checkedNet['attrid'][id] = true;
					}
				});
			}
			//console.log("individual networks");
//			console.log(checkedGr, halfcheckedGr, checkedNet);
			//console.log("groups");
			
			function filterFunction(edge){
				//console.log("Checking edge:");
				//console.log(edge);

//				if( edge.data.networkGroupName === "Uploaded" ){
//					console.log( edge );
//					console.log( checkedGr );
//				}	
				
				var grId = edge.data.networkGroupId;
				var idAttr = "netid";
				
				if( grId === null || grId === undefined || grId === "" || (edge.data.attributeId != null && edge.data.attributeId + 0 === edge.data.attributeId) ){
					grId = edge.data.attributeGroupId;
					idAttr = "attrid";
				}
				
				//console.log("Check the group " + edge.data.networkGroupId + " first, which is faster:");
				if ( checkedGr[idAttr][grId] ) {
					//console.log("all of group " + grId + " checked");
//					console.log('checked group');
					return true;
					
				} else if( halfcheckedGr[idAttr][grId] ){
//					console.log('half checked group');
					
					if( $("#networks_tab .checktree_top_level["+ idAttr +"=" + grId + "]").find(".checkbox.half_checked").size() > 0 ){
						//console.log("Still have to verify each individual network since group " + grId + " is half checked");
						var networksMap = edge.data.networkIdToWeight;
						//console.log("Edge has networks: ");
						//console.log(networksMap);
						
						for (var id in networksMap) {
						    if ( checkedNet[idAttr][id] ) {
						    	//console.log("The edge (" + edge.data.id + ") has at least one checked network (" + id + ")");
						    	return true;
						    }
						}
						
						if( checkedNet[idAttr][edge.data.attributeId] ){
							return true;
						}
					} else {
						//console.log("Group " + grId + " is fully checked so edge is OK");
						return true;
					}
				} else {
					//console.log("No networks checked for edge");
					return false;
				}
			}
			
			var filterList = [];
			$.each(edges, function(i, edge){
				if( filterFunction(edge) ){
					//console.log('call ff true', edge);
					filterList.push(edge.data.id);
				} else {
					//console.log('call ff false', edge);
				}
			});
			
			$.each(nodes, function(i, node){
				if( node.data.attribute ){
					if( $("#networks_widget .checktree_attr[attrid="+ node.data.attributeId +"] :checkbox:first").is(":checked") ){
						filterList.push(node.data.id);
					}
				} else {
					filterList.push(node.data.id);
				}
			});
			
			if (count === $("#networks_tab .checktree_top_level > .checkbox.checked").length) {
				//console.log("All of the enabled checkboxes are checked");
				_vis.removeFilter(false);
				CytowebUtil._lastFilter = null;
			} else {
				CytowebUtil._lastFilter = filterList;
				_vis.filter(filterList, true);
			}
		}
	}
};

CytowebUtil.firstNeighborsNodes = {};
CytowebUtil.firstNeighborsEdges = {};

CytowebUtil.highlightFirstNeighbors = function(nodes) {
	if (_vis) {
		CytowebUtil.firstNeighborsNodes = {};
		CytowebUtil.firstNeighborsEdges = {};
		
		if (nodes == null) {
			nodes = _vis.selected("nodes");
		}
		
		if (nodes != null && nodes.length > 0) {
			var fn = _vis.firstNeighbors(nodes, true);
			var neighbors = fn.neighbors;
			var edges = fn.edges;
			
			$.each(edges, function(i, edge){
				CytowebUtil.firstNeighborsEdges[edge.data.id] = true;
			});
			
			$.each(neighbors, function(i, node){
				CytowebUtil.firstNeighborsNodes[node.data.id] = true;
			});
			
			$.each(nodes, function(i, node){
				CytowebUtil.firstNeighborsNodes[node.data.id] = true;
			});
			
			CytowebUtil.neighborsHighlighted = true;
			$("#menu_neighbors_clear").removeClass("ui-state-disabled");
			CytowebUtil.updateBypass();
		}
	}
};

CytowebUtil.clearFirstNeighborsHighlight = function() {
	if (_vis) {
		
		CytowebUtil.firstNeighborsNodes = {};
		CytowebUtil.firstNeighborsEdges = {};
		
		CytowebUtil.neighborsHighlighted = false;
		$("#menu_neighbors_clear").addClass("ui-state-disabled");
		CytowebUtil.updateBypass();
	}
};CytowebUtil.goColor = {};
CytowebUtil.goEdgesColor = {};
CytowebUtil.goHighlight = {};
CytowebUtil.goEdgesHighlight = {};
CytowebUtil.goHighlightConnectedNodes = {};

CytowebUtil.updateGOColoursForGenes = function(){
	
    var cache = {};

    $("#genes_widget .go_list .go").add("body > .qtip .go_list .go").each(function(){
        var colour = $(this).find(".colour");
        var ocid = $(this).attr("ocid");
        var hasGo = $(this).hasClass("has");
        
        if( !hasGo ){
        	return; // skip go entries that the gene doesn't have
        }
        
        if( cache[ocid] == undefined ){
            var colouring = $("#go_tab .colouring[ocid=" + ocid + "]");
        
            if( colouring.size() > 0 ){
                var c = colouring.attr("colour");
            
                colour.css({
                    backgroundColor: c,
                    borderColor: c
                }).addClass("coloured");
                
                cache[ocid] = c;
                
            } else {
                colour.css({
                    backgroundColor: "",
                    borderColor: ""
                }).removeClass("coloured");
                
                cache[ocid] = false;
            }
        } else if( cache[ocid] ) {
            var c = cache[ocid];
            
            colour.css({
                backgroundColor: c,
                borderColor: c
            }).addClass("coloured");
            
        } else {
            colour.css({
                backgroundColor: "",
                borderColor: ""
            }).removeClass("coloured");
        }
        
    });
};

CytowebUtil.updateGO = function(draw){
    if(_vis) {
        var nodes = CytowebUtil.nodes();
        var edges = CytowebUtil.edges();
        CytowebUtil.goColor = {};
        CytowebUtil.goHighlight = {};
        CytowebUtil.goEdgesColor = {};
        CytowebUtil.goEdgesHighlight = {};
        
        if( draw == undefined ){
            draw = true;
        }
        
        $("#networks_tab .checktree_attr .bar").css("background-color", "");
        
        $("#go_tab .colouring").reverse().each(function(){
            var colouring = $(this);
            var colouring_ocid = $(this).find(".annotation").attr("ocid");
            var is_query_genes = $(this).hasClass("query");
            var tr = $("#go_tab tr[ocid="+ colouring_ocid +"]");
            var attr_id = parseInt( tr.attr("attrid") );
            var is_attr = !isNaN(attr_id);
            
            var colour = colouring.attr("colour");
            
            $.each(nodes, function(i, node) {
                var ocids = node.data.ocids;
                
                if( is_query_genes ){
                    if( node.data.queryGene == true || node.data.queryGene == "true" ){
                    	CytowebUtil.goColor[node.data.id] = colour;
                    }
                } else if( is_attr ){
                	if( node.data.attributeId == attr_id ){
                		CytowebUtil.goColor[node.data.id] = colour;
                		$("#networks_tab .checktree_attr[attrid="+ attr_id +"] .bar").css("background-color", colour);
                	}
                } else {
                    $.each(ocids, function(j, ocid){
                        if( ocid == "" || ocid != colouring_ocid ){
                            return;
                        }
                    
                        CytowebUtil.goColor[node.data.id] = colour;
                    });
                }
            });
            
            $.each(edges, function(i, edge){
            	if( is_attr ){
            		if( edge.data.attributeId == attr_id ){
            			CytowebUtil.goEdgesColor[edge.data.id] = colour;
            		}
            	}
            });
        });
        
        if( draw ){
		    CytowebUtil.updateBypass();
		}
	}
};

CytowebUtil._highlight = false;

CytowebUtil._highlightGO = function(go_id, is_query_genes){
    if(_vis) {
        
        var colouring = $("#go_tab .colouring[ocid=" + go_id + "]");
        var tr = $("#go_tab tr[ocid="+ go_id +"]");
        var colour;
        var attr_id = parseInt( tr.attr("attrid") );
        var is_attr = attr_id != null && !isNaN(attr_id);
        
        is_query_genes = ( is_query_genes ? true : false );
        CytowebUtil.goHighlight = {};
        
        if( is_query_genes ){
            colour = $("#go_tab").attr("querycolour");
        } else if( colouring.size() > 0 ){
            colour = colouring.attr("colour");
        } else {
            colour = $("#go_tab").attr("colour");
        }
        
        var nodes = CytowebUtil.nodes();
        var node_ids = [];
        for(var i in nodes){
            var node = nodes[i];
            
            var go_ids = node.data.ocids;
            var query = ( node.data.queryGene == "true" || node.data.queryGene == true );
            
            CytowebUtil.goHighlight[node.data.id] = false;
            
            if( is_query_genes ){
                if( query ){
                	CytowebUtil.goHighlight[node.data.id] = true;
            		CytowebUtil.goColor[node.data.id] = colour;	
                }
            } else if( is_attr ){
            	if( node.data.attributeId == attr_id ){
            		CytowebUtil.goHighlight[node.data.id] = true;
            		CytowebUtil.goColor[node.data.id] = colour;
            	}
            	
            	if( $.inArray(attr_id, node.data.attrids) >= 0 ){
            		CytowebUtil.goHighlight[node.data.id] = true;
            	}
            } else {
                for(var j in go_ids){
                    var id = go_ids[j];
                    
                    if( id == go_id ){
                    	CytowebUtil.goColor[node.data.id] = colour;
                    	CytowebUtil.goHighlight[node.data.id] = true;
                        break;
                    }
                }
            }
            
            if( CytowebUtil.goHighlight[node.data.id] ){
            	node_ids.push( node.data.id );
            }
        }
        
        var edges = CytowebUtil.edges();
    	$.each(edges, function(i, e){
    		if( is_attr ){
	    		if( e.data.attributeId == attr_id ){
	    			CytowebUtil.goEdgesColor[e.data.id] = colour;
	    			CytowebUtil.goEdgesHighlight[e.data.id] = true;
	    		} else {
	    			CytowebUtil.goEdgesHighlight[e.data.id] = false;
	    		}
    		} else {
    			var srcId = e.data.source;
    			var tgtId = e.data.target;
    			CytowebUtil.goEdgesHighlight[e.data.id] = CytowebUtil.goHighlight[srcId] && CytowebUtil.goHighlight[tgtId];
    		}
    	});
        
        CytowebUtil.updateBypass();
    }
};

CytowebUtil.highlightGO = function(go_id, is_query_genes){
    if( CytowebUtil._highlight ){
        CytowebUtil.updateGO(false);
    }
    
    CytowebUtil._highlight = true;
    
    CytowebUtil._highlightGO(go_id, is_query_genes);
};

CytowebUtil.unhighlightGO = function(go_id, is_query_genes){
	CytowebUtil._highlight = false;
    CytowebUtil.updateGO();
};CytowebUtil.recomputeLayout = function() {
//console.log("recompute layout");
	
	if(_vis) {
		$("#graph").showLoader({ message: "Resetting layout..." });

		var layout = { name: "ForceDirected", options: { weightAttr: "weight", restLength: 25, seed: 7 } };

		var layoutListener = function() {
			_vis.removeListener("layout", layoutListener);
			$("#graph").hideLoader();
		};
		
		_vis.addListener("layout", layoutListener);
		_vis.layout(layout);
		
	}
	
	
};
//Mapping network groups to edge colors:
CytowebUtil._edgeColorMapper = {
		attrName: "code",
		entries: [
	          { attrValue: "coexp"  	, value: "#d0b7d5" }, //Co-expression
	          { attrValue: "coloc"  	, value: "#a0b3dc" }, //Co-localization
	          { attrValue: "gi"     	, value: "#90e190" }, //Genetic interactions
	          { attrValue: "path"   	, value: "#9bd8de" }, //Pathway
	          { attrValue: "pi"     	, value: "#eaa2a2" }, //Physical interactions
	          { attrValue: "predict"	, value: "#f6c384" }, //Predicted
	          { attrValue: "spd"    	, value: "#dad4a2" }, //Shared protein domains
	          { attrValue: "spd_attr"	, value: "#D0D0D0" }, 
	          { attrValue: "reg"  		, value: "#D0D0D0" }, //Regulatory
	          { attrValue: "reg_attr"  	, value: "#D0D0D0" }, //Regulatory
	          { attrValue: "user"   	, value: "#f0ec86" }, //User-defined
		]
};

CytowebUtil.DEF_EDGE_OPACITY = 0.85;
CytowebUtil.AUX_EDGE_OPACITY = 0.4;
CytowebUtil.DEF_MERGED_EDGE_OPACITY = 0.6;

CytowebUtil.VISUAL_STYLE_OPACITY = {
	defaultValue: CytowebUtil.DEF_EDGE_OPACITY, 
       discreteMapper: {
		   attrName: "networkGroupCode",
		   entries: [
		          { attrValue: "coexp", value: CytowebUtil.AUX_EDGE_OPACITY },
		          { attrValue: "coloc", value: CytowebUtil.AUX_EDGE_OPACITY }
			]
       }
};

CytowebUtil.shapeMapper = function(data){
	if( data.attribute ){
		return "DIAMOND";
	}
	
	return null;
};

CytowebUtil.nodeColorMapper = {
	attrName: "attribute",
	entries: [ { attrValue: true, value: "#808080" } ]
};

CytowebUtil.nodeLabelGlowColorMapper = {
	attrName: "attribute",
	entries: [ { attrValue: true, value: "#808080" } ]
};

CytowebUtil.imageMapper = function(data){
	if( data.queryGene ){
		return absoluteUrl("img/etc/node_stripe.png");
	}
}

CytowebUtil.VISUAL_STYLE = {
		global: {
			backgroundColor: "#ffffff",
			selectionFillColor: "#aaaaaa",
			selectionLineColor: "#333333",
			selectionFillOpacity: 0.25,
			selectionLineOpacity: 0.5,
			selectionLineWidth: 1
		},
		nodes: {
			shape: {
				defaultValue: "ELLIPSE",
				customMapper: { functionName: "shapeMapper" }
			},
			image: {
				customMapper: { functionName: "imageMapper" }
			},
			color: { defaultValue: "#808080", discreteMapper: CytowebUtil.nodeColorMapper },
			opacity: 1,
			size: { defaultValue: 30, continuousMapper: { attrName: "score", minValue: 20, maxValue: 45 } },
			borderColor: "#777777",
			borderWidth: 1,
			label: { passthroughMapper: { attrName: "symbol" } },
			labelFontColor: "#ffffff",
			labelFontWeight: "bold",
			labelGlowColor: { defaultValue: "#808080", discreteMapper: CytowebUtil.nodeLabelGlowColorMapper },
            labelGlowOpacity: 1,
            labelGlowBlur: 4,
			labelGlowStrength: 32,
            labelHorizontalAnchor: "center",
            labelVerticalAnchor: "middle",
			selectionBorderColor: "#ffff77",
			selectionBorderWidth: 3,
			selectionGlowColor: "#ffff33",
			selectionGlowOpacity: 0,
			hoverBorderColor: "#aae6ff",
			hoverBorderWidth: 3,
			hoverGlowColor: "#aae6ff",
			hoverGlowOpacity: 0
		},
		edges: {
			color: {
				defaultValue: "#999999",
		        discreteMapper: CytowebUtil._edgeColorMapper
			},
			width: { defaultValue: 1.5, continuousMapper: { attrName: "weight", minValue: 1.5, maxValue: 5 } },
			mergeWidth: { defaultValue: 1, continuousMapper: { attrName: "weight", minValue: 2, maxValue: 5 } },
			opacity: CytowebUtil.VISUAL_STYLE_OPACITY,
			mergeOpacity: CytowebUtil.DEF_MERGED_EDGE_OPACITY,
			curvature: 16,
			selectionGlowColor: "#ffff33",
			selectionGlowOpacity: 0
		}
};

CytowebUtil.OPTIONS = {
		layout: { name: "Preset", options: { fitToScreen: false } },
		panZoomControlVisible: true,
		edgesMerged: false,
		nodeLabelsVisible: true,
		edgeLabelsVisible: false,
		nodeTooltipsEnabled: false,
		edgeTooltipsEnabled: false,
		visualStyle: CytowebUtil.VISUAL_STYLE
};

CytowebUtil.SCROLL_DELAY = 500;CytowebUtil._tooltip_timeout = null;

CytowebUtil._showTooltip = function(evt){
    $("#menu_close_tooltips").removeClass("ui-state-disabled");
    CytowebUtil._makeTooltip(evt);
};

CytowebUtil._makeTooltip = function(evt){
	//console.log("making tooltip");
	
    var outer_padding = 4;

    function sanitise(id){
    	
    	// replace : with \\: for jquery
    	
    	return id.replace(/\:/g, "-");
    	
    }
    
    var merged = evt.target.merged;
    
//    console.log(evt);
    var id;
    
    var prev_qtip;
    if( evt.group == "nodes" ){
    	id = sanitise(evt.target.data.id);
        prev_qtip = $(".qtip[nodeid=" + id + "]:visible");
    } else if( !merged ){  
        id = sanitise(evt.target.data.id);
    	prev_qtip = $(".qtip[edgeid=" + id + "]:visible");
    } else {
    	id = sanitise( evt.target.data.source[0] + "-" + evt.target.data.target[0] );
    	prev_qtip = $(".qtip[medgeid=" + id + "]:visible");
    }
    
     
    
    if( prev_qtip.size() > 0 ){
    	//console.log("show existing previous tooltip");
    	
        var title = prev_qtip.find(".qtip-title");
        var content = prev_qtip.find(".qtip-content");
        
        function flash(ele){
            var bg = ele.css("background-color");
            
            if( !ele.hasClass("highlighting") ){
            
                ele.addClass("highlighting");
            
                ele.animate({
                    backgroundColor: "#ffff88"
                }, ANI_SPD, function(){
                
                    setTimeout(function(){
                        ele.animate({
                            backgroundColor: bg
                        }, ANI_SPD, function(){
                            ele.removeClass("highlighting");
                        });
                    }, 1000);
                    
                });
            
            }
        }
        
        flash(title);
    
        return;
    }

    var source;
    var target;
    
    
    
    var x;
    var zoom = _vis.zoom();
    if( evt.group == "nodes" ){
        x = evt.target.x + $("#graph").offset().left - evt.target.size*zoom/2;
    } else {
    	source = _vis.node(evt.target.data.source);
        target = _vis.node(evt.target.data.target);
        
        
        x = evt.mouseX + $("#graph").offset().left;
    }

    
    var y;
    if( evt.group == "nodes" ){
        y = evt.target.y + $("#graph").offset().top;
    } else {
        y = evt.mouseY + $("#graph").offset().top;;
    }
    
    // The tooltip anchor:
    var maxH = 220;
    var maxW = 315;
    var screenHeight = $("html").height();
    var screenWidth = $("html").width();
    var top = (maxH + y + 2.2*outer_padding <= screenHeight);
    var left = (x <= maxW + 2.2*outer_padding);
    
    var position;
    
    if( left ){
        position = "left";
        
        if( evt.group == "nodes" ){
            x += evt.target.size*zoom;
        }
    } else {
        position = "right";
    }
    
    if( top ){
        position += "Top";
    } else {
        position += "Bottom";
    }
    
    var text = "";
    var title;

    if( evt.group == "nodes" ){
    	//console.log("making content for node");
    	
        title =  evt.target.data.symbol;
        
        if( evt.target.data.attribute ){
        	text = $("#networks_tab").find("li[attrid=" + evt.target.data.attributeId + "] .text:first").html();
        } else {
	        var score = "" + $("#genes_tab").find("li[id=gene" + evt.target.data.id + "]").not(".source_true").find(".score_text:first").text();
	        
	        if( score != "" ){
	            title += ' (rank: ' + score + ')';
	        }
	        
	        text = $("#genes_tab").find("li[id=gene" + evt.target.data.id + "] .text:first").html();
        }
    } else {
//    	console.log("making content for edge");
    	
        title = source.data.symbol + ' - ' + target.data.symbol;
  
        var edges = [];
        
        if( merged ){
            edges = evt.target.edges;
        } else {
            edges.push( evt.target );
        }
        //console.log("data for tooltip included edges:");

        for(var i in edges){
            var edge = edges[i];
            text += CytowebUtil._getLinkInfo(edge);
        }
        

    }
    
    // remove old ones
    $("body").children(".qtip").not(".ripped_out").remove();
    $("body").children(".qtip").not(":visible").remove();
     
    // Create the tooltip:
    $("body").qtip({
        content: {
            text: text,
            title: {
                text: '<span class="qtip-title-text">' + title + '</span> <div class="ui-state-default ui-corner-all minimise"> <span class="ui-icon"></span> </div> ',
                button: '<div class="ui-state-default ui-corner-all"> <span class="ui-icon ui-icon-close"></span> </div>'
            }
        },
        position: {
            target: false,
            type: "absolute",
            corner: {
                tooltip: position,
                target: "leftTop"
            },
            adjust: {
                mouse: false,
                x: x,
                y: y,
                scroll: false,
                resize: false
            }
        },
        show: {
            delay: 0,
            when: false,
            effect: { type: "fade", length: 0 },
            ready: true // Show the tooltip when ready
        },
        hide: {
            delay: 0,
            effect: { type: "fade", length: 0 },
            when: { event: "unfocus" }, // Hide when clicking anywhere else
            fixed: true // Make it fixed so it can be hovered over
        },
        style: {
           border: { width: 1, radius: 8 },
           width: {
                min: ( evt.group == "nodes" ? 0 : maxW ),
                max: ( maxW )
           },
           screen: true,
           padding: outer_padding, 
           textAlign: 'left',
           name: 'light', // Style it according to the preset 'cream' style,
           tip: true      // Give it a speech bubble tip with automatic corner detection
        }
    });
    
    $("body").children(".qtip:last").each(function(){
        var qtip = $(this);
        
        qtip.find(".qtip-button").bind("mousedown", function(){
            qtip.qtip("api").beforeHide = function(){

            }
        });
        
        function update_qtip_position(){
            if( !qtip.hasClass("ripped_out") ){
                qtip.qtip("api").updatePosition();
            }
        }
        
        function make_show_more(syn){
            var variance = 20; // px diff before hide
            var show_msg = "&#9658; Show more";
            var hide_msg = "&#9650; Show less";
        
            var syn_link = $('<a href="#" class="action_link">' + show_msg + '</a>');
            var old_syn_height = syn.height();
        
            syn.addClass("short");
                    
            if( old_syn_height - syn.height() > variance ){
                syn.after(syn_link);
                
                syn_link.click(function(){
                    if( syn.hasClass("short") ){
                        syn_link.html(hide_msg);
                    } else {
                        syn_link.html(show_msg);
                    }
                    syn.toggleClass("short");
                    update_qtip_position();
                    return false;
                });
            } else {
                syn.removeClass("short");
            }
        }
        
        if( evt.group == "nodes" ){
            var syn = qtip.find(".synonyms");
            make_show_more(syn);
            
            var descr = qtip.find(".description");
            make_show_more(descr);
            
//            var go_list = qtip.find(".go_list");
//            make_show_more(go_list);
            
            qtip.attr("nodeid", id);
        } else {
            qtip.find(".network").addClass("short").each(function(){
                var net = $(this);
                    
                net.find(".label").bind("click", function(){
                    net.toggleClass("short");
                    update_qtip_position();
                }).bind("mouseover", function(){
                    net.addClass("hover");
                }).bind("mouseout", function(){
                    net.removeClass("hover");
                });
                
                net.find(".per_cent_bar").bind("click", function(){
                    net.toggleClass("short");
                    update_qtip_position();
                }).bind("mouseover", function(){
                    net.addClass("hover");
                }).bind("mouseout", function(){
                    net.removeClass("hover");
                });
                
            });
            
            if( merged ){
            	qtip.attr("medgeid", id);
            } else {
            	qtip.attr("edgeid", id);
            }
            
        }
        
        qtip.find(".minimise").each(function(){
            var open_class = "ui-icon-arrowthickstop-1-n";
            var close_class = "ui-icon-arrowthickstop-1-s";
            var icon = $(this).find(".ui-icon");
            
            icon.addClass(open_class);
            $(this).addClass("ui-state-disabled");
        
            $(this).bind("click", function(){
                
                if( $(this).hasClass("ui-state-disabled") ){
                	return false;
                }
                
                if( icon.hasClass(open_class) ){
                    icon.removeClass(open_class);
                    icon.addClass(close_class);
                    qtip.find(".qtip-content").addClass("collapsed_min");
                } else {
                    icon.removeClass(close_class);
                    icon.addClass(open_class);
                    qtip.find(".qtip-content").removeClass("collapsed_min");
                }
            
                return false;
            }); 
        });
        
        function update_menu(){
            if( $(".qtip:visible").size() > 1 ){
                $("#menu_close_tooltips").removeClass("ui-state-disabled");
            } else {
                $("#menu_close_tooltips").addClass("ui-state-disabled");
            }
        }
        
        qtip.find(".qtip-button").bind("mousedown", function(){
            update_menu();
            removeHighlight();
        });
        
        qtip.qtip("api").beforeHide = function(){
            update_menu();
        }
        
        qtip.draggable({
            containment: "window",
            handle: ".qtip-title",
            cursor: "move",
            start: function(){
                qtip.addClass("ripped_out");
                qtip.find(".qtip-tip").hide();
                qtip.find(".qtip-title .minimise").removeClass("ui-state-disabled");
                
                qtip.qtip("api").beforeHide = function(){
                    if( qtip.hasClass("ripped_out") ){
                        return false;
                    }
                }
                
                qtip.qtip("api").beforePositionUpdate = function(){
                    if( qtip.hasClass("ripped_out") ){
                        return false;
                    }
                }
                
                track("Gene/edge tooltip", "Drag");
            }
        });
        
        function removeHighlight(){
        	qtip.removeClass("mouseover");
        	
        	if( qtip.hasClass("ui-draggable-dragging") ){
        		return; // don't remove highlight while moving qtips
        	}
        	
        	clearTimeout(CytowebUtil._qtipMouseoverTimeout);
        	CytowebUtil._qtipMouseoverTimeout = setTimeout(function(){
    			CytowebUtil.unhighlightGenesAndEdges();
        	}, 100);
        }
        
        function highlight(){
        	if( qtip.hasClass("ui-draggable-dragging") ){
        		return; // don't highlight while moving qtips
        	}
        	
        	qtip.addClass("mouseover");
        	clearTimeout(CytowebUtil._qtipMouseoverTimeout);
        	
        	var id = evt.target.data.id;
        	
        	CytowebUtil._qtipMouseoverTimeout = setTimeout(function(){
        		if( evt.group == "nodes" ){
            		CytowebUtil.highlightGene( id );
            	} else {
            		CytowebUtil.highlightEdge( id );
            	}
        	}, 500);
        	
        }
        
        qtip.find(".qtip-content").bind("mouseenter", function(){
        	highlight();
        });
        
        qtip.bind("mouseleave", function(){
        	removeHighlight();
        });
        
        qtip.qtip("api").updatePosition();
    });
    
};

CytowebUtil._qtipMouseoverTimeout = null;

CytowebUtil._getLinkInfo = function(e){
	//console.log("getting edge info");
	
    var info = '<ul class="network_list tooltip">';
    var c = e.color;
    var w = CytowebUtil._round_weight(e.data['weight']);
    
    var grId = e.data.networkGroupId;
    var grName = $("#networks_widget .checktree_network_group[netid="+grId+"] .name:first").text();
    
    info += '<li class="network_group"><div class="label">' +
            '<div class="per_cent_text"><span>Weight</span></div>' +
            '<div class="network_name">' + grName +'</div>' +
            '</div>' +
            //'<div class="per_cent_bar"> <div class="bar" style="background-color:'+c+';width:'+w+'%">&nbsp;</div> </div>' +
            '<ul style="display: block;">';
        
        var nn = e.data.networkIdToWeight;
        var weight_to_id = {};
        var weights = [];
        
        $.each(nn, function(id, w) {
            if( !weight_to_id[w] ){
            	weight_to_id[w] = [];
            }
            
			weight_to_id[w].push(id);
			weights.push( w );
        });
 
        weights.sort();
        
        for(var i = weights.length - 1; i >= 0; i--){
        	var weight = weights[i];
        	var ids = weight_to_id[weight];
        	
        	for(var j in ids){
        		var id = ids[j];
					var weight_rounded = CytowebUtil._round_weight( weight );
					var network = $("#networks_widget .checktree_network[netid="+id+"]");
					var n =  network.find(".name").text();
					var desc = network.find(".text:first").html();
				
				info += '<li class="network">' +
					'<div class="label">' +
						'<div class="per_cent_text"><span tooltip="Network weight">' + weight_rounded + '</span></div>' + 
						'<div class="network_name">' + n +'</div>' +
					'</div>' +
					'<div class="per_cent_bar"> <div class="bar" style="background-color:'+c+';width:'+weight_rounded+'%">&nbsp;</div> </div>' +
					'<div class="description">' + desc + '</div>' +
				'</li>';
    	}
    }
    				
    
    info += '</ul></li></ul>';
    
    return info;
};
CytowebUtil.generateReport = function(token, params) {
	
	if( !CytowebUtil.checkVersionFor("report") ){
		return;
	}
			
	$("#print_form [name=golegend]").remove();
	$("#print_form").append('<textarea name="golegend">' + $("#go_tab > .header").html() + '</textarea>');
	
	var legend = $('<div></div>');
	var names = [];
	var colors = {};
	$("#networks_widget .checktree_top_level").each(function(){
		var name = $(this).find(".label:first .name").text();
		var color = $(this).find(".bar:first").css("background-color");
		
		names.push(name);
		colors[name] = color;
	});
	
	names.sort();
	$.each(names, function(i, name){
		var color = colors[name];
		legend.append('<div class="entry"> <span class="colour" style="border-left-color: '+ color +'"></span> <span class="name">'+ name +'</span> </div>');
	});
	
	$("#print_form [name=netlegend]").remove();
	$("#print_form").append('<textarea name="netlegend">' + legend.html() + '</textarea>');
	
	var svg = _vis.svg();
	$("#print_form [name=svg]").remove();
	$("#print_form").append('<input type="hidden" name="svg" value=\'' + svg + '\' />');
	
	$("#print_form").submit();

	
};
CytowebUtil.searchWithSelectedGenes = function(addGenes){
	
	var useOnlySelectedGenes = !addGenes;
	
	if(_vis){
		var selected_genes = _vis.selected("nodes");
		var genes_box = $("#gene_area");
		var list = "";
		var addGenes = !useOnlySelectedGenes;
		
		var genes_to_add = [];

		if( addGenes ){
			list = "" + genes_box.val();
			
			// append newline if not already there for new genes
			if( list.charAt( list.length - 1 ) != "\n" ){
				list += "\n";
			}
		}
		
		$.each(selected_genes, function(i, gene){
			if( !gene.data.attribute ){
				genes_to_add.push( gene );
			}
		});
		
		var genes_we_have = {};
		if( addGenes ){			
			$.each( list.split("\n"), function(i, gene){
				genes_we_have[ gene.trim() ] = true;
			} );
		}
		
		$.each(genes_to_add, function(i, gene){
			var name = gene.data.symbol;
			
			if( !genes_we_have[ name ] ){
				list += name + (i < genes_to_add.length - 1 ? "\n" : "");
			}
		});
		
		genes_box.val( list ).blur();
		setTimeout(function(){
			$("#findBtn").click();
		}, 10);
	}
	
};

CytowebUtil.addSelectedGenesToSearch = function(){
	CytowebUtil.searchWithSelectedGenes(true);
};
CytowebUtil._onSelectEdgesChanged = function(scroll) {    
	$("#networks_widget .label").removeClass("selected");
	var edgesArray = _vis.selected("edges");
	
	$.each(edgesArray, function(i, edge) {
		
		$.each(edge.data.networkIdToWeight, function(id, weight){
			var label = $("#network"+ id + " .label");
			label.addClass("selected");
		});
		
		
	});
};

CytowebUtil.selectAll = function(){
	if(_vis){
		var genes = CytowebUtil.genes();
		var gene_ids = [];
		
		$.each(genes, function(i, gene){
			gene_ids.push( gene.data.id );
		});
		
		_vis.select( gene_ids );
	}
};

CytowebUtil.selectNone = function(){
	if(_vis){
		_vis.deselect("nodes");
	}
};

CytowebUtil.selectNode = function(nodeId) {
	if(_vis) {
		_vis.select("nodes", [nodeId]);
	}
};

CytowebUtil.deselectNode = function(nodeId) {
	if(_vis) {
		_vis.deselect("nodes", [nodeId]);
	}
};

CytowebUtil.selectQueryGenes = function(){
	if(_vis){
		var query_nodes = [];
		var nodes = CytowebUtil.nodes();
		
		$.each(nodes, function(i, node){
			if( node.data.queryGene ){
				query_nodes.push(node);
			}
		});
		
		_vis.deselect("nodes");
		_vis.select(query_nodes);
	}
};

CytowebUtil._onSelectNodesChanged = function() {
	if (_vis) {
		var nodes = $.grep(_vis.selected("nodes"), function(node){
			return !node.data.attribute;
		});
		var attrs = $.grep(_vis.selected("nodes"), function(node){
			return node.data.attribute;
		});
		var selected = nodes != null && nodes.length > 0;
		var selectedAttrs = attrs != null && attrs.length > 0;
		var query_nodes = false;
		var all_query_nodes = true;
		var all_nodes = false;
		var no_nodes = true;
		var non_query_nodes = false;
		var vis_nodes = CytowebUtil.genes();
		
		var selected_query_nodes = [];
		$.each(nodes, function(i, node){
			if( node.data.queryGene ){
				query_nodes = true;
				selected_query_nodes.push(node);
			} else {
				non_query_nodes = true;
			}
			
			no_nodes = false;
		});
		
		var vis_query_nodes = [];
		$.each(vis_nodes, function(i, node){
			if( node.data.queryGene ){
				vis_query_nodes.push(node);
			}
		});
		
		all_query_nodes = vis_query_nodes.length == selected_query_nodes.length;
		var all_and_only_query_nodes = all_query_nodes && nodes.length == vis_query_nodes.length;
		
		all_nodes = vis_nodes.length == nodes.length;
		
		if( all_nodes ){
			$("#genes_tab_select_all").addClass("active").siblings().removeClass("active");
		} else if( all_and_only_query_nodes ){
			$("#genes_tab_select_query").addClass("active").siblings().removeClass("active");
		} else if( no_nodes ){
			$("#genes_tab_select_none").addClass("active").siblings().removeClass("active");
		} else {
			$("#genes_tab_select_none").removeClass("active").siblings().removeClass("active");
		}
		
		if( selected ){
			$("#search_with_selected_button").removeClass("ui-state-disabled");
		} else {
			$("#search_with_selected_button").addClass("ui-state-disabled");
		}
		
		if( selected || selectedAttrs ){
			$("#menu_neighbors").removeClass("ui-state-disabled");
		} else {
			$("#menu_neighbors").addClass("ui-state-disabled");
		}
		
		// Enable/disable related menu items:
		if (selected) {
			
			$("#menu_search_selected").removeClass("ui-state-disabled");
			
			$("#menu_add_selected").removeClass("ui-state-disabled");
			$("#menu_remove_selected").removeClass("ui-state-disabled");
		} else {
			
			$("#menu_search_selected").addClass("ui-state-disabled");
			$("#menu_add_selected").addClass("ui-state-disabled");
			$("#menu_add_selected").addClass("ui-state-disabled");
			$("#menu_remove_selected").addClass("ui-state-disabled");
		}
	}
};
CytowebUtil._shift_down = false;

$(function(){
	$("body").bind("keydown", function(e){
	    CytowebUtil._shift_down = e.shiftKey;
	}).bind("keyup", function(e){
	    CytowebUtil._shift_down = e.shiftKey;
	});
});CytowebUtil.updateNetworksTab = function() {
//console.log("update networks tab");
	
	// Get the networks legend:
	var legend = {/*color->networkGroupId*/};
	var edges = CytowebUtil.edges();
	var networksList = {};
	
	if (edges) {
		$.each(edges, function(i, e) {
			// Considering only one data attribute per color in the mapper!
			legend[e.color] = e.data['networkGroupId'];
			
			for(var id in e.data.networkIdToWeight){
				networksList[id] = true;
			}
		});
	}
//console.log(networksList);
	
	// Set network colors:
	for (var netColor in legend) {
	    var netId = legend[netColor];
		$("#networks_widget > .checktree_network_group[netid="+ netId +"]  .per_cent_bar > .bar").css('background-color', netColor);
    }
	// Greyed out networks with no correspondent edges:
	var spacer = '<div class="checkbox_spacer not_found">&nbsp;</div>';
	
	$("#networks_widget .checktree_network").each(function(){
		var id = $(this).attr("netid");
		
		if ( !networksList[id] ) {
			$(this).find(".bar:first").addClass("disabled");
			$(this).find(".checkbox").remove();
			$(this).find("input[type=checkbox]").remove();
			$(this).find(".arrow:first").after(spacer);
		}
	});

	$("#networks_widget .checktree_network_group").each(function(){
		if ($(this).find(".checktree_network .checkbox").length === 0) {
			// No enabled network for this network group:
			$(this).find(".bar:first").addClass("disabled");
			$(this).find(".checkbox").remove();
			$(this).find("input[type=checkbox]").remove();
			$(this).find(".arrow:first").after(spacer);
		}
	});
	
	var tooltip = $.i18n("networks_tab.not_found.tooltip");
	$("#networks_widget .not_found").attr("tooltip", tooltip);
	$("#networks_widget .not_found").parent().find(".per_cent_bar div").attr("tooltip", tooltip);
};

CytowebUtil._round_weight = function(w){
    w = Math.round(w*1000)/10;
    
    if( (w * 10) % 10 == 0 ){
        w += ".0";
    }
    
	w = w < 0.1 ? "&lt; 0.1" : w;
	
	return w;
};

CytowebUtil.body_click = function(){
    $("body").trigger("click");
};

CytowebUtil.visualization = function() {
	return _vis;
};


CytowebUtil._nodesById = null;

CytowebUtil.getNodeById = function(id) {
	if (CytowebUtil._nodesById == null) {
		CytowebUtil._nodesById = {};
		var nodes = CytowebUtil.nodes();
		$.each(nodes, function(i, n) {
			CytowebUtil._nodesById[n.data.id] = n;
		});
	}
	return CytowebUtil._nodesById[id];
};CytowebUtil.neighborsHighlighted = false;
CytowebUtil.geneHighlighted = false;
CytowebUtil.highlightedGenes = {};
CytowebUtil.highlightedGeneEdges = {};

CytowebUtil.highlightGene = function(gene_id){
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.highlightedGenes[gene_id] = true;
	CytowebUtil.geneHighlighted = true;
	
	var fn = _vis.firstNeighbors( [_vis.node(gene_id)] , true);
	var neighbors = fn.neighbors;
	var edges = fn.edges;
	
	CytowebUtil.highlightedGeneEdges = {};
	$.each(edges, function(i, e){
		CytowebUtil.highlightedGeneEdges[e.data.id] = true;
	});
	
	CytowebUtil.halfHighlightedGenes = {};
	$.each(neighbors, function(i, n){
		CytowebUtil.halfHighlightedGenes[n.data.id] = true;
	});
	
	CytowebUtil.updateBypass();
};

CytowebUtil.unhighlightGene = function(gene_id){
	if( gene_id != null ){
		CytowebUtil.highlightedGenes[gene_id] = false;
		CytowebUtil.highlightedGeneEdges = {};
		CytowebUtil.halfHighlightedGenes = {};
	} else {
		CytowebUtil.highlightedGenes = {};
		CytowebUtil.highlightedGeneEdges = {};
		CytowebUtil.halfHighlightedGenes = {};
		CytowebUtil.geneHighlighted = false;
	}
	
	CytowebUtil.updateBypass();
};

CytowebUtil.highlightEdge = function(edge_id){
	CytowebUtil.geneHighlighted = true;

	var edge = _vis.edge(edge_id);
	var src_id = edge.data.source;
	var tgt_id = edge.data.target;
	
	CytowebUtil.halfHighlightedGenes = {};
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.halfHighlightedGenes[src_id] = true;
	CytowebUtil.halfHighlightedGenes[tgt_id] = true;
	
	CytowebUtil.highlightedGeneEdges = {};
	CytowebUtil.highlightedGeneEdges[edge_id] = true;
	
	CytowebUtil.updateBypass();
};

CytowebUtil.unhighlightGenesAndEdges = function(){
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.highlightedGeneEdges = {};
	CytowebUtil.halfHighlightedGenes = {};
	CytowebUtil.geneHighlighted = false;
	
	CytowebUtil.updateBypass();
}

CytowebUtil.unhighlightEdge = function(edge_id){
	if( edge_id != null ){
		CytowebUtil.highlightedGenes = {};
		CytowebUtil.highlightedGeneEdges[edge_id] = false;
		CytowebUtil.halfHighlightedGenes = {};
	} else {
		CytowebUtil.highlightedGenes = {};
		CytowebUtil.highlightedGeneEdges = {};
		CytowebUtil.halfHighlightedGenes = {};
		CytowebUtil.geneHighlighted = false;
	}
	
	CytowebUtil.updateBypass();
};

CytowebUtil.highlightNetworkGroup = function(group_id){
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.highlightedGeneEdges = {};
	CytowebUtil.halfHighlightedGenes = {};
	CytowebUtil.geneHighlighted = true;
	
	var edges = CytowebUtil.edges();
	$.each(edges, function(i, e){
		if( e.data.networkGroupId == group_id ){
			CytowebUtil.highlightedGeneEdges[e.data.id] = true;
			CytowebUtil.halfHighlightedGenes[e.data.source] = true;
			CytowebUtil.halfHighlightedGenes[e.data.target] = true;
		}
	});
	
	CytowebUtil.updateBypass();
};

CytowebUtil.unhighlightNetworkGroup = function(){
	CytowebUtil.unhighlightGenesAndEdges();
};

CytowebUtil.highlightNetwork = function(net_id){
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.highlightedGeneEdges = {};
	CytowebUtil.halfHighlightedGenes = {};
	CytowebUtil.geneHighlighted = true;
	
	var edges = CytowebUtil.edges();
	$.each(edges, function(i, e){
		if( e.data.networkIdToWeight[net_id] != null ){
			CytowebUtil.highlightedGeneEdges[e.data.id] = true;
			CytowebUtil.halfHighlightedGenes[e.data.source] = true;
			CytowebUtil.halfHighlightedGenes[e.data.target] = true;
		}
	});
	
	CytowebUtil.updateBypass();
};

CytowebUtil.unhighlightNetwork = function(){
	CytowebUtil.unhighlightGenesAndEdges();
};

CytowebUtil.highlightAttributeGroup = function(group_id){
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.highlightedGeneEdges = {};
	CytowebUtil.halfHighlightedGenes = {};
	CytowebUtil.geneHighlighted = true;
	
	var edges = CytowebUtil.edges();
	$.each(edges, function(i, e){
		if( e.data.attributeGroupId == group_id ){
			CytowebUtil.highlightedGeneEdges[e.data.id] = true;
			CytowebUtil.halfHighlightedGenes[e.data.source] = true;
			CytowebUtil.halfHighlightedGenes[e.data.target] = true;
		}
	});
	
	CytowebUtil.updateBypass();
};

CytowebUtil.unhighlightAttributeGroup = function(){
	CytowebUtil.unhighlightGenesAndEdges();
};

CytowebUtil.highlightAttribute = function(attr_id){
	CytowebUtil.highlightedGenes = {};
	CytowebUtil.highlightedGeneEdges = {};
	CytowebUtil.halfHighlightedGenes = {};
	CytowebUtil.geneHighlighted = true;
	
	var edges = CytowebUtil.edges();
	$.each(edges, function(i, e){
		if( e.data.attributeId == attr_id ){
			CytowebUtil.highlightedGeneEdges[e.data.id] = true;
			CytowebUtil.halfHighlightedGenes[e.data.source] = true;
			CytowebUtil.halfHighlightedGenes[e.data.target] = true;
		}
	});
	
	CytowebUtil.updateBypass();
};

CytowebUtil.unhighlightAttribute = function(){
	CytowebUtil.unhighlightGenesAndEdges();
};

CytowebUtil.mergeEdges = function(value) {
	if (value === CytowebUtil.OPTIONS.edgesMerged) return;
	
	if(_vis) {
		CytowebUtil.OPTIONS.edgesMerged = value;
		_vis.edgesMerged(value);
	}
};

CytowebUtil.showNodeLabels = function(value) {
	if (value === CytowebUtil.OPTIONS.nodeLabelsVisible) return;

	if(_vis) {
		CytowebUtil.OPTIONS.nodeLabelsVisible = value;
		_vis.nodeLabelsVisible(value);
	}
};

CytowebUtil.showPanZoomControl = function(value) {
	if (value === CytowebUtil.OPTIONS.panZoomControlVisible) return;
	
	if(_vis) {
		CytowebUtil.OPTIONS.panZoomControlVisible = value;
		_vis.panZoomControlVisible(value);
	}
};

CytowebUtil.transparentEdges = function(value) {
	if(_vis) {
		var style = _vis.visualStyle();
		
		if( value ){
			style.edges.opacity = CytowebUtil.VISUAL_STYLE_OPACITY;
		} else {
			style.edges.opacity = CytowebUtil.DEF_EDGE_OPACITY;
		}

		_vis.visualStyle(style);
	}
};

CytowebUtil.showPublicationLabels = function(value){
	if( _vis ){
		var style = _vis.visualStyle();
		
		if( value ){
			style.nodes.labelGlowOpacity = 0;
			style.nodes.labelFontColor = "#000000";
			style.nodes.labelVerticalAnchor = "bottom";
		} else {
			style.nodes.labelGlowOpacity = 1;
			style.nodes.labelFontColor = "#ffffff";
			style.nodes.labelVerticalAnchor = "middle";
		}

		_vis.visualStyle(style);
	}
};$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
	$.browserupdate({
		siteName: "GeneMANIA",
		browsers: {
			chrome: {
				name: "Chrome",
				version: 15,
				url: "http://www.google.com/chrome"
			},
			firefox: {
				name: "Firefox",
				version: 8,
				url: "http://firefox.com"
			},
			safari: {
				name: "Safari",
				version: 5.1,
				url: "http://www.apple.com/safari/download/"
			},
			msie: {
				name: "Internet Explorer",
				version: 9,
				url: "http://windows.microsoft.com/en-US/internet-explorer/products/ie/home"
			}
		}
	});
	
});$(function(){
   // debug box
    $("#debug_info").dialog({
    	autoOpen: false,
    	title: "Debug information",
    	width: 400,
    	height: 400
    });
    
    $("#debug_info .tabs").tabs();
    
    var timeout;
    var count = 0;
    var times = 3;
    var time = 1000;
    
   
    function toggle_debug(){
    	count = 0;
    	clearTimeout(timeout);
    	
    	if( $("#debug_info").is(":visible") ){
			$("#debug_info").dialog("close");
		} else {
			$("#debug_info").dialog("open");
		}
    }
	
	$("html").bind("keydown", function(e){
		if( e.which == 192 && e.shiftKey ){ // backtab with shift`
			toggle_debug();			
		} else if( e.which == 192 ){
			count++;

			if( count == 1 ){
				clearTimeout(timeout);
				timeout = setTimeout(function(){
					count = 0;
				}, time);
			} else if( count >= times ){
				toggle_debug();
			}
		}
	});
	
	$("#debug_info_networks button").click(function(){
		var networks = $("#debug_info_networks textarea").val().split("\n");
		var ids = [];
		
		$.each(networks, function(i, network){
			var id = parseInt(network);
			
			if( !isNaN(id) ){
				ids.push(id);
			}
		});
		
		if( $(".query_network_checkbox[value=" + ids[0] + "]").size() == 0 ){
			$(this).after(' <span class="done_message">Networks set</span>');
			return;
		}
		
		$(".query_network_checkbox").attr("checked", null);
		
		$.each(ids, function(i, id){
			$(".query_network_checkbox[value=" + id + "]").attr("checked", true);
    	});
	    	
	 
    	updateParentChecks();
    	refreshAllGroupCounts();
    	
    	$(this).siblings(".done_message").remove();
		$(this).after(' <span class="done_message">Networks set</span>');
		var input = $(this);
		setTimeout(function(){
			input.siblings(".done_message").fadeOut(500);
		}, 2000);
	});
});$(function(){
	if( $("#error_page").size() == 0 ){
		return;
	}
	
	$("body").bind("keydown", function(e){
		if( e.which == 192 && e.shiftKey ){ // backtab ` or ~
			
			$(".error_details").toggle();
			
		}
	});
	
	if( $("#error_page").hasClass("email") ){
	
		var msg = "An exception was caught by the GeneMANIA web application.  It is logged below.\n\n";
		
		var i = 0;
		$(".error_details").children().each(function(){
			
			if( i % 2 == 0){
				msg += "====\n";
			}
			
			msg += $(this).text() + "\n";
			i++;
		});
		
		$.ajax({
			type: "POST",
			url: absoluteUrl("json/mail"),
			data: { 
				subject: "GeneMANIA exception: " + $(".error_details").attr("exception"),
				message: msg
			},
			success: function(){
				//console.log("sent email");
				$("#loading_message").attr("class", "").addClass("sent");
			},
			error: function(){
				//console.log("could not send mail");
				$("#loading_message").attr("class", "").addClass("error");
			}
		});
		//console.log("error js");
	
	}
});/*
 * Google Analytics - Asynchronous Tracking 
 * http://code.google.com/apis/analytics/docs/tracking/asyncTracking.html
 */
var _gaq = _gaq || [];
_gaq.push(['_setAccount', '${GA_ACCT_NR}']);
_gaq.push(['_trackPageview']);
_gaq.push(['_setVar', window.navigator.userAgent]);

(function() {
    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
    (document.getElementsByTagName('head')[0] || document.getElementsByTagName('body')[0]).appendChild(ga);
})();$(function(){
	
	if( $("#results_page").size() == 0 ){
		return; // ignore this js file if on front page
	}
	
    /**********************************************
     * GENES TAB
     **********************************************/

    $("#genes_tab .checktree").checkTree({
        labelAction: "select",
        collapsedText: "&nbsp;",
        expandedText: "&nbsp;",
        onExpand: function(li){ update_sort_and_expand_by_arrow_state(); },
        onCollapse: function(li) { update_sort_and_expand_by_arrow_state(); }
    });
    
    $("#genes_tab .label").click(function(){
    	var nodeId = $(this).parent("li").attr("id").substring("gene".length);

    	if ($(this).hasClass("selected")) {
    		CytowebUtil.selectNode(nodeId);
    		track("Genes", "Select entry");
    	} else {
    		CytowebUtil.deselectNode(nodeId);
    		track("Genes", "Deselect entry");
    	}
    });
    
    var show_timeout;
    var clear_timout;
    var time = 150;
    
    $("#genes_tab .label").bind("mouseover", function(){
        var nodeId = $(this).parent("li").attr("gene");
        clearTimeout(clear_timout);
        
        show_timeout = setTimeout(function(){
            CytowebUtil.highlightGene(nodeId);
        }, time);
        
    }).bind("mouseout", function(e){
        var nodeId = $(this).parent("li").attr("gene");
        clearTimeout(show_timeout);
        
        clear_timout = setTimeout(function(){
        	CytowebUtil.unhighlightGene();
        }, time);
    });
	
	// TODO remove if we decide to show source gene scores
	// increase source genes score so sort by score puts them at the top
	// (the number isn't shown anyway)
	$("#genes_tab .score_text.source_score_true").each(function(){
	    var int_val = parseInt( $(this).text() );
	    
	    $(this).text( int_val + 1 );
	});
	
    $("#genes_tab_expand_all").click(function(){
        $("#genes_widget").modifyCheckTree({expandAll: true});
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Genes", "Expand", "All");
        return false;
    });
    
    $("#genes_tab_expand_none").click(function(){
        $("#genes_widget").modifyCheckTree({expandNone: true});
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Genes", "Expand", "None");
        return false;
    });
    
    $("#genes_tab_sort_by_name").click(function(){
        $("#genes_widget").listSort({
           value: ".gene_name:first",
           descending: false
       });
       $(this).addClass("active").siblings().removeClass("active");
       
       track("Genes", "Sort", "By name");
       return false;
    }).click();
    
    $("#genes_tab_sort_by_score").click(function(){
        $("#genes_widget").listSort({
            value: ".source_score_true",
            descending: true
        });
        $("#genes_widget").listSort({
        	value: ".score_text:first",
        	descending: $("#genes_tab .score_text").attr("score") == null
        });
        
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Genes", "Sort", "By score");
        return false;
    }).click();
    
    
    
    // TODO: remove when we support p-values
    
    // ranks of result genes
    var rank = 0;
    var prevScore = null;
    $("#genes_widget .source_false .score_text").each(function(i){ 
    	var $ele = $(this);
    	var score = parseFloat( $ele.text() );
    	
    	if( score === prevScore ){ 
    		// then use the same rank as before
    	} else { 
    		rank++; // otherwise, we're the next in line
    	}
    	
    	$ele.attr("score", score);
    	$ele.html( rank );
    });
    
    
    
    // ranks of query genes are  0
    $("#genes_widget .source_true .score_text").each(function(i){
    	var $ele = $(this);
    	var score = parseFloat( $ele.text() );
    	var rank = 0;
    	
    	$ele.attr("score", score);
    	$ele.html( rank );
    });
    
	$("#genes_tab .arrow").bind("mousedown", function(){
		if( !$(this).hasClass("expanded") ){
			track("Genes", "Expand", "Arrow open");
		} else {
			track("Genes", "Expand", "Arrow close");
		}
	});
    
    function update_sort_and_expand_by_arrow_state(){
        if( $("#genes_tab .arrow.collapsed").size() == 0 ){
            $("#genes_tab_expand_all").addClass("active").siblings().removeClass("active");
        } else if( $("#genes_tab .arrow.expanded").size() == 0 ){
            $("#genes_tab_expand_none").addClass("active").siblings().removeClass("active");
        } else {
            $("#genes_tab_expand_none").add("#genes_tab_expand_all").removeClass("active");
        }
    }
    update_sort_and_expand_by_arrow_state();
    
    function empty_description(ele){
        return ele.text().match(/\s*[Nn]\/[Aa]\s*/);
    }
    
    $("#genes_tab .mini_description").each(function(){
        if( empty_description($(this)) ){
            $(this).hide();
        }
    });
    
    $("#genes_tab .description").each(function(){
        if( empty_description($(this)) ){
            $(this).html('<span class="ui-icon ui-icon-info"></span> No description is available.');
        }
    });
    
    $("#search_with_selected_button").click(function(){
    	
    	if( !$(this).hasClass("ui-state-disabled") ){
    		CytowebUtil.searchWithSelectedGenes();
    	}
    	
    });
    
    $("#genes_tab_select_all").click(function(){
    	CytowebUtil.selectAll();
    	return false;
    });
    
    $("#genes_tab_select_none").click(function(){
    	CytowebUtil.selectNone();
    	return false;
    });
    
    $("#genes_tab_select_query").click(function(){
    	CytowebUtil.selectQueryGenes();
    	return false;
    });
    
    
    
    progress("genes");
    
});
$(function(){
	if( $("#results_page").size() == 0 ){
		return; // ignore this js file if on front page
	}

    // use n colours max where n is the number of colours defined here
    var colours = [ "#c26661", "#639bc7", "#e9ae3d", "#907cc7", "#6fc17c", "#d28bcc", "#8dcbc3" ];
    
    var query_colour = "#333333";
    var full_colour = "#000000";
    var border_colour = "#000000";
    
    function set_default_colours(){
        $("#go_tab").attr("colour", full_colour);
    }
    
    $("#go_tab").attr("querycolour", query_colour);
    
    function update_next_colours(){
        $("#go_tab").attr("colour", colours[0]);
    }
    update_next_colours();
    
    var NO_COLOURS_MSG = "<p>No colours have been added.</p>";

    $("#go_tab .content table tr").append('<td class="add_button"></td>');
    
    $("#go_tab .sort_list").append(NO_COLOURS_MSG);
    
    // convert to lower case
    $("#go_tab .content table tbody .annotation").each(function(){
        $(this).html( ("" + $(this).text()).replace("_", " ") );
        $(this).attr( "value", $(this).text() );
    });
    
    // convert to lower case
    $("#go_tab .content table tbody .pval").each(function(){
        $(this).html( $(this).text().toLowerCase() );
    });
    
    // make value sort by numerator
    $("#go_tab .content table tbody .coverage").each(function(){
        
        var val = ("" + $(this).attr("value")).match(/^(\d+)\/(\d+)$/);
        
        if(val){
            var top = parseFloat( val[1] );
            var bot = parseFloat( val[2] );
            $(this).attr("value", top);
        }
    });
    
    $("#go_tab .sort_list").disableSelection().sortable({
        sort: function(){
            update_priorities();
        },
        start: function(){
            $("#go_tab .overlay").show();
        },
        stop: function(){
            $("#go_tab .overlay").hide();
            update_legend();
            update_graph_colouring();
            $("#go_tab .colouring.hover").trigger("mouseup");
        },
        axis: "y",
        containment: "parent",
        items: ".colouring",
        tolerance: "pointer"
    });
    
    $("#go_tab tr").bind("click", function(e){
    	var tr = $(this);
    	var button = $(this).find(".button");
    	var targetIsButton = $(e.target).parents().add(e.target).is(button);
    	
    	if( !targetIsButton ){
    		button.click();
    	}
    });

    function update_priorities(){
        $("#go_tab .sort_list .colouring").not(".ui-sortable-helper").each(function(){
            var ones_before = $(this).prevAll(".colouring").not(".ui-sortable-helper").size();
            var ordinal = ones_before + 1;
            $(this).find(".priority").html( ordinal );
        });
    }
    
    function update_graph_colouring(){
        CytowebUtil.updateGO();
    }
    
    function update_go_list_colouring(){
        CytowebUtil.updateGOColoursForGenes();
    }
    
    var legend_update_semaphore = 0;
    function update_legend(){

        if( $("#go_legend").is(":visible") ){
            build_legend();
        } else {
        
            var i = ++legend_update_semaphore;
            $("#menu_go_legend").one("click", function(){
                if( i == legend_update_semaphore ){
                    build_legend();
                    legend_update_semaphore = 0;
                }
            });
        }
        
        function build_legend(){
            $("#go_legend .content").children().not(".instructions").remove();
            
            if( $("#go_tab .colouring").size() == 0 ){
                $("#go_legend .instructions").show();
            } else {
                $("#go_legend .instructions").hide();
                $("#go_legend .content").append(' <table> <tbody></tbody> </table> ');
            }
            
            $("#go_tab .colouring").each(function(){
                
                var colour = $(this).attr("colour");
                
                var entry = $('<tr>\
                    <td><div class="legend_square"></div></td>\
                    <td>' + $(this).find(".annotation").text() + '</td>\
                    </tr>');
                
                entry.find(".legend_square").css({
                    backgroundColor: colour,
                    borderColor: colour
                });
                
                $("#go_legend .content table tbody").append(entry);
                
                
            });
        }
    }
    
    var colouring_mouse_down = false;
    function add_to_colouring(tr){
        if( $("#go_tab .colouring").size() == 0 ){
            $("#go_tab .sort_list").empty(); // remove help msg if necessary
        }
        
        var is_query_genes = tr.hasClass("query");
        var colour = ( is_query_genes ? query_colour : colours.shift() );
        var button = tr.find(".button");
        
        var ocid = tr.find(".annotation").attr("ocid");
        var colouring = $('<div class="colouring' + (is_query_genes ? ' query' : '') + '" colour="' + colour + '" ocid="' + ocid + '">\
                                <div class="grip" tooltip="Click and drag to reorder in the functions coloring list."> <span class="ui-icon ui-icon-grip-solid-horizontal"></span> </div>\
                                <div class="reorder_icon"> <span class="ui-icon ui-icon-grip-solid-horizontal"></span> </div>\
                                <div class="colour"></div>\
                                <div class="priority"></div>\
                                <div class="annotation" ocid="' + ocid + '">' + tr.find(".annotation").text() + '</div>\
                                <div class="ui-state-default ui-corner-all button" tooltip="Click to remove from the functions coloring list."> <span class="ui-icon ui-icon-minus"></span> </div>\
                        </div>');
        colouring.find(".colour").css({
            "background-color": colour,
            "border-color": colour
        });
        $("#go_tab .sort_list").prepend(colouring);
        
        button.css({
        	background: colour
        });
        
        function inside_colouring(e){
            return $(e.toElement).parents(".colouring").attr("ocid") == ocid;
        }

        
        var mouse_out = true;

        colouring.bind("mouseover", function(e){
        
            if( mouse_out ){        
                if( !colouring_mouse_down ){                
                    colouring.addClass("hover");
                    
                    $("#go_tab .content table td[ocid=" + ocid + "]").trigger("mouseover");
                }
                mouse_out = false;
            }
        }).bind("mousedown", function(e){
            colouring_mouse_down = true;
        }).bind("mouseup", function(e){
            colouring_mouse_down = false;
            
            update_priorities();
            
            if( mouse_out ){
                colouring.removeClass("hover");
                

                $("#go_tab .content table td[ocid=" + ocid + "]").trigger("mouseout");
            }
        }).bind("mouseout", function(e){
            if( !mouse_out && !inside_colouring(e) ){
                mouse_out = true;
                
                update_priorities();
                
                if( !colouring_mouse_down ){
                    colouring.removeClass("hover");
                    
                    $("#go_tab .content table td[ocid=" + ocid + "]").trigger("mouseout");
                }
            }
        });

        colouring.find(".button").bind("click", function(){
            tr.find(".button").click();
            $("#go_tab .content table td[ocid=" + ocid + "]").trigger("mouseout");
        });
        
        if( colours.length == 0 ){
            disable_buttons();
            set_default_colours();
        }
        
        update_next_colours();
        update_priorities();
        update_legend();
        resize_sidebar();
        update_graph_colouring();
        update_go_list_colouring();
        
        track("Functions", "Add colouring");
    }
    
    function remove_from_colouring(tr){
        $("#go_tab .sort_list").find(".annotation").each(function(){
            if( $(this).attr("ocid") == tr.find(".annotation").attr("ocid") ){
                if( !tr.hasClass("query") ){
                    var colouring = $(this).parent(".colouring");
                    colours.unshift( colouring.attr("colour") );
                }
                $(this).parent(".colouring").remove();
            }
        });
        
        if( $("#go_tab .sort_list").children().size() == 0 ){
            $("#go_tab .sort_list").append(NO_COLOURS_MSG); // add help msg
        }
        
        tr.find(".button").attr("style", "");
        
        if( !tr.hasClass("query") ){
            update_next_colours();
            enable_buttons();
        }
        
        update_priorities();
        update_legend();
        resize_sidebar();
        update_graph_colouring();
        update_go_list_colouring();
        
        track("Functions", "Remove colouring");
    }
    
    function resize_sidebar(){
        $(window).trigger("resize");
    }
    
    function disable_buttons(){
        $("#go_tab .content .button").not(".ui-state-active").addClass("ui-state-disabled");
        $("#go_tab tr.query .button").removeClass("ui-state-disabled");
    }
    
    function enable_buttons(){
        $("#go_tab .content .button").removeClass("ui-state-disabled");
    }
    
    $("#go_tab .content table > tbody > tr > td.add_button").each(function(){
        var button = $('<div tooltip="Click to add to or remove from the functions coloring list." class="ui-state-default ui-corner-all button"><span class="ui-icon ui-icon-plus"></span></div>');
        var tr = $(this).parent();
        
        $(this).append(button);
        
        button.bind("click", function(){
            if( $(this).hasClass("ui-state-disabled") ){
                return;
            }
        
            if( ! $(this).hasClass("ui-state-active") ){
                $(this).addClass("ui-state-active");
                add_to_colouring(tr);
            } else {
                $(this).removeClass("ui-state-active");
                remove_from_colouring(tr);
            }
            
            $(this).parent().trigger("mouseout");
        });
        
    });
    
    $.tablesorter.addParser({ 
        // set a unique id 
        id: 'exponent', 
        is: function(s) { 
            return ("" + s).match( /[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?/ ) != null;
        }, 
        format: function(s) { 
            return s;
        }, 
        type: 'numeric' 
    }); 
    
    $("#go_tab .content table").tablesorter({
        textExtraction: function(node){
            var ele = $(node);
            var val = "" + ele.attr("value");
            return val;
        }
    });
    
    $("#go_tab .content table .header.pval").click();
    
    var timeout = undefined;
    var ctimeout = undefined;
    var go_id = undefined;
    
    $("#go_tab tbody tr").each(function(){
    
        $(this).find(".annotation, .pval, .coverage, .add_button").each(function(){
            
            var time = 200;
            var ele = $(this);
            var anno = $(this).parent().find(".annotation");
            var ocid = anno.attr("ocid");
            var tr = $(this).parent();
            var is_query_genes = tr.hasClass("query");
            
            function same_row(e){
                if( !e.toElement || !e.fromElement ){
                    return false;
                }
                
                function get_ocid(ele){
                    var anno = $(ele).parents("tr:first").find(".annotation");
                    return anno.attr("ocid");
                }
                
                var ocid_to = get_ocid(e.toElement);
                var ocid_from = get_ocid(e.fromElement);
                
                if( ocid_to == undefined || ocid_from == undefined ){
                    return false;
                }
                
                return ocid_to == ocid_from;
            }
            
            $(this).bind("mouseover", function(e){
                if( !same_row(e) ){
                    
                    var colouring = $("#go_tab .colouring[ocid=" + ocid + "]");
                    var colour;
                
                    if( is_query_genes ){
                        colour = $("#go_tab").attr("querycolour");
                    } else if( colouring.size() > 0 ){
                        colour = colouring.attr("colour");
                    } else {
                        colour = $("#go_tab").attr("colour");
                    }
                
                    var bg_colour = $.Color(colour).toHSL().modify([null, null, 0.8]).toHEX();
                    
                    go_id = ocid;
                    tr.addClass("preview").children().css({
                        backgroundColor: bg_colour //, 
                        //borderColor: border_colour
                    });
                    
                    if( tr.prev().size() > 0 ){
//                        tr.prev().addClass("before_preview").children().css({
//                            borderBottomColor: border_colour
//                        });
                    } else {
//                        tr.parents("table:first").find("thead tr").addClass("before_preview").children().css({
//                            borderBottomColor: border_colour
//                        });
                    }
                    timeout = setTimeout(function(){
                        if( go_id == ocid ){
                            CytowebUtil.highlightGO( ocid, is_query_genes );
                            timeout = undefined;
                        }
                    }, time);
                    clearTimeout(ctimeout);
                }
            }).bind("mouseout", function(e){
                
                if( !same_row(e) ){
                
                    go_id = undefined;
                    tr.removeClass("preview").children().css({
                        backgroundColor: "",
                        borderColor: ""
                    });
                    tr.parents("table:first").find(".before_preview").removeClass("before_preview").children().css({
                        backgroundColor: "",
                        borderColor: ""
                    });
                    
                    ctimeout = setTimeout(function(){
                        
                        if( go_id != ocid ){
                            CytowebUtil.unhighlightGO( ocid, is_query_genes );
                        }
                    }, time/2);
                    clearTimeout(timeout);
                }
            });
        });
    });    
    
    $("#go_tab .content").append('<div class="overlay"></div>');
    $("#go_tab .content .overlay").hide();
    
    
    progress("ontology");
    
    
});$(function(){

	if( $("#results_page").size() == 0 ){
		return; // ignore this js file if on front page
	}

	$.menu_popup = {};
	
	// Store the created panels:
	$.menu_popup.panels = { /* menu_obj -> panel_id */ }
    
    $.menu_popup.defaults = {
        hiddenToolTip: "Use the toggle button above to restore the panel when hidden",
        title: "",
        content: "",
        resizable: false,
        minHeight: 80,
        minWidth: 120,
        maxHeight: 520,
        maxWidth: 400,
        onResize: undefined, // function
        onStartResize: undefined, // function
        onEndResize: undefined // function
    };
	
	$.fn.menuPopup = function(id, options) {
		options = $.extend({}, $.menu_popup.defaults, options);
		var menu_item = $(this);
		var popup = $("#"+id);

		if (popup.length === 0) {
			popup = $('<div id="'+id+'" class="menu_popup outline over_flash">' +
					      '<div class="ui-widget-header title ui-corner-top">' +
					          '<div class="floating normal_width button close ui-state-default ui-corner-all"><a href="#"><span class="ui-icon ui-icon-close"></span></a></div>' +
					      '<label>'+options.title+'</label></div>' +
					      '<div class="content"></div>' +
					  '</div>');
			
			$("body").append(popup);
			
			popup.find(".content").append(options.content);
			
			popup.find(".close a").click(function(){
	        	menu_item.click();
	        	return false;
	        });
			
			popup.draggable({
//		        handle: '.title',
		        cursor: 'auto',
		        scroll: false,
		        //containment: 'window'
		        stop: function(){

		            var x1 = popup.offset().left;
		            var x2 = x1 + popup.width();
		            var y1 = popup.offset().top;
		            var y2 = y1 + popup.height();
		            
		            var maxX = $("html").width();
		            var maxY = $("html").height();
		            
		            if(0 < x1 && x2 < maxX && 0 < y1 && y2 < maxY) {
		                // in bounds
		                $.tooltip.hide(toolTipOptions);
		            } else {
		                // out of bounds
		                $.tooltip.show($.extend({
		                    message: options.hiddenToolTip,
		                    object: menu_item,
		                    arrowAlign: "left",
		                    positionAtCursor: false,
		                    delayAfterHover: 200,
		                    showLength: 10000
		                }, toolTipOptions));
		            }
		        }
		    });
		    
			popup.bind("mouseover", function(){
				$(this).css("z-index", 99999);
			}).bind("mouseout", function(){
				$(this).css("z-index", "");
			});
			
			if (options.resizable) {
				popup.resizable({ 
					mimWidth: options.minWidth, 
					minHeight: options.minHeight,
					maxWidth: options.maxWidth, 
					maxHeight: options.maxHeight,
					start: function(evt, ui) {
					    if (options.onStartResize) {
					    	options.onStartResize(evt, ui);
					    }
				    },
					resize: function(evt, ui) {
				    	var content = $(this).find(".content");
				    	var h = $(this).height() - $(this).find(".ui-widget-header").height();
				    	content.css("height", h);
				    	
					    if (options.onResize) {
					    	options.onResize(evt, ui);
					    }
				    },
					end: function(evt, ui) {
					    if (options.onEndResize) {
					    	options.onEndResize(evt, ui);
					    }
				    }
               });
			}
		}
		
		popup.show().each(function() {
			$(this).css( "top", menu_item.offset().top + menu_item.outerHeight() );
			$(this).css( "left", menu_item.offset().left );
		})
	};
    
});$(function(){	
	
    /**********************************************
    disable native language spellcheck
    otherwise, when we add one, there could be
    two underlines at once
    
    NOTE: affects only items with "widget" css
    class
    **********************************************/			
    // firefox fix
    $(".widget").attr("spellcheck", false);
    
    
});$(function(){
    
	if( $("#results_page").size() == 0 ){
		return; // ignore this js file if on front page
	}
    
    /**********************************************
    menus
    **********************************************/
	
	
    $(".menu").menu({
        onMenuItemOpen: function(li){
            if( li.attr("id") == "menu_search" ) {
                //$("#menu_search_input").width( $("#menu_search_input").parent().width() * 0.95 );
            }
        },
        
        onMenuItemCheck: function(li){
        	if(li.hasClass("ui-state-disabled")){
        		return;
        	}
        	
            switch( li.attr("id") ) {
            case "menu_show_labels":
            	CytowebUtil.showNodeLabels(true);
            	track("Visualisation", "Show labels");
                break;
            case "menu_publication_labels":
            	CytowebUtil.showPublicationLabels(true);
            	track("Visualisation", "Enable publication labels");
                break;
            case "menu_show_panzoom":
            	CytowebUtil.showPanZoomControl(true);
            	track("Visualisation", "Show panzoom");
            	break;
            case "menu_merge_links":
            	CytowebUtil.mergeEdges(true);
            	track("Visualisation", "Merge edges");
            	break;
            case "menu_transparent_edges":
            	CytowebUtil.transparentEdges(true);
            	track("Visualisation", "Show edges as semitransparent");
            	break;
            }
        },
        
        onMenuItemUncheck: function(li){
        	if(li.hasClass("ui-state-disabled")){
        		return;
        	}
        	
            switch( li.attr("id") ) {
            case "menu_show_labels":
            	CytowebUtil.showNodeLabels(false);
            	track("Visualisation", "Hide labels");
                break;
            case "menu_publication_labels":
            	CytowebUtil.showPublicationLabels(false);
            	track("Visualisation", "Disable show publication labels");
                break;
            case "menu_show_panzoom":
            	CytowebUtil.showPanZoomControl(false);
            	track("Visualisation", "Hide panzoom");
            	break;
            case "menu_merge_links":
            	CytowebUtil.mergeEdges(false);
            	track("Visualisation", "Unmerge edges");
            	break;
            case "menu_transparent_edges":
            	CytowebUtil.transparentEdges(false);
            	track("Visualisation", "Show edges as opaque");
            	break;
            }
        },
        
        onMenuItemClick: function(li) {
        	if(li.hasClass("ui-state-disabled")){
        		return;
        	}
        	
            switch( li.attr("id") ) {
	            case "menu_remove_selected":
	        		CytowebUtil.removeSelectedGenes();
	        		track("Search", "Search removing selected genes");
	                break;
	            case "menu_search_selected":
	        		CytowebUtil.searchWithSelectedGenes();
	        		track("Search", "Search with selected genes");
	                break;
	            case "menu_add_selected":
	        		CytowebUtil.addSelectedGenesToSearch();
	        		track("Search", "Add selected genes to search");
	                break;
            	case "menu_generate_report":
            		CytowebUtil.generateReport();
            		track("File", "Generate report");
	                break;
            	case "menu_export_params":
            		CytowebUtil.exportParams();
            		track("File", "Save params as TXT");
	                break;
            	case "menu_export_params_json":
            		CytowebUtil.exportParamsJson();
            		track("File", "Save params as JSON");
	                break;
            	case "menu_export_networks":
            		CytowebUtil.exportNetworks();
            		track("File", "Save networks as TXT");
	                break;
            	case "menu_export_genes":
            		CytowebUtil.exportGenes();
            		track("File", "Save genes as TXT");
	                break;
            	case "menu_export_go":
            		CytowebUtil.exportGo();
            		track("File", "Save functions as TXT");
	                break;
            	case "menu_export_interactions":
            		CytowebUtil.exportInteractions();
            		track("File", "Save interactions as TXT");
	                break;
            	case "menu_export_network":
            		CytowebUtil.exportNetwork();
            		track("File", "Save as TXT");
	                break;
            	case "menu_export_attributes":
            		CytowebUtil.exportAttributes();
            		track("File", "Save attributes as TXT");
	                break;
	            case "menu_export_svg":
	            	CytowebUtil.exportSvg();
	            	track("File", "Save as SVG");
	                break;
	            case "menu_reset_layout":
	            	CytowebUtil.recomputeLayout();
	            	track("Visualisation", "Recompute layout");
	                break;
	            case "menu_neighbors":
	            	CytowebUtil.highlightFirstNeighbors();
	            	track("Visualisation", "Highlight first neighbours");
	            	break;
	            case "menu_neighbors_clear":
	            	CytowebUtil.clearFirstNeighborsHighlight();
	            	track("Visualisation", "Unhighlight first neighbours");
	            	break;
	            case "menu_close_tooltips":
	                $(".qtip").hide().remove();
	                $("#menu_close_tooltips").addClass("ui-state-disabled");
	                track("Tooltips", "Close");
	                break;
            }
        },
        
        onMenuItemSelect: function(li) {
        	if(li.hasClass("ui-state-disabled")){
        		return;
        	}
        	
            switch( li.attr("id") ) {
	            case "menu_legend":
	            	net_legend_popup();
	            	
	                $("#legend").fadeIn(ANI_SPD, function(){
	            		$("#legend .content").show();
	            	});
	                
	                track("Networks legend", "Show");
	                break;

	           case "menu_go_legend":
	                go_legend_popup();
	                
	                $("#go_legend").fadeIn(ANI_SPD, function(){
	            		$("#go_legend .content").show();
	            	});
	                
	                track("Functions legend", "Show");
	                break;
            }
        },
        
        onMenuItemDeselect: function(li) {
        	if(li.hasClass("ui-state-disabled")){
        		return;
        	}
        	
            switch( li.attr("id") ) {
	            case "menu_legend":
	                $("#legend .content").fadeOut(ANI_SPD, function(){
	                    $("#legend").effect("transfer", { to: "#menu_legend" }, ANI_SPD);
	                    $("#legend").fadeOut(ANI_SPD);
	                });
	                
	                $.tooltip.hide(toolTipOptions);
	                track("Networks legend", "Hide");
	                break;
	           case "menu_go_legend":
	                $("#go_legend .content").fadeOut(ANI_SPD, function(){
	                    $("#go_legend").effect("transfer", { to: "#menu_go_legend" }, ANI_SPD);
	                    $("#go_legend").fadeOut(ANI_SPD);
	                });
	                
	                $.tooltip.hide(toolTipOptions);
	                track("Functions legend", "Hide");
	                break;
	        }
        }
    });
    
    function net_legend_popup(){
    	var li = $("#menu_legend");
    	li.menuPopup("legend", { title: "Networks legend", 
            hiddenToolTip: "Use the toggle button above to restore the network legend when hidden." });

		$("#legend .content").html('<table> <thead><tr><th>Colour</th><th>Name</th></tr></thead> <tbody></tbody></table>');
		
		$("#networks_widget").children("li").each(function(){
			var colour = $(this).find(".bar:first").css("background-color");
			if (! $(this).find("input").attr("disabled")) {
				var name = $(this).find(".name:first").text();
				var weight = $(this).find(".per_cent_text:first").text();
				$("#legend .content tbody").append('<tr>' +
					'<td><div class="legend_square" style="background-color: ' + colour + '"></div></td>' +
					'<td>' + name + '</td>' +
					'</tr>');
			}
		});
		
		$("#legend table").tablesorter({
			sortList: [ [1,0] ]
		}); 
    }
    
    function go_legend_popup(){
        $("#menu_go_legend").menuPopup("go_legend", {
            title: "Functions legend", 
            hiddenToolTip: "Use the toggle button above to restore the functions legend when hidden.",
            resizable: false
        });
        
        if( $("#go_legend .instructions").size() == 0 ){
            $("#go_legend .content").append('<div class="header instructions"><p>No colorings have been added.</p><p><a class="action_link" href="#">Open the functions tab</a> to add colorings.</p></div>');
            
            $("#go_legend .instructions a").bind("click", function(){
                $("#side_bar .ui-tabs-nav a[href=#go_tab]").click();
                return false;
            });
        }
    }
    
    $("#menu_search_item").click(function(){
        return false;
    });
    
    $("#gene_search").focus(function(){
        $(this).removeClass("light");
        $(this).val("");
    }).blur(function(){
        $(this).addClass("light");
        $(this).val("Find a gene by typing its name here");
    }).blur();
    
    // default enabled/checked
    
    $(window).load(function(){
    	$("#menu_neighbors").addClass("ui-state-disabled");
    	$("#menu_neighbors_clear").addClass("ui-state-disabled");
    	$("#menu_search_selected").addClass("ui-state-disabled");
    	$("#menu_add_selected").addClass("ui-state-disabled");
    	$("#menu_remove_selected").addClass("ui-state-disabled");
    	$("#menu_show_labels").add("#menu_show_panzoom").add("#menu_transparent_edges").find(".ui-menu-check-icon").addClass("ui-menu-checked");
        go_legend_popup();
        net_legend_popup();
        $("#selected_info").hide();
        $("#go_legend").hide();
        $("#legend").hide();
        $("body").click();
    });
    
    // Do not close the menu when user clicks this kind of menu item:
    $(".menu-option").toggle(function(){ /*do nothing*/ }, function(){ /*do nothing*/ });
});$(function(){

	if( $("#results_page").size() == 0 ){
		return; // ignore this js file if on front page
	}

    /**********************************************
     * NETWORKS TAB
     **********************************************/

	var _ignoreFilter = false;
	var _filterNetworks = function() {
		if (!_ignoreFilter) {
			CytowebUtil.filterNetworks();
		}
	};
	
	$("#networks_tab .checktree").checkTree({
        labelAction: "",
        checkedText: "<input type=\"checkbox\" class=\"widget\" checked=\"true\" />",
        halfCheckedText: "<input type=\"checkbox\" class=\"widget semitransparent\" checked=\"true\" />",
        uncheckedText: "<input type=\"checkbox\" class=\"widget\" />",
        collapsedText: "&nbsp;",
        expandedText: "&nbsp;",
        onCheck: function onCheckHandler(li) { _filterNetworks(); update_enable_by_checkbox_state(); },
    	onUnCheck: function onUnCheckHandler(li) { _filterNetworks(); update_enable_by_checkbox_state(); },
    	onExpand: function onExpandHandler(li) { update_expand_by_arrow_state(); },
    	onCollapse: function onCollapseHandler(li) { update_expand_by_arrow_state(); }
    });
	
	$("#networks_tab .checkbox").bind("mousedown", function(){
		if( !$(this).hasClass("checked") ){
			track("Networks", "Enable", "Check");
		} else {
			track("Networks", "Enable", "Uncheck");
		}
	});
	
	$("#networks_tab .arrow").bind("mousedown", function(){
		if( !$(this).hasClass("expanded") ){
			track("Networks", "Expand", "Arrow open");
		} else {
			track("Networks", "Expand", "Arrow close");
		}
	});
	
    // tool tips for networks tab check boxes
    $("#networks_tab").find(".checkbox").each(function(){
        $(this).attr("tooltip", "These checkboxes toggle whether networks are displayed.");
        $(this).addClass("checked");
    });
    
    $("#networks_tab").find(".widget").each(function(){
        $(this).attr("checked", "true");
    });
	
    // tool tips for check boxes in advanced
    $("#advanced_options").find(".checkbox").each(function(){
        $(this).attr("tooltip", "These checkboxes toggle whether networks are used by GeneMANIA.");
    });
    
    // bar graphs - build per cent bar based on % value found in tree
    $(".per_cent_bar").each(function(){
        var weight = $(this).parent().closest(":has(.per_cent_text)").find(".per_cent_text:first").attr("weight");
        var width = parseFloat(weight) * 100;
		if (! isNaN(width)) {
			$(this).find(".bar").css("width", width + "%");
		}
    });

    
    $("#networks_tab_check_all").click(function(){
        // It avoids the plugin calling the filter function for each top level checkbox:
    	_ignoreFilter = true;
    	$("#networks_widget").modifyCheckTree({checkAll: true});
    	// Now we cal filter the network, but only once:
    	_ignoreFilter = false;
    	_filterNetworks();
    	
    	$(this).addClass("active").siblings().removeClass("active");
    	
    	track("Networks", "Enable", "All");
    	return false;
    });
    
    $("#networks_tab_check_none").click(function(){
    	_ignoreFilter = true;
        $("#networks_widget").modifyCheckTree({checkNone: true});
        _ignoreFilter = false;
    	_filterNetworks();
    	
    	$(this).addClass("active").siblings().removeClass("active");
    	
    	track("Networks", "Enable", "None");
    	return false;
    });
    
    function update_enable_by_checkbox_state(){
        if( $("#networks_tab .checkbox.checked").size() == 0 ){
            $("#networks_tab_check_none").addClass("active").siblings().removeClass("active");
        } else if( $("#networks_tab .checkbox.checked").size() == $("#networks_tab .checkbox").size() ){
            $("#networks_tab_check_all").addClass("active").siblings().removeClass("active");
        } else {
            $("#networks_tab_check_none").add("#networks_tab_check_all").removeClass("active");
        }
    }
    update_enable_by_checkbox_state();
    
    $("#networks_tab_expand_all").click(function(){
        $("#networks_widget").modifyCheckTree({expandAll: true});
        
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Networks", "Expand", "All");
    	return false;
    });
    
    $("#networks_tab_expand_top_level").click(function(){
        $("#networks_widget").modifyCheckTree({expandTopLevel: true});
        
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Networks", "Expand", "Only top level");
    	return false;
    });
    
    $("#networks_tab_expand_none").click(function(){
        $("#networks_widget").modifyCheckTree({expandNone: true});
        
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Networks", "Expand", "None");
    	return false;
    });
    
    function update_expand_by_arrow_state(){
        if( $("#networks_tab .arrow.collapsed").size() == 0 ){
            $("#networks_tab_expand_all").addClass("active").siblings().removeClass("active");
        } else if( $("#networks_tab .arrow.expanded").size() == 0 ){
            $("#networks_tab_expand_none").addClass("active").siblings().removeClass("active");
        } else if( $("#networks_tab .checktree_top_level > .arrow.expanded").size() == $("#networks_tab .checktree_top_level > .arrow").size() 
        && $("#networks_tab .checktree_top_level > .arrow.expanded").size() == $("#networks_tab .arrow.expanded").size() ) {
            $("#networks_tab_expand_top_level").addClass("active").siblings().removeClass("active");
        } else {
            $("#networks_tab_expand_none").add("#networks_tab_expand_all").add("#networks_tab_expand_top_level").removeClass("active");
        }
    }
    update_expand_by_arrow_state();
    
    $("#networks_tab_sort_by_weight").click(function(){
        $("#networks_widget").listSort({
            value: ".per_cent_text:first",
            descending: true,
            attribute: "weight"
        });
        
        $(this).addClass("active").siblings().removeClass("active");
        
    	track("Networks", "Sort", "By weight");
    	return false;
    }).click();
    
    $("#networks_tab_sort_by_name").click(function(){
        $("#networks_widget").listSort({
            value: ".name:first",
            descending: false
        });
        
        $(this).addClass("active").siblings().removeClass("active");
        
        track("Networks", "Sort", "By name");
    	return false;
    });
   
    var highlightTimeout;
    var highlightDelay = 150;
    
    function unhighlight(){
    	clearTimeout(highlightTimeout);
    	highlightTimeout = setTimeout(function(){
    		CytowebUtil.unhighlightGenesAndEdges();
    	}, highlightDelay);
    }
    
    $("#networks_widget .checktree_network_group > .label").bind("mouseover", function(){
    	$(this).addClass("highlight");
    	var id = $(this).parent().attr("netid");
    	
    	clearTimeout(highlightTimeout);
    	highlightTimeout = setTimeout(function(){
    		CytowebUtil.highlightNetworkGroup( id );
    	}, highlightDelay);
    }).bind("mouseout", function(){
    	$(this).removeClass("highlight");
    	
    	unhighlight();
    });
    
    $("#networks_widget .checktree_network > .label").bind("mouseover", function(){
    	$(this).addClass("highlight");
    	var id = $(this).parent().attr("netid");
    	
    	clearTimeout(highlightTimeout);
    	highlightTimeout = setTimeout(function(){
    		CytowebUtil.highlightNetwork( id );
    	}, highlightDelay);
    }).bind("mouseout", function(){
    	$(this).removeClass("highlight");
    	
    	unhighlight();
    });
    
    $("#networks_widget .checktree_attr_group > .label").bind("mouseover", function(){
    	$(this).addClass("highlight");
    	var id = $(this).parent().attr("attrid");
    	
    	clearTimeout(highlightTimeout);
    	highlightTimeout = setTimeout(function(){
    		CytowebUtil.highlightAttributeGroup( id );
    	}, highlightDelay);
    }).bind("mouseout", function(){
    	$(this).removeClass("highlight");
    	
    	unhighlight();
    });
    
    $("#networks_widget .checktree_attr > .label").bind("mouseover", function(){
    	$(this).addClass("highlight");
    	var id = $(this).parent().attr("attrid");
    	
    	clearTimeout(highlightTimeout);
    	highlightTimeout = setTimeout(function(){
    		CytowebUtil.highlightAttribute( id );
    	}, highlightDelay);
    }).bind("mouseout", function(){
    	$(this).removeClass("highlight");
    	
    	unhighlight();
    });
    
    progress("networks");
    
});$(function(){
	
	if( $("#print_page").size() <= 0 ){
		return;
	}
	
	if( $.browser.msie ){
		
		var timeout = undefined;
		var callback = function(){
		
			if( $(".graph_area object").size() == 0 ){
				setTimeout(callback, 10);
			} else {
				var svg = $(".graph_area object");
				var parent = $(".graph_area");
				
				var height = parent.height();
				var width = parent.height() / svg.height() * svg.width();
				
				svg.height(height);
				svg.width(width);
				svg.css({
					position: "relative",
					left: "50%",
					marginLeft: (-1/2 * width) + "px"
				});
			
				
			}
			
		};
		callback();
		
	}
	
	$("#transferred_go_legend .colouring").each(function(){
		var name = $(this).find(".annotation").text();
		var ocid = $(this).attr("ocid");
		var colour = $(this).attr("colour");
		
		var entry = $('<div class="entry" ocid="' + ocid + '" colour="' + colour + '">'+
					'<span class="colour"></span>' +
					'<span class="name">' + name + '</span>' +
				'</div>');
		
		$(".go.legend").append(entry);
		entry.find(".colour").css("border-color", colour);
	});
	
	$("#go_table th").unbind("click");
	
	$("#genes_widget .go .colour").html('');
	
	
	
	$("#genes_widget .go_list").each(function(){
		$(this).find(".colour").each(function(i){
			if( $(this).parent().hasClass("has") ){
				$(this).append('<span class="number">'+ (i+1) +'</span>');
			}
		});
	});
	
	$("#go_table td.annotation").each(function(i){
		$(this).prepend('<span class="colour"><span class="number"></span></span>');
	});
	
	$(".legend.go .entry").each(function(){
		var id = $(this).attr("ocid");
		var colour = $(this).attr("colour");
		
		$("#genes_widget .go[ocid=" + id + "].has .colour").css({
			"border-color": colour
		});
		
		$("#go_table .annotation[ocid=" + id + "] .colour").css({
			"border-color":  colour
		});
		
	});
	
	$(".print_selection .button").bind("click", function(){		
		window.print();
	});
	
	$(".print_selection input").bind("click", function(){
		
		var section = $(this).attr("section");
		var checked = $(this).is(":checked");
		
		if( checked ){
			$(".section[section=" + section + "]")
				.removeClass("do_not_print")
				.find("h2 input").attr("checked", "checked");
		} else {
			$(".section[section=" + section + "]")
				.addClass("do_not_print")
				.find("h2 input").removeAttr("checked");
		}
		
		adjust_breaks();
		
	});
	
	$("h2").append('<span class="print button"><label>Remove from print</label></span>');
	
	function adjust_breaks(){
		$("h2:visible:first").removeClass("break");
		$("h2:visible").not(":first").addClass("break");
	}
	adjust_breaks();
	
	$("h2 .print").bind("click", function(){
		var section = $(this).parents(".section").attr("section");
		
		$(".section[section=" + section + "]").addClass("do_not_print");
		
		$(".print_selection input[section=" + section + "]").removeAttr("checked");
	});
	
	$("h2").append('<span class="save button"><label>Save</label></span>');
	$(".print_selection .item").each(function(){
		var section = $(this).find("[section]").attr("section");
		
		$(this).prepend('<span class="save button" section="' + section + '"><label>Save</label></span>');
		$(this).find(".save").bind("click", function(){
			var export_fn = "export" + section.toUpperCase().charAt(0) + section.substr(1);
			
			CytowebUtil[export_fn](false);
		});
	});
	
	$("h2 .save").bind("click", function(){
		var section = $(this).parents(".section").attr("section");
		
		$(".print_selection .save[section=" + section + "]").click();
	});
	
	//sort genes
	$("#genes_widget").listSort({
        value: ".source_score_true",
        descending: true
    });
    $("#genes_widget").listSort({
    	value: ".score_text:first",
    	descending: true
    });
    
    //sort networks
    $("#networks_widget").listSort({
        value: ".per_cent_text:first",
        descending: true 
    });
	
    //sort go
    
    $.tablesorter.addParser({ 
        // set a unique id 
        id: 'exponent', 
        is: function(s) { 
            return ("" + s).match( /[-+]?[0-9]*\.?[0-9]+([eE][-+]?[0-9]+)?/ ) != null;
        }, 
        format: function(s) { 
            return s;
        }, 
        type: 'numeric' 
    }); 
    
    $("#go_table").tablesorter({
        textExtraction: function(node){
            var ele = $(node);
            var val = "" + ele.attr("value");
            return val;
        }
    });
    
    $("#go_table .header.pval").click();
    
    $("#go_table th").unbind("click");
    
    setTimeout(function(){
    	$("#go_table td.annotation").each(function(i){
    		$(this).find(".number").append(i + 1);
    	});
    }, 50);
    
    // sort interactions
    
    $("#interactions_table").tablesorter({
    	sortList: [ [3,0], [2,1] ]
    });
    $("#interactions_table th").unbind("click");
});var _layout;

$(function(){    

	if( $("#results_page").size() == 0 ){
		return; // ignore this js file if on front page
	}

	$("#relatedGenes").bind("mouseenter", function(){
		$(this).addClass("active");
	}).bind("mouseleave", function(){
		$(this).removeClass("active", ANI_SPD);
	});
	
    /**********************************************
    side bar layout
    **********************************************/
    
    var MIN_GRAPH_WIDTH = parseInt( $("#cytoscape_lite").css("min-width") );
    var MIN_SIDE_BAR_WIDTH = parseInt( $("#side_bar").css("min-width") );
    var SPACING = 12;
    
    var resizing = false;
    var layout = $("body").layout({
        name: "defaultLayout",
        
        defaults: {
            size: "auto",
            resizable: false,
            fxName: "hide",
            fxSpeed: 0,
            contentIgnoreSelector: ".qtip"
        },
        
        north: {
            paneSelector: "#header"
        },
        
        center: {
            paneSelector: "#cytoscape_lite"
        },
        
        east: {
            size: MIN_SIDE_BAR_WIDTH,
            minSize: MIN_SIDE_BAR_WIDTH,
            maxSize: get_max_side_size(),
            paneSelector: "#side_bar",
            resizable: true,
            resizerTip: "",
            spacing_open: SPACING,
            onresize_start: function(){
        		track("Sidebar", "Resize");
            },
            onresize_end: function(){
                $("#overlay").remove();
            }
        },
        
        south: {
            paneSelector: "#footer",
            closable: false
        }
    });
    _layout = layout;

    
    $("[resizer=east]").bind("mousedown", function(){
    	$("body").append('<div id="overlay"></div>');
    }).bind("mouseup", function(){
    	$("#overlay").remove();
    });
    
    $("#overlay").live("mouseup mousedown click", function(){
    	$("#overlay").remove();
    });
    
    
    function get_max_side_size(){
        var max = $(window).width() - MIN_GRAPH_WIDTH - 4*SPACING;
        var min = (layout == null ? null : layout.options.east.minSize);
        
        if( min != null && max < min ){
        	return min;
        }
        
        return max;
    }
    
    $(window).bind("resize", function(event){
		layout.options.east.maxSize = get_max_side_size();
        
        // workaround for small window size hides sidebar
        if( !$("#side_bar").is(":visible") && !$("#side_bar_toggle_closed").is(":visible") ){
        	layout.show("east");
        }
        
        // fix resizer location if the layout plugin loses it
        $("[resizer=east]").each(function(){
        	var right = parseInt( $(this).offset.right );
        	var should_be_right = $("#side_bar").width()
        	
        	if( right != should_be_right ){
        		$(this).css( "right",  should_be_right );
        	}
        });
    });
    
    // overlap fixes
    // plugin sets all to index 2
    $("#header").css("z-index", 4);
    $("#cytoscape_lite").css("z-index", 3);
    $("#side_bar").css("z-index", 2);
    $("#footer").css("z-index", 3);
    
    
    
    // tabs
    $(".tabs").tabs({
        show: function(){
            $(window).trigger("resize"); // trigger resize for tab contents fitting
        }
    });
    
    $("#side_bar .ui-tabs-nav a").bind("click", function(){
    	var name = $(this).text() + " tab";
    	track(name, "Open");
    });
    
    $(window).bind("load", function(){
    	$(window).trigger("resize");
    });
    
    
    /**********************************************
    side bar toggle open, closed
    **********************************************/
    
    $("#side_bar_toggle_open").click(function(){
        layout.hide("east");
        $("#side_bar_toggle_closed").show();
        $(this).mouseout(); // fixes bug on older browsers where mouseout isn't sent after animation
        track("Sidebar", "Close");
    });
    
    $("#side_bar_toggle_closed").click(function(){
        $(this).hide();
        layout.show("east");
        $(this).mouseout(); // fixes bug on older browsers where mouseout isn't sent after animation
        track("Sidebar", "Open");
    }).hide();
    
    /**********************************************
    fix sizes of side bar contents on window resize
    **********************************************/
    
    // resize tab contents on contents resize
    $(window).bind("resize", function(){
        
       fix();
       setTimeout(function(){
            fix();
       }, 100); // 100 ms is empirically good 
       
        function fix(){
           $(".tabs").add(".menu").each(function(){
                var totalHeight = $(this).innerHeight();
                var tabHeaderHeight = $(this).find("ul:first").outerHeight({margin: true});
                
                $(this).find(".tab, .menu_area").each(function(){
                    var headerHeight = 0;
                    
                    $(this).children(".header").each(function(){
                    	if ($(this).css("display") != "none") {
                    		headerHeight += $(this).outerHeight({margin: true});
                    	}
                    });
                     
                    $(this).find(".content").height( totalHeight - ( headerHeight + tabHeaderHeight ) );
                });
            });
            
            $("#side_bar").each(function(){
                var max = $("html").width() - parseInt( $("#cytoscape_lite").css("min-width") ) - 50;
                var min = $("#side_bar").resizable("option", "minWidth");
                
                if( max <  min) {
                    max = min;
                }
                
                $(this).resizable("option", "maxWidth", max); // - 50 for unknown spacing for resizer
                if( $(this).width() > max ) {
                    $(this).width( max );
                }
            });
        }   
       
    }).resize(); // fire once for initial size
    
    /**********************************************
    loader
    **********************************************/
    
    // hide loader when both cytolite and page loaded
    
    var cytowebLoaded = false;
    var cytowebError = false;
    var pageLoaded = false;
    
    $(window).load(function(){
        pageLoaded = true;
        checkLoader();
    });
    
    onCytoscapeWebLoaded = function(error){
    	cytowebLoaded = true;
    	cytowebError = error;
    	var done = checkLoader();
    	if (done) {
    		// Trigger resize to recalculate the tab contents height:
    		$(window).resize();
    	}
    }
    
    function checkLoader() {
    	if( cytowebLoaded && pageLoaded ) {
            if (cytowebError) {
            	$("#cytoscape_lite .ui-tabs-nav").hide();
            	$("#networks_tab .content .checkbox").hide();
            	$("#networks_filter_menu").hide();
            	$("#genes_tab .label").unbind("click").unbind("mouseover");
            	$("#side_bar .ui-tabs-nav li").eq(2).hide();
            	$("#cytoscape_lite .ui-menu-nav").hide();
            }
            return true;
    	}
    	return false;
    }
      
    CytowebUtil.refresh();
    
    progress("layout");
});
$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}

    /***************************************************************************
	 * expandable advanced behaviour
	 **************************************************************************/
    // expand
	var width_set = false;
    $("#advanced_options_closed .advanced_options_toggle a").click(function() {
    	// refreshTreeFlag = false;
    	// switch
        $("#advanced_options_closed").hide();
        $("#advanced_options").show();
        $("#advanced_options_open").show();
        $("#relatedGenes").addClass("advanced_open");
        $("#cytoscape_lite").addClass("advanced_open");
        $("#side_bar").addClass("advanced_open");
        // DO NOT USE the slide option!!! It rebuilds the tree, causing a bug
		// that
        // prevents the default networks to be checked.
        // ***********************************************************************
        // $("#advanced_options_open").show("slide", { direction: "up" },
		// ANI_SPD);
        // ***********************************************************************

        if( !width_set ){
        	var padding = $("#advanced_options").outerWidth() - $("#advanced_options").width();
        	var width = $("#advanced_line").prev().width() - padding;
	        $("#advanced_options").width( width );
	        width_set = true;
        }
        
        track("Advanced options", "Open");
        
        adjustNetworksLoaderCSS();

        return false;
    });
    
    // close advanced on hide
    $("#advanced_options_open .advanced_options_toggle a").click(function() {	
        close_advanced_options();
        
        return false;

    });
    
    if( $("#results_page").size() > 0 ){
    
		$("#cytoscape_lite").add("#side_bar").add("#footer").add(".ui-layout-resizer").bind("mousedown", function(e){
			if( $("#advanced_options_open").is(":visible") ){
				close_advanced_options();
			}
		});
    
    }
    
    function close_advanced_options(){
        // switch
        $("#advanced_options_closed").show();
        $("#advanced_options_open").hide();
        // $("#advanced_options_open").fadeOut(ANI_SPD);
        $("#advanced_options").hide();
        $("#relatedGenes").removeClass("advanced_open");
        $("#cytoscape_lite").removeClass("advanced_open", ANI_SPD);
        $("#side_bar").removeClass("advanced_open", ANI_SPD);
        // DO NOT USE the slide option!!! It rebuilds the tree, causing a bug
		// that
        // prevents the default networks to be checked.
        // ***********************************************************************
        // $("#advanced_options").hide("slide", { direction: "up" }, ANI_SPD);
        // ***********************************************************************
        
        track("Advanced options", "Close");
    }
    
    
	$("#advanced_options_closed .advanced_options_toggle a").bind("click", function(){
		loadNetworks();
		loadAttributeGroups();
	});
});
function loadAttributeGroups(){
	var organism = $("#species_select").val();
	
	$(".attribute_group[organism=" + organism + "]").show();
	$(".attribute_group[organism!=" + organism + "]").hide();
	
	if( $(".attribute_group:visible").size() == 0 ){
		$("#no_attr_groups").show();
	} else {
		$("#no_attr_groups").hide();
	}
}function loadDefaultOrganismGenes() {
	clearGeneError();

	var genes_string = $("#species_select :selected").attr("defgenes");
	
	// console.log("**** "+genes)
	$("#gene_list").val(genes_string);
	$("#gene_text").val(genes_string.replace("\n", "; "));// console.log("&&&
															// "+$("#gene_text").val());
	$("#gene_area").val(genes_string);// console.log("###
													// "+$("#gene_area").val());
}$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
    /***************************************************************************
	 * gene entry box behaviour
	 **************************************************************************/
    var genesOpen = false;
    
    // on focus of text line, replace with text area (i.e. bigger box)
    $("#gene_text").focus(function(){

        // tab focus fix (i.e. press tab and focus gene_text from gene_area =>
		// creates bad state)
        if(genesOpen) {
            $(this).css("visibility", "visible").blur();
        } else {
        
            // replace widget
            $("#gene_closed").css("visibility", "hidden");
            $("#gene_open").show();
            
            // fancy ani
            $("#gene_area").fadeIn(ANI_SPD, function(){
                $(this).focus(); // must focus after ani
//                var scrollToVal = $(this).attr("scrollHeight");
//                $(this).attr("scrollTop", scrollToVal);
                genesOpen = true;
                
                $("#uploadArea").hide();
            });
        
        }
    });
    
    var gene_timeout;
    var gene_delay = 750;
    
    function update_text_from_area(){
		// replace text
        var text = new String( $("#gene_area").val() );

        // remove blank lines
        while( text.search(/\n(\s)*\n/) >= 0 ) {
            text = text.replace(/\n(\s)*\n/g, "\n");
        }
        // left trim
        text = $.trim(text);
                
        // newline <-> ; for gene_text box summary
        var newlineToSemi = text.replace(/\n/g, "; ");
        var noSemiAtEnd = $.trim(newlineToSemi).replace(/;$/, ""); 
        $("#gene_text").val(noSemiAtEnd);
    }
    
    // on blur of text area, show text line instead (i.e. shrink bigger box)
    var gene_area_val;
    
    function trigger_hide_tooltip(){
		$("#gene_area").trigger("hidetooltip");
    }
    
    $(window).bind("blur", function(e){
    	trigger_hide_tooltip();
    });
    
   
    $("body").bind("mousedown mouseup", function(e){ 
    
    	var target = $(e.target);

    	var inside_area = false;
    	var inside_tooltip = false;
    	target.parents().andSelf().each(function(){
    		if(  $(this).hasClass("gene_error_tooltip") ){
    			inside_tooltip = true;
    		}
    	
    		if( $(this).attr("id") == "gene_area" ){
    			inside_area = true;
    		}
    	});
    	
    	if( !inside_area && !inside_tooltip ){
    		trigger_hide_tooltip();
    	} 
    	
    	if( inside_tooltip ) {
    		return false;
    	}
    	
    });
    
    if( $("#gene_area").val() != "" ){
// console.log("some genes already entered");
    	update_text_from_area();
		validateGeneSymbols();
    }
    
    updateGeneCount();
    
    var too_many_genes_message = "<p>Because you have more than " + maxNumberOfGenes() + " genes in your list, the web " + 
	"version of GeneMANIA does not support your query.</p> " +
	"<p>We suggest you try the <a href='http://pages.genemania.org/plugin'>GeneMANIA Cytoscape plugin</a>, which can handle very large number of genes.</p>";

    function adjust_icons_from_scroll(){
    	
    	$("#gene_validation_icons").css({
    		top: -1 * $("#gene_area").scrollTop(),
    		left: -1 * $("#gene_area").scrollLeft()
    	});
    	
    }
    
    
    var last_gene_area_val = "";
    var last_organism = $("#species_select").val();
    var last_valid_max_gene_list = true;
    $("#gene_area").focus(function(){
    	$("#gene_area").trigger("showtooltip");
    	$("#gene_area").trigger("adjusttooltipposition");
    }).blur(function(e){
        // replace widget
    	$("#gene_closed").css("visibility", "visible");
        $("#gene_open").fadeOut(ANI_SPD);

		update_text_from_area();
		validateGeneSymbols();
        
        $(this).hide();
        genesOpen = false; // TODO remove this genesOpen variable and use
							// $("#genes_open").is(':visible') instead
		
    	$("#gene_list").val($("#gene_text").val());
    	
    	$("#uploadArea").show();
    	
    }).bind("keyup keydown paste change", function(e){	
		
		// console.log("--\ngene key event");
		// console.log(e);
		
		// console.log("caret location: ");
		// console.log("index: ");
		// console.log( $(this).caret() );
		// console.log( $(this).val()[ $(this).caret().end - 1 ] + "|" +
		// $(this).val()[ $(this).caret().end ] );
		
    	if( e.type != "keydown" ){
    	
	    	var number_of_genes = numberOfGenes();
	    	var max_number_of_genes = maxNumberOfGenes();
	    	
	    	updateGeneCount();
  
	    	if( number_of_genes > max_number_of_genes ){
	    		$("#gene_validation_icons .icon").attr("type", "empty").attr("class", "icon empty");
	    		
	    		$("#gene_selection").removeClass("loading");
	    		
	    		if( last_valid_max_gene_list ){
		    		showGeneError("error", "Too many genes", too_many_genes_message);
		    	}
	    		
	    		last_valid_max_gene_list = false;
	    		
	    		return;
	    	} else {
	    		last_valid_max_gene_list = true;
	    	}
    	}
    	
		function key_is_meta(code){
			switch(code){
				case 37: // left
				case 38: // up
				case 39: // right
				case 40: // down
				case 17: // control
				case 18: // alt
				case 91: // command
				case 27: // escape
				case 16: // shift
				case 20: // caps lock
					return true;
				default:
					return false;
			}
		}
		
		function key_is_enter(code){
			return code == 13;
		}
		
		function key_is_backspace(code){
			return code == 8;
		}
		
		function key_is_delete(code){
			return code == 46;
		}
		
		function key_edits(code){
			return !key_is_meta(code);
		}
		
		// don't trigger anything on just a meta key (shift, alt, etc)
		if( !key_edits(e.which) ){
			// do nothing
		} else if( $("#gene_area").is(":visible") ) {
			
			// clear all icons iff blank
			if( $("#gene_area").val() == "" || $("#gene_area").val().match(/^\s*$/g) ){
				// console.log("empty val icons");
				$("#gene_validation_icons .icon").attr("type", "empty").attr("class", "icon").attr("tooltip", "");
				
		    	adjust_icons_from_scroll();
			}
		
			function update_qtip_position(){
				// update position
				$(".qtip[qtipfor=gene_selection]").each(function(){
					$(this).qtip("api").updatePosition();
				});
			}
			
			var cursor_char = $(this).caret().end;
			var val = $(this).val();
			
			function lines(include_last_char){
				var cursor_ch = cursor_char;
				
				if( include_last_char == undefined ){
					include_last_char = true;
				}
				
				if( !include_last_char ){
					cursor_ch--;
				}
			
				var line = 0;
				for(var i = 0; i < val.length && i < cursor_ch; i++){
					if( val[i] == "\n" ){
						line++;
					}
				}
				line_cache = line;
				
				return line;
			}
						
			function adjust_to_line(forwards){		
				adjustValidationIcons( (forwards ? 1 : -1) * line() );
			}
			
			// handle new or removed lines
			if( e.type == "keydown" ){			
				if( key_is_enter(e.which) ){
				
					var line = lines(true);
					
					if( cursor_char > 0 &&  val[ cursor_char - 1 ] == "\n" ){
						// console.log("pressed enter key before newline");
						adjustValidationIcons( line - 1 );	
					} else if( val[ cursor_char ] == "\n" || val.substring(cursor_char).match(/^\s*\n.*/g) ){
						// console.log("pressed enter key after newline");
						adjustValidationIcons( line );
					} else {
						// console.log("just adjust the icons; normal enter key
						// press");
						resetValidationIcon( line );
						resetValidationIcon( line + 1 );
						adjustValidationIcons();
					}
					
					update_qtip_position();
				} else if( key_is_backspace(e.which) && cursor_char > 0 && val[cursor_char - 1] == "\n" ){
					var line = lines(false);
				
					// console.log("pressed backspace key after newline");
					
					if( (val[cursor_char - 1] + "") != "\n" ){
						resetValidationIcon( line );
					}
					
					adjustValidationIcons( -1 * line );
					update_qtip_position();
				} else if( key_is_delete(e.which) && val[cursor_char] == "\n" ){
					var line = lines(false);
				
					// console.log("pressed delete key before newline");
					
					if( ("" + val[cursor_char + 1]) != "\n" && cursor_char - 1 > 0 && val[cursor_char - 1] != "\n" ){
						resetValidationIcon( line );
					}
					
					resetValidationIcon( line + 1 );
					adjustValidationIcons( -1 * line );
					update_qtip_position();
				} 
			}
					
			if( e.type != "keydown" ){
				var new_ga_val = $("#gene_area").val();
				var new_org = $("#species_select").val();
				
		    	if( e.type == "paste" || new_org != last_organism || last_gene_area_val != new_ga_val ){
		    		clearTimeout(gene_timeout);
		    		clip_icons();
		    		setLoadingIconsForGenes();
		    		gene_timeout = setTimeout(function(){
						update_text_from_area();
						last_gene_area_val = new_ga_val;
						validateGeneSymbols();
					}, gene_delay);
		    	}
			}
    	
		}
    });
    
    function hscroll_on(){
    	// remove listener so we don't get a loop
		$("#gene_area").unbind("scroll", scroll_callback);
		
		var scroll = $("#gene_area").scrollLeft();
		var ret = false;
		
		$("#gene_area").scrollLeft(1);
		if( $("#gene_area").scrollLeft() == 1 ){
			$("#gene_area").scrollLeft(scroll);
			ret = true;
		}
		
		// add back the listener
		$("#gene_area").bind("scroll", scroll_callback);
		return ret;
	}
    
    function clip_icons(){
    	if( hscroll_on() ){
    		var height = $("#gene_area").height() - $.scrollBarSize();
    		var top = $("#gene_area").scrollTop();
    		
//    		console.log("scroll is on");
    		$("#gene_validation_icons").css("clip", "rect(" + top + "px,14px," + (top + height) + "px,0px)");
    	} else {
//    		console.log("scroll is off");
    		$("#gene_validation_icons").css("clip", "auto");
    	}
    }
    
    function scroll_callback(){
    	adjust_icons_from_scroll();
    	clip_icons();
    }
    $("#gene_area").bind("scroll", scroll_callback);

    // replace separators with newlines as you type
    // makes the cursor disappear on some browsers
    // won't work if genes need separators in their names
    // $("#gene_area").keypress(sepToNewLine).keydown(sepToNewLine).keyup(sepToNewLine);
    
    // Avoid bug introduced in FireFox 3.6 for Mac OS: the Flash player steals
	// the keydown event,
    // preventing the user from editing the textarea when it overlaps the
	// CytoWeb area.
    var ua = navigator.userAgent;
    var isBuggyFF = ua.indexOf("Firefox") > -1 && ua.indexOf("Mac OS X") > -1;
    var doNotExpand = isBuggyFF && $("#graphBox").length > 0;
     
    
    $("#gene_area").autoexpand({
        minLines: 5,
        maxLines: doNotExpand ? 5 : 20
    });
    
    /***************************************************************************
	 * escape key to blur
	 **************************************************************************/
    
    $("#gene_area").add("#species_select").add("#species_text").keydown(function(e) {
        if(e.which && e.which == 27) {
            $(this).trigger("blur");
        }
    });
    
    $(".default_genes_link").live("mousedown", function(){
    	loadDefaultOrganismGenes();
    	var link = $(this);
    	
    	$("#gene_area").trigger("edit").trigger("paste");
    	
    	update_text_from_area();
    	updateGeneCount();
    	
		// setTimeout(function(){
			
			if( !link.hasClass("open") ){
    			validateGeneSymbols();
    		}
    		
    		if( link.hasClass("also_submit") ){
    			link.parents(".qtip").hide();
    			$("#findBtn").click();
    		}
		// }, gene_delay);
    		
    }).live("click", function(){
    	return false;
    });
    
});

function makeNetworkHtml(organism, group, network, error, types){

	if( types == null ){
		types = "networks";
	}
	
	var type = types.substring(0, types.length - 1);
	
	function empty(str){
		if(str == undefined || str == null || str == ""){
			return true;
		} else {
			return false;
		}
	}

	function add_commas(nStr)
	{
		nStr += '';
		x = nStr.split('.');
		x1 = x[0];
		x2 = x.length > 1 ? '.' + x[1] : '';
		var rgx = /(\d+)(\d{3})/;
		while (rgx.test(x1)) {
			x1 = x1.replace(rgx, '$1' + ',' + '$2');
		}
		return x1 + x2;
	}
	
	var html = "";
	
	if( network.metadata == null ){
		network.metadata = {
			authors: ""
		};
	}
	
	var url = network.metadata.sourceUrl;
	
	html = '<tr>';
	
	var authors, first_author, last_author, year, interactions;
	
	if( !error ){
		var authors = network.metadata.authors.split(",");
		
		var first_author = authors[0].trim();
		var last_author = authors[authors.length - 1].trim();
		
		if( empty(first_author) ){
			first_author = network.name;
		}
		
		if( empty(last_author) ){
			last_author = network.name;
		}
		
		year = network.metadata.yearPublished;
		
		interactions = network.metadata.interactionCount;
	} else {
		authors = network.name;
		first_author = network.name;
		last_author = network.name;
		interactions = 0;
		year = 0;
	}
	
	html += '<td class="first_author">' + first_author + '</td>';
	html += '<td class="last_author">' + last_author + '</td>';
	html += '<td class="date">' + year + '</td>';
	html += '<td class="interactions">' + interactions + '</td>';
	
	html += '<td class="content">';
	html += 	'<div class="query_network ' + ( empty(error) ? 'valid' : 'error' ) + '" organism="' + organism + '" group="' + group.id + '" network="' + network.id + '">' +
				( !error ?
						'<input type="checkbox" class="query_network_checkbox" name="'+ types +'" value="' + network.id + '" group="' +
		            	group.id + '" organism="' + organism + '" id="net' + network.id + '" '
		            	+ (network.defaultSelected ? ' default="true" ' : ' default="false" ')
		            	+ ( $("#selected_networks ["+ type +"=" + network.id + "]").size() > 0 ||
		            		( ($("#selected_networks").children().size() == 0 || $("#selected_networks").attr("organism") != $("#species_select").val()) && network.defaultSelected )
		            			?
		            		' checked="true" '
		            			: ''
		            		)
		            	+ '/>' 
					:
						'<span class="uploadError uploadStatusIcon"></span>'
				) +
	
				( network.id <= 0 ?
						'<span class="trashIcn"><span class="ui-icon ui-icon-trash"></span></span>'
					:
						''
				) +
            	            	
            	'<label>' + network.name + ( network.id <= 0 && !empty(network.metadata.comment) ? '<span class="date">&nbsp;' + network.metadata.comment + '</span>' : '' ) + '</label>' +
            	'<div class="query_network_info text" organism="' + organism + '">' +
            		
            		( !empty(error) ?
            				( network.metadata.interactionCount > 0 ?
            						'<p>' + $.i18n("search_networks.info.error") + '</p>'
            					:
            						( ("" + error).toLowerCase() == "method not allowed" ?
            								'<p>' + $.i18n("search_networks.info.too_big_file") + '  You may be interested in trying the <a href="http://pages.genemania.org/plugin/">GeneMANIA plugin</a> for Cytoscape, which supports large files.' + '</p>'
            							:
            								'<p>' + $.i18n("search_networks.info.no_interactions") + '</p>'
            						)
            				)
            			:
            				''
            		) +
            	
            		( !empty(network.description) && types != "attrgroups" ?
            				'<p class="description">' + network.description + '</p>'
            			:
            				''
            		) + 
            		
            		( !empty(network.metadata.authors) || !empty(network.metadata.url) || !empty(network.metadata.publicationName) ?
            				'<p class="ref">' + 
            					( !empty(network.metadata.url) && !empty(network.metadata.title) ? ' <a class="external_link" target="_blank" href="' + network.metadata.url + '">' + network.metadata.title + '</a>' : '' ) +
            					( !empty(network.metadata.authors) ? ' <span class="authors">' + network.metadata.authors.substring(0, network.metadata.authors.search(","))  + ' ' +  $.i18n("search_networks.info.etal") + '</span> ' : '' ) +
            					( !empty(network.metadata.yearPublished) ? ' <span class="year">(' + network.metadata.yearPublished + ').</span> ' : '' ) +
            					( !empty(network.metadata.publicationName) ? ' <span class="publication">' + network.metadata.publicationName + '.</span> ' : '' ) + 
            				'</p>'
            			:
            				''
            		) +
            		
            		( !empty(network.metadata.comment) && network.id > 0 ? 
            				'<p class="notice">' +
            					'<b>' + $.i18n("search_networks.info.comment_title") + '</b>' +
            					' <span class="comment">' + network.metadata.comment + '</span> ' +
            				'</p>'
        				:
        					''
            		) +
            		
            		( !empty(network.publicationName) && !empty(network.publicationUrl) ?
            				'<p><b>' + $.i18n("search_networks.info.source_title") + '</b>' +
            					( network.description + " " + $.i18n("search_networks.info.from") + " " ) +
            					( '<a class="external_link" target="_blank" href="' + network.publicationUrl + '">' + network.publicationName + '</a>' ) +
            				'</p>'
            			:
            				''
            		) +
            		
            		( empty(error) && ( !empty(network.metadata.source) || !empty(network.metadata.processingDescription) ) ?
                			'<p><b>' + $.i18n("search_networks.info.source_title") + '</b>' +
                				( !empty(network.metadata.processingDescription) ? '<a href="http://pages.genemania.org/help/#GeneMANIA_network_categories" target="_blank">' + network.metadata.processingDescription + '</a>' : "" ) +
                				( !empty(network.metadata.interactionCount) ? (!empty(network.metadata.processingDescription) ? ' ' + $.i18n("search_networks.info.with") + ' ' : '') + add_commas(network.metadata.interactionCount) + ' ' + $.i18n("search_networks.info.interactions") : '' ) +
                				( network.metadata.source.toLowerCase() != 'collaborator' || !empty(network.metadata.reference) ? 
                						( ' ' + $.i18n("search_networks.info.from") + ' ' ) +
                						( network.metadata.source.toLowerCase() == 'collaborator' || network.id < 0 ?
                								network.metadata.source
                							:
                								( !empty(network.metadata.sourceUrl)
                									?
                										' <a class="external_link" target="_blank" href="' + network.metadata.sourceUrl + '">' + $.i18n("network_source." + network.metadata.source) + '</a> '
                									:
                										$.i18n("network_source." + network.metadata.source)
                								)
                						)
                					:
                						' <span class="reference">' + network.metadata.reference + '</span> '
                				) +
                				( network.id <= 0 && !empty(network.metadata.comment) ?
                						 network.metadata.comment
            						:
            							''
                				) +
                			'</p>'
                		:
                			''
            		);
	
        			if( network.tags != null && network.tags.length > 0 ){
        				html +=	'<p><b>' + $.i18n("search_networks.info.tags_title") + '</b><span class="tags">';
        				
        					for(var k in network.tags){
        						var tag = network.tags[k];
        						html += tag.name.toLowerCase() + ( k < network.tags.length - 1 ? ",&#160;&#160;" : "");
        					}
        				
        				html += '</span></p>';
        			}
        			
        			if( network.metadata.invalidInteractions != null && network.metadata.invalidInteractions.length > 0 ){
        				html += '<p><b>' + $.i18n("search_networks.info.invalid_interactions_title") + '</b>';
        					
        					var interactions = [];
        					for(var k in network.metadata.invalidInteractions){
        						var interaction = network.metadata.invalidInteractions[k];
        						interactions.push(interaction);
        					}
        					
        					interactions.sort();
        					for(var k in interactions){
        						var interaction = interactions[k];
        						html += interaction + "; &#160;";
        					}
        				
        				html += '</p>';
        			}
    html +=     '</div>' +
            '</div>';
    html += '</td>';
    
    html += '</tr>';
    
    return html;
}

var _load_networks_last_org;
function loadNetworks(){

	$("#posted_networks").remove();
	
	var organism = $("#species_select").val();
	var first_load = $(".query_networks[organism=" + organism + "]").size() == 0;
    
//	console.log("loading networks for organism with id " + organism + " and need to fetch " + first_load);
	
	$("#networks_section_loading").show();
	
	function update_counts(){
		var org = $("#species_select").val();
		$("#groupsPanel").children(".query_network_group[organism="+org+"]").each(function(){
			updateChecktreeItem(this);
		});
		refreshAllGroupCounts();
	}
	
	function sort(){
		var sort_method = $("#advanced_options_open .selected_sorting.sort");
		
		if( sort_method.size() == 0 ){
			sort_method = $("#advanced_options_open .sort:first");
		}
		
		sort_method.click();
	}
	
	function show_only_selected_group(){
		var current_group = $(".query_network_group[organism=" + organism + "].selected");
		
		if( current_group.size() == 0 ){
			current_group = $(".query_network_group[organism=" + organism + "]:first");
			current_group.addClass("selected");
		}
		
		var id = current_group.attr("group");
		
		$(".query_network_group[organism=" + organism + "]").show();
		$(".query_network_group[organism!=" + organism + "]").hide();
		
		$(".query_networks[organism=" + organism + "]").filter("[group=" + id + "]").show();
		$(".query_networks[organism=" + organism + "]").filter("[group!=" + id + "]").hide();
		$(".query_networks[organism!=" + organism + "]").hide();
	}
	
	function finshed_loading(){
		
		function show(){
			setTimeout(function(){
				$("#networks_section_loading").hide();
				$("#networks_section").trigger("load");
			}, 100);
		}
		
	
		if( organism == undefined || organism != _load_networks_last_org ){
			setTimeout(function(){
				validateNetworks();
				show_only_selected_group();
				update_counts();
				sort();
				show();
			}, 10);
			
		} else {
			
			show();
		}
		
		_load_networks_last_org = organism;
	}
	
	
	
	if( first_load ){

		function populate_cache(data){
			var network_groups = data.networkGroups;
			var attr_groups = data.attributeGroups;
			var groups_map = {}; // name => group
			var group_names = [];
			var group_names_to_put_at_end = [];
 //console.log(network_groups);
			
			for(var i in network_groups){
				var group = network_groups[i];
				groups_map[group.name] = group;
				
				if(group.name != "Other" && group.name != "Uploaded"){
					group_names.push(group.name);
				} else {
					group_names_to_put_at_end.push(group.name);
				}
			}
			
			var attr_group_name = "Attributes";
			var attr_group_id = -1;
			if( attr_groups && attr_groups.length > 0 ){
				groups_map[attr_group_name] = {
					name: attr_group_name,
					id: attr_group_id,
					attributeGroups: attr_groups
				};
				group_names.push(attr_group_name);
			}
			
			group_names.sort();
			
			for(var i in group_names_to_put_at_end){
				group_names.push( group_names_to_put_at_end[i] );
			}
			
			$.each(group_names, function(i){
				var name = group_names[i];
        		var group = groups_map[name];
        		
        		$("#groupsPanel").append(
        			'<div class="query_network_group" organism="' + organism + '" group="' + group.id + '">' +
                        '<span class="network-group-count text"><span class="selected_count" organism="' + organism + '" group="' + group.id + '" id="countSelectedNetworksFor' + group.id + '"></span>/<span class="all_count" organism="' + organism + '" group="' + group.id + '" id="countAllNetworksFor' + group.id + '"></span></span>' +
                        	'<input class="query_group_checkbox" type="checkbox" value="' + group.name + '" organism="' + organism + '" group="' + group.id + '" />' +
                        '<label id="networkGroupLabel' + group.id + '">' + group.name + '</label>' +
                    '</div>'
                );
        		
        		$("#networksPanel").append( '<div class="query_networks" id="networksFor' + group.id + '" organism="' + organism + '" + group="' + group.id + '"></div>' );
        		var networks = group.interactionNetworks;
        		var attrGroups = group.attributeGroups;
        		
        		var html = "<table class='network_sorter'>";
        		html += '<thead> <th class="first_author">First author</th> <th class="last_author">Last author</th> <th class="date">Date</th> <th class="interactions">Interactions</th> <th class="content">Content</th> </thead>';
        		html += '<tbody>';
        		for(var j in networks){
        			var network = networks[j];
        			html += makeNetworkHtml(organism, group, network, null);
        		}
        		for(var j in attrGroups){
        			var attrGroup = attrGroups[j];
        			html += makeNetworkHtml(organism, group, attrGroup, null, "attrgroups");
        		}
        		html += "</tbody>";
        		html += "</table>";
        		
        		var container = $("#networksPanel .query_networks[group=" + group.id + "][organism='"+ organism +"']");
        	    container.append(html);
        	    container.find("table.network_sorter").tablesorter();
        	});

			
			finshed_loading();
		}
		
		$.ajax({
    		cache: false,
    		success: function(data, status, request){
    			//console.log(data);
    			populate_cache(data);
    		},
    		dataType: "json",
    		data: {},
    		type: "GET",
    		url: absoluteUrl("json/feature_groups/" + organism)
    	});
	} else {
		finshed_loading();
	}
}
function loadDefaultOrganismNetworks() {
	// console.log("load networks");
		
	startNetworksLoader();

	var currentSpeciesIndex = $("#species_select").attr("selectedIndex");
}

var _networks_reloading_count = 0;

function adjustNetworksLoaderCSS(){
	if( !jQuery ){
		return;
	}
	
	$("#networks_section_loading").css({
		left: $("#networks_section").position().left,
		top: $("#networks_section").position().top,
		width: $("#networks_section").width(),
		height: $("#networks_section").height() + ( $.browser.msie ? -8 : 0 ) // -8
																				// is a
																				// hack
																				// to
																				// fix
																				// ie
																				// since
																				// css
																				// hacks
																				// don't
																				// work
																				// :[
	});
}

function startNetworksLoader(){
	_networks_reloading_count++;
	// console.log("++ " + _networks_reloading_count);
		
	// $("#networks_section_loading").show();
	adjustNetworksLoaderCSS();
		
	return _networks_reloading_count;
}

function areNetworksReloading(){
	return _networks_reloading_count != 0;
}

function endNetworksLoader(){
	setTimeout(function(){
		_networks_reloading_count--;
		
		if( _networks_reloading_count < 0 ){
			_networks_reloading_count = 0;
		}
		
		// console.log("-- " + _networks_reloading_count);
		
		if(_networks_reloading_count <= 0){
			$("#networks_section_loading").hide();
		}
	}, 500);
}

function refreshCounts(grpId) {
	//console.log("refresh indiv");

	var organism = $("#species_select").val();
	
	var allNetworks = $(".query_networks[group="+grpId+"][organism='"+ organism +"']").find(".query_network.valid").find(":checkbox");
	var allNetworksCount = allNetworks.size();
	var selectedNetworksCount = allNetworks.filter(":checked").size();
	$("[organism='"+ organism +"']#countAllNetworksFor" + grpId).text(allNetworksCount);
	$("[organism='"+ organism +"']#countSelectedNetworksFor" + grpId).text(selectedNetworksCount);
}

function updateParentChecks(){
	
	$(".query_group_checkbox").each(function(){
		var organism = $(this).attr("organism");
		var group = $(this).attr("group");
		var inputs = $(".query_network_checkbox[organism="+ organism +"][group="+ group +"]");
		
		var checked = inputs.filter(":checked").size() > 0;
		var half_checked = inputs.size() != inputs.filter(":checked").size();
		
		if( checked ){
			$(this).attr("checked", true);
		} else {
			$(this).attr("checked", null);
		}
		
		if( half_checked ){
			$(this).addClass("half_checked");
		} else {
			$(this).removeClass("half_checked");
		}
	});
	
}

function updateChecktreeItem(self) {
	var groupId = $(self).attr("group");
	var organism = $(self).attr("organism");
	var children_inputs = $(".query_networks[group="+groupId+"][organism="+organism+"]").find(".valid").find(":checkbox");
	var parent_input = $(".query_group_checkbox[group="+groupId+"][organism="+organism+"]");
	var all = children_inputs.size();
	var checked = children_inputs.filter(":checked").size();
	
//	console.log("updateChecktreeItem");
//	
//	console.log(this);
//	
//	console.log("group: " + groupId);
//	console.log("organism: " + organism);
//	
//	console.log("all: " + all);
//	console.log("checked: " + checked);
	
	parent_input.attr("checked", checked > 0);
	if (checked == 0 || all == checked) {
		parent_input.removeClass("half_checked");
	} else {
		parent_input.addClass("half_checked");
	}
	
	if( parent_input.is(":checked") && !parent_input.hasClass("half_checked") ){
		children_inputs.attr("checked", true);
	} else if( !parent_input.is(":checked") ){
		children_inputs.removeAttr("checked");
	}
	
	if( all == 0 ){
//		console.log("remove parent check");
		parent_input.removeAttr("checked");
	}
}

function refreshAllGroupCounts() {
	// console.log("refresh all");

	var org = $("#species_select").val();
	var inputs = $("#networksPanel").children(".query_networks[organism=" + org + "]").find(":checkbox");
	var totalNetworksCount = inputs.size();
	var totalSelectedNetworksCount = inputs.filter(":checked").size();
	
	// refresh regular networks
	$("#groupsPanel").children(".query_network_group[organism="+org+"]").each(function() {
		var grpId = $(this).attr("group");
		if(grpId != 0){
			refreshCounts(grpId);
		}
	});
	$("#totalNetworksCount").text(totalNetworksCount);
	$("#totalSelectedNetworksCount").text(totalSelectedNetworksCount);
	
	if( totalNetworksCount == totalSelectedNetworksCount ) {
	    $("#network_selection_select_all").addClass("selected_sorting");
	    $("#network_selection_select_none").add("#network_selection_select_default").removeClass("selected_sorting");
	} else if( totalSelectedNetworksCount == 0 ) {
	    $("#network_selection_select_none").addClass("selected_sorting");
	    $("#network_selection_select_default").add("#network_selection_select_all").removeClass("selected_sorting");
	} else if( default_networks_selected() ) {
	    $("#network_selection_select_default").addClass("selected_sorting");
	    $("#network_selection_select_none").add("#network_selection_select_all").removeClass("selected_sorting");
	} else {
	    $("#network_selection_select_none").add("#network_selection_select_all").add("#network_selection_select_default").removeClass("selected_sorting");
	}
	
	
	var section = $("#networksPanel").children(".query_networks[group=0][organism="+org+"]");
	var selectedUserNetworksCount = section.find("input:checked").size();
	var userNetworksCount = section.find(".query_network.valid").size();

	$(".network-group-count .selected_count[group=0][organism="+org+"]").html(selectedUserNetworksCount);
	$(".network-group-count .all_count[group=0][organism="+org+"]").html(userNetworksCount);
	
	
	
	// console.log("end hide");
}

$(function(){
	
	$(".trashIcn").live("mousedown", function(){
		var container = $(this).parents(".query_network");
    	var id = parseInt( container.attr("network") );
    	
    	deleteUserNetwork( container );
    	return false;
    });
	
});

function sortQueryNetworks(criteria) {
	var org = $("#species_select").val();
	
	$("#networks_section .sort").removeClass("selected_sorting");
	$("#networks_section .sort[by="+ criteria +"]").addClass("selected_sorting");
	
	var tables = $(".query_networks[organism='" + org + "'] table.network_sorter");
	var columnToSort = tables.eq(0).find("td." + criteria + ":first").index();
	tables.each(function(i, table){
		var th = $(this).find("th." + criteria);
		
		if( th.hasClass("headerSortDown") || th.hasClass("headerSortUp") ){
			return false;
		}
		
		var order = 0;
		if( th.hasClass("interactions") || th.hasClass("date") ){
			order = 1;
		}
		
		var sorting = [[columnToSort,order]];
		try{
			$(this).trigger("sorton", [sorting]);
		} catch(e){}
		
	});
	
}

function restoreDefaultNetworks() {
	// console.log("--\n setting default networks");

	// reset all networks
	$("#networkTree input[organism=" + $("#species_select").val() + "]").filter("[default=false]").removeAttr("checked");
	// update networks
	$("#networkTree input[organism=" + $("#species_select").val() + "]").filter("[default=true]").attr("checked", "checked");
	// update groups
    $(".query_network_group").each(function() {
    	updateChecktreeItem(this);
    });
}

function default_networks_selected() {
	
	var inputs = $(".query_network_checkbox[organism=" + $("#species_select").val() + "]");
	var defaults = inputs.filter("[default=true]");
	var not_defaults = inputs.not("[default=true]");
	var unchecked_defaults = defaults.not(":checked");
	var checked_not_defaults = not_defaults.filter(":checked");
	
	return unchecked_defaults.size() == 0 && checked_not_defaults.size() == 0;
}

function refreshNetworks() {
	// console.log("refresh networks...");
	
	// restore previously selected networks
    var checkedList = window["checkedQueryNetworks"];

    if (checkedList != null) {
    	$(".query_networks input[type=checkbox]").attr("checked", false);
    	$.each(checkedList, function(i, id) {
            $("#"+id).attr("checked", true);
        });
        window["checkedQueryNetworks"] = null;
    }
	
    // Complete the networks tree styles:
    $(".query_network_group").each(function() {
    	updateChecktreeItem(this);
    	var grpId = $(this).attr("id").substring("queryGroup".length);
    	refreshCounts(grpId);
    });
    
    // validate again:
    validateNetworks();
}

function addNetworkCheckBoxListeners(){
	$(".query_network_group").live("click", function(evt) {
		if( $(evt.target).hasClass("query_group_checkbox") ){
			return;
		}
		
		var orgId = $(this).attr("organism");
		var groupId = $(this).attr("group");
		
		$(".query_networks").hide();
		$(".query_networks[organism=" + orgId + "][group=" + groupId + "]").show();
	
		// Highlight this item...
		$(".query_network_group, .query_network").removeClass("selected");
		$(this).removeClass("select_hover");
		$(this).addClass("selected");
		// ...and the info:
		$(".query_network_info[organism=" + $("#species_select").val() + "]").hide();
		
	    // adjust long network names
		$(".query_network label").each(function() {
			if($(this).width() > 270) {
				$(this).attr("tooltip", $(this).text());
			}
		});
	});
	
	$(".query_network_group input").live("click", function() {
		var checked = $(this).attr("checked") || $(this).hasClass("half_checked");
		
		var orgId = $(this).attr("organism");
		var groupId = $(this).attr("group");
		
		$(".query_networks[organism=" + orgId + "][group=" + groupId + "] input:checkbox").attr("checked", checked);
		
		updateChecktreeItem(this);
		validateTree();
		refreshCounts(groupId);
	});
	
	$(".query_network input").live("click", function(evt) {
		var groupId = $(this).attr("group");
		var organism = $(this).attr("organism");
		updateChecktreeItem(this);
		validateTree();
		refreshCounts(groupId);
		evt.stopPropagation();
	});
	
	$(".query_network").live("click", function(evt) {
		if( $(evt.target).hasClass("query_network_checkbox") || $(evt.target).parents().andSelf().hasClass("query_network_info") ){
			return;
		}
		
		var info = $(this).find(".query_network_info");
		
		$(".query_network").removeClass("selected");
		var visible = info.is(":visible");
		$(".query_network_info").filter("[organism=" + $("#species_select").val() + "]").hide();
		if (!visible) {
			$(this).addClass("selected");
			info.show();
			
			$(this).parents(".query_networks:first").scrollTo($(this), 50);
		}
	});
}

function updateInputNetworks() {
    refreshAllGroupCounts();
}

var queryNetworksSortingCriteria = "FirstAuthor";
var queryNetworksDescendingSorting = false;

$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
    $(".default_networks_link").live("click", function(){
    	
    	$("#network_selection_select_default").trigger("click");
    	
    	var link = $(this);
   		
		if( link.hasClass("also_submit") ){
			link.parents(".qtip").hide();
			$("#findBtn").click();
		}
    	
    	return false;
    });
    
   
    
    $("#network_selection_select_all").click(function() {
    	$(".query_network_group input").filter("[organism=" + $("#species_select").val() + "]").removeClass("half_checked");
    	$("#networkTree input").filter("[organism=" + $("#species_select").val() + "]").attr("checked", true);
    	
    	var uploaded_group_input = $(".query_network_group input").filter("[organism=" + $("#species_select").val() + "][group=0]");
    	var uploaded_inputs = $(".query_network input").filter("[organism=" + $("#species_select").val() + "][group=0]");
    	if( uploaded_inputs.size() == 0 ){
    		uploaded_group_input.removeAttr("checked");
    	}
    	
    	validateTree();
    	refreshAllGroupCounts(); 
    	
    	return false;
    });
    
    $("#network_selection_select_none").click(function() {
    	$(".query_network_group input").filter("[organism=" + $("#species_select").val() + "]").removeClass("half_checked");
    	$("#networkTree input").filter("[organism=" + $("#species_select").val() + "]").attr("checked", false);
    	validateTree();
    	refreshAllGroupCounts(); 
    	
    	return false;
    });
    
    $("#network_selection_select_default").click(function() {
    	restoreDefaultNetworks();
    	validateTree();
    	refreshAllGroupCounts(); 
    	
    	return false;
    }).addClass("selected_sorting");
    
    $(".query_network, .query_network_group").live("mouseover", function() {
    	$(this).addClass("select_hover")	
    });
    
    $(".query_network, .query_network_group").live("mouseout", function() {
    	$(this).removeClass("select_hover")	
    });
    
	// query networks sorting
	$("#networks_section .sort").click(function(){
		
		var criteria = $(this).attr("by");
		sortQueryNetworks(criteria);
		return false;
	});
    
	addNetworkCheckBoxListeners();
	
    
    
});$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
    

	/***************************************************************************
	 * Number of gene results
	 **************************************************************************/
	$("#threshold").change(function(){
		track("Number of gene results", "Change", $(this).val());
	});
});
$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
    
    /***************************************************************************
	 * species entry behaviour
	 **************************************************************************/

	// autocomplete setup
	
    var species = new Array();
    var alias = new Array();
    var species_structs = [];
    $("#species_select > option").each(function(){
        var name = $.trim( $(this).text() );
        var alias = $.trim( $(this).attr("alias") );
        var id = parseInt( $(this).attr('value') );

        species_structs.push({
            name: name,
            alias: alias,
            id: id
        });
    });

    species_structs.sort(function(a, b){
        return a.id - b.id;
    });

    for( var i = 0; i < species_structs.length; i++ ){
        var struct = species_structs[i];

        species.push( struct.name );
        alias.push( struct.name );
    }

    
    var speciesAndAliases = new Array();
    for(var i in species) {
    	speciesAndAliases.push( species[i] );
    }
    for(var i in alias) {
    	speciesAndAliases.push( alias[i] );
    }

    $("#species_text").autocomplete(speciesAndAliases, {
        minChars: 2,
        matchContains: true,
        matchSubset: true,
        max: 5,
        selectFirst: true,
        autoFill: false,
        matchCase: false,
        focusFieldAfterCancel: false,
        focusFieldAfterSelect: false,
        delay: 10,
        onSetValue: updateSpecies,
        scroll: false
    }).bind("keydown", function(e){
    	if( e.which == 13 ){
    		return false; // do not submit form on enter
    	}
    });
    
    function validSpecies(spc) {
        for(var i in species) {
            if( spc == species[i] ) {
                return true;
            }
        }
        return false;
    }
    
    function validAlias(als) {
    	var ret = false;
    	if(als) {
	    	for(var i in alias) {
	            if( als == alias[i] ) {
	                ret = true;
	                break;
	            }
	        }
    	}
        return ret;
    }

    function getSpeciesFromAlias(als) {
    	var ret = null;
    	if(als) {
	    	for(var i in alias) {
	            if(alias[i] == als ) {
	                ret = species[i];
	                break;
	            }
	        }
    	}
        return ret;
    }
    
    function getIdForSpecies(spc){
    	for(var i in species){
    		if( species[i] == spc ){
    			return parseInt(i) + 1;
    		}
    	}
    	
    	return undefined;
    }
    
    function getIdForAlias(als){
    	for(var i in alias){
    		if( alias[i] == als ){
    			return parseInt(i) + 1;
    		}
    	}
    	
    	return undefined;
    }
    
    // clear on click of text box for convenience
    $("#species_text").focus(function() {
        $(this).val("");
    }).blur(function(){
    	updateSpecies();
    });
    
    function updateSpecies() {
        var val;
        
        try{
        	val = $("#species_text").val().trim();
        } catch(e){
        	val = null;
        }
        
//     console.log("--");
//     console.log(val);
        
        if( !val  || val == "" ) {
//     	console.log("grab from select on empty");
            resetSpeciesToSelectBoxVal();
        } else if( validSpecies(val) ) {
//     	console.log("set value to select on valid species");
            $("#species_select").val( getIdForSpecies(val) )
//          console.log(val)
//          console.log(getIdForSpecies(val));

            setTimeout(function(){ $("#species_select").trigger('change') }, 10);
        } else if( validAlias(val) ) {
//     	console.log("set value to select on valid alias");
            $("#species_text").val( getSpeciesFromAlias(val) );
			$("#species_select").val( getIdForAlias(val) );

            setTimeout(function(){ $("#species_select").trigger('change') }, 10);
        } else {
//     	console.log("grab from select on invalid");
        	resetSpeciesToSelectBoxVal();
        }
        
    }
    
    $("#species_select").change(function(){
		var value = $(this).val();
		var option = $("#species_select option[value=" + value + "]");
		
		$("#posted_networks").remove();
		
		track("Species", "Change", option.text());
	});
    
    function resetSpeciesToSelectBoxVal() {
    	$("#species_text").val( $("#species_select option:selected").text() );
    }
    
    // on drop down button click, show menu
    $("#species_drop_down_closed").click(function() {
        // replace button
        $(this).hide();
        $("#species_drop_down_open").show();
        
        // replace widget
        $("#species_closed").css("visibility", "hidden");
        $("#species_open").show();
        
        // fancy ani
        $("#species_select").fadeIn(ANI_SPD, function(){
            $(this).focus();
            // $(this).scrollTop( $("#species_select
			// option:selected").offset().top );
        });
    });
    
    // on select blur, show the text box again
    $("#species_select").blur(function(){
        // replace button
        $("#species_drop_down_open").hide();
        $("#species_drop_down_closed").show();
    
        // replace widget
        $("#species_open").fadeOut(ANI_SPD);
        $("#species_closed").css("visibility", "visible");
        
        // fancy ani
        $(this).hide("slide", { direction: "up" }, ANI_SPD);
    });
    
    // close on select of menu
    $("#species_select").click(function(){
        $(this).blur();
    });
    
    // reset text value with value from select
	var prev_species = $("#species_select").val();
    $("#species_select").change(function() {
	    var current_species = $(this).val();
	    
	    if( current_species != prev_species ){
			$("#species_text").val( $("#species_select option:selected").text() );
			
			clearErrors();
			
			if( $("#advanced_options_open").is(":visible") ){
				setTimeout(function(){
					loadNetworks();
				}, 10);
				loadAttributeGroups();
			} 			
			
			invalidateGeneForOrganismChange();
			
			if( genesTouched() ){
				last_organism = $("#species_select").val();
				validateGeneSymbols();
			}
        }
        
	    prev_species = $("#species_select").val();
    });
    
//	if( $("#species_text").val() !="" ){
//    	$("#species_text").trigger("blur");
//    }
    
});
$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
	// find button setup
	   
	$("#stopBtn").hide().bind("click", function(){
		$("#loading_line").css("visibility", "hidden");
		
		if( window.stop ){
			window.stop();
		} else if( $.browser.msie ){
			document.execCommand("Stop");
		}
		
		$("#findBtn").show();
    	$("#stopBtn").hide();
	});
    
    $("#findBtn").click(function(){
// console.log("--\n find button clicked");
    
    	if( isSubmittable() ){
    		$("#loading_line").css("visibility", "visible");
    		
    		// disable stop button for now
    		// $("#findBtn").hide();
    		// $("#stopBtn").show();
// console.log("find button in submittable state");
// console.log("remove networks for other organisms");
    		 $(".query_networks[organism!=" + $("#species_select").val() + "]").remove();
    		 $(".query_network_group[organism!=" + $("#species_select").val() + "]").remove();
    		 
    		$("#relatedGenes").submit();
    	} else {
// console.log("find button in NON submitable state");
    		
    		$("#loading_line").css("visibility", "hidden");
    		
    		function show_tooltip(msg, type){
				$("#findBtn").qtip({
					content: {
						text: msg,
						title: {
							text: 'Warning',
							button: '<div class="ui-state-error ui-corner-all"> <span class="ui-icon ui-icon-close"></span> </div>'
						}
					},
					show: {
						delay: 0,
						when: false,
						effect: { type: "fade", length: 0 },
						ready: true // Show the tooltip when ready
					},
					hide: {
						delay: 0,
						effect: { type: "fade", length: 0 },
						when: { event: "unfocus" }, // Hide when clicking
													// anywhere else
						fixed: true // Make it fixed so it can be hovered over
					},
					style: {
					   border: { width: 1, radius: 8 },
					   width: { min: 0
					   },
					   screen: true,
					   padding: 8, 
					   textAlign: 'left',
					   name: 'red', 
					   tip: true      // Give it a speech bubble tip with
										// automatic corner detection
					},
					position: {
						type: "absolute",
						adjust: { 
							screen: true
						},
						corner: {
						 	target: 'topMiddle',
						 	tooltip: 'bottomRight'
					  	}
					}
				});
				
				var qtip = $("body .qtip:last");
				
				qtip.qtip("api").onHide = function(){
					qtip.qtip("api").destroy();
				};
				
				qtip.attr("qtipfor", "findBtn");
				qtip.attr("errortype", type);
			}
    		
    		if( isValidating() ){
    			$("#loading_line").css("visibility", "visible");
// console.log("validating find button form submission");
    			
    			var interval = setInterval(function(){  			
    				if( !isValidating() ){
    					clearInterval(interval);
    					
    					if( inGeneErrorState() ){
    						$("#loading_line").css("visibility", "hidden");
// console.log("found error after waiting for find button validtion");
    					} else if( isSubmittable() ) {
// console.log("submit after waiting for find button validation");
    						$("#relatedGenes").submit();
    					} else {
// console.log("try again after waiting for find button validation");
    						$("#findBtn").click();
    					}
    					
    					
    				}
    			}, 100);
    		} else if( !validMaxGeneList() ){
    			show_tooltip(too_many_genes_message, "too_many_genes");
    		} else if( noGenesEntered() ){
    			validateGeneSymbols();
    			show_tooltip('<p>Please enter at least one gene, or <a href="#" class="action_link default_genes_link also_submit">try this example gene list</a>.</p>', "empty");
    		} else if( inGeneErrorState() ){
    			show_tooltip('<p>Please enter at least one valid gene, and then try again.</p>', "no_valid_genes");
    		} else if( areNetworksReloading() ) {
    			$("#loading_line").css("visibility", "visible");
// console.log("find button validation waiting for networks to reload");
    			
    			var interval = setInterval(function(){  			
    				if( !areNetworksReloading() ){
    					clearInterval(interval);
    					
    					if( isSubmittable() ){
// console.log("submit after waiting for submit button");
							$("#relatedGenes").submit();
						} else {
// console.log("try again after waiting for submit button");
							$("#findBtn").click();
						}
    				}
    			}, 100);
    		} else if( uploadingNetwork() ) {
    			$("#loading_line").css("visibility", "visible");
    			// console.log("find button validation waiting for networks to reload");
    			    			
    			    			var interval = setInterval(function(){  			
    			    				if( !uploadingNetwork() ){
    			    					clearInterval(interval);
    			    					
    			    					if( isSubmittable() ){
    			// console.log("submit after waiting for submit button");
    										$("#relatedGenes").submit();
    									} else {
    			// console.log("try again after waiting for submit button");
    										$("#findBtn").click();
    									}
    			    				}
    			    			}, 100);
    		} else if( noNetworksSelected() ) {
// console.log("no networks selected for find button");
    			
    			show_tooltip('<p>Please enable at least one network in the advanced options, or <a href="#" class="action_link default_networks_link also_submit">use the default networks</a>.</p>', "no_networks_selected");
    			
    		} else {
// console.log("something else is wrong for submitting the find button");
    		}
    		
    		
    		
    	}
    	
    	
    	
    });
	
});function refreshUserNetworksEventHandlers() {
	// console.log("refresh handlers...");

	$("div[id^='descriptionFor']").click(
		function(evt) {
			evt.stopPropagation();
		}
	);
	
	var UNNAMED_STRING = "Unnamed network upload";
	
	$("input[id^='networkNameEdit']").blur( function() {
		var networkId = $(this).attr("id").substring("networkNameEdit".length);
		var networkName = $(this).val();
		if ($.trim(networkName) == "") { networkName = UNNAMED_STRING; }
		var networkDescription = $("#networkDescriptionEdit"+networkId).val();
		updateUserNetworkInfo(networkId, networkName, networkDescription);
	});
	
	$("input[id^='networkNameEdit']").keyup(
		function() {
			var netId = $(this).attr("id").substring("networkNameEdit".length);
			var name = $(this).val();
			if( $.trim(name) == "" ) { name = UNNAMED_STRING; }
			
			$("#" + netId + " label").html(name);
		}
	).bind("blur", function(){
		if( $.trim($(this)).val() == "" ){
			$(this).val(UNNAMED_STRING);
		}
	}).bind("focus", function(){
		if( $.trim($(this).val()) == UNNAMED_STRING ){
			$(this).val("");
		}
	}).bind("keydown", function(e){
		if( e.which == 13 ){
			return false;
		}
	});
	
	$("textarea[id^='networkDescriptionEdit']").blur(function() {
		var networkId = $(this).attr("id").substring("networkDescriptionEdit".length);
		var networkName = $("#networkNameEdit"+networkId).val();
		var networkDescription = $(this).val();
		updateUserNetworkInfo(networkId, networkName, networkDescription);
	});
	
	endNetworksLoader();
}

function showUploadProgress(networkName) {
	var org = $("#species_select").val();
	var progresDivEl = "<div class='uploading_query_network' organism='" + org + "' name='" + networkName + "'><span style='float:left'><span class='uploadStatusIcon upload_progress'/></span><label>" + networkName + "</label></div>";

	
	$(".query_networks[group=0][organism="+org+"]").append(progresDivEl);
	$(".query_network_group[group=0][organism="+org+"]").click();
}

function showUploadCompleted(data, error) {

	var name = $(".uploading_query_network").attr("name");
	var organism = $(".uploading_query_network").attr("organism");
	$(".uploading_query_network").remove();
	
	if( data.error && data.network == null ){
		var date = new Date();
		
		function pad(num){
			return ( num < 10 ? "0" : "" ) + num;
		}
		
		var date_str = date.getFullYear() + "-" + pad(date.getMonth() + 1) + "-" + pad(date.getDate()) + " at " + pad(date.getHours()) + ":" + pad(date.getMinutes()) + ":" + pad(date.getSeconds());
		
		data.network = {
			name: name,
			id: 0,
			metadata: {
				comment: date_str
			},
			tags: []
		};
	}

	var group = {
		id: 0
	};
	

	
	var html = makeNetworkHtml(organism, group, data.network, data.error);
	
	var container = $("#networksPanel .query_networks[group=" + group.id + "][organism=" + organism + "] table");
    container.append(html);
	refreshAllGroupCounts();
	updateChecktreeItem( $(".query_group_checkbox[group=" + group.id + "][organism=" + organism + "]") );
	validateTree();
}

function deleteUserNetwork( query_network_div ) {
	
	var container = query_network_div;
	var network_id = container.attr("network");
	var organism = query_network_div.attr("organism");
	
	if(network_id > 0){
		console.log("can't delete non user network");
		return;
	}
	
	function remove() {
		var organism_id = parseInt( $("#species_select").val() );
		
		container.addClass("deleting");
		
		if( network_id == 0 ){
			container.remove();
			console.log("don't post to the server the deletion of failed uploads");
			return;
		}
		
		$.ajax({
			dataType: "json",
			data: {
				organism_id: organism_id,
				network_id: network_id
			},
			error: function(request, status, error){
				console.log("delete error");
				container.remove();
				refreshAllGroupCounts();
				updateChecktreeItem( $(".query_group_checkbox[group=0][organism=" + organism + "]") );
			},
			success: function(data, status, request){
				
				if( data.error ){
					console.log(data.error);
				}
				
				container.remove();
				refreshAllGroupCounts();
				updateChecktreeItem( $(".query_group_checkbox[group=0][organism=" + organism + "]") );
			},
			type: "POST",
			url: absoluteUrl("json/delete_network")
		});
		
		
	}

	$('<div><p>Are you sure you want to remove this network?</p></div>').dialog({
		title: "Confirmation of removal",
		buttons: {
			"Yes, remove it.": function(){ remove(); $(this).dialog("close"); },
			"No, keep it.": function(){ $(this).dialog("close"); }
		},
		modal: true,
		closeOnEscape: true,
    	resizable: false,
    	width: 300,
    	minHeight: 0
	});
}

function updateUserNetworkInfo(networkId, networkName, networkDescription) {
	var organismId = $("#species_select").val();
	if (networkDescription == null) { networkDescription = '' };
	networkDescription = $.trim(networkDescription.replace(/(\n|\r)+/, '').replace(/(\n|\r)+$/, '')); // trim
																										// newline
																										// chars
																										// and
																										// spaces
	$.post("json/upload", { operation: "update", organism: organismId, networkId: networkId, networkName: networkName, networkDescription: networkDescription } );
}$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
    
    var networkHelpDialog = $("#uploadHelpDialog").dialog({
    	autoOpen: false,
    	modal: true,
    	closeOnEscape: true,
    	resizable: false,
    	width: 400,
    	position: ["center", "center"],
    	buttons: {
			"OK, bring me back to GeneMANIA.": function(){ networkHelpDialog.dialog("close"); }
		}
    });
    
	$("#uploadHelpBtn").click(function() {
		networkHelpDialog.dialog("open");
		return false;
	});
	
    /* upload button */

    _uploading = false;
    function process_upload(data, status, request, error){
    	if(error == null){
    		error = data.error;
    	}
//    	console.log(data);
    	showUploadCompleted(data, error);
    	$("#uploadArea").removeClass("disabled");
    	
    	if( data.network != null ){
    		if( error ){
    			track("Upload", "Error", error);
    		} else {
    			track("Upload", "Success");
    		}
    	} else {
    		track("Upload", "Error", "No network was received from the server");
    	}
    	
    	_uploading = false;
    	$("body").trigger("uploadcomplete");
    }
    
    function upload(file, fileName){
    	_uploading = true;
    	
    	var organismId = $("#species_select").val();
    	
    	$("#uploadArea").addClass("disabled");
    	showUploadProgress(fileName);
    	
    	$.ajax({
    		url: absoluteUrl("json/upload_network"),
    		type: "POST",
    		data: { organism_id: organismId, file: file, file_name: fileName },
    		dataType: "json",
    		success: function(data, status, request){
//    			console.log(data);
//    			console.log(status);
//    			console.log(request);
    			
    			process_upload(data, status, request, null);
    		},
    		error: function(request, status, error){
//    			console.log(error);
//    			console.log(status);
//    			console.log(request);
    			
    			process_upload({ error: error }, status, request, error);
    		}
    	});
    }
    jsUpload = function(file, name){
    	track("Upload", "Start", "File size (bytes)", file.length);
    	upload(file, name);
    }
    
    if( FlashDetect.versionAtLeast(MIN_FLASH_VERSION) && $.cookiesenabled() ) {
    	// Use the flash upload component
    	var options = {
                swfPath: absoluteUrl("swf/Importer"),
                flashInstallerPath: absoluteUrl("swf/playerProductInstall"),
                data: function(data){
//    			console.log(data);
    				
    				track("Upload", "Start", "File size (bytes)", data.bytes.length);
    		
    				var str = "";
    				for(var i = 0; i < data.bytes.length; i++){
//    					console.log("character");
    					
    					var ch = String.fromCharCode( data.bytes[i] );
    					str += ch;
//    					
//    					console.log("character");
//    					console.log(ch);
    				}
//    				console.log(str);
    			
        			upload(str, data.metadata.name);
				},
	            ready: function(){
	            	// when the flash component is loaded
	            },
	            typeFilter: function(){
	                return "*";
	            },
	            binary: function(metadata){
	            	return true; // to return data.string and not data.bytes
	            } 
            };
            
        new org.cytoscapeweb.demo.Importer("uploadOverlay", options);
    } else {
    	$("#uploadOverlay").hide();
    }
    
	$("#uploadBtn").bind("click", function(){
		$('<div><p>You need to have <a href="http://get.adobe.com/flashplayer/">Flash</a> installed and cookies enabled ' +
				'to use the upload feature.  Please <a href="http://get.adobe.com/flashplayer/">install Flash</a>, enable cookies, ' +
				'and then reload GeneMANIA.</p> ' +
				'<p>You will need <a href="http://get.adobe.com/flashplayer/">Flash</a> installed if you want to view ' +
				'GeneMANIA\'s visualization, anyway.  So, it is advised that you install <a href="http://get.adobe.com/flashplayer/">Flash</a>. ' +
		'</div>').dialog({
			title: "You need Flash and cookies to upload",
			buttons: {
				"OK, return me to GeneMANIA": function(){ $(this).dialog("close"); }
			},
			modal: true,
			closeOnEscape: true,
	    	resizable: false,
	    	width: 300,
	    	minHeight: 0
		});
    });
});
$(function(){
	
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
    
	/***************************************************************************
	 * radio buttons for weighting
	 **************************************************************************/
	$(".network_weighting_group input").change(function(){
		var id = $(this).attr("id");
		var label = $("label[for=" + id + "]");
		
		track("Network weighting", "Change", label.text());
	});
});
$(function(){

    /**********************************************
    tool tips
    **********************************************/
   
    toolTipOptions = {
    	toolTipId: "tool_tip",
    	toolTipClass: "tool_tip",
    	delayAfterHover: 50
    };

    $("body").tooltip(toolTipOptions);
     
});var _genes_touched;
$(function(){
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
	_genes_touched = $("#results_page").size() > 0 || $("#user_network_exception").size() > 0 || $("#gene_area").val().length > 0;

	$("#gene_text").add("#gene_area").bind("focus edit", function(){
		$(this).trigger("touch");
	}).one("touch", function(){
		//console.log("--\ntouch triggered on gene_text");
		_genes_touched = true;
	}).bind("keydown", function(){
		$(this).trigger("edit");
	}).bind("edit", function(){
		//console.log("--\nedit triggered on genes");
		_between_edit_and_validated = true;
	});
});

function genesTouched(){
	return _genes_touched;
}

function inErrorState(){
	var error = $(".input_error").length > 0;
	return error;
}

function inGeneErrorState(){
	return $("#gene_text").hasClass("input_error");
}

function noGenesEntered(){
	var no_genes = $("#gene_text").val() == "";
	return no_genes;
}

function isSubmittable(){
	return _genes_touched && !inErrorState() && !noGenesEntered() && !isValidating() && !areNetworksReloading() && validMaxGeneList() && !uploadingNetwork();
}

function validMinGeneList() {
	return $("#gene_area").val().match(/\S/);
}

function validMaxGeneList() {
	return numberOfGenes() <= maxNumberOfGenes();
}

function noNetworksSelected(){
	return $("#networks_section").hasClass("input_error");
}

function uploadingNetwork(){
	return _uploading;
}

function invalidateGeneForOrganismChange(){
	_lastValidationGeneSet = "";
	_lastValidationOrg = "";
}

function isValidating(){
	return _validating_active_count != 0 || _between_edit_and_validated;
}function resetValidationIcon(line){
	return; // not needed with new grey out mechanism
	$("#gene_validation_icons .icon").quickeq(line)
		.attr("type", "empty")
		.attr("class", "icon")
		.attr("tooltip", "")
		.attr("gene", "");
}

function setIcons(val_result){
	for(var line in val_result.genes){
		var possible_gene = val_result.genes[line];
		setIcon(line, possible_gene.type, possible_gene.name, possible_gene.other);
	}
	
	$("#gene_validation_icons .icon").quickeq( val_result.size ).nextAll().andSelf()
		.attr("type", "empty")
		.attr("class", "icon")
		.attr("tooltip", "")
		.attr("gene", "")
		.attr("other", "");
}

$(function(){
	$("#gene_validation_icons .icon").css("height", geneAreaLineHeight());
});

var _number_of_icons;
$(function(){
	get_number_of_icons();	
});

function get_number_of_icons(){
	if( _number_of_icons == null ){
		_number_of_icons = $("#gene_validation_icons .icon").size();
	}
	return _number_of_icons;
}

function adjustValidationIcons(only_past_this_line){
	
	//console.log("--\nadjusting validation icons: " + only_past_this_line); 
	
	if(only_past_this_line != undefined){
		//console.log("just enter / bkspc " + only_past_this_line); 
		
		if( only_past_this_line > 0 ){
			var inserted_icon = $('<div class="icon empty inserted" type="empty" line="' + only_past_this_line + '"></div>');
			inserted_icon.append('<div class="image"></div>');
			inserted_icon.css("height", geneAreaLineHeight());
			
			$("#gene_validation_icons .icon").quickeq( only_past_this_line )
			.after(inserted_icon)
			.nextAll().each(function(){
				var line = parseInt( $(this).attr("line") );
			
				//console.log("incr " + line);
				$(this).attr("line", line + 1);
			});
		} else if ( only_past_this_line < 0 ){
			var at_line = $("#gene_validation_icons .icon").quickeq( -1 * only_past_this_line );
				
			at_line.nextAll().each(function(){
				var next = $(this).next();
				
				if( next.size() != 0 ){
					var next_type = next.attr("type"); 
					var next_class = next.attr("class");
					var next_tooltip = next.attr("tooltip");
					var next_other = next.attr("other");
				
					//console.log("decr " + line);
					$(this).attr("type", next_type).attr("class", next_class).attr("tooltip", next_tooltip).attr("other", next_other);
				}

			});
		}
		
	
	}
	
}

function setLoadingIconsForGenes(){
	$("#gene_validation_icons .icon").addClass("loading");
	$("#gene_selection").addClass("loading");
}

function removeLoadingIconsForGenes(){
	$("#gene_validation_icons .loading").each(function(){
		$(this).removeClass("loading");
	});
	$("#gene_selection").removeClass("loading");
}

function setIcon(line, type, gene, str){
	var icons = $("#gene_validation_icons .icon");
	var icon = icons.quickeq(line);
	
	type = type.toLowerCase();
	gene = gene.toLowerCase();
	str = str.toLowerCase();
	
	if( icon.size() == 0 ){
		//console.log("No such icon with line " + line);
	}
	
	icon.removeClass( icon.attr("type") );
	
	icon.addClass(type);
	icon.attr("type", type);
	icon.attr("gene", gene);
	icon.attr("other", str);
	
	var tooltip;
	if( type == "valid" ){
		tooltip = "The gene, \"" + gene + "\", is recognized.";
	} else if ( type == "invalid"  ) {
		tooltip = "The gene, \"" + gene + "\", is unrecognized.";
	} else if( type == "synonym" ) {
		tooltip = "The gene, \"" + gene + "\", is a synonym" + ( str != undefined ? " of \"" + str + "\"" : "" ) + ".";
	} else if( type == "duplicate" ) {
		tooltip = "The gene, \"" + gene + "\", is a duplicate.";
	} else if( type == "loading" ) {
		tooltip = "The gene, \"" + gene + "\", is being checked.";
	} else if( type == "empty" ){
		tooltip = "This line is empty and contains no gene.";
	}
	
	icon.attr("tooltip", tooltip);

}
function validateGeneSymbols() {
	
	if( !validMaxGeneList() ){
//console.log("too many genes to validate");
		_between_edit_and_validated = false;
		$("#gene_area").trigger("validationcomplete");
		return; // don't validate on huge lists
	}
	
	var msg = "";
	var type = "warning";
	var code = 0;
	var org = $('#species_select').val();
	var geneSet = $("#gene_area").val();
	var errMsg = "";
	var details = "";
	var val_result;
		
	var counting_sem;
	function get_sem(){
		_validating_active_count++;
		counting_sem = ++_validating % 1000000;
		return counting_sem;
	}
	
	function release_sem(){
		_validating_active_count--;
	}
	
	get_sem();
//console.log("--\nvalidating gene symbols");
//console.log("semaphor id: "+counting_sem);
//console.log("number of active validation threads: "+_validating_active_count);
	
	function latest_sem(){
//console.log("sem : " + counting_sem);
		return counting_sem == _validating;
	}

	
	// don't validate if the gene list is the same
	if (_lastValidationGeneSet.toLowerCase() == geneSet.toLowerCase() && _lastValidationOrg.toLowerCase() == org.toLowerCase()
	&& validMinGeneList() // need to show error even if the same
	) {
		if( latest_sem() ){
			_between_edit_and_validated = false;
		}
		release_sem();
//console.log("validation thread died before starting; same as last");
//console.log( geneSet );
//console.log( org );
//console.log("with last");
//console.log( _lastValidationGeneSet );
//console.log( _lastValidationOrg );
		
		removeLoadingIconsForGenes();
		updateErrorMessage();
		
		$("#gene_area").trigger("validationcomplete");
		return;
	}
	
	_lastValidationGeneSet = geneSet;
	_lastValidationOrg = org;
	
	// min/max validation 
	if (! validMinGeneList()) {
		showGeneError("error", "Please enter at least one gene symbol.", undefined, false);
			
		if( latest_sem() ){
			_between_edit_and_validated = false;
		}
		
		release_sem();
//console.log("empty die early: "+_validating_active_count);
		
		removeLoadingIconsForGenes();
		
		// We don't need to call the server side if there is no gene!
		$("#gene_area").trigger("validationcomplete");
		return;
	}
	
	function ajax_handler(data){
		if( !latest_sem() ){
			release_sem();
			_between_edit_and_validated = false;
			removeLoadingIconsForGenes();
			$("#gene_area").trigger("validationcomplete");
			return; // if we're not the last validation thread created, just die
		}
		
		var errMsg = "";
		details = "";
		var MAX_GENES = 10;
		
		removeLoadingIconsForGenes();
		setIcons(data);
		updateErrorMessage();
		
		release_sem();
//console.log("releasing semaphore with new active thread count of "+_validating_active_count);
		
		_between_edit_and_validated = false;
		$("#gene_area").trigger("validationcomplete");
		
	}
	
	$.ajax({
		cache: false,
		success: function(data, status, request){
//			console.log(data);
			ajax_handler(data);
		},
		dataType: "json",
		data: { organism: org, genes: geneSet },
		type: "POST",
		url: absoluteUrl("json/gene_validation")
	});
}


function maxNumberOfGenes(){
	return get_number_of_icons();
}

function numberOfGenes(){
	return $("#gene_area").val().split("\n").length;
}

function clearGeneError(){
	clearError( $("#gene_text"), $("#gene_error") );
	clearError( $("#gene_open") );
	
		
	$(".qtip[qtipfor=gene_selection]").remove();
}

function showGeneError(type, msg, details, show_tooltip) {
	clearGeneError();

	// update tooltips for both gene inputs
	setError({ type: type, msg: msg, details: details }, $("#gene_text"), $("#gene_error"), true);
	setError({ type: type }, $("#gene_open"));


	if( show_tooltip == undefined || show_tooltip ){
		var prev_qtip = $(".qtip[qtipfor=gene_selection]");
		var had_prev_qtip = prev_qtip.size() > 0;
		prev_qtip.remove();

		$("#gene_area").qtip({
			content: {
				title: { 
					text: msg
				},
				text: details
			},
			show: {
				delay: 0,
				when: { event: "showtooltip", target: $("#gene_area") },
				effect: { type: "fade", length: 0 },
				ready: true // Show the tooltip when ready
			},
			hide: {
				delay: 0,
				effect: { type: "fade", length: 0 },
				when: { event: "hidetooltip", target: $("#gene_area") }, // Hide when clicking anywhere else
				fixed: true // Make it fixed so it can be hovered over
			},
			style: {
			   border: { width: 1, radius: 8 },
			   width: { min: 360, max: 360 },
			   screen: true,
			   padding: 8, 
			   textAlign: 'left',
			   name: 'cream',
			   tip: true      // Give it a speech bubble tip with automatic corner detection
			},
			position: {
				type: "absolute",
				adjust: {
					y: -12,
					screen: true,
					scroll: true
				},
				corner: {
					target: 'leftTop',
					tooltip: 'rightTop'
				}
			}
		});
		
		var qtip = $("body").children(".qtip:last");
	
		qtip.hide();
		qtip.attr("qtipfor", "gene_selection").addClass("gene_error_tooltip");
		
		// fix alt tab bug
		qtip.qtip("api").beforeShow = function(){
			if( !$("#gene_area").is(":visible") ){
				return false;
			}
		};
		
		if( $("#gene_area").is(":visible") ){
			$("#gene_area").trigger("showtooltip");
		}
		
		$("#gene_area").bind("adjusttooltipposition", function(){
			$(".qtip[qtipfor=gene_selection]").each(function(){
				$(this).qtip("api").updatePosition();
			});
		});
	}
	

}

function updateGeneCount(){
	var count;
	
	if( $("#gene_area").val() == "" || $("#gene_area").val().match(/^\s*$/g) ){
		count = 0;
	} else {
		count = numberOfGenes();
	}
	
	$("#gene_count").html(count);
}

var _geneAreaLineHeightCached = undefined;
function geneAreaLineHeight(){
	
	if( _geneAreaLineHeightCached == undefined ){
		var div = $("<div>hello&nbsp;</div>").css({
			"font" : $("#gene_area").css("font"),
			"position" : "absolute",
			"left" : 0,
			"top" : 0,
			"font-size" : $("#gene_area").css("font-size")
		});
		
		if( $.browser.msie && parseFloat($.browser.version) <= 8 ){
        	// ie8 does this for garbage and returns 1px
        } else {
        	div.css({
        		"line-height": $("#gene_area").css("line-height")
        	});
        }
		
		$("body").append(div);
		
		
		var lineHeight = parseFloat( div.css("height") );
		
		if( isNaN(lineHeight) ){
			lineHeight = div.height();
		}
		
		div.remove();
		_geneAreaLineHeightCached = lineHeight;
	}
	
	return _geneAreaLineHeightCached;
}

function validateNetworks() {
	
	//console.log("validate networks for organism " + $("#species_select").val());
	var msg;
	var networskError = ($("#networkTree input:checked").filter("[organism=" + $("#species_select").val() + "]").length == 0);
	if (networskError) {
		msg = "With no networks selected, the default networks will be used.";
		setError({ type: "warning", msg: msg }, $("#networks_section"), $("#networks_section_error"));
		$("#networks_section:visible");
	} else {
		clearError( $("#networks_section"), $("#networks_section_error") );
	}
	
}

function updateSubmitButton() {
	
}

function validateTree() {
	updateInputNetworks();
	validateNetworks();
}function updateErrorMessage(){
			// Message and details:
			var details = "";
			var errMsg = "";
			var type = "warning";
			var msg = "";
			
			function gene_names(icons, use_other){
				var ret = "";
				var name_map = {}; // name => count
				var other_map = {}; // name => other
				
				icons.each(function(){
					var name = $(this).attr("gene").toLowerCase();
					var other = $(this).attr("other").toLowerCase();
					
					if( name_map[name] != null ){
						name_map[name] = name_map[name] + 1;
					} else {
						name_map[name] = 1;
					}
					
					other_map[name] = other;
				});
				
				for(var name in name_map){
					var count = name_map[name];
					var other = other_map[name];
					
					ret += "<strong>" + name + "</strong>" + ( count > 1 ? " (" + (use_other ? other : count + "x" ) + ")" : "" ) + "; ";
				}
 				
				return ret;
			}
			
			var synonyms = $("#gene_validation_icons .icon[type=synonym]");
			var synonyms_size = synonyms.size();
			
			var duplicates = $("#gene_validation_icons .icon[type=duplicate]");
			var duplicates_size = duplicates.size();
			
			var invalids = $("#gene_validation_icons .icon[type=invalid]");
			var invalids_size = invalids.size();
			
			var valids = $("#gene_validation_icons .icon[type=valid]");
			var valids_size = valids.size();
			
			var invalid_types = 0;
			
			if (synonyms_size > 0) { // synonyms
				type = "warning";
				errMsg = "There " + (synonyms_size == 1 ? "is a" : "are" ) + " duplicated gene symbol" + (synonyms_size > 1 ? "s" : "") + ".";
				
				invalid_types++;
			} 
			
			if (duplicates_size > 0) { // duplicates
				type = "warning";
				errMsg = "There " + (duplicates_size == 1 ? "is a" : "are" ) + " duplicated gene symbol" + (duplicates_size > 1 ? "s" : "") + ".";
				
				invalid_types++;
			} 
			
			if ( invalids_size > 0 ) { // invalid symbols
				
				if (valids_size <= 0) {
					type = "error";
					errMsg = "None of the symbols entered were recognized.";
				} else {
					type = "warning";
					errMsg = "There " + (valids_size == 1 ? "is an" : "are" ) + " unrecognized gene symbol" + (valids_size > 1 ? "s" : "") + ".";
				}
				
				invalid_types++;
			}
			
	
			if( !validMinGeneList() ){
				invalid_types++;
			}
			
			if ( invalid_types > 1 ) { // multiple errors
				type = "warning";
				errMsg = "There are multiple warnings.";
			}
			
			
			if (invalids_size > 0) {
				details += "<p>Unrecognized gene symbols ("+invalids_size+"):</p><p class='invalid_genes_list ui-corner-all'>"+gene_names(invalids)+"</p>";
			}
			if (synonyms_size > 0) {
				details += "<p>Synonyms ("+synonyms_size+"):</p><p class='invalid_genes_list ui-corner-all'>"+gene_names(synonyms, true)+"</p>";
			}
			if (duplicates_size > 0) {
				details += "<p>Duplicates ("+duplicates_size+"):</p><p class='invalid_genes_list ui-corner-all'>"+gene_names(duplicates)+"</p>";
			}

			
			// show tooltip
			if ( invalid_types >= 1 ) {
				showGeneError(type, errMsg, details);
			} else if ( synonyms_size == 0 && invalids_size == 0 && duplicates_size == 0 ) {
				// Clear only if there is no client side errors already set:
				clearGeneError();
			}
}
$(function(){
	if( $("#phrase_and_go").size() == 0 && $("#query_line").size() == 0 ){
		return; // ignore this js file if no query area
	}
	
	if( $("#user_network_exception").size() > 0 ){
		
		var title = "We couldn't complete your query";
		var details = "<p>You deleted some of your uploaded networks while we were processing your query.  Please resend your query by clicking here.</p><p>Please note that your networks are lost when you clear your browser cookies.</p>";
		
		$("#findBtn").qtip({
			content: {
				title: { 
					text: title
				},
				text: details
			},
			show: {
                delay: 0,
                when: false,
                effect: { type: "fade", length: 0 },
                ready: true // Show the tooltip when ready
            },
            hide: {
                delay: 0,
                effect: { type: "fade", length: 0 },
                when: { event: "unfocus" }, // Hide when clicking anywhere else
                fixed: true // Make it fixed so it can be hovered over
            },
			style: {
			   border: { width: 1, radius: 8 },
			   width: { min: 360, max: 360 },
			   screen: true,
			   padding: 8, 
			   textAlign: 'left',
			   name: 'cream',
			   tip: true      // Give it a speech bubble tip with automatic corner detection
			},
			position: {
				type: "absolute",
				adjust: {
					screen: true,
					scroll: true
				},
				corner: {
					target: 'topMiddle',
					tooltip: 'bottomRight'
				}
			}
		});
	}
});
function setError(error, target, msgField, ellipsis) {
	var type = error.type;
	var msg = error.msg;
	var details = error.details;
	if (details == "") { details = null; }
	
	clearError(target, msgField);
	target.addClass("input_"+type);	
	
	if (msgField && msg) {
		msgField.show();
		msgField.addClass(type+"_msg");
		msgField.html('<span class="ellipsis_text">'+msg+'  </span>'); // Do NOT remove extra spaces!!! The ThreeDots plugin needs it.
		if (false && ellipsis) { // disable three dots for now since it doesn't seem to work the 1st time
			msgField.ThreeDots({ max_rows: 1, 
				                 alt_text_e: false, 
				                 alt_text_t: false, 
				                 whole_word: false });
		}
		msgField.show();
	}
}

function clearError(target, msgField) {
	// remove class and warning msg and hide any tool tip shown on target
	
	if (msgField) {
		msgField.hide();
		msgField.html('');
		msgField.removeAttr("warning");
		msgField.removeClass("warning_msg");
		msgField.removeAttr("error");
		msgField.removeClass("error_msg");
		msgField.removeAttr("threedots");
	}
	target.removeClass("input_error");
	target.removeClass("input_warning");
}

/** Remove error highlights. */
function clearErrors() {
	clearGeneError();
	clearError( $("#gene_open") );
	clearError( $("#networks_section"), $("#networks_section_error") );
}
	



var _lastValidationOrg = "";
var _lastValidationGeneSet = "";
var _validating = 0;
var _validating_active_count = 0;
var _between_edit_and_validated = false;
